@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ExampleFO2PDF.java 1036809 2010-11-19 11:25:15Z spepping $ */
+/* $Id: ExampleFO2PDF.java 1356646 2012-07-03 09:46:41Z mehdi $ */
 
 package embedding;
 
@@ -26,19 +26,16 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
 
-//JAXP
+import javax.xml.transform.Result;
+import javax.xml.transform.Source;
 import javax.xml.transform.Transformer;
 import javax.xml.transform.TransformerFactory;
-import javax.xml.transform.Source;
-import javax.xml.transform.Result;
-import javax.xml.transform.stream.StreamSource;
 import javax.xml.transform.sax.SAXResult;
+import javax.xml.transform.stream.StreamSource;
 
-
-// FOP
+import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.apps.Fop;
-import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FopFactory;
 import org.apache.fop.apps.FormattingResults;
 import org.apache.fop.apps.MimeConstants;
@@ -50,7 +47,7 @@ import org.apache.fop.apps.PageSequenceResults;
 public class ExampleFO2PDF {
 
     // configure fopFactory as desired
-    private FopFactory fopFactory = FopFactory.newInstance();
+    private final FopFactory fopFactory = FopFactory.newInstance(new File(".").toURI());
 
     /**
      * Converts an FO file to a PDF file using FOP
