@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: UnicodeClasses.java 1296590 2012-03-03 09:46:03Z gadams $ */
+/* $Id: UnicodeClasses.java 1479969 2013-05-07 16:23:13Z vhennebert $ */
 
 package org.apache.fop.hyphenation;
 
@@ -179,8 +179,8 @@ public final class UnicodeClasses {
         }
         String scheme = unidata.getScheme();
         if (scheme == null || !(scheme.equals("file") || scheme.equals("http"))) {
-            throw new FileNotFoundException
-            ("URI with file or http scheme required for UNIDATA input directory");
+            throw new FileNotFoundException(
+            "URI with file or http scheme required for UNIDATA input directory");
         }
 
         File f = new File(outfilePath);
