@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AFPGraphics2D.java 1357883 2012-07-05 20:29:53Z gadams $ */
+/* $Id: AFPGraphics2D.java 1617052 2014-08-10 06:55:01Z gadams $ */
 
 package org.apache.fop.afp;
 
@@ -32,7 +32,6 @@ import java.awt.Paint;
 import java.awt.Rectangle;
 import java.awt.Shape;
 import java.awt.Stroke;
-import java.awt.TexturePaint;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Ellipse2D;
 import java.awt.geom.Line2D;
@@ -52,13 +51,13 @@ import org.apache.xmlgraphics.image.loader.ImageSize;
 import org.apache.xmlgraphics.image.loader.impl.ImageRendered;
 import org.apache.xmlgraphics.java2d.AbstractGraphics2D;
 import org.apache.xmlgraphics.java2d.GraphicContext;
+import org.apache.xmlgraphics.java2d.GraphicsConfigurationWithTransparency;
 import org.apache.xmlgraphics.java2d.StrokingTextHandler;
 import org.apache.xmlgraphics.java2d.TextHandler;
 import org.apache.xmlgraphics.util.UnitConv;
 
 import org.apache.fop.afp.goca.GraphicsSetLineType;
 import org.apache.fop.afp.modca.GraphicsObject;
-import org.apache.fop.afp.svg.AFPGraphicsConfiguration;
 import org.apache.fop.afp.util.CubicBezierApproximator;
 import org.apache.fop.fonts.FontInfo;
 import org.apache.fop.render.afp.AFPImageHandlerRenderedImage;
@@ -93,25 +92,25 @@ public class AFPGraphics2D extends AbstractGraphics2D implements NativeImageHand
     private static final int Y3 = 5;
 
     /** graphics object */
-    private GraphicsObject graphicsObj = null;
+    private GraphicsObject graphicsObj;
 
     /** Fallback text handler */
     protected TextHandler fallbackTextHandler = new StrokingTextHandler();
 
     /** Custom text handler */
-    protected TextHandler customTextHandler = null;
+    protected TextHandler customTextHandler;
 
     /** AFP resource manager */
-    private AFPResourceManager resourceManager = null;
+    private AFPResourceManager resourceManager;
 
     /** AFP resource info */
-    private AFPResourceInfo resourceInfo = null;
+    private AFPResourceInfo resourceInfo;
 
     /** Current AFP state */
-    private AFPPaintingState paintingState = null;
+    private AFPPaintingState paintingState;
 
     /** AFP graphics configuration */
-    private final AFPGraphicsConfiguration graphicsConfig = new AFPGraphicsConfiguration();
+    private final GraphicsConfigurationWithTransparency graphicsConfig = new GraphicsConfigurationWithTransparency();
 
     /** The AFP FontInfo */
     private FontInfo fontInfo;
@@ -333,14 +332,14 @@ public class AFPGraphics2D extends AbstractGraphics2D implements NativeImageHand
             return true;
         }
         LOG.debug("NYI: applyPaint() " + paint + " fill=" + fill);
-        if (paint instanceof TexturePaint) {
+//      if (paint instanceof TexturePaint) {
 //            TexturePaint texturePaint = (TexturePaint)paint;
 //            BufferedImage bufferedImage = texturePaint.getImage();
 //            AffineTransform at = paintingState.getTransform();
 //            int x = (int)Math.round(at.getTranslateX());
 //            int y = (int)Math.round(at.getTranslateY());
 //            drawImage(bufferedImage, x, y, null);
-        }
+//      }
         return false;
     }
 
