@@ -15,11 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: AFPPaintingState.java 959945 2010-07-02 10:44:18Z jeremias $ */
+/* $Id: AFPPaintingState.java 1617052 2014-08-10 06:55:01Z gadams $ */
 
 package org.apache.fop.afp;
 
 import java.awt.Point;
+import java.io.IOException;
+import java.io.ObjectInputStream;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -34,33 +36,38 @@ import org.apache.fop.util.AbstractPaintingState;
  * This keeps information about the current painting state when writing to an
  * AFP datastream.
  */
-public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState implements
-        Cloneable {
+public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState {
 
     private static final long serialVersionUID = 8206711712452344473L;
 
     private static Log log = LogFactory.getLog("org.apache.xmlgraphics.afp");
 
     /** the portrait rotation */
-    private int portraitRotation = 0;
+    private int portraitRotation;
 
     /** the landscape rotation */
     private int landscapeRotation = 270;
 
     /** color image support */
-    private boolean colorImages = false;
+    private boolean colorImages;
 
     /** dithering quality setting (0.0f..1.0f) */
     private float ditheringQuality;
 
+    /** image encoding quality setting (0.0f..1.0f) */
+    private float bitmapEncodingQuality;
+
     /** color image handler */
-    private ColorConverter colorConverter = GrayScaleColorConverter.getInstance();
+    private transient ColorConverter colorConverter;
 
     /**
      * true if certain image formats may be embedded unchanged in their native
      * format.
      */
-    private boolean nativeImagesSupported = false;
+    private boolean nativeImagesSupported;
+
+    private boolean canEmbedJpeg;
+
     /**
      * true if CMYK images (requires IOCA FS45 suppport on the target platform)
      * may be generated
@@ -73,14 +80,42 @@ public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState
     /** the output resolution */
     private int resolution = 240; // 240 dpi
 
+    /**
+     * A configurable value to correct the line width so that the output matches the expected. Different
+     * devices may need different values.
+     */
+    private float lineWidthCorrection = AFPConstants.LINE_WIDTH_CORRECTION;
+
+    /** determines whether GOCA is enabled or disabled  */
+    private boolean gocaEnabled = true;
+    /** determines whether to stroke text in GOCA mode or to use text operators where possible */
+    private boolean strokeGocaText;
+
+
+    /** use page segment with F11 and F45 images*/
+    private boolean pSeg;
+
+    /** use FS45 images*/
+    private boolean fs45;
+
     /** the current page */
-    private transient AFPPagePaintingState pagePaintingState = new AFPPagePaintingState();
+    private transient AFPPagePaintingState pagePaintingState;
 
     // /** reference orientation */
     // private int orientation = 0;
 
     /** a unit converter */
-    private final transient AFPUnitConverter unitConv = new AFPUnitConverter(this);
+    private final transient AFPUnitConverter unitConv;
+
+    public AFPPaintingState() {
+        colorConverter = GrayScaleColorConverter.getInstance();
+        pagePaintingState = new AFPPagePaintingState();
+        unitConv = new AFPUnitConverter(this);
+    }
+
+    private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
+        ois.defaultReadObject();
+    }
 
     /**
      * Sets the rotation to be used for portrait pages, valid values are 0
@@ -214,6 +249,24 @@ public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState
         return this.nativeImagesSupported;
     }
 
+    /**
+     * Set whether or not JPEG images can be embedded within an AFP document.
+     *
+     * @param canEmbed true if the JPEG image can be embedded
+     */
+    public void setCanEmbedJpeg(boolean canEmbed) {
+        canEmbedJpeg = canEmbed;
+    }
+
+    /**
+     * Returns true if JPEGs can be embedded in an AFP document.
+     *
+     * @return true if JPEG embedding is allowed
+     */
+    public boolean canEmbedJpeg() {
+        return canEmbedJpeg;
+    }
+
     /**
      * Controls whether CMYK images (IOCA FS45) are enabled. By default, support
      * is disabled for wider compatibility. When disabled, any CMYK image is
@@ -254,6 +307,25 @@ public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState
         this.ditheringQuality = quality;
     }
 
+    /**
+     * Gets the image encoding quality setting to use when encoding bitmap images.
+     * @return the encoding quality (a value between 0.0f and 1.0f, 1.0 meaning loss-less)
+     */
+    public float getBitmapEncodingQuality() {
+        return this.bitmapEncodingQuality;
+    }
+
+    /**
+     * Sets the image encoding quality setting to use when encoding bitmap images.
+     * @param quality Defines the desired quality level for the conversion.
+     *                  Valid values: a value between 0.0f (lowest) and 1.0f (best, loss-less)
+     */
+    public void setBitmapEncodingQuality(float quality) {
+        quality = Math.max(quality, 0.0f);
+        quality = Math.min(quality, 1.0f);
+        this.bitmapEncodingQuality = quality;
+    }
+
     /**
      * Sets the output/device resolution
      *
@@ -267,6 +339,18 @@ public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState
         this.resolution = resolution;
     }
 
+    /**
+     * Sets the line width correction
+     *
+     * @param correction the line width multiplying factor correction
+     */
+    public void setLineWidthCorrection(float correction) {
+        if (log.isDebugEnabled()) {
+            log.debug("line width correction set to: " + correction);
+        }
+        this.lineWidthCorrection = correction;
+    }
+
     /**
      * Returns the output/device resolution.
      *
@@ -276,12 +360,89 @@ public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState
         return this.resolution;
     }
 
+    /**
+     * Returns the line width correction.
+     * @return the correction
+     */
+    public float getLineWidthCorrection() {
+        return this.lineWidthCorrection;
+    }
+
+    /**
+     * Controls whether GOCA is enabled or disabled.
+     * @param enabled true if GOCA is enabled, false if it is disabled
+     */
+    public void setGOCAEnabled(boolean enabled) {
+        this.gocaEnabled = enabled;
+    }
+
+    /**
+     * Indicates whether GOCA is enabled or disabled.
+     * @return true if GOCA is enabled, false if GOCA is disabled
+     */
+    public boolean isGOCAEnabled() {
+        return this.gocaEnabled;
+    }
+
+    /**
+     * Controls whether to stroke text in GOCA mode or to use text operators where possible.
+     * @param stroke true to stroke, false to paint with text operators where possible
+     */
+    public void setStrokeGOCAText(boolean stroke) {
+        this.strokeGocaText = stroke;
+    }
+
+    /**
+     * Indicates whether to stroke text in GOCA mode or to use text operators where possible.
+     * @return true to stroke, false to paint with text operators where possible
+     */
+    public boolean isStrokeGOCAText() {
+        return this.strokeGocaText;
+    }
+
+    /**
+     * Whether FS11 and SF45 non-inline images should be wrapped in a page segment
+     * @return true iff images should be wrapped
+     */
+    public boolean getWrapPSeg() {
+        return pSeg;
+    }
+
+    /**
+     * Sets whether FS11 and FS45 non-inline images should be wrapped in a page segment
+     * @param pSeg true iff images should be wrapped
+     */
+    public void setWrapPSeg(boolean pSeg) {
+        this.pSeg = pSeg;
+    }
+
+
+    /**
+     * gets whether images should be FS45
+     * @return true iff images should be FS45
+     */
+    public boolean getFS45() {
+        return fs45;
+    }
+
+    /**
+     * sets whether images should be FS45
+     * @param fs45 true iff images should be FS45
+     */
+    public void setFS45(boolean fs45) {
+        this.fs45 = fs45;
+    }
+
+
+
     /** {@inheritDoc} */
+    @Override
     protected AbstractData instantiateData() {
         return new AFPData();
     }
 
     /** {@inheritDoc} */
+    @Override
     protected AbstractPaintingState instantiate() {
         return new AFPPaintingState();
     }
@@ -423,6 +584,7 @@ public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState
     }
 
     /** {@inheritDoc} */
+    @Override
     public Object clone() {
         AFPPaintingState paintingState = (AFPPaintingState) super.clone();
         paintingState.pagePaintingState = (AFPPagePaintingState) this.pagePaintingState.clone();
@@ -436,6 +598,7 @@ public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState
     }
 
     /** {@inheritDoc} */
+    @Override
     public String toString() {
         return "AFPPaintingState{" + "portraitRotation=" + portraitRotation
                 + ", landscapeRotation=" + landscapeRotation + ", colorImages=" + colorImages
@@ -448,19 +611,19 @@ public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState
      */
     private class AFPPagePaintingState implements Cloneable {
         /** page width */
-        private int width = 0;
+        private int width;
 
         /** page height */
-        private int height = 0;
+        private int height;
 
         /** page fonts */
         private AFPPageFonts fonts = new AFPPageFonts();
 
         /** page font count */
-        private int fontCount = 0;
+        private int fontCount;
 
         /** page orientation */
-        private int orientation = 0;
+        private int orientation;
 
         /**
          * Returns the page width
@@ -548,6 +711,7 @@ public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState
         }
 
         /** {@inheritDoc} */
+        @Override
         public Object clone() {
             AFPPagePaintingState state = new AFPPagePaintingState();
             state.width = this.width;
@@ -559,6 +723,7 @@ public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState
         }
 
         /** {@inheritDoc} */
+        @Override
         public String toString() {
             return "AFPPagePaintingState{width=" + width + ", height=" + height + ", orientation="
                     + orientation + ", fonts=" + fonts + ", fontCount=" + fontCount + "}";
@@ -568,15 +733,17 @@ public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState
     /**
      * Block level state data
      */
+    // @SuppressFBWarnings("SE_INNER_CLASS")
     private class AFPData extends org.apache.fop.util.AbstractPaintingState.AbstractData {
         private static final long serialVersionUID = -1789481244175275686L;
 
         /** The current fill status */
-        private boolean filled = false;
+        private boolean filled;
 
-        private String imageUri = null;
+        private String imageUri;
 
         /** {@inheritDoc} */
+        @Override
         public Object clone() {
             AFPData obj = (AFPData) super.clone();
             obj.filled = this.filled;
@@ -585,15 +752,17 @@ public class AFPPaintingState extends org.apache.fop.util.AbstractPaintingState
         }
 
         /** {@inheritDoc} */
+        @Override
         public String toString() {
             return "AFPData{" + super.toString() + ", filled=" + filled + ", imageUri=" + imageUri
                     + "}";
         }
 
         /** {@inheritDoc} */
+        @Override
         protected AbstractData instantiate() {
             return new AFPData();
         }
     }
 
-}
\ No newline at end of file
+}
