@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AFPResourceManager.java 1195952 2011-11-01 12:20:21Z phancock $ */
+/* $Id: AFPResourceManager.java 1720357 2015-12-16 13:46:42Z ssteiner $ */
 
 package org.apache.fop.afp;
 
@@ -31,18 +31,26 @@ import org.apache.commons.io.IOUtils;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.fop.afp.AFPResourceLevel.ResourceType;
 import org.apache.fop.afp.fonts.AFPFont;
 import org.apache.fop.afp.fonts.CharacterSet;
 import org.apache.fop.afp.modca.AbstractNamedAFPObject;
 import org.apache.fop.afp.modca.AbstractPageObject;
+import org.apache.fop.afp.modca.ActiveEnvironmentGroup;
 import org.apache.fop.afp.modca.IncludeObject;
 import org.apache.fop.afp.modca.IncludedResourceObject;
+import org.apache.fop.afp.modca.ObjectContainer;
 import org.apache.fop.afp.modca.PageSegment;
 import org.apache.fop.afp.modca.Registry;
 import org.apache.fop.afp.modca.ResourceGroup;
 import org.apache.fop.afp.modca.ResourceObject;
+import org.apache.fop.afp.modca.triplets.EncodingTriplet;
+import org.apache.fop.afp.modca.triplets.FullyQualifiedNameTriplet;
+import org.apache.fop.afp.util.AFPResourceAccessor;
 import org.apache.fop.afp.util.AFPResourceUtil;
-import org.apache.fop.afp.util.ResourceAccessor;
+import org.apache.fop.apps.io.InternalResourceResolver;
+import org.apache.fop.fonts.FontType;
+import org.apache.fop.render.afp.AFPFontConfig;
 
 /**
  * Manages the creation and storage of document resources
@@ -63,24 +71,21 @@ public class AFPResourceManager {
     private final AFPDataObjectFactory dataObjectFactory;
 
     /** Maintain a reference count of instream objects for referencing purposes */
-    private int instreamObjectCount = 0;
-
-    /** a mapping of resourceInfo --> include name */
-    private final Map<AFPResourceInfo, String> includeNameMap
-        = new java.util.HashMap<AFPResourceInfo, String>();
-
-    /** a mapping of resourceInfo --> page segment name */
-    private Map<AFPResourceInfo, String> pageSegmentMap
-        = new java.util.HashMap<AFPResourceInfo, String>();
+    private int instreamObjectCount;
 
+    /** Mapping of resourceInfo to AbstractCachedObject */
+    private final Map<AFPResourceInfo, AbstractCachedObject> includeObjectCache
+            = new java.util.HashMap<AFPResourceInfo, AbstractCachedObject>();
     private AFPResourceLevelDefaults resourceLevelDefaults = new AFPResourceLevelDefaults();
 
     /**
      * Main constructor
+     *
+     * @param resourceResolver  the associated {@link InternalResourceResolver} instance
      */
-    public AFPResourceManager() {
+    public AFPResourceManager(InternalResourceResolver resourceResolver) {
         this.factory = new Factory();
-        this.streamer = new AFPStreamer(factory);
+        this.streamer = new AFPStreamer(factory, resourceResolver);
         this.dataObjectFactory = new AFPDataObjectFactory(factory);
     }
 
@@ -118,13 +123,13 @@ public class AFPResourceManager {
     }
 
     /**
-     * Sets the default resource group file path
+     * Sets the default resource group URI.
      *
-     * @param filePath the default resource group file path
+     * @param uri the default resource group URI
      */
 
-    public void setDefaultResourceGroupFilePath(String filePath) {
-        streamer.setDefaultResourceGroupFilePath(filePath);
+    public void setDefaultResourceGroupUri(URI uri) {
+        streamer.setDefaultResourceGroupUri(uri);
     }
 
     /**
@@ -138,21 +143,7 @@ public class AFPResourceManager {
     public boolean tryIncludeObject(AFPDataObjectInfo dataObjectInfo) throws IOException {
         AFPResourceInfo resourceInfo = dataObjectInfo.getResourceInfo();
         updateResourceInfoUri(resourceInfo);
-
-        String objectName = includeNameMap.get(resourceInfo);
-        if (objectName != null) {
-            // an existing data resource so reference it by adding an include to the current page
-            includeObject(dataObjectInfo, objectName);
-            return true;
-        }
-
-        objectName = pageSegmentMap.get(resourceInfo);
-        if (objectName != null) {
-            // an existing data resource so reference it by adding an include to the current page
-            includePageSegment(dataObjectInfo, objectName);
-            return true;
-        }
-        return false;
+        return includeCachedObject(resourceInfo, dataObjectInfo.getObjectAreaInfo());
     }
 
     /**
@@ -193,7 +184,7 @@ public class AFPResourceManager {
 
         useInclude &= resourceGroup != null;
         if (useInclude) {
-            boolean usePageSegment = dataObjectInfo.isCreatePageSegment();
+            final boolean usePageSegment = dataObjectInfo.isCreatePageSegment();
 
             // if it is to reside within a resource group at print-file or external level
             if (resourceLevel.isPrintFile() || resourceLevel.isExternal()) {
@@ -211,23 +202,110 @@ public class AFPResourceManager {
 
             // add data object into its resource group destination
             resourceGroup.addObject(namedObj);
-
-            // create the include object
-            String objectName = namedObj.getName();
-            if (usePageSegment) {
-                includePageSegment(dataObjectInfo, objectName);
-                pageSegmentMap.put(resourceInfo, objectName);
-            } else {
-                includeObject(dataObjectInfo, objectName);
-                // record mapping of resource info to data object resource name
-                includeNameMap.put(resourceInfo, objectName);
-            }
+            includeObject(namedObj, dataObjectInfo);
         } else {
             // not to be included so inline data object directly into the current page
             dataStream.getCurrentPage().addObject(namedObj);
         }
     }
 
+    private abstract class AbstractCachedObject {
+        protected String objectName;
+        protected AFPDataObjectInfo dataObjectInfo;
+
+        public AbstractCachedObject(String objectName, AFPDataObjectInfo dataObjectInfo) {
+            this.objectName = objectName;
+            this.dataObjectInfo = dataObjectInfo;
+
+
+        }
+        protected abstract void includeObject();
+    }
+
+    private class CachedPageSegment extends AbstractCachedObject {
+
+        public CachedPageSegment(String objectName, AFPDataObjectInfo dataObjectInfo) {
+           super(objectName, dataObjectInfo);
+        }
+
+        protected void includeObject() {
+            includePageSegment(dataObjectInfo, objectName);
+        }
+
+    }
+
+    private class CachedObject extends AbstractCachedObject {
+
+        public CachedObject(String objectName, AFPDataObjectInfo dataObjectInfo) {
+           super(objectName, dataObjectInfo);
+        }
+
+        protected void includeObject() {
+            AFPResourceManager.this.includeObject(dataObjectInfo, objectName);
+        }
+
+    }
+
+
+    private void includeObject(AbstractNamedAFPObject namedObj, AFPDataObjectInfo dataObjectInfo) {
+
+        // create the include object
+        String objectName = namedObj.getName();
+
+        AbstractCachedObject cachedObject;
+
+        if (dataObjectInfo.isCreatePageSegment()) {
+            cachedObject = new CachedPageSegment(objectName, dataObjectInfo);
+        } else {
+            cachedObject = new CachedObject(objectName, dataObjectInfo);
+        }
+
+        cachedObject.includeObject();
+
+        includeObjectCache.put(dataObjectInfo.getResourceInfo(), cachedObject);
+
+        //The data field of dataObjectInfo is not further required
+        // therefore we are safe to null the reference, saving memory
+        dataObjectInfo.setData(null);
+
+    }
+
+    /**
+     * Returns {@code true} if the passed {@link AFPResourceInfo} instance is already cached.
+     *
+     * @param resourceInfo  the resource info to check
+     * @return  {@code true} if the object is cached
+     */
+    public boolean isObjectCached(AFPResourceInfo resourceInfo) {
+        return includeObjectCache.containsKey(resourceInfo);
+    }
+
+    /**
+     * {@asf.todo}
+     *
+     * @param resourceInfo  the resource info to check
+     * @param areaInfo  the area info to check
+     * @return  {@code true} if ...
+     */
+    public boolean includeCachedObject(AFPResourceInfo resourceInfo, AFPObjectAreaInfo areaInfo) {
+
+        String objectName;
+
+        AbstractCachedObject cachedObject = includeObjectCache.get(resourceInfo);
+
+        if (cachedObject != null) {
+            if (areaInfo != null) {
+                cachedObject.dataObjectInfo.setObjectAreaInfo(areaInfo);
+            }
+            cachedObject.includeObject();
+
+            return true;
+        } else {
+            return false;
+        }
+    }
+
+
     private void updateResourceInfoUri(AFPResourceInfo resourceInfo) {
         String uri = resourceInfo.getUri();
         if (uri == null) {
@@ -257,13 +335,21 @@ public class AFPResourceManager {
         if (afpFont.isEmbeddable()) {
             //Embed fonts (char sets and code pages)
             if (charSet.getResourceAccessor() != null) {
-                ResourceAccessor accessor = charSet.getResourceAccessor();
-                createIncludedResource(
-                        charSet.getName(), accessor,
-                        ResourceObject.TYPE_FONT_CHARACTER_SET);
-                createIncludedResource(
-                        charSet.getCodePage(), accessor,
-                        ResourceObject.TYPE_CODE_PAGE);
+                AFPResourceAccessor accessor = charSet.getResourceAccessor();
+                if (afpFont.getFontType() == FontType.TRUETYPE) {
+
+                    createIncludedResource(afpFont.getFontName(),
+                            ((AFPFontConfig.AFPTrueTypeFont) afpFont).getUri(), accessor,
+                            ResourceObject.TYPE_OBJECT_CONTAINER, true,
+                            ((AFPFontConfig.AFPTrueTypeFont) afpFont).getTTC());
+                } else {
+                    createIncludedResource(
+                            charSet.getName(), accessor,
+                            ResourceObject.TYPE_FONT_CHARACTER_SET);
+                    createIncludedResource(
+                            charSet.getCodePage(), accessor,
+                            ResourceObject.TYPE_CODE_PAGE);
+                }
             }
         }
     }
@@ -284,7 +370,7 @@ public class AFPResourceManager {
      * @param resourceObjectType the resource object type ({@link ResourceObject}.*)
      * @throws IOException if an I/O error occurs while loading the resource
      */
-    public void createIncludedResource(String resourceName, ResourceAccessor accessor,
+    public void createIncludedResource(String resourceName, AFPResourceAccessor accessor,
                 byte resourceObjectType) throws IOException {
         URI uri;
         try {
@@ -294,7 +380,7 @@ public class AFPResourceManager {
                     + " (" + e.getMessage() + ")");
         }
 
-        createIncludedResource(resourceName, uri, accessor, resourceObjectType);
+        createIncludedResource(resourceName, uri, accessor, resourceObjectType, false, null);
     }
 
     /**
@@ -305,36 +391,83 @@ public class AFPResourceManager {
      * @param resourceObjectType the resource object type ({@link ResourceObject}.*)
      * @throws IOException if an I/O error occurs while loading the resource
      */
-    public void createIncludedResource(String resourceName, URI uri, ResourceAccessor accessor,
-                byte resourceObjectType) throws IOException {
-        AFPResourceLevel resourceLevel = new AFPResourceLevel(AFPResourceLevel.PRINT_FILE);
+    public void createIncludedResource(String resourceName, URI uri, AFPResourceAccessor accessor,
+                                       byte resourceObjectType, boolean truetype, String ttc) throws IOException {
+        AFPResourceLevel resourceLevel = new AFPResourceLevel(ResourceType.PRINT_FILE);
 
         AFPResourceInfo resourceInfo = new AFPResourceInfo();
         resourceInfo.setLevel(resourceLevel);
         resourceInfo.setName(resourceName);
         resourceInfo.setUri(uri.toASCIIString());
 
-        String objectName = includeNameMap.get(resourceInfo);
-        if (objectName == null) {
+        AbstractCachedObject cachedObject = includeObjectCache.get(resourceInfo);
+        if (cachedObject == null) {
             if (log.isDebugEnabled()) {
                 log.debug("Adding included resource: " + resourceName);
             }
-            IncludedResourceObject resourceContent = new IncludedResourceObject(
-                        resourceName, accessor, uri);
-
-            ResourceObject resourceObject = factory.createResource(resourceName);
-            resourceObject.setDataObject(resourceContent);
-            resourceObject.setType(resourceObjectType);
 
             ResourceGroup resourceGroup = streamer.getResourceGroup(resourceLevel);
-            resourceGroup.addObject(resourceObject);
+
+            if (truetype) {
+                ResourceObject res = factory.createResource();
+                res.setType(ResourceObject.TYPE_OBJECT_CONTAINER);
+
+                ActiveEnvironmentGroup.setupTruetypeMDR(res, false);
+
+                ObjectContainer oc = factory.createObjectContainer();
+                InputStream is = accessor.createInputStream(uri);
+
+                if (ttc != null) {
+                    oc.setData(extractTTC(ttc, is));
+                } else {
+                    oc.setData(IOUtils.toByteArray(is));
+                }
+
+                ActiveEnvironmentGroup.setupTruetypeMDR(oc, true);
+
+                res.addTriplet(new EncodingTriplet(1200));
+
+                res.setFullyQualifiedName(FullyQualifiedNameTriplet.TYPE_REPLACE_FIRST_GID_NAME,
+                        FullyQualifiedNameTriplet.FORMAT_CHARSTR, resourceName, true);
+
+                res.setDataObject(oc);
+                resourceGroup.addObject(res);
+            } else {
+                ResourceObject resourceObject = factory.createResource(resourceName);
+                IncludedResourceObject resourceContent = new IncludedResourceObject(
+                        resourceName, accessor, uri);
+                resourceObject.setDataObject(resourceContent);
+                resourceObject.setType(resourceObjectType);
+                resourceGroup.addObject(resourceObject);
+            }
+
+            //TODO what is the data object?
+            cachedObject = new CachedObject(resourceName, null);
+
             // record mapping of resource info to data object resource name
-            includeNameMap.put(resourceInfo, resourceName);
+            includeObjectCache.put(resourceInfo, cachedObject);
         } else {
             //skip, already created
         }
     }
 
+    private byte[] extractTTC(String ttc, InputStream is) throws IOException {
+//        TrueTypeCollection trueTypeCollection = new TrueTypeCollection(is);
+//        for (TrueTypeFont ttf : trueTypeCollection.getFonts()) {
+//            String name = ttf.getNaming().getFontFamily();
+//            if (name.equals(ttc)) {
+//                ByteArrayOutputStream bos = new ByteArrayOutputStream();
+//                TTFSubsetter s = new TTFSubsetter(ttf, null);
+//                for (int i = 0; i < 256 * 256; i++) {
+//                    s.addCharCode(i);
+//                }
+//                s.writeToStream(bos);
+//                return bos.toByteArray();
+//            }
+//        }
+        throw new IOException(ttc + " not supported");
+    }
+
     /**
      * Creates an included resource extracting the named resource from an external source.
      * @param resourceName the name of the resource
@@ -343,18 +476,17 @@ public class AFPResourceManager {
      * @throws IOException if an I/O error occurs while loading the resource
      */
     public void createIncludedResourceFromExternal(final String resourceName,
-            final URI uri, final ResourceAccessor accessor) throws IOException {
+            final URI uri, final AFPResourceAccessor accessor) throws IOException {
 
-        AFPResourceLevel resourceLevel = new AFPResourceLevel(AFPResourceLevel.PRINT_FILE);
+        AFPResourceLevel resourceLevel = new AFPResourceLevel(ResourceType.PRINT_FILE);
 
         AFPResourceInfo resourceInfo = new AFPResourceInfo();
         resourceInfo.setLevel(resourceLevel);
         resourceInfo.setName(resourceName);
         resourceInfo.setUri(uri.toASCIIString());
 
-        String resource = includeNameMap.get(resourceInfo);
-        if (resource == null) {
-
+        AbstractCachedObject cachedObject = includeObjectCache.get(resourceInfo);
+        if (cachedObject == null) {
             ResourceGroup resourceGroup = streamer.getResourceGroup(resourceLevel);
 
             //resourceObject delegates write commands to copyNamedResource()
@@ -380,11 +512,9 @@ public class AFPResourceManager {
                 @Override
                 protected void writeEnd(OutputStream os) throws IOException { }
             };
-
             resourceGroup.addObject(resourceObject);
-
-            includeNameMap.put(resourceInfo, resourceName);
-
+            cachedObject = new CachedObject(resourceName, null);
+            includeObjectCache.put(resourceInfo, cachedObject);
         }
     }
 
