@@ -15,14 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: CharacterSet.java 1338605 2012-05-15 09:07:02Z mehdi $ */
+/* $Id: CharacterSet.java 1694450 2015-08-06 10:59:51Z ssteiner $ */
 
 package org.apache.fop.afp.fonts;
 
+import java.awt.Rectangle;
 import java.io.UnsupportedEncodingException;
 import java.nio.charset.CharacterCodingException;
-import java.util.HashMap;
-import java.util.Map;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -30,7 +29,7 @@ import org.apache.commons.logging.LogFactory;
 import org.apache.fop.afp.AFPConstants;
 import org.apache.fop.afp.AFPEventProducer;
 import org.apache.fop.afp.fonts.CharactersetEncoder.EncodedChars;
-import org.apache.fop.afp.util.ResourceAccessor;
+import org.apache.fop.afp.util.AFPResourceAccessor;
 import org.apache.fop.afp.util.StringUtils;
 
 /**
@@ -39,16 +38,16 @@ import org.apache.fop.afp.util.StringUtils;
  * font description information for identifying the characters, font metric
  * information for positioning the characters, and character shape information
  * for presenting the character images.
- * <p/>
+ * <br>
  * Presenting a graphic character on a presentation surface requires
  * information on the rotation and position of character on the physical
  * or logical page.
- * <p/>
+ * <br>
  * This class proivdes font metric information for a particular font
  * as identified by the character set name. This information is obtained
  * directly from the AFP font files which must be installed in the path
  * specified in the afp-fonts xml definition file.
- * <p/>
+ * <br>
  */
 public class CharacterSet {
 
@@ -63,6 +62,8 @@ public class CharacterSet {
 
     private static final int MAX_NAME_LEN = 8;
 
+    /** The current orientation (currently only 0 is supported by FOP) */
+    public static final int SUPPORTED_ORIENTATION = 0;
 
     /** The code page to which the character set relates */
     protected final String codePage;
@@ -77,13 +78,10 @@ public class CharacterSet {
     protected final String name;
 
     /** The path to the installed fonts */
-    private final ResourceAccessor accessor;
-
-    /** The current orientation (currently only 0 is supported by FOP) */
-    private final String currentOrientation = "0";
+    private final AFPResourceAccessor accessor;
 
     /** The collection of objects for each orientation */
-    private final Map<String, CharacterSetOrientation> characterSetOrientations;
+    private CharacterSetOrientation characterSetOrientation;
 
     /** The nominal vertical size (in millipoints) for bitmap fonts. 0 for outline fonts. */
     private int nominalVerticalSize;
@@ -100,7 +98,7 @@ public class CharacterSet {
      * @param eventProducer for handling AFP related events
      */
     CharacterSet(String codePage, String encoding, CharacterSetType charsetType, String name,
-            ResourceAccessor accessor, AFPEventProducer eventProducer) {
+            AFPResourceAccessor accessor, AFPEventProducer eventProducer) {
         if (name.length() > MAX_NAME_LEN) {
             String msg = "Character set name '" + name + "' must be a maximum of "
                 + MAX_NAME_LEN + " characters";
@@ -108,26 +106,34 @@ public class CharacterSet {
             throw new IllegalArgumentException(msg);
         }
 
-        if (name.length() < MAX_NAME_LEN) {
-            this.name = StringUtils.rpad(name, ' ', MAX_NAME_LEN);
+        // the character set name must be 8 chars long
+        this.name = padName(name);
+        if (codePage == null) {
+            this.codePage = null;
         } else {
-            this.name = name;
+            // the code page name must be 8 chars long
+            this.codePage = padName(codePage);
         }
-        this.codePage = codePage;
-        this.encoding = encoding;
-        this.encoder = CharactersetEncoder.newInstance(encoding, charsetType);
-        this.accessor = accessor;
 
-        this.characterSetOrientations = new HashMap<String, CharacterSetOrientation>(4);
+        this.encoding = encoding;
+        this.encoder = charsetType.getEncoder(encoding);
+        this.accessor = accessor;
     }
 
+     // right pad short names with space
+     private String padName(String name) {
+         return name.length() < MAX_NAME_LEN ? StringUtils.rpad(name, ' ', MAX_NAME_LEN) : name;
+     }
+
     /**
      * Add character set metric information for the different orientations
      *
      * @param cso the metrics for the orientation
      */
     public void addCharacterSetOrientation(CharacterSetOrientation cso) {
-        characterSetOrientations.put(String.valueOf(cso.getOrientation()), cso);
+        if (cso.getOrientation() == SUPPORTED_ORIENTATION) {
+            characterSetOrientation = cso;
+        }
     }
 
     /**
@@ -161,10 +167,27 @@ public class CharacterSet {
      * @return the ascender value in millipoints
      */
     public int getAscender() {
-
         return getCharacterSetOrientation().getAscender();
     }
 
+    /**
+     * Return the width to use for an underscore (_) character.
+     *
+     * @return  the width of an underscore character
+     */
+    public int getUnderscoreWidth() {
+        return getCharacterSetOrientation().getUnderscoreWidth();
+    }
+
+    /**
+     * Return the position for an underscore (_) character.
+     *
+     * @return the position of an underscore character
+     */
+    public int getUnderscorePosition() {
+        return getCharacterSetOrientation().getUnderscorePosition();
+    }
+
     /**
      * Cap height is the average height of the uppercase characters in
      * a font. This value is specified by the designer of a font and is
@@ -173,7 +196,6 @@ public class CharacterSet {
      * @return the cap height value in millipoints
      */
     public int getCapHeight() {
-
         return getCharacterSetOrientation().getCapHeight();
     }
 
@@ -189,42 +211,14 @@ public class CharacterSet {
         return getCharacterSetOrientation().getDescender();
     }
 
-    /**
-     * Returns the first character in the character set
-     *
-     * @return the first character in the character set (Unicode codepoint)
-     */
-    public char getFirstChar() {
-        return getCharacterSetOrientation().getFirstChar();
-    }
-
-    /**
-     * Returns the last character in the character set
-     *
-     * @return the last character in the character set (Unicode codepoint)
-     */
-    public char getLastChar() {
-        return getCharacterSetOrientation().getLastChar();
-    }
-
     /**
      * Returns the resource accessor to load the font resources with.
      * @return the resource accessor to load the font resources with
      */
-    public ResourceAccessor getResourceAccessor() {
+    public AFPResourceAccessor getResourceAccessor() {
         return this.accessor;
     }
 
-    /**
-     * Get the width (in 1/1000ths of a point size) of all characters
-     *
-     * @return the widths of all characters
-     */
-    public int[] getWidths() {
-
-        return getCharacterSetOrientation().getWidths();
-    }
-
     /**
      * XHeight refers to the height of the lower case letters above the baseline.
      *
@@ -240,13 +234,16 @@ public class CharacterSet {
      * identified by the parameter passed.
      *
      * @param character the Unicode character from which the width will be calculated
+     * @param size  the font size
      * @return the width of the character
      */
-    public int getWidth(char character) {
-        return getCharacterSetOrientation().getWidth(character);
+    public int getWidth(char character, int size) {
+        return getCharacterSetOrientation().getWidth(character, size);
     }
 
-
+    public Rectangle getCharacterBox(char character, int size) {
+        return getCharacterSetOrientation().getCharacterBox(character, size);
+    }
 
     /**
      * Returns the AFP character set identifier
@@ -267,6 +264,7 @@ public class CharacterSet {
         try {
             nameBytes = name.getBytes(AFPConstants.EBCIDIC_ENCODING);
         } catch (UnsupportedEncodingException usee) {
+            // @SuppressFBWarnings("DM_DEFAULT_ENCODING")
             nameBytes = name.getBytes();
             LOG.warn(
                 "UnsupportedEncodingException translating the name " + name);
@@ -305,9 +303,7 @@ public class CharacterSet {
      * @return characterSetOrentation The current orientation metrics.
      */
     private CharacterSetOrientation getCharacterSetOrientation() {
-        CharacterSetOrientation c
-            = characterSetOrientations.get(currentOrientation);
-        return c;
+        return characterSetOrientation;
     }
 
     /**
@@ -363,4 +359,12 @@ public class CharacterSet {
         return getCharacterSetOrientation().getEmSpaceIncrement();
     }
 
+    /**
+     * Returns the nominal character increment.
+     * @return the nominal character increment
+     */
+    public int getNominalCharIncrement() {
+        return getCharacterSetOrientation().getNominalCharIncrement();
+    }
+
 }
