@@ -15,15 +15,19 @@
  * limitations under the License.
  */
 
-/* $Id: CharacterSetBuilder.java 946585 2010-05-20 09:52:27Z jeremias $ */
+/* $Id: CharacterSetBuilder.java 1694450 2015-08-06 10:59:51Z ssteiner $ */
 
 package org.apache.fop.afp.fonts;
 
-import java.io.FileNotFoundException;
+import java.awt.Rectangle;
 import java.io.IOException;
 import java.io.InputStream;
+import java.net.MalformedURLException;
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.WeakHashMap;
@@ -34,15 +38,17 @@ import org.apache.commons.logging.LogFactory;
 import org.apache.xmlgraphics.image.loader.util.SoftMapCache;
 
 import org.apache.fop.afp.AFPConstants;
-import org.apache.fop.afp.util.ResourceAccessor;
+import org.apache.fop.afp.AFPEventProducer;
+import org.apache.fop.afp.util.AFPResourceAccessor;
 import org.apache.fop.afp.util.StructuredFieldReader;
+import org.apache.fop.apps.io.InternalResourceResolver;
 import org.apache.fop.fonts.Typeface;
 
 /**
  * The CharacterSetBuilder is responsible building the a CharacterSet instance that holds
- *  the font metric data.  The data is either read from disk and passed to a CharacterSet (*)
- *  or a FopCharacterSet is instantiated that is composed of a Typeface instance configured
- *  with this data.<p/>
+ * the font metric data.  The data is either read from disk and passed to a CharacterSet (*)
+ * or a FopCharacterSet is instantiated that is composed of a Typeface instance configured
+ * with this data.<br>
  * -*- For referenced fonts CharacterSetBuilder is responsible for reading the font attributes
  * from binary code page files and the character set metric files. In IBM font structure, a
  * code page maps each character of text to the characters in a character set.
@@ -52,24 +58,18 @@ import org.apache.fop.fonts.Typeface;
  * outline pattern) of the character in the character set specified. The image
  * in the character set is the image that is printed in the document. To be a
  * valid code page for a particular character set, all character IDs in the code
- * page must be included in that character set. <p/>This class will read the
- * font information from the binary code page files and character set metric
- * files in order to determine the correct metrics to use when rendering the
- * formatted object. <p/>
- *
+ * page must be included in that character set.<br>
+ * This class will read the font information from the binary code page files and character
+ * set metric files in order to determine the correct metrics to use when rendering the
+ * formatted object.
  */
-public class CharacterSetBuilder {
+public abstract class CharacterSetBuilder {
 
     /**
      * Static logging instance
      */
     protected static final Log LOG = LogFactory.getLog(CharacterSetBuilder.class);
 
-    /**
-     * Singleton reference
-     */
-    private static CharacterSetBuilder instance;
-
     /**
      * Template used to convert lists to arrays.
      */
@@ -106,26 +106,24 @@ public class CharacterSetBuilder {
     /**
      * The collection of code pages
      */
-    private final Map/*<String, Map<String, String>>*/ codePagesCache
-            = new WeakHashMap/*<String, Map<String, String>>*/();
+    private final Map<String, Map<String, String>> codePagesCache
+            = Collections.synchronizedMap(new WeakHashMap<String, Map<String, String>>());
 
     /**
      * Cache of charactersets
      */
     private final SoftMapCache characterSetsCache = new SoftMapCache(true);
 
-
-    private CharacterSetBuilder() { }
+    /** Default constructor. */
+    private CharacterSetBuilder() {
+    }
 
     /**
      * Factory method for the single-byte implementation of AFPFontReader.
      * @return AFPFontReader
      */
-    public static CharacterSetBuilder getInstance() {
-        if (instance == null) {
-            instance = new CharacterSetBuilder();
-        }
-        return instance;
+    public static CharacterSetBuilder getSingleByteInstance() {
+        return SingleByteLoader.getInstance();
     }
 
     /**
@@ -133,34 +131,32 @@ public class CharacterSetBuilder {
      * @return AFPFontReader
      */
     public static CharacterSetBuilder getDoubleByteInstance() {
-        return new DoubleByteLoader();
+        return DoubleByteLoader.getInstance();
     }
 
 
     /**
      * Returns an InputStream to a given file path and filename
      *
-     * * @param accessor the resource accessor
-     * @param filename the file name
+     * @param accessor the resource accessor
+     * @param uriStr the URI
+     * @param eventProducer for handling AFP related events
      * @return an inputStream
-     *
      * @throws IOException in the event that an I/O exception of some sort has occurred
      */
-    protected InputStream openInputStream(ResourceAccessor accessor, String filename)
+    private InputStream openInputStream(AFPResourceAccessor accessor, String uriStr,
+            AFPEventProducer eventProducer)
             throws IOException {
         URI uri;
         try {
-            uri = new URI(filename.trim());
+            uri = InternalResourceResolver.cleanURI(uriStr.trim());
         } catch (URISyntaxException e) {
-            throw new FileNotFoundException("Invalid filename: "
-                    + filename + " (" + e.getMessage() + ")");
+            throw new MalformedURLException("Invalid uri: " + uriStr + " (" + e.getMessage() + ")");
         }
-
         if (LOG.isDebugEnabled()) {
             LOG.debug("Opening " + uri);
         }
-        InputStream inputStream = accessor.createInputStream(uri);
-        return inputStream;
+        return accessor.createInputStream(uri);
     }
 
     /**
@@ -168,7 +164,7 @@ public class CharacterSetBuilder {
      *
      * @param inputStream the inputstream to close
      */
-    protected void closeInputStream(InputStream inputStream) {
+    private void closeInputStream(InputStream inputStream) {
         try {
             if (inputStream != null) {
                 inputStream.close();
@@ -180,30 +176,82 @@ public class CharacterSetBuilder {
     }
 
     /**
-     * Load the font details and metrics into the CharacterSetMetric object,
-     * this will use the actual afp code page and character set files to load
-     * the object with the necessary metrics.
+     * Load the font details and metrics into the CharacterSetMetric object, this will use the
+     * actual afp code page and character set files to load the object with the necessary metrics.
+     *
      * @param characterSetName name of the characterset
      * @param codePageName name of the code page file
      * @param encoding encoding name
      * @param accessor used to load codepage and characterset
+     * @param eventProducer for handling AFP related events
      * @return CharacterSet object
      * @throws IOException if an I/O error occurs
      */
-    public CharacterSet build(String characterSetName, String codePageName,
-            String encoding, ResourceAccessor accessor) throws IOException {
+    public CharacterSet buildSBCS(String characterSetName, String codePageName, String encoding,
+            AFPResourceAccessor accessor, AFPEventProducer eventProducer) throws IOException {
+        return processFont(characterSetName, codePageName, encoding, CharacterSetType.SINGLE_BYTE,
+                accessor, eventProducer);
+    }
 
+    /**
+     * Load the font details and metrics into the CharacterSetMetric object, this will use the
+     * actual afp code page and character set files to load the object with the necessary metrics.
+     * This method is to be used for double byte character sets (DBCS).
+     *
+     * @param characterSetName name of the characterset
+     * @param codePageName name of the code page file
+     * @param encoding encoding name
+     * @param charsetType the characterset type
+     * @param accessor used to load codepage and characterset
+     * @param eventProducer for handling AFP related events
+     * @return CharacterSet object
+     * @throws IOException if an I/O error occurs
+     */
+    public CharacterSet buildDBCS(String characterSetName, String codePageName, String encoding,
+            CharacterSetType charsetType, AFPResourceAccessor accessor, AFPEventProducer eventProducer)
+            throws IOException {
+        return processFont(characterSetName, codePageName, encoding, charsetType, accessor,
+                eventProducer);
+    }
+
+    /**
+     * Load the font details and metrics into the CharacterSetMetric object, this will use the
+     * actual afp code page and character set files to load the object with the necessary metrics.
+     *
+     * @param characterSetName the CharacterSetMetric object to populate
+     * @param codePageName the name of the code page to use
+     * @param encoding name of the encoding in use
+     * @param typeface base14 font name
+     * @param eventProducer for handling AFP related events
+     * @return CharacterSet object
+     * @throws IOException if an I/O error occurs
+     */
+    public CharacterSet build(String characterSetName, String codePageName, String encoding,
+            Typeface typeface, AFPEventProducer eventProducer) throws IOException {
+        return new FopCharacterSet(codePageName, encoding, characterSetName, typeface,
+                eventProducer);
+    }
+
+    public CharacterSet build(String characterSetName, String codePageName, String encoding,
+                              Typeface typeface, AFPResourceAccessor accessor, AFPEventProducer eventProducer)
+        throws IOException {
+        return new FopCharacterSet(codePageName, encoding, characterSetName, typeface, accessor, eventProducer);
+    }
+
+    private CharacterSet processFont(String characterSetName, String codePageName, String encoding,
+            CharacterSetType charsetType, AFPResourceAccessor accessor, AFPEventProducer eventProducer)
+            throws IOException {
         // check for cached version of the characterset
-        String descriptor = characterSetName + "_" + encoding + "_" + codePageName;
-        CharacterSet characterSet = (CharacterSet)characterSetsCache.get(descriptor);
-
+        URI charSetURI = accessor.resolveURI(characterSetName);
+        String cacheKey = charSetURI.toASCIIString() + "_" + characterSetName + "_" + codePageName;
+        CharacterSet characterSet = (CharacterSet) characterSetsCache.get(cacheKey);
         if (characterSet != null) {
             return characterSet;
         }
 
         // characterset not in the cache, so recreating
-        characterSet = new CharacterSet(
-                codePageName, encoding, characterSetName, accessor);
+        characterSet = new CharacterSet(codePageName, encoding, charsetType, characterSetName,
+                accessor, eventProducer);
 
         InputStream inputStream = null;
 
@@ -214,16 +262,19 @@ public class CharacterSetBuilder {
              * information to map the unicode character id to the graphic
              * chracter global identifier.
              */
+            Map<String, String> codePage;
+            // TODO: This could have performance implications if several threads want to use the
+            // codePagesCache to retrieve different codepages.
+            synchronized (codePagesCache) {
+                codePage = codePagesCache.get(codePageName);
 
-            Map/*<String,String>*/ codePage
-                = (Map/*<String,String>*/)codePagesCache.get(codePageName);
-
-            if (codePage == null) {
-                codePage = loadCodePage(codePageName, encoding, accessor);
-                codePagesCache.put(codePageName, codePage);
+                if (codePage == null) {
+                    codePage = loadCodePage(codePageName, encoding, accessor, eventProducer);
+                    codePagesCache.put(codePageName, codePage);
+                }
             }
 
-            inputStream = openInputStream(accessor, characterSetName);
+            inputStream = openInputStream(accessor, characterSetName, eventProducer);
 
             StructuredFieldReader structuredFieldReader = new StructuredFieldReader(inputStream);
 
@@ -239,24 +290,22 @@ public class CharacterSetBuilder {
                 CharacterSetOrientation[] characterSetOrientations
                     = processFontOrientation(structuredFieldReader);
 
-                int metricNormalizationFactor;
+                double metricNormalizationFactor;
                 if (fontControl.isRelative()) {
                     metricNormalizationFactor = 1;
                 } else {
                     int dpi = fontControl.getDpi();
-                    metricNormalizationFactor = 1000 * 72000
+                    metricNormalizationFactor = 1000.0d * 72000.0d
                         / fontDescriptor.getNominalFontSizeInMillipoints() / dpi;
                 }
-
+                ValueNormalizer normalizer = new ValueNormalizer(metricNormalizationFactor);
                 //process D3AC89 Font Position
-                processFontPosition(structuredFieldReader, characterSetOrientations,
-                        metricNormalizationFactor);
-
+                processFontPosition(structuredFieldReader, characterSetOrientations, normalizer);
                 //process D38C89 Font Index (per orientation)
                 for (int i = 0; i < characterSetOrientations.length; i++) {
-                    processFontIndex(structuredFieldReader,
-                            characterSetOrientations[i], codePage, metricNormalizationFactor);
-                    characterSet.addCharacterSetOrientation(characterSetOrientations[i]);
+                    CharacterSetOrientation characterSetOrientation = characterSetOrientations[i];
+                    processFontIndex(structuredFieldReader, characterSetOrientation, codePage, normalizer);
+                    characterSet.addCharacterSetOrientation(characterSetOrientation);
                 }
             } else {
                 throw new IOException("Missing D3AE89 Font Control structured field.");
@@ -265,25 +314,21 @@ public class CharacterSetBuilder {
         } finally {
             closeInputStream(inputStream);
         }
-        characterSetsCache.put(descriptor, characterSet);
+        characterSetsCache.put(cacheKey, characterSet);
         return characterSet;
-
     }
 
-    /**
-     * Load the font details and metrics into the CharacterSetMetric object,
-     * this will use the actual afp code page and character set files to load
-     * the object with the necessary metrics.
-     *
-     * @param characterSetName the CharacterSetMetric object to populate
-     * @param codePageName the name of the code page to use
-     * @param encoding name of the encoding in use
-     * @param typeface base14 font name
-     * @return CharacterSet object
-     */
-    public CharacterSet build(String characterSetName, String codePageName,
-            String encoding, Typeface typeface) {
-       return new FopCharacterSet(codePageName, encoding, characterSetName, typeface);
+    private static class ValueNormalizer {
+
+        private final double factor;
+
+        public ValueNormalizer(double factor) {
+            this.factor = factor;
+        }
+
+        public int normalize(int value) {
+            return (int) Math.round(value *  factor);
+        }
     }
 
     /**
@@ -295,19 +340,24 @@ public class CharacterSetBuilder {
      * @param encoding
      *            the encoding to use for the character decoding
      * @param accessor the resource accessor
+     * @param eventProducer for handling AFP related events
      * @return a code page mapping (key: GCGID, value: Unicode character)
      * @throws IOException if an I/O exception of some sort has occurred.
      */
-    protected Map/*<String,String>*/ loadCodePage(String codePage, String encoding,
-        ResourceAccessor accessor) throws IOException {
+    protected Map<String, String> loadCodePage(String codePage, String encoding,
+            AFPResourceAccessor accessor, AFPEventProducer eventProducer) throws IOException {
 
         // Create the HashMap to store code page information
-        Map/*<String,String>*/ codePages = new java.util.HashMap/*<String,String>*/();
+        Map<String, String> codePages = new HashMap<String, String>();
 
         InputStream inputStream = null;
         try {
-            inputStream = openInputStream(accessor, codePage.trim());
-
+            inputStream = openInputStream(accessor, codePage.trim(), eventProducer);
+        } catch (IOException e) {
+            eventProducer.codePageNotFound(this, e);
+            throw e;
+        }
+        try {
             StructuredFieldReader structuredFieldReader = new StructuredFieldReader(inputStream);
             byte[] data = structuredFieldReader.getNext(CHARACTER_TABLE_SF);
 
@@ -351,9 +401,8 @@ public class CharacterSetBuilder {
      * @return a class representing the font descriptor
      * @throws IOException if an I/O exception of some sort has occurred.
      */
-    protected static FontDescriptor processFontDescriptor(
-                StructuredFieldReader structuredFieldReader)
-    throws IOException {
+    private static FontDescriptor processFontDescriptor(
+            StructuredFieldReader structuredFieldReader) throws IOException {
 
         byte[] fndData = structuredFieldReader.getNext(FONT_DESCRIPTOR_SF);
         return new FontDescriptor(fndData);
@@ -367,8 +416,8 @@ public class CharacterSetBuilder {
      * @return the FontControl
      * @throws IOException if an I/O exception of some sort has occurred.
      */
-    protected FontControl processFontControl(StructuredFieldReader structuredFieldReader)
-    throws IOException {
+    private FontControl processFontControl(StructuredFieldReader structuredFieldReader)
+            throws IOException {
 
         byte[] fncData = structuredFieldReader.getNext(FONT_CONTROL_SF);
 
@@ -399,7 +448,7 @@ public class CharacterSetBuilder {
      * @return CharacterSetOrientation array
      * @throws IOException if an I/O exception of some sort has occurred.
      */
-    protected CharacterSetOrientation[] processFontOrientation(
+    private CharacterSetOrientation[] processFontOrientation(
         StructuredFieldReader structuredFieldReader) throws IOException {
 
         byte[] data = structuredFieldReader.getNext(FONT_ORIENTATION_SF);
@@ -407,7 +456,7 @@ public class CharacterSetBuilder {
         int position = 0;
         byte[] fnoData = new byte[26];
 
-        List orientations = new java.util.ArrayList();
+        List<CharacterSetOrientation> orientations = new ArrayList<CharacterSetOrientation>();
 
         // Read data, ignoring bytes 0 - 2
         for (int index = 3; index < data.length; index++) {
@@ -416,25 +465,18 @@ public class CharacterSetBuilder {
             position++;
 
             if (position == 26) {
-
                 position = 0;
 
                 int orientation = determineOrientation(fnoData[2]);
-                //  Space Increment
-                int space = ((fnoData[8] & 0xFF ) << 8) + (fnoData[9] & 0xFF);
-                //  Em-Space Increment
-                int em = ((fnoData[14] & 0xFF ) << 8) + (fnoData[15] & 0xFF);
-
-                CharacterSetOrientation cso = new CharacterSetOrientation(orientation);
-                cso.setSpaceIncrement(space);
-                cso.setEmSpaceIncrement(em);
-                orientations.add(cso);
+                int spaceIncrement = getUBIN(fnoData, 8);
+                int emIncrement = getUBIN(fnoData, 14);
+                int nominalCharacterIncrement = getUBIN(fnoData, 20);
 
+                orientations.add(new CharacterSetOrientation(orientation, spaceIncrement,
+                        emIncrement, nominalCharacterIncrement));
             }
         }
-
-        return (CharacterSetOrientation[]) orientations
-            .toArray(EMPTY_CSO_ARRAY);
+        return orientations.toArray(EMPTY_CSO_ARRAY);
     }
 
     /**
@@ -449,8 +491,8 @@ public class CharacterSetBuilder {
      *                  font metric values
      * @throws IOException if an I/O exception of some sort has occurred.
      */
-    protected void processFontPosition(StructuredFieldReader structuredFieldReader,
-        CharacterSetOrientation[] characterSetOrientations, double metricNormalizationFactor)
+    private void processFontPosition(StructuredFieldReader structuredFieldReader,
+        CharacterSetOrientation[] characterSetOrientations, ValueNormalizer normalizer)
             throws IOException {
 
         byte[] data = structuredFieldReader.getNext(FONT_POSITION_SF);
@@ -468,48 +510,34 @@ public class CharacterSetBuilder {
                 if (position == 9) {
                     CharacterSetOrientation characterSetOrientation
                             = characterSetOrientations[characterSetOrientationIndex];
-
                     int xHeight = getSBIN(fpData, 2);
                     int capHeight = getSBIN(fpData, 4);
                     int ascHeight = getSBIN(fpData, 6);
                     int dscHeight = getSBIN(fpData, 8);
-
                     dscHeight = dscHeight * -1;
-
-                    characterSetOrientation.setXHeight(
-                            (int)Math.round(xHeight * metricNormalizationFactor));
-                    characterSetOrientation.setCapHeight(
-                            (int)Math.round(capHeight * metricNormalizationFactor));
-                    characterSetOrientation.setAscender(
-                            (int)Math.round(ascHeight * metricNormalizationFactor));
-                    characterSetOrientation.setDescender(
-                            (int)Math.round(dscHeight * metricNormalizationFactor));
+                    int underscoreWidth = getUBIN(fpData, 17);
+                    int underscorePosition = getSBIN(fpData, 20);
+                    characterSetOrientation.setXHeight(normalizer.normalize(xHeight));
+                    characterSetOrientation.setCapHeight(normalizer.normalize(capHeight));
+                    characterSetOrientation.setAscender(normalizer.normalize(ascHeight));
+                    characterSetOrientation.setDescender(normalizer.normalize(dscHeight));
+                    characterSetOrientation.setUnderscoreWidth(normalizer.normalize(underscoreWidth));
+                    characterSetOrientation.setUnderscorePosition(normalizer.normalize(underscorePosition));
                 }
             } else if (position == 22) {
                 position = 0;
                 characterSetOrientationIndex++;
                 fpData[position] = data[index];
             }
-
             position++;
         }
 
     }
 
-    /**
-     * Process the font index details for the character set orientation.
-     *
-     * @param structuredFieldReader the structured field reader
-     * @param cso the CharacterSetOrientation object to populate
-     * @param codepage the map of code pages
-     * @param metricNormalizationFactor factor to apply to the metrics to get normalized
-     *                  font metric values
-     * @throws IOException if an I/O exception of some sort has occurred.
-     */
-    protected void processFontIndex(StructuredFieldReader structuredFieldReader,
-        CharacterSetOrientation cso, Map/*<String,String>*/ codepage,
-        double metricNormalizationFactor)
-        throws IOException {
+
+    private void processFontIndex(StructuredFieldReader structuredFieldReader, CharacterSetOrientation cso,
+            Map<String, String> codepage, ValueNormalizer normalizer)
+            throws IOException {
 
         byte[] data = structuredFieldReader.getNext(FONT_INDEX_SF);
 
@@ -518,8 +546,6 @@ public class CharacterSetBuilder {
         byte[] gcgid = new byte[8];
         byte[] fiData = new byte[20];
 
-        char lowest = 255;
-        char highest = 0;
         String firstABCMismatch = null;
 
         // Read data, ignoring bytes 0 - 2
@@ -538,19 +564,21 @@ public class CharacterSetBuilder {
 
                 String gcgiString = new String(gcgid, AFPConstants.EBCIDIC_ENCODING);
 
-                String idx = (String) codepage.get(gcgiString);
+                String idx = codepage.get(gcgiString);
 
                 if (idx != null) {
 
                     char cidx = idx.charAt(0);
                     int width = getUBIN(fiData, 0);
+                    int ascendHt = getSBIN(fiData, 2);
+                    int descendDp = getSBIN(fiData, 4);
                     int a = getSBIN(fiData, 10);
                     int b = getUBIN(fiData, 12);
                     int c = getSBIN(fiData, 14);
                     int abc = a + b + c;
                     int diff = Math.abs(abc - width);
                     if (diff != 0 && width != 0) {
-                        double diffPercent = 100 * diff / (double)width;
+                        double diffPercent = 100 * diff / (double) width;
                         if (diffPercent > 2) {
                             if (LOG.isTraceEnabled()) {
                                 LOG.trace(gcgiString + ": "
@@ -562,27 +590,16 @@ public class CharacterSetBuilder {
                             }
                         }
                     }
-
-                    if (cidx < lowest) {
-                        lowest = cidx;
-                    }
-
-                    if (cidx > highest) {
-                        highest = cidx;
-                    }
-
-                    int normalizedWidth = (int)Math.round(width * metricNormalizationFactor);
-
-                    cso.setWidth(cidx, normalizedWidth);
-
+                    int normalizedWidth = normalizer.normalize(width);
+                    int x0 = normalizer.normalize(a);
+                    int y0 = normalizer.normalize(-descendDp);
+                    int dx = normalizer.normalize(b);
+                    int dy = normalizer.normalize(ascendHt + descendDp);
+                    cso.setCharacterMetrics(cidx, normalizedWidth, new Rectangle(x0, y0, dx, dy));
                 }
-
             }
         }
 
-        cso.setFirstChar(lowest);
-        cso.setLastChar(highest);
-
         if (LOG.isDebugEnabled() && firstABCMismatch != null) {
             //Debug level because it usually is no problem.
             LOG.debug("Font has metrics inconsitencies where A+B+C doesn't equal the"
@@ -610,7 +627,7 @@ public class CharacterSetBuilder {
         private int dpi;
         private int unitsPerEm;
 
-        private boolean isRelative = false;
+        private boolean isRelative;
 
         public int getDpi() {
             return dpi;
@@ -651,27 +668,50 @@ public class CharacterSetBuilder {
         }
     }
 
+    private static final class SingleByteLoader extends CharacterSetBuilder {
+
+        private static final SingleByteLoader INSTANCE = new SingleByteLoader();
+
+        private SingleByteLoader() {
+            super();
+        }
+
+        private static SingleByteLoader getInstance() {
+            return INSTANCE;
+        }
+    }
+
     /**
      * Double-byte (CID Keyed font (Type 0)) implementation of AFPFontReader.
      */
-    private static class DoubleByteLoader extends CharacterSetBuilder {
+    private static final class DoubleByteLoader extends CharacterSetBuilder {
 
-        protected Map/*<String,String>*/ loadCodePage(String codePage, String encoding,
-                ResourceAccessor accessor) throws IOException {
+        private static final DoubleByteLoader INSTANCE = new DoubleByteLoader();
 
+        private DoubleByteLoader() {
+        }
+
+        static DoubleByteLoader getInstance() {
+            return INSTANCE;
+        }
+
+        @Override
+        protected Map<String, String> loadCodePage(String codePage, String encoding,
+                AFPResourceAccessor accessor, AFPEventProducer eventProducer) throws IOException {
             // Create the HashMap to store code page information
-            Map/*<String,String>*/ codePages = new java.util.HashMap/*<String,String>*/();
-
+            Map<String, String> codePages = new HashMap<String, String>();
             InputStream inputStream = null;
             try {
-                inputStream = openInputStream(accessor, codePage.trim());
-
-                StructuredFieldReader structuredFieldReader
-                    = new StructuredFieldReader(inputStream);
+                inputStream = super.openInputStream(accessor, codePage.trim(), eventProducer);
+            } catch (IOException e) {
+                eventProducer.codePageNotFound(this, e);
+                throw e;
+            }
+            try {
+                StructuredFieldReader structuredFieldReader = new StructuredFieldReader(inputStream);
                 byte[] data;
                 while ((data = structuredFieldReader.getNext(CHARACTER_TABLE_SF)) != null) {
                     int position = 0;
-
                     byte[] gcgiBytes = new byte[8];
                     byte[] charBytes = new byte[2];
                     // Read data, ignoring bytes 0 - 2
@@ -694,17 +734,14 @@ public class CharacterSetBuilder {
                                     AFPConstants.EBCIDIC_ENCODING);
                             String charString = new String(charBytes, encoding);
                             codePages.put(gcgiString, charString);
-
-                        }
-                        else {
+                        } else {
                             position++;
                         }
                     }
                 }
             } finally {
-                closeInputStream(inputStream);
+                super.closeInputStream(inputStream);
             }
-
             return codePages;
         }
 
