@@ -15,30 +15,29 @@
  * limitations under the License.
  */
 
-/* $Id: CharacterSetOrientation.java 1169780 2011-09-12 14:53:34Z vhennebert $ */
+/* $Id: CharacterSetOrientation.java 1679676 2015-05-16 02:10:42Z adelmelle $ */
 
 package org.apache.fop.afp.fonts;
 
-import java.util.Arrays;
+import java.awt.Rectangle;
 
 /**
  * The IBM Font Object Content Architecture (FOCA) supports presentation
  * of character shapes by defining their characteristics, which include
  * Font-Description information for identifying the characters, Font-Metric
  * information for positioning the characters, and Character-Shape
- * information for presenting the character images.
+ * information for presenting the character images.<br>
  *
  * Presenting a graphic character on a presentation surface requires
  * that you communicate this information clearly to rotate and position
- * characters correctly on the physical or logical page.
+ * characters correctly on the physical or logical page.<br>
  *
  * This class provides font metric information for a particular font
- * as by the orientation.
+ * as by the orientation.<br>
  *
  * This information is obtained directly from the AFP font files which must
  * be installed in the classpath under in the location specified by the path
  * attribute in the afp-font.xml file.
- * <p/>
  */
 public class CharacterSetOrientation {
 
@@ -60,44 +59,42 @@ public class CharacterSetOrientation {
     /**
      * The character widths in the character set (indexed using Unicode codepoints)
      */
-    private int[] charsWidths = null;
+    private IntegerKeyStore<CharacterMetrics> characterMetrics;
 
     /**
      * The height of lowercase letters
      */
     private int xHeight;
 
-    /**
-     * The first character (Unicode codepoint)
-     */
-    private char firstChar;
-
-    /**
-     * The last character (Unicode codepoint)
-     */
-    private char lastChar;
-
-
-    /**
-     * The character set orientation
-     */
-    private int orientation = 0;
-
+    /** The character set orientation */
+    private final int orientation;
     /** space increment */
-    private int spaceIncrement;
+    private final int spaceIncrement;
     /** em space increment */
-    private int emSpaceIncrement = -1;
+    private final int emSpaceIncrement;
+    /** Nominal Character Increment */
+    private final int nomCharIncrement;
 
+    private int underscoreWidth;
+
+    private int underscorePosition;
 
     /**
      * Constructor for the CharacterSetOrientation, the orientation is
      * expressed as the degrees rotation (i.e 0, 90, 180, 270)
-     * @param orientation the character set orientation
+     *
+     * @param orientation   the character set orientation
+     * @param spaceIncrement    the space increment
+     * @param emSpaceIncrement  the em space increment
+     * @param nomCharIncrement  the nominal character increment
      */
-    public CharacterSetOrientation(int orientation) {
+    public CharacterSetOrientation(int orientation, int spaceIncrement, int emSpaceIncrement,
+            int nomCharIncrement) {
         this.orientation = orientation;
-        charsWidths = new int[256];
-        Arrays.fill(charsWidths, -1);
+        this.spaceIncrement = spaceIncrement;
+        this.emSpaceIncrement = emSpaceIncrement;
+        this.nomCharIncrement = nomCharIncrement;
+        this.characterMetrics = new IntegerKeyStore<CharacterMetrics>();
     }
 
     /**
@@ -137,19 +134,21 @@ public class CharacterSetOrientation {
     }
 
     /**
-     * The first character in the character set
-     * @return the first character (Unicode codepoint)
+     * {@asf.todo}
+     *
+     * @return  the underscore width
      */
-    public char getFirstChar() {
-        return firstChar;
+    public int getUnderscoreWidth() {
+        return underscoreWidth;
     }
 
     /**
-     * The last character in the character set
-     * @return the last character (Unicode codepoint)
+     * {@asf.todo}
+     *
+     * @return  the underscore position
      */
-    public char getLastChar() {
-        return lastChar;
+    public int getUnderscorePosition() {
+        return underscorePosition;
     }
 
     /**
@@ -160,17 +159,6 @@ public class CharacterSetOrientation {
         return orientation;
     }
 
-    /**
-     * Get the width (in 1/1000ths of a point size) of all characters
-     * in this character set.
-     * @return the widths of all characters
-     */
-    public int[] getWidths() {
-        int[] arr = new int[(getLastChar() - getFirstChar()) + 1];
-        System.arraycopy(charsWidths, getFirstChar(), arr, 0, (getLastChar() - getFirstChar()) + 1);
-        return arr;
-    }
-
     /**
      * XHeight refers to the height of the lower case letters above
      * the baseline.
@@ -184,15 +172,42 @@ public class CharacterSetOrientation {
      * Get the width (in 1/1000ths of a point size) of the character
      * identified by the parameter passed.
      * @param character the Unicode character to evaluate
+     * @param size  the font size
      * @return the widths of the character
      */
-    public int getWidth(char character) {
-        if (character >= charsWidths.length) {
-            throw new IllegalArgumentException("Invalid character: "
-                    + character + " (" + Integer.toString(character)
-                    + "), maximum is " + (charsWidths.length - 1));
+    public int getWidth(char character, int size) {
+        CharacterMetrics cm = getCharacterMetrics(character);
+        return cm == null ? -1 : size * cm.width;
+    }
+
+    private CharacterMetrics getCharacterMetrics(char character) {
+        return characterMetrics.get((int) character);
+    }
+
+    /**
+     * Get the character box (rectangle with dimensions in 1/1000ths of a point size) of the character
+     * identified by the parameter passed.
+     * @param character the Unicode character to evaluate
+     * @param size  the font size
+     * @return the character box
+     */
+    public Rectangle getCharacterBox(char character, int size) {
+        CharacterMetrics cm = getCharacterMetrics(character);
+        return scale(cm == null ? getFallbackCharacterBox() : cm.characterBox, size);
+    }
+
+    private static Rectangle scale(Rectangle rectangle, int size) {
+        if (rectangle == null) {
+            return null;
+        } else {
+        return new Rectangle((int) (size * rectangle.getX()), (int) (size * rectangle.getY()),
+                (int) (size * rectangle.getWidth()), (int) (size * rectangle.getHeight()));
         }
-        return charsWidths[character];
+    }
+
+    private Rectangle getFallbackCharacterBox() {
+        // TODO replace with something sensible
+        return new Rectangle(0, 0, 0, 0);
     }
 
     /**
@@ -232,19 +247,19 @@ public class CharacterSetOrientation {
     }
 
     /**
-     * The first character in the character set
-     * @param firstChar the first character
+     * TODO
+     * @param underscoreWidth the underscore width value in millipoints
      */
-    public void setFirstChar(char firstChar) {
-        this.firstChar = firstChar;
+    public void setUnderscoreWidth(int underscoreWidth) {
+        this.underscoreWidth = underscoreWidth;
     }
 
     /**
-     * The last character in the character set
-     * @param lastChar the last character
+     * TODO
+     * @param underscorePosition the underscore position value in millipoints
      */
-    public void setLastChar(char lastChar) {
-        this.lastChar = lastChar;
+    public void setUnderscorePosition(int underscorePosition) {
+        this.underscorePosition = underscorePosition;
     }
 
     /**
@@ -252,18 +267,10 @@ public class CharacterSetOrientation {
      * identified by the parameter passed.
      * @param character the Unicode character for which the width is being set
      * @param width the widths of the character
+     * @param   characterBox    the character box
      */
-    public void setWidth(char character, int width) {
-        if (character >= charsWidths.length) {
-            // Increase the size of the array if necessary
-            //  TODO Can we remove firstChar? surely firstChar==0 at this stage?
-            int[] arr = new int[(character - firstChar) + 1];
-            System.arraycopy(charsWidths, 0, arr, 0, charsWidths.length);
-            Arrays.fill(arr, charsWidths.length, character - firstChar, -1);
-            charsWidths = arr;
-        }
-        charsWidths[character] = width;
-
+    public void setCharacterMetrics(char character, int width, Rectangle characterBox) {
+        characterMetrics.put((int) character, new CharacterMetrics(width, characterBox));
     }
 
     /**
@@ -283,14 +290,6 @@ public class CharacterSetOrientation {
         return this.spaceIncrement;
     }
 
-    /**
-     * Sets the space increment.
-     * @param value the space increment
-     */
-    public void setSpaceIncrement(int value) {
-        this.spaceIncrement = value;
-    }
-
     /**
      * Returns the em space increment.
      * @return the em space increment
@@ -300,11 +299,22 @@ public class CharacterSetOrientation {
     }
 
     /**
-     * Sets the em space increment.
-     * @param value the em space increment
+     * Returns the nominal character increment.
+     * @return the nominal character increment
      */
-    public void setEmSpaceIncrement(int value) {
-        this.emSpaceIncrement = value;
+    public int getNominalCharIncrement() {
+        return this.nomCharIncrement;
     }
 
+    private static class CharacterMetrics {
+
+        public final int width;
+
+        public final Rectangle characterBox;
+
+        public CharacterMetrics(int width, Rectangle characterBox) {
+            this.width = width;
+            this.characterBox = characterBox;
+        }
+    }
 }
