@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: GraphicsData.java 1297404 2012-03-06 10:17:54Z vhennebert $ */
+/* $Id: GraphicsData.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.afp.goca;
 
@@ -36,7 +36,7 @@ public final class GraphicsData extends AbstractGraphicsDrawingOrderContainer {
     //+16 to avoid unnecessary, practically empty GraphicsData instances.
 
     /** the graphics segment */
-    private GraphicsChainedSegment currentSegment = null;
+    private GraphicsChainedSegment currentSegment;
 
     private boolean segmentedData;
 
