@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractDataObject.java 1195952 2011-11-01 12:20:21Z phancock $ */
+/* $Id: AbstractDataObject.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.afp.modca;
 
@@ -38,7 +38,7 @@ public abstract class AbstractDataObject extends AbstractNamedAFPObject
         implements Startable, Completable {
 
     /** the object environment group */
-    protected ObjectEnvironmentGroup objectEnvironmentGroup = null;
+    protected ObjectEnvironmentGroup objectEnvironmentGroup;
 
     /** the object factory */
     protected final Factory factory;
