@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractNamedAFPObject.java 1151452 2011-07-27 12:50:12Z phancock $ */
+/* $Id: AbstractNamedAFPObject.java 1679676 2015-05-16 02:10:42Z adelmelle $ */
 
 package org.apache.fop.afp.modca;
 
@@ -34,7 +34,7 @@ public abstract class AbstractNamedAFPObject extends AbstractTripletStructuredOb
     /**
      * The actual name of the object
      */
-    protected String name = null;
+    protected String name;
 
     /**
      * Default constructor
@@ -82,6 +82,7 @@ public abstract class AbstractNamedAFPObject extends AbstractTripletStructuredOb
         try {
             nameBytes = name.getBytes(AFPConstants.EBCIDIC_ENCODING);
         } catch (UnsupportedEncodingException usee) {
+            // @SuppressFBWarnings("DM_DEFAULT_ENCODING")
             nameBytes = name.getBytes();
             LOG.warn(
                 "Constructor:: UnsupportedEncodingException translating the name "
