@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractPageObject.java 1296526 2012-03-03 00:18:45Z gadams $ */
+/* $Id: AbstractPageObject.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.afp.modca;
 
@@ -52,10 +52,10 @@ import org.apache.fop.afp.ptoca.PtocaProducer;
 public abstract class AbstractPageObject extends AbstractNamedAFPObject implements Completable {
 
     /** The active environment group for the page */
-    protected ActiveEnvironmentGroup activeEnvironmentGroup = null;
+    protected ActiveEnvironmentGroup activeEnvironmentGroup;
 
     /** The current presentation text object */
-    private PresentationTextObject currentPresentationTextObject = null;
+    private PresentationTextObject currentPresentationTextObject;
 
     /** The list of objects within this resource container */
     protected List/*<AbstractStructuredObject>*/ objects = new java.util.ArrayList();
@@ -67,10 +67,10 @@ public abstract class AbstractPageObject extends AbstractNamedAFPObject implemen
     private int height;
 
     /** The page rotation */
-    protected int rotation = 0;
+    protected int rotation;
 
     /** The page state */
-    protected boolean complete = false;
+    protected boolean complete;
 
     /** The width resolution */
     private int widthRes;
@@ -223,19 +223,15 @@ public abstract class AbstractPageObject extends AbstractNamedAFPObject implemen
     /**
      * Creates a TagLogicalElement on the page.
      *
-     * @param name
-     *            the name of the tag
-     * @param value
-     *            the value of the tag
-     * @param tleID
-     *            unique ID within AFP stream
+     * @param state the state of the TLE
      */
-    public void createTagLogicalElement(String name, String value, int tleID) {
-        TagLogicalElement tle = new TagLogicalElement(name, value, tleID);
+    public void createTagLogicalElement(TagLogicalElement.State state) {
+        TagLogicalElement tle = new TagLogicalElement(state);
         List list = getTagLogicalElements();
         list.add(tle);
     }
 
+
     /**
      * Creates a NoOperation on the page.
      *
