@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractResourceGroupContainer.java 1296418 2012-03-02 19:56:33Z gadams $ */
+/* $Id: AbstractResourceGroupContainer.java 1617052 2014-08-10 06:55:01Z gadams $ */
 
 package org.apache.fop.afp.modca;
 
@@ -26,20 +26,18 @@ import java.util.Iterator;
 
 import org.apache.fop.afp.Completable;
 import org.apache.fop.afp.Factory;
-import org.apache.fop.afp.Streamable;
 
 
 /**
  * An abstract container of resource objects
  */
-public abstract class AbstractResourceGroupContainer extends AbstractPageObject
-implements Streamable {
+public abstract class AbstractResourceGroupContainer extends AbstractPageObject {
 
     /** The container started state */
-    protected boolean started = false;
+    protected boolean started;
 
     /** the resource group object */
-    protected ResourceGroup resourceGroup = null;
+    protected ResourceGroup resourceGroup;
 
     /**
      * Default constructor
