@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: GraphicsObject.java 1339723 2012-05-17 17:22:20Z gadams $ */
+/* $Id: GraphicsObject.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.afp.modca;
 
@@ -62,7 +62,7 @@ import org.apache.fop.afp.goca.GraphicsSetProcessColor;
 public class GraphicsObject extends AbstractDataObject {
 
     /** the graphics data */
-    private GraphicsData currentData = null;
+    private GraphicsData currentData;
 
     /** list of objects contained within this container */
     protected List<GraphicsData> objects
@@ -73,7 +73,7 @@ public class GraphicsObject extends AbstractDataObject {
 
 
     /** color  converter  */
-    private ColorConverter colorConverter = null;
+    private ColorConverter colorConverter;
 
     /**
      * Default constructor
@@ -195,8 +195,8 @@ public class GraphicsObject extends AbstractDataObject {
      * @param lineWidth the line width multiplier
      */
     public void setLineWidth(float lineWidth) {
-        float epsilon = Float.intBitsToFloat ( 0x00800000 ); // Float.MIN_NORMAL (JDK1.6)
-        if ( Math.abs ( graphicsState.lineWidth - lineWidth ) > epsilon ) {
+        float epsilon = Float.intBitsToFloat(0x00800000); // Float.MIN_NORMAL (JDK1.6)
+        if (Math.abs(graphicsState.lineWidth - lineWidth) > epsilon) {
             addObject(new GraphicsSetFractionalLineWidth(lineWidth));
             graphicsState.lineWidth = lineWidth;
         }
