@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: IncludedResourceObject.java 1005350 2010-10-07 07:41:48Z jeremias $ */
+/* $Id: IncludedResourceObject.java 1371639 2012-08-10 10:16:22Z mehdi $ */
 
 package org.apache.fop.afp.modca;
 
@@ -26,16 +26,15 @@ import java.net.URI;
 
 import org.apache.commons.io.IOUtils;
 
+import org.apache.fop.afp.util.AFPResourceAccessor;
 import org.apache.fop.afp.util.AFPResourceUtil;
-import org.apache.fop.afp.util.ResourceAccessor;
-
 
 /**
  * Encapsulates an included resource object that is loaded from an external file.
  */
 public class IncludedResourceObject extends AbstractNamedAFPObject {
 
-    private ResourceAccessor resourceAccessor;
+    private final AFPResourceAccessor resourceAccessor;
     private URI uri;
 
     /**
@@ -45,7 +44,7 @@ public class IncludedResourceObject extends AbstractNamedAFPObject {
      * @param uri the URI of the external file
      */
     public IncludedResourceObject(String name,
-            ResourceAccessor resourceAccessor, URI uri) {
+            AFPResourceAccessor resourceAccessor, URI uri) {
         super(name);
         this.resourceAccessor = resourceAccessor;
         this.uri = uri;
