@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: MapCodedFont.java 985571 2010-08-14 19:28:26Z jeremias $ */
+/* $Id: MapCodedFont.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.afp.modca;
 
@@ -292,7 +292,7 @@ public class MapCodedFont extends AbstractStructuredObject {
         /**
          * The scale (only specified for outline fonts)
          */
-        private int scale = 0;
+        private int scale;
     }
 
 }
