@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: StringUtils.java 1297404 2012-03-06 10:17:54Z vhennebert $ */
+/* $Id: StringUtils.java 1616668 2014-08-08 03:48:54Z gadams $ */
 
 package org.apache.fop.afp.util;
 
@@ -39,7 +39,7 @@ public final class StringUtils {
     public static String lpad(String input, char padding, int length) {
 
         if (input == null) {
-            input = new String();
+            input = "";
         }
 
         if (input.length() >= length) {
@@ -66,7 +66,7 @@ public final class StringUtils {
     public static String rpad(String input, char padding, int length) {
 
         if (input == null) {
-            input = new String();
+            input = "";
         }
 
         if (input.length() >= length) {
