@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Area.java 1311120 2012-04-08 23:48:11Z gadams $ */
+/* $Id: Area.java 1642793 2014-12-02 00:29:57Z lbernardo $ */
 
 package org.apache.fop.area;
 
@@ -111,6 +111,8 @@ public class Area extends AreaTreeObject implements Serializable {
     /** the area's block-progression-dimension */
     protected int bpd;
 
+    protected int effectiveIPD = -1;
+
     /**
      * Resolved bidirectional level for area.
      */
@@ -208,6 +210,10 @@ public class Area extends AreaTreeObject implements Serializable {
         return getBorderAndPaddingWidthStart() + getIPD() + getBorderAndPaddingWidthEnd();
     }
 
+    public int getEffectiveAllocIPD() {
+        return getBorderAndPaddingWidthStart() + getEffectiveIPD() + getBorderAndPaddingWidthEnd();
+    }
+
     /**
      * Get the allocation block progression dimension of this area.
      * This adds the content, borders, padding and spaces to find the
@@ -225,7 +231,7 @@ public class Area extends AreaTreeObject implements Serializable {
      *
      * @param bidiLevel the bidirectional embedding level
      */
-    public void setBidiLevel ( int bidiLevel ) {
+    public void setBidiLevel(int bidiLevel) {
         this.bidiLevel = bidiLevel;
     }
 
@@ -412,9 +418,9 @@ public class Area extends AreaTreeObject implements Serializable {
      *
      * @param traits the map of traits
      */
-    public void setTraits ( Map traits ) {
-        if ( traits != null ) {
-            this.traits = new TreeMap<Integer, Object>( traits );
+    public void setTraits(Map traits) {
+        if (traits != null) {
+            this.traits = new TreeMap<Integer, Object>(traits);
         } else {
             this.traits = null;
         }
@@ -499,4 +505,14 @@ public class Area extends AreaTreeObject implements Serializable {
         sb.append("}");
         return sb.toString();
     }
+
+    public int getEffectiveIPD() {
+        return 0;
+    }
+
+    public void activateEffectiveIPD() {
+        if (effectiveIPD != -1) {
+            ipd = effectiveIPD;
+        }
+    }
 }
