@@ -15,10 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: Block.java 1293736 2012-02-26 02:29:01Z gadams $ */
+/* $Id: Block.java 1642793 2014-12-02 00:29:57Z lbernardo $ */
 
 package org.apache.fop.area;
 
+import java.io.IOException;
+import java.io.ObjectInputStream;
+import java.util.Locale;
 
 // block areas hold either more block areas or line
 // areas can also be used as a block spacer
@@ -61,10 +64,19 @@ public class Block extends BlockParent {
     private int positioning = STACK;
 
     /** if true, allow BPD update */
-    protected transient boolean allowBPDUpdate = true;
+    protected transient boolean allowBPDUpdate;
 
-    // a block with may contain the dominant styling info in
-    // terms of most lines or blocks with info
+    private Locale locale;
+
+    private String location;
+
+    public Block() {
+        allowBPDUpdate = true;
+    }
+
+    private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
+        ois.defaultReadObject();
+    }
 
     /**
      * Add the block to this block area.
@@ -140,5 +152,53 @@ public class Block extends BlockParent {
         return (endIndent != null ? endIndent : 0);
     }
 
+    /**
+     * Sets the language information coming from the FO that generated this area.
+     */
+    public void setLocale(Locale locale) {
+        this.locale = locale;
+    }
+
+    /**
+     * Returns the language information for the FO that generated this area.
+     */
+    public Locale getLocale() {
+        return locale;
+    }
+
+    /**
+     * Sets the location in the source XML of the FO that generated this area.
+     *
+     * @location the line and column location
+     */
+    public void setLocation(String location) {
+        this.location = location;
+    }
+
+    /**
+     * Returns the location in the source XML of the FO that generated this area.
+     *
+     * @return the line and column location, {@code null} if that information is not available
+     */
+    public String getLocation() {
+        return location;
+    }
+
+    // maybe this can be done in the parent?
+    public int getEffectiveIPD() {
+        int eIPD = super.getEffectiveIPD();
+        if (eIPD != 0) {
+            effectiveIPD = eIPD;
+        }
+        return eIPD;
+    }
+
+    // maybe this can be done in the parent?
+    public void activateEffectiveIPD() {
+        super.activateEffectiveIPD();
+        if (effectiveIPD != -1) {
+            ipd = effectiveIPD;
+        }
+    }
 }
 
