@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,34 +15,41 @@
  * limitations under the License.
  */
 
-/* $Id: Span.java 557337 2007-07-18 17:37:14Z adelmelle $ */
- 
+/* $Id: Span.java 1466146 2013-04-09 17:31:41Z vhennebert $ */
+
 package org.apache.fop.area;
 
+import java.util.Iterator;
 import java.util.List;
 
+import org.apache.fop.fo.Constants;
+import org.apache.fop.traits.WritingModeTraitsGetter;
+
 /**
  * The span-reference-area.
  * This is a block-area with 0 border and padding that is stacked
  * within the main-reference-area
  * This object holds one or more normal-flow-reference-area children
  * based on the column-count trait in effect for this span.
- * See fo:region-body definition in the XSL Rec for more information. 
+ * See fo:region-body definition in the XSL Rec for more information.
  */
 public class Span extends Area {
+
+    private static final long serialVersionUID = -5551430053660081549L;
+
     // the list of flow reference areas in this span area
-    private List flowAreas;
+    private List<NormalFlow> flowAreas;
     private int colCount;
     private int colGap;
     private int colWidth; // width for each normal flow, calculated value
     private int curFlowIdx;  // n-f-r-a currently being processed, zero-based
-    
+
     /**
      * Create a span area with the number of columns for this span area.
      *
      * @param colCount the number of columns in the span
-     * @param colGap the column gap between each column 
-     * @param ipd the total ipd of the span 
+     * @param colGap the column gap between each column
+     * @param ipd the total ipd of the span
      */
     public Span(int colCount, int colGap, int ipd) {
         addTrait(Trait.IS_REFERENCE_AREA, Boolean.TRUE);
@@ -57,7 +64,7 @@ public class Span extends Area {
      * Create the normal flows for this Span
      */
     private void createNormalFlows() {
-        flowAreas = new java.util.ArrayList(colCount);        
+        flowAreas = new java.util.ArrayList<NormalFlow>(colCount);
         colWidth = (ipd - ((colCount - 1) * colGap)) / colCount;
 
         for (int i = 0; i < colCount; i++) {
@@ -102,30 +109,30 @@ public class Span extends Area {
      */
     public NormalFlow getNormalFlow(int colRequested) {
         if (colRequested >= 0 && colRequested < colCount) {
-            return (NormalFlow) flowAreas.get(colRequested);
+            return flowAreas.get(colRequested);
         } else { // internal error
-            throw new IllegalArgumentException("Invalid column number " 
-                    + colRequested + " requested; only 0-" + (colCount - 1) 
+            throw new IllegalArgumentException("Invalid column number "
+                    + colRequested + " requested; only 0-" + (colCount - 1)
                     + " available.");
         }
     }
-    
+
     /**
      * Get the NormalFlow area currently being processed
      *
      * @return the current NormalFlow
      */
     public NormalFlow getCurrentFlow() {
-        return getNormalFlow(curFlowIdx); 
+        return getNormalFlow(curFlowIdx);
     }
-    
+
     /** @return the index of the current normal flow */
     public int getCurrentFlowIndex() {
         return curFlowIdx;
     }
-    
+
     /**
-     * Indicate to the Span that the next column is being 
+     * Indicate to the Span that the next column is being
      * processed.
      *
      * @return the new NormalFlow (in the next column)
@@ -138,17 +145,17 @@ public class Span extends Area {
             throw new IllegalStateException("(Internal error.) No more flows left in span.");
         }
     }
-    
+
     /**
-     * Indicates if the Span has unprocessed flows. 
+     * Indicates if the Span has unprocessed flows.
      *
-     * @return true if Span can increment to the next flow, 
+     * @return true if Span can increment to the next flow,
      * false otherwise.
      */
     public boolean hasMoreFlows() {
-        return (curFlowIdx < colCount - 1); 
+        return (curFlowIdx < colCount - 1);
     }
-    
+
     /**
      * Called to notify the span that all its flows have been fully generated so it can update
      * its own BPD extent.
@@ -160,7 +167,7 @@ public class Span extends Area {
         }
         bpd = maxFlowBPD;
     }
-    
+
     /**
      * Indicates whether any child areas have been added to this span area.
      *
@@ -179,8 +186,31 @@ public class Span extends Area {
         }
         return (areaCount == 0);
     }
-    
+
+    /**
+     * Sets the writing mode traits for the main reference area of
+     * this span area.
+     * @param wmtg a WM traits getter
+     */
+    public void setWritingModeTraits(WritingModeTraitsGetter wmtg) {
+        switch (wmtg.getColumnProgressionDirection().getEnumValue()) {
+        case Constants.EN_RL:
+            setBidiLevel(1);
+            for (Iterator<NormalFlow> it = flowAreas.iterator(); it.hasNext();) {
+                it.next().setBidiLevel(1);
+            }
+            break;
+        default:
+            resetBidiLevel();
+            for (Iterator<NormalFlow> it = flowAreas.iterator(); it.hasNext();) {
+                it.next().resetBidiLevel();
+            }
+            break;
+        }
+    }
+
     /** {@inheritDoc} */
+    @Override
     public String toString() {
         StringBuffer sb = new StringBuffer(super.toString());
         if (colCount > 1) {
