@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Trait.java 1296526 2012-03-03 00:18:45Z gadams $ */
+/* $Id: Trait.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.area;
 
@@ -169,8 +169,11 @@ public final class Trait implements Serializable {
     /** shift direction trait */
     public static final Integer SHIFT_DIRECTION = 42;
 
+    /** For optional content groups. */
+    public static final Integer LAYER = 43;
+
     /** Maximum value used by trait keys */
-    public static final int MAX_TRAIT_KEY = 42;
+    public static final int MAX_TRAIT_KEY = 43;
 
     private static final TraitInfo[] TRAIT_INFO = new TraitInfo[MAX_TRAIT_KEY + 1];
 
@@ -242,6 +245,7 @@ public final class Trait implements Serializable {
                 new TraitInfo("block-progression-direction", Direction.class));
         put(SHIFT_DIRECTION,
                 new TraitInfo("shift-direction", Direction.class));
+        put(LAYER, new TraitInfo("layer", String.class));
 
     }
 
@@ -487,13 +491,13 @@ public final class Trait implements Serializable {
         private static final long serialVersionUID = 8452078676273242870L;
 
         /** The background color if any. */
-        private Color color = null;
+        private Color color;
 
         /** The background image url if any. */
-        private String url = null;
+        private String url;
 
         /** The background image if any. */
-        private ImageInfo imageInfo = null;
+        private ImageInfo imageInfo;
 
         /** Background repeat enum for images. */
         private int repeat;
@@ -504,6 +508,10 @@ public final class Trait implements Serializable {
         /** Background vertical offset for images. */
         private int vertical;
 
+        private int imageTargetWidth;
+
+        private int imageTargetHeight;
+
         /**
          * Returns the background color.
          * @return background color, null if n/a
@@ -651,9 +659,31 @@ public final class Trait implements Serializable {
                 sb.append(",horiz=").append(horiz);
                 sb.append(",vertical=").append(vertical);
             }
+            if (imageTargetWidth != 0) {
+                sb.append(",target-width=").append(Integer.toString(imageTargetWidth));
+            }
+            if (imageTargetHeight != 0) {
+                sb.append(",target-height=").append(Integer.toString(imageTargetHeight));
+            }
             return sb.toString();
         }
 
+        public void setImageTargetWidth(int value) {
+            imageTargetWidth = value;
+        }
+
+        public int getImageTargetWidth() {
+            return imageTargetWidth;
+        }
+
+        public void setImageTargetHeight(int value) {
+            imageTargetHeight = value;
+        }
+
+        public int getImageTargetHeight() {
+            return imageTargetHeight;
+        }
+
     }
 }
 
