@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: InlineArea.java 1293736 2012-02-26 02:29:01Z gadams $ */
+/* $Id: InlineArea.java 1681384 2015-05-23 21:07:13Z adelmelle $ */
 
 package org.apache.fop.area.inline;
 
@@ -41,6 +41,7 @@ public class InlineArea extends Area {
      * that can be used in order to re-compute adjustments when a
      * page-number or a page-number-citation is resolved
      */
+    // @SuppressFBWarnings("SE_INNER_CLASS")
     protected class InlineAdjustingInfo implements Serializable {
 
         private static final long serialVersionUID = -5601387735459712149L;
@@ -81,32 +82,32 @@ public class InlineArea extends Area {
     /**
      * offset position from before edge of parent area
      */
-    protected int blockProgressionOffset = 0;
+    protected int blockProgressionOffset;
 
     /**
      * parent area
      * it is needed in order to recompute adjust ratio and indents
      * when a page-number or a page-number-citation is resolved
      */
-    private Area parentArea = null;
+    private Area parentArea;
 
     /**
      * ipd variation of child areas: if this area has not already
      * been added and cannot notify its parent area, store the variation
      * and wait for the parent area to be set
      */
-    private int storedIPDVariation = 0;
+    private int storedIPDVariation;
 
     /**
      * The adjustment information object
      */
-    protected InlineAdjustingInfo adjustingInfo = null;
+    protected InlineAdjustingInfo adjustingInfo;
 
     /**
      * Default constructor for inline area.
      */
     public InlineArea() {
-        this (  0, -1 );
+        this (0, -1);
     }
 
     /**
@@ -114,7 +115,7 @@ public class InlineArea extends Area {
      * @param blockProgressionOffset a block progression offset or zero
      * @param bidiLevel a resolved bidi level or -1
      */
-    protected InlineArea ( int blockProgressionOffset, int bidiLevel ) {
+    protected InlineArea(int blockProgressionOffset, int bidiLevel) {
         this.blockProgressionOffset = blockProgressionOffset;
         setBidiLevel(bidiLevel);
     }
@@ -136,6 +137,14 @@ public class InlineArea extends Area {
         adjustingInfo = new InlineAdjustingInfo(stretch, shrink, adjustment);
     }
 
+    /**
+     * Sets the adjustment information from an existing object
+     * @param adjustingInfo the existing adjustment object
+     */
+    public void setAdjustingInfo(InlineAdjustingInfo adjustingInfo) {
+        this.adjustingInfo = adjustingInfo;
+    }
+
     /**
      * Modify the adjustment value in the adjustment information object
      * @param adjustment the new adjustment value
@@ -251,8 +260,10 @@ public class InlineArea extends Area {
                     + ": increase by " + ipdVariation + " mpt.");
         }
 
-        increaseIPD(ipdVariation);
-        notifyIPDVariation(ipdVariation);
+        if (ipdVariation != 0) {
+            increaseIPD(ipdVariation);
+            notifyIPDVariation(ipdVariation);
+        }
     }
 
     /**
@@ -261,11 +272,12 @@ public class InlineArea extends Area {
      * @param ipdVariation the difference between new and old ipd
      */
     protected void notifyIPDVariation(int ipdVariation) {
-        if (getParentArea() instanceof InlineArea) {
-            ((InlineArea) getParentArea()).handleIPDVariation(ipdVariation);
-        } else if (getParentArea() instanceof LineArea) {
-            ((LineArea) getParentArea()).handleIPDVariation(ipdVariation);
-        } else if (getParentArea() == null) {
+        Area parentArea = getParentArea();
+        if (parentArea instanceof InlineArea) {
+            ((InlineArea) parentArea).handleIPDVariation(ipdVariation);
+        } else if (parentArea instanceof LineArea) {
+            ((LineArea) parentArea).handleIPDVariation(ipdVariation);
+        } else if (parentArea == null) {
             // parent area not yet set: store the variations
             storedIPDVariation += ipdVariation;
         }
@@ -303,9 +315,9 @@ public class InlineArea extends Area {
      * @param runs current list of inline runs
      * @return modified list of inline runs, having appended new run
      */
-    public List collectInlineRuns ( List runs ) {
+    public List collectInlineRuns(List runs) {
         assert runs != null;
-        runs.add ( new InlineRun ( this, new int[] {getBidiLevel()}) );
+        runs.add(new InlineRun(this, new int[] {getBidiLevel()}));
         return runs;
     }
 
@@ -314,8 +326,8 @@ public class InlineArea extends Area {
      * @param ia inline area to test
      * @return true if specified inline area is an ancestor or same as this area
      */
-    public boolean isAncestorOrSelf ( InlineArea ia ) {
-        return ( ia == this ) || isAncestor ( ia );
+    public boolean isAncestorOrSelf(InlineArea ia) {
+        return (ia == this) || isAncestor(ia);
     }
 
     /**
@@ -323,12 +335,12 @@ public class InlineArea extends Area {
      * @param ia inline area to test
      * @return true if specified inline area is an ancestor of this area
      */
-    public boolean isAncestor ( InlineArea ia ) {
-        for ( Area p = getParentArea(); p != null;) {
-            if ( p == ia ) {
+    public boolean isAncestor(InlineArea ia) {
+        for (Area p = getParentArea(); p != null;) {
+            if (p == ia) {
                 return true;
-            } else if ( p instanceof InlineArea ) {
-                p = ( (InlineArea) p ).getParentArea();
+            } else if (p instanceof InlineArea) {
+                p = ((InlineArea) p).getParentArea();
             } else {
                 p = null;
             }
