@@ -15,10 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: InlineParent.java 1296526 2012-03-03 00:18:45Z gadams $ */
+/* $Id: InlineParent.java 1617052 2014-08-10 06:55:01Z gadams $ */
 
 package org.apache.fop.area.inline;
 
+import java.io.IOException;
+import java.io.ObjectInputStream;
+import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
 
@@ -35,7 +38,7 @@ public class InlineParent extends InlineArea {
     /**
      * The list of inline areas added to this inline parent.
      */
-    protected List<InlineArea> inlines = new java.util.ArrayList<InlineArea>();
+    protected List<InlineArea> inlines = new ArrayList<InlineArea>();
 
     /** Controls whether the IPD is automatically adjusted based on the area's children. */
     protected transient boolean autoSize;
@@ -50,6 +53,10 @@ public class InlineParent extends InlineArea {
      */
     private int maxAfterEdge;
 
+    private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
+        ois.defaultReadObject();
+    }
+
     @Override
     public void addChildArea(Area c) {
         assert c instanceof InlineArea;
@@ -63,7 +70,7 @@ public class InlineParent extends InlineArea {
         if (autoSize) {
             increaseIPD(childArea.getAllocIPD());
         }
-        updateLevel ( childArea.getBidiLevel() );
+        updateLevel(childArea.getBidiLevel());
         int childOffset = childArea.getVirtualOffset();
         minChildOffset = Math.min(minChildOffset, childOffset);
         maxAfterEdge = Math.max(maxAfterEdge, childOffset + childArea.getVirtualBPD());
@@ -113,10 +120,10 @@ public class InlineParent extends InlineArea {
     }
 
     @Override
-    public List collectInlineRuns ( List runs ) {
-        for ( Iterator<InlineArea> it = getChildAreas().iterator(); it.hasNext();) {
+    public List collectInlineRuns(List runs) {
+        for (Iterator<InlineArea> it = getChildAreas().iterator(); it.hasNext();) {
             InlineArea ia = it.next();
-            runs = ia.collectInlineRuns ( runs );
+            runs = ia.collectInlineRuns(runs);
         }
         return runs;
     }
@@ -126,20 +133,20 @@ public class InlineParent extends InlineArea {
      * signalling that they will inherit the level of their parent text area.
      */
     public void resetChildrenLevel() {
-        for ( Iterator it = inlines.iterator(); it.hasNext();) {
-            ( (InlineArea) it.next() ) .resetBidiLevel();
+        for (Iterator it = inlines.iterator(); it.hasNext();) {
+            ((InlineArea) it.next()) .resetBidiLevel();
         }
     }
 
-    private void updateLevel ( int newLevel ) {
-        if ( newLevel >= 0 ) {
+    private void updateLevel(int newLevel) {
+        if (newLevel >= 0) {
             int curLevel = getBidiLevel();
-            if ( curLevel >= 0 ) {
-                if ( newLevel < curLevel ) {
-                    setBidiLevel ( newLevel );
+            if (curLevel >= 0) {
+                if (newLevel < curLevel) {
+                    setBidiLevel(newLevel);
                 }
             } else {
-                setBidiLevel ( newLevel );
+                setBidiLevel(newLevel);
             }
         }
     }
