@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: CommandLineOptions.java 1341585 2012-05-22 18:26:15Z gadams $ */
+/* $Id: CommandLineOptions.java 1636469 2014-11-03 23:01:33Z lbernardo $ */
 
 package org.apache.fop.cli;
 
@@ -24,6 +24,7 @@ import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.PrintStream;
+import java.net.URI;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Vector;
@@ -39,9 +40,11 @@ import org.apache.fop.Version;
 import org.apache.fop.accessibility.Accessibility;
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
+import org.apache.fop.apps.FopConfParser;
 import org.apache.fop.apps.FopFactory;
+import org.apache.fop.apps.FopFactoryBuilder;
+import org.apache.fop.apps.FopFactoryConfig;
 import org.apache.fop.apps.MimeConstants;
-import org.apache.fop.fonts.FontManager;
 import org.apache.fop.pdf.PDFAMode;
 import org.apache.fop.pdf.PDFEncryptionManager;
 import org.apache.fop.pdf.PDFEncryptionParams;
@@ -51,11 +54,10 @@ import org.apache.fop.render.awt.AWTRenderer;
 import org.apache.fop.render.intermediate.IFContext;
 import org.apache.fop.render.intermediate.IFDocumentHandler;
 import org.apache.fop.render.intermediate.IFSerializer;
-import org.apache.fop.render.pdf.PDFConfigurationConstants;
+import org.apache.fop.render.pdf.PDFEncryptionOption;
 import org.apache.fop.render.print.PagesMode;
 import org.apache.fop.render.print.PrintRenderer;
 import org.apache.fop.render.xml.XMLRenderer;
-import org.apache.fop.util.CommandLineLogger;
 
 /**
  * Options parses the commandline arguments
@@ -87,67 +89,65 @@ public class CommandLineOptions {
     /* for area tree XML output, only down to block area level */
     private Boolean suppressLowLevelAreas = Boolean.FALSE;
     /* user configuration file */
-    private File userConfigFile = null;
+    private File userConfigFile;
     /* input fo file */
-    private File fofile = null;
+    private File fofile;
     /* xsltfile (xslt transformation as input) */
-    private File xsltfile = null;
+    private File xsltfile;
     /* xml file (xslt transformation as input) */
-    private File xmlfile = null;
+    private File xmlfile;
     /* area tree input file */
-    private File areatreefile = null;
+    private File areatreefile;
     /* intermediate format input file */
-    private File iffile = null;
+    private File iffile;
     /* area tree input file */
-    private File imagefile = null;
+    private File imagefile;
     /* output file */
-    private File outfile = null;
+    private File outfile;
     /* input mode */
     private int inputmode = NOT_SET;
     /* output mode */
-    private String outputmode = null;
+    private String outputmode;
     /* true if System.in (stdin) should be used for the input file */
-    private boolean useStdIn = false;
+    private boolean useStdIn;
     /* true if System.out (stdout) should be used for the output file */
-    private boolean useStdOut = false;
+    private boolean useStdOut;
     /* true if a catalog resolver should be used for entity and uri resolution */
-    private boolean useCatalogResolver = false;
+    private boolean useCatalogResolver;
     /* rendering options (for the user agent) */
     private Map renderingOptions = new java.util.HashMap();
     /* target resolution (for the user agent) */
-    private int targetResolution = 0;
+    private float targetResolution = FopFactoryConfig.DEFAULT_TARGET_RESOLUTION;
+
+    private boolean strictValidation = true;
     /* control memory-conservation policy */
-    private boolean conserveMemoryPolicy = false;
+    private boolean conserveMemoryPolicy;
     /* true if a complex script features are enabled */
     private boolean useComplexScriptFeatures = true;
+    /* set to true if -dpi used in command line */
+    private boolean overrideTargetResolution;
 
-    private FopFactory factory = FopFactory.newInstance();
+    private FopFactory factory;
     private FOUserAgent foUserAgent;
 
     private InputHandler inputHandler;
 
     private Log log;
 
-    private Vector xsltParams = null;
+    private Vector xsltParams;
 
-    private String mimicRenderer = null;
+    private String mimicRenderer;
 
-    private boolean flushCache = false;
+    private boolean flushCache;
+
+    private URI baseURI = new File(".").getAbsoluteFile().toURI();
+
+    private String cacheName;
 
     /**
      * Construct a command line option object.
      */
     public CommandLineOptions() {
-        LogFactory logFactory = LogFactory.getFactory();
-
-        // Enable the simple command line logging when no other logger is
-        // defined.
-        if (System.getProperty("org.apache.commons.logging.Log") == null) {
-            logFactory.setAttribute("org.apache.commons.logging.Log",
-                                            CommandLineLogger.class.getName());
-            setLogLevel("info");
-        }
-
         log = LogFactory.getLog("FOP");
     }
 
@@ -166,7 +166,7 @@ public class CommandLineOptions {
         try {
             optionsParsed = parseOptions(args);
             if (optionsParsed) {
-                if (showConfiguration == Boolean.TRUE) {
+                if (showConfiguration.equals(Boolean.TRUE)) {
                     dumpConfiguration();
                 }
                 checkSettings();
@@ -177,15 +177,13 @@ public class CommandLineOptions {
                 //Factory config is set up, now we can create the user agent
                 foUserAgent = factory.newFOUserAgent();
                 foUserAgent.getRendererOptions().putAll(renderingOptions);
-                if (targetResolution != 0) {
-                    foUserAgent.setTargetResolution(targetResolution);
-                }
                 addXSLTParameter("fop-output-format", getOutputFormat());
                 addXSLTParameter("fop-version", Version.getVersion());
                 foUserAgent.setConserveMemoryPolicy(conserveMemoryPolicy);
-                if (!useComplexScriptFeatures) {
-                    foUserAgent.setComplexScriptFeaturesEnabled(false);
-                }
+                // TODO: Handle this!!
+                //if (!useComplexScriptFeatures) {
+                //    foUserAgent.setComplexScriptFeaturesEnabled(false);
+                //}
             } else {
                 return false;
             }
@@ -225,9 +223,7 @@ public class CommandLineOptions {
         } else if (MimeConstants.MIME_FOP_IF.equals(outputmode)
                 && mimicRenderer != null) {
             // render from FO to Intermediate Format
-            IFSerializer serializer = new IFSerializer();
-            serializer.setContext(new IFContext(foUserAgent));
-
+            IFSerializer serializer = new IFSerializer(new IFContext(foUserAgent));
             IFDocumentHandler targetHandler
                 = foUserAgent.getRendererFactory().createDocumentHandler(
                         foUserAgent, mimicRenderer);
@@ -286,19 +282,19 @@ public class CommandLineOptions {
             } else if (args[i].equals("-s")) {
                 suppressLowLevelAreas = Boolean.TRUE;
             } else if (args[i].equals("-d")) {
-                setLogOption("debug", "debug");
+                // nop. Left there for backwards compatibility
             } else if (args[i].equals("-r")) {
-                factory.setStrictValidation(false);
+                strictValidation = false;
             } else if (args[i].equals("-conserve")) {
                 conserveMemoryPolicy = true;
             } else if (args[i].equals("-flush")) {
                 flushCache = true;
             } else if (args[i].equals("-cache")) {
-                parseCacheOption(args, i);
+                i = i + parseCacheOption(args, i);
             } else if (args[i].equals("-dpi")) {
                 i = i + parseResolution(args, i);
             } else if (args[i].equals("-q") || args[i].equals("--quiet")) {
-                setLogOption("quiet", "error");
+                // nop. Left there for backwards compatibility
             } else if (args[i].equals("-fo")) {
                 i = i + parseFOInputOption(args, i);
             } else if (args[i].equals("-xsl")) {
@@ -403,6 +399,7 @@ public class CommandLineOptions {
                 i = i + parseUnknownOption(args, i);
             } else {
                 printUsage(System.err);
+                // @SuppressFBWarnings("DM_EXIT")
                 System.exit(1);
             }
         }
@@ -415,7 +412,7 @@ public class CommandLineOptions {
             throw new FOPException("if you use '-cache', you must specify "
               + "the name of the font cache file");
         } else {
-            factory.getFontManager().setCacheFile(new File(args[i + 1]));
+            cacheName = args[i + 1];
             return 1;
         }
     }
@@ -448,6 +445,7 @@ public class CommandLineOptions {
                     "if you use '-dpi', you must specify a resolution (dots per inch)");
         } else {
             this.targetResolution = Integer.parseInt(args[i + 1]);
+            this.overrideTargetResolution = true;
             return 1;
         }
     }
@@ -463,6 +461,7 @@ public class CommandLineOptions {
                 this.useStdIn = true;
             } else {
                 fofile = new File(filename);
+                baseURI = getBaseURI(fofile);
             }
             return 1;
         }
@@ -492,11 +491,16 @@ public class CommandLineOptions {
                 this.useStdIn = true;
             } else {
                 xmlfile = new File(filename);
+                baseURI = getBaseURI(xmlfile);
             }
             return 1;
         }
     }
 
+    private URI getBaseURI(File file) {
+        return file.getAbsoluteFile().getParentFile().toURI();
+    }
+
     private int parseAWTOutputOption(String[] args, int i) throws FOPException {
         setOutputMode(MimeConstants.MIME_FOP_AWT_PREVIEW);
         return 0;
@@ -701,6 +705,7 @@ public class CommandLineOptions {
                 for (int j = 0; j < mimes.length; j++) {
                     System.out.println("  " + mimes[j]);
                 }
+                // @SuppressFBWarnings("DM_EXIT")
                 System.exit(0);
             }
         }
@@ -723,6 +728,7 @@ public class CommandLineOptions {
                 this.useStdIn = true;
             } else {
                 fofile = new File(filename);
+                baseURI = getBaseURI(fofile);
             }
         } else if (outputmode == null) {
             outputmode = MimeConstants.MIME_PDF;
@@ -781,6 +787,7 @@ public class CommandLineOptions {
                 this.useStdIn = true;
             } else {
                 areatreefile = new File(filename);
+                baseURI = getBaseURI(areatreefile);
             }
             return 1;
         }
@@ -797,6 +804,7 @@ public class CommandLineOptions {
                 this.useStdIn = true;
             } else {
                 iffile = new File(filename);
+                baseURI = getBaseURI(iffile);
             }
             return 1;
         }
@@ -813,21 +821,21 @@ public class CommandLineOptions {
                 this.useStdIn = true;
             } else {
                 imagefile = new File(filename);
+                baseURI = getBaseURI(imagefile);
             }
             return 1;
         }
     }
 
     private PDFEncryptionParams getPDFEncryptionParams() throws FOPException {
-        PDFEncryptionParams params = (PDFEncryptionParams)renderingOptions.get(
-                        PDFConfigurationConstants.ENCRYPTION_PARAMS);
+        PDFEncryptionParams params = (PDFEncryptionParams) renderingOptions.get(PDFEncryptionOption.ENCRYPTION_PARAMS);
         if (params == null) {
             if (!PDFEncryptionManager.checkAvailableAlgorithms()) {
                 throw new FOPException("PDF encryption requested but it is not available."
                         + " Please make sure MD5 and RC4 algorithms are available.");
             }
             params = new PDFEncryptionParams();
-            renderingOptions.put(PDFConfigurationConstants.ENCRYPTION_PARAMS, params);
+            renderingOptions.put(PDFEncryptionOption.ENCRYPTION_PARAMS, params);
         }
         return params;
     }
@@ -860,7 +868,7 @@ public class CommandLineOptions {
             throw new FOPException("You must specify a PDF profile");
         } else {
             String profile = args[i + 1];
-            PDFAMode pdfAMode = PDFAMode.valueOf(profile);
+            PDFAMode pdfAMode = PDFAMode.getValueOf(profile);
             if (pdfAMode != null && pdfAMode != PDFAMode.DISABLED) {
                 if (renderingOptions.get("pdf-a-mode") != null) {
                     throw new FOPException("PDF/A mode already set");
@@ -868,7 +876,7 @@ public class CommandLineOptions {
                 renderingOptions.put("pdf-a-mode", pdfAMode.getName());
                 return 1;
             } else {
-                PDFXMode pdfXMode = PDFXMode.valueOf(profile);
+                PDFXMode pdfXMode = PDFXMode.getValueOf(profile);
                 if (pdfXMode != null && pdfXMode != PDFXMode.DISABLED) {
                     if (renderingOptions.get("pdf-x-mode") != null) {
                         throw new FOPException("PDF/X mode already set");
@@ -889,27 +897,6 @@ public class CommandLineOptions {
         }
     }
 
-    private void setLogOption (String option, String level) {
-        if (log instanceof CommandLineLogger
-            || System.getProperty("org.apache.commons.logging.Log") == null) {
-            setLogLevel(level);
-        } else if (log != null) {
-            log.warn("The option " + option + " can only be used");
-            log.warn("with FOP's command line logger,");
-            log.warn("which is the default on the command line.");
-            log.warn("Configure other loggers using Java system properties.");
-        }
-    }
-
-    private void setLogLevel(String level) {
-        // Set the level for future loggers.
-        LogFactory.getFactory().setAttribute("level", level);
-        if (log instanceof CommandLineLogger) {
-            // Set the level for the logger created already.
-            ((CommandLineLogger) log).setLogLevel(level);
-        }
-    }
-
     private void setInputFormat(int format) throws FOPException {
         if (inputmode == NOT_SET || inputmode == format) {
             inputmode = format;
@@ -1027,13 +1014,33 @@ public class CommandLineOptions {
      * @throws IOException
      */
     private void setUserConfig() throws FOPException, IOException {
+        FopFactoryBuilder fopFactoryBuilder;
         if (userConfigFile == null) {
-            return;
+            fopFactoryBuilder = new FopFactoryBuilder(baseURI);
+            fopFactoryBuilder.setStrictFOValidation(strictValidation);
+            fopFactoryBuilder.setTargetResolution(targetResolution);
+            fopFactoryBuilder.setComplexScriptFeatures(useComplexScriptFeatures);
+        } else {
+            try {
+                FopConfParser fopConfParser = new FopConfParser(userConfigFile, baseURI);
+                fopFactoryBuilder = fopConfParser.getFopFactoryBuilder();
+                if (this.overrideTargetResolution) {
+                    fopFactoryBuilder.setTargetResolution(targetResolution);
+                }
+            } catch (SAXException e) {
+                throw new FOPException(e);
+            }
+            if (!strictValidation) {
+                fopFactoryBuilder.setStrictFOValidation(strictValidation);
+            }
+            if (!useComplexScriptFeatures) {
+                // note: by default cs is enabled so we need to disable it
+                fopFactoryBuilder.setComplexScriptFeatures(useComplexScriptFeatures);
+            }
         }
-        try {
-            factory.setUserConfig(userConfigFile);
-        } catch (SAXException e) {
-            throw new FOPException(e);
+        factory = fopFactoryBuilder.build();
+        if (cacheName != null) {
+            factory.getFontManager().setCacheFile(URI.create(cacheName));
         }
      }
 
@@ -1180,9 +1187,7 @@ public class CommandLineOptions {
                     + "[-awt|-pdf|-mif|-rtf|-tiff|-png|-pcl|-ps|-txt|-at [mime]|-print] <outfile>\n"
             + " [OPTIONS]  \n"
             + "  -version          print FOP version and exit\n"
-            + "  -d                debug mode   \n"
             + "  -x                dump configuration settings  \n"
-            + "  -q                quiet mode  \n"
             + "  -c cfg.xml        use additional configuration file cfg.xml\n"
             + "  -l lang           the language to use for user information \n"
             + "  -nocs             disable complex script features\n"
@@ -1390,13 +1395,7 @@ public class CommandLineOptions {
     }
 
     private void flushCache() throws FOPException {
-        FontManager fontManager = factory.getFontManager();
-        File cacheFile = fontManager.getCacheFile();
-        if (!fontManager.deleteCache()) {
-            System.err.println("Failed to flush the font cache file '"
-                    + cacheFile + "'.");
-            System.exit(1);
-        }
+        factory.getFontManager().deleteCache();
     }
 }
 
