@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: InputHandler.java 1297232 2012-03-05 21:13:28Z gadams $ */
+/* $Id: InputHandler.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.cli;
 
@@ -51,7 +51,6 @@ import org.apache.fop.ResourceEventProducer;
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.apps.Fop;
-import org.apache.fop.apps.FopFactory;
 import org.apache.fop.render.awt.viewer.Renderable;
 
 /**
@@ -65,8 +64,8 @@ public class InputHandler implements ErrorListener, Renderable {
     protected File sourcefile;
     private File stylesheet;  // for XML/XSLT usage
     private Vector xsltParams; // for XML/XSLT usage
-    private EntityResolver entityResolver = null;
-    private URIResolver uriResolver = null;
+    private EntityResolver entityResolver;
+    private URIResolver uriResolver;
 
     /** the logger */
     protected Log log = LogFactory.getLog(InputHandler.class);
@@ -103,25 +102,11 @@ public class InputHandler implements ErrorListener, Renderable {
     public void renderTo(FOUserAgent userAgent, String outputFormat, OutputStream out)
                 throws FOPException {
 
-        FopFactory factory = userAgent.getFactory();
         Fop fop;
         if (out != null) {
-            fop = factory.newFop(outputFormat, userAgent, out);
+            fop = userAgent.newFop(outputFormat, out);
         } else {
-            fop = factory.newFop(outputFormat, userAgent);
-        }
-
-        // if base URL was not explicitly set in FOUserAgent, obtain here
-        if (fop.getUserAgent().getBaseURL() == null && sourcefile != null) {
-            String baseURL = null;
-
-            try {
-                baseURL = new File(sourcefile.getAbsolutePath())
-                        .getParentFile().toURI().toURL().toExternalForm();
-            } catch (Exception e) {
-                baseURL = "";
-            }
-            fop.getUserAgent().setBaseURL(baseURL);
+            fop = userAgent.newFop(outputFormat);
         }
 
         // Resulting SAX events (the generated FO) must be piped through to FOP
