@@ -29,14 +29,12 @@ import org.apache.commons.logging.LogFactory;
 import org.apache.fop.complexscripts.bidi.BidiClass;
 import org.apache.fop.complexscripts.bidi.BidiConstants;
 import org.apache.fop.complexscripts.fonts.GlyphDefinitionTable;
+import org.apache.fop.complexscripts.util.CharAssociation;
 import org.apache.fop.complexscripts.util.GlyphContextTester;
 import org.apache.fop.complexscripts.util.GlyphSequence;
 import org.apache.fop.complexscripts.util.ScriptContextTester;
+import org.apache.fop.util.CharUtilities;
 
-// CSOFF: AvoidNestedBlocksCheck
-// CSOFF: NoWhitespaceAfterCheck
-// CSOFF: InnerAssignmentCheck
-// CSOFF: SimplifyBooleanReturnCheck
 // CSOFF: LineLengthCheck
 
 /**
@@ -48,10 +46,10 @@ import org.apache.fop.complexscripts.util.ScriptContextTester;
 public class ArabicScriptProcessor extends DefaultScriptProcessor {
 
     /** logging instance */
-    private static final Log log = LogFactory.getLog(ArabicScriptProcessor.class);                                      // CSOK: ConstantNameCheck
+    private static final Log log = LogFactory.getLog(ArabicScriptProcessor.class);
 
     /** features to use for substitutions */
-    private static final String[] gsubFeatures =                                                                        // CSOK: ConstantNameCheck
+    private static final String[] GSUB_FEATURES =
     {
         "calt",                                                 // contextual alternates
         "ccmp",                                                 // glyph composition/decomposition
@@ -64,7 +62,7 @@ public class ArabicScriptProcessor extends DefaultScriptProcessor {
     };
 
     /** features to use for positioning */
-    private static final String[] gposFeatures =                                                                        // CSOK: ConstantNameCheck
+    private static final String[] GPOS_FEATURES =
     {
         "curs",                                                 // cursive positioning
         "kern",                                                 // kerning
@@ -75,56 +73,56 @@ public class ArabicScriptProcessor extends DefaultScriptProcessor {
     private static class SubstitutionScriptContextTester implements ScriptContextTester {
         private static Map/*<String,GlyphContextTester>*/ testerMap = new HashMap/*<String,GlyphContextTester>*/();
         static {
-            testerMap.put ( "fina", new GlyphContextTester() {
-                    public boolean test ( String script, String language, String feature, GlyphSequence gs, int index, int flags ) {
-                        return inFinalContext ( script, language, feature, gs, index, flags );
+            testerMap.put("fina", new GlyphContextTester() {
+                    public boolean test(String script, String language, String feature, GlyphSequence gs, int index, int flags) {
+                        return inFinalContext(script, language, feature, gs, index, flags);
                     }
-                } );
-            testerMap.put ( "init", new GlyphContextTester() {
-                    public boolean test ( String script, String language, String feature, GlyphSequence gs, int index, int flags ) {
-                        return inInitialContext ( script, language, feature, gs, index, flags );
+                });
+            testerMap.put("init", new GlyphContextTester() {
+                    public boolean test(String script, String language, String feature, GlyphSequence gs, int index, int flags) {
+                        return inInitialContext(script, language, feature, gs, index, flags);
                     }
-                } );
-            testerMap.put ( "isol", new GlyphContextTester() {
-                    public boolean test ( String script, String language, String feature, GlyphSequence gs, int index, int flags ) {
-                        return inIsolateContext ( script, language, feature, gs, index, flags );
+                });
+            testerMap.put("isol", new GlyphContextTester() {
+                    public boolean test(String script, String language, String feature, GlyphSequence gs, int index, int flags) {
+                        return inIsolateContext(script, language, feature, gs, index, flags);
                     }
-                } );
-            testerMap.put ( "liga", new GlyphContextTester() {
-                    public boolean test ( String script, String language, String feature, GlyphSequence gs, int index, int flags ) {
-                        return inLigatureContext ( script, language, feature, gs, index, flags );
+                });
+            testerMap.put("liga", new GlyphContextTester() {
+                    public boolean test(String script, String language, String feature, GlyphSequence gs, int index, int flags) {
+                        return inLigatureContext(script, language, feature, gs, index, flags);
                     }
-                } );
-            testerMap.put ( "medi", new GlyphContextTester() {
-                    public boolean test ( String script, String language, String feature, GlyphSequence gs, int index, int flags ) {
-                        return inMedialContext ( script, language, feature, gs, index, flags );
+                });
+            testerMap.put("medi", new GlyphContextTester() {
+                    public boolean test(String script, String language, String feature, GlyphSequence gs, int index, int flags) {
+                        return inMedialContext(script, language, feature, gs, index, flags);
                     }
-                } );
+                });
         }
-        public GlyphContextTester getTester ( String feature ) {
-            return (GlyphContextTester) testerMap.get ( feature );
+        public GlyphContextTester getTester(String feature) {
+            return (GlyphContextTester) testerMap.get(feature);
         }
     }
 
     private static class PositioningScriptContextTester implements ScriptContextTester {
         private static Map/*<String,GlyphContextTester>*/ testerMap = new HashMap/*<String,GlyphContextTester>*/();
-        public GlyphContextTester getTester ( String feature ) {
-            return (GlyphContextTester) testerMap.get ( feature );
+        public GlyphContextTester getTester(String feature) {
+            return (GlyphContextTester) testerMap.get(feature);
         }
     }
 
     private final ScriptContextTester subContextTester;
     private final ScriptContextTester posContextTester;
 
-    ArabicScriptProcessor ( String script ) {
-        super ( script );
+    ArabicScriptProcessor(String script) {
+        super(script);
         this.subContextTester = new SubstitutionScriptContextTester();
         this.posContextTester = new PositioningScriptContextTester();
     }
 
     /** {@inheritDoc} */
     public String[] getSubstitutionFeatures() {
-        return gsubFeatures;
+        return GSUB_FEATURES;
     }
 
     /** {@inheritDoc} */
@@ -134,7 +132,7 @@ public class ArabicScriptProcessor extends DefaultScriptProcessor {
 
     /** {@inheritDoc} */
     public String[] getPositioningFeatures() {
-        return gposFeatures;
+        return GPOS_FEATURES;
     }
 
     /** {@inheritDoc} */
@@ -144,26 +142,28 @@ public class ArabicScriptProcessor extends DefaultScriptProcessor {
 
     /** {@inheritDoc} */
     @Override
-    public GlyphSequence reorderCombiningMarks ( GlyphDefinitionTable gdef, GlyphSequence gs, int[][] gpa, String script, String language ) {
+    public GlyphSequence reorderCombiningMarks(GlyphDefinitionTable gdef, GlyphSequence gs, int[] widths, int[][] gpa, String script, String language) {
         // a side effect of BIDI reordering is to order combining marks before their base, so we need to override the default here to
         // prevent double reordering
         return gs;
     }
 
-    private static boolean inFinalContext ( String script, String language, String feature, GlyphSequence gs, int index, int flags ) {
-        GlyphSequence.CharAssociation a = gs.getAssociation ( index );
-        int[] ca = gs.getCharacterArray ( false );
+    private static boolean inFinalContext(String script, String language, String feature, GlyphSequence gs, int index, int flags) {
+        CharAssociation a = gs.getAssociation(index);
+        int[] ca = gs.getCharacterArray(false);
         int   nc = gs.getCharacterCount();
-        if ( nc == 0 ) {
+        if (nc == 0) {
             return false;
         } else {
             int s = a.getStart();
             int e = a.getEnd();
-            if ( ! hasFinalPrecedingContext ( ca, nc, s, e ) ) {
+            if (!hasFinalPrecedingContext(ca, nc, s, e)) {
                 return false;
-            } else if ( forcesFinalThisContext ( ca, nc, s, e ) ) {
+            } else if (!hasFinalThisContext(ca, nc, s, e)) {
+                return false;
+            } else if (forceFinalThisContext(ca, nc, s, e)) {
                 return true;
-            } else if ( ! hasFinalFollowingContext ( ca, nc, s, e ) ) {
+            } else if (!hasFinalSucceedingContext(ca, nc, s, e)) {
                 return false;
             } else {
                 return true;
@@ -171,18 +171,20 @@ public class ArabicScriptProcessor extends DefaultScriptProcessor {
         }
     }
 
-    private static boolean inInitialContext ( String script, String language, String feature, GlyphSequence gs, int index, int flags ) {
-        GlyphSequence.CharAssociation a = gs.getAssociation ( index );
-        int[] ca = gs.getCharacterArray ( false );
+    private static boolean inInitialContext(String script, String language, String feature, GlyphSequence gs, int index, int flags) {
+        CharAssociation a = gs.getAssociation(index);
+        int[] ca = gs.getCharacterArray(false);
         int   nc = gs.getCharacterCount();
-        if ( nc == 0 ) {
+        if (nc == 0) {
             return false;
         } else {
             int s = a.getStart();
             int e = a.getEnd();
-            if ( ! hasInitialPrecedingContext ( ca, nc, s, e ) ) {
+            if (!hasInitialPrecedingContext(ca, nc, s, e)) {
                 return false;
-            } else if ( ! hasInitialFollowingContext ( ca, nc, s, e ) ) {
+            } else if (!hasInitialThisContext(ca, nc, s, e)) {
+                return false;
+            } else if (!hasInitialSucceedingContext(ca, nc, s, e)) {
                 return false;
             } else {
                 return true;
@@ -190,30 +192,30 @@ public class ArabicScriptProcessor extends DefaultScriptProcessor {
         }
     }
 
-    private static boolean inIsolateContext ( String script, String language, String feature, GlyphSequence gs, int index, int flags ) {
-        GlyphSequence.CharAssociation a = gs.getAssociation ( index );
+    private static boolean inIsolateContext(String script, String language, String feature, GlyphSequence gs, int index, int flags) {
+        CharAssociation a = gs.getAssociation(index);
         int   nc = gs.getCharacterCount();
-        if ( nc == 0 ) {
+        if (nc == 0) {
             return false;
-        } else if ( ( a.getStart() == 0 ) && ( a.getEnd() == nc ) ) {
+        } else if ((a.getStart() == 0) && (a.getEnd() == nc)) {
             return true;
         } else {
             return false;
         }
     }
 
-    private static boolean inLigatureContext ( String script, String language, String feature, GlyphSequence gs, int index, int flags ) {
-        GlyphSequence.CharAssociation a = gs.getAssociation ( index );
-        int[] ca = gs.getCharacterArray ( false );
+    private static boolean inLigatureContext(String script, String language, String feature, GlyphSequence gs, int index, int flags) {
+        CharAssociation a = gs.getAssociation(index);
+        int[] ca = gs.getCharacterArray(false);
         int   nc = gs.getCharacterCount();
-        if ( nc == 0 ) {
+        if (nc == 0) {
             return false;
         } else {
             int s = a.getStart();
             int e = a.getEnd();
-            if ( ! hasLigaturePrecedingContext ( ca, nc, s, e ) ) {
+            if (!hasLigaturePrecedingContext(ca, nc, s, e)) {
                 return false;
-            } else if ( ! hasLigatureFollowingContext ( ca, nc, s, e ) ) {
+            } else if (!hasLigatureSucceedingContext(ca, nc, s, e)) {
                 return false;
             } else {
                 return true;
@@ -221,20 +223,20 @@ public class ArabicScriptProcessor extends DefaultScriptProcessor {
         }
     }
 
-    private static boolean inMedialContext ( String script, String language, String feature, GlyphSequence gs, int index, int flags ) {
-        GlyphSequence.CharAssociation a = gs.getAssociation ( index );
-        int[] ca = gs.getCharacterArray ( false );
+    private static boolean inMedialContext(String script, String language, String feature, GlyphSequence gs, int index, int flags) {
+        CharAssociation a = gs.getAssociation(index);
+        int[] ca = gs.getCharacterArray(false);
         int   nc = gs.getCharacterCount();
-        if ( nc == 0 ) {
+        if (nc == 0) {
             return false;
         } else {
             int s = a.getStart();
             int e = a.getEnd();
-            if ( ! hasMedialPrecedingContext ( ca, nc, s, e ) ) {
+            if (!hasMedialPrecedingContext(ca, nc, s, e)) {
                 return false;
-            } else if ( ! hasMedialThisContext ( ca, nc, s, e ) ) {
+            } else if (!hasMedialThisContext(ca, nc, s, e)) {
                 return false;
-            } else if ( ! hasMedialFollowingContext ( ca, nc, s, e ) ) {
+            } else if (!hasMedialSucceedingContext(ca, nc, s, e)) {
                 return false;
             } else {
                 return true;
@@ -242,209 +244,257 @@ public class ArabicScriptProcessor extends DefaultScriptProcessor {
         }
     }
 
-    private static boolean hasFinalPrecedingContext ( int[] ca, int nc, int s, int e ) {
-        int chp = 0;
+    private static boolean hasFinalPrecedingContext(int[] ca, int nc, int s, int e) {
+        int chp = 0;    // preceding non-NSM char in [0,s) searching back from s
         int clp = 0;
-        for ( int i = s; i > 0; i-- ) {
+        for (int i = s; i > 0; i--) {
             int k = i - 1;
-            if ( ( k >= 0 ) && ( k < nc ) ) {
+            if ((k >= 0) && (k < nc)) {
                 chp = ca [ k ];
-                clp = BidiClass.getBidiClass ( chp );
-                if ( clp != BidiConstants.NSM ) {
+                clp = BidiClass.getBidiClass(chp);
+                if (clp != BidiConstants.NSM) {
                     break;
                 }
             }
         }
-        if ( clp != BidiConstants.AL ) {
-            return false;
-        } else if ( hasIsolateInitial ( chp ) ) {
+        if (clp != BidiConstants.AL) {
+            return isZWJ(chp);
+        } else if (hasIsolateInitial(chp)) {
             return false;
         } else {
             return true;
         }
     }
 
-    private static boolean forcesFinalThisContext ( int[] ca, int nc, int s, int e ) {
-        int chl = 0;
+    private static boolean hasFinalThisContext(int[] ca, int nc, int s, int e) {
+        int chl = 0;    // last non-{NSM,ZWJ} char in [s,e)
         int cll = 0;
-        for ( int i = 0, n = e - s; i < n; i++ ) {
+        for (int i = 0, n = e - s; i < n; i++) {
             int k = n - i - 1;
             int j = s + k;
-            if ( ( j >= 0 ) && ( j < nc ) ) {
+            if ((j >= 0) && (j < nc)) {
                 chl = ca [ j ];
-                cll = BidiClass.getBidiClass ( chl );
-                if ( cll != BidiConstants.NSM ) {
+                cll = BidiClass.getBidiClass(chl);
+                if ((cll != BidiConstants.NSM) && !isZWJ(chl)) {
                     break;
                 }
             }
         }
-        if ( cll != BidiConstants.AL ) {
+        if (cll != BidiConstants.AL) {
             return false;
         }
-        if ( hasIsolateInitial ( chl ) ) {
+        if (hasIsolateFinal(chl)) {
+            return false;
+        } else {
+            return true;
+        }
+    }
+
+    private static boolean forceFinalThisContext(int[] ca, int nc, int s, int e) {
+        int chl = 0;    // last non-{NSM,ZWJ} char in [s,e)
+        int cll = 0;
+        for (int i = 0, n = e - s; i < n; i++) {
+            int k = n - i - 1;
+            int j = s + k;
+            if ((j >= 0) && (j < nc)) {
+                chl = ca [ j ];
+                cll = BidiClass.getBidiClass(chl);
+                if ((cll != BidiConstants.NSM) && !isZWJ(chl)) {
+                    break;
+                }
+            }
+        }
+        if (cll != BidiConstants.AL) {
+            return false;
+        }
+        if (hasIsolateInitial(chl)) {
             return true;
         } else {
             return false;
         }
     }
 
-    private static boolean hasFinalFollowingContext ( int[] ca, int nc, int s, int e ) {
-        int chf = 0;
-        int clf = 0;
-        for ( int i = e, n = nc; i < n; i++ ) {
-            chf = ca [ i ];
-            clf = BidiClass.getBidiClass ( chf );
-            if ( clf != BidiConstants.NSM ) {
+    private static boolean hasFinalSucceedingContext(int[] ca, int nc, int s, int e) {
+        int chs = 0;    // succeeding non-NSM char in [e,nc) searching forward from e
+        int cls = 0;
+        for (int i = e, n = nc; i < n; i++) {
+            chs = ca [ i ];
+            cls = BidiClass.getBidiClass(chs);
+            if (cls != BidiConstants.NSM) {
                 break;
             }
         }
-        if ( clf != BidiConstants.AL ) {
-            return true;
-        } else if ( hasIsolateFinal ( chf ) ) {
+        if (cls != BidiConstants.AL) {
+            return !isZWJ(chs);
+        } else if (hasIsolateFinal(chs)) {
             return true;
         } else {
             return false;
         }
     }
 
-    private static boolean hasInitialPrecedingContext ( int[] ca, int nc, int s, int e ) {
-        int chp = 0;
+    private static boolean hasInitialPrecedingContext(int[] ca, int nc, int s, int e) {
+        int chp = 0;    // preceding non-NSM char in [0,s) searching back from s
         int clp = 0;
-        for ( int i = s; i > 0; i-- ) {
+        for (int i = s; i > 0; i--) {
             int k = i - 1;
-            if ( ( k >= 0 ) && ( k < nc ) ) {
+            if ((k >= 0) && (k < nc)) {
                 chp = ca [ k ];
-                clp = BidiClass.getBidiClass ( chp );
-                if ( clp != BidiConstants.NSM ) {
+                clp = BidiClass.getBidiClass(chp);
+                if (clp != BidiConstants.NSM) {
                     break;
                 }
             }
         }
-        if ( clp != BidiConstants.AL ) {
-            return true;
-        } else if ( hasIsolateInitial ( chp ) ) {
+        if (clp != BidiConstants.AL) {
+            return !isZWJ(chp);
+        } else if (hasIsolateInitial(chp)) {
             return true;
         } else {
             return false;
         }
     }
 
-    private static boolean hasInitialFollowingContext ( int[] ca, int nc, int s, int e ) {
-        int chf = 0;
+    private static boolean hasInitialThisContext(int[] ca, int nc, int s, int e) {
+        int chf = 0;    // first non-{NSM,ZWJ} char in [s,e)
         int clf = 0;
-        for ( int i = e, n = nc; i < n; i++ ) {
-            chf = ca [ i ];
-            clf = BidiClass.getBidiClass ( chf );
-            if ( clf != BidiConstants.NSM ) {
-                break;
-            }
-        }
-        if ( clf != BidiConstants.AL ) {
-            return false;
-        } else if ( hasIsolateFinal ( chf ) ) {
-            return false;
-        } else {
-            return true;
-        }
-    }
-
-    private static boolean hasMedialPrecedingContext ( int[] ca, int nc, int s, int e ) {
-        int chp = 0;
-        int clp = 0;
-        for ( int i = s; i > 0; i-- ) {
-            int k = i - 1;
-            if ( ( k >= 0 ) && ( k < nc ) ) {
-                chp = ca [ k ];
-                clp = BidiClass.getBidiClass ( chp );
-                if ( clp != BidiConstants.NSM ) {
-                    break;
-                }
-            }
-        }
-        if ( clp != BidiConstants.AL ) {
-            return false;
-        } else if ( hasIsolateInitial ( chp ) ) {
-            return false;
-        } else {
-            return true;
-        }
-    }
-
-    private static boolean hasMedialThisContext ( int[] ca, int nc, int s, int e ) {
-        int chf = 0;    // first non-NSM char in [s,e)
-        int clf = 0;
-        for ( int i = 0, n = e - s; i < n; i++ ) {
+        for (int i = 0, n = e - s; i < n; i++) {
             int k = s + i;
-            if ( ( k >= 0 ) && ( k < nc ) ) {
+            if ((k >= 0) && (k < nc)) {
                 chf = ca [ s + i ];
-                clf = BidiClass.getBidiClass ( chf );
-                if ( clf != BidiConstants.NSM ) {
+                clf = BidiClass.getBidiClass(chf);
+                if ((clf != BidiConstants.NSM) && !isZWJ(chf)) {
                     break;
                 }
             }
         }
-        if ( clf != BidiConstants.AL ) {
+        if (clf != BidiConstants.AL) {
             return false;
         }
-        int chl = 0;    // last non-NSM char in [s,e)
-        int cll = 0;
-        for ( int i = 0, n = e - s; i < n; i++ ) {
-            int k = n - i - 1;
-            int j = s + k;
-            if ( ( j >= 0 ) && ( j < nc ) ) {
-                chl = ca [ j ];
-                cll = BidiClass.getBidiClass ( chl );
-                if ( cll != BidiConstants.NSM ) {
-                    break;
-                }
-            }
-        }
-        if ( cll != BidiConstants.AL ) {
-            return false;
-        }
-        if ( hasIsolateFinal ( chf ) ) {
-            return false;
-        } else if ( hasIsolateInitial ( chl ) ) {
+        if (hasIsolateInitial(chf)) {
             return false;
         } else {
             return true;
         }
     }
 
-    private static boolean hasMedialFollowingContext ( int[] ca, int nc, int s, int e ) {
-        int chf = 0;
-        int clf = 0;
-        for ( int i = e, n = nc; i < n; i++ ) {
-            chf = ca [ i ];
-            clf = BidiClass.getBidiClass ( chf );
-            if ( clf != BidiConstants.NSM ) {
+    private static boolean hasInitialSucceedingContext(int[] ca, int nc, int s, int e) {
+        int chs = 0;    // succeeding non-NSM char in [e,nc) searching forward from e
+        int cls = 0;
+        for (int i = e, n = nc; i < n; i++) {
+            chs = ca [ i ];
+            cls = BidiClass.getBidiClass(chs);
+            if (cls != BidiConstants.NSM) {
                 break;
             }
         }
-        if ( clf != BidiConstants.AL ) {
-            return false;
-        } else if ( hasIsolateFinal ( chf ) ) {
+        if (cls != BidiConstants.AL) {
+            return isZWJ(chs);
+        } else if (hasIsolateFinal(chs)) {
             return false;
         } else {
             return true;
         }
     }
 
-    private static boolean hasLigaturePrecedingContext ( int[] ca, int nc, int s, int e ) {
+    private static boolean hasMedialPrecedingContext(int[] ca, int nc, int s, int e) {
+        int chp = 0;    // preceding non-NSM char in [0,s) searching back from s
+        int clp = 0;
+        for (int i = s; i > 0; i--) {
+            int k = i - 1;
+            if ((k >= 0) && (k < nc)) {
+                chp = ca [ k ];
+                clp = BidiClass.getBidiClass(chp);
+                if (clp != BidiConstants.NSM) {
+                    break;
+                }
+            }
+        }
+        if (clp != BidiConstants.AL) {
+            return isZWJ(chp);
+        } else if (hasIsolateInitial(chp)) {
+            return false;
+        } else {
+            return true;
+        }
+    }
+
+    private static boolean hasMedialThisContext(int[] ca, int nc, int s, int e) {
+        int chf = 0;    // first non-{NSM,ZWJ} char in [s,e)
+        int clf = 0;
+        for (int i = 0, n = e - s; i < n; i++) {
+            int k = s + i;
+            if ((k >= 0) && (k < nc)) {
+                chf = ca [ s + i ];
+                clf = BidiClass.getBidiClass(chf);
+                if ((clf != BidiConstants.NSM) && !isZWJ(chf)) {
+                    break;
+                }
+            }
+        }
+        if (clf != BidiConstants.AL) {
+            return false;
+        }
+        int chl = 0;    // last non-{NSM,ZWJ} char in [s,e)
+        int cll = 0;
+        for (int i = 0, n = e - s; i < n; i++) {
+            int k = n - i - 1;
+            int j = s + k;
+            if ((j >= 0) && (j < nc)) {
+                chl = ca [ j ];
+                cll = BidiClass.getBidiClass(chl);
+                if ((cll != BidiConstants.NSM) && !isZWJ(chl)) {
+                    break;
+                }
+            }
+        }
+        if (cll != BidiConstants.AL) {
+            return false;
+        }
+        if (hasIsolateFinal(chf)) {
+            return false;
+        } else if (hasIsolateInitial(chl)) {
+            return false;
+        } else {
+            return true;
+        }
+    }
+
+    private static boolean hasMedialSucceedingContext(int[] ca, int nc, int s, int e) {
+        int chs = 0;    // succeeding non-NSM char in [e,nc) searching forward from e
+        int cls = 0;
+        for (int i = e, n = nc; i < n; i++) {
+            chs = ca [ i ];
+            cls = BidiClass.getBidiClass(chs);
+            if (cls != BidiConstants.NSM) {
+                break;
+            }
+        }
+        if (cls != BidiConstants.AL) {
+            return isZWJ(chs);
+        } else if (hasIsolateFinal(chs)) {
+            return false;
+        } else {
+            return true;
+        }
+    }
+
+    private static boolean hasLigaturePrecedingContext(int[] ca, int nc, int s, int e) {
         return true;
     }
 
-    private static boolean hasLigatureFollowingContext ( int[] ca, int nc, int s, int e ) {
-        int chf = 0;
-        int clf = 0;
-        for ( int i = e, n = nc; i < n; i++ ) {
-            chf = ca [ i ];
-            clf = BidiClass.getBidiClass ( chf );
-            if ( clf != BidiConstants.NSM ) {
+    private static boolean hasLigatureSucceedingContext(int[] ca, int nc, int s, int e) {
+        int chs = 0;    // succeeding non-NSM char in [e,nc) searching forward from e
+        int cls = 0;
+        for (int i = e, n = nc; i < n; i++) {
+            chs = ca [ i ];
+            cls = BidiClass.getBidiClass(chs);
+            // TBD - does ZWJ have impact here?
+            if (cls != BidiConstants.NSM) {
                 break;
             }
         }
-        if ( clf == BidiConstants.AL ) {
+        if (cls == BidiConstants.AL) {
             return true;
         } else  {
             return false;
@@ -455,7 +505,7 @@ public class ArabicScriptProcessor extends DefaultScriptProcessor {
      * Ordered array of Unicode scalars designating those Arabic (Script) Letters
      * which exhibit an isolated form in word initial position.
      */
-    private static int[] isolatedInitials = {
+    private static final int[] ISOLATED_INITIALS = {
         0x0621, // HAMZA
         0x0622, // ALEF WITH MADDA ABOVE
         0x0623, // ALEF WITH HAMZA ABOVE
@@ -504,20 +554,24 @@ public class ArabicScriptProcessor extends DefaultScriptProcessor {
         0x06EF  // REH WITH INVERTED V
     };
 
-    private static boolean hasIsolateInitial ( int ch ) {
-        return Arrays.binarySearch ( isolatedInitials, ch ) >= 0;
+    private static boolean hasIsolateInitial(int ch) {
+        return Arrays.binarySearch(ISOLATED_INITIALS, ch) >= 0;
     }
 
     /**
      * Ordered array of Unicode scalars designating those Arabic (Script) Letters
      * which exhibit an isolated form in word final position.
      */
-    private static int[] isolatedFinals = {
+    private static final int[] ISOLATED_FINALS = {
         0x0621  // HAMZA
     };
 
-    private static boolean hasIsolateFinal ( int ch ) {
-        return Arrays.binarySearch ( isolatedFinals, ch ) >= 0;
+    private static boolean hasIsolateFinal(int ch) {
+        return Arrays.binarySearch(ISOLATED_FINALS, ch) >= 0;
+    }
+
+    private static boolean isZWJ(int ch) {
+        return ch == CharUtilities.ZERO_WIDTH_JOINER;
     }
 
 }
