@@ -22,13 +22,9 @@ package org.apache.fop.complexscripts.scripts;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.fop.complexscripts.util.CharAssociation;
 import org.apache.fop.complexscripts.util.GlyphSequence;
 
-// CSOFF: AvoidNestedBlocksCheck
-// CSOFF: NoWhitespaceAfterCheck
-// CSOFF: WhitespaceAfter
-// CSOFF: InnerAssignmentCheck
-// CSOFF: SimplifyBooleanReturnCheck
 // CSOFF: LineLengthCheck
 
 /**
@@ -40,10 +36,10 @@ import org.apache.fop.complexscripts.util.GlyphSequence;
 public class DevanagariScriptProcessor extends IndicScriptProcessor {
 
     /** logging instance */
-    private static final Log log = LogFactory.getLog(DevanagariScriptProcessor.class);                                  // CSOK: ConstantNameCheck
+    private static final Log log = LogFactory.getLog(DevanagariScriptProcessor.class);
 
-    DevanagariScriptProcessor ( String script ) {
-        super ( script );
+    DevanagariScriptProcessor(String script) {
+        super(script);
     }
 
     @Override
@@ -53,12 +49,12 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
 
     @Override
     // find rightmost pre-base matra
-    protected int findPreBaseMatra ( GlyphSequence gs ) {
+    protected int findPreBaseMatra(GlyphSequence gs) {
         int   ng = gs.getGlyphCount();
         int   lk = -1;
-        for ( int i = ng; i > 0; i-- ) {
+        for (int i = ng; i > 0; i--) {
             int k = i - 1;
-            if ( containsPreBaseMatra ( gs, k ) ) {
+            if (containsPreBaseMatra(gs, k)) {
                 lk = k;
                 break;
             }
@@ -68,15 +64,15 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
 
     @Override
     // find leftmost pre-base matra target, starting from source
-    protected int findPreBaseMatraTarget ( GlyphSequence gs, int source ) {
+    protected int findPreBaseMatraTarget(GlyphSequence gs, int source) {
         int   ng = gs.getGlyphCount();
         int   lk = -1;
-        for ( int i = ( source < ng ) ? source : ng; i > 0; i-- ) {
+        for (int i = (source < ng) ? source : ng; i > 0; i--) {
             int k = i - 1;
-            if ( containsConsonant ( gs, k ) ) {
-                if ( containsHalfConsonant ( gs, k ) ) {
+            if (containsConsonant(gs, k)) {
+                if (containsHalfConsonant(gs, k)) {
                     lk = k;
-                } else if ( lk == -1 ) {
+                } else if (lk == -1) {
                     lk = k;
                 } else {
                     break;
@@ -86,39 +82,39 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
         return lk;
     }
 
-    private static boolean containsPreBaseMatra ( GlyphSequence gs, int k ) {
-        GlyphSequence.CharAssociation a = gs.getAssociation ( k );
-        int[] ca = gs.getCharacterArray ( false );
-        for ( int i = a.getStart(), e = a.getEnd(); i < e; i++ ) {
-            if ( isPreM ( ca [ i ] ) ) {
+    private static boolean containsPreBaseMatra(GlyphSequence gs, int k) {
+        CharAssociation a = gs.getAssociation(k);
+        int[] ca = gs.getCharacterArray(false);
+        for (int i = a.getStart(), e = a.getEnd(); i < e; i++) {
+            if (isPreM(ca [ i ])) {
                 return true;
             }
         }
         return false;
     }
 
-    private static boolean containsConsonant ( GlyphSequence gs, int k ) {
-        GlyphSequence.CharAssociation a = gs.getAssociation ( k );
-        int[] ca = gs.getCharacterArray ( false );
-        for ( int i = a.getStart(), e = a.getEnd(); i < e; i++ ) {
-            if ( isC ( ca [ i ] ) ) {
+    private static boolean containsConsonant(GlyphSequence gs, int k) {
+        CharAssociation a = gs.getAssociation(k);
+        int[] ca = gs.getCharacterArray(false);
+        for (int i = a.getStart(), e = a.getEnd(); i < e; i++) {
+            if (isC(ca [ i ])) {
                 return true;
             }
         }
         return false;
     }
 
-    private static boolean containsHalfConsonant ( GlyphSequence gs, int k ) {
-        Boolean half = (Boolean) gs.getAssociation ( k ) . getPredication ( "half" );
-        return ( half != null ) ? half.booleanValue() : false;
+    private static boolean containsHalfConsonant(GlyphSequence gs, int k) {
+        Boolean half = (Boolean) gs.getAssociation(k) .getPredication("half");
+        return (half != null) ? half.booleanValue() : false;
     }
 
     @Override
-    protected int findReph ( GlyphSequence gs ) {
+    protected int findReph(GlyphSequence gs) {
         int   ng = gs.getGlyphCount();
         int   li = -1;
-        for ( int i = 0; i < ng; i++ ) {
-            if ( containsReph ( gs, i ) ) {
+        for (int i = 0; i < ng; i++) {
+            if (containsReph(gs, i)) {
                 li = i;
                 break;
             }
@@ -127,58 +123,58 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
     }
 
     @Override
-    protected int findRephTarget ( GlyphSequence gs, int source ) {
+    protected int findRephTarget(GlyphSequence gs, int source) {
         int   ng = gs.getGlyphCount();
         int   c1 = -1;
         int   c2 = -1;
         // first candidate target is after first non-half consonant
-        for ( int i = 0; i < ng; i++ ) {
-            if ( ( i != source ) && containsConsonant ( gs, i ) ) {
-                if ( ! containsHalfConsonant ( gs, i ) ) {
+        for (int i = 0; i < ng; i++) {
+            if ((i != source) && containsConsonant(gs, i)) {
+                if (!containsHalfConsonant(gs, i)) {
                     c1 = i + 1;
                     break;
                 }
             }
         }
         // second candidate target is after last non-prebase matra after first candidate or before first syllable or vedic mark
-        for ( int i = ( c1 >= 0 ) ? c1 : 0; i < ng; i++ ) {
-            if ( containsMatra ( gs, i ) && ! containsPreBaseMatra ( gs, i ) ) {
+        for (int i = (c1 >= 0) ? c1 : 0; i < ng; i++) {
+            if (containsMatra(gs, i) && !containsPreBaseMatra(gs, i)) {
                 c2 = i + 1;
-            } else if ( containsOtherMark ( gs, i ) ) {
+            } else if (containsOtherMark(gs, i)) {
                 c2 = i;
                 break;
             }
         }
-        if ( c2 >= 0 ) {
+        if (c2 >= 0) {
             return c2;
-        } else if ( c1 >= 0 ) {
+        } else if (c1 >= 0) {
             return c1;
         } else {
             return source;
         }
     }
 
-    private static boolean containsReph ( GlyphSequence gs, int k ) {
-        Boolean rphf = (Boolean) gs.getAssociation ( k ) . getPredication ( "rphf" );
-        return ( rphf != null ) ? rphf.booleanValue() : false;
+    private static boolean containsReph(GlyphSequence gs, int k) {
+        Boolean rphf = (Boolean) gs.getAssociation(k) .getPredication("rphf");
+        return (rphf != null) ? rphf.booleanValue() : false;
     }
 
-    private static boolean containsMatra ( GlyphSequence gs, int k ) {
-        GlyphSequence.CharAssociation a = gs.getAssociation ( k );
-        int[] ca = gs.getCharacterArray ( false );
-        for ( int i = a.getStart(), e = a.getEnd(); i < e; i++ ) {
-            if ( isM ( ca [ i ] ) ) {
+    private static boolean containsMatra(GlyphSequence gs, int k) {
+        CharAssociation a = gs.getAssociation(k);
+        int[] ca = gs.getCharacterArray(false);
+        for (int i = a.getStart(), e = a.getEnd(); i < e; i++) {
+            if (isM(ca [ i ])) {
                 return true;
             }
         }
         return false;
     }
 
-    private static boolean containsOtherMark ( GlyphSequence gs, int k ) {
-        GlyphSequence.CharAssociation a = gs.getAssociation ( k );
-        int[] ca = gs.getCharacterArray ( false );
-        for ( int i = a.getStart(), e = a.getEnd(); i < e; i++ ) {
-            switch ( typeOf ( ca [ i ] ) ) {
+    private static boolean containsOtherMark(GlyphSequence gs, int k) {
+        CharAssociation a = gs.getAssociation(k);
+        int[] ca = gs.getCharacterArray(false);
+        for (int i = a.getStart(), e = a.getEnd(); i < e; i++) {
+            switch (typeOf(ca [ i ])) {
             case C_T:   // tone (e.g., udatta, anudatta)
             case C_A:   // accent (e.g., acute, grave)
             case C_O:   // other (e.g., candrabindu, anusvara, visarga, etc)
@@ -191,18 +187,18 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
     }
 
     private static class DevanagariSyllabizer extends DefaultSyllabizer {
-        DevanagariSyllabizer ( String script, String language ) {
-            super ( script, language );
+        DevanagariSyllabizer(String script, String language) {
+            super(script, language);
         }
         @Override
         // | C ...
-        protected int findStartOfSyllable ( int[] ca, int s, int e ) {
-            if ( ( s < 0 ) || ( s >= e ) ) {
+        protected int findStartOfSyllable(int[] ca, int s, int e) {
+            if ((s < 0) || (s >= e)) {
                 return -1;
             } else {
-                while ( s < e ) {
+                while (s < e) {
                     int c = ca [ s ];
-                    if ( isC ( c ) ) {
+                    if (isC(c)) {
                         break;
                     } else {
                         s++;
@@ -213,29 +209,29 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
         }
         @Override
         // D* L? | ...
-        protected int findEndOfSyllable ( int[] ca, int s, int e ) {
-            if ( ( s < 0 ) || ( s >= e ) ) {
+        protected int findEndOfSyllable(int[] ca, int s, int e) {
+            if ((s < 0) || (s >= e)) {
                 return -1;
             } else {
                 int nd = 0;
                 int nl = 0;
                 int i;
                 // consume dead consonants
-                while ( ( i = isDeadConsonant ( ca, s, e ) ) > s ) {
+                while ((i = isDeadConsonant(ca, s, e)) > s) {
                     s = i;
                     nd++;
                 }
                 // consume zero or one live consonant
-                if ( ( i = isLiveConsonant ( ca, s, e ) ) > s ) {
+                if ((i = isLiveConsonant(ca, s, e)) > s) {
                     s = i;
                     nl++;
                 }
-                return ( ( nd > 0 ) || ( nl > 0 ) ) ? s : -1;
+                return ((nd > 0) || (nl > 0)) ? s : -1;
             }
         }
         // D := ( C N? H )?
-        private int isDeadConsonant ( int[] ca, int s, int e ) {
-            if ( s < 0 ) {
+        private int isDeadConsonant(int[] ca, int s, int e) {
+            if (s < 0) {
                 return -1;
             } else {
                 int c;
@@ -244,9 +240,9 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
                 int nh = 0;
                 do {
                     // C
-                    if ( ( s + i ) < e ) {
+                    if ((s + i) < e) {
                         c = ca [ s + i ];
-                        if ( isC ( c ) ) {
+                        if (isC(c)) {
                             i++;
                             nc++;
                         } else {
@@ -254,29 +250,29 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
                         }
                     }
                     // N?
-                    if ( ( s + i ) < e ) {
+                    if ((s + i) < e) {
                         c = ca [ s + 1 ];
-                        if ( isN ( c ) ) {
+                        if (isN(c)) {
                             i++;
                         }
                     }
                     // H
-                    if ( ( s + i ) < e ) {
+                    if ((s + i) < e) {
                         c = ca [ s + i ];
-                        if ( isH ( c ) ) {
+                        if (isH(c)) {
                             i++;
                             nh++;
                         } else {
                             break;
                         }
                     }
-                } while ( false );
-                return ( nc > 0 ) && ( nh > 0 ) ? s + i : -1;
+                } while (false);
+                return (nc > 0) && (nh > 0) ? s + i : -1;
             }
         }
         // L := ( (C|V) N? X* )?; where X = ( MATRA | ACCENT MARK | TONE MARK | OTHER MARK )
-        private int isLiveConsonant ( int[] ca, int s, int e ) {
-            if ( s < 0 ) {
+        private int isLiveConsonant(int[] ca, int s, int e) {
+            if (s < 0) {
                 return -1;
             } else {
                 int c;
@@ -286,12 +282,12 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
                 int nx = 0;
                 do {
                     // C
-                    if ( ( s + i ) < e ) {
+                    if ((s + i) < e) {
                         c = ca [ s + i ];
-                        if ( isC ( c ) ) {
+                        if (isC(c)) {
                             i++;
                             nc++;
-                        } else if ( isV ( c ) ) {
+                        } else if (isV(c)) {
                             i++;
                             nv++;
                         } else {
@@ -299,37 +295,37 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
                         }
                     }
                     // N?
-                    if ( ( s + i ) < e ) {
+                    if ((s + i) < e) {
                         c = ca [ s + i ];
-                        if ( isN ( c ) ) {
+                        if (isN(c)) {
                             i++;
                         }
                     }
                     // X*
-                    while ( ( s + i ) < e ) {
+                    while ((s + i) < e) {
                         c = ca [ s + i ];
-                        if ( isX ( c ) ) {
+                        if (isX(c)) {
                             i++;
                             nx++;
                         } else {
                             break;
                         }
                     }
-                } while ( false );
+                } while (false);
                 // if no X but has H, then ignore C|I
-                if ( nx == 0 ) {
-                    if ( ( s + i ) < e ) {
+                if (nx == 0) {
+                    if ((s + i) < e) {
                         c = ca [ s + i ];
-                        if ( isH ( c ) ) {
-                            if ( nc > 0 ) {
+                        if (isH(c)) {
+                            if (nc > 0) {
                                 nc--;
-                            } else if ( nv > 0 ) {
+                            } else if (nv > 0) {
                                 nv--;
                             }
                         }
                     }
                 }
-                return ( ( nc > 0 ) || ( nv > 0 ) ) ? s + i : -1;
+                return ((nc > 0) || (nv > 0)) ? s + i : -1;
             }
         }
     }
@@ -352,10 +348,10 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
     static final short C_M_TYPE     = 0x00FF;       // type mask
     static final short C_M_FLAGS    = 0x7F00;       // flag mask
     // devanagari block range
-    static final int ccaStart       =  0x0900;      // first code point mapped by cca                                   // CSOK: ConstantNameCheck
-    static final int ccaEnd         =  0x0980;      // last code point + 1 mapped by cca                                // CSOK: ConstantNameCheck
+    static final int CCA_START       =  0x0900;      // first code point mapped by cca
+    static final int CCA_END         =  0x0980;      // last code point + 1 mapped by cca
     // devanagari character type lookups
-    static final short[] cca = {                                                                                        // CSOK: ConstantNameCheck
+    static final short[] CCA = {
         C_O,                        // 0x0900       // INVERTED CANDRABINDU
         C_O,                        // 0x0901       // CANDRABINDU
         C_O,                        // 0x0902       // ANUSVARA
@@ -404,8 +400,8 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
         C_C,                        // 0x092D       // BHA
         C_C,                        // 0x092E       // MA
         C_C,                        // 0x092F       // YA
-        C_C|C_R,                    // 0x0930       // RA                                                               // CSOK: WhitespaceAround
-        C_C|C_R|C_N,                // 0x0931       // RRA          = 0930+093C                                         // CSOK: WhitespaceAround
+        C_C | C_R,                  // 0x0930       // RA
+        C_C | C_R | C_N,            // 0x0931       // RRA          = 0930+093C
         C_C,                        // 0x0932       // LA
         C_C,                        // 0x0933       // LLA
         C_C,                        // 0x0934       // LLLA
@@ -419,7 +415,7 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
         C_N,                        // 0x093C       // NUKTA
         C_S,                        // 0x093D       // AVAGRAHA
         C_M,                        // 0x093E       // AA
-        C_M|C_PRE,                  // 0x093F       // I                                                               // CSOK: WhitespaceAround
+        C_M | C_PRE,                // 0x093F       // I
         C_M,                        // 0x0940       // II
         C_M,                        // 0x0941       // U
         C_M,                        // 0x0942       // UU
@@ -444,14 +440,14 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
         C_M,                        // 0x0955       // CANDRA LONG E
         C_M,                        // 0x0956       // UE
         C_M,                        // 0x0957       // UUE
-        C_C|C_N,                    // 0x0958       // QA                                                               // CSOK: WhitespaceAround
-        C_C|C_N,                    // 0x0959       // KHHA                                                             // CSOK: WhitespaceAround
-        C_C|C_N,                    // 0x095A       // GHHA                                                             // CSOK: WhitespaceAround
-        C_C|C_N,                    // 0x095B       // ZA                                                               // CSOK: WhitespaceAround
-        C_C|C_N,                    // 0x095C       // DDDHA                                                            // CSOK: WhitespaceAround
-        C_C|C_N,                    // 0x095D       // RHA                                                              // CSOK: WhitespaceAround
-        C_C|C_N,                    // 0x095E       // FA                                                               // CSOK: WhitespaceAround
-        C_C|C_N,                    // 0x095F       // YYA                                                              // CSOK: WhitespaceAround
+        C_C | C_N,                  // 0x0958       // QA
+        C_C | C_N,                  // 0x0959       // KHHA
+        C_C | C_N,                  // 0x095A       // GHHA
+        C_C | C_N,                  // 0x095B       // ZA
+        C_C | C_N,                  // 0x095C       // DDDHA
+        C_C | C_N,                  // 0x095D       // RHA
+        C_C | C_N,                  // 0x095E       // FA
+        C_C | C_N,                  // 0x095F       // YYA
         C_V,                        // 0x0960       // VOCALIC RR
         C_V,                        // 0x0961       // VOCALIC LL
         C_M,                        // 0x0962       // VOCALIC RR
@@ -486,30 +482,30 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
         C_C                         // 0x097F       // BBA (SINDHI)
     };
     static int typeOf(int c) {
-        if ( ( c >= ccaStart ) && ( c < ccaEnd ) ) {
-            return cca [ c - ccaStart ] & C_M_TYPE;
+        if ((c >= CCA_START) && (c < CCA_END)) {
+            return CCA [ c - CCA_START ] & C_M_TYPE;
         } else {
             return C_U;
         }
     }
     static boolean isType(int c, int t) {
-        return typeOf ( c ) == t;
+        return typeOf(c) == t;
     }
     static boolean hasFlag(int c, int f) {
-        if ( ( c >= ccaStart ) && ( c < ccaEnd ) ) {
-            return ( cca [ c - ccaStart ] & f ) == f;
+        if ((c >= CCA_START) && (c < CCA_END)) {
+            return (CCA [ c - CCA_START ] & f) == f;
         } else {
             return false;
         }
     }
     static boolean isC(int c) {
-        return isType(c,C_C);
+        return isType(c, C_C);
     }
     static boolean isR(int c) {
-        return isType(c,C_C) && hasR(c);
+        return isType(c, C_C) && hasR(c);
     }
     static boolean isV(int c) {
-        return isType(c,C_V);
+        return isType(c, C_V);
     }
     static boolean isN(int c) {
         return c == 0x093C;
@@ -518,13 +514,13 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
         return c == 0x094D;
     }
     static boolean isM(int c) {
-        return isType(c,C_M);
+        return isType(c, C_M);
     }
     static boolean isPreM(int c) {
-        return isType(c,C_M) && hasFlag(c,C_PRE);
+        return isType(c, C_M) && hasFlag(c, C_PRE);
     }
     static boolean isX(int c) {
-        switch ( typeOf ( c ) ) {
+        switch (typeOf(c)) {
         case C_M: // matra (combining vowel)
         case C_A: // accent mark
         case C_T: // tone mark
@@ -535,10 +531,10 @@ public class DevanagariScriptProcessor extends IndicScriptProcessor {
         }
     }
     static boolean hasR(int c) {
-        return hasFlag(c,C_R);
+        return hasFlag(c, C_R);
     }
     static boolean hasN(int c) {
-        return hasFlag(c,C_N);
+        return hasFlag(c, C_N);
     }
 
 }
