@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FODimension.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: FODimension.java 1480018 2013-05-07 18:53:29Z vhennebert $ */
 
 package org.apache.fop.datatypes;
 
@@ -41,6 +41,17 @@ public class FODimension {
         this.bpd = bpd;
     }
 
+    /** @return ipd */
+    public int getIPD() {
+        return ipd;
+    }
+
+    /** @return bpd */
+    public int getBPD() {
+        return bpd;
+    }
+
+    /** {@inheritDoc} */
     public String toString() {
         StringBuffer sb = new StringBuffer(super.toString());
         sb.append(" {ipd=").append(Integer.toString(ipd));
