@@ -15,16 +15,18 @@
  * limitations under the License.
  */
 
-/* $Id: LengthBase.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: LengthBase.java 1303891 2012-03-22 17:04:12Z vhennebert $ */
 
 package org.apache.fop.datatypes;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.expr.PropertyException;
+import org.apache.fop.util.CompareUtil;
 
 /**
  * Models a length which can be used as a factor in a percentage length
@@ -62,7 +64,7 @@ public class LengthBase implements PercentBase {
     /**
      * logging instance
      */
-    protected static Log log = LogFactory.getLog(LengthBase.class);
+    protected static final Log log = LogFactory.getLog(LengthBase.class);
 
     /**
      * The FO for which this property is to be calculated.
@@ -142,5 +144,30 @@ public class LengthBase implements PercentBase {
     public Length getBaseLength() {
         return baseLength;
     }
+
+    @Override
+    public int hashCode() {
+        final int prime = 31;
+        int result = 1;
+        result = prime * result + CompareUtil.getHashCode(baseLength);
+        result = prime * result + baseType;
+        result = prime * result + CompareUtil.getHashCode(fobj);
+        return result;
+    }
+
+    @Override
+    public boolean equals(Object obj) {
+        if (this == obj) {
+            return true;
+        }
+        if (!(obj instanceof LengthBase)) {
+            return false;
+        }
+        LengthBase other = (LengthBase) obj;
+        return CompareUtil.equal(baseLength, other.baseLength)
+                && baseType == other.baseType
+                && CompareUtil.equal(fobj, other.fobj);
+    }
+
 }
 
