@@ -15,12 +15,10 @@
  * limitations under the License.
  */
 
-/* $Id: Numeric.java 992386 2010-09-03 17:26:28Z vhennebert $ */
+/* $Id: Numeric.java 1477872 2013-04-30 23:25:18Z lbernardo $ */
 
 package org.apache.fop.datatypes;
 
-import org.apache.fop.fo.expr.PropertyException;
-
 /**
  * An interface for classes that can participate in numeric operations.
  * All the numeric operation (+, -, *, ...) are expressed in terms of
@@ -39,7 +37,7 @@ public interface Numeric {
      * @return the computed value.
      * @throws PropertyException if a property exception occurs
      */
-    double getNumericValue() throws PropertyException;
+    double getNumericValue();
 
     /**
      * Return the value of this Numeric
@@ -47,7 +45,7 @@ public interface Numeric {
      * @return the computed value.
      * @throws PropertyException if a property exception occurs
      */
-    double getNumericValue(PercentBaseContext context) throws PropertyException;
+    double getNumericValue(PercentBaseContext context);
 
     /**
      * Return the dimension of this numeric. Plain numbers has a dimension of
