@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: EventExceptionManager.java 1058988 2011-01-14 12:58:53Z spepping $ */
+/* $Id: EventExceptionManager.java 1502902 2013-07-13 22:26:09Z lbernardo $ */
 
 package org.apache.fop.events;
 
@@ -53,7 +53,7 @@ public final class EventExceptionManager {
      */
     public static void throwException(Event event, String exceptionClass) throws Throwable {
         if (exceptionClass != null) {
-            ExceptionFactory factory = (ExceptionFactory)EXCEPTION_FACTORIES.get(exceptionClass);
+            ExceptionFactory factory = EXCEPTION_FACTORIES.get(exceptionClass);
             if (factory != null) {
                 throw factory.createException(event);
             } else {
