@@ -15,13 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id: FONode.java 830293 2009-10-27 19:07:52Z vhennebert $ */
+/* $Id: FONode.java 1722664 2016-01-02 20:53:51Z adelmelle $ */
 
 package org.apache.fop.fo;
 
 // Java
 import java.util.ListIterator;
 import java.util.Map;
+import java.util.Stack;
 
 import org.xml.sax.Attributes;
 import org.xml.sax.Locator;
@@ -32,8 +33,11 @@ import org.apache.commons.logging.LogFactory;
 
 import org.apache.xmlgraphics.util.QName;
 
+import org.apache.fop.accessibility.StructureTreeElement;
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
+import org.apache.fop.complexscripts.bidi.DelimitedTextRange;
+import org.apache.fop.fo.expr.PropertyException;
 import org.apache.fop.fo.extensions.ExtensionAttachment;
 import org.apache.fop.fo.extensions.ExtensionElementMapping;
 import org.apache.fop.fo.extensions.InternalElementMapping;
@@ -69,7 +73,7 @@ public abstract class FONode implements Cloneable {
     protected Locator locator;
 
     /** Logger for fo-tree related messages **/
-    protected static Log log = LogFactory.getLog(FONode.class);
+    protected static final Log log = LogFactory.getLog(FONode.class);
 
     /**
      * Base constructor
@@ -306,7 +310,7 @@ public abstract class FONode implements Cloneable {
      *
      * @throws FOPException if there's a problem during processing
      */
-    protected void startOfNode() throws FOPException {
+    public void startOfNode() throws FOPException {
         // do nothing by default
    }
 
@@ -316,13 +320,13 @@ public abstract class FONode implements Cloneable {
      * has been reached.
      * The default implementation simply calls {@link #finalizeNode()}, without
      * sending any event to the {@link FOEventHandler}.
-     * <br/><i>Note: the recommended way to override this method in subclasses is</i>
-     * <br/><br/><code>super.endOfNode(); // invoke finalizeNode()
-     * <br/>getFOEventHandler().endXXX(); // send endOfNode() notification</code>
+     * <p><i>Note: the recommended way to override this method in subclasses is</i></p>
+     * <p><code>super.endOfNode(); // invoke finalizeNode()</code></p>
+     * <p><code>getFOEventHandler().endXXX(); // send endOfNode() notification</code></p>
      *
      * @throws FOPException if there's a problem during processing
      */
-    protected void endOfNode() throws FOPException {
+    public void endOfNode() throws FOPException {
         this.finalizeNode();
     }
 
@@ -400,6 +404,27 @@ public abstract class FONode implements Cloneable {
         return new OneCharIterator(CharUtilities.CODE_EOT);
     }
 
+    /**
+     * Helper function to obtain standard usage prefix for FOP related
+     * namespace URIs.
+     * @param namespaceURI URI of node found
+     *         (e.g., "http://www.w3.org/1999/XSL/Format")
+     * @return the prefix or null if none
+     */
+    public static String getNodePrefix(String namespaceURI) {
+        if (namespaceURI.equals(FOElementMapping.URI)) {
+            return "fo";
+        } else if (namespaceURI.equals(ExtensionElementMapping.URI)) {
+            return "fox";
+        } else if (namespaceURI.equals(InternalElementMapping.URI)) {
+            return "foi";
+        } else if (namespaceURI.equals(SVGElementMapping.URI)) {
+            return "svg";
+        } else {
+            return null;
+        }
+    }
+
     /**
      * Helper function to standardize the names of all namespace URI - local
      * name pairs in text messages.
@@ -412,14 +437,9 @@ public abstract class FONode implements Cloneable {
      * with the unabbreviated URI otherwise.
      */
     public static String getNodeString(String namespaceURI, String localName) {
-        if (namespaceURI.equals(FOElementMapping.URI)) {
-            return "fo:" + localName;
-        } else if (namespaceURI.equals(ExtensionElementMapping.URI)) {
-            return "fox:" + localName;
-        } else if (namespaceURI.equals(InternalElementMapping.URI)) {
-            return "foi:" + localName;  // used FOP internally for accessibility
-        } else if (namespaceURI.equals(SVGElementMapping.URI)) {
-            return "svg:" + localName;
+        String prefix = getNodePrefix(namespaceURI);
+        if (prefix != null) {
+            return prefix + ":" + localName;
         } else {
             return "(Namespace URI: \"" + namespaceURI + "\", "
                     + "Local Name: \"" + localName + "\")";
@@ -523,16 +543,35 @@ public abstract class FONode implements Cloneable {
      *
      * @param loc org.xml.sax.Locator object of the error (*not* parent node)
      * @param parentName the name of the parent element
-     * @param nsURI namespace URI of incoming invalid node
-     * @param lName local name (i.e., no prefix) of incoming node
+     * @param nsURI namespace URI of incoming offending node
+     * @param lName local name (i.e., no prefix) of incoming offending node
      * @param ruleViolated name of the rule violated (used to lookup a resource in a bundle)
      * @throws ValidationException the validation error provoked by the method call
      */
     protected void invalidChildError(Locator loc, String parentName, String nsURI, String lName,
                 String ruleViolated)
                 throws ValidationException {
-        getFOValidationEventProducer().invalidChild(this, parentName,
-                new QName(nsURI, lName), ruleViolated, loc);
+        String prefix = getNodePrefix(nsURI);
+        QName qn; // qualified name of offending node
+        if (prefix != null) {
+            qn = new QName(nsURI, prefix, lName);
+        } else {
+            qn = new QName(nsURI, lName);
+        }
+        getFOValidationEventProducer().invalidChild(this, parentName, qn, ruleViolated, loc);
+    }
+
+    /**
+     * Helper function to return "not supported child" exceptions. Note that the child is valid, just not
+     * supported yet by FOP.
+     *
+     * @param loc org.xml.sax.Locator object of the error (*not* parent node)
+     * @param nsURI namespace URI of incoming invalid node
+     * @param lName local name (i.e., no prefix) of incoming node
+     * @throws ValidationException the validation error provoked by the method call
+     */
+    protected void notSupportedChildError(Locator loc, String nsURI, String lName) throws ValidationException {
+        getFOValidationEventProducer().notSupportedChild(this, getName(), new QName(nsURI, lName), loc);
     }
 
     /**
@@ -575,6 +614,22 @@ public abstract class FONode implements Cloneable {
         getFOValidationEventProducer().missingProperty(this, getName(), propertyName, locator);
     }
 
+
+
+    /**
+     * Helper function to throw an error caused by an invalid property
+     *
+     * @param propertyName the name of the property.
+     * @param propertyValue the value of the property.
+     * @param e optional property parsing exception.
+     * @throws ValidationException the validation error provoked by the method call
+     */
+    protected void invalidPropertyValueError(String propertyName, String propertyValue, Exception e)
+                throws ValidationException {
+        getFOValidationEventProducer().invalidPropertyValue(this, getName(), propertyName,
+                propertyValue,  new PropertyException(e), locator);
+    }
+
     /**
      * Helper function to return "Error(line#/column#)" string for
      * above exception messages
@@ -607,7 +662,7 @@ public abstract class FONode implements Cloneable {
         if (loc == null) {
             return "Unknown location";
         } else {
-            return loc.getLineNumber() + "/" + loc.getColumnNumber();
+            return loc.getLineNumber() + ":" + loc.getColumnNumber();
         }
     }
 
@@ -633,8 +688,9 @@ public abstract class FONode implements Cloneable {
      * this method only in exceptional conditions because this method may perform quite extensive
      * information gathering inside the FO tree.
      * @return a String containing context information
-     * @deprecated Not localized! Should rename getContextInfoAlt() to getContextInfo() when done!
      */
+    // [GA] remove deprecation - no alternative specified
+    //  @deprecated Not localized! Should rename getContextInfoAlt() to getContextInfo() when done!
     public String getContextInfo() {
         StringBuffer sb = new StringBuffer();
         if (getLocalName() != null) {
@@ -797,8 +853,9 @@ public abstract class FONode implements Cloneable {
 
     /**
      * This method is overridden by extension elements and allows the extension element to return
-     * a {@link ContentHandlerFactory}. This factory can create ContentHandler implementations that handle
-     * foreign XML content by either building up a specific DOM, a Java object or something else.
+     * a {@link ContentHandlerFactory}. This factory can create ContentHandler implementations that
+     * handle foreign XML content by either building up a specific DOM, a Java object or something
+     * else.
      *
      * @return the <code>ContentHandlerFactory</code> or <code>null</code> if not applicable
      */
@@ -862,9 +919,140 @@ public abstract class FONode implements Cloneable {
     }
 
     /**
-     * Base iterator interface over a FO's children
+     * Determine if node has a delimited text range boundary. N.B. that we report
+     * this to be true by default, while specific subclasses override this method to report false.
+     * @param boundary one of {EN_BEFORE, EN_AFTER, or EN_BOTH} enumeration constants
+     * @return true if indicated boundary (or boundaries) constitute a delimited text range
+     * boundary.
      */
-    public interface FONodeIterator extends ListIterator {
+    public boolean isDelimitedTextRangeBoundary(int boundary) {
+        return true;
+    }
+
+    /**
+     * Collect the sequence of delimited text ranges, where each new
+     * range is pushed onto RANGES.
+     * @param ranges a stack of delimited text ranges
+     * @return the (possibly) updated stack of delimited text ranges
+     */
+    public Stack<DelimitedTextRange> collectDelimitedTextRanges(Stack<DelimitedTextRange> ranges) {
+        // if boundary before, then push new range
+        if (isRangeBoundaryBefore()) {
+            maybeNewRange(ranges);
+        }
+        // get current range, if one exists
+        DelimitedTextRange currentRange;
+        if (ranges.size() > 0) {
+            currentRange = ranges.peek();
+        } else {
+            currentRange = null;
+        }
+        // proceses this node
+        ranges = collectDelimitedTextRanges(ranges, currentRange);
+        // if boundary after, then push new range
+        if (isRangeBoundaryAfter()) {
+            maybeNewRange(ranges);
+        }
+        return ranges;
+    }
+
+    /**
+     * Collect the sequence of delimited text ranges, where each new
+     * range is pushed onto RANGES, where default implementation collects ranges
+     * of child nodes.
+     * @param ranges a stack of delimited text ranges
+     * @param currentRange the current range or null (if none)
+     * @return the (possibly) updated stack of delimited text ranges
+     */
+    protected Stack<DelimitedTextRange> collectDelimitedTextRanges(
+            Stack<DelimitedTextRange> ranges, DelimitedTextRange currentRange) {
+
+        for (FONodeIterator it = getChildNodes(); (it != null) && it.hasNext();) {
+            ranges = it.next().collectDelimitedTextRanges(ranges);
+        }
+        return ranges;
+    }
+
+    /**
+     * Determine if this node is a new bidi RANGE block item.
+     * @return true if this node is a new bidi RANGE block item
+     */
+    public boolean isBidiRangeBlockItem() {
+        return false;
+    }
+
+    /**
+     * Conditionally add a new delimited text range to RANGES. Always add new
+     * range unless there are no ranges on the stack yet and this node is not a block item.
+     * @param ranges stack of delimited text ranges
+     * @return new range (if constructed and pushed onto stack) or current range (if any) or null
+     */
+    private DelimitedTextRange maybeNewRange(Stack<DelimitedTextRange> ranges) {
+        DelimitedTextRange rCur = !ranges.empty() ? ranges.peek() : null;
+        DelimitedTextRange rNew;
+        if ((rCur != null) || isBidiRangeBlockItem()) {
+            rNew = new DelimitedTextRange(this);
+        } else {
+            rNew = null;
+        }
+        if (rNew != null) {
+            ranges.push(rNew);
+        } else {
+            rNew = rCur;
+        }
+        return rNew;
+    }
+
+    private boolean isRangeBoundaryBefore() {
+        return isDelimitedTextRangeBoundary(Constants.EN_BEFORE);
+    }
+
+    private boolean isRangeBoundaryAfter() {
+        return isDelimitedTextRangeBoundary(Constants.EN_AFTER);
+    }
+
+    /**
+     * Base iterator interface over a FO's children, offering three methods on top of the base interface
+     * methods {@see java.util.ListIterator}.
+     */
+    public interface FONodeIterator extends ListIterator<FONode> {
+
+        /** @return the next node */
+        FONode next();
+
+        /** @return the previous node */
+        FONode previous();
+
+        /**
+         * Replace the node at the current index with the given <code>newNode</code>.
+         *
+         * @param newNode the new node
+         */
+        void set(FONode newNode);
+
+        /**
+         * Add the given <code>newNode</code> at the current position.
+         *
+         * @param newNode the new node
+         */
+        void add(FONode newNode);
+
+        /** @return <code>true</code> if there is a next node, <code>false</code> otherwise */
+        boolean hasNext();
+
+        /** @return <code>true</code> if there is a previous node, <code>false</code> otherwise */
+        boolean hasPrevious();
+
+        /** @return the current index */
+        int nextIndex();
+
+        /** @return the previous index */
+        int previousIndex();
+
+        /**
+         * Removes the node at the current position.
+         */
+        void remove();
 
         /**
          * Returns the parent node for this iterator's list
@@ -872,23 +1060,7 @@ public abstract class FONode implements Cloneable {
          *
          * @return  the parent node
          */
-        FObj parentNode();
-
-        /**
-         * Convenience method with return type of FONode
-         * (semantically equivalent to: <code>(FONode) next();</code>)
-         *
-         * @return the next node (if any), as a type FONode
-         */
-        FONode nextNode();
-
-        /**
-         * Convenience method with return type of FONode
-         * (semantically equivalent to: <code>(FONode) previous();</code>)
-         *
-         * @return the previous node (if any), as a type FONode
-         */
-        FONode previousNode();
+        FObj parent();
 
         /**
          * Returns the first node in the list, and decreases the index,
@@ -897,7 +1069,7 @@ public abstract class FONode implements Cloneable {
          *
          * @return the first node in the list
          */
-        FONode firstNode();
+        FONode first();
 
         /**
          * Returns the last node in the list, and advances the
@@ -906,8 +1078,26 @@ public abstract class FONode implements Cloneable {
          *
          * @return the last node in the list
          */
-        FONode lastNode();
+        FONode last();
 
     }
 
+    /**
+     * Sets the structure tree element.
+     *
+     * @param structureTreeElement set.
+     */
+    public void setStructureTreeElement(StructureTreeElement structureTreeElement) {
+        throw new UnsupportedOperationException();
+    }
+
+    /**
+     * Returns the structure tree element associated to this object.
+     *
+     * @return the structure tree element
+     */
+    public StructureTreeElement getStructureTreeElement() {
+        return null;
+    }
+
 }
