@@ -15,13 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id: FOText.java 1293736 2012-02-26 02:29:01Z gadams $ */
+/* $Id: FOText.java 1619960 2014-08-23 03:50:42Z gadams $ */
 
 package org.apache.fop.fo;
 
 import java.awt.Color;
 import java.nio.CharBuffer;
-import java.util.Map;
+import java.text.CharacterIterator;
+import java.text.StringCharacterIterator;
 import java.util.NoSuchElementException;
 import java.util.Stack;
 
@@ -38,16 +39,20 @@ import org.apache.fop.fo.properties.CommonTextDecoration;
 import org.apache.fop.fo.properties.KeepProperty;
 import org.apache.fop.fo.properties.Property;
 import org.apache.fop.fo.properties.SpaceProperty;
+import org.apache.fop.fonts.TextFragment;
 import org.apache.fop.util.CharUtilities;
 
 /**
  * A text node (PCDATA) in the formatting object tree.
  */
-public class FOText extends FONode implements CharSequence {
+public class FOText extends FONode implements CharSequence, TextFragment {
 
     /** the <code>CharBuffer</code> containing the text */
     private CharBuffer charBuffer;
 
+    // cached iterator
+    private CharacterIterator charIterator;
+
     // The value of FO traits (refined properties) that apply to #PCDATA
     // (aka implicit sequence of fo:character)
     private CommonFont commonFont;
@@ -71,19 +76,19 @@ public class FOText extends FONode implements CharSequence {
      * Points to the previous FOText object created within the current
      * block. If this is "null", this is the first such object.
      */
-    private FOText prevFOTextThisBlock = null;
+    private FOText prevFOTextThisBlock;
 
     /**
      * Points to the next FOText object created within the current
      * block. If this is "null", this is the last such object.
      */
-    private FOText nextFOTextThisBlock = null;
+    private FOText nextFOTextThisBlock;
 
     /**
      * Points to the ancestor Block object. This is used to keep track of
      * which FOText nodes are descendants of the same block.
      */
-    private Block ancestorBlock = null;
+    private Block ancestorBlock;
 
     /** Holds the text decoration values. May be null */
     private CommonTextDecoration textDecoration;
@@ -93,9 +98,6 @@ public class FOText extends FONode implements CharSequence {
     /* bidi levels */
     private int[] bidiLevels;
 
-    /* advanced script processing state */
-    private Map/*<MapRange,String>*/ mappings;
-
     private static final int IS_WORD_CHAR_FALSE = 0;
     private static final int IS_WORD_CHAR_TRUE = 1;
     private static final int IS_WORD_CHAR_MAYBE = 2;
@@ -114,15 +116,15 @@ public class FOText extends FONode implements CharSequence {
             PropertyList list, Locator locator) throws FOPException {
         if (charBuffer == null) {
             // buffer not yet initialized, do so now
-            int newLength = ( length < 16 ) ? 16 : length;
+            int newLength = (length < 16) ? 16 : length;
             charBuffer = CharBuffer.allocate(newLength);
         } else {
             // allocate a larger buffer, and transfer contents
             int requires = charBuffer.position() + length;
             int capacity = charBuffer.capacity();
-            if ( requires > capacity ) {
+            if (requires > capacity) {
                 int newCapacity = capacity * 2;
-                if ( requires > newCapacity ) {
+                if (requires > newCapacity) {
                     newCapacity = requires;
                 }
                 CharBuffer newBuffer = CharBuffer.allocate(newCapacity);
@@ -193,8 +195,8 @@ public class FOText extends FONode implements CharSequence {
     }
 
     /** {@inheritDoc} */
-    protected void endOfNode() throws FOPException {
-        if ( charBuffer != null ) {
+    public void endOfNode() throws FOPException {
+        if (charBuffer != null) {
             charBuffer.rewind();
         }
         super.endOfNode();
@@ -510,10 +512,10 @@ public class FOText extends FONode implements CharSequence {
 
     private class TextCharIterator extends CharIterator {
 
-        private int currentPosition = 0;
+        private int currentPosition;
 
-        private boolean canRemove = false;
-        private boolean canReplace = false;
+        private boolean canRemove;
+        private boolean canReplace;
 
         public TextCharIterator() {
         }
@@ -652,19 +654,42 @@ public class FOText extends FONode implements CharSequence {
         return country;
     }
 
-    /** @return the language trait */
+    @Override
+    public synchronized CharacterIterator getIterator() {
+        if (charIterator != null) {
+            charIterator = new StringCharacterIterator(toString());
+        }
+        return charIterator;
+    }
+
+    @Override
+    public int getBeginIndex() {
+        return 0;
+    }
+
+    @Override
+    public int getEndIndex() {
+        return length();
+    }
+
+    @Override
     public String getLanguage() {
         return language;
     }
 
-    /** @return the script trait */
+    @Override
     public String getScript() {
         return script;
     }
 
+    @Override
+    public int getBidiLevel() {
+        return length() > 0 ? bidiLevelAt(0) : -1;
+    }
+
     /** {@inheritDoc} */
     public String toString() {
-        if ( charBuffer == null ) {
+        if (charBuffer == null) {
             return "";
         } else {
             CharBuffer cb = charBuffer.duplicate();
@@ -717,7 +742,7 @@ public class FOText extends FONode implements CharSequence {
     }
 
     @Override
-    public boolean isDelimitedTextRangeBoundary ( int boundary ) {
+    public boolean isDelimitedTextRangeBoundary(int boundary) {
         return false;
     }
 
@@ -726,7 +751,7 @@ public class FOText extends FONode implements CharSequence {
         this.structureTreeElement = structureTreeElement;
     }
 
-    /** @return the structure tree element. */
+    @Override
     public StructureTreeElement getStructureTreeElement() {
         return structureTreeElement;
     }
@@ -737,16 +762,16 @@ public class FOText extends FONode implements CharSequence {
      * @param start the starting index of interval
      * @param end the ending index of interval
      */
-    public void setBidiLevel ( int level, int start, int end ) {
-        if ( start < end ) {
-            if ( bidiLevels == null ) {
+    public void setBidiLevel(int level, int start, int end) {
+        if (start < end) {
+            if (bidiLevels == null) {
                 bidiLevels = new int [ length() ];
             }
-            for ( int i = start, n = end; i < n; i++ ) {
+            for (int i = start, n = end; i < n; i++) {
                 bidiLevels [ i ] = level;
             }
-            if ( parent != null ) {
-                ( (FObj) parent ).setBidiLevel ( level );
+            if (parent != null) {
+                ((FObj) parent).setBidiLevel(level);
             }
         } else {
             assert start < end;
@@ -771,12 +796,12 @@ public class FOText extends FONode implements CharSequence {
      * @return a (possibly empty) array of bidi levels or null
      * in case no bidi levels have been assigned
      */
-    public int[] getBidiLevels ( int start, int end ) {
-        if ( this.bidiLevels != null ) {
+    public int[] getBidiLevels(int start, int end) {
+        if (this.bidiLevels != null) {
             assert start <= end;
             int n = end - start;
             int[] bidiLevels = new int [ n ];
-            for ( int i = 0; i < n; i++ ) {
+            for (int i = 0; i < n; i++) {
                 bidiLevels[i] = this.bidiLevels [ start + i ];
             }
             return bidiLevels;
@@ -794,107 +819,21 @@ public class FOText extends FONode implements CharSequence {
      * @throws IndexOutOfBoundsException if position is not non-negative integer
      * or is greater than or equal to length
      */
-    public int bidiLevelAt ( int position ) throws IndexOutOfBoundsException {
-        if ( ( position < 0 ) || ( position >= length() ) ) {
+    public int bidiLevelAt(int position) throws IndexOutOfBoundsException {
+        if ((position < 0) || (position >= length())) {
             throw new IndexOutOfBoundsException();
-        } else if ( bidiLevels != null ) {
+        } else if (bidiLevels != null) {
             return bidiLevels [ position ];
         } else {
             return -1;
         }
     }
 
-    /**
-     * Add characters mapped by script substitution processing.
-     * @param start index in character buffer
-     * @param end index in character buffer
-     * @param mappedChars sequence of character codes denoting substituted characters
-     */
-    public void addMapping ( int start, int end, CharSequence mappedChars ) {
-        if ( mappings == null ) {
-            mappings = new java.util.HashMap();
-        }
-        mappings.put ( new MapRange ( start, end ), mappedChars.toString() );
-    }
-
-    /**
-     * Determine if characters over specific interval  have a mapping.
-     * @param start index in character buffer
-     * @param end index in character buffer
-     * @return true if a mapping exist such that the mapping's interval is coincident to
-     * [start,end)
-     */
-    public boolean hasMapping ( int start, int end ) {
-        return ( mappings != null ) && ( mappings.containsKey ( new MapRange ( start, end ) ) );
-    }
-
-    /**
-     * Obtain mapping of characters over specific interval.
-     * @param start index in character buffer
-     * @param end index in character buffer
-     * @return a string of characters representing the mapping over the interval
-     * [start,end)
-     */
-    public String getMapping ( int start, int end ) {
-        if ( mappings != null ) {
-            return (String) mappings.get ( new MapRange ( start, end ) );
-        } else {
-            return null;
-        }
-    }
-
-    /**
-     * Obtain length of mapping of characters over specific interval.
-     * @param start index in character buffer
-     * @param end index in character buffer
-     * @return the length of the mapping (if present) or zero
-     */
-    public int getMappingLength ( int start, int end ) {
-        if ( mappings != null ) {
-            return ( (String) mappings.get ( new MapRange ( start, end ) ) ) .length();
-        } else {
-            return 0;
-        }
-    }
-
-    /**
-     * Obtain bidirectional levels of mapping of characters over specific interval.
-     * @param start index in character buffer
-     * @param end index in character buffer
-     * @return a (possibly empty) array of bidi levels or null
-     * in case no bidi levels have been assigned
-     */
-    public int[] getMappingBidiLevels ( int start, int end ) {
-        if ( hasMapping ( start, end ) ) {
-            int   nc = end - start;
-            int   nm = getMappingLength ( start, end );
-            int[] la = getBidiLevels ( start, end );
-            if ( la == null ) {
-                return null;
-            } else if ( nm == nc ) {            // mapping is same length as mapped range
-                return la;
-            } else if ( nm > nc ) {             // mapping is longer than mapped range
-                int[] ma = new int [ nm ];
-                System.arraycopy ( la, 0, ma, 0, la.length );
-                for ( int i = la.length,
-                          n = ma.length, l = ( i > 0 ) ? la [ i - 1 ] : 0; i < n; i++ ) {
-                    ma [ i ] = l;
-                }
-                return ma;
-            } else {                            // mapping is shorter than mapped range
-                int[] ma = new int [ nm ];
-                System.arraycopy ( la, 0, ma, 0, ma.length );
-                return ma;
-            }
-        } else {
-            return getBidiLevels ( start, end );
-        }
-    }
-
     @Override
-    protected Stack collectDelimitedTextRanges ( Stack ranges, DelimitedTextRange currentRange ) {
-        if ( currentRange != null ) {
-            currentRange.append ( charIterator(), this );
+    protected Stack<DelimitedTextRange> collectDelimitedTextRanges(Stack<DelimitedTextRange> ranges,
+        DelimitedTextRange currentRange) {
+        if (currentRange != null) {
+            currentRange.append(charIterator(), this);
         }
         return ranges;
     }
@@ -907,12 +846,12 @@ public class FOText extends FONode implements CharSequence {
             this.end = end;
         }
         public int hashCode() {
-            return ( start * 31 ) + end;
+            return (start * 31) + end;
         }
-        public boolean equals ( Object o ) {
-            if ( o instanceof MapRange ) {
+        public boolean equals(Object o) {
+            if (o instanceof MapRange) {
                 MapRange r = (MapRange) o;
-                return ( r.start == start ) && ( r.end == end );
+                return (r.start == start) && (r.end == end);
             } else {
                 return false;
             }
