@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FOTreeBuilder.java 1296496 2012-03-02 22:19:46Z gadams $ */
+/* $Id: FOTreeBuilder.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.fo;
 
@@ -40,6 +40,7 @@ import org.apache.fop.apps.FormattingResults;
 import org.apache.fop.fo.ElementMapping.Maker;
 import org.apache.fop.fo.extensions.ExtensionElementMapping;
 import org.apache.fop.fo.pagination.Root;
+import org.apache.fop.render.pdf.extensions.PDFElementMapping;
 import org.apache.fop.util.ContentHandlerFactory;
 import org.apache.fop.util.ContentHandlerFactory.ObjectBuiltListener;
 import org.apache.fop.util.ContentHandlerFactory.ObjectSource;
@@ -59,7 +60,7 @@ public class FOTreeBuilder extends DefaultHandler {
     protected ElementMappingRegistry elementMappingRegistry;
 
     /** The root of the formatting object tree */
-    protected Root rootFObj = null;
+    protected Root rootFObj;
 
     /** Main DefaultHandler that handles the FO namespace. */
     protected MainFOHandler mainFOHandler;
@@ -79,10 +80,11 @@ public class FOTreeBuilder extends DefaultHandler {
     /** The user agent for this processing run. */
     private FOUserAgent userAgent;
 
-    private boolean used = false;
+    private boolean used;
     private boolean empty = true;
 
     private int depth;
+    private boolean errorinstart;
 
     /**
      * <code>FOTreeBuilder</code> constructor
@@ -99,7 +101,7 @@ public class FOTreeBuilder extends DefaultHandler {
             throws FOPException {
 
         this.userAgent = foUserAgent;
-        this.elementMappingRegistry = userAgent.getFactory().getElementMappingRegistry();
+        this.elementMappingRegistry = userAgent.getElementMappingRegistry();
         //This creates either an AreaTreeHandler and ultimately a Renderer, or
         //one of the RTF-, MIF- etc. Handlers.
         foEventHandler = foUserAgent.getRendererFactory().createFOEventHandler(
@@ -172,20 +174,28 @@ public class FOTreeBuilder extends DefaultHandler {
     public void startElement(String namespaceURI, String localName, String rawName,
                              Attributes attlist) throws SAXException {
         this.depth++;
-        delegate.startElement(namespaceURI, localName, rawName, attlist);
+        errorinstart = false;
+        try {
+            delegate.startElement(namespaceURI, localName, rawName, attlist);
+        } catch (SAXException e) {
+            errorinstart = true;
+            throw e;
+        }
     }
 
     /** {@inheritDoc} */
     public void endElement(String uri, String localName, String rawName)
                 throws SAXException {
-        this.delegate.endElement(uri, localName, rawName);
-        this.depth--;
-        if (depth == 0) {
-            if (delegate != mainFOHandler) {
-                //Return from sub-handler back to main handler
-                delegate.endDocument();
-                delegate = mainFOHandler;
-                delegate.endElement(uri, localName, rawName);
+        if (!errorinstart) {
+            this.delegate.endElement(uri, localName, rawName);
+            this.depth--;
+            if (depth == 0) {
+                if (delegate != mainFOHandler) {
+                    //Return from sub-handler back to main handler
+                    delegate.endDocument();
+                    delegate = mainFOHandler;
+                    delegate.endElement(uri, localName, rawName);
+                }
             }
         }
     }
@@ -232,13 +242,13 @@ public class FOTreeBuilder extends DefaultHandler {
     private class MainFOHandler extends DefaultHandler {
 
         /** Current formatting object being handled */
-        protected FONode currentFObj = null;
+        protected FONode currentFObj;
 
         /** Current propertyList for the node being handled */
         protected PropertyList currentPropertyList;
 
         /** Current marker nesting-depth */
-        private int nestedMarkerDepth = 0;
+        private int nestedMarkerDepth;
 
         /** {@inheritDoc} */
         public void startElement(String namespaceURI, String localName, String rawName,
@@ -261,7 +271,8 @@ public class FOTreeBuilder extends DefaultHandler {
                 }
             } else { // check that incoming node is valid for currentFObj
                 if (currentFObj.getNamespaceURI().equals(FOElementMapping.URI)
-                    || currentFObj.getNamespaceURI().equals(ExtensionElementMapping.URI)) {
+                    || currentFObj.getNamespaceURI().equals(ExtensionElementMapping.URI)
+                    || currentFObj.getNamespaceURI().equals(PDFElementMapping.NAMESPACE)) {
                     currentFObj.validateChildNode(locator, namespaceURI, localName);
                 }
             }
@@ -321,7 +332,8 @@ public class FOTreeBuilder extends DefaultHandler {
             // fo:characters can potentially be removed during
             // white-space handling.
             // Do not notify the FOEventHandler.
-            if (currentFObj.getNameId() != Constants.FO_CHARACTER) {
+            if (currentFObj.getNameId() != Constants.FO_CHARACTER
+                    && (!builderContext.inMarker() || currentFObj.getNameId() == Constants.FO_MARKER)) {
                 currentFObj.startOfNode();
             }
         }
@@ -343,7 +355,8 @@ public class FOTreeBuilder extends DefaultHandler {
             // fo:characters can potentially be removed during
             // white-space handling.
             // Do not notify the FOEventHandler.
-            if (currentFObj.getNameId() != Constants.FO_CHARACTER) {
+            if (currentFObj.getNameId() != Constants.FO_CHARACTER
+                    && (!builderContext.inMarker() || currentFObj.getNameId() == Constants.FO_MARKER)) {
                 currentFObj.endOfNode();
             }
 
