@@ -15,15 +15,15 @@
  * limitations under the License.
  */
 
-/* $Id: FObj.java 825566 2009-10-15 17:07:08Z acumiskey $ */
+/* $Id: FObj.java 1722725 2016-01-03 14:30:10Z adelmelle $ */
 
 package org.apache.fop.fo;
 
 import java.util.Collections;
-import java.util.Iterator;
+import java.util.HashMap;
 import java.util.List;
-import java.util.ListIterator;
 import java.util.Map;
+import java.util.Map.Entry;
 import java.util.NoSuchElementException;
 import java.util.Set;
 
@@ -35,6 +35,7 @@ import org.apache.xmlgraphics.util.QName;
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.fo.extensions.ExtensionAttachment;
 import org.apache.fop.fo.flow.Marker;
+import org.apache.fop.fo.properties.Property;
 import org.apache.fop.fo.properties.PropertyMaker;
 
 /**
@@ -44,30 +45,34 @@ import org.apache.fop.fo.properties.PropertyMaker;
 public abstract class FObj extends FONode implements Constants {
 
     /** the list of property makers */
-    private static final PropertyMaker[] propertyListTable
+    private static final PropertyMaker[] PROPERTY_LIST_TABLE
                             = FOPropertyMapping.getGenericMappings();
 
-    /**
-     * pointer to the descendant subtree
-     */
+    /** pointer to the descendant subtree */
     protected FONode firstChild;
 
+    /** pointer to the end of the descendant subtree */
+    protected FONode lastChild;
+
     /** The list of extension attachments, null if none */
-    private List/*<ExtensionAttachment>*/ extensionAttachments = null;
+    private List<ExtensionAttachment> extensionAttachments;
 
     /** The map of foreign attributes, null if none */
-    private Map/*<QName,String>*/ foreignAttributes = null;
+    private Map<QName, String> foreignAttributes;
 
     /** Used to indicate if this FO is either an Out Of Line FO (see rec)
      *  or a descendant of one. Used during FO validation.
      */
-    private boolean isOutOfLineFODescendant = false;
+    private boolean isOutOfLineFODescendant;
 
     /** Markers added to this element. */
-    private Map markers = null;
+    private Map<String, Marker> markers;
+
+    private int bidiLevel = -1;
 
     // The value of properties relevant for all fo objects
-    private String id = null;
+    private String id;
+    private String layer;
     // End of property values
 
     /**
@@ -108,7 +113,7 @@ public abstract class FObj extends FONode implements Constants {
      * @return the requested Property Maker
      */
     public static PropertyMaker getPropertyMakerFor(int propId) {
-        return propertyListTable[propId];
+        return PROPERTY_LIST_TABLE[propId];
     }
 
     /** {@inheritDoc} */
@@ -117,11 +122,35 @@ public abstract class FObj extends FONode implements Constants {
                     throws FOPException {
         setLocator(locator);
         pList.addAttributesToList(attlist);
-        if (!inMarker()
-                || "marker".equals(elementName)) {
-            pList.setWritingMode();
+        if (!inMarker() || "marker".equals(elementName)) {
             bind(pList);
         }
+        warnOnUnknownProperties(attlist, elementName, pList);
+    }
+
+    private void warnOnUnknownProperties(Attributes attlist, String objName, PropertyList propertyList)
+            throws FOPException {
+        Map<String, Property> unknowns = propertyList.getUnknownPropertyValues();
+        for (Entry<String, Property> entry : unknowns.entrySet()) {
+            FOValidationEventProducer producer = FOValidationEventProducer.Provider.get(getUserAgent()
+                    .getEventBroadcaster());
+            producer.warnOnInvalidPropertyValue(this, objName,
+                    getAttributeNameForValue(attlist, entry.getValue(), propertyList), entry.getKey(), null,
+                    getLocator());
+        }
+    }
+
+    private String getAttributeNameForValue(Attributes attList, Property value, PropertyList propertyList)
+            throws FOPException {
+        for (int i = 0; i < attList.getLength(); i++) {
+            String attributeName = attList.getQName(i);
+            String attributeValue = attList.getValue(i);
+            Property prop = propertyList.getPropertyForAttribute(attList, attributeName, attributeValue);
+            if (prop != null && prop.equals(value)) {
+                return attributeName;
+            }
+        }
+        return "unknown";
     }
 
     /**
@@ -142,13 +171,14 @@ public abstract class FObj extends FONode implements Constants {
      */
     public void bind(PropertyList pList) throws FOPException {
         id = pList.get(PR_ID).getString();
+        layer = pList.get(PR_X_LAYER).getString();
     }
 
     /**
      * {@inheritDoc}
      * @throws FOPException FOP Exception
      */
-    protected void startOfNode() throws FOPException {
+    public void startOfNode() throws FOPException {
         if (id != null) {
             checkId(id);
         }
@@ -165,7 +195,7 @@ public abstract class FObj extends FONode implements Constants {
      */
     private void checkId(String id) throws ValidationException {
         if (!inMarker() && !id.equals("")) {
-            Set idrefs = getBuilderContext().getIDReferences();
+            Set<String> idrefs = getBuilderContext().getIDReferences();
             if (!idrefs.contains(id)) {
                 idrefs.add(id);
             } else {
@@ -197,13 +227,19 @@ public abstract class FObj extends FONode implements Constants {
             } else {
                 if (firstChild == null) {
                     firstChild = child;
+                    lastChild = child;
                 } else {
-                    FONode prevChild = firstChild;
-                    while (prevChild.siblings != null
-                            && prevChild.siblings[1] != null) {
-                        prevChild = prevChild.siblings[1];
+                    if (lastChild == null) {
+                        FONode prevChild = firstChild;
+                        while (prevChild.siblings != null
+                                && prevChild.siblings[1] != null) {
+                            prevChild = prevChild.siblings[1];
+                        }
+                        FONode.attachSiblings(prevChild, child);
+                    } else {
+                        FONode.attachSiblings(lastChild, child);
+                        lastChild = child;
                     }
-                    FONode.attachSiblings(prevChild, child);
                 }
             }
         }
@@ -231,13 +267,20 @@ public abstract class FObj extends FONode implements Constants {
             if (firstChild != null) {
                 firstChild.siblings[0] = null;
             }
-        } else {
+        } else if (child.siblings != null) {
             FONode prevChild = child.siblings[0];
             prevChild.siblings[1] = nextChild;
             if (nextChild != null) {
                 nextChild.siblings[0] = prevChild;
             }
         }
+        if (child == lastChild) {
+            if (child.siblings != null) {
+                lastChild = siblings[0];
+            } else {
+                lastChild = null;
+            }
+        }
     }
 
     /**
@@ -255,7 +298,7 @@ public abstract class FObj extends FONode implements Constants {
     /**
      * Check if this formatting object generates reference areas.
      * @return true if generates reference areas
-     * @todo see if needed
+     * TODO see if needed
      */
     public boolean generatesReferenceAreas() {
         return false;
@@ -282,7 +325,7 @@ public abstract class FObj extends FONode implements Constants {
      * at the passed-in node (= first call to iterator.next() will
      * return childNode)
      * @param childNode First node in the iterator
-     * @return A ListIterator or null if childNode isn't a child of
+     * @return A FONodeIterator or null if childNode isn't a child of
      * this FObj.
      */
     public FONodeIterator getChildNodes(FONode childNode) {
@@ -292,7 +335,7 @@ public abstract class FObj extends FONode implements Constants {
                 return it;
             } else {
                 while (it.hasNext()
-                        && it.nextNode().siblings[1] != childNode) {
+                        && it.next().siblings[1] != childNode) {
                     //nop
                 }
                 if (it.hasNext()) {
@@ -327,8 +370,8 @@ public abstract class FObj extends FONode implements Constants {
         String mcname = marker.getMarkerClassName();
         if (firstChild != null) {
             // check for empty childNodes
-            for (Iterator iter = getChildNodes(); iter.hasNext();) {
-                FONode node = (FONode) iter.next();
+            for (FONodeIterator iter = getChildNodes(); iter.hasNext();) {
+                FONode node = iter.next();
                 if (node instanceof FObj
                         || (node instanceof FOText
                                 && ((FOText) node).willCreateArea())) {
@@ -342,7 +385,7 @@ public abstract class FObj extends FONode implements Constants {
             }
         }
         if (markers == null) {
-            markers = new java.util.HashMap();
+            markers = new HashMap<String, Marker>();
         }
         if (!markers.containsKey(mcname)) {
             markers.put(mcname, marker);
@@ -362,13 +405,13 @@ public abstract class FObj extends FONode implements Constants {
     /**
      * @return the collection of Markers attached to this object
      */
-    public Map getMarkers() {
+    public Map<String, Marker> getMarkers() {
         return markers;
     }
 
     /** {@inheritDoc} */
     protected String getContextInfoAlt() {
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         if (getLocalName() != null) {
             sb.append(getName());
             sb.append(", ");
@@ -398,13 +441,13 @@ public abstract class FObj extends FONode implements Constants {
         if (getLocator() != null) {
             return super.gatherContextInfo();
         } else {
-            ListIterator iter = getChildNodes();
+            FONodeIterator iter = getChildNodes();
             if (iter == null) {
                 return null;
             }
-            StringBuffer sb = new StringBuffer();
+            StringBuilder sb = new StringBuilder();
             while (iter.hasNext()) {
-                FONode node = (FONode) iter.next();
+                FONode node = iter.next();
                 String s = node.gatherContextInfo();
                 if (s != null) {
                     if (sb.length() > 0) {
@@ -420,7 +463,8 @@ public abstract class FObj extends FONode implements Constants {
     /**
      * Convenience method for validity checking.  Checks if the
      * incoming node is a member of the "%block;" parameter entity
-     * as defined in Sect. 6.2 of the XSL 1.0 & 1.1 Recommendations
+     * as defined in Sect. 6.2 of the XSL 1.0 &amp; 1.1 Recommendations
+     *
      * @param nsURI namespace URI of incoming node
      * @param lName local name (i.e., no prefix) of incoming node
      * @return true if a member, false if not
@@ -439,7 +483,8 @@ public abstract class FObj extends FONode implements Constants {
     /**
      * Convenience method for validity checking.  Checks if the
      * incoming node is a member of the "%inline;" parameter entity
-     * as defined in Sect. 6.2 of the XSL 1.0 & 1.1 Recommendations
+     * as defined in Sect. 6.2 of the XSL 1.0 &amp; 1.1 Recommendations
+     *
      * @param nsURI namespace URI of incoming node
      * @param lName local name (i.e., no prefix) of incoming node
      * @return true if a member, false if not
@@ -480,12 +525,12 @@ public abstract class FObj extends FONode implements Constants {
     /**
      * Convenience method for validity checking.  Checks if the
      * incoming node is a member of the neutral item list
-     * as defined in Sect. 6.2 of the XSL 1.0 & 1.1 Recommendations
+     * as defined in Sect. 6.2 of the XSL 1.0 &amp; 1.1 Recommendations
      * @param nsURI namespace URI of incoming node
      * @param lName local name (i.e., no prefix) of incoming node
      * @return true if a member, false if not
      */
-    boolean isNeutralItem(String nsURI, String lName) {
+    protected boolean isNeutralItem(String nsURI, String lName) {
         return (FO_URI.equals(nsURI)
                 && ("multi-switch".equals(lName)
                         || "multi-properties".equals(lName)
@@ -529,7 +574,17 @@ public abstract class FObj extends FONode implements Constants {
 
     /** @return whether this object has an id set */
     public boolean hasId() {
-        return id != null && id.length() > 0;
+        return (id != null && id.length() > 0);
+    }
+
+    /** @return the "layer" property. */
+    public String getLayer() {
+        return layer;
+    }
+
+    /** @return whether this object has an layer set */
+    public boolean hasLayer() {
+        return (layer != null && layer.length() > 0);
     }
 
     /** {@inheritDoc} */
@@ -542,6 +597,78 @@ public abstract class FObj extends FONode implements Constants {
         return "fo";
     }
 
+    /** {@inheritDoc} */
+    public boolean isBidiRangeBlockItem() {
+        String ns = getNamespaceURI();
+        String ln = getLocalName();
+        return !isNeutralItem(ns, ln) && isBlockItem(ns, ln);
+    }
+
+    /**
+     * Recursively set resolved bidirectional level of FO (and its ancestors) if
+     * and only if it is non-negative and if either the current value is reset (-1)
+     * or the new value is less than the current value.
+     * @param bidiLevel a non-negative bidi embedding level
+     */
+    public void setBidiLevel(int bidiLevel) {
+
+        assert bidiLevel >= 0;
+
+        if ((this.bidiLevel < 0) || (bidiLevel < this.bidiLevel)) {
+            this.bidiLevel = bidiLevel;
+            if ((parent != null) && !isBidiPropagationBoundary()) {
+                FObj foParent = (FObj) parent;
+                int parentBidiLevel = foParent.getBidiLevel();
+                if ((parentBidiLevel < 0) || (bidiLevel < parentBidiLevel)) {
+                    foParent.setBidiLevel(bidiLevel);
+                }
+            }
+        }
+    }
+
+    /**
+     * Obtain resolved bidirectional level of FO.
+     * @return either a non-negative bidi embedding level or -1
+     * in case no bidi levels have been assigned
+     */
+    public int getBidiLevel() {
+        return bidiLevel;
+    }
+
+    /**
+     * Obtain resolved bidirectional level of FO or nearest FO
+     * ancestor that has a resolved level.
+     * @return either a non-negative bidi embedding level or -1
+     * in case no bidi levels have been assigned to this FO or
+     * any ancestor
+     */
+    public int getBidiLevelRecursive() {
+        for (FONode fn = this; fn != null; fn = fn.getParent()) {
+            if (fn instanceof FObj) {
+                int level = ((FObj) fn).getBidiLevel();
+                if (level >= 0) {
+                    return level;
+                }
+            }
+            if (isBidiInheritanceBoundary()) {
+                break;
+            }
+        }
+        return -1;
+    }
+
+    protected boolean isBidiBoundary(boolean propagate) {
+        return false;
+    }
+
+    private boolean isBidiInheritanceBoundary() {
+        return isBidiBoundary(false);
+    }
+
+    private boolean isBidiPropagationBoundary() {
+        return isBidiBoundary(true);
+    }
+
     /**
      * Add a new extension attachment to this FObj.
      * (see org.apache.fop.fo.FONode for details)
@@ -554,7 +681,7 @@ public abstract class FObj extends FONode implements Constants {
                     "Parameter attachment must not be null");
         }
         if (extensionAttachments == null) {
-            extensionAttachments = new java.util.ArrayList/*<ExtensionAttachment>*/();
+            extensionAttachments = new java.util.ArrayList<ExtensionAttachment>();
         }
         if (log.isDebugEnabled()) {
             log.debug("ExtensionAttachment of category "
@@ -591,7 +718,7 @@ public abstract class FObj extends FONode implements Constants {
             throw new NullPointerException("Parameter attributeName must not be null");
         }
         if (foreignAttributes == null) {
-            foreignAttributes = new java.util.HashMap/*<QName,String>*/();
+            foreignAttributes = new java.util.HashMap<QName, String>();
         }
         foreignAttributes.put(attributeName, value);
     }
@@ -610,8 +737,8 @@ public abstract class FObj extends FONode implements Constants {
         return (super.toString() + "[@id=" + this.id + "]");
     }
 
-    /** Basic {@link FONodeIterator} implementation */
-    public class FObjIterator implements FONodeIterator {
+    /** Basic {@link FONode.FONodeIterator} implementation */
+    public static class FObjIterator implements FONodeIterator {
 
         private static final int F_NONE_ALLOWED = 0;
         private static final int F_SET_ALLOWED = 1;
@@ -630,12 +757,12 @@ public abstract class FObj extends FONode implements Constants {
         }
 
         /** {@inheritDoc} */
-        public FObj parentNode() {
+        public FObj parent() {
             return parentNode;
         }
 
         /** {@inheritDoc} */
-        public Object next() {
+        public FONode next() {
             if (currentNode != null) {
                 if (currentIndex != 0) {
                     if (currentNode.siblings != null
@@ -654,7 +781,7 @@ public abstract class FObj extends FONode implements Constants {
         }
 
         /** {@inheritDoc} */
-        public Object previous() {
+        public FONode previous() {
             if (currentNode.siblings != null
                     && currentNode.siblings[0] != null) {
                 currentIndex--;
@@ -667,9 +794,8 @@ public abstract class FObj extends FONode implements Constants {
         }
 
         /** {@inheritDoc} */
-        public void set(Object o) {
+        public void set(FONode newNode) {
             if ((flags & F_SET_ALLOWED) == F_SET_ALLOWED) {
-                FONode newNode = (FONode) o;
                 if (currentNode == parentNode.firstChild) {
                     parentNode.firstChild = newNode;
                 } else {
@@ -679,14 +805,16 @@ public abstract class FObj extends FONode implements Constants {
                         && currentNode.siblings[1] != null) {
                     FONode.attachSiblings(newNode, currentNode.siblings[1]);
                 }
+                if (currentNode == parentNode.lastChild) {
+                    parentNode.lastChild = newNode;
+                }
             } else {
                 throw new IllegalStateException();
             }
         }
 
         /** {@inheritDoc} */
-        public void add(Object o) {
-            FONode newNode = (FONode) o;
+        public void add(FONode newNode) {
             if (currentIndex == -1) {
                 if (currentNode != null) {
                     FONode.attachSiblings(newNode, currentNode);
@@ -694,12 +822,18 @@ public abstract class FObj extends FONode implements Constants {
                 parentNode.firstChild = newNode;
                 currentIndex = 0;
                 currentNode = newNode;
+                if (parentNode.lastChild == null) {
+                    parentNode.lastChild = newNode;
+                }
             } else {
                 if (currentNode.siblings != null
                         && currentNode.siblings[1] != null) {
-                    FONode.attachSiblings((FONode) o, currentNode.siblings[1]);
+                    FONode.attachSiblings(newNode, currentNode.siblings[1]);
+                }
+                FONode.attachSiblings(currentNode, newNode);
+                if (currentNode == parentNode.lastChild) {
+                    parentNode.lastChild = newNode;
                 }
-                FONode.attachSiblings(currentNode, (FONode) o);
             }
             flags &= F_NONE_ALLOWED;
         }
@@ -750,7 +884,7 @@ public abstract class FObj extends FONode implements Constants {
         }
 
         /** {@inheritDoc} */
-        public FONode lastNode() {
+        public FONode last() {
             while (currentNode != null
                     && currentNode.siblings != null
                     && currentNode.siblings[1] != null) {
@@ -761,20 +895,10 @@ public abstract class FObj extends FONode implements Constants {
         }
 
         /** {@inheritDoc} */
-        public FONode firstNode() {
+        public FONode first() {
             currentNode = parentNode.firstChild;
             currentIndex = 0;
             return currentNode;
         }
-
-        /** {@inheritDoc} */
-        public FONode nextNode() {
-            return (FONode) next();
-        }
-
-        /** {@inheritDoc} */
-        public FONode previousNode() {
-            return (FONode) previous();
-        }
     }
 }
