@@ -15,11 +15,16 @@
  * limitations under the License.
  */
 
-/* $Id: PropertyList.java 693841 2008-09-10 13:24:00Z adelmelle $ */
+/* $Id: PropertyList.java 1616312 2014-08-06 19:19:31Z gadams $ */
 
 package org.apache.fop.fo;
 
 // Java
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Map;
+import java.util.Set;
+
 import org.xml.sax.Attributes;
 
 import org.apache.commons.logging.Log;
@@ -27,10 +32,10 @@ import org.apache.commons.logging.LogFactory;
 
 import org.apache.xmlgraphics.util.QName;
 
-import org.apache.fop.apps.FopFactory;
+import org.apache.fop.apps.FOPException;
+import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.fo.expr.PropertyException;
 import org.apache.fop.fo.properties.CommonAbsolutePosition;
-import org.apache.fop.fo.properties.CommonAccessibility;
 import org.apache.fop.fo.properties.CommonAural;
 import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
 import org.apache.fop.fo.properties.CommonFont;
@@ -47,17 +52,25 @@ import org.apache.fop.fo.properties.PropertyMaker;
  */
 public abstract class PropertyList {
 
-    // writing-mode index
-    private int writingMode;
-
     private static boolean[] inheritableProperty;
 
+    static {
+        inheritableProperty = new boolean[Constants.PROPERTY_COUNT + 1];
+        PropertyMaker maker = null;
+        for (int prop = 1; prop <= Constants.PROPERTY_COUNT; prop++) {
+            maker = findMaker(prop);
+            inheritableProperty[prop] = (maker != null && maker.isInherited());
+        }
+    }
+
     /** reference to the parent FO's propertyList **/
-    protected PropertyList parentPropertyList = null;
-    private FObj fobj = null;
+    protected PropertyList parentPropertyList;
+    private FObj fobj;
 
     private static Log log = LogFactory.getLog(PropertyList.class);
 
+    private final UnknownPropertyHandler unknownPropertyHandler = new UnknownPropertyHandler();
+
     /**
      * Basic constructor.
      * @param fObjToAttach  the FO this PropertyList should be attached to
@@ -87,6 +100,25 @@ public abstract class PropertyList {
         }
     }
 
+    /**
+     * Adds an unknown property value to the property list so that if
+     * necessary, a warning can be displayed.
+     * @param propertyValue The unknown property value
+     * @param output The output of the property to validate
+     * @param property The original property containing the full value
+     */
+    public void validatePropertyValue(String propertyValue, Property output, Property property) {
+        unknownPropertyHandler.validatePropertyValue(propertyValue, output, property);
+    }
+
+    /**
+     * Gets the current list of unknown property values
+     * @return The set containing the list of unknown property values
+     */
+    public Map<String, Property> getUnknownPropertyValues() {
+        return unknownPropertyHandler.getUnknownPropertyValues();
+    }
+
     /**
      * @return the FObj object attached to the parentPropetyList
      */
@@ -110,6 +142,36 @@ public abstract class PropertyList {
         return p;
     }
 
+    /**
+     * A class to handle unknown shorthand property values e.g. border="solit 1pt"
+     */
+    private static class UnknownPropertyHandler {
+
+        /**
+         * A list of unknown properties identified by the value and property in which its featured
+         */
+        private Map<String, Property> unknownPropertyValues = new HashMap<String, Property>();
+
+        /**
+         * A list of known properties which have already been processed
+         */
+        private Set<Property> knownProperties = new HashSet<Property>();
+
+        void validatePropertyValue(String propertyValue, Property output, Property property) {
+            if (!knownProperties.contains(property) && output == null) {
+                if (propertyValue != null) {
+                    unknownPropertyValues.put(propertyValue, property);
+                }
+            } else {
+                knownProperties.add(property);
+            }
+        }
+
+        Map<String, Property> getUnknownPropertyValues() {
+            return unknownPropertyValues;
+        }
+    }
+
     /**
      * Return the value explicitly specified on this FO.
      * @param propId The ID of the property whose value is desired.
@@ -221,51 +283,60 @@ public abstract class PropertyList {
     }
 
     /**
-     * Set writing mode for this FO.
-     * Use that from the nearest ancestor, including self, which generates
-     * reference areas, or from root FO if no ancestor found.
-     * @throws PropertyException ...
-     */
-    public void setWritingMode() throws PropertyException {
-        FObj p = fobj.findNearestAncestorFObj();
-        // If this is a reference area or the root, use the property value.
-        if (fobj.generatesReferenceAreas() || p == null) {
-            writingMode = get(Constants.PR_WRITING_MODE).getEnum();
-        } else {
-            // Otherwise get the writing mode value from the parent.
-            writingMode = getParentPropertyList().getWritingMode();
-        }
-    }
-
-    /**
-     * Return the "writing-mode" property value.
-     * @return the "writing-mode" property value.
-     */
-    public int getWritingMode() {
-        return writingMode;
-    }
-
-
-    /**
-     * Uses the stored writingMode.
+     * Select a writing mode dependent property ID based on value of writing mode property.
      * @param lrtb the property ID to return under lrtb writingmode.
      * @param rltb the property ID to return under rltb writingmode.
      * @param tbrl the property ID to return under tbrl writingmode.
+     * @param tblr the property ID to return under tblr writingmode.
      * @return one of the property IDs, depending on the writing mode.
      */
-    public int getWritingMode(int lrtb, int rltb, int tbrl) {
-        switch (writingMode) {
-            case Constants.EN_LR_TB: return lrtb;
-            case Constants.EN_RL_TB: return rltb;
-            case Constants.EN_TB_RL: return tbrl;
+    public int selectFromWritingMode(int lrtb, int rltb, int tbrl, int tblr) {
+        int propID;
+        try {
+            switch (get(Constants.PR_WRITING_MODE).getEnum()) {
+            case Constants.EN_LR_TB:
+                propID = lrtb;
+                break;
+            case Constants.EN_RL_TB:
+                propID = rltb;
+                break;
+            case Constants.EN_TB_RL:
+                propID = tbrl;
+                break;
+            case Constants.EN_TB_LR:
+                propID = tblr;
+                break;
             default:
-                //nop
+            propID = -1;
+                break;
+            }
+        } catch (PropertyException e) {
+            propID = -1;
         }
-        return -1;
+        return propID;
+    }
+
+    private String addAttributeToList(Attributes attributes,
+                                    String attributeName) throws ValidationException {
+        String attributeValue = attributes.getValue(attributeName);
+        if (attributeValue != null) {
+            convertAttributeToProperty(attributes, attributeName, attributeValue);
+        }
+        return attributeValue;
     }
 
     /**
-     * Adds the attributes, passed in by the parser to the PropertyList
+     * <p>Adds the attributes, passed in by the parser to the PropertyList.</p>
+     * <p>Note that certain attributes are given priority in terms of order of
+     * processing due to conversion dependencies, where the order is as follows:</p>
+     * <ol>
+     * <li>writing-mode</li>
+     * <li>column-number</li>
+     * <li>number-columns-spanned</li>
+     * <li>font</li>
+     * <li>font-size</li>
+     * <li><emph>all others in order of appearance</emph></li>
+     * </ol>
      *
      * @param attributes Collection of attributes passed to us from the parser.
      * @throws ValidationException if there is an attribute that does not
@@ -273,52 +344,48 @@ public abstract class PropertyList {
      */
     public void addAttributesToList(Attributes attributes)
                     throws ValidationException {
+        /*
+         * Give writing-mode highest conversion priority.
+         */
+        addAttributeToList(attributes, "writing-mode");
+
         /*
          * If column-number/number-columns-spanned are specified, then we
          * need them before all others (possible from-table-column() on any
          * other property further in the list...
          */
-        String attributeName = "column-number";
-        String attributeValue = attributes.getValue(attributeName);
-        convertAttributeToProperty(attributes, attributeName,
-            attributeValue);
-        attributeName = "number-columns-spanned";
-        attributeValue = attributes.getValue(attributeName);
-        convertAttributeToProperty(attributes, attributeName,
-            attributeValue);
+        addAttributeToList(attributes, "column-number");
+        addAttributeToList(attributes, "number-columns-spanned");
 
         /*
          * If font-size is set on this FO, must set it first, since
          * other attributes specified in terms of "ems" depend on it.
          */
-        attributeName = "font";
-        attributeValue = attributes.getValue(attributeName);
-        convertAttributeToProperty(attributes, attributeName,
-                attributeValue);
-        if (attributeValue == null) {
+        String checkValue = addAttributeToList(attributes, "font");
+        if (checkValue == null || "".equals(checkValue)) {
             /*
              * font shorthand wasn't specified, so still need to process
              * explicit font-size
              */
-            attributeName = "font-size";
-            attributeValue = attributes.getValue(attributeName);
-            convertAttributeToProperty(attributes, attributeName,
-                    attributeValue);
+            addAttributeToList(attributes, "font-size");
         }
 
         String attributeNS;
-        FopFactory factory = getFObj().getUserAgent().getFactory();
+        String attributeName;
+        String attributeValue;
+        FOUserAgent userAgent = getFObj().getUserAgent();
         for (int i = 0; i < attributes.getLength(); i++) {
             /* convert all attributes with the same namespace as the fo element
-             * the "xml:lang" property is a special case */
+             * the "xml:lang" and "xml:base" properties are special cases */
             attributeNS = attributes.getURI(i);
             attributeName = attributes.getQName(i);
             attributeValue = attributes.getValue(i);
             if (attributeNS == null || attributeNS.length() == 0
-                    || "xml:lang".equals(attributeName)) {
+                    || "xml:lang".equals(attributeName)
+                    || "xml:base".equals(attributeName)) {
                 convertAttributeToProperty(attributes, attributeName, attributeValue);
-            } else if (!factory.isNamespaceIgnored(attributeNS)) {
-                ElementMapping mapping = factory.getElementMappingRegistry().getElementMapping(
+            } else if (!userAgent.isNamespaceIgnored(attributeNS)) {
+                ElementMapping mapping = userAgent.getElementMappingRegistry().getElementMapping(
                         attributeNS);
                 QName attr = new QName(attributeNS, attributeName);
                 if (mapping != null) {
@@ -355,6 +422,27 @@ public abstract class PropertyList {
                 && findSubPropertyName(propertyName) != null));
     }
 
+    public Property getPropertyForAttribute(Attributes attributes, String attributeName, String attributeValue)
+            throws FOPException {
+        if (attributeValue != null) {
+            if (attributeName.startsWith("xmlns:") || "xmlns".equals(attributeName)) {
+                return null;
+            }
+            String basePropertyName = findBasePropertyName(attributeName);
+            String subPropertyName = findSubPropertyName(attributeName);
+
+            int propId = FOPropertyMapping.getPropertyId(basePropertyName);
+            int subpropId = FOPropertyMapping.getSubPropertyId(subPropertyName);
+
+            if (propId == -1 || (subpropId == -1 && subPropertyName != null)) {
+                return null;
+            }
+
+            return getExplicit(propId);
+        }
+        return null;
+    }
+
     /**
      *
      * @param attributes Collection of attributes
@@ -368,15 +456,14 @@ public abstract class PropertyList {
                                             String attributeValue)
                     throws ValidationException {
 
+        if (attributeName.startsWith("xmlns:")
+                || "xmlns".equals(attributeName)) {
+            /* Ignore namespace declarations if the XML parser/XSLT processor
+             * reports them as 'regular' attributes */
+            return;
+        }
+
         if (attributeValue != null) {
-
-            if (attributeName.startsWith("xmlns:")
-                    || "xmlns".equals(attributeName)) {
-                //Ignore namespace declarations if the XML parser/XSLT processor
-                //reports them as 'regular' attributes
-                return;
-            }
-
             /* Handle "compound" properties, ex. space-before.minimum */
             String basePropertyName = findBasePropertyName(attributeName);
             String subPropertyName = findSubPropertyName(attributeName);
@@ -541,15 +628,6 @@ public abstract class PropertyList {
      * @return isInherited value from the requested Property.Maker
      */
     private boolean isInherited(int propId) {
-        if (inheritableProperty == null) {
-            inheritableProperty = new boolean[Constants.PROPERTY_COUNT + 1];
-            PropertyMaker maker = null;
-            for (int prop = 1; prop <= Constants.PROPERTY_COUNT; prop++) {
-                maker = findMaker(prop);
-                inheritableProperty[prop] = (maker != null && maker.isInherited());
-            }
-        }
-
         return inheritableProperty[propId];
     }
 
@@ -557,8 +635,7 @@ public abstract class PropertyList {
      * @param propId Id of property
      * @return the Property.Maker for this property
      */
-    private PropertyMaker findMaker(int propId) {
-
+    private static PropertyMaker findMaker(int propId) {
         if (propId < 1 || propId > Constants.PROPERTY_COUNT) {
             return null;
         } else {
@@ -603,15 +680,6 @@ public abstract class PropertyList {
         return new CommonMarginInline(this);
     }
 
-    /**
-     * Constructs a CommonAccessibility object.
-     * @return the CommonAccessibility object
-     * @throws PropertyException if there's a problem while processing the properties
-     */
-    public CommonAccessibility getAccessibilityProps() throws PropertyException {
-        return new CommonAccessibility(this);
-    }
-
     /**
      * Constructs a CommonAural object.
      * @return the CommonAural object
@@ -659,4 +727,3 @@ public abstract class PropertyList {
         return CommonTextDecoration.createFromPropertyList(this);
     }
 }
-
