@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RecursiveCharIterator.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: RecursiveCharIterator.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.fo;
 
@@ -32,12 +32,12 @@ public class RecursiveCharIterator extends CharIterator {
     /** parent node for whose child nodes this iterator iterates */
     private FONode fobj;
     /** iterator for the child nodes */
-    private Iterator childIter = null;
+    private Iterator childIter;
 
     /** current child object that is being managed by childIter*/
     private FONode curChild;
     /** CharIterator for curChild's characters */
-    private CharIterator curCharIter = null;
+    private CharIterator curCharIter;
 
     /**
      * Constructor which creates an iterator for all child nodes
