@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,21 +15,24 @@
  * limitations under the License.
  */
 
-/* $Id: SVGElementMapping.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: SVGElementMapping.java 1664570 2015-03-06 09:41:07Z lbernardo $ */
 
 package org.apache.fop.fo.extensions.svg;
 
 import java.util.HashMap;
+
 import javax.xml.parsers.SAXParserFactory;
 
-import org.apache.fop.fo.FONode;
-import org.apache.fop.fo.ElementMapping;
+import org.w3c.dom.DOMImplementation;
 
-import org.apache.batik.util.XMLResourceDescriptor;
-import org.apache.batik.dom.svg.SVGDOMImplementation;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-import org.w3c.dom.DOMImplementation;
+
+import org.apache.batik.anim.dom.SVGDOMImplementation;
+import org.apache.batik.util.XMLResourceDescriptor;
+
+import org.apache.fop.fo.ElementMapping;
+import org.apache.fop.fo.FONode;
 
 /**
  * Setup the SVG element mapping.
@@ -37,13 +40,13 @@ import org.w3c.dom.DOMImplementation;
  * that create the SVG Document.
  */
 public class SVGElementMapping extends ElementMapping {
-    
+
     /** the SVG namespace */
     public static final String URI = SVGDOMImplementation.SVG_NAMESPACE_URI;
-    
+
     /** logging instance */
     protected Log log = LogFactory.getLog(SVGElementMapping.class);
-    
+
     private boolean batikAvailable = true;
 
     /** Main constructor. */
@@ -80,7 +83,7 @@ public class SVGElementMapping extends ElementMapping {
                 XMLResourceDescriptor.setXMLParserClassName(
                   getAParserClassName());
 
-                foObjs = new HashMap();
+                foObjs = new HashMap<String, Maker>();
                 foObjs.put("svg", new SE());
                 foObjs.put(DEFAULT, new SVGMaker());
             } catch (Throwable t) {
