@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Block.java 1293736 2012-02-26 02:29:01Z gadams $ */
+/* $Id: Block.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.fo.flow;
 
@@ -51,8 +51,8 @@ public class Block extends FObjMixed implements BreakPropertySet,
         CommonAccessibilityHolder {
 
     // used for FO validation
-    private boolean blockOrInlineItemFound = false;
-    private boolean initialPropertySetFound = false;
+    private boolean blockOrInlineItemFound;
+    private boolean initialPropertySetFound;
 
     // The value of FO traits (refined properties) that apply to fo:block.
     private CommonAccessibility commonAccessibility;
@@ -140,13 +140,13 @@ public class Block extends FObjMixed implements BreakPropertySet,
     }
 
     /** {@inheritDoc} */
-    protected void startOfNode() throws FOPException {
+    public void startOfNode() throws FOPException {
         super.startOfNode();
         getFOEventHandler().startBlock(this);
     }
 
     /** {@inheritDoc} */
-    protected void endOfNode() throws FOPException {
+    public void endOfNode() throws FOPException {
         super.endOfNode();
         getFOEventHandler().endBlock(this);
     }
