@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Marker.java 1324916 2012-04-11 18:52:19Z gadams $ */
+/* $Id: Marker.java 1617052 2014-08-10 06:55:01Z gadams $ */
 
 package org.apache.fop.fo.flow;
 
@@ -83,7 +83,7 @@ public class Marker extends FObjMixed {
     }
 
     /** {@inheritDoc} */
-    protected void startOfNode() {
+    public void startOfNode() {
         FOTreeBuilderContext builderContext = getBuilderContext();
         // Push a new property list maker which will make MarkerPropertyLists.
         savePropertyListMaker = builderContext.getPropertyListMaker();
@@ -97,7 +97,7 @@ public class Marker extends FObjMixed {
     }
 
     /** {@inheritDoc} */
-    protected void endOfNode() throws FOPException {
+    public void endOfNode() throws FOPException {
         super.endOfNode();
         // Pop the MarkerPropertyList maker.
         getBuilderContext().setPropertyListMaker(savePropertyListMaker);
@@ -361,13 +361,13 @@ public class Marker extends FObjMixed {
                 = new PropertyCache<MarkerAttribute>();
 
         /** namespace */
-        protected String namespace;
+        private String namespace;
         /** qualfied name */
-        protected String qname;
+        private String qname;
         /** local name */
-        protected String name;
+        private String name;
         /** value */
-        protected String value;
+        private String value;
 
         /**
          * Main constructor
@@ -420,18 +420,14 @@ public class Marker extends FObjMixed {
 
             if (o instanceof MarkerAttribute) {
                 MarkerAttribute attr = (MarkerAttribute) o;
-                return ((attr.namespace == this.namespace)
-                            || (attr.namespace != null
-                                    && attr.namespace.equals(this.namespace)))
-                    && ((attr.qname == this.qname)
-                            || (attr.qname != null
-                                    && attr.qname.equals(this.qname)))
-                    && ((attr.name == this.name)
-                            || (attr.name != null
-                                    && attr.name.equals(this.name)))
-                    && ((attr.value == this.value)
-                            || (attr.value != null
-                                    && attr.value.equals(this.value)));
+                return (attr.namespace != null
+                                    && attr.namespace.equals(this.namespace))
+                    && (attr.qname != null
+                                    && attr.qname.equals(this.qname))
+                    && (attr.name != null
+                                    && attr.name.equals(this.name))
+                    && (attr.value != null
+                                    && attr.value.equals(this.value));
             } else {
                 return false;
             }
