@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: CollapsingBorderResolver.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: CollapsingBorderResolver.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.fo.flow.table;
 
@@ -62,7 +62,7 @@ class CollapsingBorderResolver implements BorderResolver {
 
     /* TODO Temporary hack for resolved borders in header */
     /* Currently the normal border is always used. */
-    private List/*<GridUnit>*/ headerLastRow = null;
+    private List/*<GridUnit>*/ headerLastRow;
     /* End of temporary hack */
 
     /**
