@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TableRow.java 985227 2010-08-13 15:03:17Z spepping $ */
+/* $Id: TableRow.java 1465599 2013-04-08 11:51:52Z vhennebert $ */
 
 package org.apache.fop.fo.flow.table;
 
@@ -98,13 +98,13 @@ public class TableRow extends TableCellContainer implements BreakPropertySet {
     }
 
     /** {@inheritDoc} */
-    protected void startOfNode() throws FOPException {
+    public void startOfNode() throws FOPException {
         super.startOfNode();
         getFOEventHandler().startRow(this);
     }
 
     /** {@inheritDoc} */
-    protected void endOfNode() throws FOPException {
+    public void endOfNode() throws FOPException {
         super.endOfNode();
         getFOEventHandler().endRow(this);
     }
