@@ -15,32 +15,39 @@
  * limitations under the License.
  */
 
-/* $Id: PageSequence.java 758234 2009-03-25 12:08:04Z acumiskey $ */
+/* $Id: PageSequence.java 1723297 2016-01-06 12:13:52Z ssteiner $ */
 
 package org.apache.fop.fo.pagination;
 
-// Java
+import java.util.Locale;
 import java.util.Map;
+import java.util.Stack;
 
 import org.xml.sax.Locator;
 
 import org.apache.fop.apps.FOPException;
+import org.apache.fop.complexscripts.bidi.DelimitedTextRange;
+import org.apache.fop.datatypes.Numeric;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.ValidationException;
+import org.apache.fop.fo.properties.CommonHyphenation;
+import org.apache.fop.traits.Direction;
+import org.apache.fop.traits.WritingMode;
+import org.apache.fop.traits.WritingModeTraits;
+import org.apache.fop.traits.WritingModeTraitsGetter;
 
 /**
  * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_page-sequence">
  * <code>fo:page-sequence</code></a> object.
  */
-public class PageSequence extends AbstractPageSequence {
+public class PageSequence extends AbstractPageSequence implements WritingModeTraitsGetter {
 
-    // The value of properties relevant for fo:page-sequence.
-    private String country;
-    private String language;
     private String masterReference;
-    //private int writingMode; //XSL 1.1
-    // End of property values
+    private Numeric referenceOrientation;
+    private WritingModeTraits writingModeTraits;
+
+    private Locale locale;
 
     // There doesn't seem to be anything in the spec requiring flows
     // to be in the order given, only that they map to the regions
@@ -48,7 +55,7 @@ public class PageSequence extends AbstractPageSequence {
     // the set of flows includes StaticContent flows also
 
     /** Map of flows to their flow name (flow-name, Flow) */
-    private Map/*<String, Flow>*/ flowMap;
+    private Map<String, FONode> flowMap;
 
     /**
      * The currentSimplePageMaster is either the page master for the
@@ -68,7 +75,7 @@ public class PageSequence extends AbstractPageSequence {
     /**
      * The fo:flow object for this page-sequence.
      */
-    private Flow mainFlow = null;
+    private Flow mainFlow;
 
     /**
      * Create a PageSequence instance that is a child of the
@@ -83,20 +90,23 @@ public class PageSequence extends AbstractPageSequence {
     /** {@inheritDoc} */
     public void bind(PropertyList pList) throws FOPException {
         super.bind(pList);
-        country = pList.get(PR_COUNTRY).getString();
-        language = pList.get(PR_LANGUAGE).getString();
+        String country = pList.get(PR_COUNTRY).getString();
+        String language = pList.get(PR_LANGUAGE).getString();
+        locale = CommonHyphenation.toLocale(language, country);
         masterReference = pList.get(PR_MASTER_REFERENCE).getString();
-        //writingMode = pList.getWritingMode();
-
+        referenceOrientation = pList.get(PR_REFERENCE_ORIENTATION).getNumeric();
+        writingModeTraits = new WritingModeTraits(
+            WritingMode.valueOf(pList.get(PR_WRITING_MODE).getEnum()),
+            pList.getExplicit(PR_WRITING_MODE) != null);
         if (masterReference == null || masterReference.equals("")) {
             missingPropertyError("master-reference");
         }
     }
 
     /** {@inheritDoc} */
-    protected void startOfNode() throws FOPException {
+    public void startOfNode() throws FOPException {
         super.startOfNode();
-        flowMap = new java.util.HashMap/*<String, Flow>*/();
+        flowMap = new java.util.HashMap<String, FONode>();
 
         this.simplePageMaster
             = getRoot().getLayoutMasterSet().getSimplePageMaster(masterReference);
@@ -112,7 +122,7 @@ public class PageSequence extends AbstractPageSequence {
     }
 
     /** {@inheritDoc} */
-    protected void endOfNode() throws FOPException {
+    public void endOfNode() throws FOPException {
         if (mainFlow == null) {
            missingChildElementError("(title?,static-content*,flow)");
         }
@@ -151,7 +161,7 @@ public class PageSequence extends AbstractPageSequence {
 
     /**
      * {@inheritDoc}
-     * @todo see if addChildNode() should also be called for fo's other than
+     * TODO see if addChildNode() should also be called for fo's other than
      *  fo:flow.
      */
     public void addChildNode(FONode child) throws FOPException {
@@ -167,7 +177,7 @@ public class PageSequence extends AbstractPageSequence {
             break;
         case FO_STATIC_CONTENT:
             addFlow((StaticContent)child);
-            flowMap.put(((StaticContent)child).getFlowName(), child);
+            flowMap.put(((Flow)child).getFlowName(), (Flow)child);
             break;
         default:
             super.addChildNode(child);
@@ -183,7 +193,7 @@ public class PageSequence extends AbstractPageSequence {
      * @throws org.apache.fop.fo.ValidationException if the fo:flow maps
      * to an invalid page-region
      */
-     private void addFlow(Flow flow) throws ValidationException {
+    private void addFlow(Flow flow) throws ValidationException {
         String flowName = flow.getFlowName();
 
         if (hasFlowName(flowName)) {
@@ -191,14 +201,20 @@ public class PageSequence extends AbstractPageSequence {
                     flowName, flow.getLocator());
         }
 
-        if (!getRoot().getLayoutMasterSet().regionNameExists(flowName)
-            && !flowName.equals("xsl-before-float-separator")
-            && !flowName.equals("xsl-footnote-separator")) {
+        if (!hasRegion(flowName) && !flowName.equals("xsl-before-float-separator")
+                && !flowName.equals("xsl-footnote-separator")) {
             getFOValidationEventProducer().flowNameNotMapped(this, flow.getName(),
                     flowName, flow.getLocator());
         }
     }
 
+    private boolean hasRegion(String flowName) {
+        LayoutMasterSet set = getRoot().getLayoutMasterSet();
+        PageSequenceMaster psm = set.getPageSequenceMaster(masterReference);
+        return (psm != null) ? psm.getLayoutMasterSet().regionNameExists(flowName)
+                : set.getSimplePageMaster(masterReference).regionNameExists(flowName);
+    }
+
     /**
      * Get the static content FO node from the flow map.
      * This gets the static content flow for the given flow name.
@@ -239,7 +255,7 @@ public class PageSequence extends AbstractPageSequence {
     }
 
     /** @return the flow map for this page-sequence */
-    public Map getFlowMap() {
+    public Map<String, FONode> getFlowMap() {
         return this.flowMap;
     }
 
@@ -254,15 +270,14 @@ public class PageSequence extends AbstractPageSequence {
      * @return the SimplePageMaster to use for this page
      * @throws PageProductionException if there's a problem determining the page master
      */
-    public SimplePageMaster getNextSimplePageMaster(int page,
-                                                    boolean isFirstPage,
-                                                    boolean isLastPage,
-                                                    boolean isBlank) throws PageProductionException {
+    public SimplePageMaster getNextSimplePageMaster(
+        int page, boolean isFirstPage, boolean isLastPage, boolean isBlank)
+        throws PageProductionException {
 
         if (pageSequenceMaster == null) {
             return simplePageMaster;
         }
-        boolean isOddPage = ((page % 2) == 1);
+        boolean isOddPage = ((page % 2) != 0);
         if (log.isDebugEnabled()) {
             log.debug("getNextSimplePageMaster(page=" + page
                     + " isOdd=" + isOddPage
@@ -271,7 +286,7 @@ public class PageSequence extends AbstractPageSequence {
                     + " isBlank=" + isBlank + ")");
         }
         return pageSequenceMaster.getNextSimplePageMaster(isOddPage,
-            isFirstPage, isLastPage, isBlank);
+            isFirstPage, isLastPage, isBlank, getMainFlow().getFlowName());
     }
 
     /**
@@ -293,8 +308,8 @@ public class PageSequence extends AbstractPageSequence {
     }
 
     /**
-     * Get the value of the <code>master-reference</code> property.
-     * @return the "master-reference" property
+     * Get the value of the <code>master-reference</code> trait.
+     * @return the "master-reference" trait
      */
     public String getMasterReference() {
         return masterReference;
@@ -313,20 +328,122 @@ public class PageSequence extends AbstractPageSequence {
         return FO_PAGE_SEQUENCE;
     }
 
-    /**
-     * Get the value of the <code>country</code> property.
-     * @return the country property value
-     */
-    public String getCountry() {
-        return this.country;
+    public Locale getLocale() {
+        return locale;
     }
 
     /**
-     * Get the value of the <code>language</code> property.
-     * @return the language property value
+     * Get the value of the <code>reference-orientation</code> trait.
+     * @return the reference orientation trait value
      */
-    public String getLanguage() {
-        return this.language;
+    public int getReferenceOrientation() {
+        if (referenceOrientation != null) {
+            return referenceOrientation.getValue();
+        } else {
+            return 0;
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public Direction getInlineProgressionDirection() {
+        if (writingModeTraits != null) {
+            return writingModeTraits.getInlineProgressionDirection();
+        } else {
+            return Direction.LR;
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public Direction getBlockProgressionDirection() {
+        if (writingModeTraits != null) {
+            return writingModeTraits.getBlockProgressionDirection();
+        } else {
+            return Direction.TB;
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public Direction getColumnProgressionDirection() {
+        if (writingModeTraits != null) {
+            return writingModeTraits.getColumnProgressionDirection();
+        } else {
+            return Direction.LR;
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public Direction getRowProgressionDirection() {
+        if (writingModeTraits != null) {
+            return writingModeTraits.getRowProgressionDirection();
+        } else {
+            return Direction.TB;
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public Direction getShiftDirection() {
+        if (writingModeTraits != null) {
+            return writingModeTraits.getShiftDirection();
+        } else {
+            return Direction.TB;
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public WritingMode getWritingMode() {
+        if (writingModeTraits != null) {
+            return writingModeTraits.getWritingMode();
+        } else {
+            return WritingMode.LR_TB;
+        }
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public boolean getExplicitWritingMode() {
+        if (writingModeTraits != null) {
+            return writingModeTraits.getExplicitWritingMode();
+        } else {
+            return false;
+        }
+    }
+
+    @Override
+    protected Stack<DelimitedTextRange> collectDelimitedTextRanges(Stack<DelimitedTextRange> ranges,
+        DelimitedTextRange currentRange) {
+        // collect ranges from static content flows
+        Map<String, FONode> flows = getFlowMap();
+        if (flows != null) {
+            for (FONode fn : flows.values()) {
+                if (fn instanceof StaticContent) {
+                    ranges = ((StaticContent) fn).collectDelimitedTextRanges(ranges);
+                }
+            }
+        }
+        // collect ranges in main flow
+        Flow main = getMainFlow();
+        if (main != null) {
+            ranges = main.collectDelimitedTextRanges(ranges);
+        }
+        return ranges;
+    }
+
+    @Override
+    protected boolean isBidiBoundary(boolean propagate) {
+        return true;
     }
 
     /**
@@ -337,4 +454,14 @@ public class PageSequence extends AbstractPageSequence {
         this.flowMap.clear();
     }
 
+    public SimplePageMaster getLastSimplePageMaster(int page, boolean isFirstPage, boolean isBlank) {
+        boolean isOddPage = ((page % 2) != 0); // please findbugs...
+        log.debug("getNextSimplePageMaster(page=" + page + " isOdd=" + isOddPage + " isFirst="
+                + isFirstPage + " isLast=true" + " isBlank=" + isBlank + ")");
+        if (pageSequenceMaster == null) {
+            return simplePageMaster;
+        }
+        return pageSequenceMaster.getLastSimplePageMaster(isOddPage, isFirstPage, isBlank, getMainFlow()
+                .getFlowName());
+    }
 }
