@@ -15,11 +15,12 @@
  * limitations under the License.
  */
 
-/* $Id: PageSequenceMaster.java 698280 2008-09-23 18:48:34Z adelmelle $ */
+/* $Id: PageSequenceMaster.java 1723297 2016-01-06 12:13:52Z ssteiner $ */
 
 package org.apache.fop.fo.pagination;
 
 // Java
+import java.util.Collections;
 import java.util.List;
 
 import org.xml.sax.Locator;
@@ -44,9 +45,10 @@ public class PageSequenceMaster extends FObj {
     // End of property values
 
     private LayoutMasterSet layoutMasterSet;
-    private List subSequenceSpecifiers;
+    private List<SubSequenceSpecifier> subSequenceSpecifiers;
     private SubSequenceSpecifier currentSubSequence;
     private int currentSubSequenceNumber = -1;
+    private BlockLevelEventProducer blockLevelEventProducer;
 
     // The terminology may be confusing. A 'page-sequence-master' consists
     // of a sequence of what the XSL spec refers to as
@@ -60,9 +62,11 @@ public class PageSequenceMaster extends FObj {
      * given {@link FONode}.
      *
      * @param parent {@link FONode} that is the parent of this object
+     * @param blockLevelEventProducer event producer
      */
-    public PageSequenceMaster(FONode parent) {
+    public PageSequenceMaster(FONode parent, BlockLevelEventProducer blockLevelEventProducer) {
         super(parent);
+        this.blockLevelEventProducer = blockLevelEventProducer;
     }
 
     /** {@inheritDoc} */
@@ -75,14 +79,14 @@ public class PageSequenceMaster extends FObj {
     }
 
     /** {@inheritDoc} */
-    protected void startOfNode() throws FOPException {
-        subSequenceSpecifiers = new java.util.ArrayList();
+    public void startOfNode() throws FOPException {
+        subSequenceSpecifiers = new java.util.ArrayList<SubSequenceSpecifier>();
         layoutMasterSet = parent.getRoot().getLayoutMasterSet();
         layoutMasterSet.addPageSequenceMaster(masterName, this);
     }
 
     /** {@inheritDoc} */
-    protected void endOfNode() throws FOPException {
+    public void endOfNode() throws FOPException {
         if (firstChild == null) {
             missingChildElementError("(single-page-master-reference|"
                     + "repeatable-page-master-reference|repeatable-page-master-alternatives)+");
@@ -113,6 +117,10 @@ public class PageSequenceMaster extends FObj {
         subSequenceSpecifiers.add(pageMasterReference);
     }
 
+    public LayoutMasterSet getLayoutMasterSet() {
+        return layoutMasterSet;
+    }
+
     /**
      * Returns the next subsequence specifier
      * @return a subsequence specifier
@@ -121,12 +129,15 @@ public class PageSequenceMaster extends FObj {
         currentSubSequenceNumber++;
         if (currentSubSequenceNumber >= 0
                 && currentSubSequenceNumber < subSequenceSpecifiers.size()) {
-            return (SubSequenceSpecifier)subSequenceSpecifiers
-              .get(currentSubSequenceNumber);
+            return subSequenceSpecifiers.get(currentSubSequenceNumber);
         }
         return null;
     }
 
+    List<SubSequenceSpecifier> getSubSequenceSpecifier() {
+        return Collections.unmodifiableList(subSequenceSpecifiers);
+    }
+
     /**
      * Resets the subsequence specifiers subsystem.
      */
@@ -134,8 +145,8 @@ public class PageSequenceMaster extends FObj {
         currentSubSequenceNumber = -1;
         currentSubSequence = null;
         if (subSequenceSpecifiers != null) {
-            for (int i = 0; i < subSequenceSpecifiers.size(); i++) {
-                ((SubSequenceSpecifier)subSequenceSpecifiers.get(i)).reset();
+            for (SubSequenceSpecifier subSequenceSpecifier : subSequenceSpecifiers) {
+                subSequenceSpecifier.reset();
             }
         }
     }
@@ -150,7 +161,7 @@ public class PageSequenceMaster extends FObj {
             if (!success) {
                 if (currentSubSequenceNumber > 0) {
                     currentSubSequenceNumber--;
-                    currentSubSequence = (SubSequenceSpecifier)subSequenceSpecifiers
+                    currentSubSequence = subSequenceSpecifiers
                         .get(currentSubSequenceNumber);
                 } else {
                     currentSubSequence = null;
@@ -178,52 +189,58 @@ public class PageSequenceMaster extends FObj {
      * @param isFirstPage True if the next page is the first
      * @param isLastPage True if the next page is the last
      * @param isBlankPage True if the next page is blank
+     * @param mainFlowName the name of the main flow of the page sequence
      * @return the requested page master
      * @throws PageProductionException if there's a problem determining the next page master
      */
     public SimplePageMaster getNextSimplePageMaster(boolean isOddPage,
                                                     boolean isFirstPage,
                                                     boolean isLastPage,
-                                                    boolean isBlankPage)
+                                                    boolean isBlankPage,
+                                                    String mainFlowName)
                                                       throws PageProductionException {
         if (currentSubSequence == null) {
             currentSubSequence = getNextSubSequence();
             if (currentSubSequence == null) {
-                BlockLevelEventProducer eventProducer = BlockLevelEventProducer.Provider.get(
-                        getUserAgent().getEventBroadcaster());
-                eventProducer.missingSubsequencesInPageSequenceMaster(this,
+                blockLevelEventProducer.missingSubsequencesInPageSequenceMaster(this,
                         masterName, getLocator());
             }
+            if (currentSubSequence.isInfinite() && !currentSubSequence.canProcess(mainFlowName)) {
+                throw new PageProductionException(
+                "The current sub-sequence will not terminate whilst processing then main flow");
+            }
         }
-        String pageMasterName = currentSubSequence
-            .getNextPageMasterName(isOddPage, isFirstPage, isLastPage, isBlankPage);
+
+        SimplePageMaster pageMaster = currentSubSequence
+            .getNextPageMaster(isOddPage, isFirstPage, isLastPage, isBlankPage);
+
         boolean canRecover = true;
-        while (pageMasterName == null) {
+
+        while (pageMaster == null) {
             SubSequenceSpecifier nextSubSequence = getNextSubSequence();
+
             if (nextSubSequence == null) {
-                BlockLevelEventProducer eventProducer = BlockLevelEventProducer.Provider.get(
-                        getUserAgent().getEventBroadcaster());
-                eventProducer.pageSequenceMasterExhausted(this,
-                        masterName, canRecover, getLocator());
+                //Sub-sequence exhausted so attempt to reuse it
+                blockLevelEventProducer.pageSequenceMasterExhausted(this,
+                        masterName, canRecover & currentSubSequence.isReusable(), getLocator());
                 currentSubSequence.reset();
+                if (!currentSubSequence.canProcess(mainFlowName)) {
+                    throw new PageProductionException(
+                    "The last simple-page-master does not reference the main flow");
+                }
                 canRecover = false;
             } else {
                 currentSubSequence = nextSubSequence;
             }
-            pageMasterName = currentSubSequence
-                .getNextPageMasterName(isOddPage, isFirstPage, isLastPage, isBlankPage);
-        }
-        SimplePageMaster pageMaster = this.layoutMasterSet
-            .getSimplePageMaster(pageMasterName);
-        if (pageMaster == null) {
-            BlockLevelEventProducer eventProducer = BlockLevelEventProducer.Provider.get(
-                    getUserAgent().getEventBroadcaster());
-            eventProducer.noMatchingPageMaster(this,
-                    masterName, pageMasterName, getLocator());
+
+            pageMaster = currentSubSequence
+                .getNextPageMaster(isOddPage, isFirstPage, isLastPage, isBlankPage);
         }
+
         return pageMaster;
     }
 
+
     /** {@inheritDoc} */
     public String getLocalName() {
         return "page-sequence-master";
@@ -236,5 +253,25 @@ public class PageSequenceMaster extends FObj {
     public int getNameId() {
         return FO_PAGE_SEQUENCE_MASTER;
     }
+
+    public SimplePageMaster getLastSimplePageMaster(boolean isOddPage, boolean isFirstPage, boolean isBlank,
+            String flowName) {
+        if (currentSubSequence == null) {
+            currentSubSequence = getNextSubSequence();
+            if (currentSubSequence == null) {
+                blockLevelEventProducer.missingSubsequencesInPageSequenceMaster(this, masterName,
+                        getLocator());
+            }
+            if (currentSubSequence.isInfinite() && !currentSubSequence.canProcess(flowName)) {
+                throw new PageProductionException(
+                        "The current sub-sequence will not terminate whilst processing the main flow");
+            }
+        }
+
+        SimplePageMaster pageMaster = currentSubSequence.getLastPageMaster(isOddPage, isFirstPage, isBlank,
+                blockLevelEventProducer);
+        return pageMaster;
+    }
+
 }
 
