@@ -15,11 +15,12 @@
  * limitations under the License.
  */
 
-/* $Id: RepeatablePageMasterAlternatives.java 698280 2008-09-23 18:48:34Z adelmelle $ */
+/* $Id: RepeatablePageMasterAlternatives.java 1723297 2016-01-06 12:13:52Z ssteiner $ */
 
 package org.apache.fop.fo.pagination;
 
 // Java
+import java.util.ArrayList;
 import java.util.List;
 
 import org.xml.sax.Locator;
@@ -30,6 +31,7 @@ import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.ValidationException;
 import org.apache.fop.fo.properties.Property;
+import org.apache.fop.layoutmgr.BlockLevelEventProducer;
 
 /**
  * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_repeatable-page-master-alternatives">
@@ -46,11 +48,11 @@ public class RepeatablePageMasterAlternatives extends FObj
 
     private static final int INFINITE = -1;
 
-    private int numberConsumed = 0;
+    private int numberConsumed;
 
-    private List conditionalPageMasterRefs;
-    private boolean hasPagePositionLast = false;
-    private boolean hasPagePositionOnly = false;
+    private List<ConditionalPageMasterReference> conditionalPageMasterRefs;
+    private boolean hasPagePositionLast;
+    private boolean hasPagePositionOnly;
 
     /**
      * Base constructor
@@ -67,8 +69,8 @@ public class RepeatablePageMasterAlternatives extends FObj
     }
 
     /** {@inheritDoc} */
-    protected void startOfNode() throws FOPException {
-        conditionalPageMasterRefs = new java.util.ArrayList();
+    public void startOfNode() throws FOPException {
+        conditionalPageMasterRefs = new java.util.ArrayList<ConditionalPageMasterReference>();
 
         assert parent.getName().equals("fo:page-sequence-master"); //Validation by the parent
         PageSequenceMaster pageSequenceMaster = (PageSequenceMaster)parent;
@@ -76,7 +78,7 @@ public class RepeatablePageMasterAlternatives extends FObj
     }
 
     /** {@inheritDoc} */
-    protected void endOfNode() throws FOPException {
+    public void endOfNode() throws FOPException {
         if (firstChild == null) {
            missingChildElementError("(conditional-page-master-reference+)");
         }
@@ -114,30 +116,43 @@ public class RepeatablePageMasterAlternatives extends FObj
     }
 
     /** {@inheritDoc} */
-    public String getNextPageMasterName(boolean isOddPage,
+    public SimplePageMaster getNextPageMaster(boolean isOddPage,
                                         boolean isFirstPage,
                                         boolean isLastPage,
                                         boolean isBlankPage) {
-        if (getMaximumRepeats() != INFINITE) {
-            if (numberConsumed < getMaximumRepeats()) {
-                numberConsumed++;
-            } else {
-                return null;
-            }
-        } else {
-            numberConsumed++;
+
+        if (!isInfinite() && numberConsumed >= getMaximumRepeats()) {
+            return null;
         }
 
-        for (int i = 0; i < conditionalPageMasterRefs.size(); i++) {
-            ConditionalPageMasterReference cpmr
-                = (ConditionalPageMasterReference)conditionalPageMasterRefs.get(i);
+        numberConsumed++;
+
+        for (ConditionalPageMasterReference cpmr : conditionalPageMasterRefs) {
             if (cpmr.isValid(isOddPage, isFirstPage, isLastPage, isBlankPage)) {
-                return cpmr.getMasterReference();
+                return cpmr.getMaster();
             }
         }
+
+
         return null;
     }
 
+    public SimplePageMaster getLastPageMaster(boolean isOddPage, boolean isFirstPage, boolean isBlankPage,
+                                              BlockLevelEventProducer blockLevelEventProducer) {
+        for (ConditionalPageMasterReference cpmr : conditionalPageMasterRefs) {
+            if (cpmr.isValid(isOddPage, isFirstPage, true, isBlankPage)) {
+                return cpmr.getMaster();
+            }
+        }
+        blockLevelEventProducer.lastPageMasterReferenceMissing(this, getLocator());
+        for (ConditionalPageMasterReference cpmr : conditionalPageMasterRefs) {
+            if (cpmr.isValid(isOddPage, isFirstPage, false, isBlankPage)) {
+                return cpmr.getMaster();
+            }
+        }
+        throw new PageProductionException("Last page master not found: oddpage=" + isOddPage
+                + " firstpage=" + isFirstPage + " blankpage=" + isBlankPage);
+    }
 
     /**
      * Adds a new conditional page master reference.
@@ -191,4 +206,53 @@ public class RepeatablePageMasterAlternatives extends FObj
         return FO_REPEATABLE_PAGE_MASTER_ALTERNATIVES;
     }
 
+
+
+    /** {@inheritDoc} */
+    public void resolveReferences(LayoutMasterSet layoutMasterSet) throws ValidationException {
+        for (ConditionalPageMasterReference conditionalPageMasterReference
+                : conditionalPageMasterRefs) {
+            conditionalPageMasterReference.resolveReferences(layoutMasterSet);
+        }
+
+    }
+
+    /** {@inheritDoc} */
+    public boolean canProcess(String flowName) {
+
+        boolean willTerminate = true;
+
+
+        //Look for rest spm that cannot terminate
+        ArrayList<ConditionalPageMasterReference> rest
+                = new ArrayList<ConditionalPageMasterReference>();
+        for (ConditionalPageMasterReference cpmr
+                : conditionalPageMasterRefs) {
+            if (cpmr.isValid(true, false, false, false)
+                    || cpmr.isValid(false, false, false, false)) {
+                rest.add(cpmr);
+            }
+        }
+        if (!rest.isEmpty()) {
+            willTerminate = false;
+            for (ConditionalPageMasterReference cpmr : rest) {
+                willTerminate |= cpmr.getMaster().getRegion(FO_REGION_BODY).getRegionName()
+                        .equals(flowName);
+            }
+        }
+
+
+        return willTerminate;
+    }
+
+    /** {@inheritDoc} */
+    public boolean isInfinite() {
+        return getMaximumRepeats() == INFINITE;
+    }
+
+    /** {@inheritDoc} */
+    public boolean isReusable() {
+        return false;
+    }
+
 }
