@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RepeatablePageMasterReference.java 698670 2008-09-24 17:45:51Z adelmelle $ */
+/* $Id: RepeatablePageMasterReference.java 1723297 2016-01-06 12:13:52Z ssteiner $ */
 
 package org.apache.fop.fo.pagination;
 
@@ -28,6 +28,7 @@ import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.ValidationException;
 import org.apache.fop.fo.properties.Property;
+import org.apache.fop.layoutmgr.BlockLevelEventProducer;
 
 /**
  * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_repeatable-page-master-reference">
@@ -40,12 +41,14 @@ public class RepeatablePageMasterReference extends FObj
 
     // The value of properties relevant for fo:repeatable-page-master-reference.
     private String masterReference;
+    // The simple page master referenced
+    private SimplePageMaster master;
     private Property maximumRepeats;
     // End of property values
 
     private static final int INFINITE = -1;
 
-    private int numberConsumed = 0;
+    private int numberConsumed;
 
     /**
      * Base constructor
@@ -67,7 +70,7 @@ public class RepeatablePageMasterReference extends FObj
     }
 
     /** {@inheritDoc} */
-    protected void startOfNode() throws FOPException {
+    public void startOfNode() throws FOPException {
         PageSequenceMaster pageSequenceMaster = (PageSequenceMaster) parent;
 
         if (masterReference == null) {
@@ -87,18 +90,20 @@ public class RepeatablePageMasterReference extends FObj
     }
 
     /** {@inheritDoc} */
-    public String getNextPageMasterName(boolean isOddPage,
+    public SimplePageMaster getNextPageMaster(boolean isOddPage,
                                         boolean isFirstPage,
                                         boolean isLastPage,
                                         boolean isEmptyPage) {
-        if (getMaximumRepeats() != INFINITE) {
-            if (numberConsumed < getMaximumRepeats()) {
-                numberConsumed++;
-            } else {
-                return null;
-            }
+        if (getMaximumRepeats() != INFINITE && numberConsumed >= getMaximumRepeats()) {
+           return null;
         }
-        return masterReference;
+        numberConsumed++;
+        return master;
+    }
+
+    public SimplePageMaster getLastPageMaster(boolean isOddPage, boolean isFirstPage, boolean isEmptyPage,
+                                              BlockLevelEventProducer blockLevelEventProducer) {
+        return getNextPageMaster(isOddPage, isFirstPage, true, isEmptyPage);
     }
 
     /**
@@ -159,4 +164,31 @@ public class RepeatablePageMasterReference extends FObj
     }
 
 
+    /** {@inheritDoc} */
+    public void resolveReferences(LayoutMasterSet layoutMasterSet) throws ValidationException {
+        master = layoutMasterSet.getSimplePageMaster(masterReference);
+        if (master == null) {
+            BlockLevelEventProducer.Provider.get(
+                getUserAgent().getEventBroadcaster())
+                .noMatchingPageMaster(this, parent.getName(), masterReference, getLocator());
+        }
+
+    }
+
+    /** {@inheritDoc} */
+    public boolean canProcess(String flowName) {
+        assert master != null;
+        return master.getRegion(FO_REGION_BODY).getRegionName().equals(flowName);
+    }
+
+    /** {@inheritDoc} */
+    public boolean isInfinite() {
+        return getMaximumRepeats() == INFINITE;
+    }
+
+    /** {@inheritDoc} */
+    public boolean isReusable() {
+        return false;
+    }
+
 }
