@@ -15,12 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: Root.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: Root.java 1723297 2016-01-06 12:13:52Z ssteiner $ */
 
 package org.apache.fop.fo.pagination;
 
 // java
 import java.util.List;
+import java.util.Locale;
 
 import org.xml.sax.Locator;
 
@@ -33,31 +34,36 @@ import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.ValidationException;
 import org.apache.fop.fo.extensions.destination.Destination;
 import org.apache.fop.fo.pagination.bookmarks.BookmarkTree;
+import org.apache.fop.fo.properties.CommonAccessibility;
+import org.apache.fop.fo.properties.CommonAccessibilityHolder;
+import org.apache.fop.fo.properties.CommonHyphenation;
 
 /**
  * Class modeling the <a href="http://www.w3.org/TR/xsl/#fo_root">
  * <code>fo:root</code></a> formatting object.
  * Contains page masters, page-sequences.
  */
-public class Root extends FObj {
-    // The value of properties relevant for fo:root.
+public class Root extends FObj implements CommonAccessibilityHolder {
+
+    private CommonAccessibility commonAccessibility;
+
     private int mediaUsage;
-    // End of property values
 
     private LayoutMasterSet layoutMasterSet;
     private Declarations declarations;
-    private BookmarkTree bookmarkTree = null;
-    private List destinationList;
-    private List pageSequences;
+    private BookmarkTree bookmarkTree;
+    private List<Destination> destinationList;
+    private List<PageSequence> pageSequences;
+    private Locale locale;
 
     // temporary until above list populated
-    private boolean pageSequenceFound = false;
+    private boolean pageSequenceFound;
 
     /**
      * Keeps count of page number from over PageSequence instances
      */
-    private int endingPageNumberOfPreviousSequence = 0;
-    private int totalPagesGenerated = 0;
+    private int endingPageNumberOfPreviousSequence;
+    private int totalPagesGenerated;
 
     /**
      * Context class used while building the FO tree.
@@ -67,7 +73,17 @@ public class Root extends FObj {
     /**
      * FOEventHandler object for this FO Tree
      */
-    private FOEventHandler foEventHandler = null;
+    private FOEventHandler foEventHandler;
+
+    private PageSequence lastSeq;
+
+    public void setLastSeq(PageSequence seq) {
+        lastSeq = seq;
+    }
+
+    public PageSequence getLastSeq() {
+          return lastSeq;
+    }
 
     /**
      * Base constructor
@@ -77,20 +93,31 @@ public class Root extends FObj {
      */
     public Root(FONode parent) {
         super(parent);
-        pageSequences = new java.util.ArrayList();
+        pageSequences = new java.util.ArrayList<PageSequence>();
     }
 
     /** {@inheritDoc} */
     public void bind(PropertyList pList) throws FOPException {
+        super.bind(pList);
+        commonAccessibility = CommonAccessibility.getInstance(pList);
         mediaUsage = pList.get(PR_MEDIA_USAGE).getEnum();
+        String language = pList.get(PR_LANGUAGE).getString();
+        String country = pList.get(PR_COUNTRY).getString();
+        locale = CommonHyphenation.toLocale(language, country);
+    }
+
+     /** {@inheritDoc} */
+    public void startOfNode() throws FOPException {
+        foEventHandler.startRoot(this);
     }
 
     /** {@inheritDoc} */
-    protected void endOfNode() throws FOPException {
+    public void endOfNode() throws FOPException {
         if (!pageSequenceFound || layoutMasterSet == null) {
             missingChildElementError("(layout-master-set, declarations?, "
                 + "bookmark-tree?, (page-sequence|fox:external-document)+)");
         }
+        foEventHandler.endRoot(this);
     }
 
     /**
@@ -144,13 +171,22 @@ public class Root extends FObj {
     }
 
 
-    /** @inheritDoc */
+    /**
+     * @param loc location in the source file
+     * @param child the {@link FONode} to validate against
+     * @throws ValidationException if the incoming node is not a valid child for the given FO
+     */
     protected void validateChildNode(Locator loc, FONode child) throws ValidationException {
         if (child instanceof AbstractPageSequence) {
             pageSequenceFound = true;
         }
     }
 
+    /** {@inheritDoc} */
+    public CommonAccessibility getCommonAccessibility() {
+        return commonAccessibility;
+    }
+
     /**
      * Sets the FOEventHandler object that this Root is attached to
      * @param foEventHandler the FOEventHandler object
@@ -207,7 +243,8 @@ public class Root extends FObj {
      * @param additionalPages the total pages generated by the sequence (for statistics)
      * @throws IllegalArgumentException for negative additional page counts
      */
-    public void notifyPageSequenceFinished(int lastPageNumber, int additionalPages) {
+    public void notifyPageSequenceFinished(int lastPageNumber, int additionalPages)
+            throws IllegalArgumentException {
 
         if (additionalPages >= 0) {
             totalPagesGenerated += additionalPages;
@@ -238,7 +275,7 @@ public class Root extends FObj {
             return null;
         }
         if (currentIndex < (pageSequences.size() - 1)) {
-            return (PageSequence)pageSequences.get(currentIndex + 1);
+            return pageSequences.get(currentIndex + 1);
         } else {
             return null;
         }
@@ -290,7 +327,7 @@ public class Root extends FObj {
      */
     public void addDestination(Destination destination) {
         if (destinationList == null) {
-          destinationList = new java.util.ArrayList();
+          destinationList = new java.util.ArrayList<Destination>();
         }
         destinationList.add(destination);
     }
@@ -329,4 +366,10 @@ public class Root extends FObj {
         return FO_ROOT;
     }
 
+
+    /** @return locale proprty. */
+    public Locale getLocale() {
+        return locale;
+    }
+
 }
