@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: DimensionPropertyMaker.java 1297284 2012-03-05 23:29:29Z gadams $ */
+/* $Id: DimensionPropertyMaker.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.fo.properties;
 
@@ -28,7 +28,7 @@ import org.apache.fop.fo.expr.PropertyException;
  */
 public class DimensionPropertyMaker extends CorrespondingPropertyMaker {
 
-    private int[][] extraCorresponding = null;
+    private int[][] extraCorresponding;
 
     /**
      * Instantiate a dimension property maker.
@@ -43,13 +43,13 @@ public class DimensionPropertyMaker extends CorrespondingPropertyMaker {
      * @param extraCorresponding an array of four element integer arrays
      */
     public void setExtraCorresponding(int[][] extraCorresponding) {
-        if ( extraCorresponding == null ) {
+        if (extraCorresponding == null) {
             throw new NullPointerException();
         }
-        for ( int i = 0; i < extraCorresponding.length; i++ ) {
+        for (int i = 0; i < extraCorresponding.length; i++) {
             int[] eca = extraCorresponding[i];
-            if ( ( eca == null ) || ( eca.length != 4 ) ) {
-                throw new IllegalArgumentException ( "bad sub-array @ [" + i + "]" );
+            if ((eca == null) || (eca.length != 4)) {
+                throw new IllegalArgumentException("bad sub-array @ [" + i + "]");
             }
         }
         this.extraCorresponding = extraCorresponding;
