@@ -15,11 +15,12 @@
  * limitations under the License.
  */
 
-/* $Id: EnumLength.java 824686 2009-10-13 10:52:29Z vhennebert $ */
+/* $Id: EnumLength.java 1303891 2012-03-22 17:04:12Z vhennebert $ */
 
 package org.apache.fop.fo.properties;
 
 import org.apache.fop.datatypes.PercentBaseContext;
+import org.apache.fop.util.CompareUtil;
 
 /**
  * A length quantity in XSL which is specified as an enum, such as "auto"
@@ -27,6 +28,10 @@ import org.apache.fop.datatypes.PercentBaseContext;
 public class EnumLength extends LengthProperty {
     private Property enumProperty;
 
+    /**
+     * Construct an enumerated length from an enum property.
+     * @param enumProperty the enumeration property
+     */
     public EnumLength(Property enumProperty) {
         this.enumProperty = enumProperty;
     }
@@ -38,6 +43,7 @@ public class EnumLength extends LengthProperty {
         return enumProperty.getEnum();
     }
 
+    /** @return true if absolute */
     public boolean isAbsolute() {
         return false;
     }
@@ -88,5 +94,20 @@ public class EnumLength extends LengthProperty {
         return enumProperty.getObject();
     }
 
+    @Override
+    public int hashCode() {
+        return enumProperty.hashCode();
+    }
 
+    @Override
+    public boolean equals(Object obj) {
+        if (this == obj) {
+            return true;
+        }
+        if (!(obj instanceof EnumLength)) {
+            return false;
+        }
+        EnumLength other = (EnumLength) obj;
+        return CompareUtil.equal(enumProperty, other.enumProperty);
+    }
 }
