@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: KeepProperty.java 1303891 2012-03-22 17:04:12Z vhennebert $ */
+/* $Id: KeepProperty.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.fo.properties;
 
@@ -33,7 +33,7 @@ public final class KeepProperty extends Property implements CompoundDatatype {
     private static final PropertyCache<KeepProperty> CACHE
             = new PropertyCache<KeepProperty>();
 
-    private boolean isCachedValue = false;
+    private boolean isCachedValue;
     private Property withinLine;
     private Property withinColumn;
     private Property withinPage;
