@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: NumberProperty.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: NumberProperty.java 1303891 2012-03-22 17:04:12Z vhennebert $ */
 
 package org.apache.fop.fo.properties;
 
@@ -28,6 +28,7 @@ import org.apache.fop.datatypes.PercentBaseContext;
 import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.expr.PropertyException;
+import org.apache.fop.util.CompareUtil;
 
 /**
  * Class for handling numeric properties
@@ -68,6 +69,9 @@ public final class NumberProperty extends Property implements Numeric {
 
     }
 
+    /**
+     * A positive integer property maker.
+     */
     public static class PositiveIntegerMaker extends PropertyMaker {
 
         /**
@@ -103,7 +107,8 @@ public final class NumberProperty extends Property implements Numeric {
     }
 
     /** cache holding all canonical NumberProperty instances */
-    private static final PropertyCache cache = new PropertyCache(NumberProperty.class);
+    private static final PropertyCache<NumberProperty> CACHE
+            = new PropertyCache<NumberProperty>();
 
     private final Number number;
 
@@ -140,8 +145,7 @@ public final class NumberProperty extends Property implements Numeric {
      * @return  the canonical NumberProperty
      */
     public static NumberProperty getInstance(Double num) {
-        return (NumberProperty)cache.fetch(
-                    new NumberProperty(num.doubleValue()));
+        return CACHE.fetch(new NumberProperty(num.doubleValue()));
     }
 
     /**
@@ -151,8 +155,7 @@ public final class NumberProperty extends Property implements Numeric {
      * @return  the canonical NumberProperty
      */
     public static NumberProperty getInstance(Integer num) {
-        return (NumberProperty)cache.fetch(
-                    new NumberProperty(num.intValue()));
+        return CACHE.fetch(new NumberProperty(num.intValue()));
     }
 
     /**
@@ -162,8 +165,7 @@ public final class NumberProperty extends Property implements Numeric {
      * @return  the canonical NumberProperty
      */
     public static NumberProperty getInstance(double num) {
-        return (NumberProperty)cache.fetch(
-                    new NumberProperty(num));
+        return CACHE.fetch(new NumberProperty(num));
     }
 
     /**
@@ -173,8 +175,7 @@ public final class NumberProperty extends Property implements Numeric {
      * @return  the canonical NumberProperty
      */
     public static NumberProperty getInstance(int num) {
-        return (NumberProperty)cache.fetch(
-                    new NumberProperty(num));
+        return CACHE.fetch(new NumberProperty(num));
     }
 
     /**
@@ -265,21 +266,21 @@ public final class NumberProperty extends Property implements Numeric {
     }
 
     /** {@inheritDoc} */
-    public int hashCode() {
-        return number.hashCode();
+    @Override
+    public boolean equals(Object obj) {
+        if (this == obj) {
+            return true;
+        }
+        if (!(obj instanceof NumberProperty)) {
+            return false;
+        }
+        NumberProperty other = (NumberProperty) obj;
+        return CompareUtil.equal(number, other.number);
     }
 
     /** {@inheritDoc} */
-    public boolean equals(Object o) {
-        if (o == this) {
-            return true;
-        }
-        if (o instanceof NumberProperty) {
-            NumberProperty np = (NumberProperty) o;
-            return (np.number == this.number
-                    || (this.number != null
-                        && this.number.equals(np.number)));
-        }
-        return false;
+    @Override
+    public int hashCode() {
+        return number.hashCode();
     }
 }
