@@ -15,16 +15,22 @@
  * limitations under the License.
  */
 
-/* $Id: CustomFont.java 721430 2008-11-28 11:13:12Z acumiskey $ */
+/* $Id: CustomFont.java 1695082 2015-08-10 14:15:48Z rmeyer $ */
 
 package org.apache.fop.fonts;
 
 import java.io.IOException;
+import java.io.InputStream;
+import java.net.URI;
+import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import javax.xml.transform.Source;
+import org.apache.fop.apps.io.InternalResourceResolver;
 
 
 /**
@@ -33,31 +39,60 @@ import javax.xml.transform.Source;
 public abstract class CustomFont extends Typeface
             implements FontDescriptor, MutableFont {
 
-    private String fontName = null;
-    private String fullName = null;
-    private Set familyNames = null; //Set<String>
-    private String fontSubName = null;
-    private String embedFileName = null;
-    private String embedResourceName = null;
-    private FontResolver resolver = null;
+    /** Fallback thickness for underline and strikeout when not provided by the font. */
+    private static final int DEFAULT_LINE_THICKNESS = 50;
 
-    private int capHeight = 0;
-    private int xHeight = 0;
-    private int ascender = 0;
-    private int descender = 0;
+    private URI fontFileURI;
+    private String fontName;
+    private String fullName;
+    private Set<String> familyNames;
+    private String fontSubName;
+    private URI embedFileURI;
+    private String embedResourceName;
+    private final InternalResourceResolver resourceResolver;
+    private EmbeddingMode embeddingMode = EmbeddingMode.AUTO;
+
+    private int capHeight;
+    private int xHeight;
+    private int ascender;
+    private int descender;
     private int[] fontBBox = {0, 0, 0, 0};
     private int flags = 4;
-    private int weight = 0; //0 means unknown weight
-    private int stemV = 0;
-    private int italicAngle = 0;
-    private int missingWidth = 0;
+    private int weight; //0 means unknown weight
+    private int stemV;
+    private int italicAngle;
+    private int missingWidth;
     private FontType fontType = FontType.TYPE1;
-    private int firstChar = 0;
+    private int firstChar;
     private int lastChar = 255;
 
-    private Map kerning;
+    private int underlinePosition;
+
+    private int underlineThickness;
+
+    private int strikeoutPosition;
+
+    private int strikeoutThickness;
+
+    private Map<Integer, Map<Integer, Integer>> kerning;
 
     private boolean useKerning = true;
+    /** the character map, mapping Unicode ranges to glyph indices. */
+    protected List<CMapSegment> cmap = new ArrayList<CMapSegment>();
+    private boolean useAdvanced = true;
+
+    /**
+     * @param resourceResolver the URI resource resolver for controlling file access
+     */
+    public CustomFont(InternalResourceResolver resourceResolver) {
+        this.resourceResolver = resourceResolver;
+    }
+
+
+    /** {@inheritDoc} */
+    public URI getFontURI() {
+        return fontFileURI;
+    }
 
     /** {@inheritDoc} */
     public String getFontName() {
@@ -78,7 +113,7 @@ public abstract class CustomFont extends Typeface
      * Returns the font family names.
      * @return the font family names (a Set of Strings)
      */
-    public Set getFamilyNames() {
+    public Set<String> getFamilyNames() {
         return Collections.unmodifiableSet(this.familyNames);
     }
 
@@ -100,29 +135,31 @@ public abstract class CustomFont extends Typeface
     }
 
     /**
-     * Returns an URI representing an embeddable font file. The URI will often
-     * be a filename or an URL.
+     * Returns an URI representing an embeddable font file.
+     *
      * @return URI to an embeddable font file or null if not available.
      */
-    public String getEmbedFileName() {
-        return embedFileName;
+    public URI getEmbedFileURI() {
+        return embedFileURI;
     }
 
     /**
-     * Returns a Source representing an embeddable font file.
-     * @return Source for an embeddable font file
+
+     * Returns the embedding mode for this font.
+     * @return embedding mode
+     */
+    public EmbeddingMode getEmbeddingMode() {
+        return embeddingMode;
+    }
+
+    /**
+     * Returns an {@link InputStream} representing an embeddable font file.
+     *
+     * @return {@link InputStream} for an embeddable font file
      * @throws IOException if embedFileName is not null but Source is not found
      */
-    public Source getEmbedFileSource() throws IOException {
-        Source result = null;
-        if (resolver != null && embedFileName != null) {
-            result = resolver.resolve(embedFileName);
-            if (result == null) {
-                throw new IOException("Unable to resolve Source '"
-                        + embedFileName + "' for embedded font");
-            }
-        }
-        return result;
+    public InputStream getInputStream() throws IOException {
+        return resourceResolver.getResource(embedFileURI);
     }
 
     /**
@@ -275,16 +312,30 @@ public abstract class CustomFont extends Typeface
     /**
      * {@inheritDoc}
      */
-    public final Map getKerningInfo() {
+    public final Map<Integer, Map<Integer, Integer>> getKerningInfo() {
         if (hasKerningInfo()) {
             return kerning;
         } else {
-            return java.util.Collections.EMPTY_MAP;
+            return Collections.emptyMap();
         }
     }
 
+    /**
+     * Used to determine if advanced typographic features are enabled.
+     * By default, this is false, but may be overridden by subclasses.
+     * @return true if enabled.
+     */
+    public boolean isAdvancedEnabled() {
+        return useAdvanced;
+    }
+
     /* ---- MutableFont interface ---- */
 
+    /** {@inheritDoc} */
+    public void setFontURI(URI uri) {
+        this.fontFileURI = uri;
+    }
+
     /** {@inheritDoc} */
     public void setFontName(String name) {
         this.fontName = name;
@@ -296,8 +347,8 @@ public abstract class CustomFont extends Typeface
     }
 
     /** {@inheritDoc} */
-    public void setFamilyNames(Set names) {
-        this.familyNames = new java.util.HashSet(names);
+    public void setFamilyNames(Set<String> names) {
+        this.familyNames = new HashSet<String>(names);
     }
 
     /**
@@ -311,8 +362,8 @@ public abstract class CustomFont extends Typeface
     /**
      * {@inheritDoc}
      */
-    public void setEmbedFileName(String path) {
-        this.embedFileName = path;
+    public void setEmbedURI(URI path) {
+        this.embedFileURI = path;
     }
 
     /**
@@ -322,6 +373,13 @@ public abstract class CustomFont extends Typeface
         this.embedResourceName = name;
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    public void setEmbeddingMode(EmbeddingMode embeddingMode) {
+        this.embeddingMode = embeddingMode;
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -426,17 +484,16 @@ public abstract class CustomFont extends Typeface
     }
 
     /**
-     * Sets the font resolver. Needed for URI resolution.
-     * @param resolver the font resolver
+     * {@inheritDoc}
      */
-    public void setResolver(FontResolver resolver) {
-        this.resolver = resolver;
+    public void setAdvancedEnabled(boolean enabled) {
+        this.useAdvanced = enabled;
     }
 
     /** {@inheritDoc} */
-    public void putKerningEntry(Integer key, Map value) {
+    public void putKerningEntry(Integer key, Map<Integer, Integer> value) {
         if (kerning == null) {
-            kerning = new java.util.HashMap();
+            kerning = new HashMap<Integer, Map<Integer, Integer>>();
         }
         this.kerning.put(key, value);
     }
@@ -446,12 +503,81 @@ public abstract class CustomFont extends Typeface
      * @param kerningMap the kerning map (Map<Integer, Map<Integer, Integer>, the integers are
      *                          character codes)
      */
-    public void replaceKerningMap(Map kerningMap) {
+    public void replaceKerningMap(Map<Integer, Map<Integer, Integer>> kerningMap) {
         if (kerningMap == null) {
-            this.kerning = Collections.EMPTY_MAP;
+            this.kerning = Collections.emptyMap();
         } else {
             this.kerning = kerningMap;
         }
     }
 
+    /**
+     * Sets the character map for this font. It maps all available Unicode characters
+     * to their glyph indices inside the font.
+     * @param cmap the character map
+     */
+    public void setCMap(CMapSegment[] cmap) {
+        this.cmap.clear();
+        for (CMapSegment c : cmap) {
+            this.cmap.add(c);
+        }
+    }
+
+    /**
+     * Returns the character map for this font. It maps all available Unicode characters
+     * to their glyph indices inside the font.
+     * @return the character map
+     */
+    public CMapSegment[] getCMap() {
+        return cmap.toArray(new CMapSegment[cmap.size()]);
+    }
+
+    public int getUnderlinePosition(int size) {
+        return (underlinePosition == 0)
+                ? getDescender(size) / 2
+                : size * underlinePosition;
+    }
+
+    public void setUnderlinePosition(int underlinePosition) {
+        this.underlinePosition = underlinePosition;
+    }
+
+    public int getUnderlineThickness(int size) {
+        return size * ((underlineThickness == 0) ? DEFAULT_LINE_THICKNESS : underlineThickness);
+    }
+
+    public void setUnderlineThickness(int underlineThickness) {
+        this.underlineThickness = underlineThickness;
+    }
+
+    public int getStrikeoutPosition(int size) {
+        return (strikeoutPosition == 0)
+                ? getXHeight(size) / 2
+                : size * strikeoutPosition;
+    }
+
+    public void setStrikeoutPosition(int strikeoutPosition) {
+        this.strikeoutPosition = strikeoutPosition;
+    }
+
+    public int getStrikeoutThickness(int size) {
+        return (strikeoutThickness == 0) ? getUnderlineThickness(size) : size * strikeoutThickness;
+    }
+
+    public void setStrikeoutThickness(int strikeoutThickness) {
+        this.strikeoutThickness = strikeoutThickness;
+    }
+
+    /**
+     * Returns a Map of used Glyphs.
+     * @return Map Map of used Glyphs
+     */
+    public abstract Map<Integer, Integer> getUsedGlyphs();
+
+    /**
+     * Returns the character from it's original glyph index in the font
+     * @param glyphIndex The original index of the character
+     * @return The character
+     */
+    public abstract char getUnicodeFromGID(int glyphIndex);
 }
