@@ -15,13 +15,12 @@
  * limitations under the License.
  */
 
-/* $Id: FontManagerConfigurator.java 1296526 2012-03-03 00:18:45Z gadams $ */
+/* $Id: FontManagerConfigurator.java 1542190 2013-11-15 07:13:53Z vhennebert $ */
 
 package org.apache.fop.fonts;
 
-import java.io.File;
-import java.net.MalformedURLException;
 import java.net.URI;
+import java.net.URISyntaxException;
 import java.util.List;
 import java.util.regex.Pattern;
 
@@ -30,7 +29,11 @@ import org.apache.avalon.framework.configuration.ConfigurationException;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.xmlgraphics.io.ResourceResolver;
+
 import org.apache.fop.apps.FOPException;
+import org.apache.fop.apps.io.InternalResourceResolver;
+import org.apache.fop.apps.io.ResourceResolverFactory;
 import org.apache.fop.fonts.substitute.FontSubstitutions;
 import org.apache.fop.fonts.substitute.FontSubstitutionsConfigurator;
 import org.apache.fop.util.LogUtil;
@@ -45,24 +48,25 @@ public class FontManagerConfigurator {
 
     private final Configuration cfg;
 
-    private URI baseURI = null;
+    private final URI baseURI;
+
+    private final URI fallbackURI;
+
+    private final ResourceResolver resourceResolver;
 
     /**
      * Main constructor
      * @param cfg the font manager configuration object
+     * @param baseURI the URI against which to resolve relative URIs
+     * @param fallbackURI the URI to use as a fallback if font-base is unspecified
+     * @param resourceResolver the resource resolver
      */
-    public FontManagerConfigurator(Configuration cfg) {
-        this.cfg = cfg;
-    }
-
-    /**
-     * Main constructor
-     * @param cfg the font manager configuration object
-     * @param baseURI the base URI of the configuration
-     */
-    public FontManagerConfigurator(Configuration cfg, URI baseURI) {
+    public FontManagerConfigurator(Configuration cfg, URI baseURI, URI fallbackURI,
+            ResourceResolver resourceResolver) {
         this.cfg = cfg;
         this.baseURI = baseURI;
+        this.fallbackURI = fallbackURI;
+        this.resourceResolver = resourceResolver;
     }
 
     /**
@@ -72,33 +76,34 @@ public class FontManagerConfigurator {
      * @throws FOPException if an exception occurs while processing the configuration
      */
     public void configure(FontManager fontManager, boolean strict) throws FOPException {
+        if (cfg.getChild("font-base", false) != null) {
+            try {
+                URI fontBase = InternalResourceResolver.getBaseURI(cfg.getChild("font-base")
+                                                                      .getValue(null));
+                fontManager.setResourceResolver(ResourceResolverFactory.createInternalResourceResolver(
+                        baseURI.resolve(fontBase), resourceResolver));
+            } catch (URISyntaxException use) {
+                LogUtil.handleException(log, use, true);
+            }
+        } else {
+            fontManager.setResourceResolver(ResourceResolverFactory.createInternalResourceResolver(
+                    fallbackURI, resourceResolver));
+        }
         // caching (fonts)
         if (cfg.getChild("use-cache", false) != null) {
             try {
-                fontManager.setUseCache(cfg.getChild("use-cache").getValueAsBoolean());
-            } catch (ConfigurationException e) {
-                LogUtil.handleException(log, e, true);
-            }
-        }
-        if (cfg.getChild("cache-file", false) != null) {
-            try {
-                fontManager.setCacheFile(new File(cfg.getChild("cache-file").getValue()));
-            } catch (ConfigurationException e) {
-                LogUtil.handleException(log, e, true);
-            }
-        }
-        if (cfg.getChild("font-base", false) != null) {
-            String path = cfg.getChild("font-base").getValue(null);
-            if (baseURI != null) {
-                path = baseURI.resolve(path).normalize().toString();
-            }
-            try {
-                fontManager.setFontBaseURL(path);
-            } catch (MalformedURLException mfue) {
+                if (!cfg.getChild("use-cache").getValueAsBoolean()) {
+                    fontManager.disableFontCache();
+                } else {
+                    if (cfg.getChild("cache-file", false) != null) {
+
+                        fontManager.setCacheFile(URI.create(cfg.getChild("cache-file").getValue()));
+                    }
+                }
+            } catch (ConfigurationException mfue) {
                 LogUtil.handleException(log, mfue, true);
             }
         }
-
         // [GA] permit configuration control over base14 kerning; without this,
         // there is no way for a user to enable base14 kerning other than by
         // programmatic API;
@@ -114,7 +119,6 @@ public class FontManagerConfigurator {
         // global font configuration
         Configuration fontsCfg = cfg.getChild("fonts", false);
         if (fontsCfg != null) {
-
             // font substitution
             Configuration substitutionsCfg = fontsCfg.getChild("substitutions", false);
             if (substitutionsCfg != null) {
@@ -122,7 +126,6 @@ public class FontManagerConfigurator {
                 new FontSubstitutionsConfigurator(substitutionsCfg).configure(substitutions);
                 fontManager.setFontSubstitutions(substitutions);
             }
-
             // referenced fonts (fonts which are not to be embedded)
             Configuration referencedFontsCfg = fontsCfg.getChild("referenced-fonts", false);
             if (referencedFontsCfg != null) {
@@ -130,7 +133,6 @@ public class FontManagerConfigurator {
                         referencedFontsCfg, strict);
                 fontManager.setReferencedFontsMatcher(matcher);
             }
-
         }
     }
 
@@ -159,6 +161,24 @@ public class FontManagerConfigurator {
         return orMatcher;
     }
 
+    /**
+     * Creates a font triplet matcher from a configuration object.
+     * @param fontFamilies the list of font families
+     * @param strict true for strict configuraton error handling
+     * @return the font matcher
+     * @throws FOPException if an error occurs while building the matcher
+     */
+    public static FontTriplet.Matcher createFontsMatcher(
+            List<String> fontFamilies, boolean strict) throws FOPException {
+        List<FontTriplet.Matcher> matcherList = new java.util.ArrayList<FontTriplet.Matcher>();
+        for (String fontFamily : fontFamilies) {
+                matcherList.add(new FontFamilyRegExFontTripletMatcher(fontFamily));
+        }
+        FontTriplet.Matcher orMatcher = new OrFontTripletMatcher(
+                matcherList.toArray(new FontTriplet.Matcher[matcherList.size()]));
+        return orMatcher;
+    }
+
     private static class OrFontTripletMatcher implements FontTriplet.Matcher {
 
         private final FontTriplet.Matcher[] matchers;
