@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FontUtil.java 985537 2010-08-14 17:17:00Z jeremias $ */
+/* $Id: FontUtil.java 1502902 2013-07-13 22:26:09Z lbernardo $ */
 
 package org.apache.fop.fonts;
 
@@ -40,7 +40,7 @@ public final class FontUtil {
         int weight = 400;
         try {
             weight = Integer.parseInt(text);
-            weight = ((int)weight / 100) * 100;
+            weight = (weight / 100) * 100;
             weight = Math.max(weight, 100);
             weight = Math.min(weight, 900);
         } catch (NumberFormatException nfe) {
