@@ -15,24 +15,24 @@
  * limitations under the License.
  */
 
-/* $Id: LazyFont.java 1357883 2012-07-05 20:29:53Z gadams $ */
+/* $Id: LazyFont.java 1641827 2014-11-26 15:00:57Z lbernardo $ */
 
 package org.apache.fop.fonts;
+import java.awt.Rectangle;
 import java.io.IOException;
 import java.io.InputStream;
-import java.net.URL;
+import java.net.URI;
+import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import javax.xml.transform.Source;
-import javax.xml.transform.stream.StreamSource;
-
 import org.xml.sax.InputSource;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
 import org.apache.fop.apps.FOPException;
+import org.apache.fop.apps.io.InternalResourceResolver;
 import org.apache.fop.complexscripts.fonts.Positionable;
 import org.apache.fop.complexscripts.fonts.Substitutable;
 
@@ -43,49 +43,50 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
 
     private static Log log = LogFactory.getLog(LazyFont.class);
 
-    private String metricsFileName;
-    private String fontEmbedPath;
-    private boolean useKerning;
-    private boolean useAdvanced;
-    private EncodingMode encodingMode = EncodingMode.AUTO;
-    private EmbeddingMode embeddingMode = EmbeddingMode.AUTO;
-    private boolean embedded = true;
-    private String subFontName;
+    private final FontUris fontUris;
+
+    private final boolean useKerning;
+    private final boolean useAdvanced;
+    private final EncodingMode encodingMode;
+    private final EmbeddingMode embeddingMode;
+    private final String subFontName;
+    private final boolean embedded;
+    private final InternalResourceResolver resourceResolver;
 
     private boolean isMetricsLoaded;
     private Typeface realFont;
     private FontDescriptor realFontDescriptor;
 
-    private FontResolver resolver;
-
     /**
      * Main constructor
      * @param fontInfo  the font info to embed
-     * @param resolver the font resolver to handle font URIs
+     * @param resourceResolver the font resolver to handle font URIs
      */
-    public LazyFont(EmbedFontInfo fontInfo, FontResolver resolver) {
+    public LazyFont(EmbedFontInfo fontInfo, InternalResourceResolver resourceResolver,
+            boolean useComplexScripts) {
 
-        this.metricsFileName = fontInfo.getMetricsFile();
-        this.fontEmbedPath = fontInfo.getEmbedFile();
+        this.fontUris = fontInfo.getFontUris();
         this.useKerning = fontInfo.getKerning();
-        if ( resolver != null ) {
-            this.useAdvanced = resolver.isComplexScriptFeaturesEnabled();
+        if (resourceResolver != null) {
+            this.useAdvanced = useComplexScripts;
         } else {
             this.useAdvanced = fontInfo.getAdvanced();
         }
-        this.encodingMode = fontInfo.getEncodingMode();
-        this.embeddingMode = fontInfo.getEmbeddingMode();
+        this.encodingMode = fontInfo.getEncodingMode() != null ? fontInfo.getEncodingMode()
+                : EncodingMode.AUTO;
+        this.embeddingMode = fontInfo.getEmbeddingMode() != null ? fontInfo.getEmbeddingMode()
+                : EmbeddingMode.AUTO;
         this.subFontName = fontInfo.getSubFontName();
         this.embedded = fontInfo.isEmbedded();
-        this.resolver = resolver;
+        this.resourceResolver = resourceResolver;
     }
 
     /** {@inheritDoc} */
     public String toString() {
         StringBuffer sbuf = new StringBuffer(super.toString());
         sbuf.append('{');
-        sbuf.append("metrics-url=" + metricsFileName);
-        sbuf.append(",embed-url=" + fontEmbedPath);
+        sbuf.append("metrics-url=" + fontUris.getMetrics());
+        sbuf.append(",embed-url=" + fontUris.getEmbed());
         sbuf.append(",kerning=" + useKerning);
         sbuf.append(",advanced=" + useAdvanced);
         sbuf.append('}');
@@ -95,75 +96,38 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
     private void load(boolean fail) {
         if (!isMetricsLoaded) {
             try {
-                if (metricsFileName != null) {
+                if (fontUris.getMetrics() != null) {
                     /**@todo Possible thread problem here */
                     FontReader reader = null;
-                    if (resolver != null) {
-                        Source source = resolver.resolve(metricsFileName);
-                        if (source == null) {
-                            String err
-                                = "Cannot load font: failed to create Source from metrics file "
-                                    + metricsFileName;
-                            if (fail) {
-                                throw new RuntimeException(err);
-                            } else {
-                                log.error(err);
-                            }
-                            return;
-                        }
-                        InputStream in = null;
-                        if (source instanceof StreamSource) {
-                            in = ((StreamSource) source).getInputStream();
-                        }
-                        if (in == null && source.getSystemId() != null) {
-                            in = new java.net.URL(source.getSystemId()).openStream();
-                        }
-                        if (in == null) {
-                            String err = "Cannot load font: After URI resolution, the returned"
-                                + " Source object does not contain an InputStream"
-                                + " or a valid URL (system identifier) for metrics file: "
-                                + metricsFileName;
-                            if (fail) {
-                                throw new RuntimeException(err);
-                            } else {
-                                log.error(err);
-                            }
-                            return;
-                        }
-                        InputSource src = new InputSource(in);
-                        src.setSystemId(source.getSystemId());
-                        reader = new FontReader(src);
-                    } else {
-                        reader = new FontReader(new InputSource(
-                                    new URL(metricsFileName).openStream()));
-                    }
+                    InputStream in = resourceResolver.getResource(fontUris.getMetrics());
+                    InputSource src = new InputSource(in);
+                    src.setSystemId(fontUris.getMetrics().toASCIIString());
+                    reader = new FontReader(src, resourceResolver);
                     reader.setKerningEnabled(useKerning);
                     reader.setAdvancedEnabled(useAdvanced);
                     if (this.embedded) {
-                        reader.setFontEmbedPath(fontEmbedPath);
+                        reader.setFontEmbedURI(fontUris.getEmbed());
                     }
-                    reader.setResolver(resolver);
                     realFont = reader.getFont();
                 } else {
-                    if (fontEmbedPath == null) {
+                    if (fontUris.getEmbed() == null) {
                         throw new RuntimeException("Cannot load font. No font URIs available.");
                     }
-                    realFont = FontLoader.loadFont(fontEmbedPath, subFontName,
-                            embedded, embeddingMode, encodingMode,
-                            useKerning, useAdvanced, resolver);
+                    realFont = FontLoader.loadFont(fontUris, subFontName, embedded,
+                            embeddingMode, encodingMode, useKerning, useAdvanced, resourceResolver);
                 }
                 if (realFont instanceof FontDescriptor) {
                     realFontDescriptor = (FontDescriptor) realFont;
                 }
             } catch (FOPException fopex) {
-                log.error("Failed to read font metrics file " + metricsFileName, fopex);
+                log.error("Failed to read font metrics file " + fontUris.getMetrics(), fopex);
                 if (fail) {
-                    throw new RuntimeException(fopex.getMessage());
+                    throw new RuntimeException(fopex);
                 }
             } catch (IOException ioex) {
-                log.error("Failed to read font metrics file " + metricsFileName, ioex);
+                log.error("Failed to read font metrics file " + fontUris.getMetrics(), ioex);
                 if (fail) {
-                    throw new RuntimeException(ioex.getMessage());
+                    throw new RuntimeException(ioex);
                 }
             }
             realFont.setEventListener(this.eventListener);
@@ -191,7 +155,9 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
      * {@inheritDoc}
      */
     public char mapChar(char c) {
-        load(true);
+        if (!isMetricsLoaded) {
+            load(true);
+        }
         return realFont.mapChar(c);
     }
 
@@ -207,7 +173,9 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
      * {@inheritDoc}
      */
     public boolean hasChar(char c) {
-        load(true);
+        if (!isMetricsLoaded) {
+            load(true);
+        }
         return realFont.hasChar(c);
     }
 
@@ -220,6 +188,12 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
     }
 
     // ---- FontMetrics interface ----
+    /** {@inheritDoc} */
+    public URI getFontURI() {
+        load(true);
+        return realFont.getFontURI();
+    }
+
     /** {@inheritDoc} */
     public String getFontName() {
         load(true);
@@ -284,11 +258,33 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
         return realFont.getXHeight(size);
     }
 
+    public int getUnderlinePosition(int size) {
+        load(true);
+        return realFont.getUnderlinePosition(size);
+    }
+
+    public int getUnderlineThickness(int size) {
+        load(true);
+        return realFont.getUnderlineThickness(size);
+    }
+
+    public int getStrikeoutPosition(int size) {
+        load(true);
+        return realFont.getStrikeoutPosition(size);
+    }
+
+    public int getStrikeoutThickness(int size) {
+        load(true);
+        return realFont.getStrikeoutThickness(size);
+    }
+
     /**
      * {@inheritDoc}
      */
     public int getWidth(int i, int size) {
-        load(true);
+        if (!isMetricsLoaded) {
+            load(true);
+        }
         return realFont.getWidth(i, size);
     }
 
@@ -300,6 +296,11 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
         return realFont.getWidths();
     }
 
+    public Rectangle getBoundingBox(int glyphIndex, int size) {
+        load(true);
+        return realFont.getBoundingBox(glyphIndex, size);
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -316,6 +317,12 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
         return realFont.getKerningInfo();
     }
 
+    /** {@inheritDoc} */
+    public boolean hasFeature(int tableType, String script, String language, String feature) {
+        load(true);
+        return realFont.hasFeature(tableType, script, language, feature);
+    }
+
     // ---- FontDescriptor interface ----
     /**
      * {@inheritDoc}
@@ -398,7 +405,7 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
      */
     public boolean performsSubstitution() {
         load(true);
-        if ( realFontDescriptor instanceof Substitutable ) {
+        if (realFontDescriptor instanceof Substitutable) {
             return ((Substitutable)realFontDescriptor).performsSubstitution();
         } else {
             return false;
@@ -408,10 +415,12 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
     /**
      * {@inheritDoc}
      */
-    public CharSequence performSubstitution ( CharSequence cs, String script, String language ) {
+    public CharSequence performSubstitution(CharSequence cs, String script, String language, List associations,
+                                            boolean retainControls) {
         load(true);
-        if ( realFontDescriptor instanceof Substitutable ) {
-            return ((Substitutable)realFontDescriptor).performSubstitution(cs, script, language);
+        if (realFontDescriptor instanceof Substitutable) {
+            return ((Substitutable)realFontDescriptor).performSubstitution(cs,
+                script, language, associations, retainControls);
         } else {
             return cs;
         }
@@ -420,12 +429,14 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
     /**
      * {@inheritDoc}
      */
-    public CharSequence reorderCombiningMarks
-        ( CharSequence cs, int[][] gpa, String script, String language ) {
-        load(true);
-        if ( realFontDescriptor instanceof Substitutable ) {
+    public CharSequence reorderCombiningMarks(
+        CharSequence cs, int[][] gpa, String script, String language, List associations) {
+        if (!isMetricsLoaded) {
+            load(true);
+        }
+        if (realFontDescriptor instanceof Substitutable) {
             return ((Substitutable)realFontDescriptor)
-                .reorderCombiningMarks(cs, gpa, script, language);
+                .reorderCombiningMarks(cs, gpa, script, language, associations);
         } else {
             return cs;
         }
@@ -435,8 +446,10 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
      * {@inheritDoc}
      */
     public boolean performsPositioning() {
-        load(true);
-        if ( realFontDescriptor instanceof Positionable ) {
+        if (!isMetricsLoaded) {
+            load(true);
+        }
+        if (realFontDescriptor instanceof Positionable) {
             return ((Positionable)realFontDescriptor).performsPositioning();
         } else {
             return false;
@@ -447,9 +460,11 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
      * {@inheritDoc}
      */
     public int[][]
-        performPositioning ( CharSequence cs, String script, String language, int fontSize ) {
-        load(true);
-        if ( realFontDescriptor instanceof Positionable ) {
+        performPositioning(CharSequence cs, String script, String language, int fontSize) {
+        if (!isMetricsLoaded) {
+            load(true);
+        }
+        if (realFontDescriptor instanceof Positionable) {
             return ((Positionable)realFontDescriptor)
                 .performPositioning(cs, script, language, fontSize);
         } else {
@@ -461,9 +476,11 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
      * {@inheritDoc}
      */
     public int[][]
-        performPositioning ( CharSequence cs, String script, String language ) {
-        load(true);
-        if ( realFontDescriptor instanceof Positionable ) {
+        performPositioning(CharSequence cs, String script, String language) {
+        if (!isMetricsLoaded) {
+            load(true);
+        }
+        if (realFontDescriptor instanceof Positionable) {
             return ((Positionable)realFontDescriptor)
                 .performPositioning(cs, script, language);
         } else {
@@ -476,8 +493,10 @@ public class LazyFont extends Typeface implements FontDescriptor, Substitutable,
      */
     public boolean isSubsetEmbedded() {
         load(true);
+        if (realFont.isMultiByte() && this.embeddingMode == EmbeddingMode.FULL) {
+            return false;
+        }
         return realFont.isMultiByte();
     }
-
 }
 
