@@ -15,18 +15,29 @@
  * limitations under the License.
  */
 
-/* $Id: SingleByteFont.java 777459 2009-05-22 10:38:49Z vhennebert $ */
+/* $Id: SingleByteFont.java 1695082 2015-08-10 14:15:48Z rmeyer $ */
 
 package org.apache.fop.fonts;
 
-import java.util.Iterator;
+import java.awt.Rectangle;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.TreeSet;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.xmlgraphics.fonts.Glyphs;
+
+import org.apache.fop.apps.io.InternalResourceResolver;
+import org.apache.fop.fonts.truetype.OpenFont.PostScriptVersion;
+import org.apache.fop.util.CharUtilities;
+
 /**
  * Generic SingleByte font
  */
@@ -35,29 +46,56 @@ public class SingleByteFont extends CustomFont {
     /** logger */
     private  static Log log = LogFactory.getLog(SingleByteFont.class);
 
-    private SingleByteEncoding mapping;
-    private boolean useNativeEncoding = false;
+    protected SingleByteEncoding mapping;
+    private boolean useNativeEncoding;
 
-    private int[] width = null;
+    protected int[] width;
 
-    private Map unencodedCharacters;
-    //Map<Character, UnencodedCharacter>
-    private List additionalEncodings;
+    private Rectangle[] boundingBoxes;
 
+    private Map<Character, UnencodedCharacter> unencodedCharacters;
+    private List<SimpleSingleByteEncoding> additionalEncodings;
+    private Map<Character, Character> alternativeCodes;
 
-    /**
-     * Main constructor.
-     */
-    public SingleByteFont() {
+    private PostScriptVersion ttPostScriptVersion;
+
+    private int usedGlyphsCount;
+    private LinkedHashMap<Integer, String> usedGlyphNames;
+    private Map<Integer, Integer> usedGlyphs;
+    private Map<Integer, Character> usedCharsIndex;
+    private Map<Character, Integer> charGIDMappings;
+
+    public SingleByteFont(InternalResourceResolver resourceResolver) {
+        super(resourceResolver);
         setEncoding(CodePointMapping.WIN_ANSI_ENCODING);
     }
 
+    public SingleByteFont(InternalResourceResolver resourceResolver, EmbeddingMode embeddingMode) {
+        this(resourceResolver);
+        setEmbeddingMode(embeddingMode);
+        if (embeddingMode != EmbeddingMode.FULL) {
+            usedGlyphNames = new LinkedHashMap<Integer, String>();
+            usedGlyphs = new HashMap<Integer, Integer>();
+            usedCharsIndex = new HashMap<Integer, Character>();
+            charGIDMappings = new HashMap<Character, Integer>();
+
+            // The zeroth value is reserved for .notdef
+            usedGlyphs.put(0, 0);
+            usedGlyphsCount++;
+        }
+    }
+
     /** {@inheritDoc} */
     public boolean isEmbeddable() {
-        return (!(getEmbedFileName() == null
+        return (!(getEmbedFileURI() == null
                 && getEmbedResourceName() == null));
     }
 
+    /** {@inheritDoc} */
+    public boolean isSubsetEmbedded() {
+        return getEmbeddingMode() != EmbeddingMode.FULL;
+    }
+
     /** {@inheritDoc} */
     public String getEncodingName() {
         return this.mapping.getName();
@@ -76,7 +114,7 @@ public class SingleByteFont extends CustomFont {
         if (i < 256) {
             int idx = i - getFirstChar();
             if (idx >= 0 && idx < width.length) {
-                return size * width[i - getFirstChar()];
+                return size * width[idx];
             }
         } else if (this.additionalEncodings != null) {
             int encodingIndex = (i / 256) - 1;
@@ -84,8 +122,7 @@ public class SingleByteFont extends CustomFont {
             int codePoint = i % 256;
             NamedCharacter nc = encoding.getCharacterForIndex(codePoint);
             UnencodedCharacter uc
-                = (UnencodedCharacter)this.unencodedCharacters.get(
-                        new Character(nc.getSingleUnicodeValue()));
+                = this.unencodedCharacters.get(Character.valueOf(nc.getSingleUnicodeValue()));
             return size * uc.getWidth();
         }
         return 0;
@@ -98,30 +135,130 @@ public class SingleByteFont extends CustomFont {
         return arr;
     }
 
-    /** {@inheritDoc} */
-    public char mapChar(char c) {
-        notifyMapOperation();
+    public Rectangle getBoundingBox(int glyphIndex, int size) {
+        Rectangle bbox = null;
+        if (glyphIndex < 256) {
+            int idx = glyphIndex - getFirstChar();
+            if (idx >= 0 && idx < boundingBoxes.length) {
+                bbox =  boundingBoxes[idx];
+            }
+        } else if (this.additionalEncodings != null) {
+            int encodingIndex = (glyphIndex / 256) - 1;
+            SimpleSingleByteEncoding encoding = getAdditionalEncoding(encodingIndex);
+            int codePoint = glyphIndex % 256;
+            NamedCharacter nc = encoding.getCharacterForIndex(codePoint);
+            UnencodedCharacter uc = this.unencodedCharacters.get(Character.valueOf(nc.getSingleUnicodeValue()));
+            bbox = uc.getBBox();
+        }
+        return bbox == null ? null : new Rectangle(bbox.x * size, bbox.y * size, bbox.width * size, bbox.height * size);
+    }
+
+    /**
+     * Lookup a character using its alternative names. If found, cache it so we
+     * can speed up lookups.
+     * @param c the character
+     * @return the suggested alternative character present in the font
+     */
+    private char findAlternative(char c) {
+        char d;
+        if (alternativeCodes == null) {
+            alternativeCodes = new java.util.HashMap<Character, Character>();
+        } else {
+            Character alternative = alternativeCodes.get(c);
+            if (alternative != null) {
+                return alternative;
+            }
+        }
+        String charName = Glyphs.charToGlyphName(c);
+        String[] charNameAlternatives = Glyphs.getCharNameAlternativesFor(charName);
+        if (charNameAlternatives != null && charNameAlternatives.length > 0) {
+            for (int i = 0; i < charNameAlternatives.length; i++) {
+                if (log.isDebugEnabled()) {
+                    log.debug("Checking alternative for char " + c + " (charname="
+                            + charName + "): " + charNameAlternatives[i]);
+                }
+                String s = Glyphs.getUnicodeSequenceForGlyphName(charNameAlternatives[i]);
+                if (s != null) {
+                    d = lookupChar(s.charAt(0));
+                    if (d != SingleByteEncoding.NOT_FOUND_CODE_POINT) {
+                        alternativeCodes.put(c, d);
+                        return d;
+                    }
+                }
+            }
+        }
+
+        return SingleByteEncoding.NOT_FOUND_CODE_POINT;
+    }
+
+    private char lookupChar(char c) {
         char d = mapping.mapChar(c);
         if (d != SingleByteEncoding.NOT_FOUND_CODE_POINT) {
             return d;
         }
 
-        //Check unencoded characters which are available in the font by character name
+        // Check unencoded characters which are available in the font by
+        // character name
         d = mapUnencodedChar(c);
-        if (d != SingleByteEncoding.NOT_FOUND_CODE_POINT) {
-            return d;
+        return d;
+    }
+
+    private boolean isSubset() {
+        return getEmbeddingMode() == EmbeddingMode.SUBSET;
+    }
+
+    /** {@inheritDoc} */
+    @Override
+    public char mapChar(char c) {
+        notifyMapOperation();
+        char d = lookupChar(c);
+        if (d == SingleByteEncoding.NOT_FOUND_CODE_POINT) {
+            // Check for alternative
+            d = findAlternative(c);
+            if (d != SingleByteEncoding.NOT_FOUND_CODE_POINT) {
+                return d;
+            } else {
+                this.warnMissingGlyph(c);
+                return Typeface.NOT_FOUND;
+            }
+        }
+        if (isEmbeddable() && isSubset()) {
+            mapChar(d, c);
+        }
+        return d;
+    }
+
+    private int mapChar(int glyphIndex, char unicode) {
+        // Reencode to a new subset font or get the reencoded value
+        // IOW, accumulate the accessed characters and build a character map for them
+        Integer subsetCharSelector = usedGlyphs.get(glyphIndex);
+        if (subsetCharSelector == null) {
+            int selector = usedGlyphsCount;
+            usedGlyphs.put(glyphIndex, selector);
+            usedCharsIndex.put(selector, unicode);
+            charGIDMappings.put(unicode, glyphIndex);
+            usedGlyphsCount++;
+            return selector;
+        } else {
+            return subsetCharSelector;
+        }
+    }
+
+    private char getUnicode(int index) {
+        Character mapValue = usedCharsIndex.get(index);
+        if (mapValue != null) {
+            return mapValue.charValue();
+        } else {
+            return CharUtilities.NOT_A_CHARACTER;
         }
-        this.warnMissingGlyph(c);
-        return Typeface.NOT_FOUND;
     }
 
     private char mapUnencodedChar(char ch) {
         if (this.unencodedCharacters != null) {
-            UnencodedCharacter unencoded
-                = (UnencodedCharacter)this.unencodedCharacters.get(new Character(ch));
+            UnencodedCharacter unencoded = this.unencodedCharacters.get(Character.valueOf(ch));
             if (unencoded != null) {
                 if (this.additionalEncodings == null) {
-                    this.additionalEncodings = new java.util.ArrayList();
+                    this.additionalEncodings = new ArrayList<SimpleSingleByteEncoding>();
                 }
                 SimpleSingleByteEncoding encoding = null;
                 char mappedStart = 0;
@@ -150,6 +287,7 @@ public class SingleByteFont extends CustomFont {
     }
 
     /** {@inheritDoc} */
+    @Override
     public boolean hasChar(char c) {
         char d = mapping.mapChar(c);
         if (d != SingleByteEncoding.NOT_FOUND_CODE_POINT) {
@@ -160,6 +298,11 @@ public class SingleByteFont extends CustomFont {
         if (d != SingleByteEncoding.NOT_FOUND_CODE_POINT) {
             return true;
         }
+        // Check if an alternative exists
+        d = findAlternative(c);
+        if (d != SingleByteEncoding.NOT_FOUND_CODE_POINT) {
+            return true;
+        }
         return false;
     }
 
@@ -223,18 +366,25 @@ public class SingleByteFont extends CustomFont {
         this.width[index - getFirstChar()] = w;
     }
 
+    public void setBoundingBox(int index, Rectangle bbox) {
+        if (this.boundingBoxes == null) {
+            this.boundingBoxes = new Rectangle[getLastChar() - getFirstChar() + 1];
+        }
+        this.boundingBoxes[index - getFirstChar()] = bbox;
+    }
+
     /**
      * Adds an unencoded character (one that is not supported by the primary encoding).
      * @param ch the named character
      * @param width the width of the character
      */
-    public void addUnencodedCharacter(NamedCharacter ch, int width) {
+    public void addUnencodedCharacter(NamedCharacter ch, int width, Rectangle bbox) {
         if (this.unencodedCharacters == null) {
-            this.unencodedCharacters = new java.util.HashMap();
+            this.unencodedCharacters = new HashMap<Character, UnencodedCharacter>();
         }
         if (ch.hasSingleUnicodeValue()) {
-            UnencodedCharacter uc = new UnencodedCharacter(ch, width);
-            this.unencodedCharacters.put(new Character(ch.getSingleUnicodeValue()), uc);
+            UnencodedCharacter uc = new UnencodedCharacter(ch, width, bbox);
+            this.unencodedCharacters.put(Character.valueOf(ch.getSingleUnicodeValue()), uc);
         } else {
             //Cannot deal with unicode sequences, so ignore this character
         }
@@ -248,10 +398,8 @@ public class SingleByteFont extends CustomFont {
      */
     public void encodeAllUnencodedCharacters() {
         if (this.unencodedCharacters != null) {
-            Set sortedKeys = new java.util.TreeSet(this.unencodedCharacters.keySet());
-            Iterator iter = sortedKeys.iterator();
-            while (iter.hasNext()) {
-                Character ch = (Character)iter.next();
+            Set<Character> sortedKeys = new TreeSet<Character>(this.unencodedCharacters.keySet());
+            for (Character ch : sortedKeys) {
                 char mapped = mapChar(ch.charValue());
                 assert mapped != Typeface.NOT_FOUND;
             }
@@ -287,7 +435,7 @@ public class SingleByteFont extends CustomFont {
     public SimpleSingleByteEncoding getAdditionalEncoding(int index)
             throws IndexOutOfBoundsException {
         if (hasAdditionalEncodings()) {
-            return (SimpleSingleByteEncoding)this.additionalEncodings.get(index);
+            return this.additionalEncodings.get(index);
         } else {
             throw new IndexOutOfBoundsException("No additional encodings available");
         }
@@ -303,8 +451,8 @@ public class SingleByteFont extends CustomFont {
         int[] arr = new int[enc.getLastChar() - enc.getFirstChar() + 1];
         for (int i = 0, c = arr.length; i < c; i++) {
             NamedCharacter nc = enc.getCharacterForIndex(enc.getFirstChar() + i);
-            UnencodedCharacter uc = (UnencodedCharacter)this.unencodedCharacters.get(
-                    new Character(nc.getSingleUnicodeValue()));
+            UnencodedCharacter uc = this.unencodedCharacters.get(
+                    Character.valueOf(nc.getSingleUnicodeValue()));
             arr[i] = uc.getWidth();
         }
         return arr;
@@ -312,12 +460,14 @@ public class SingleByteFont extends CustomFont {
 
     private static final class UnencodedCharacter {
 
-        private NamedCharacter character;
-        private int width;
+        private final NamedCharacter character;
+        private final int width;
+        private final Rectangle bbox;
 
-        public UnencodedCharacter(NamedCharacter character, int width) {
+        public UnencodedCharacter(NamedCharacter character, int width, Rectangle bbox) {
             this.character = character;
             this.width = width;
+            this.bbox = bbox;
         }
 
         public NamedCharacter getCharacter() {
@@ -328,11 +478,75 @@ public class SingleByteFont extends CustomFont {
             return this.width;
         }
 
+        public Rectangle getBBox() {
+            return bbox;
+        }
+
         /** {@inheritDoc} */
+        @Override
         public String toString() {
             return getCharacter().toString();
         }
     }
 
+    /**
+     * Sets the version of the PostScript table stored in the TrueType font represented by
+     * this instance.
+     *
+     * @param version version of the <q>post</q> table
+     */
+    public void setTrueTypePostScriptVersion(PostScriptVersion version) {
+        ttPostScriptVersion = version;
+    }
+
+    /**
+     * Returns the version of the PostScript table stored in the TrueType font represented by
+     * this instance.
+     *
+     * @return the version of the <q>post</q> table
+     */
+    public PostScriptVersion getTrueTypePostScriptVersion() {
+        assert getFontType() == FontType.TRUETYPE;
+        return ttPostScriptVersion;
+    }
+
+    /**
+     * Returns a Map of used Glyphs.
+     * @return Map Map of used Glyphs
+     */
+    public Map<Integer, Integer> getUsedGlyphs() {
+        return Collections.unmodifiableMap(usedGlyphs);
+    }
+
+    public char getUnicodeFromSelector(int selector) {
+        return getUnicode(selector);
+    }
+
+    public int getGIDFromChar(char ch) {
+        return charGIDMappings.get(ch);
+    }
+
+    public char getUnicodeFromGID(int glyphIndex) {
+        int selector = usedGlyphs.get(glyphIndex);
+        return usedCharsIndex.get(selector);
+    }
+
+    public void mapUsedGlyphName(int gid, String value) {
+        usedGlyphNames.put(gid, value);
+    }
+
+    public Map<Integer, String> getUsedGlyphNames() {
+        return usedGlyphNames;
+    }
+
+    public String getGlyphName(int idx) {
+        if (idx < mapping.getCharNameMap().length) {
+            return mapping.getCharNameMap()[idx];
+        } else {
+            int selector = usedGlyphs.get(idx);
+            char theChar = usedCharsIndex.get(selector);
+            return unencodedCharacters.get(theChar).getCharacter().getName();
+        }
+    }
 }
 
