@@ -15,11 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: TTFReader.java 1357883 2012-07-05 20:29:53Z gadams $ */
+/* $Id: TTFReader.java 1617052 2014-08-10 06:55:01Z gadams $ */
 
 package org.apache.fop.fonts.apps;
 
+import java.io.FileInputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.util.Map;
 import java.util.Set;
 
@@ -30,17 +32,12 @@ import org.w3c.dom.Element;
 import org.xml.sax.Attributes;
 import org.xml.sax.SAXException;
 
-import org.apache.commons.logging.LogFactory;
-
 import org.apache.fop.Version;
 import org.apache.fop.fonts.CMapSegment;
 import org.apache.fop.fonts.FontUtil;
 import org.apache.fop.fonts.truetype.FontFileReader;
+import org.apache.fop.fonts.truetype.OFFontLoader;
 import org.apache.fop.fonts.truetype.TTFFile;
-import org.apache.fop.util.CommandLineLogger;
-
-// CSOFF: InnerAssignmentCheck
-// CSOFF: LineLengthCheck
 
 /**
  * A tool which reads TTF files and generates
@@ -121,14 +118,6 @@ public class TTFReader extends AbstractFontReader {
         Map options = new java.util.HashMap();
         String[] arguments = parseArguments(options, args);
 
-        // Enable the simple command line logging when no other logger is
-        // defined.
-        LogFactory logFactory = LogFactory.getFactory();
-        if (System.getProperty("org.apache.commons.logging.Log") == null) {
-            logFactory.setAttribute("org.apache.commons.logging.Log",
-                                            CommandLineLogger.class.getName());
-        }
-
         determineLogLevel(options);
 
         TTFReader app = new TTFReader();
@@ -216,15 +205,22 @@ public class TTFReader extends AbstractFontReader {
      * @return The TTF as an object, null if the font is incompatible.
      * @throws IOException In case of an I/O problem
      */
-    public TTFFile loadTTF(String fileName, String fontName, boolean useKerning, boolean useAdvanced) throws IOException {
+    public TTFFile loadTTF(String fileName, String fontName, boolean useKerning, boolean useAdvanced)
+            throws IOException {
         TTFFile ttfFile = new TTFFile(useKerning, useAdvanced);
         log.info("Reading " + fileName + "...");
-
-        FontFileReader reader = new FontFileReader(fileName);
-        boolean supported = ttfFile.readFont(reader, fontName);
-        if (!supported) {
-            return null;
+        InputStream stream = new FileInputStream(fileName);
+        try {
+            FontFileReader reader = new FontFileReader(stream);
+            String header = OFFontLoader.readHeader(reader);
+            boolean supported = ttfFile.readFont(reader, header, fontName);
+            if (!supported) {
+                return null;
+            }
+        } finally {
+            stream.close();
         }
+
         log.info("Font Family: " + ttfFile.getFamilyNames());
         if (ttfFile.isCFF()) {
             throw new UnsupportedOperationException(
@@ -401,6 +397,11 @@ public class TTFReader extends AbstractFontReader {
         for (int i = 0; i < wx.length; i++) {
             Element wxel = doc.createElement("wx");
             wxel.setAttribute("w", String.valueOf(wx[i]));
+            int[] bbox = ttf.getBBox(i);
+            wxel.setAttribute("xMin", String.valueOf(bbox[0]));
+            wxel.setAttribute("yMin", String.valueOf(bbox[1]));
+            wxel.setAttribute("xMax", String.valueOf(bbox[2]));
+            wxel.setAttribute("yMax", String.valueOf(bbox[3]));
             el.appendChild(wxel);
         }
     }
@@ -461,11 +462,12 @@ public class TTFReader extends AbstractFontReader {
                 h2 = ttf.getAnsiKerning().get(kpx1);
             }
 
-            for (Integer kpx2 : h2.keySet()) {
+            for (Map.Entry<Integer, Integer> e : h2.entrySet()) {
+                Integer kpx2 = e.getKey();
                 if (isCid || kpx2.intValue() < 256) {
                     el2 = doc.createElement("pair");
                     el2.setAttribute("kpx2", kpx2.toString());
-                    Integer val = (Integer)h2.get(kpx2);
+                    Integer val = e.getValue();
                     el2.setAttribute("kern", val.toString());
                     el.appendChild(el2);
                 }
