@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FontQualifier.java 1296526 2012-03-03 00:18:45Z gadams $ */
+/* $Id: FontQualifier.java 1616668 2014-08-08 03:48:54Z gadams $ */
 
 package org.apache.fop.fonts.substitute;
 
@@ -41,13 +41,13 @@ public class FontQualifier {
     private static Log log = LogFactory.getLog(FontQualifier.class);
 
     /** font family attribute value */
-    private AttributeValue fontFamilyAttributeValue = null;
+    private AttributeValue fontFamilyAttributeValue;
 
     /** font style attribute value */
-    private AttributeValue fontStyleAttributeValue = null;
+    private AttributeValue fontStyleAttributeValue;
 
     /** font weight attribute value */
-    private AttributeValue fontWeightAttributeValue = null;
+    private AttributeValue fontWeightAttributeValue;
 
     /**
      * Default constructor
@@ -285,7 +285,7 @@ public class FontQualifier {
      * {@inheritDoc}
      */
     public String toString() {
-        String str = new String();
+        String str = "";
         if (fontFamilyAttributeValue != null) {
             str += "font-family=" + fontFamilyAttributeValue;
         }
