@@ -15,23 +15,28 @@
  * limitations under the License.
  */
 
-/* $Id: Type1FontLoader.java 1293736 2012-02-26 02:29:01Z gadams $ */
+/* $Id: Type1FontLoader.java 1641827 2014-11-26 15:00:57Z lbernardo $ */
 
 package org.apache.fop.fonts.type1;
 
 import java.awt.geom.RectangularShape;
 import java.io.IOException;
 import java.io.InputStream;
+import java.net.URISyntaxException;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 
 import org.apache.commons.io.IOUtils;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 
+import org.apache.fop.apps.io.InternalResourceResolver;
 import org.apache.fop.fonts.CodePointMapping;
+import org.apache.fop.fonts.EmbeddingMode;
 import org.apache.fop.fonts.FontLoader;
-import org.apache.fop.fonts.FontResolver;
 import org.apache.fop.fonts.FontType;
+import org.apache.fop.fonts.FontUris;
 import org.apache.fop.fonts.SingleByteEncoding;
 import org.apache.fop.fonts.SingleByteFont;
 
@@ -40,19 +45,27 @@ import org.apache.fop.fonts.SingleByteFont;
  */
 public class Type1FontLoader extends FontLoader {
 
+    private static final Log log = LogFactory.getLog(Type1FontLoader.class);
+
     private SingleByteFont singleFont;
 
+    private EmbeddingMode embeddingMode;
+
+    private final FontUris fontUris;
+
     /**
      * Constructs a new Type 1 font loader.
      * @param fontFileURI the URI to the PFB file of a Type 1 font
      * @param embedded indicates whether the font is embedded or referenced
      * @param useKerning indicates whether to load kerning information if available
-     * @param resolver the font resolver used to resolve URIs
+     * @param resourceResolver the font resolver used to resolve URIs
      * @throws IOException In case of an I/O error
      */
-    public Type1FontLoader(String fontFileURI, boolean embedded, boolean useKerning,
-            FontResolver resolver) throws IOException {
-        super(fontFileURI, embedded, useKerning, true, resolver);
+    public Type1FontLoader(FontUris fontUris, boolean embedded, EmbeddingMode embeddingMode,
+            boolean useKerning, InternalResourceResolver resourceResolver) throws IOException {
+        super(fontUris.getEmbed(), embedded, useKerning, true, resourceResolver);
+        this.embeddingMode = embeddingMode;
+        this.fontUris = fontUris;
     }
 
     private String getPFMURI(String pfbURI) {
@@ -71,17 +84,28 @@ public class Type1FontLoader extends FontLoader {
         PFMFile pfm = null;
 
         InputStream afmIn = null;
-        String afmUri = null;
-        for (int i = 0; i < AFM_EXTENSIONS.length; i++) {
-            try {
-                afmUri = this.fontFileURI.substring(0, this.fontFileURI.length() - 4)
-                        + AFM_EXTENSIONS[i];
-                afmIn = openFontUri(resolver, afmUri);
-                if (afmIn != null) {
-                    break;
+        String fontFileStr = fontFileURI.toASCIIString();
+        String partialAfmUri = fontFileStr.substring(0, fontFileStr.length() - 4);
+        String afmUri = (fontUris.getAfm() != null) ? fontUris.getAfm().toASCIIString() : null;
+        if (afmUri == null) {
+            for (String afmExtension : AFM_EXTENSIONS) {
+                try {
+                    afmUri = partialAfmUri + afmExtension;
+                    afmIn = resourceResolver.getResource(afmUri);
+                    if (afmIn != null) {
+                        break;
+                    }
+                } catch (IOException ioe) {
+                    // Ignore, AFM probably not available under the URI
+                } catch (URISyntaxException e) {
+                    // Ignore, AFM probably not available under the URI
                 }
-            } catch (IOException ioe) {
-                // Ignore, AFM probably not available under the URI
+            }
+        } else {
+            try {
+                afmIn = resourceResolver.getResource(afmUri);
+            } catch (URISyntaxException e) {
+                throw new IOException(e);
             }
         }
         if (afmIn != null) {
@@ -93,12 +117,15 @@ public class Type1FontLoader extends FontLoader {
             }
         }
 
-        String pfmUri = getPFMURI(this.fontFileURI);
+        String pfmUri = (fontUris.getPfm() == null) ? getPFMURI(fontFileStr) : fontUris.getPfm()
+                .toASCIIString();
         InputStream pfmIn = null;
         try {
-            pfmIn = openFontUri(resolver, pfmUri);
+            pfmIn = resourceResolver.getResource(pfmUri);
         } catch (IOException ioe) {
             // Ignore, PFM probably not available under the URI
+        } catch (URISyntaxException e) {
+            // Ignore, PFM probably not available under the URI
         }
         if (pfmIn != null) {
             try {
@@ -126,11 +153,10 @@ public class Type1FontLoader extends FontLoader {
         if (afm == null && pfm == null) {
             throw new IllegalArgumentException("Need at least an AFM or a PFM!");
         }
-        singleFont = new SingleByteFont();
+        singleFont = new SingleByteFont(resourceResolver, embeddingMode);
         singleFont.setFontType(FontType.TYPE1);
-        singleFont.setResolver(this.resolver);
         if (this.embedded) {
-            singleFont.setEmbedFileName(this.fontFileURI);
+            singleFont.setEmbedURI(this.fontFileURI);
         }
         returnFont = singleFont;
 
@@ -163,7 +189,21 @@ public class Type1FontLoader extends FontLoader {
                 addUnencodedBasedOnAFM(afm);
             }
         } else {
-            if (pfm.getCharSet() >= 0 && pfm.getCharSet() <= 2) {
+            if (pfm.getCharSet() == 2 && !pfm.getCharSetName().equals("Symbol")) {
+                int[] table = new int[256];
+                String[] charNameMap = new String[256];
+                int j = 0;
+                for (int i = pfm.getFirstChar(); i < pfm.getLastChar(); i++) {
+                    if (j < table.length) {
+                        table[j] = i;
+                        table[j + 1] = i;
+                        j += 2;
+                    }
+                    charNameMap[i] = String.format("x%03o", i);
+                }
+                CodePointMapping mapping  = new CodePointMapping("custom", table, charNameMap);
+                singleFont.setEncoding(mapping);
+            } else if (pfm.getCharSet() >= 0 && pfm.getCharSet() <= 2) {
                 singleFont.setEncoding(pfm.getCharSetName() + "Encoding");
             } else {
                 log.warn("The PFM reports an unsupported encoding ("
@@ -193,12 +233,16 @@ public class Type1FontLoader extends FontLoader {
         for (AFMCharMetrics metrics : charMetrics) {
             String charName = metrics.getCharName();
             if (charName != null && !glyphNames.contains(charName)) {
-                singleFont.addUnencodedCharacter(metrics.getCharacter(),
-                        (int)Math.round(metrics.getWidthX()));
+                addUnencodedCharacter(singleFont, metrics);
             }
         }
     }
 
+    private static void addUnencodedCharacter(SingleByteFont font, AFMCharMetrics metrics) {
+        font.addUnencodedCharacter(metrics.getCharacter(),
+                (int) Math.round(metrics.getWidthX()), metrics.getBBox());
+    }
+
     /**
      * Adds characters not encoded in the font's primary encoding. This method is used when
      * the primary encoding is built based on the character codes in the AFM rather than
@@ -210,8 +254,7 @@ public class Type1FontLoader extends FontLoader {
         for (int i = 0, c = afm.getCharCount(); i < c; i++) {
             AFMCharMetrics metrics = (AFMCharMetrics)charMetrics.get(i);
             if (!metrics.hasCharCode() && metrics.getCharacter() != null) {
-                singleFont.addUnencodedCharacter(metrics.getCharacter(),
-                        (int)Math.round(metrics.getWidthX()));
+                addUnencodedCharacter(singleFont, metrics);
             }
         }
     }
@@ -257,7 +300,10 @@ public class Type1FontLoader extends FontLoader {
             } else {
                 returnFont.setStemV(80); // Arbitrary value
             }
-            returnFont.setItalicAngle((int) afm.getWritingDirectionMetrics(0).getItalicAngle());
+            AFMWritingDirectionMetrics metrics = afm.getWritingDirectionMetrics(0);
+            returnFont.setItalicAngle((int) metrics.getItalicAngle());
+            returnFont.setUnderlinePosition(metrics.getUnderlinePosition().intValue());
+            returnFont.setUnderlineThickness(metrics.getUnderlineThickness().intValue());
         } else {
             returnFont.setFontBBox(pfm.getFontBBox());
             returnFont.setStemV(pfm.getStemV());
@@ -359,6 +405,7 @@ public class Type1FontLoader extends FontLoader {
             for (AFMCharMetrics chm : afm.getCharMetrics()) {
                 if (chm.hasCharCode()) {
                     singleFont.setWidth(chm.getCharCode(), (int) Math.round(chm.getWidthX()));
+                    singleFont.setBoundingBox(chm.getCharCode(), chm.getBBox());
                 }
             }
             if (useKerning) {
@@ -383,10 +430,7 @@ public class Type1FontLoader extends FontLoader {
         List<AFMCharMetrics> chars = afm.getCharMetrics();
         for (AFMCharMetrics charMetrics : chars) {
             if (charMetrics.getCharCode() >= 0) {
-                String u = charMetrics.getUnicodeSequence();
-                if (u != null && u.length() == 1) {
-                    mappingCount++;
-                }
+                mappingCount++;
             }
         }
         // ...and now build the table.
@@ -399,6 +443,10 @@ public class Type1FontLoader extends FontLoader {
                 String unicodes = charMetrics.getUnicodeSequence();
                 if (unicodes == null) {
                     log.info("No Unicode mapping for glyph: " + charMetrics);
+                    table[idx] = charMetrics.getCharCode();
+                    idx++;
+                    table[idx] = charMetrics.getCharCode();
+                    idx++;
                 } else if (unicodes.length() == 1) {
                     table[idx] = charMetrics.getCharCode();
                     idx++;
