@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: HyphenationTree.java 1297284 2012-03-05 23:29:29Z gadams $ */
+/* $Id: HyphenationTree.java 1617052 2014-08-10 06:55:01Z gadams $ */
 
 package org.apache.fop.hyphenation;
 
@@ -27,13 +27,16 @@ import java.io.FileReader;
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
-import java.io.Serializable;
 import java.net.MalformedURLException;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
+import java.util.List;
 
 import org.xml.sax.InputSource;
 
+import org.apache.commons.io.IOUtils;
+
 /**
  * <p>This tree structure stores the hyphenation patterns in an efficient
  * way for fast lookup. It provides the provides the method to
@@ -41,8 +44,7 @@ import org.xml.sax.InputSource;
  *
  * <p>This work was authored by Carlos Villegas (cav@uniscope.co.jp).</p>
  */
-public class HyphenationTree extends TernaryTree
-            implements PatternConsumer, Serializable {
+public class HyphenationTree extends TernaryTree implements PatternConsumer {
 
     private static final long serialVersionUID = -7842107987915665573L;
 
@@ -74,6 +76,10 @@ public class HyphenationTree extends TernaryTree
         vspace.alloc(1);    // this reserves index 0, which we don't use
     }
 
+    private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
+        ois.defaultReadObject();
+    }
+
     /**
      * Packs the values by storing them in 4 bits, two values into a byte
      * Values range is from 0 to 9. We use zero as terminator,
@@ -321,7 +327,110 @@ public class HyphenationTree extends TernaryTree
     public Hyphenation hyphenate(String word, int remainCharCount,
                                  int pushCharCount) {
         char[] w = word.toCharArray();
-        return hyphenate(w, 0, w.length, remainCharCount, pushCharCount);
+        if (isMultiPartWord(w, w.length)) {
+            List<char[]> words = splitOnNonCharacters(w);
+            return new Hyphenation(new String(w),
+                    getHyphPointsForWords(words, remainCharCount, pushCharCount));
+        } else {
+            return hyphenate(w, 0, w.length, remainCharCount, pushCharCount);
+        }
+    }
+
+    private boolean isMultiPartWord(char[] w, int len) {
+        int wordParts = 0;
+        for (int i = 0; i < len; i++) {
+            char[] c = new char[2];
+            c[0] = w[i];
+            int nc = classmap.find(c, 0);
+            if (nc > 0) {
+                if (wordParts > 1) {
+                    return true;
+                }
+                wordParts = 1;
+            } else {
+                if (wordParts == 1) {
+                    wordParts++;
+                }
+            }
+        }
+        return false;
+    }
+
+    private List<char[]> splitOnNonCharacters(char[] word) {
+        List<Integer> breakPoints = getNonLetterBreaks(word);
+        if (breakPoints.size() == 0) {
+            return Collections.emptyList();
+        }
+        List<char[]> words = new ArrayList<char[]>();
+        for (int ibreak = 0; ibreak < breakPoints.size(); ibreak++) {
+            char[] newWord = getWordFromCharArray(word, ((ibreak == 0)
+                    ? 0 : breakPoints.get(ibreak - 1)), breakPoints.get(ibreak));
+            words.add(newWord);
+        }
+        if (word.length - breakPoints.get(breakPoints.size() - 1) - 1 > 1) {
+            char[] newWord = getWordFromCharArray(word, breakPoints.get(breakPoints.size() - 1),
+                    word.length);
+            words.add(newWord);
+        }
+        return words;
+    }
+
+    private List<Integer> getNonLetterBreaks(char[] word) {
+        char[] c = new char[2];
+        List<Integer> breakPoints = new ArrayList<Integer>();
+        boolean foundLetter = false;
+        for (int i = 0; i < word.length; i++) {
+            c[0] = word[i];
+            if (classmap.find(c, 0) < 0) {
+                if (foundLetter) {
+                    breakPoints.add(i);
+                }
+            } else {
+                foundLetter = true;
+            }
+        }
+        return breakPoints;
+    }
+
+    private char[] getWordFromCharArray(char[] word, int startIndex, int endIndex) {
+        char[] newWord = new char[endIndex - ((startIndex == 0) ? startIndex : startIndex + 1)];
+        int iChar = 0;
+        for (int i = (startIndex == 0) ? 0 : startIndex + 1; i < endIndex; i++) {
+            newWord[iChar++] = word[i];
+        }
+        return newWord;
+    }
+
+    private int[] getHyphPointsForWords(List<char[]> nonLetterWords, int remainCharCount,
+            int pushCharCount) {
+        int[] breaks = new int[0];
+        for (int iNonLetterWord = 0; iNonLetterWord < nonLetterWords.size(); iNonLetterWord++) {
+            char[] nonLetterWord = nonLetterWords.get(iNonLetterWord);
+            Hyphenation curHyph = hyphenate(nonLetterWord, 0, nonLetterWord.length,
+                    (iNonLetterWord == 0) ? remainCharCount : 1,
+                    (iNonLetterWord == nonLetterWords.size() - 1) ? pushCharCount : 1);
+            if (curHyph == null) {
+                continue;
+            }
+            int[] combined = new int[breaks.length + curHyph.getHyphenationPoints().length];
+            int[] hyphPoints = curHyph.getHyphenationPoints();
+            int foreWordsSize = calcForeWordsSize(nonLetterWords, iNonLetterWord);
+            for (int i = 0; i < hyphPoints.length; i++) {
+                hyphPoints[i] += foreWordsSize;
+            }
+            System.arraycopy(breaks, 0, combined, 0, breaks.length);
+            System.arraycopy(hyphPoints, 0, combined, breaks.length, hyphPoints.length);
+            breaks = combined;
+        }
+        return breaks;
+    }
+
+    private int calcForeWordsSize(List<char[]> nonLetterWords, int iNonLetterWord) {
+        int result = 0;
+        for (int i = 0; i < iNonLetterWord; i++) {
+            result += nonLetterWords.get(i).length + 1;
+        }
+        return result;
     }
 
     /**
@@ -389,6 +498,7 @@ public class HyphenationTree extends TernaryTree
                 }
             }
         }
+
         len = iLength;
         if (len < (remainCharCount + pushCharCount)) {
             // word is too short to be hyphenated
@@ -533,24 +643,44 @@ public class HyphenationTree extends TernaryTree
                     + "b:\tbenchmark\n"
                     + "q:\tquit\n\n"
                     + "Command:");
-            String token = in.readLine().trim();
+            String token = in.readLine();
+            if (token == null) {
+                break;
+            }
+            token = token.trim();
             if (token.equals("f")) {
                 System.out.print("Pattern: ");
-                token = in.readLine().trim();
+                token = in.readLine();
+                if (token == null) {
+                    break;
+                }
+                token = token.trim();
                 System.out.println("Values: " + ht.findPattern(token));
             } else if (token.equals("s")) {
                 System.out.print("Minimun value: ");
-                token = in.readLine().trim();
+                token = in.readLine();
+                if (token == null) {
+                    break;
+                }
+                token = token.trim();
                 minCharCount = Integer.parseInt(token);
             } else if (token.equals("l")) {
                 ht = new HyphenationTree();
                 System.out.print("XML file name: ");
-                token = in.readLine().trim();
+                token = in.readLine();
+                if (token == null) {
+                    break;
+                }
+                token = token.trim();
                 ht.loadPatterns(token);
             } else if (token.equals("L")) {
                 ObjectInputStream ois = null;
                 System.out.print("Object file name: ");
-                token = in.readLine().trim();
+                token = in.readLine();
+                if (token == null) {
+                    break;
+                }
+                token = token.trim();
                 try {
                     ois = new ObjectInputStream(new FileInputStream(token));
                     ht = (HyphenationTree)ois.readObject();
@@ -567,7 +697,11 @@ public class HyphenationTree extends TernaryTree
                 }
             } else if (token.equals("w")) {
                 System.out.print("Object file name: ");
-                token = in.readLine().trim();
+                token = in.readLine();
+                if (token == null) {
+                    break;
+                }
+                token = token.trim();
                 ObjectOutputStream oos = null;
                 try {
                     oos = new ObjectOutputStream(new FileOutputStream(token));
@@ -590,7 +724,11 @@ public class HyphenationTree extends TernaryTree
                 }
             } else if (token.equals("h")) {
                 System.out.print("Word: ");
-                token = in.readLine().trim();
+                token = in.readLine();
+                if (token == null) {
+                    break;
+                }
+                token = token.trim();
                 System.out.print("Hyphenation points: ");
                 System.out.println(ht.hyphenate(token, minCharCount,
                                                 minCharCount));
@@ -600,12 +738,16 @@ public class HyphenationTree extends TernaryTree
                     break;
                 }
                 System.out.print("Word list filename: ");
-                token = in.readLine().trim();
+                token = in.readLine();
+                if (token == null) {
+                    break;
+                }
+                token = token.trim();
                 long starttime = 0;
                 int counter = 0;
+                BufferedReader reader = null;
                 try {
-                    BufferedReader reader
-                        = new BufferedReader(new FileReader(token));
+                    reader = new BufferedReader(new FileReader(token));
                     String line;
 
                     starttime = System.currentTimeMillis();
@@ -625,6 +767,8 @@ public class HyphenationTree extends TernaryTree
                 } catch (Exception ioe) {
                     System.out.println("Exception " + ioe);
                     ioe.printStackTrace();
+                } finally {
+                    IOUtils.closeQuietly(reader);
                 }
                 long endtime = System.currentTimeMillis();
                 long result = endtime - starttime;
