@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,12 +15,16 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractBaseLayoutManager.java 557337 2007-07-18 17:37:14Z adelmelle $ */
+/* $Id: AbstractBaseLayoutManager.java 1678425 2015-05-08 18:52:41Z adelmelle $ */
 
 package org.apache.fop.layoutmgr;
 
+import java.util.List;
+import java.util.Stack;
+
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.datatypes.LengthBase;
 import org.apache.fop.datatypes.PercentBaseContext;
 import org.apache.fop.fo.FObj;
@@ -29,27 +33,28 @@ import org.apache.fop.fo.FObj;
  * The base class for nearly all LayoutManagers.
  * Provides the functionality for merging the {@link LayoutManager}
  * and the {@link org.apache.fop.datatypes.PercentBaseContext} interfaces
- * into a common base calls for all higher LayoutManagers.
+ * into a common base class for all higher LayoutManagers.
  */
-public abstract class AbstractBaseLayoutManager 
+public abstract class AbstractBaseLayoutManager
     implements LayoutManager, PercentBaseContext {
-    
-    /** Indicator if this LM generates reference areas */
-    protected boolean generatesReferenceArea = false;
-    /** Indicator if this LM generates block areas */
-    protected boolean generatesBlockArea = false;
-    /** The formatting object for this LM */
-    protected FObj fobj = null;
+
+    /** Indicator if this LM generates reference areas. */
+    protected boolean generatesReferenceArea;
+    /** Indicator if this LM generates block areas. */
+    protected boolean generatesBlockArea;
+    /** The formatting object for this LM. */
+    protected final FObj fobj;
 
     /**
      * logging instance
      */
-    private static Log log = LogFactory.getLog(AbstractBaseLayoutManager.class);
+    private static final Log LOG = LogFactory.getLog(AbstractBaseLayoutManager.class);
 
     /**
      * Abstract base layout manager.
      */
     public AbstractBaseLayoutManager() {
+        fobj = null;
     }
 
     /**
@@ -58,20 +63,18 @@ public abstract class AbstractBaseLayoutManager
      * @param fo the formatting object for this layout manager
      */
     public AbstractBaseLayoutManager(FObj fo) {
-        fobj = fo;
-        setGeneratesReferenceArea(fo.generatesReferenceAreas());
-        if (getGeneratesReferenceArea()) {
-            setGeneratesBlockArea(true);
+        if (fo == null) {
+            throw new IllegalStateException("Null formatting object found.");
         }
+        this.fobj = fo;
+        setGeneratesReferenceArea(fo.generatesReferenceAreas());
     }
 
     // --------- Property Resolution related functions --------- //
-    
-    /**
-     * {@inheritDoc} 
-     */
-    public int getBaseLength(int lengthBase, FObj fobj) {
-        if (fobj == this.fobj) {
+
+    /** {@inheritDoc} */
+    public int getBaseLength(int lengthBase, FObj fobjx) {
+        if (fobjx == this.fobj) {
             switch (lengthBase) {
             case LengthBase.CONTAINING_BLOCK_WIDTH:
                 return getAncestorBlockAreaIPD();
@@ -82,20 +85,20 @@ public abstract class AbstractBaseLayoutManager
             case LengthBase.CONTAINING_REFAREA_WIDTH:
                 return getReferenceAreaIPD();
             default:
-                log.error(new Exception("Unknown base type for LengthBase:" + lengthBase));
+                LOG.error("Unknown base type for LengthBase:" + lengthBase);
                 return 0;
             }
         } else {
             LayoutManager lm = getParent();
-            while (lm != null && fobj != lm.getFObj()) {
+            while (lm != null && fobjx != lm.getFObj()) {
                 lm = lm.getParent();
             }
             if (lm != null) {
-                return lm.getBaseLength(lengthBase, fobj);
+                return lm.getBaseLength(lengthBase, fobjx);
             }
         }
-        log.error("Cannot find LM to handle given FO for LengthBase. ("
-                + fobj.getContextInfo() + ")");
+        LOG.error("Cannot find LM to handle given FO for LengthBase. ("
+                + fobjx.getContextInfo() + ")");
         return 0;
     }
 
@@ -112,9 +115,7 @@ public abstract class AbstractBaseLayoutManager
             }
             lm = lm.getParent();
         }
-        if (lm == null) {
-            log.error("No parent LM found");
-        }
+        LOG.error("No parent LM found");
         return 0;
     }
 
@@ -131,9 +132,7 @@ public abstract class AbstractBaseLayoutManager
             }
             lm = lm.getParent();
         }
-        if (lm == null) {
-            log.error("No parent LM found");
-        }
+        LOG.error("No parent LM found");
         return 0;
     }
 
@@ -146,7 +145,7 @@ public abstract class AbstractBaseLayoutManager
         if (lm != null) {
             return lm.getContentAreaIPD();
         }
-        log.error("No parent LM found");
+        LOG.error("No parent LM found");
         return 0;
     }
 
@@ -159,7 +158,7 @@ public abstract class AbstractBaseLayoutManager
         if (lm != null) {
             return lm.getContentAreaBPD();
         }
-        log.error("No parent LM found");
+        LOG.error("No parent LM found");
         return 0;
     }
 
@@ -176,9 +175,7 @@ public abstract class AbstractBaseLayoutManager
             }
             lm = lm.getParent();
         }
-        if (lm == null) {
-            log.error("No parent LM found");
-        }
+        LOG.error("No parent LM found");
         return 0;
     }
 
@@ -195,52 +192,50 @@ public abstract class AbstractBaseLayoutManager
             }
             lm = lm.getParent();
         }
-        if (lm == null) {
-            log.error("No parent LM found");
-        }
+        LOG.error("No parent LM found");
         return 0;
     }
 
     /**
      * {@inheritDoc}
+     * <i>NOTE: Should be overridden by subclasses.
+     * Default implementation throws an <code>UnsupportedOperationException</code>.</i>
      */
     public int getContentAreaIPD() {
-        log.error("getContentAreaIPD called when it should have been overwritten");
-        return 0;
+        throw new UnsupportedOperationException(
+                "getContentAreaIPD() called when it should have been overridden");
     }
-   
+
     /**
      * {@inheritDoc}
+     * <i>NOTE: Should be overridden by subclasses.
+     * Default implementation throws an <code>UnsupportedOperationException</code>.</i>
      */
     public int getContentAreaBPD() {
-        log.error("getContentAreaBPD called when it should have been overwritten");
-        return 0;
+        throw new UnsupportedOperationException(
+                "getContentAreaBPD() called when it should have been overridden");
     }
-    
-    /**
-     * {@inheritDoc}
-     */
+
+    /** {@inheritDoc} */
     public boolean getGeneratesReferenceArea() {
         return generatesReferenceArea;
     }
-   
+
     /**
      * Lets implementing LM set the flag indicating if they
      * generate reference areas.
-     * @param generatesReferenceArea if true the areas generates by this LM are 
+     * @param generatesReferenceArea if true the areas generates by this LM are
      * reference areas.
      */
     protected void setGeneratesReferenceArea(boolean generatesReferenceArea) {
         this.generatesReferenceArea = generatesReferenceArea;
     }
-   
-    /**
-     * {@inheritDoc}
-     */
+
+    /** {@inheritDoc} */
     public boolean getGeneratesBlockArea() {
         return generatesBlockArea;
     }
-   
+
     /**
      * Lets implementing LM set the flag indicating if they
      * generate block areas.
@@ -249,19 +244,40 @@ public abstract class AbstractBaseLayoutManager
     protected void setGeneratesBlockArea(boolean generatesBlockArea) {
         this.generatesBlockArea = generatesBlockArea;
     }
-   
-    /**
-     * {@inheritDoc}
-     */
+
+    /** {@inheritDoc} */
     public boolean getGeneratesLineArea() {
         return false;
     }
-    
+
     /**
      * {@inheritDoc}
      */
     public FObj getFObj() {
         return fobj;
     }
-   
+
+    /** {@inheritDoc} */
+    public void reset() {
+        throw new UnsupportedOperationException("Not implemented");
+    }
+
+    /** {@inheritDoc} */
+    public boolean isRestartable() {
+        return false;
+    }
+
+    /** {@inheritDoc} */
+    public List getNextKnuthElements(LayoutContext context, int alignment, Stack lmStack,
+            Position positionAtIPDChange, LayoutManager restartAtLM) {
+        throw new UnsupportedOperationException("Not implemented");
+    }
+
+    public void preserveChildrenAtEndOfLayout() {
+
+    }
+
+    public void recreateChildrenLMs() {
+
+    }
 }
