@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractPageSequenceLayoutManager.java 808157 2009-08-26 18:50:10Z vhennebert $ */
+/* $Id: AbstractPageSequenceLayoutManager.java 1723297 2016-01-06 12:13:52Z ssteiner $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -59,9 +59,9 @@ public abstract class AbstractPageSequenceLayoutManager extends AbstractLayoutMa
     protected Page curPage;
 
     /** the current page number */
-    protected int currentPageNum = 0;
+    protected int currentPageNum;
     /** The stating page number */
-    protected int startPageNum = 0;
+    protected int startPageNum;
 
     /**
      * Constructor
@@ -226,15 +226,14 @@ public abstract class AbstractPageSequenceLayoutManager extends AbstractLayoutMa
     public RetrieveMarker resolveRetrieveMarker(RetrieveMarker rm) {
         AreaTreeModel areaTreeModel = areaTreeHandler.getAreaTreeModel();
         String name = rm.getRetrieveClassName();
-        int pos = rm.getRetrievePosition();
         int boundary = rm.getRetrieveBoundary();
 
         // get marker from the current markers on area tree
-        Marker mark = (Marker)getCurrentPV().getMarker(name, pos);
+        Marker mark = getCurrentPV().resolveMarker(rm);
         if (mark == null && boundary != EN_PAGE) {
             // go back over pages until mark found
             // if document boundary then keep going
-            boolean doc = boundary == EN_DOCUMENT;
+            boolean doc = (boundary == EN_DOCUMENT);
             int seq = areaTreeModel.getPageSequenceCount();
             int page = areaTreeModel.getPageCount(seq) - 1;
             while (page < 0 && doc && seq > 1) {
@@ -243,7 +242,11 @@ public abstract class AbstractPageSequenceLayoutManager extends AbstractLayoutMa
             }
             while (page >= 0) {
                 PageViewport pv = areaTreeModel.getPage(seq, page);
-                mark = (Marker)pv.getMarker(name, Constants.EN_LEWP);
+                int originalPosition = rm.getPosition();
+                rm.changePositionTo(Constants.EN_LEWP);
+                mark = pv.resolveMarker(rm);
+                // this is probably not necessary since the RM will not be used again, but to be safe...
+                rm.changePositionTo(originalPosition);
                 if (mark != null) {
                     break;
                 }
@@ -275,11 +278,10 @@ public abstract class AbstractPageSequenceLayoutManager extends AbstractLayoutMa
     /**
      * Makes a new page
      *
-     * @param bIsBlank whether this page is blank or not
-     * @param bIsLast whether this page is the last page or not
+     * @param isBlank whether this page is blank or not
      * @return a new page
      */
-    protected Page makeNewPage(boolean isBlank, boolean isLast) {
+    protected Page makeNewPage(boolean isBlank) {
         if (curPage != null) {
             finishPage();
         }
@@ -293,6 +295,7 @@ public abstract class AbstractPageSequenceLayoutManager extends AbstractLayoutMa
                     + (isBlank ? "*" : "") + "]");
         }
 
+        addIDToPage(pageSeq.getRoot().getId());
         addIDToPage(pageSeq.getId());
         return curPage;
     }
@@ -359,27 +362,32 @@ public abstract class AbstractPageSequenceLayoutManager extends AbstractLayoutMa
 
         if (forcePageCount == Constants.EN_EVEN) {
             if ((currentPageNum - startPageNum + 1) % 2 != 0) { // we have an odd number of pages
-                curPage = makeNewPage(true, false);
+                curPage = makeNewPage(true);
             }
         } else if (forcePageCount == Constants.EN_ODD) {
             if ((currentPageNum - startPageNum + 1) % 2 == 0) { // we have an even number of pages
-                curPage = makeNewPage(true, false);
+                curPage = makeNewPage(true);
             }
         } else if (forcePageCount == Constants.EN_END_ON_EVEN) {
             if (currentPageNum % 2 != 0) { // we are now on an odd page
-                curPage = makeNewPage(true, false);
+                curPage = makeNewPage(true);
             }
         } else if (forcePageCount == Constants.EN_END_ON_ODD) {
             if (currentPageNum % 2 == 0) { // we are now on an even page
-                curPage = makeNewPage(true, false);
+                curPage = makeNewPage(true);
             }
-        } else if (forcePageCount == Constants.EN_NO_FORCE) {
+        } /*  else if (forcePageCount == Constants.EN_NO_FORCE) {
             // i hope: nothing special at all
-        }
+        } */
 
         if (curPage != null) {
             finishPage();
         }
+
+        while (forcePageCount != Constants.EN_NO_FORCE && getCurrentPageNum() < getLastPageNumber()) {
+            curPage = makeNewPage(true);
+            finishPage();
+        }
     }
 
     /** {@inheritDoc} */
@@ -387,4 +395,7 @@ public abstract class AbstractPageSequenceLayoutManager extends AbstractLayoutMa
         throw new IllegalStateException();
     }
 
+    protected int getLastPageNumber() {
+        return currentPageNum;
+    }
 }
