@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BlockLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
+/* $Id: BlockLayoutManager.java 1562429 2014-01-29 12:52:26Z vhennebert $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -31,6 +31,8 @@ import org.apache.fop.area.Area;
 import org.apache.fop.area.Block;
 import org.apache.fop.area.LineArea;
 import org.apache.fop.datatypes.Length;
+import org.apache.fop.fo.FONode;
+import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
 import org.apache.fop.fo.properties.KeepProperty;
 import org.apache.fop.fonts.Font;
 import org.apache.fop.fonts.FontInfo;
@@ -43,31 +45,22 @@ import org.apache.fop.traits.SpaceVal;
 /**
  * LayoutManager for a block FO.
  */
-public class BlockLayoutManager extends BlockStackingLayoutManager
-            implements ConditionalElementListener {
+public class BlockLayoutManager extends SpacedBorderedPaddedBlockLayoutManager
+        implements BreakOpportunity {
 
-    /**
-     * logging instance
-     */
+    /** logging instance */
     private static Log log = LogFactory.getLog(BlockLayoutManager.class);
 
     private Block curBlockArea;
 
     /** Iterator over the child layout managers. */
-    protected ListIterator proxyLMiter;
+    protected ListIterator<LayoutManager> proxyLMiter;
 
     private int lead = 12000;
     private Length lineHeight;
     private int follow = 2000;
     //private int middleShift = 0;
 
-    private boolean discardBorderBefore;
-    private boolean discardBorderAfter;
-    private boolean discardPaddingBefore;
-    private boolean discardPaddingAfter;
-    private MinOptMax effSpaceBefore;
-    private MinOptMax effSpaceAfter;
-
     /**
      * Creates a new BlockLayoutManager.
      * @param inBlock the block FO object to create the layout manager for.
@@ -78,52 +71,83 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     }
 
     /** {@inheritDoc} */
+    @Override
     public void initialize() {
         super.initialize();
-        FontInfo fi = getBlockFO().getFOEventHandler().getFontInfo();
-        FontTriplet[] fontkeys = getBlockFO().getCommonFont().getFontState(fi);
+        org.apache.fop.fo.flow.Block fo = getBlockFO();
+        FontInfo fi = fo.getFOEventHandler().getFontInfo();
+        FontTriplet[] fontkeys = fo.getCommonFont().getFontState(fi);
         Font initFont = fi.getFontInstance(fontkeys[0],
                 getBlockFO().getCommonFont().fontSize.getValue(this));
         lead = initFont.getAscender();
         follow = -initFont.getDescender();
         //middleShift = -fs.getXHeight() / 2;
-        lineHeight = getBlockFO().getLineHeight().getOptimum(this).getLength();
-        startIndent = getBlockFO().getCommonMarginBlock().startIndent.getValue(this);
-        endIndent = getBlockFO().getCommonMarginBlock().endIndent.getValue(this);
-        foSpaceBefore = new SpaceVal(getBlockFO().getCommonMarginBlock().spaceBefore, this)
-                            .getSpace();
-        foSpaceAfter = new SpaceVal(getBlockFO().getCommonMarginBlock().spaceAfter, this)
-                            .getSpace();
-        bpUnit = 0; // non-standard extension
-        if (bpUnit == 0) {
-            // use optimum space values
-            adjustedSpaceBefore = getBlockFO().getCommonMarginBlock().spaceBefore.getSpace()
-                                        .getOptimum(this).getLength().getValue(this);
-            adjustedSpaceAfter = getBlockFO().getCommonMarginBlock().spaceAfter.getSpace()
-                                        .getOptimum(this).getLength().getValue(this);
-        } else {
-            // use minimum space values
-            adjustedSpaceBefore = getBlockFO().getCommonMarginBlock().spaceBefore.getSpace()
-                                        .getMinimum(this).getLength().getValue(this);
-            adjustedSpaceAfter = getBlockFO().getCommonMarginBlock().spaceAfter.getSpace()
-                                        .getMinimum(this).getLength().getValue(this);
-        }
+        lineHeight = fo.getLineHeight().getOptimum(this).getLength();
+        startIndent = fo.getCommonMarginBlock().startIndent.getValue(this);
+        endIndent = fo.getCommonMarginBlock().endIndent.getValue(this);
+        foSpaceBefore = new SpaceVal(fo.getCommonMarginBlock().spaceBefore, this).getSpace();
+        foSpaceAfter = new SpaceVal(fo.getCommonMarginBlock().spaceAfter, this).getSpace();
+        // use optimum space values
+        adjustedSpaceBefore = fo.getCommonMarginBlock().spaceBefore.getSpace()
+                                    .getOptimum(this).getLength().getValue(this);
+        adjustedSpaceAfter = fo.getCommonMarginBlock().spaceAfter.getSpace()
+                                    .getOptimum(this).getLength().getValue(this);
+    }
+
+    @Override
+    protected CommonBorderPaddingBackground getCommonBorderPaddingBackground() {
+        return getBlockFO().getCommonBorderPaddingBackground();
     }
 
     /** {@inheritDoc} */
+    @Override
     public List getNextKnuthElements(LayoutContext context, int alignment) {
         return getNextKnuthElements(context, alignment, null, null, null);
     }
 
     /** {@inheritDoc} */
+    @Override
     public List getNextKnuthElements(LayoutContext context, int alignment, Stack lmStack,
             Position restartPosition, LayoutManager restartAtLM) {
         resetSpaces();
-        if (lmStack == null) {
-            return super.getNextKnuthElements(context, alignment);
+        return super.getNextKnuthElements(
+                context, alignment, lmStack, restartPosition, restartAtLM);
+    }
+
+    /**
+     * Overridden to take into account that the childLM may be the block's
+     * {@link LineLayoutManager}.
+     * {@inheritDoc}
+     */
+    @Override
+    protected List<ListElement> getNextChildElements(LayoutManager childLM, LayoutContext context,
+           LayoutContext childLC, int alignment, Stack lmStack, Position restartPosition,
+           LayoutManager restartAtLM) {
+
+        childLC.copyPendingMarksFrom(context);
+
+        if (childLM instanceof LineLayoutManager) {
+            childLC.setRefIPD(getContentAreaIPD());
         } else {
-            return super.getNextKnuthElements(context, alignment, lmStack, restartPosition,
-                    restartAtLM);
+            // nop; will have been properly set by makeChildLayoutContext()
+        }
+
+        if (childLM == this.childLMs.get(0)) {
+            childLC.setFlags(LayoutContext.SUPPRESS_BREAK_BEFORE);
+            //Handled already by the parent (break collapsing, see above)
+        }
+
+        if (lmStack == null) {
+            return childLM.getNextKnuthElements(childLC, alignment);
+        } else {
+            if (childLM instanceof LineLayoutManager) {
+                assert (restartPosition instanceof LeafPosition);
+                return ((LineLayoutManager) childLM).getNextKnuthElements(childLC, alignment,
+                        (LeafPosition) restartPosition);
+            } else {
+                return childLM.getNextKnuthElements(childLC, alignment,
+                        lmStack, restartPosition, restartAtLM);
+            }
         }
     }
 
@@ -151,7 +175,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
          */
         public ProxyLMiter() {
             super(BlockLayoutManager.this);
-            listLMs = new java.util.ArrayList(10);
+            listLMs = new java.util.ArrayList<LayoutManager>(10);
         }
 
         /**
@@ -166,7 +190,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
          * @return true if new child lms were added
          */
         protected boolean createNextChildLMs(int pos) {
-            List newLMs = createChildLMs(pos + 1 - listLMs.size());
+            List<LayoutManager> newLMs = createChildLMs(pos + 1 - listLMs.size());
             if (newLMs != null) {
                 listLMs.addAll(newLMs);
             }
@@ -174,13 +198,12 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
+    @Override
     public boolean createNextChildLMs(int pos) {
 
         while (proxyLMiter.hasNext()) {
-            LayoutManager lm = (LayoutManager) proxyLMiter.next();
+            LayoutManager lm = proxyLMiter.next();
             if (lm instanceof InlineLevelLayoutManager) {
                 LineLayoutManager lineLM = createLineManager(lm);
                 addChildLM(lineLM);
@@ -203,10 +226,10 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     private LineLayoutManager createLineManager(LayoutManager firstlm) {
         LineLayoutManager llm;
         llm = new LineLayoutManager(getBlockFO(), lineHeight, lead, follow);
-        List inlines = new java.util.ArrayList();
+        List<LayoutManager> inlines = new java.util.ArrayList<LayoutManager>();
         inlines.add(firstlm);
         while (proxyLMiter.hasNext()) {
-            LayoutManager lm = (LayoutManager) proxyLMiter.next();
+            LayoutManager lm = proxyLMiter.next();
             if (lm instanceof InlineLevelLayoutManager) {
                 inlines.add(lm);
             } else {
@@ -219,23 +242,26 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     }
 
     /** {@inheritDoc} */
+    @Override
     public KeepProperty getKeepTogetherProperty() {
         return getBlockFO().getKeepTogether();
     }
 
     /** {@inheritDoc} */
+    @Override
     public KeepProperty getKeepWithPreviousProperty() {
         return getBlockFO().getKeepWithPrevious();
     }
 
     /** {@inheritDoc} */
+    @Override
     public KeepProperty getKeepWithNextProperty() {
         return getBlockFO().getKeepWithNext();
     }
 
     /** {@inheritDoc} */
-    public void addAreas(PositionIterator parentIter,
-            LayoutContext layoutContext) {
+    @Override
+    public void addAreas(PositionIterator parentIter, LayoutContext layoutContext) {
         getParentArea(null);
 
         // if this will create the first block area in a page
@@ -246,7 +272,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
 
         LayoutManager childLM;
         LayoutManager lastLM = null;
-        LayoutContext lc = new LayoutContext(0);
+        LayoutContext lc = LayoutContext.offspringOf(layoutContext);
         lc.setSpaceAdjust(layoutContext.getSpaceAdjust());
         // set space after in the LayoutContext for children
         if (layoutContext.getSpaceAfter() > 0) {
@@ -256,14 +282,12 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
 
         // "unwrap" the NonLeafPositions stored in parentIter
         // and put them in a new list;
-        LinkedList positionList = new LinkedList();
+        LinkedList<Position> positionList = new LinkedList<Position>();
         Position pos;
-        boolean spaceBefore = false;
-        boolean spaceAfter = false;
         Position firstPos = null;
         Position lastPos = null;
         while (parentIter.hasNext()) {
-            pos = (Position) parentIter.next();
+            pos = parentIter.next();
             //log.trace("pos = " + pos.getClass().getName() + "; " + pos);
             if (pos.getIndex() >= 0) {
                 if (firstPos == null) {
@@ -276,109 +300,23 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
                 //Not all elements are wrapped
                 innerPosition = pos.getPosition();
             }
-            if (innerPosition == null) {
-                // pos was created by this BlockLM and was inside an element
-                // representing space before or after
-                // this means the space was not discarded
-                if (positionList.size() == 0) {
-                    // pos was in the element representing space-before
-                    spaceBefore = true;
-                    //log.trace(" space before");
-                } else {
-                    // pos was in the element representing space-after
-                    spaceAfter = true;
-                    //log.trace(" space-after");
-                }
-            } else if (innerPosition.getLM() == this
-                    && !(innerPosition instanceof MappingPosition)) {
-                // pos was created by this BlockLM and was inside a penalty
-                // allowing or forbidding a page break
-                // nothing to do
-                //log.trace(" penalty");
-            } else {
+
+            if (innerPosition != null
+                    && (innerPosition.getLM() != this
+                        || innerPosition instanceof MappingPosition)) {
                 // innerPosition was created by another LM
                 positionList.add(innerPosition);
                 lastLM = innerPosition.getLM();
-                //log.trace(" " + innerPosition.getClass().getName());
             }
         }
 
         addId();
 
-        addMarkersToPage(true, isFirst(firstPos), isLast(lastPos));
+        registerMarkers(true, isFirst(firstPos), isLast(lastPos));
 
-        if (bpUnit == 0) {
-            // the Positions in positionList were inside the elements
-            // created by the LineLM
-            childPosIter = new StackingIter(positionList.listIterator());
-        } else {
-            // the Positions in positionList were inside the elements
-            // created by the BlockLM in the createUnitElements() method
-            //if (((Position) positionList.getLast()) instanceof
-                  // LeafPosition) {
-            //    // the last item inside positionList is a LeafPosition
-            //    // (a LineBreakPosition, more precisely); this means that
-            //    // the whole paragraph is on the same page
-            //    childPosIter = new KnuthPossPosIter(storedList, 0,
-                  // storedList.size());
-            //} else {
-            //    // the last item inside positionList is a Position;
-            //    // this means that the paragraph has been split
-            //    // between consecutive pages
-            LinkedList splitList = new LinkedList();
-            int splitLength = 0;
-            int iFirst = ((MappingPosition) positionList.getFirst()).getFirstIndex();
-            int iLast = ((MappingPosition) positionList.getLast()).getLastIndex();
-            // copy from storedList to splitList all the elements from
-            // iFirst to iLast
-            ListIterator storedListIterator = storedList.listIterator(iFirst);
-            while (storedListIterator.nextIndex() <= iLast) {
-                KnuthElement element = (KnuthElement) storedListIterator
-                        .next();
-                // some elements in storedList (i.e. penalty items) were created
-                // by this BlockLM, and must be ignored
-                if (element.getLayoutManager() != this) {
-                    splitList.add(element);
-                    splitLength += element.getWidth();
-                    lastLM = element.getLayoutManager();
-                }
-            }
-            //log.debug("Adding areas from " + iFirst + " to " + iLast);
-            //log.debug("splitLength= " + splitLength
-            //                   + " (" + neededUnits(splitLength) + " units') "
-            //                   + (neededUnits(splitLength) * bpUnit - splitLength)
-            //                   + " spacing");
-            // add space before and / or after the paragraph
-            // to reach a multiple of bpUnit
-            if (spaceBefore && spaceAfter) {
-                foSpaceBefore = new SpaceVal(getBlockFO()
-                                    .getCommonMarginBlock().spaceBefore, this).getSpace();
-                foSpaceAfter = new SpaceVal(getBlockFO()
-                                    .getCommonMarginBlock().spaceAfter, this).getSpace();
-                adjustedSpaceBefore = (neededUnits(splitLength
-                        + foSpaceBefore.getMin()
-                        + foSpaceAfter.getMin())
-                        * bpUnit - splitLength) / 2;
-                adjustedSpaceAfter = neededUnits(splitLength
-                        + foSpaceBefore.getMin()
-                        + foSpaceAfter.getMin())
-                        * bpUnit - splitLength - adjustedSpaceBefore;
-                } else if (spaceBefore) {
-                adjustedSpaceBefore = neededUnits(splitLength
-                        + foSpaceBefore.getMin())
-                        * bpUnit - splitLength;
-                } else {
-                adjustedSpaceAfter = neededUnits(splitLength
-                        + foSpaceAfter.getMin())
-                        * bpUnit - splitLength;
-                }
-            //log.debug("spazio prima = " + adjustedSpaceBefore
-                  // + " spazio dopo = " + adjustedSpaceAfter + " totale = " +
-                  // (adjustedSpaceBefore + adjustedSpaceAfter + splitLength));
-            childPosIter = new KnuthPossPosIter(splitList, 0, splitList
-                    .size());
-            //}
-        }
+        // the Positions in positionList were inside the elements
+        // created by the LineLM
+        childPosIter = new PositionIterator(positionList.listIterator());
 
         while ((childLM = childPosIter.getNextChildLM()) != null) {
             // set last area flag
@@ -389,9 +327,8 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
             childLM.addAreas(childPosIter, lc);
         }
 
-        addMarkersToPage(false, isFirst(firstPos), isLast(lastPos));
+        registerMarkers(false, isFirst(firstPos), isLast(lastPos));
 
-        TraitSetter.addPtr(curBlockArea, getBlockFO().getPtr());  // used for accessibility
         TraitSetter.addSpaceBeforeAfter(curBlockArea, layoutContext.getSpaceAdjust(),
                 effSpaceBefore, effSpaceAfter);
         flush();
@@ -415,12 +352,15 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
      * @param childArea area to get the parent area for
      * @return the parent area
      */
+    @Override
     public Area getParentArea(Area childArea) {
         if (curBlockArea == null) {
             curBlockArea = new Block();
 
             curBlockArea.setIPD(super.getContentAreaIPD());
 
+            curBlockArea.setBidiLevel(getBlockFO().getBidiLevelRecursive());
+
             TraitSetter.addBreaks(curBlockArea,
                     getBlockFO().getBreakBefore(), getBlockFO().getBreakAfter());
 
@@ -440,15 +380,17 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
                     getBlockFO().getCommonBorderPaddingBackground(),
                     startIndent, endIndent,
                     this);
+            TraitSetter.setLayer(curBlockArea, getBlockFO().getLayer());
 
+            curBlockArea.setLocale(getBlockFO().getCommonHyphenation().getLocale());
+            curBlockArea.setLocation(FONode.getLocatorString(getBlockFO().getLocator()));
             setCurrentArea(curBlockArea); // ??? for generic operations
         }
         return curBlockArea;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
+    @Override
     public void addChildArea(Area childArea) {
         if (curBlockArea != null) {
             if (childArea instanceof LineArea) {
@@ -463,6 +405,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
      * Force current area to be added to parent area.
      * {@inheritDoc}
      */
+    @Override
     protected void flush() {
         if (curBlockArea != null) {
             TraitSetter.addBackground(curBlockArea,
@@ -486,6 +429,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
      * Returns the IPD of the content area
      * @return the IPD of the content area
      */
+    @Override
     public int getContentAreaIPD() {
         if (curBlockArea != null) {
             return curBlockArea.getIPD();
@@ -498,6 +442,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
      * Returns the BPD of the content area
      * @return the BPD of the content area
      */
+    @Override
     public int getContentAreaBPD() {
         if (curBlockArea != null) {
             return curBlockArea.getBPD();
@@ -505,62 +450,16 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         return -1;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
+    @Override
     public boolean getGeneratesBlockArea() {
         return true;
     }
 
     /** {@inheritDoc} */
-    public void notifySpace(RelSide side, MinOptMax effectiveLength) {
-        if (RelSide.BEFORE == side) {
-            if (log.isDebugEnabled()) {
-                log.debug(this + ": Space " + side + ", "
-                        + this.effSpaceBefore + "-> " + effectiveLength);
-            }
-            this.effSpaceBefore = effectiveLength;
-        } else {
-            if (log.isDebugEnabled()) {
-                log.debug(this + ": Space " + side + ", "
-                        + this.effSpaceAfter + "-> " + effectiveLength);
-            }
-            this.effSpaceAfter = effectiveLength;
-        }
-    }
-
-    /** {@inheritDoc} */
-    public void notifyBorder(RelSide side, MinOptMax effectiveLength) {
-        if (effectiveLength == null) {
-            if (RelSide.BEFORE == side) {
-                this.discardBorderBefore = true;
-            } else {
-                this.discardBorderAfter = true;
-            }
-        }
-        if (log.isDebugEnabled()) {
-            log.debug(this + ": Border " + side + " -> " + effectiveLength);
-        }
-    }
-
-    /** {@inheritDoc} */
-    public void notifyPadding(RelSide side, MinOptMax effectiveLength) {
-        if (effectiveLength == null) {
-            if (RelSide.BEFORE == side) {
-                this.discardPaddingBefore = true;
-            } else {
-                this.discardPaddingAfter = true;
-            }
-        }
-        if (log.isDebugEnabled()) {
-            log.debug(this + ": Padding " + side + " -> " + effectiveLength);
-        }
-    }
-
-    /** {@inheritDoc} */
+    @Override
     public boolean isRestartable() {
         return true;
     }
 
 }
-
