@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BlockLevelEventProducer.java 1327157 2012-04-17 16:15:44Z gadams $ */
+/* $Id: BlockLevelEventProducer.java 1723297 2016-01-06 12:13:52Z ssteiner $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -43,8 +43,7 @@ public interface BlockLevelEventProducer extends EventProducer {
          * @return the event producer
          */
         public static BlockLevelEventProducer get(EventBroadcaster broadcaster) {
-            return (BlockLevelEventProducer)broadcaster.getEventProducerFor(
-                    BlockLevelEventProducer.class);
+            return broadcaster.getEventProducerFor(BlockLevelEventProducer.class);
         }
     }
 
@@ -131,6 +130,23 @@ public interface BlockLevelEventProducer extends EventProducer {
             int amount, boolean clip, boolean canRecover,
             Locator loc) throws LayoutException;
 
+    /**
+     * Contents overflow a static region viewport.
+     * @param source the event source
+     * @param elementName the formatting object
+     * @param page the page number/name where the overflow happened
+     * @param amount the amount by which the contents overflow (in mpt)
+     * @param clip true if the content will be clipped
+     * @param canRecover indicates whether FOP can recover from this problem and continue working
+     * @param loc the location of the error or null
+     * @throws LayoutException the layout error provoked by the method call
+     * @event.severity FATAL
+     */
+    void staticRegionOverflow(Object source, String elementName,
+            String page,
+            int amount, boolean clip, boolean canRecover,
+            Locator loc) throws LayoutException;
+
     /**
      * Indicates that FOP doesn't support flows that are not mapped to region-body, yet.
      * @param source the event source
@@ -186,4 +202,21 @@ public interface BlockLevelEventProducer extends EventProducer {
      * @event.severity WARN
      */
     void nonRestartableContentFlowingToNarrowerPage(Object source);
+
+    /**
+     * A feasible layout has reached the given number of parts (columns or pages).
+     *
+     * @param source the event source
+     * @param partCount the number of parts that the layout has reached
+     * @event.severity INFO
+     */
+    void layoutHasReachedParts(Object source, int partCount);
+
+    /**
+     * Last page master reference missing.
+     *
+     * @param source the event source
+     * @event.severity WARN
+     */
+    void lastPageMasterReferenceMissing(Object source, Locator loc);
 }
