@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BreakingAlgorithm.java 1297008 2012-03-05 11:19:47Z vhennebert $ */
+/* $Id: BreakingAlgorithm.java 1642793 2014-12-02 00:29:57Z lbernardo $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -123,9 +123,9 @@ public abstract class BreakingAlgorithm {
     /** Force the algorithm to find a set of breakpoints, even if no feasible breakpoints
      * exist.
      */
-    private boolean force =  false;
+    private boolean force;
     /** If set to true, doesn't ignore break possibilities which are definitely too short. */
-    protected boolean considerTooShort = false;
+    protected boolean considerTooShort;
 
     /** When in forced mode, the best node leading to a too long line. The line will be
      * too long anyway, but this one will lead to a paragraph with fewest demerits.
@@ -162,12 +162,12 @@ public abstract class BreakingAlgorithm {
     /**
      * The lowest available line in the set of active nodes.
      */
-    protected int startLine = 0;
+    protected int startLine;
 
     /**
      * The highest + 1 available line in the set of active nodes.
      */
-    protected int endLine = 0;
+    protected int endLine;
 
     /**
      * The total width of all elements handled so far.
@@ -177,12 +177,12 @@ public abstract class BreakingAlgorithm {
     /**
      * The total stretch of all elements handled so far.
      */
-    protected int totalStretch = 0;
+    protected int totalStretch;
 
     /**
      * The total shrink of all elements handled so far.
      */
-    protected int totalShrink = 0;
+    protected int totalShrink;
 
     /**
      * Best records.
@@ -226,49 +226,49 @@ public abstract class BreakingAlgorithm {
      */
     public class KnuthNode {
         /** index of the breakpoint represented by this node */
-        public final int position;                              // CSOK: VisibilityModifier
+        public final int position;
 
         /** number of the line ending at this breakpoint */
-        public final int line;                                  // CSOK: VisibilityModifier
+        public final int line;
 
         /** fitness class of the line ending at this breakpoint. One of 0, 1, 2, 3. */
-        public final int fitness;                               // CSOK: VisibilityModifier
+        public final int fitness;
 
         /** accumulated width of the KnuthElements up to after this breakpoint. */
-        public final int totalWidth;                            // CSOK: VisibilityModifier
+        public final int totalWidth;
 
         /** accumulated stretchability of the KnuthElements up to after this breakpoint. */
-        public final int totalStretch;                          // CSOK: VisibilityModifier
+        public final int totalStretch;
 
         /** accumulated shrinkability of the KnuthElements up to after this breakpoint. */
-        public final int totalShrink;                           // CSOK: VisibilityModifier
+        public final int totalShrink;
 
         /** adjustment ratio if the line ends at this breakpoint */
-        public final double adjustRatio;                        // CSOK: VisibilityModifier
+        public final double adjustRatio;
 
         /** available stretch of the line ending at this breakpoint */
-        public final int availableShrink;                       // CSOK: VisibilityModifier
+        public final int availableShrink;
 
         /** available shrink of the line ending at this breakpoint */
-        public final int availableStretch;                      // CSOK: VisibilityModifier
+        public final int availableStretch;
 
         /** difference between target and actual line width */
-        public final int difference;                            // CSOK: VisibilityModifier
+        public final int difference;
 
         /** minimum total demerits up to this breakpoint */
-        public double totalDemerits;                            // CSOK: VisibilityModifier
+        public double totalDemerits;
 
         /** best node for the preceding breakpoint */
-        public KnuthNode previous;                              // CSOK: VisibilityModifier
+        public KnuthNode previous;
 
         /** next possible node in the same line */
-        public KnuthNode next;                                  // CSOK: VisibilityModifier
+        public KnuthNode next;
 
         /**
          * Holds the number of subsequent recovery attempty that are made to get content fit
          * into a line.
          */
-        public int fitRecoveryCounter = 0;                      // CSOK: VisibilityModifier
+        public int fitRecoveryCounter;
 
         /**
          * Construct node.
@@ -285,8 +285,7 @@ public abstract class BreakingAlgorithm {
          * @param totalDemerits a real number
          * @param previous a node
          */
-        public KnuthNode(                                        // CSOK: ParameterNumber
-                int position, int line, int fitness,
+        public KnuthNode(int position, int line, int fitness,
                 int totalWidth, int totalStretch, int totalShrink,
                 double adjustRatio, int availableShrink, int availableStretch,
                 int difference, double totalDemerits, KnuthNode previous) {
@@ -320,12 +319,12 @@ public abstract class BreakingAlgorithm {
     protected class BestRecords {
         private static final double INFINITE_DEMERITS = Double.POSITIVE_INFINITY;
 
-        private double[] bestDemerits = new double[4];
-        private KnuthNode[] bestNode = new KnuthNode[4];
-        private double[] bestAdjust = new double[4];
-        private int[] bestDifference = new int[4];
-        private int[] bestAvailableShrink = new int[4];
-        private int[] bestAvailableStretch = new int[4];
+        private final double[] bestDemerits = new double[4];
+        private final KnuthNode[] bestNode = new KnuthNode[4];
+        private final double[] bestAdjust = new double[4];
+        private final int[] bestDifference = new int[4];
+        private final int[] bestAvailableShrink = new int[4];
+        private final int[] bestAvailableStretch = new int[4];
         /** Points to the fitness class which currently leads to the best demerits. */
         private int bestIndex = -1;
 
@@ -460,6 +459,10 @@ public abstract class BreakingAlgorithm {
         return this.partOverflowRecoveryActivated;
     }
 
+    protected KnuthNode getLastTooLong() {
+        return lastTooLong;
+    }
+
     /**
      * Empty method, hook for subclasses. Called before determining the optimal
      * breakpoints corresponding to a given active node.
@@ -532,14 +535,15 @@ public abstract class BreakingAlgorithm {
         // index of the first KnuthBox in the sequence, in case of non-centered
         // alignment. For centered alignment, we need to take into account preceding
         // penalties+glues used for the filler spaces
-        int firstBoxIndex = startIndex;
+        int previousPosition = startIndex;
         if (alignment != Constants.EN_CENTER) {
-            firstBoxIndex = par.getFirstBoxIndex(startIndex);
+            int firstBoxIndex = par.getFirstBoxIndex(startIndex);
+            previousPosition = (firstBoxIndex >= par.size()) ? startIndex : firstBoxIndex - 1;
         }
-        firstBoxIndex = (firstBoxIndex < 0) ? 0 : firstBoxIndex;
+        previousPosition = (previousPosition < 0) ? 0 : previousPosition;
 
         // create an active node representing the starting point
-        addNode(0, createNode(firstBoxIndex, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, null));
+        addNode(0, createNode(previousPosition, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, null));
         KnuthNode lastForced = getNode(0);
 
         if (log.isTraceEnabled()) {
@@ -554,6 +558,9 @@ public abstract class BreakingAlgorithm {
                     elementIndex, previousIsBox, allowedBreaks).isBox();
 
             if (activeNodeCount == 0) {
+                if (handlingFloat()) {
+                    return handleFloat();
+                }
                 if (getIPDdifference() != 0) {
                     return handleIpdChange();
                 }
@@ -681,8 +688,7 @@ public abstract class BreakingAlgorithm {
      * @param previous active node for the preceding breakpoint
      * @return a new node
      */
-    protected KnuthNode createNode(                              // CSOK: ParameterNumber
-            int position, int line, int fitness,
+    protected KnuthNode createNode(int position, int line, int fitness,
             int totalWidth, int totalStretch, int totalShrink,
             double adjustRatio, int availableShrink, int availableStretch,
             int difference, double totalDemerits, KnuthNode previous) {
@@ -942,8 +948,11 @@ public abstract class BreakingAlgorithm {
                     log.trace("\tline=" + line);
                 }
 
+                if (element.isForcedBreak() && handlingFloat()) {
+                    disableFloatHandling(); // so that we do not create a float edge position later
+                }
                 // The line would be too long.
-                if (r < -1 || element.isForcedBreak()) {
+                if (r < -1 || element.isForcedBreak() || handlingFloat()) {
                     deactivateNode(node, line);
                 }
 
@@ -985,14 +994,14 @@ public abstract class BreakingAlgorithm {
      * @param node  the node
      * @param line  the line number
      * @param elementIdx    the position index of the element
-     * @param difference    the difference between content-length and avaialable width
+     * @param difference    the difference between content-length and available width
      * @param r     the adjustment ratio
      * @param demerits  demerits produced by the node
      * @param fitnessClass  the fitness class
      * @param availableShrink   the available amount of shrink
      * @param availableStretch  tha available amount of stretch
      */
-    protected void forceNode(KnuthNode node,                    // CSOK: ParameterNumber
+    protected void forceNode(KnuthNode node,
                              int line,
                              int elementIdx,
                              int difference,
@@ -1025,6 +1034,13 @@ public abstract class BreakingAlgorithm {
             }
         }
 
+        createForcedNodes(node, line, elementIdx, difference, r, demerits, fitnessClass, availableShrink,
+                availableStretch, newWidth, newStretch, newShrink);
+    }
+
+    protected void createForcedNodes(KnuthNode node, int line, int elementIdx, int difference, double r,
+            double demerits, int fitnessClass, int availableShrink, int availableStretch, int newWidth,
+            int newStretch, int newShrink) {
         if (r <= -1) {
             log.debug("Considering tooLong, demerits=" + demerits);
             if (lastTooLong == null || demerits < lastTooLong.totalDemerits) {
@@ -1039,10 +1055,9 @@ public abstract class BreakingAlgorithm {
         } else {
             if (lastTooShort == null || demerits <= lastTooShort.totalDemerits) {
                 if (considerTooShort) {
-                    //consider possibilities which are too short
-                    best.addRecord(demerits, node, r,
-                            availableShrink, availableStretch,
-                            difference, fitnessClass);
+                    // consider possibilities which are too short
+                    best.addRecord(demerits, node, r, availableShrink, availableStretch, difference,
+                            fitnessClass);
                 }
                 lastTooShort = createNode(elementIdx, line + 1, fitnessClass,
                         newWidth, newStretch, newShrink,
@@ -1448,4 +1463,15 @@ public abstract class BreakingAlgorithm {
         return this.alignmentLast;
     }
 
+    protected boolean handlingFloat() {
+        return false;
+    }
+
+    protected int handleFloat() {
+        throw new IllegalStateException();
+    }
+
+    protected void disableFloatHandling() {
+        throw new IllegalStateException();
+    }
 }
