@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: LMiter.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: LMiter.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -23,28 +23,37 @@ import java.util.List;
 import java.util.ListIterator;
 import java.util.NoSuchElementException;
 
-public class LMiter implements ListIterator {
+/** An iterator for layout managers. */
+public class LMiter implements ListIterator<LayoutManager> {
 
-
-    protected List listLMs;
-    protected int curPos = 0;
+    /** list of layout managers */
+    protected List<LayoutManager> listLMs;
+    /** current position in iteration */
+    protected int curPos;
     /** The LayoutManager to which this LMiter is attached **/
     private LayoutManager lp;
 
+    /**
+     * Construct a layout manager iterator.
+     * @param lp the associated layout manager (parent)
+     */
     public LMiter(LayoutManager lp) {
         this.lp = lp;
         listLMs = lp.getChildLMs();
     }
 
+    /** {@inheritDoc} */
     public boolean hasNext() {
-        return (curPos < listLMs.size()) ? true : lp.createNextChildLMs(curPos);
+        return (curPos < listLMs.size()) || lp.createNextChildLMs(curPos);
     }
 
+    /** {@inheritDoc} */
     public boolean hasPrevious() {
         return (curPos > 0);
     }
 
-    public Object previous() throws NoSuchElementException {
+    /** {@inheritDoc} */
+    public LayoutManager previous() throws NoSuchElementException {
         if (curPos > 0) {
             return listLMs.get(--curPos);
         } else {
@@ -52,7 +61,8 @@ public class LMiter implements ListIterator {
         }
     }
 
-    public Object next() throws NoSuchElementException {
+    /** {@inheritDoc} */
+    public LayoutManager next() throws NoSuchElementException {
         if (curPos < listLMs.size()) {
             return listLMs.get(curPos++);
         } else {
@@ -60,7 +70,8 @@ public class LMiter implements ListIterator {
         }
     }
 
-    public void remove() throws NoSuchElementException {
+    /** {@inheritDoc} */
+     public void remove() throws NoSuchElementException {
         if (curPos > 0) {
             listLMs.remove(--curPos);
             // Note: doesn't actually remove it from the base!
@@ -70,18 +81,22 @@ public class LMiter implements ListIterator {
     }
 
 
-    public void add(Object o) throws UnsupportedOperationException {
+    /** {@inheritDoc} */
+   public void add(LayoutManager lm) throws UnsupportedOperationException {
         throw new UnsupportedOperationException("LMiter doesn't support add");
     }
 
-    public void set(Object o) throws UnsupportedOperationException {
+    /** {@inheritDoc} */
+    public void set(LayoutManager lm) throws UnsupportedOperationException {
         throw new UnsupportedOperationException("LMiter doesn't support set");
     }
 
+    /** {@inheritDoc} */
     public int nextIndex() {
         return curPos;
     }
 
+    /** {@inheritDoc} */
     public int previousIndex() {
         return curPos - 1;
     }
