@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: LayoutManagerMapping.java 680369 2008-07-28 14:24:51Z jeremias $ */
+/* $Id: LayoutManagerMapping.java 1642793 2014-12-02 00:29:57Z lbernardo $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -31,6 +31,7 @@ import org.apache.commons.logging.LogFactory;
 import org.apache.fop.area.AreaTreeHandler;
 import org.apache.fop.fo.FOElementMapping;
 import org.apache.fop.fo.FONode;
+import org.apache.fop.fo.FONode.FONodeIterator;
 import org.apache.fop.fo.FOText;
 import org.apache.fop.fo.FObjMixed;
 import org.apache.fop.fo.extensions.ExternalDocument;
@@ -40,6 +41,7 @@ import org.apache.fop.fo.flow.Block;
 import org.apache.fop.fo.flow.BlockContainer;
 import org.apache.fop.fo.flow.Character;
 import org.apache.fop.fo.flow.ExternalGraphic;
+import org.apache.fop.fo.flow.Float;
 import org.apache.fop.fo.flow.Footnote;
 import org.apache.fop.fo.flow.Inline;
 import org.apache.fop.fo.flow.InlineContainer;
@@ -48,6 +50,8 @@ import org.apache.fop.fo.flow.InstreamForeignObject;
 import org.apache.fop.fo.flow.Leader;
 import org.apache.fop.fo.flow.ListBlock;
 import org.apache.fop.fo.flow.ListItem;
+import org.apache.fop.fo.flow.MultiCase;
+import org.apache.fop.fo.flow.MultiSwitch;
 import org.apache.fop.fo.flow.PageNumber;
 import org.apache.fop.fo.flow.PageNumberCitation;
 import org.apache.fop.fo.flow.PageNumberCitationLast;
@@ -71,10 +75,10 @@ import org.apache.fop.layoutmgr.inline.BidiLayoutManager;
 import org.apache.fop.layoutmgr.inline.CharacterLayoutManager;
 import org.apache.fop.layoutmgr.inline.ContentLayoutManager;
 import org.apache.fop.layoutmgr.inline.ExternalGraphicLayoutManager;
+import org.apache.fop.layoutmgr.inline.FloatLayoutManager;
 import org.apache.fop.layoutmgr.inline.FootnoteLayoutManager;
-import org.apache.fop.layoutmgr.inline.ICLayoutManager;
+import org.apache.fop.layoutmgr.inline.InlineContainerLayoutManager;
 import org.apache.fop.layoutmgr.inline.InlineLayoutManager;
-import org.apache.fop.layoutmgr.inline.InlineLevelLayoutManager;
 import org.apache.fop.layoutmgr.inline.InstreamForeignObjectLM;
 import org.apache.fop.layoutmgr.inline.LeaderLayoutManager;
 import org.apache.fop.layoutmgr.inline.PageNumberCitationLastLayoutManager;
@@ -93,11 +97,12 @@ import org.apache.fop.util.CharUtilities;
 public class LayoutManagerMapping implements LayoutManagerMaker {
 
     /** logging instance */
-    protected static Log log = LogFactory.getLog(LayoutManagerMapping.class);
+    private static final Log LOG = LogFactory.getLog(LayoutManagerMapping.class);
 
     /** The map of LayoutManagerMakers */
-    private Map makers = new HashMap();
+    private final Map makers = new HashMap();
 
+    /** default constructor */
     public LayoutManagerMapping() {
         initialize();
     }
@@ -110,14 +115,14 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         registerMaker(FObjMixed.class, new Maker());
         registerMaker(BidiOverride.class, new BidiOverrideLayoutManagerMaker());
         registerMaker(Inline.class, new InlineLayoutManagerMaker());
-        registerMaker(Footnote.class, new FootnodeLayoutManagerMaker());
+        registerMaker(Footnote.class, new FootnoteLayoutManagerMaker());
         registerMaker(InlineContainer.class,
                    new InlineContainerLayoutManagerMaker());
         registerMaker(BasicLink.class, new BasicLinkLayoutManagerMaker());
         registerMaker(Block.class, new BlockLayoutManagerMaker());
         registerMaker(Leader.class, new LeaderLayoutManagerMaker());
         registerMaker(RetrieveMarker.class, new RetrieveMarkerLayoutManagerMaker());
-        registerMaker(RetrieveTableMarker.class, new Maker());
+        registerMaker(RetrieveTableMarker.class, new RetrieveTableMarkerLayoutManagerMaker());
         registerMaker(Character.class, new CharacterLayoutManagerMaker());
         registerMaker(ExternalGraphic.class,
                    new ExternalGraphicLayoutManagerMaker());
@@ -141,6 +146,9 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         registerMaker(TableHeader.class, new Maker());
         registerMaker(Wrapper.class, new WrapperLayoutManagerMaker());
         registerMaker(Title.class, new InlineLayoutManagerMaker());
+        registerMaker(MultiCase.class, new MultiCaseLayoutManagerMaker());
+        registerMaker(MultiSwitch.class, new MultiSwitchLayoutManagerMaker());
+        registerMaker(Float.class, new FloatLayoutManagerMaker());
     }
 
     /**
@@ -152,17 +160,15 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         makers.put(clazz, maker);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void makeLayoutManagers(FONode node, List lms) {
         Maker maker = (Maker) makers.get(node.getClass());
         if (maker == null) {
             if (FOElementMapping.URI.equals(node.getNamespaceURI())) {
-                log.error("No LayoutManager maker for class " + node.getClass());
+                LOG.error("No LayoutManager maker for class " + node.getClass());
             } else {
-                if (log.isDebugEnabled()) {
-                    log.debug("Skipping the creation of a layout manager for " + node.getClass());
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Skipping the creation of a layout manager for " + node.getClass());
                 }
             }
         } else {
@@ -170,9 +176,7 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public LayoutManager makeLayoutManager(FONode node) {
         List lms = new ArrayList();
         makeLayoutManagers(node, lms);
@@ -188,30 +192,31 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         return (LayoutManager) lms.get(0);
     }
 
+    /** {@inheritDoc} */
     public PageSequenceLayoutManager makePageSequenceLayoutManager(
         AreaTreeHandler ath, PageSequence ps) {
         return new PageSequenceLayoutManager(ath, ps);
     }
 
-    /*
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
+    public ExternalDocumentLayoutManager makeExternalDocumentLayoutManager(
+        AreaTreeHandler ath, ExternalDocument ed) {
+        return new ExternalDocumentLayoutManager(ath, ed);
+    }
+
+    /** {@inheritDoc} */
     public FlowLayoutManager makeFlowLayoutManager(
             PageSequenceLayoutManager pslm, Flow flow) {
         return new FlowLayoutManager(pslm, flow);
     }
 
-    /*
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public ContentLayoutManager makeContentLayoutManager(PageSequenceLayoutManager pslm,
                                                          Title title) {
         return new ContentLayoutManager(pslm, title);
     }
 
-    /*
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public StaticContentLayoutManager makeStaticContentLayoutManager(
             PageSequenceLayoutManager pslm, StaticContent sc, SideRegion reg) {
         return new StaticContentLayoutManager(pslm, sc, reg);
@@ -223,13 +228,21 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         return new StaticContentLayoutManager(pslm, sc, block);
     }
 
+    /** a layout manager maker base class */
     public static class Maker {
+        /**
+         * Create a layout manager.
+         * @param node the associated FO node
+         * @param lms a list of layout managers to which new manager is to be added
+         */
         public void make(FONode node, List lms) {
             // no layout manager
         }
     }
 
+    /** a layout manager maker */
     public static class FOTextLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
             FOText foText = (FOText) node;
             if (foText.length() > 0) {
@@ -238,71 +251,67 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         }
     }
 
+    /** a layout manager maker */
     public static class BidiOverrideLayoutManagerMaker extends Maker {
-        // public static class BidiOverrideLayoutManagerMaker extends FObjMixedLayoutManagerMaker {
-        public void make(BidiOverride node, List lms) {
-            if (false) {
-                // this is broken; it does nothing
-                // it should make something like an InlineStackingLM
-                super.make(node, lms);
-            } else {
-                ArrayList childList = new ArrayList();
-                // this is broken; it does nothing
-                // it should make something like an InlineStackingLM
-                super.make(node, childList);
-                for (int count = childList.size() - 1; count >= 0; count--) {
-                    LayoutManager lm = (LayoutManager) childList.get(count);
-                    if (lm instanceof InlineLevelLayoutManager) {
-                        LayoutManager blm = new BidiLayoutManager
-                            (node, (InlineLayoutManager) lm);
-                        lms.add(blm);
-                    } else {
-                        lms.add(lm);
-                    }
-                }
+        /** {@inheritDoc} */
+        public void make(FONode node, List lms) {
+            if (node instanceof BidiOverride) {
+                lms.add(new BidiLayoutManager((BidiOverride) node));
             }
         }
     }
 
+    /** a layout manager maker */
     public static class InlineLayoutManagerMaker extends Maker {
-         public void make(FONode node, List lms) {
-             lms.add(new InlineLayoutManager((InlineLevel) node));
-         }
+        /** {@inheritDoc} */
+        public void make(FONode node, List lms) {
+            lms.add(new InlineLayoutManager((InlineLevel) node));
+        }
     }
 
-    public static class FootnodeLayoutManagerMaker extends Maker {
+    /** a layout manager maker */
+    public static class FootnoteLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
             lms.add(new FootnoteLayoutManager((Footnote) node));
         }
     }
 
+    /** a layout manager maker */
     public static class InlineContainerLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
-            ArrayList childList = new ArrayList();
-            super.make(node, childList);
-            lms.add(new ICLayoutManager((InlineContainer) node, childList));
+            lms.add(new InlineContainerLayoutManager((InlineContainer) node));
         }
     }
 
+    /** a layout manager maker */
     public static class BasicLinkLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
             lms.add(new BasicLinkLayoutManager((BasicLink) node));
         }
     }
 
+    /** a layout manager maker */
     public static class BlockLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
          public void make(FONode node, List lms) {
              lms.add(new BlockLayoutManager((Block) node));
          }
     }
 
+    /** a layout manager maker */
     public static class LeaderLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
             lms.add(new LeaderLayoutManager((Leader) node));
         }
     }
 
+    /** a layout manager maker */
     public static class CharacterLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
             Character foCharacter = (Character) node;
             if (foCharacter.getCharacter() != CharUtilities.CODE_EOT) {
@@ -311,7 +320,9 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         }
     }
 
+    /** a layout manager maker */
     public static class ExternalGraphicLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
             ExternalGraphic eg = (ExternalGraphic) node;
             if (!eg.getSrc().equals("")) {
@@ -320,49 +331,65 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         }
     }
 
+    /** a layout manager maker */
     public static class BlockContainerLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
             lms.add(new BlockContainerLayoutManager((BlockContainer) node));
          }
     }
 
+    /** a layout manager maker */
     public static class ListItemLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
          public void make(FONode node, List lms) {
              lms.add(new ListItemLayoutManager((ListItem) node));
          }
     }
 
+    /** a layout manager maker */
     public static class ListBlockLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
             lms.add(new ListBlockLayoutManager((ListBlock) node));
         }
     }
 
+    /** a layout manager maker */
     public static class InstreamForeignObjectLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
             lms.add(new InstreamForeignObjectLM((InstreamForeignObject) node));
         }
     }
 
+    /** a layout manager maker */
     public static class PageNumberLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
          public void make(FONode node, List lms) {
              lms.add(new PageNumberLayoutManager((PageNumber) node));
          }
     }
 
+    /** a layout manager maker */
     public static class PageNumberCitationLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
          public void make(FONode node, List lms) {
             lms.add(new PageNumberCitationLayoutManager((PageNumberCitation) node));
          }
     }
 
+    /** a layout manager maker */
     public static class PageNumberCitationLastLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
            lms.add(new PageNumberCitationLastLayoutManager((PageNumberCitationLast) node));
         }
     }
 
+    /** a layout manager maker */
     public static class TableLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
             Table table = (Table) node;
             TableLayoutManager tlm = new TableLayoutManager(table);
@@ -370,7 +397,9 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         }
     }
 
+    /** a layout manager maker */
     public class RetrieveMarkerLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
             Iterator baseIter;
             baseIter = node.getChildNodes();
@@ -384,7 +413,27 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         }
     }
 
+    public class RetrieveTableMarkerLayoutManagerMaker extends Maker {
+        public void make(FONode node, List lms) {
+            FONodeIterator baseIter = node.getChildNodes();
+            if (baseIter == null) {
+                // this happens when the retrieve-table-marker cannot be resolved yet
+                RetrieveTableMarker rtm = (RetrieveTableMarker) node;
+                RetrieveTableMarkerLayoutManager rtmlm = new RetrieveTableMarkerLayoutManager(rtm);
+                lms.add(rtmlm);
+                return;
+            }
+            while (baseIter.hasNext()) {
+                // this happens when the retrieve-table-marker has been resolved
+                FONode child = (FONode) baseIter.next();
+                makeLayoutManagers(child, lms);
+            }
+        }
+    }
+
+    /** a layout manager maker */
     public class WrapperLayoutManagerMaker extends Maker {
+        /** {@inheritDoc} */
         public void make(FONode node, List lms) {
             //We insert the wrapper LM before it's children so an ID
             //on the node can be registered on a page.
@@ -401,9 +450,26 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         }
     }
 
-    public ExternalDocumentLayoutManager makeExternalDocumentLayoutManager(
-        AreaTreeHandler ath, ExternalDocument ed) {
-        return new ExternalDocumentLayoutManager(ath, ed);
+    public class MultiSwitchLayoutManagerMaker extends Maker {
+
+        @Override
+        public void make(FONode node, List lms) {
+            lms.add(new MultiSwitchLayoutManager((MultiSwitch) node));
+        }
+    }
+
+    public class MultiCaseLayoutManagerMaker extends Maker {
+
+        @Override
+        public void make(FONode node, List lms) {
+            lms.add(new MultiCaseLayoutManager((MultiCase) node));
+        }
+    }
+
+    public static class FloatLayoutManagerMaker extends Maker {
+        public void make(FONode node, List lms) {
+            lms.add(new FloatLayoutManager((Float) node));
+        }
     }
 
 }
