@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PageBreakingAlgorithm.java 1296496 2012-03-02 22:19:46Z gadams $ */
+/* $Id: PageBreakingAlgorithm.java 1723297 2016-01-06 12:13:52Z ssteiner $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -29,7 +29,9 @@ import org.apache.commons.logging.LogFactory;
 
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FObj;
+import org.apache.fop.layoutmgr.AbstractBreaker.FloatPosition;
 import org.apache.fop.layoutmgr.AbstractBreaker.PageBreakPosition;
+import org.apache.fop.layoutmgr.WhitespaceManagementPenalty.Variant;
 import org.apache.fop.traits.MinOptMax;
 import org.apache.fop.util.ListUtil;
 
@@ -38,66 +40,77 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
     /** the logger for the class */
     private static Log log = LogFactory.getLog(PageBreakingAlgorithm.class);
 
-    private LayoutManager topLevelLM;
-    private PageProvider pageProvider;
-    private PageBreakingLayoutListener layoutListener;
+    private final LayoutManager topLevelLM;
+    private final PageProvider pageProvider;
+    private final PageBreakingLayoutListener layoutListener;
     /** List of PageBreakPosition elements. */
-    private LinkedList<PageBreakPosition> pageBreaks = null;
+    private LinkedList<PageBreakPosition> pageBreaks;
 
     /** Footnotes which are cited between the currently considered active node (previous
      * break) and the current considered break. Its type is
      * List&lt;List&lt;KnuthElement&gt;&gt;, it contains the sequences of KnuthElement
      * representing the footnotes bodies.
      */
-    private List<List<KnuthElement>> footnotesList = null;
+    private List<List<KnuthElement>> footnotesList;
     /** Cumulated bpd of unhandled footnotes. */
-    private List<Integer> lengthList = null;
+    private List<Integer> lengthList;
     /** Length of all the footnotes which will be put on the current page. */
-    private int totalFootnotesLength = 0;
+    private int totalFootnotesLength;
     /**
      * Length of all the footnotes which have already been inserted, up to the currently
      * considered element. That is, footnotes from the currently considered page plus
      * footnotes from its preceding pages.
      */
-    private int insertedFootnotesLength = 0;
+    private int insertedFootnotesLength;
 
     /** True if footnote citations have been met since the beginning of the page sequence. */
-    private boolean footnotesPending = false;
+    private boolean footnotesPending;
     /** True if the elements met after the previous break point contain footnote citations. */
-    private boolean newFootnotes = false;
+    private boolean newFootnotes;
     /** Index of the first footnote met after the previous break point. */
-    private int firstNewFootnoteIndex = 0;
+    private int firstNewFootnoteIndex;
     /** Index of the last footnote inserted on the current page. */
-    private int footnoteListIndex = 0;
+    private int footnoteListIndex;
     /** Index of the last element of the last footnote inserted on the current page. */
     private int footnoteElementIndex = -1;
 
     // demerits for a page break that splits a footnote
-    private int splitFootnoteDemerits = 5000;
+    private final int splitFootnoteDemerits = 5000;
     // demerits for a page break that defers a whole footnote to the following page
-    private int deferredFootnoteDemerits = 10000;
-    private MinOptMax footnoteSeparatorLength = null;
+    private final int deferredFootnoteDemerits = 10000;
+    private MinOptMax footnoteSeparatorLength;
 
     // the method noBreakBetween(int, int) uses these variables
     // to store parameters and result of the last call, in order
     // to reuse them and take less time
     private int storedPrevBreakIndex = -1;
     private int storedBreakIndex = -1;
-    private boolean storedValue = false;
+    private boolean storedValue;
 
     //Controls whether overflows should be warned about or not
-    private boolean autoHeight = false;
+    private boolean autoHeight;
 
     //Controls whether a single part should be forced if possible (ex. block-container)
-    private boolean favorSinglePart = false;
+    private boolean favorSinglePart;
 
     private int ipdDifference;
     private KnuthNode bestNodeForIPDChange;
+    public KnuthNode bestNodeForLastPage;
 
     //Used to keep track of switches in keep-context
     private int currentKeepContext = Constants.EN_AUTO;
     private KnuthNode lastBeforeKeepContextSwitch;
 
+    // just one float for now...
+    private boolean handlingStartOfFloat;
+    private boolean handlingEndOfFloat;
+    private int floatHeight;
+    private KnuthNode bestFloatEdgeNode;
+    private FloatPosition floatPosition;
+    private int previousFootnoteListIndex = -2;
+    private int previousFootnoteElementIndex = -2;
+    private boolean relayingFootnotes;
+
     /**
      * Construct a page breaking algorithm.
      * @param topLevelLM the top level layout manager
@@ -117,7 +130,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
      * @param favorSinglePart true if favoring single part
      * @see BreakingAlgorithm
      */
-    public PageBreakingAlgorithm(LayoutManager topLevelLM,      // CSOK: ParameterNumber
+    public PageBreakingAlgorithm(LayoutManager topLevelLM,
                                  PageProvider pageProvider,
                                  PageBreakingLayoutListener layoutListener,
                                  int alignment, int alignmentLast,
@@ -140,19 +153,31 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
      */
     protected class KnuthPageNode extends KnuthNode {
 
-        /** Additional length due to footnotes. */
-        public int totalFootnotes;                              // CSOK: VisibilityModifier
+        /** Additional length due to already inserted footnotes. */
+        public int insertedFootnotes;
+
+        /** Total length of the footnotes. */
+        public int totalFootnotes;
 
         /** Index of the last inserted footnote. */
-        public int footnoteListIndex;                           // CSOK: VisibilityModifier
+        public int footnoteListIndex;
 
         /** Index of the last inserted element of the last inserted footnote. */
-        public int footnoteElementIndex;                        // CSOK: VisibilityModifier
+        public int footnoteElementIndex;
 
-        public KnuthPageNode(int position,                      // CSOK: ParameterNumber
+        /**
+         * Pending variants of dynamic contents that were evaluated WRT this node.
+         * When computing page difference for a break element, the total width of these variants
+         * will be added to 'actualWidth'.
+         */
+        private final List<Variant> pendingVariants = new ArrayList<Variant>();
+        private int totalVariantsWidth;
+
+        public KnuthPageNode(int position,
                              int line, int fitness,
                              int totalWidth, int totalStretch, int totalShrink,
-                             int totalFootnotes, int footnoteListIndex, int footnoteElementIndex,
+                             int insertedFootnotes, int totalFootnotes,
+                             int footnoteListIndex, int footnoteElementIndex,
                              double adjustRatio, int availableShrink, int availableStretch,
                              int difference, double totalDemerits, KnuthNode previous) {
             super(position, line, fitness,
@@ -160,10 +185,16 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                   adjustRatio, availableShrink, availableStretch,
                   difference, totalDemerits, previous);
             this.totalFootnotes = totalFootnotes;
+            this.insertedFootnotes = insertedFootnotes;
             this.footnoteListIndex = footnoteListIndex;
             this.footnoteElementIndex = footnoteElementIndex;
         }
 
+        public void addVariant(Variant variant) {
+            pendingVariants.add(variant);
+            totalVariantsWidth += variant.width;
+        }
+
     }
 
     /**
@@ -172,23 +203,30 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
      */
     protected class BestPageRecords extends BestRecords {
 
-        private int[] bestFootnotesLength = new int[4];
-        private int[] bestFootnoteListIndex = new int[4];
-        private int[] bestFootnoteElementIndex = new int[4];
+        private final int[] bestInsertedFootnotesLength = new int[4];
+        private final int[] bestTotalFootnotesLength = new int[4];
+        private final int[] bestFootnoteListIndex = new int[4];
+        private final int[] bestFootnoteElementIndex = new int[4];
 
+        @Override
         public void addRecord(double demerits, KnuthNode node, double adjust,
                               int availableShrink, int availableStretch,
                               int difference, int fitness) {
             super.addRecord(demerits, node, adjust,
                             availableShrink, availableStretch,
                             difference, fitness);
-            bestFootnotesLength[fitness] = insertedFootnotesLength;
+            bestInsertedFootnotesLength[fitness] = insertedFootnotesLength;
+            bestTotalFootnotesLength[fitness] = totalFootnotesLength;
             bestFootnoteListIndex[fitness] = footnoteListIndex;
             bestFootnoteElementIndex[fitness] = footnoteElementIndex;
         }
 
-        public int getFootnotesLength(int fitness) {
-            return bestFootnotesLength[fitness];
+        public int getInsertedFootnotesLength(int fitness) {
+            return bestInsertedFootnotesLength[fitness];
+        }
+
+        public int getTotalFootnotesLength(int fitness) {
+            return bestTotalFootnotesLength[fitness];
         }
 
         public int getFootnoteListIndex(int fitness) {
@@ -198,6 +236,8 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
         public int getFootnoteElementIndex(int fitness) {
             return bestFootnoteElementIndex[fitness];
         }
+
+
     }
 
     /** {@inheritDoc} */
@@ -207,6 +247,18 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
         insertedFootnotesLength = 0;
         footnoteListIndex = 0;
         footnoteElementIndex = -1;
+        if (topLevelLM instanceof PageSequenceLayoutManager) {
+            PageSequenceLayoutManager pslm = (PageSequenceLayoutManager) topLevelLM;
+            if (pslm.handlingStartOfFloat() || pslm.handlingEndOfFloat()) {
+                pslm.retrieveFootnotes(this);
+            }
+            if (pslm.handlingStartOfFloat()) {
+                floatHeight = Math.min(pslm.getFloatHeight(), lineWidth - pslm.getFloatYOffset());
+            }
+            if (pslm.handlingEndOfFloat()) {
+                totalWidth += pslm.getOffsetDueToFloat() + insertedFootnotesLength;
+            }
+        }
     }
 
     /**
@@ -280,14 +332,15 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
 
     /** {@inheritDoc} */
     @Override
-    protected KnuthNode createNode(int position,                // CSOK: ParameterNumber
+    protected KnuthNode createNode(int position,
                                    int line, int fitness,
                                    int totalWidth, int totalStretch, int totalShrink,
                                    double adjustRatio, int availableShrink, int availableStretch,
                                    int difference, double totalDemerits, KnuthNode previous) {
         return new KnuthPageNode(position, line, fitness,
                                  totalWidth, totalStretch, totalShrink,
-                                 insertedFootnotesLength, footnoteListIndex, footnoteElementIndex,
+                                 insertedFootnotesLength, totalFootnotesLength,
+                                 footnoteListIndex, footnoteElementIndex,
                                  adjustRatio, availableShrink, availableStretch,
                                  difference, totalDemerits, previous);
     }
@@ -298,7 +351,8 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                                    int totalWidth, int totalStretch, int totalShrink) {
         return new KnuthPageNode(position, line, fitness,
                                  totalWidth, totalStretch, totalShrink,
-                                 ((BestPageRecords) best).getFootnotesLength(fitness),
+                                 ((BestPageRecords) best).getInsertedFootnotesLength(fitness),
+                                 ((BestPageRecords) best).getTotalFootnotesLength(fitness),
                                  ((BestPageRecords) best).getFootnoteListIndex(fitness),
                                  ((BestPageRecords) best).getFootnoteElementIndex(fitness),
                                  best.getAdjust(fitness), best.getAvailableShrink(fitness),
@@ -322,6 +376,12 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                 firstNewFootnoteIndex = footnotesList.size() - 1;
             }
         }
+        if (box instanceof KnuthBlockBox && ((KnuthBlockBox) box).hasFloatAnchors()) {
+            handlingStartOfFloat = true;
+        }
+        if (floatHeight != 0 && totalWidth >= floatHeight) {
+            handlingEndOfFloat = true;
+        }
     }
 
     /**
@@ -368,11 +428,6 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
 
         // compute the total length of the footnotes
         for (List<KnuthElement> noteList : elementLists) {
-
-            //Space resolution (Note: this does not respect possible stacking constraints
-            //between footnotes!)
-            SpaceResolver.resolveElementList(noteList);
-
             int noteLength = 0;
             footnotesList.add(noteList);
             for (KnuthElement element : noteList) {
@@ -393,6 +448,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
     /** {@inheritDoc} */
     @Override
     protected int restartFrom(KnuthNode restartingNode, int currentIndex) {
+
         int returnValue = super.restartFrom(restartingNode, currentIndex);
         newFootnotes = false;
         if (footnotesPending) {
@@ -405,6 +461,12 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                     resetFootnotes(((KnuthBlockBox) resetElement).getElementLists());
                 }
             }
+            assert restartingNode instanceof KnuthPageNode;
+            KnuthPageNode restartingPageNode = (KnuthPageNode) restartingNode;
+            footnoteElementIndex = restartingPageNode.footnoteElementIndex;
+            footnoteListIndex = restartingPageNode.footnoteListIndex;
+            totalFootnotesLength = restartingPageNode.totalFootnotes;
+            insertedFootnotesLength = restartingPageNode.insertedFootnotes;
         }
         return returnValue;
     }
@@ -413,13 +475,6 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
         for (int i = 0; i < elementLists.size(); i++) {
             ListUtil.removeLast(footnotesList);
             ListUtil.removeLast(lengthList);
-
-            // update totalFootnotesLength
-            if (!lengthList.isEmpty()) {
-                totalFootnotesLength = ListUtil.getLast(lengthList);
-            } else {
-                totalFootnotesLength = 0;
-            }
         }
         // update footnotesPending;
         if (footnotesList.size() == 0) {
@@ -497,12 +552,16 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
         int actualWidth = totalWidth - pageNode.totalWidth;
         int footnoteSplit;
         boolean canDeferOldFN;
-        if (element.isPenalty()) {
+        actualWidth += pageNode.totalVariantsWidth;
+        if (element instanceof WhitespaceManagementPenalty) {
+            actualWidth += handleWhitespaceManagementPenalty(pageNode,
+                    (WhitespaceManagementPenalty) element, elementIndex);
+        } else if (element.isPenalty()) {
             actualWidth += element.getWidth();
         }
         if (footnotesPending) {
             // compute the total length of the footnotes not yet inserted
-            int allFootnotes = totalFootnotesLength - pageNode.totalFootnotes;
+            int allFootnotes = totalFootnotesLength - pageNode.insertedFootnotes;
             if (allFootnotes > 0) {
                 // this page contains some footnote citations
                 // add the footnote separator width
@@ -511,15 +570,15 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                     // there is enough space to insert all footnotes:
                     // add the whole allFootnotes length
                     actualWidth += allFootnotes;
-                    insertedFootnotesLength = pageNode.totalFootnotes + allFootnotes;
+                    insertedFootnotesLength = pageNode.insertedFootnotes + allFootnotes;
                     footnoteListIndex = footnotesList.size() - 1;
                     footnoteElementIndex
                         = getFootnoteList(footnoteListIndex).size() - 1;
-                } else if (((canDeferOldFN = canDeferOldFootnotes // CSOK: InnerAssignment
-                             (pageNode, elementIndex))
+                } else if (((canDeferOldFN = canDeferOldFootnotes(
+                             pageNode, elementIndex))
                             || newFootnotes)
-                           && (footnoteSplit = getFootnoteSplit // CSOK: InnerAssignment
-                               (pageNode, getLineWidth(activeNode.line) - actualWidth,
+                           && (footnoteSplit = getFootnoteSplit(
+                               pageNode, getLineWidth(activeNode.line) - actualWidth,
                                 canDeferOldFN)) > 0) {
                     // it is allowed to break or even defer footnotes if either:
                     //  - there are new footnotes in the last piece of content, and
@@ -528,7 +587,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                     //    this is the first feasible break; in this case it is allowed
                     //    to break and defer, if necessary, old and new footnotes
                     actualWidth += footnoteSplit;
-                    insertedFootnotesLength = pageNode.totalFootnotes + footnoteSplit;
+                    insertedFootnotesLength = pageNode.insertedFootnotes + footnoteSplit;
                     // footnoteListIndex has been set in getFootnoteSplit()
                     // footnoteElementIndex has been set in getFootnoteSplit()
                 } else {
@@ -538,7 +597,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                     // that cannot be broken:
                     // add the whole allFootnotes length, so this breakpoint will be discarded
                     actualWidth += allFootnotes;
-                    insertedFootnotesLength = pageNode.totalFootnotes + allFootnotes;
+                    insertedFootnotesLength = pageNode.insertedFootnotes + allFootnotes;
                     footnoteListIndex = footnotesList.size() - 1;
                     footnoteElementIndex
                         = getFootnoteList(footnoteListIndex).size() - 1;
@@ -558,6 +617,24 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
         }
     }
 
+    /**
+     * Evaluates the variants corresponding to the given penalty until one that
+     * leads to an acceptable adjustment ratio is found. That variant will
+     * be added to the list of pending variants in the given active node.
+     */
+    private int handleWhitespaceManagementPenalty(KnuthPageNode activeNode,
+            WhitespaceManagementPenalty penalty, int elementIndex) {
+        for (Variant var : penalty.getVariants()) {
+            int difference = computeDifference(activeNode, var.getPenalty(), elementIndex);
+            double r = computeAdjustmentRatio(activeNode, difference);
+            if (r >= -1.0) {
+                activeNode.addVariant(var);
+                return var.width;
+            }
+        }
+        return 0;
+    }
+
     /**
      * Checks whether footnotes from preceding pages may be deferred to the page after
      * the given element.
@@ -569,7 +646,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
     private boolean canDeferOldFootnotes(KnuthPageNode node, int contentElementIndex) {
         return (noBreakBetween(node.position, contentElementIndex)
                 && deferredFootnotes(node.footnoteListIndex,
-                        node.footnoteElementIndex, node.totalFootnotes));
+                        node.footnoteElementIndex, node.insertedFootnotes));
     }
 
     /**
@@ -649,7 +726,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                 boolean canDeferOldFootnotes) {
         return getFootnoteSplit(activeNode.footnoteListIndex,
                                 activeNode.footnoteElementIndex,
-                                activeNode.totalFootnotes,
+                                activeNode.insertedFootnotes,
                                 availableLength, canDeferOldFootnotes);
     }
 
@@ -714,10 +791,8 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
             int prevIndex = -1;
             int index = -1;
 
-            while (!(somethingAdded && splitLength > availableLength)) {
-                if (!somethingAdded) {
-                    somethingAdded = true;
-                } else {
+            while (splitLength <= availableLength) {
+                if (somethingAdded) {
                     prevSplitLength = splitLength;
                     prevIndex = index;
                 }
@@ -733,6 +808,10 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                         // element is a box
                         splitLength += element.getWidth();
                         boxPreceding = true;
+                        if (splitLength > prevSplitLength) {
+                            // and it is non-empty
+                            somethingAdded = true;
+                        }
                     } else if (element.isGlue()) {
                         // element is a glue
                         if (boxPreceding) {
@@ -749,6 +828,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                             index = noteListIterator.previousIndex();
                             break;
                         }
+                        boxPreceding = false;
                     }
                 }
             }
@@ -758,7 +838,6 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
             // page here
             // if prevSplitLength is > 0 we can insert some footnote content in this page
             // and insert the remaining in the following one
-            //TODO: check this conditional, as the first one is always false...?
             if (!somethingAdded) {
                 // there was not enough space to add a piece of the first new footnote
                 // this is not a good break
@@ -781,7 +860,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
         if (difference > 0) {
             int maxAdjustment = totalStretch - activeNode.totalStretch;
             // add the footnote separator stretch if some footnote content will be added
-            if (((KnuthPageNode) activeNode).totalFootnotes < totalFootnotesLength) {
+            if (((KnuthPageNode) activeNode).insertedFootnotes < totalFootnotesLength) {
                 maxAdjustment += footnoteSeparatorLength.getStretch();
             }
             if (maxAdjustment > 0) {
@@ -792,7 +871,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
         } else if (difference < 0) {
             int maxAdjustment = totalShrink - activeNode.totalShrink;
             // add the footnote separator shrink if some footnote content will be added
-            if (((KnuthPageNode) activeNode).totalFootnotes < totalFootnotesLength) {
+            if (((KnuthPageNode) activeNode).insertedFootnotes < totalFootnotesLength) {
                 maxAdjustment += footnoteSeparatorLength.getShrink();
             }
             if (maxAdjustment > 0) {
@@ -863,10 +942,11 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
     @Override
     protected void finish() {
         for (int i = startLine; i < endLine; i++) {
+            // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
             for (KnuthPageNode node = (KnuthPageNode) getNode(i);
                  node != null;
                  node = (KnuthPageNode) node.next) {
-                if (node.totalFootnotes < totalFootnotesLength) {
+                if (node.insertedFootnotes < totalFootnotesLength) {
                     // layout remaining footnote bodies
                     createFootnotePages(node);
                 }
@@ -876,7 +956,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
 
     private void createFootnotePages(KnuthPageNode lastNode) {
 
-        insertedFootnotesLength = lastNode.totalFootnotes;
+        insertedFootnotesLength = lastNode.insertedFootnotes;
         footnoteListIndex = lastNode.footnoteListIndex;
         footnoteElementIndex = lastNode.footnoteElementIndex;
         int availableBPD = getLineWidth(lastNode.line);
@@ -885,16 +965,13 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
 
         // create pages containing the remaining footnote bodies
         while (insertedFootnotesLength < totalFootnotesLength) {
-            final int tmpLength = lengthList.get(footnoteListIndex);
-            // try adding some more content
-            if ((tmpLength - insertedFootnotesLength) <= availableBPD) {
-                // add a whole footnote
-                availableBPD -= tmpLength - insertedFootnotesLength;
-                insertedFootnotesLength = tmpLength;
-                footnoteElementIndex
-                    = getFootnoteList(footnoteListIndex).size() - 1;
-            } else if ((split = getFootnoteSplit                // CSOK: InnerAssignment
-                        (footnoteListIndex, footnoteElementIndex,
+            if (totalFootnotesLength - insertedFootnotesLength <= availableBPD) {
+                // All the remaining footnotes fit
+                insertedFootnotesLength = totalFootnotesLength;
+                footnoteListIndex = lengthList.size() - 1;
+                footnoteElementIndex = getFootnoteList(footnoteListIndex).size() - 1;
+            } else if ((split = getFootnoteSplit(
+                        footnoteListIndex, footnoteElementIndex,
                          insertedFootnotesLength, availableBPD, true)) > 0) {
                 // add a piece of a footnote
                 availableBPD -= split;
@@ -903,9 +980,10 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                 // footnoteElementIndex has already been set in getFootnoteSplit()
             } else {
                 // cannot add any content: create a new node and start again
+                // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
                 KnuthPageNode node = (KnuthPageNode)
                                      createNode(lastNode.position, prevNode.line + 1, 1,
-                                                insertedFootnotesLength - prevNode.totalFootnotes,
+                                                insertedFootnotesLength - prevNode.insertedFootnotes,
                                                 0, 0,
                                                 0, 0, 0,
                                                 0, 0, prevNode);
@@ -917,9 +995,10 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
             }
         }
         // create the last node
+        // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
         KnuthPageNode node = (KnuthPageNode)
                              createNode(lastNode.position, prevNode.line + 1, 1,
-                                        totalFootnotesLength - prevNode.totalFootnotes, 0, 0,
+                                        totalFootnotesLength - prevNode.insertedFootnotes, 0, 0,
                                         0, 0, 0,
                                         0, 0, prevNode);
         addNode(node.line, node);
@@ -971,6 +1050,17 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                             int total) {
         //int difference = (bestActiveNode.line < total)
         //      ? bestActiveNode.difference : bestActiveNode.difference + fillerMinWidth;
+        // Check if the given node has an attached variant of a dynamic content
+        KnuthPageNode pageNode = (KnuthPageNode) bestActiveNode;
+        KnuthPageNode previousPageNode = ((KnuthPageNode) pageNode.previous);
+        for (Variant var : previousPageNode.pendingVariants) {
+            WhitespaceManagementPenalty penalty = var.getWhitespaceManagementPenalty();
+            // A WMPenalty should not be activated more than once. The reason is simply
+            // because a dynamic content cannot occupy multiple pages at the same time.
+            if (!penalty.hasActiveVariant()) {
+                penalty.setActiveVariant(var);
+            }
+        }
         int difference = bestActiveNode.difference;
         if (difference + bestActiveNode.availableShrink < 0) {
             if (!autoHeight) {
@@ -1007,17 +1097,6 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                 difference = 0;
             }
         }
-        // compute the indexes of the first footnote list and the first element in that list
-        int firstListIndex = ((KnuthPageNode) bestActiveNode.previous).footnoteListIndex;
-        int firstElementIndex = ((KnuthPageNode) bestActiveNode.previous).footnoteElementIndex;
-        if (footnotesList != null
-                && firstElementIndex == getFootnoteList(firstListIndex).size() - 1) {
-            // advance to the next list
-            firstListIndex++;
-            firstElementIndex = 0;
-        } else {
-            firstElementIndex++;
-        }
 
         // add nodes at the beginning of the list, as they are found
         // backwards, from the last one to the first one
@@ -1025,12 +1104,31 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
             log.debug("BBA> difference=" + difference + " ratio=" + ratio
                     + " position=" + bestActiveNode.position);
         }
-        insertPageBreakAsFirst(new PageBreakPosition(this.topLevelLM,
-                bestActiveNode.position,
-                firstListIndex, firstElementIndex,
-                ((KnuthPageNode) bestActiveNode).footnoteListIndex,
-                ((KnuthPageNode) bestActiveNode).footnoteElementIndex,
-                ratio, difference));
+        if (handlingFloat() && floatPosition == null) {
+            floatPosition = new FloatPosition(this.topLevelLM, bestActiveNode.position, ratio, difference);
+        } else {
+            boolean useRelayedFootnotes = relayingFootnotes && bestActiveNode.previous.position == 0;
+            // compute the indexes of the first footnote list and the first element in that list
+            int firstListIndex = (useRelayedFootnotes) ? previousFootnoteListIndex
+                    : ((KnuthPageNode) bestActiveNode.previous).footnoteListIndex;
+            int firstElementIndex = (useRelayedFootnotes) ? previousFootnoteElementIndex
+                    : ((KnuthPageNode) bestActiveNode.previous).footnoteElementIndex;
+            if (useRelayedFootnotes) {
+                previousFootnoteListIndex = -2;
+                previousFootnoteElementIndex = -2;
+                relayingFootnotes = false;
+            }
+            if (footnotesList != null && firstElementIndex == getFootnoteList(firstListIndex).size() - 1) {
+                // advance to the next list
+                firstListIndex++;
+                firstElementIndex = 0;
+            } else {
+                firstElementIndex++;
+            }
+            insertPageBreakAsFirst(new PageBreakPosition(this.topLevelLM, bestActiveNode.position,
+                    firstListIndex, firstElementIndex, ((KnuthPageNode) bestActiveNode).footnoteListIndex,
+                    ((KnuthPageNode) bestActiveNode).footnoteElementIndex, ratio, difference));
+        }
     }
 
     /** {@inheritDoc} */
@@ -1103,6 +1201,19 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
 
     }
 
+    @Override
+    protected KnuthNode recoverFromOverflow() {
+        if (compareIPDs(getLastTooLong().line - 1) != 0) {
+            /**
+             * If the IPD of the next page changes, disable the recovery mechanism as the
+             * inline content has to be re-laid out according to the new IPD anyway.
+             */
+            return getLastTooLong();
+        } else {
+            return super.recoverFromOverflow();
+        }
+    }
+
     /** {@inheritDoc} */
     @Override
     protected int getIPDdifference() {
@@ -1135,7 +1246,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
     @Override
     protected void addNode(int line, KnuthNode node) {
         if (node.position < par.size() - 1 && line > 0
-                && (ipdDifference = compareIPDs(line - 1)) != 0) {  // CSOK: InnerAssignment
+                && (ipdDifference = compareIPDs(line - 1)) != 0) {
             log.trace("IPD changes at page " + line);
             if (bestNodeForIPDChange == null
                     || node.totalDemerits < bestNodeForIPDChange.totalDemerits) {
@@ -1148,6 +1259,9 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                  * the IPD change. No need to do any special handling.
                  */
                 ipdDifference = 0;
+            } else if (line > 0 /*&& (bestNodeForLastPage == null
+                     || node.totalDemerits < bestNodeForLastPage.totalDemerits)*/) {
+                bestNodeForLastPage = node;
             }
             super.addNode(line, node);
         }
@@ -1163,4 +1277,98 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
         }
         return pageProvider.compareIPDs(line);
     }
+
+    KnuthNode getBestNodeForLastPage() {
+        return bestNodeForLastPage;
+    }
+
+    protected boolean handlingFloat() {
+        return (handlingStartOfFloat || handlingEndOfFloat);
+    }
+
+    protected void createForcedNodes(KnuthNode node, int line, int elementIdx, int difference, double r,
+            double demerits, int fitnessClass, int availableShrink, int availableStretch, int newWidth,
+            int newStretch, int newShrink) {
+        if (handlingFloat()) {
+            if (bestFloatEdgeNode == null || demerits <= bestFloatEdgeNode.totalDemerits) {
+                bestFloatEdgeNode = createNode(elementIdx, line + 1, fitnessClass, newWidth, newStretch,
+                        newShrink, r, availableShrink, availableStretch, difference, demerits, node);
+            }
+        } else {
+            super.createForcedNodes(node, line, elementIdx, difference, r, demerits, fitnessClass,
+                    availableShrink, availableStretch, newWidth, newStretch, newShrink);
+        }
+    }
+
+    protected int handleFloat() {
+        calculateBreakPoints(bestFloatEdgeNode, par, bestFloatEdgeNode.line);
+        activeLines = null;
+        return bestFloatEdgeNode.line - 1;
+    }
+
+    protected KnuthNode getBestFloatEdgeNode() {
+        return bestFloatEdgeNode;
+    }
+
+    protected FloatPosition getFloatPosition() {
+        return floatPosition;
+    }
+
+    protected int getFloatHeight() {
+        return floatHeight;
+    }
+
+    protected boolean handlingStartOfFloat() {
+        return handlingStartOfFloat;
+    }
+
+    protected boolean handlingEndOfFloat() {
+        return handlingEndOfFloat;
+    }
+
+    /**
+     * Deactivate the given node
+     *
+     * @param node  the node
+     * @param line  the line number
+     */
+    protected void deactivateNode(KnuthNode node, int line) {
+        super.deactivateNode(node, line);
+        if (handlingEndOfFloat) {
+            floatHeight = totalWidth;
+        }
+    }
+
+    protected void disableFloatHandling() {
+        handlingEndOfFloat = false;
+        handlingStartOfFloat = false;
+    }
+
+    public void loadFootnotes(List fl, List ll, int tfl, int ifl, boolean fp, boolean nf, int fnfi, int fli,
+            int fei, MinOptMax fsl, int pfli, int pfei) {
+        footnotesList = fl;
+        lengthList = ll;
+        totalFootnotesLength = tfl;
+        insertedFootnotesLength = ifl;
+        footnotesPending = fp;
+        newFootnotes = nf;
+        firstNewFootnoteIndex = fnfi;
+        footnoteListIndex = fli;
+        footnoteElementIndex = fei;
+        footnoteSeparatorLength = fsl;
+        previousFootnoteListIndex = pfli;
+        previousFootnoteElementIndex = pfei;
+        relayingFootnotes = !(previousFootnoteListIndex == -2 && previousFootnoteElementIndex == -2);
+    }
+
+    public void relayFootnotes(PageSequenceLayoutManager pslm) {
+        if (!relayingFootnotes) {
+            previousFootnoteListIndex = ((KnuthPageNode) bestFloatEdgeNode.previous).footnoteListIndex;
+            previousFootnoteElementIndex = ((KnuthPageNode) bestFloatEdgeNode.previous).footnoteElementIndex;
+        }
+        pslm.holdFootnotes(footnotesList, lengthList, totalFootnotesLength, insertedFootnotesLength,
+                footnotesPending, newFootnotes, firstNewFootnoteIndex, footnoteListIndex,
+                footnoteElementIndex, footnoteSeparatorLength, previousFootnoteListIndex,
+                previousFootnoteElementIndex);
+    }
 }
