@@ -15,36 +15,73 @@
  * limitations under the License.
  */
 
-/* $Id: Position.java 808157 2009-08-26 18:50:10Z vhennebert $ */
+/* $Id: Position.java 1052561 2010-12-24 19:28:11Z spepping $ */
 
 package org.apache.fop.layoutmgr;
 
+/** A position. */
 public class Position {
 
     private LayoutManager layoutManager;
     private int index = -1;
 
+    /**
+     * Construct a position.
+     * @param lm the associated layout manager
+     */
     public Position(LayoutManager lm) {
         layoutManager = lm;
     }
 
+    /**
+     * Construct a position.
+     * @param lm the associated layout manager
+     * @param index the index
+     */
    public Position(LayoutManager lm, int index) {
         this(lm);
         setIndex(index);
     }
-
+    /** @return associated layout manager */
     public LayoutManager getLM() {
         return layoutManager;
     }
 
     /**
-     * Overridden by NonLeafPosition to return the Position of its
-     * child LM.
+     * @param depth the depth at which the LM in this position is found
+     * @return associated layout manager
+     */
+    public LayoutManager getLM(int depth) {
+        Position subPos = getPosition(depth);
+        if (subPos == null) {
+            return null;
+        } else {
+            return subPos.getLM();
+        }
+    }
+
+    /**
+     * Overridden by NonLeafPosition to return the Position of its child LM.
+     * @return a position or null
      */
     public Position getPosition() {
         return null;
     }
 
+    /**
+     * Overridden by NonLeafPosition to return the Position of its child LM.
+     * @param depth the depth at which the position in this position is found
+     * @return a position or null
+     */
+    public Position getPosition(int depth) {
+        Position subPos = this;
+        for (int i = 0; i < depth && subPos != null; ++i, subPos = subPos.getPosition()) {
+            // no-op
+        }
+        return subPos;
+    }
+
+    /** @return true if generates areas */
     public boolean generatesAreas() {
         return false;
     }
@@ -67,12 +104,13 @@ public class Position {
         return this.index;
     }
 
-    public String getShortLMName() {
+    /** @return short name of associated layout manager */
+    protected String getShortLMName() {
         if (getLM() != null) {
             String lm = getLM().toString();
             int idx = lm.lastIndexOf('.');
             if (idx >= 0 && lm.indexOf('@') > 0) {
-                return(lm.substring(idx + 1));
+                return lm.substring(idx + 1);
             } else {
                 return lm;
             }
