@@ -15,11 +15,10 @@
  * limitations under the License.
  */
 
-/* $Id: SpaceResolver.java 598558 2007-11-27 08:36:23Z jeremias $ */
+/* $Id: SpaceResolver.java 1642793 2014-12-02 00:29:57Z lbernardo $ */
 
 package org.apache.fop.layoutmgr;
 
-import java.util.LinkedList;
 import java.util.List;
 import java.util.ListIterator;
 
@@ -33,23 +32,23 @@ import org.apache.fop.traits.MinOptMax;
  * UnresolvedListElements descendants by the right combination of KnuthElements on an element
  * list.
  */
-public class SpaceResolver {
+public final class SpaceResolver {
 
     /** Logger instance */
-    protected static Log log = LogFactory.getLog(SpaceResolver.class);
-    
+    private static final Log LOG = LogFactory.getLog(SpaceResolver.class);
+
     private UnresolvedListElementWithLength[] firstPart;
     private BreakElement breakPoss;
     private UnresolvedListElementWithLength[] secondPart;
     private UnresolvedListElementWithLength[] noBreak;
-    
+
     private MinOptMax[] firstPartLengths;
     private MinOptMax[] secondPartLengths;
     private MinOptMax[] noBreakLengths;
-    
+
     private boolean isFirst;
     private boolean isLast;
-    
+
     /**
      * Main constructor.
      * @param first Element list before a break (optional)
@@ -58,7 +57,7 @@ public class SpaceResolver {
      * @param isFirst Resolution at the beginning of a (full) element list
      * @param isLast Resolution at the end of a (full) element list
      */
-    private SpaceResolver(List first, BreakElement breakPoss, List second, 
+    private SpaceResolver(List first, BreakElement breakPoss, List second,
             boolean isFirst, boolean isLast) {
         this.isFirst = isFirst;
         this.isLast = isLast;
@@ -94,25 +93,25 @@ public class SpaceResolver {
         //Add pending elements from higher level FOs
         if (breakPoss != null) {
             if (breakPoss.getPendingAfterMarks() != null) {
-                if (log.isTraceEnabled()) {
-                    log.trace("    adding pending before break: " 
+                if (LOG.isTraceEnabled()) {
+                    LOG.trace("    adding pending before break: "
                             + breakPoss.getPendingAfterMarks());
                 }
                 first.addAll(0, breakPoss.getPendingAfterMarks());
             }
             if (breakPoss.getPendingBeforeMarks() != null) {
-                if (log.isTraceEnabled()) {
-                    log.trace("    adding pending after break: " 
+                if (LOG.isTraceEnabled()) {
+                    LOG.trace("    adding pending after break: "
                             + breakPoss.getPendingBeforeMarks());
                 }
                 second.addAll(0, breakPoss.getPendingBeforeMarks());
             }
         }
-        if (log.isTraceEnabled()) {
-            log.trace("before: " + first);
-            log.trace("  break: " + breakPoss);
-            log.trace("after: " + second);
-            log.trace("NO-BREAK: " + toString(noBreak, noBreakLengths));
+        if (LOG.isTraceEnabled()) {
+            LOG.trace("before: " + first);
+            LOG.trace("  break: " + breakPoss);
+            LOG.trace("after: " + second);
+            LOG.trace("NO-BREAK: " + toString(noBreak, noBreakLengths));
         }
 
         if (first != null) {
@@ -134,10 +133,10 @@ public class SpaceResolver {
         }
         resolve();
     }
-    
+
     private String toString(Object[] arr1, Object[] arr2) {
         if (arr1.length != arr2.length) {
-            new IllegalArgumentException("The length of both arrays must be equal");
+            throw new IllegalArgumentException("The length of both arrays must be equal");
         }
         StringBuffer sb = new StringBuffer("[");
         for (int i = 0; i < arr1.length; i++) {
@@ -151,9 +150,9 @@ public class SpaceResolver {
         sb.append("]");
         return sb.toString();
     }
-    
+
     private void removeConditionalBorderAndPadding(
-                UnresolvedListElement[] elems, MinOptMax[] lengths, boolean reverse) {
+            UnresolvedListElement[] elems, MinOptMax[] lengths, boolean reverse) {
         for (int i = 0; i < elems.length; i++) {
             int effIndex;
             if (reverse) {
@@ -164,20 +163,20 @@ public class SpaceResolver {
             if (elems[effIndex] instanceof BorderOrPaddingElement) {
                 BorderOrPaddingElement bop = (BorderOrPaddingElement)elems[effIndex];
                 if (bop.isConditional() && !(bop.isFirst() || bop.isLast())) {
-                    if (log.isDebugEnabled()) {
-                        log.debug("Nulling conditional element: " + bop);
+                    if (LOG.isDebugEnabled()) {
+                        LOG.debug("Nulling conditional element: " + bop);
                     }
                     lengths[effIndex] = null;
                 }
             }
         }
-        if (log.isTraceEnabled() && elems.length > 0) {
-            log.trace("-->Resulting list: " + toString(elems, lengths));
+        if (LOG.isTraceEnabled() && elems.length > 0) {
+            LOG.trace("-->Resulting list: " + toString(elems, lengths));
         }
     }
-    
+
     private void performSpaceResolutionRule1(UnresolvedListElement[] elems, MinOptMax[] lengths,
-                    boolean reverse) {
+            boolean reverse) {
         for (int i = 0; i < elems.length; i++) {
             int effIndex;
             if (reverse) {
@@ -194,24 +193,24 @@ public class SpaceResolver {
             } else if (!elems[effIndex].isConditional()) {
                 break;
             }
-            if (log.isDebugEnabled()) {
-                log.debug("Nulling conditional element using 4.3.1, rule 1: " + elems[effIndex]);
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("Nulling conditional element using 4.3.1, rule 1: " + elems[effIndex]);
             }
             lengths[effIndex] = null;
         }
-        if (log.isTraceEnabled() && elems.length > 0) {
-            log.trace("-->Resulting list: " + toString(elems, lengths));
+        if (LOG.isTraceEnabled() && elems.length > 0) {
+            LOG.trace("-->Resulting list: " + toString(elems, lengths));
         }
     }
 
-    private void performSpaceResolutionRules2to3(UnresolvedListElement[] elems, 
+    private void performSpaceResolutionRules2to3(UnresolvedListElement[] elems,
             MinOptMax[] lengths, int start, int end) {
-        if (log.isTraceEnabled()) {
-            log.trace("rule 2-3: " + start + "-" + end);
+        if (LOG.isTraceEnabled()) {
+            LOG.trace("rule 2-3: " + start + "-" + end);
         }
         SpaceElement space;
         int remaining;
-        
+
         //Rule 2 (4.3.1, XSL 1.0)
         boolean hasForcing = false;
         remaining = 0;
@@ -236,8 +235,8 @@ public class SpaceResolver {
                 }
                 space = (SpaceElement)elems[i];
                 if (!space.isForcing()) {
-                    if (log.isDebugEnabled()) {
-                        log.debug("Nulling non-forcing space-specifier using 4.3.1, rule 2: " 
+                    if (LOG.isDebugEnabled()) {
+                        LOG.debug("Nulling non-forcing space-specifier using 4.3.1, rule 2: "
                                 + elems[i]);
                     }
                     lengths[i] = null;
@@ -245,7 +244,7 @@ public class SpaceResolver {
             }
             return; //If rule is triggered skip rule 3
         }
-        
+
         //Rule 3 (4.3.1, XSL 1.0)
         //Determine highes precedence
         int highestPrecedence = Integer.MIN_VALUE;
@@ -256,8 +255,8 @@ public class SpaceResolver {
             space = (SpaceElement)elems[i];
             highestPrecedence = Math.max(highestPrecedence, space.getPrecedence());
         }
-        if (highestPrecedence != 0 && log.isDebugEnabled()) {
-            log.debug("Highest precedence is " + highestPrecedence);
+        if (highestPrecedence != 0 && LOG.isDebugEnabled()) {
+            LOG.debug("Highest precedence is " + highestPrecedence);
         }
         //Suppress space-specifiers with lower precedence
         remaining = 0;
@@ -268,19 +267,19 @@ public class SpaceResolver {
             }
             space = (SpaceElement)elems[i];
             if (space.getPrecedence() != highestPrecedence) {
-                if (log.isDebugEnabled()) {
-                    log.debug("Nulling space-specifier with precedence " 
-                            + space.getPrecedence() + " using 4.3.1, rule 3: " 
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Nulling space-specifier with precedence "
+                            + space.getPrecedence() + " using 4.3.1, rule 3: "
                             + elems[i]);
                 }
                 lengths[i] = null;
             } else {
-                greatestOptimum = Math.max(greatestOptimum, space.getLength().opt);
+                greatestOptimum = Math.max(greatestOptimum, space.getLength().getOpt());
                 remaining++;
             }
         }
-        if (log.isDebugEnabled()) {
-            log.debug("Greatest optimum: " + greatestOptimum);
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Greatest optimum: " + greatestOptimum);
         }
         if (remaining <= 1) {
             return;
@@ -292,10 +291,10 @@ public class SpaceResolver {
                 continue;
             }
             space = (SpaceElement)elems[i];
-            if (space.getLength().opt < greatestOptimum) {
-                if (log.isDebugEnabled()) {
-                    log.debug("Nulling space-specifier with smaller optimum length "
-                            + "using 4.3.1, rule 3: " 
+            if (space.getLength().getOpt() < greatestOptimum) {
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Nulling space-specifier with smaller optimum length "
+                            + "using 4.3.1, rule 3: "
                             + elems[i]);
                 }
                 lengths[i] = null;
@@ -314,27 +313,26 @@ public class SpaceResolver {
                 continue;
             }
             space = (SpaceElement)elems[i];
-            min = Math.max(min, space.getLength().min);
-            max = Math.min(max, space.getLength().max);
+            min = Math.max(min, space.getLength().getMin());
+            max = Math.min(max, space.getLength().getMax());
             if (remaining > 1) {
-                if (log.isDebugEnabled()) {
-                    log.debug("Nulling non-last space-specifier using 4.3.1, rule 3, second part: " 
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Nulling non-last space-specifier using 4.3.1, rule 3, second part: "
                             + elems[i]);
                 }
                 lengths[i] = null;
                 remaining--;
             } else {
-                lengths[i].min = min;
-                lengths[i].max = max;
+                lengths[i] = MinOptMax.getInstance(min, lengths[i].getOpt(), max);
             }
         }
 
-        if (log.isTraceEnabled() && elems.length > 0) {
-            log.trace("Remaining spaces: " + remaining);
-            log.trace("-->Resulting list: " + toString(elems, lengths));
+        if (LOG.isTraceEnabled() && elems.length > 0) {
+            LOG.trace("Remaining spaces: " + remaining);
+            LOG.trace("-->Resulting list: " + toString(elems, lengths));
         }
     }
-    
+
     private void performSpaceResolutionRules2to3(UnresolvedListElement[] elems,
             MinOptMax[] lengths) {
         int start = 0;
@@ -354,15 +352,15 @@ public class SpaceResolver {
             start = i;
         }
     }
-    
+
     private boolean hasFirstPart() {
         return firstPart != null && firstPart.length > 0;
     }
-    
+
     private boolean hasSecondPart() {
         return secondPart != null && secondPart.length > 0;
     }
-    
+
     private void resolve() {
         if (breakPoss != null) {
             if (hasFirstPart()) {
@@ -387,11 +385,11 @@ public class SpaceResolver {
                 removeConditionalBorderAndPadding(firstPart, firstPartLengths, true);
                 performSpaceResolutionRule1(firstPart, firstPartLengths, true);
             }
-            
+
             if (hasFirstPart()) {
                 //Now that we've handled isFirst/isLast conditions, we need to look at the
                 //active part in its normal order so swap it back.
-                log.trace("Swapping first and second parts.");
+                LOG.trace("Swapping first and second parts.");
                 UnresolvedListElementWithLength[] tempList;
                 MinOptMax[] tempLengths;
                 tempList = secondPart;
@@ -408,93 +406,73 @@ public class SpaceResolver {
             performSpaceResolutionRules2to3(secondPart, secondPartLengths);
         }
     }
-    
+
     private MinOptMax sum(MinOptMax[] lengths) {
-        MinOptMax sum = new MinOptMax();
+        MinOptMax sum = MinOptMax.ZERO;
         for (int i = 0; i < lengths.length; i++) {
             if (lengths[i] != null) {
-                sum.add(lengths[i]);
+                sum = sum.plus(lengths[i]);
             }
         }
         return sum;
     }
-    
+
     private void generate(ListIterator iter) {
-        MinOptMax noBreakLength = new MinOptMax();
-        MinOptMax glue1; //space before break possibility if break occurs
-        //MinOptMax glue2; //difference between glue 1 and 3 when no break occurs
-        MinOptMax glue3; //space after break possibility if break occurs
-        glue1 = sum(firstPartLengths);
-        glue3 = sum(secondPartLengths);
-        noBreakLength = sum(noBreakLengths);
-        
-        //This doesn't produce the right glue2
-        //glue2 = new MinOptMax(noBreakLength);
-        //glue2.subtract(glue1);
-        //glue2.subtract(glue3);
-        
-        int glue2w = noBreakLength.opt - glue1.opt - glue3.opt;
-        int glue2stretch = (noBreakLength.max - noBreakLength.opt);
-        int glue2shrink = (noBreakLength.opt - noBreakLength.min);
-        glue2stretch -= glue1.max - glue1.opt;
-        glue2stretch -= glue3.max - glue3.opt;
-        glue2shrink -= glue1.opt - glue1.min;
-        glue2shrink -= glue3.opt - glue3.min;
-        
+        MinOptMax spaceBeforeBreak = sum(firstPartLengths);
+        MinOptMax spaceAfterBreak = sum(secondPartLengths);
+
         boolean hasPrecedingNonBlock = false;
-        if (log.isDebugEnabled()) {
-            log.debug("noBreakLength=" + noBreakLength 
-                    + ", glue1=" + glue1 
-                    + ", glue2=" + glue2w + "+" + glue2stretch + "-" + glue2shrink 
-                    + ", glue3=" + glue3);
-        }
         if (breakPoss != null) {
-            boolean forcedBreak = breakPoss.isForcedBreak();
-            if (glue1.isNonZero()) {
-                iter.add(new KnuthPenalty(0, KnuthPenalty.INFINITE, 
-                        false, (Position)null, true));
-                iter.add(new KnuthGlue(glue1.opt, glue1.max - glue1.opt, glue1.opt - glue1.min, 
-                        (Position)null, true));
-                if (forcedBreak) {
+            if (spaceBeforeBreak.isNonZero()) {
+                iter.add(new KnuthPenalty(0, KnuthPenalty.INFINITE, false, null, true));
+                iter.add(new KnuthGlue(spaceBeforeBreak, null, true));
+                if (breakPoss.isForcedBreak()) {
                     //Otherwise, the preceding penalty and glue will be cut off
-                    iter.add(new KnuthBox(0, (Position)null, true));
+                    iter.add(new KnuthBox(0, null, true));
                 }
             }
-            iter.add(new KnuthPenalty(breakPoss.getPenaltyWidth(), breakPoss.getPenaltyValue(), 
-                    false, breakPoss.getBreakClass(), 
+            iter.add(new KnuthPenalty(breakPoss.getPenaltyWidth(), breakPoss.getPenaltyValue(),
+                    false, breakPoss.getBreakClass(),
                     new SpaceHandlingBreakPosition(this, breakPoss), false));
             if (breakPoss.getPenaltyValue() <= -KnuthPenalty.INFINITE) {
                 return; //return early. Not necessary (even wrong) to add additional elements
             }
-            if (glue2w != 0 || glue2stretch != 0 || glue2shrink != 0) {
-                iter.add(new KnuthGlue(glue2w, glue2stretch, glue2shrink, 
-                        (Position)null, true));
+
+            // No break
+            // TODO: We can't use a MinOptMax for glue2,
+            // because min <= opt <= max is not always true - why?
+            MinOptMax noBreakLength = sum(noBreakLengths);
+            MinOptMax spaceSum = spaceBeforeBreak.plus(spaceAfterBreak);
+            int glue2width = noBreakLength.getOpt() - spaceSum.getOpt();
+            int glue2stretch = noBreakLength.getStretch() - spaceSum.getStretch();
+            int glue2shrink = noBreakLength.getShrink() - spaceSum.getShrink();
+
+            if (glue2width != 0 || glue2stretch != 0 || glue2shrink != 0) {
+                iter.add(new KnuthGlue(glue2width, glue2stretch, glue2shrink, null, true));
             }
         } else {
-            if (glue1.isNonZero()) {
-                throw new IllegalStateException("glue1 should be 0 in this case");
+            if (spaceBeforeBreak.isNonZero()) {
+                throw new IllegalStateException("spaceBeforeBreak should be 0 in this case");
             }
         }
         Position pos = null;
         if (breakPoss == null) {
             pos = new SpaceHandlingPosition(this);
         }
-        if (glue3.isNonZero() || pos != null) {
+        if (spaceAfterBreak.isNonZero() || pos != null) {
             iter.add(new KnuthBox(0, pos, true));
         }
-        if (glue3.isNonZero()) {
-            iter.add(new KnuthPenalty(0, KnuthPenalty.INFINITE, 
-                    false, (Position)null, true));
-            iter.add(new KnuthGlue(glue3.opt, glue3.max - glue3.opt, glue3.opt - glue3.min, 
-                    (Position)null, true));
+        if (spaceAfterBreak.isNonZero()) {
+            iter.add(new KnuthPenalty(0, KnuthPenalty.INFINITE, false, null, true));
+            iter.add(new KnuthGlue(spaceAfterBreak, null, true));
             hasPrecedingNonBlock = true;
         }
         if (isLast && hasPrecedingNonBlock) {
             //Otherwise, the preceding penalty and glue will be cut off
-            iter.add(new KnuthBox(0, (Position)null, true));
+            iter.add(new KnuthBox(0, null, true));
         }
     }
-    
+
     /**
      * Position class for break possibilities. It is used to notify layout manager about the
      * effective spaces and conditional lengths.
@@ -503,11 +481,11 @@ public class SpaceResolver {
 
         private SpaceResolver resolver;
         private Position originalPosition;
-        
+
         /**
          * Main constructor.
          * @param resolver the space resolver that provides the info about the actual situation
-         * @param breakPoss the original break possibility that creates this Position 
+         * @param breakPoss the original break possibility that creates this Position
          */
         public SpaceHandlingBreakPosition(SpaceResolver resolver, BreakElement breakPoss) {
             super(null);
@@ -518,18 +496,18 @@ public class SpaceResolver {
                 this.originalPosition = this.originalPosition.getPosition();
             }
         }
-        
+
         /** @return the space resolver */
         public SpaceResolver getSpaceResolver() {
             return this.resolver;
         }
-        
+
         /**
          * Notifies all affected layout managers about the current situation in the part to be
          * handled for area generation.
          * @param isBreakSituation true if this is a break situation.
          * @param side defines to notify about the situation whether before or after the break.
-         *             May be null if isBreakSituation is null. 
+         *             May be null if isBreakSituation is null.
          */
         public void notifyBreakSituation(boolean isBreakSituation, RelSide side) {
             if (isBreakSituation) {
@@ -548,7 +526,7 @@ public class SpaceResolver {
                 }
             }
         }
-        
+
         /** {@inheritDoc} */
         public String toString() {
             StringBuffer sb = new StringBuffer();
@@ -558,15 +536,21 @@ public class SpaceResolver {
             return sb.toString();
         }
 
-        /** 
+        /**
          * @return the original Position instance set at the BreakElement that this Position was
          *         created for.
          */
         public Position getOriginalBreakPosition() {
             return this.originalPosition;
         }
+
+        /** {@inheritDoc} */
+        public Position getPosition() {
+            return originalPosition;
+        }
+
     }
-    
+
     /**
      * Position class for no-break situations. It is used to notify layout manager about the
      * effective spaces and conditional lengths.
@@ -574,7 +558,7 @@ public class SpaceResolver {
     public static class SpaceHandlingPosition extends Position {
 
         private SpaceResolver resolver;
-        
+
         /**
          * Main constructor.
          * @param resolver the space resolver that provides the info about the actual situation
@@ -583,12 +567,12 @@ public class SpaceResolver {
             super(null);
             this.resolver = resolver;
         }
-        
+
         /** @return the space resolver */
         public SpaceResolver getSpaceResolver() {
             return this.resolver;
         }
-        
+
         /**
          * Notifies all affected layout managers about the current situation in the part to be
          * handled for area generation.
@@ -601,22 +585,20 @@ public class SpaceResolver {
                 resolver.secondPart[i].notifyLayoutManager(resolver.secondPartLengths[i]);
             }
         }
-        
+
         /** {@inheritDoc} */
         public String toString() {
-            StringBuffer sb = new StringBuffer();
-            sb.append("SpaceHandlingPosition");
-            return sb.toString();
+            return "SpaceHandlingPosition";
         }
     }
-    
+
     /**
      * Resolves unresolved elements applying the space resolution rules defined in 4.3.1.
      * @param elems the element list
      */
-    public static void resolveElementList(LinkedList elems) {
-        if (log.isTraceEnabled()) {
-            log.trace(elems);
+    public static void resolveElementList(List elems) {
+        if (LOG.isTraceEnabled()) {
+            LOG.trace(elems);
         }
         boolean first = true;
         boolean last = false;
@@ -628,8 +610,8 @@ public class SpaceResolver {
         while (iter.hasNext()) {
             ListElement el = (ListElement)iter.next();
             if (el.isUnresolvedElement()) {
-                if (log.isTraceEnabled()) {
-                    log.trace("unresolved found: " + el + " " + first + "/" + last);
+                if (LOG.isTraceEnabled()) {
+                    LOG.trace("unresolved found: " + el + " " + first + "/" + last);
                 }
                 BreakElement breakPoss = null;
                 //Clear temp lists
@@ -665,16 +647,16 @@ public class SpaceResolver {
                     }
                 }
                 //last = !iter.hasNext();
-                if (breakPoss == null && unresolvedSecond.size() == 0 && !last) {
-                    log.trace("Swap first and second parts in no-break condition,"
+                if (breakPoss == null && unresolvedSecond.isEmpty() && !last) {
+                    LOG.trace("Swap first and second parts in no-break condition,"
                             + " second part is empty.");
                     //The first list is reversed, so swap if this shouldn't happen
                     List swapList = unresolvedSecond;
                     unresolvedSecond = unresolvedFirst;
                     unresolvedFirst = swapList;
                 }
-                
-                log.debug("----start space resolution (first=" + first + ", last=" + last + ")...");
+
+                LOG.debug("----start space resolution (first=" + first + ", last=" + last + ")...");
                 SpaceResolver resolver = new SpaceResolver(
                         unresolvedFirst, breakPoss, unresolvedSecond, first, last);
                 if (!last) {
@@ -684,12 +666,12 @@ public class SpaceResolver {
                 if (!last && skipNextElement) {
                     iter.next();
                 }
-                log.debug("----end space resolution.");
+                LOG.debug("----end space resolution.");
             }
             first = false;
         }
     }
-    
+
     /**
      * Inspects an effective element list and notifies all layout managers about the state of
      * the spaces and conditional lengths.
@@ -699,7 +681,7 @@ public class SpaceResolver {
      * @param prevBreak index of the the break possibility just before this part (used to
      *                  identify a break condition, lastBreak <= 0 represents a no-break condition)
      */
-    public static void performConditionalsNotification(List effectiveList, 
+    public static void performConditionalsNotification(List effectiveList,
             int startElementIndex, int endElementIndex, int prevBreak) {
         KnuthElement el = null;
         if (prevBreak > 0) {
@@ -710,15 +692,15 @@ public class SpaceResolver {
         if (el != null && el.isPenalty()) {
             Position pos = el.getPosition();
             if (pos instanceof SpaceResolver.SpaceHandlingBreakPosition) {
-                beforeBreak = (SpaceResolver.SpaceHandlingBreakPosition)pos; 
+                beforeBreak = (SpaceResolver.SpaceHandlingBreakPosition)pos;
                 beforeBreak.notifyBreakSituation(true, RelSide.BEFORE);
             }
         }
-        el = (KnuthElement)effectiveList.get(endElementIndex);
+        el = endElementIndex > -1 ? (KnuthElement) effectiveList.get(endElementIndex) : null;
         if (el != null && el.isPenalty()) {
             Position pos = el.getPosition();
             if (pos instanceof SpaceResolver.SpaceHandlingBreakPosition) {
-                afterBreak = (SpaceResolver.SpaceHandlingBreakPosition)pos; 
+                afterBreak = (SpaceResolver.SpaceHandlingBreakPosition)pos;
                 afterBreak.notifyBreakSituation(true, RelSide.AFTER);
             }
         }
@@ -735,7 +717,6 @@ public class SpaceResolver {
             }
         }
     }
-    
-    
+
 
 }
