@@ -15,13 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id: TraitSetter.java 893238 2009-12-22 17:20:51Z vhennebert $ */
+/* $Id: TraitSetter.java 1593863 2014-05-11 22:06:49Z lbernardo $ */
 
 package org.apache.fop.layoutmgr;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.fop.accessibility.StructureTreeElement;
 import org.apache.fop.area.Area;
 import org.apache.fop.area.Trait;
 import org.apache.fop.datatypes.LengthBase;
@@ -29,9 +30,9 @@ import org.apache.fop.datatypes.PercentBaseContext;
 import org.apache.fop.datatypes.SimplePercentBaseContext;
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
+import org.apache.fop.fo.properties.CommonBorderPaddingBackground.BorderInfo;
 import org.apache.fop.fo.properties.CommonMarginBlock;
 import org.apache.fop.fo.properties.CommonTextDecoration;
-import org.apache.fop.fo.properties.CommonBorderPaddingBackground.BorderInfo;
 import org.apache.fop.fonts.Font;
 import org.apache.fop.traits.BorderProps;
 import org.apache.fop.traits.MinOptMax;
@@ -39,59 +40,62 @@ import org.apache.fop.traits.MinOptMax;
 /**
  * This is a helper class used for setting common traits on areas.
  */
-public class TraitSetter {
+public final class TraitSetter {
+
+    private TraitSetter() {
+    }
 
     /** logger */
-    protected static Log log = LogFactory.getLog(TraitSetter.class);
+    private static final Log LOG = LogFactory.getLog(TraitSetter.class);
 
     /**
      * Sets border and padding traits on areas.
      *
      * @param area      area to set the traits on
      * @param bpProps   border and padding properties
-     * @param bNotFirst True if the area is not the first area
-     * @param bNotLast  True if the area is not the last area
+     * @param isNotFirst True if the area is not the first area
+     * @param isNotLast  True if the area is not the last area
      * @param context   Property evaluation context
      */
     public static void setBorderPaddingTraits(Area area,
-            CommonBorderPaddingBackground bpProps, boolean bNotFirst, boolean bNotLast,
+            CommonBorderPaddingBackground bpProps, boolean isNotFirst, boolean isNotLast,
             PercentBaseContext context) {
-        int iBP;
-        iBP = bpProps.getPadding(CommonBorderPaddingBackground.START, bNotFirst, context);
-        if (iBP > 0) {
-            area.addTrait(Trait.PADDING_START, new Integer(iBP));
+        int padding;
+        padding = bpProps.getPadding(CommonBorderPaddingBackground.START, isNotFirst, context);
+        if (padding > 0) {
+            area.addTrait(Trait.PADDING_START, padding);
         }
-        iBP = bpProps.getPadding(CommonBorderPaddingBackground.END, bNotLast, context);
-        if (iBP > 0) {
-            area.addTrait(Trait.PADDING_END, new Integer(iBP));
+        padding = bpProps.getPadding(CommonBorderPaddingBackground.END, isNotLast, context);
+        if (padding > 0) {
+            area.addTrait(Trait.PADDING_END, padding);
         }
-        iBP = bpProps.getPadding(CommonBorderPaddingBackground.BEFORE, false, context);
-        if (iBP > 0) {
-            area.addTrait(Trait.PADDING_BEFORE, new Integer(iBP));
+        padding = bpProps.getPadding(CommonBorderPaddingBackground.BEFORE, false, context);
+        if (padding > 0) {
+            area.addTrait(Trait.PADDING_BEFORE, padding);
         }
-        iBP = bpProps.getPadding(CommonBorderPaddingBackground.AFTER, false, context);
-        if (iBP > 0) {
-            area.addTrait(Trait.PADDING_AFTER, new Integer(iBP));
+        padding = bpProps.getPadding(CommonBorderPaddingBackground.AFTER, false, context);
+        if (padding > 0) {
+            area.addTrait(Trait.PADDING_AFTER, padding);
         }
 
-        addBorderTrait(area, bpProps, bNotFirst,
+        addBorderTrait(area, bpProps, isNotFirst,
                 CommonBorderPaddingBackground.START,
-                BorderProps.SEPARATE, Trait.BORDER_START);
+                BorderProps.Mode.SEPARATE, Trait.BORDER_START, context);
 
-        addBorderTrait(area, bpProps, bNotLast,
+        addBorderTrait(area, bpProps, isNotLast,
                 CommonBorderPaddingBackground.END,
-                BorderProps.SEPARATE, Trait.BORDER_END);
+                BorderProps.Mode.SEPARATE, Trait.BORDER_END, context);
 
         addBorderTrait(area, bpProps, false,
                 CommonBorderPaddingBackground.BEFORE,
-                BorderProps.SEPARATE, Trait.BORDER_BEFORE);
+                BorderProps.Mode.SEPARATE, Trait.BORDER_BEFORE, context);
 
         addBorderTrait(area, bpProps, false,
                 CommonBorderPaddingBackground.AFTER,
-                BorderProps.SEPARATE, Trait.BORDER_AFTER);
+                BorderProps.Mode.SEPARATE, Trait.BORDER_AFTER, context);
     }
 
-    /**
+    /*
      * Sets border traits on an area.
      *
      * @param area    area to set the traits on
@@ -100,14 +104,15 @@ public class TraitSetter {
      */
     private static void addBorderTrait(Area area,
                                        CommonBorderPaddingBackground bpProps,
-                                       boolean bDiscard, int iSide, int mode,
-                                       Object oTrait) {
-        int iBP = bpProps.getBorderWidth(iSide, bDiscard);
-        if (iBP > 0) {
-            area.addTrait(oTrait,
-                    new BorderProps(bpProps.getBorderStyle(iSide),
-                            iBP, bpProps.getBorderColor(iSide),
-                            mode));
+
+                                       boolean discard, int side, BorderProps.Mode mode,
+                                       Integer traitCode, PercentBaseContext context) {
+        int width = bpProps.getBorderWidth(side, discard);
+        int radiusStart = bpProps.getBorderRadiusStart(side, discard, context);
+        int radiusEnd = bpProps.getBorderRadiusEnd(side, discard, context);
+        if (width > 0 || radiusStart > 0 || radiusEnd > 0) {
+            area.addTrait(traitCode, new BorderProps(bpProps.getBorderStyle(side), width, radiusStart,
+                    radiusEnd, bpProps.getBorderColor(side), mode));
         }
     }
 
@@ -116,30 +121,30 @@ public class TraitSetter {
      * Layout managers that create areas with borders can use this to
      * add the borders to the area.
      * @param area the area to set the traits on.
-     * @param bordProps border properties
+     * @param borderProps border properties
      * @param context Property evaluation context
      * @deprecated Call the other addBorders() method and addPadding separately.
      */
-    public static void addBorders(Area area, CommonBorderPaddingBackground bordProps,
+    public static void addBorders(Area area, CommonBorderPaddingBackground borderProps,
                                   PercentBaseContext context) {
-        BorderProps bps = getBorderProps(bordProps, CommonBorderPaddingBackground.BEFORE);
+        BorderProps bps = getBorderProps(borderProps, CommonBorderPaddingBackground.BEFORE, context);
         if (bps != null) {
             area.addTrait(Trait.BORDER_BEFORE, bps);
         }
-        bps = getBorderProps(bordProps, CommonBorderPaddingBackground.AFTER);
+        bps = getBorderProps(borderProps, CommonBorderPaddingBackground.AFTER, context);
         if (bps != null) {
             area.addTrait(Trait.BORDER_AFTER, bps);
         }
-        bps = getBorderProps(bordProps, CommonBorderPaddingBackground.START);
+        bps = getBorderProps(borderProps, CommonBorderPaddingBackground.START, context);
         if (bps != null) {
             area.addTrait(Trait.BORDER_START, bps);
         }
-        bps = getBorderProps(bordProps, CommonBorderPaddingBackground.END);
+        bps = getBorderProps(borderProps, CommonBorderPaddingBackground.END, context);
         if (bps != null) {
             area.addTrait(Trait.BORDER_END, bps);
         }
 
-        addPadding(area, bordProps, context);
+        addPadding(area, borderProps, context);
     }
 
     /**
@@ -147,33 +152,35 @@ public class TraitSetter {
      * Layout managers that create areas with borders can use this to
      * add the borders to the area.
      * @param area the area to set the traits on.
-     * @param bordProps border properties
+     * @param borderProps border properties
      * @param discardBefore true if the before border should be discarded
      * @param discardAfter true if the after border should be discarded
      * @param discardStart true if the start border should be discarded
      * @param discardEnd true if the end border should be discarded
      * @param context Property evaluation context
      */
-    public static void addBorders(Area area, CommonBorderPaddingBackground bordProps,
+    //TODO: remove evaluation context; unused, since border-widths are always absolute lengths
+    public static void addBorders(Area area, CommonBorderPaddingBackground borderProps,
                 boolean discardBefore, boolean discardAfter,
                 boolean discardStart, boolean discardEnd,
                 PercentBaseContext context) {
-        BorderProps bps = getBorderProps(bordProps, CommonBorderPaddingBackground.BEFORE);
+        BorderProps bps = getBorderProps(borderProps, CommonBorderPaddingBackground.BEFORE, context);
         if (bps != null && !discardBefore) {
             area.addTrait(Trait.BORDER_BEFORE, bps);
         }
-        bps = getBorderProps(bordProps, CommonBorderPaddingBackground.AFTER);
+        bps = getBorderProps(borderProps, CommonBorderPaddingBackground.AFTER, context);
         if (bps != null && !discardAfter) {
             area.addTrait(Trait.BORDER_AFTER, bps);
         }
-        bps = getBorderProps(bordProps, CommonBorderPaddingBackground.START);
+        bps = getBorderProps(borderProps, CommonBorderPaddingBackground.START, context);
         if (bps != null && !discardStart) {
             area.addTrait(Trait.BORDER_START, bps);
         }
-        bps = getBorderProps(bordProps, CommonBorderPaddingBackground.END);
+        bps = getBorderProps(borderProps, CommonBorderPaddingBackground.END, context);
         if (bps != null && !discardEnd) {
             area.addTrait(Trait.BORDER_END, bps);
         }
+
     }
 
     /**
@@ -234,38 +241,37 @@ public class TraitSetter {
         int padding = bordProps.getPadding(CommonBorderPaddingBackground.BEFORE,
                 discardBefore, context);
         if (padding != 0) {
-            area.addTrait(Trait.PADDING_BEFORE, new java.lang.Integer(padding));
+            area.addTrait(Trait.PADDING_BEFORE, padding);
         }
 
         padding = bordProps.getPadding(CommonBorderPaddingBackground.AFTER,
                 discardAfter, context);
         if (padding != 0) {
-            area.addTrait(Trait.PADDING_AFTER, new java.lang.Integer(padding));
+            area.addTrait(Trait.PADDING_AFTER, padding);
         }
 
         padding = bordProps.getPadding(CommonBorderPaddingBackground.START,
                 discardStart, context);
         if (padding != 0) {
-            area.addTrait(Trait.PADDING_START, new java.lang.Integer(padding));
+            area.addTrait(Trait.PADDING_START, padding);
         }
 
         padding = bordProps.getPadding(CommonBorderPaddingBackground.END,
                 discardEnd, context);
         if (padding != 0) {
-            area.addTrait(Trait.PADDING_END, new java.lang.Integer(padding));
+            area.addTrait(Trait.PADDING_END, padding);
         }
 
     }
 
-    private static BorderProps getBorderProps(CommonBorderPaddingBackground bordProps, int side) {
+    private static BorderProps getBorderProps(CommonBorderPaddingBackground bordProps,
+            int side, PercentBaseContext context) {
         int width = bordProps.getBorderWidth(side, false);
-        if (width != 0) {
-            BorderProps bps;
-            bps = new BorderProps(bordProps.getBorderStyle(side),
-                                  width,
-                                  bordProps.getBorderColor(side),
-                                  BorderProps.SEPARATE);
-            return bps;
+        int radiusStart = bordProps.getBorderRadiusStart(side, false, context);
+        int radiusEnd = bordProps.getBorderRadiusEnd(side, false, context);
+        if (width != 0 || radiusStart != 0 || radiusEnd != 0) {
+            return new BorderProps(bordProps.getBorderStyle(side), width, radiusStart, radiusEnd,
+                                  bordProps.getBorderColor(side), BorderProps.Mode.SEPARATE);
         } else {
             return null;
         }
@@ -275,14 +281,14 @@ public class TraitSetter {
         assert borderInfo != null;
         int width = borderInfo.getRetainedWidth();
         if (width != 0) {
-            BorderProps bps = new BorderProps(borderInfo.getStyle(), width, borderInfo.getColor(),
-                    (outer ? BorderProps.COLLAPSE_OUTER : BorderProps.COLLAPSE_INNER));
-            return bps;
+            return  BorderProps.makeRectangular(borderInfo.getStyle(), width, borderInfo.getColor(),
+                    (outer ? BorderProps.Mode.COLLAPSE_OUTER : BorderProps.Mode.COLLAPSE_INNER));
         } else {
             return null;
         }
     }
 
+
     /**
      * Add background to an area. This method is mainly used by table-related layout
      * managers to add background for column, body or row. Since the area corresponding to
@@ -339,7 +345,7 @@ public class TraitSetter {
                                 + backProps.backgroundPositionHorizontal.getValue(refContext));
                     } else {
                         // TODO Area IPD has to be set for this to work
-                        log.warn("Horizontal background image positioning ignored"
+                        LOG.warn("Horizontal background image positioning ignored"
                                 + " because the IPD was not set on the area."
                                 + " (Yes, it's a bug in FOP)");
                     }
@@ -356,7 +362,7 @@ public class TraitSetter {
                                 + backProps.backgroundPositionVertical.getValue(refContext));
                     } else {
                         // TODO Area BPD has to be set for this to work
-                        log.warn("Vertical background image positioning ignored"
+                        LOG.warn("Vertical background image positioning ignored"
                                 + " because the BPD was not set on the area."
                                 + " (Yes, it's a bug in FOP)");
                     }
@@ -407,7 +413,7 @@ public class TraitSetter {
                         back.setHoriz(horizontal);
                     } else {
                         //TODO Area IPD has to be set for this to work
-                        log.warn("Horizontal background image positioning ignored"
+                        LOG.warn("Horizontal background image positioning ignored"
                                 + " because the IPD was not set on the area."
                                 + " (Yes, it's a bug in FOP)");
                     }
@@ -431,12 +437,18 @@ public class TraitSetter {
                         back.setVertical(vertical);
                     } else {
                         //TODO Area BPD has to be set for this to work
-                        log.warn("Vertical background image positioning ignored"
+                        LOG.warn("Vertical background image positioning ignored"
                                 + " because the BPD was not set on the area."
                                 + " (Yes, it's a bug in FOP)");
                     }
                 }
             }
+            if (backProps.backgroungImageTargetWidth.getValue() != 0) {
+                back.setImageTargetWidth(backProps.backgroungImageTargetWidth.getValue());
+            }
+            if (backProps.backgroungImageTargetHeight.getValue() != 0) {
+                back.setImageTargetHeight(backProps.backgroungImageTargetHeight.getValue());
+            }
         }
 
         area.addTrait(Trait.BACKGROUND, back);
@@ -457,24 +469,24 @@ public class TraitSetter {
                                   int startIndent, int endIndent,
                                   PercentBaseContext context) {
         if (startIndent != 0) {
-            area.addTrait(Trait.START_INDENT, new Integer(startIndent));
+            area.addTrait(Trait.START_INDENT, startIndent);
         }
 
         int spaceStart = startIndent
                 - bpProps.getBorderStartWidth(false)
                 - bpProps.getPaddingStart(false, context);
         if (spaceStart != 0) {
-            area.addTrait(Trait.SPACE_START, new Integer(spaceStart));
+            area.addTrait(Trait.SPACE_START, spaceStart);
         }
 
         if (endIndent != 0) {
-            area.addTrait(Trait.END_INDENT, new Integer(endIndent));
+            area.addTrait(Trait.END_INDENT, endIndent);
         }
         int spaceEnd = endIndent
                 - bpProps.getBorderEndWidth(false)
                 - bpProps.getPaddingEnd(false, context);
         if (spaceEnd != 0) {
-            area.addTrait(Trait.SPACE_END, new Integer(spaceEnd));
+            area.addTrait(Trait.SPACE_END, spaceEnd);
         }
     }
 
@@ -534,7 +546,7 @@ public class TraitSetter {
             MinOptMax space, double adjust) {
         int effectiveSpace = getEffectiveSpace(adjust, space);
         if (effectiveSpace != 0) {
-            area.addTrait(spaceTrait, new Integer(effectiveSpace));
+            area.addTrait(spaceTrait, effectiveSpace);
         }
     }
 
@@ -558,7 +570,7 @@ public class TraitSetter {
      */
     public static void addFontTraits(Area area, Font font) {
         area.addTrait(Trait.FONT, font.getFontTriplet());
-        area.addTrait(Trait.FONT_SIZE, new Integer(font.getFontSize()));
+        area.addTrait(Trait.FONT_SIZE, font.getFontSize());
     }
 
     /**
@@ -588,13 +600,15 @@ public class TraitSetter {
     }
 
     /**
-     * Adds the ptr trait to the area.
+     * Sets the structure tree element associated to the given area.
+     *
      * @param area the area to set the traits on
-     * @param ptr string
+     * @param structureTreeElement the element the area is associated to in the document structure
      */
-    public static void addPtr(Area area, String ptr) {
-        if (ptr != null && ptr.length() > 0) {
-            area.addTrait(Trait.PTR, ptr);
+    public static void addStructureTreeElement(Area area,
+            StructureTreeElement structureTreeElement) {
+        if (structureTreeElement != null) {
+            area.addTrait(Trait.STRUCTURE_TREE_ELEMENT, structureTreeElement);
         }
     }
 
@@ -609,4 +623,15 @@ public class TraitSetter {
             area.addTrait(Trait.PROD_ID, id);
         }
     }
+
+    /**
+     * Sets the optional content group layer as a trait on the area.
+     * @param area the area to set the traits on
+     * @param layer the layer ID to set
+     */
+    public static void setLayer(Area area, String layer) {
+        if (layer != null && layer.length() > 0) {
+            area.addTrait(Trait.LAYER, layer);
+        }
+    }
 }
