@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractGraphicsLayoutManager.java 1293736 2012-02-26 02:29:01Z gadams $ */
+/* $Id: AbstractGraphicsLayoutManager.java 1466146 2013-04-09 17:31:41Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
@@ -24,6 +24,7 @@ import java.awt.Rectangle;
 import java.util.List;
 
 import org.apache.fop.area.Area;
+import org.apache.fop.area.inline.InlineArea;
 import org.apache.fop.area.inline.InlineViewport;
 import org.apache.fop.datatypes.LengthBase;
 import org.apache.fop.fo.FObj;
@@ -77,7 +78,7 @@ public abstract class AbstractGraphicsLayoutManager extends LeafNodeLayoutManage
         placement.y += beforeBPD;
 
         //Determine extra IPD from borders and padding
-        if ( ( bidiLevel == -1 ) || ( ( bidiLevel & 1 ) == 0 ) ) {
+        if ((bidiLevel == -1) || ((bidiLevel & 1) == 0)) {
             int startIPD = borderProps.getPadding(CommonBorderPaddingBackground.START, false, this);
             startIPD += borderProps.getBorderWidth(CommonBorderPaddingBackground.START, false);
             placement.x += startIPD;
@@ -92,7 +93,6 @@ public abstract class AbstractGraphicsLayoutManager extends LeafNodeLayoutManage
         transferForeignAttributes(viewportArea);
 
         InlineViewport vp = new InlineViewport(viewportArea, bidiLevel);
-        TraitSetter.addStructureTreeElement(vp, fobj.getStructureTreeElement());
         TraitSetter.setProducerID(vp, fobj.getId());
         vp.setIPD(imageLayout.getViewportSize().width);
         vp.setBPD(imageLayout.getViewportSize().height);
@@ -118,6 +118,21 @@ public abstract class AbstractGraphicsLayoutManager extends LeafNodeLayoutManage
         return super.getNextKnuthElements(context, alignment);
     }
 
+    @Override
+    protected InlineArea getEffectiveArea(LayoutContext layoutContext) {
+        /*
+         * If an image is in a repeated table heading, then it must be treated as real
+         * content the first time and then as artifact. Therefore we cannot re-use the
+         * area, as we have to account for the likely different values of treatAsArtifact.
+         */
+        InlineArea area = curArea != null ? curArea : getInlineArea();
+        curArea = null;
+        if (!layoutContext.treatAsArtifact()) {
+            TraitSetter.addStructureTreeElement(area, ((AbstractGraphics) fobj).getStructureTreeElement());
+        }
+        return area;
+    }
+
     /** {@inheritDoc} */
     protected AlignmentContext makeAlignmentContext(LayoutContext context) {
         final AbstractGraphics fobj = (AbstractGraphics)this.fobj;
