@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: CharacterLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
+/* $Id: CharacterLayoutManager.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
@@ -23,6 +23,7 @@ import java.util.LinkedList;
 import java.util.List;
 
 import org.apache.fop.area.Trait;
+import org.apache.fop.area.inline.InlineArea;
 import org.apache.fop.area.inline.TextArea;
 import org.apache.fop.fo.flow.Character;
 import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
@@ -48,7 +49,7 @@ public class CharacterLayoutManager extends LeafNodeLayoutManager {
     private MinOptMax letterSpaceIPD;
     private int hyphIPD;
     private Font font;
-    private CommonBorderPaddingBackground borderProps = null;
+    private CommonBorderPaddingBackground borderProps;
 
     /**
      * Constructor
@@ -60,6 +61,7 @@ public class CharacterLayoutManager extends LeafNodeLayoutManager {
     }
 
     /** {@inheritDoc} */
+    @Override
     public void initialize() {
         Character fobj = (Character)this.fobj;
         font = FontSelector.selectFontForCharacter(fobj, this);
@@ -68,48 +70,48 @@ public class CharacterLayoutManager extends LeafNodeLayoutManager {
         hyphIPD = fobj.getCommonHyphenation().getHyphIPD(font);
         borderProps = fobj.getCommonBorderPaddingBackground();
         setCommonBorderPaddingBackground(borderProps);
-        TextArea chArea = getCharacterInlineArea(fobj);
-        chArea.setBaselineOffset(font.getAscender());
-        setCurrentArea(chArea);
     }
 
-    private TextArea getCharacterInlineArea(Character node) {
+    private TextArea createCharacterArea() {
+        Character fobj = (Character) this.fobj;
         TextArea text = new TextArea();
-        char ch = node.getCharacter();
+        char ch = fobj.getCharacter();
+        int ipd = font.getCharWidth(ch);
+        int blockProgressionOffset = 0;
+        int level = fobj.getBidiLevel();
         if (CharUtilities.isAnySpace(ch)) {
             // add space unless it's zero-width:
             if (!CharUtilities.isZeroWidthSpace(ch)) {
-                text.addSpace(ch, 0, CharUtilities.isAdjustableSpace(ch));
+                text.addSpace(ch, ipd, CharUtilities.isAdjustableSpace(ch),
+                              blockProgressionOffset, level);
             }
         } else {
-            text.addWord(String.valueOf(ch), 0);
+            int[] levels = (level >= 0) ? new int[] {level} : null;
+            text.addWord(String.valueOf(ch), ipd, null, levels, null, blockProgressionOffset);
         }
-        TraitSetter.setProducerID(text, node.getId());
-        TraitSetter.addTextDecoration(text, node.getTextDecoration());
-        TraitSetter.addPtr(text, node.getPtr()); // used for accessibility
+
+        TraitSetter.setProducerID(text, fobj.getId());
+        TraitSetter.addTextDecoration(text, fobj.getTextDecoration());
+        text.setIPD(font.getCharWidth(fobj.getCharacter()));
+        text.setBPD(font.getAscender() - font.getDescender());
+        text.setBaselineOffset(font.getAscender());
+        TraitSetter.addFontTraits(text, font);
+        text.addTrait(Trait.COLOR, fobj.getColor());
         return text;
     }
 
+    @Override
+    protected InlineArea getEffectiveArea(LayoutContext layoutContext) {
+        InlineArea area = createCharacterArea();
+        if (!layoutContext.treatAsArtifact()) {
+            TraitSetter.addStructureTreeElement(area, ((Character) fobj).getStructureTreeElement());
+        }
+        return area;
+    }
+
     /** {@inheritDoc} */
     public List getNextKnuthElements(LayoutContext context, int alignment) {
-        MinOptMax ipd;
-        curArea = get(context);
-        KnuthSequence seq = new InlineKnuthSequence();
-
-        if (curArea == null) {
-            setFinished(true);
-            return null;
-        }
-
-        Character fobj = (Character)this.fobj;
-
-        ipd = MinOptMax.getInstance(font.getCharWidth(fobj.getCharacter()));
-
-        curArea.setIPD(ipd.getOpt());
-        curArea.setBPD(font.getAscender() - font.getDescender());
-
-        TraitSetter.addFontTraits(curArea, font);
-        curArea.addTrait(Trait.COLOR, fobj.getColor());
+        Character fobj = (Character) this.fobj;
 
         // TODO: may need some special handling for fo:character
         alignmentContext = new AlignmentContext(font
@@ -120,9 +122,11 @@ public class CharacterLayoutManager extends LeafNodeLayoutManager {
                                     , fobj.getDominantBaseline()
                                     , context.getAlignmentContext());
 
+        KnuthSequence seq = new InlineKnuthSequence();
         addKnuthElementsForBorderPaddingStart(seq);
 
         // create the AreaInfo object to store the computed values
+        MinOptMax ipd = MinOptMax.getInstance(font.getCharWidth(fobj.getCharacter()));
         areaInfo = new AreaInfo((short) 0, ipd, false, alignmentContext);
 
         // node is a fo:Character
@@ -147,23 +151,25 @@ public class CharacterLayoutManager extends LeafNodeLayoutManager {
 
         addKnuthElementsForBorderPaddingEnd(seq);
 
-        LinkedList returnList = new LinkedList();
+        LinkedList<KnuthSequence> returnList = new LinkedList<KnuthSequence>();
         returnList.add(seq);
         setFinished(true);
         return returnList;
     }
 
     /** {@inheritDoc} */
+    @Override
     public String getWordChars(Position pos) {
-        return ((TextArea) curArea).getText();
+        return String.valueOf(((Character) fobj).getCharacter());
     }
 
     /** {@inheritDoc} */
+    @Override
     public void hyphenate(Position pos, HyphContext hc) {
         if (hc.getNextHyphPoint() == 1) {
             // the character ends a syllable
-            areaInfo.bHyphenated = true;
-            isSomethingChanged = true;
+            areaInfo.isHyphenated = true;
+            somethingChanged = true;
         } else {
             // hc.getNextHyphPoint() returned -1 (no more hyphenation points)
             // or a number > 1;
@@ -173,42 +179,44 @@ public class CharacterLayoutManager extends LeafNodeLayoutManager {
     }
 
     /** {@inheritDoc} */
+    @Override
     public boolean applyChanges(List oldList) {
         setFinished(false);
-        return isSomethingChanged;
+        return somethingChanged;
     }
 
     /** {@inheritDoc} */
+    @Override
     public List getChangedKnuthElements(List oldList, int alignment) {
         if (isFinished()) {
             return null;
         }
 
-        LinkedList returnList = new LinkedList();
+        LinkedList<KnuthElement> returnList = new LinkedList<KnuthElement>();
 
         addKnuthElementsForBorderPaddingStart(returnList);
 
-        if (letterSpaceIPD.isStiff() || areaInfo.iLScount == 0) {
+        if (letterSpaceIPD.isStiff() || areaInfo.letterSpaces == 0) {
             // constant letter space, or no letter space
             returnList.add(new KnuthInlineBox(areaInfo.ipdArea.getOpt(),
                                         areaInfo.alignmentContext,
                                         notifyPos(new LeafPosition(this, 0)), false));
-            if (areaInfo.bHyphenated) {
-                returnList.add
-                    (new KnuthPenalty(hyphIPD, KnuthPenalty.FLAGGED_PENALTY, true,
-                                      new LeafPosition(this, -1), false));
+            if (areaInfo.isHyphenated) {
+                returnList.add(new KnuthPenalty(hyphIPD, KnuthPenalty.FLAGGED_PENALTY, true,
+                        new LeafPosition(this, -1), false));
             }
         } else {
             // adjustable letter space
             returnList.add(new KnuthInlineBox(areaInfo.ipdArea.getOpt()
-                    - areaInfo.iLScount * letterSpaceIPD.getOpt(), areaInfo.alignmentContext,
+                    - areaInfo.letterSpaces * letterSpaceIPD.getOpt(), areaInfo.alignmentContext,
                     notifyPos(new LeafPosition(this, 0)), false));
             returnList.add(new KnuthPenalty(0, KnuthElement.INFINITE, false,
                     new LeafPosition(this, -1), true));
-            returnList.add(new KnuthGlue(letterSpaceIPD.mult(areaInfo.iLScount),
+            returnList.add(new KnuthGlue(letterSpaceIPD.mult(areaInfo.letterSpaces),
                     new LeafPosition(this, -1), true));
-            returnList.add(new KnuthInlineBox(0, null, notifyPos(new LeafPosition(this, -1)), true));
-            if (areaInfo.bHyphenated) {
+            returnList.add(
+                    new KnuthInlineBox(0, null, notifyPos(new LeafPosition(this, -1)), true));
+            if (areaInfo.isHyphenated) {
                 returnList.add(new KnuthPenalty(hyphIPD, KnuthPenalty.FLAGGED_PENALTY, true,
                         new LeafPosition(this, -1), false));
             }
@@ -221,4 +229,3 @@ public class CharacterLayoutManager extends LeafNodeLayoutManager {
     }
 
 }
-
