@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ContentLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
+/* $Id: ContentLayoutManager.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
@@ -61,12 +61,13 @@ public class ContentLayoutManager extends AbstractBaseLayoutManager
     private Area holder;
     private int stackSize;
     private LayoutManager parentLM;
-    private InlineLevelLayoutManager childLM = null;
+    private InlineLevelLayoutManager childLM;
 
     /**
      * Constructs a new ContentLayoutManager
      *
      * @param area  The parent area
+     * @param parentLM the parent layout manager
      */
     public ContentLayoutManager(Area area, LayoutManager parentLM) {
         holder = area;
@@ -100,15 +101,17 @@ public class ContentLayoutManager extends AbstractBaseLayoutManager
         }
     }
 
+    /** {@inheritDoc} */
     public void initialize() {
         // Empty
     }
 
-    public void fillArea(LayoutManager curLM) {
+    private void fillArea(LayoutManager curLM) {
 
         int ipd = 1000000;
 
-        LayoutContext childLC = new LayoutContext(LayoutContext.NEW_AREA);
+        LayoutContext childLC = LayoutContext.newInstance();
+        childLC.setFlags(LayoutContext.NEW_AREA);
         childLC.setLeadingSpace(new SpaceSpecifier(false));
         childLC.setTrailingSpace(new SpaceSpecifier(false));
         childLC.setRefIPD(ipd);
@@ -141,7 +144,7 @@ public class ContentLayoutManager extends AbstractBaseLayoutManager
             middlefollow = maxtb - lineLead;
         }
 
-        LayoutContext lc = new LayoutContext(0);
+        LayoutContext lc = LayoutContext.newInstance();
 
         lc.setFlags(LayoutContext.RESOLVE_LEADING_SPACE, true);
         lc.setLeadingSpace(new SpaceSpecifier(false));
@@ -150,6 +153,7 @@ public class ContentLayoutManager extends AbstractBaseLayoutManager
         curLM.addAreas(contentPosIter, lc);
     }
 
+    /** {@inheritDoc} */
     public void addAreas(PositionIterator posIter, LayoutContext context) {
         // add the content areas
         // the area width has already been adjusted, and it must remain unchanged
@@ -157,12 +161,13 @@ public class ContentLayoutManager extends AbstractBaseLayoutManager
         int savedIPD = ((InlineArea)holder).getIPD();
         // set to zero the ipd adjustment ratio, to avoid spaces in the pattern
         // to be modified
-        LayoutContext childContext = new LayoutContext(context);
+        LayoutContext childContext = LayoutContext.copyOf(context);
         childContext.setIPDAdjust(0.0);
         childLM.addAreas(posIter, childContext);
         ((InlineArea)holder).setIPD(savedIPD);
     }
 
+    /** @return stack size */
     public int getStackingSize() {
         return stackSize;
     }
@@ -248,6 +253,7 @@ public class ContentLayoutManager extends AbstractBaseLayoutManager
         }
     }
 
+    /** {@inheritDoc} */
     public List getNextKnuthElements(LayoutContext context, int alignment) {
         List contentList = new LinkedList();
         List returnedList;
@@ -264,7 +270,7 @@ public class ContentLayoutManager extends AbstractBaseLayoutManager
                     Object obj = returnedList.remove(0);
                     if (obj instanceof KnuthSequence) {
                         KnuthSequence ks = (KnuthSequence)obj;
-                        for (Iterator it = ks.iterator(); it.hasNext(); ) {
+                        for (Iterator it = ks.iterator(); it.hasNext();) {
                             contentElement = (KnuthElement)it.next();
                             stackSize += contentElement.getWidth();
                             contentList.add(contentElement);
@@ -282,41 +288,59 @@ public class ContentLayoutManager extends AbstractBaseLayoutManager
         return contentList;
     }
 
+    /** {@inheritDoc} */
     public List addALetterSpaceTo(List oldList) {
         return oldList;
     }
 
-    /**
-     * Remove the word space represented by the given elements
-     *
-     * @param oldList the elements representing the word space
-     */
-    public void removeWordSpace(List oldList) {
-        // do nothing
-        log.warn(this.getClass().getName() + " should not receive a call to removeWordSpace(list)");
+    /** {@inheritDoc} */
+    public List addALetterSpaceTo(List oldList, int depth) {
+        return addALetterSpaceTo(oldList);
     }
 
+    /** {@inheritDoc} */
     public String getWordChars(Position pos) {
         return "";
     }
 
+    /** {@inheritDoc} */
     public void hyphenate(Position pos, HyphContext hc) {
     }
 
+    /** {@inheritDoc} */
     public boolean applyChanges(List oldList) {
         return false;
     }
 
-    public List getChangedKnuthElements(List oldList,
-                                              /*int flaggedPenalty,*/
-                                              int alignment) {
+    /** {@inheritDoc} */
+    public boolean applyChanges(List oldList, int depth) {
+        return applyChanges(oldList);
+    }
+
+    /** {@inheritDoc} */
+    public List getChangedKnuthElements(List oldList, int alignment) {
         return null;
     }
 
+    /** {@inheritDoc} */
+    public List getChangedKnuthElements(List oldList, int alignment, int depth) {
+        return getChangedKnuthElements(oldList, alignment);
+    }
+
+    /** {@inheritDoc} */
     public PageSequenceLayoutManager getPSLM() {
         return parentLM.getPSLM();
     }
 
+
+    public boolean hasLineAreaDescendant() {
+        return true;
+    }
+
+    public int getBaselineOffset() {
+        return childLM.getBaselineOffset();
+    }
+
     // --------- Property Resolution related functions --------- //
 
     /**
