@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: InlineLayoutManager.java 1334058 2012-05-04 16:52:35Z gadams $ */
+/* $Id: InlineLayoutManager.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
@@ -72,11 +72,11 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
      */
     private static Log log = LogFactory.getLog(InlineLayoutManager.class);
 
-    private CommonMarginInline inlineProps = null;
-    private CommonBorderPaddingBackground borderProps = null;
+    private CommonMarginInline inlineProps;
+    private CommonBorderPaddingBackground borderProps;
 
-    private boolean areaCreated = false;
-    private LayoutManager lastChildLM = null; // Set when return last breakposs;
+    private boolean areaCreated;
+    private LayoutManager lastChildLM; // Set when return last breakposs;
 
     private Font font;
 
@@ -93,7 +93,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
     /** The keep-together property */
     //private KeepProperty keepTogether;
 
-    private AlignmentContext alignmentContext = null;
+    private AlignmentContext alignmentContext;
 
     /**
      * Create an inline layout manager.
@@ -213,6 +213,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         }
         if (fobj instanceof Inline || fobj instanceof BasicLink) {
             TraitSetter.setProducerID(area, fobj.getId());
+            TraitSetter.setLayer(area, fobj.getLayer());
         }
         return area;
     }
@@ -256,9 +257,9 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
     }
 
     /** {@inheritDoc} */
-    @Override                                                   // CSOK: MethodLength
-    public List getNextKnuthElements
-        (LayoutContext context, int alignment) {
+    @Override
+    public List getNextKnuthElements(
+        LayoutContext context, int alignment) {
         LayoutManager curLM;
 
         // the list returned by child LM
@@ -283,7 +284,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
                                     , context.getAlignmentContext());
         }
 
-        childLC = new LayoutContext(context);
+        childLC = LayoutContext.copyOf(context);
         childLC.setAlignmentContext(alignmentContext);
 
         if (context.startsNewArea()) {
@@ -439,7 +440,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
 
         addId();
 
-        setChildContext(new LayoutContext(context)); // Store current value
+        setChildContext(LayoutContext.copyOf(context)); // Store current value
 
         // "Unwrap" the NonLeafPositions stored in parentIter and put
         // them in a new list.  Set lastLM to be the LayoutManager
@@ -480,7 +481,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
             context.getLeadingSpace().addSpace(new SpaceVal(getSpaceStart(), this));
         }
 
-        addMarkersToPage(
+        registerMarkers(
                 true,
                 !areaCreated,
                 lastPos == null || isLast(lastPos));
@@ -542,7 +543,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         setTraits(areaCreated, lastPos == null || !isLast(lastPos));
         parentLayoutManager.addChildArea(getCurrentArea());
 
-        addMarkersToPage(
+        registerMarkers(
                 false,
                 !areaCreated,
                 lastPos == null || isLast(lastPos));
