@@ -15,10 +15,11 @@
  * limitations under the License.
  */
 
-/* $Id: KnuthInlineBox.java 985537 2010-08-14 17:17:00Z jeremias $ */
+/* $Id: KnuthInlineBox.java 1642793 2014-12-02 00:29:57Z lbernardo $ */
 
 package org.apache.fop.layoutmgr.inline;
 
+import org.apache.fop.layoutmgr.FloatContentLayoutManager;
 import org.apache.fop.layoutmgr.FootnoteBodyLayoutManager;
 import org.apache.fop.layoutmgr.KnuthBox;
 import org.apache.fop.layoutmgr.Position;
@@ -28,8 +29,10 @@ import org.apache.fop.layoutmgr.Position;
  */
 public class KnuthInlineBox extends KnuthBox {
 
-    private FootnoteBodyLayoutManager footnoteBodyLM = null;
-    private AlignmentContext alignmentContext = null;
+    private FootnoteBodyLayoutManager footnoteBodyLM;
+    private AlignmentContext alignmentContext;
+    private FloatContentLayoutManager floatContentLM;
+
 
     /**
      * Create a new KnuthBox.
@@ -39,8 +42,8 @@ public class KnuthInlineBox extends KnuthBox {
      * @param pos              the Position stored in this box
      * @param auxiliary        is this box auxiliary?
      */
-    public KnuthInlineBox
-        (int width, AlignmentContext alignmentContext, Position pos, boolean auxiliary) {
+    public KnuthInlineBox(
+        int width, AlignmentContext alignmentContext, Position pos, boolean auxiliary) {
         super(width, pos, auxiliary);
         this.alignmentContext = alignmentContext;
     }
@@ -72,4 +75,17 @@ public class KnuthInlineBox extends KnuthBox {
     public boolean isAnchor() {
         return (footnoteBodyLM != null);
     }
+
+    public void setFloatContentLM(FloatContentLayoutManager fclm) {
+        floatContentLM = fclm;
+    }
+
+    public FloatContentLayoutManager getFloatContentLM() {
+        return floatContentLM;
+    }
+
+    public boolean isFloatAnchor() {
+        return (floatContentLM != null);
+    }
+
 }
