@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RowPainter.java 1328515 2012-04-20 21:26:43Z gadams $ */
+/* $Id: RowPainter.java 1681435 2015-05-24 11:14:22Z adelmelle $ */
 
 package org.apache.fop.layoutmgr.table;
 
@@ -50,9 +50,9 @@ import org.apache.fop.layoutmgr.TraitSetter;
 class RowPainter {
     private static Log log = LogFactory.getLog(RowPainter.class);
     private int colCount;
-    private int currentRowOffset = 0;
+    private int currentRowOffset;
     /** Currently handled row (= last encountered row). */
-    private EffRow currentRow = null;
+    private EffRow currentRow;
     private LayoutContext layoutContext;
     /**
      * Index of the first row of the current part present on the current page.
@@ -79,7 +79,7 @@ class RowPainter {
     private CellPart[] lastCellParts;
 
     /** y-offset of the current table part. */
-    private int tablePartOffset = 0;
+    private int tablePartOffset;
     /** See {@link RowPainter#registerPartBackgroundArea(Block)}. */
     private CommonBorderPaddingBackground tablePartBackground;
     /** See {@link RowPainter#registerPartBackgroundArea(Block)}. */
@@ -269,6 +269,7 @@ class RowPainter {
                 } else {
                     borderAfterWhich = ConditionalBorder.REST;
                 }
+                assert (currentGU instanceof EmptyGridUnit);
                 addAreaForEmptyGridUnit((EmptyGridUnit)currentGU,
                         currentRow.getIndex(), i,
                         actualRowHeight,
@@ -303,11 +304,16 @@ class RowPainter {
                     borderAfterWhich = ConditionalBorder.REST;
                 }
 
+                // when adding the areas for the TableCellLayoutManager this helps with the isLast trait
+                // if, say, the first cell of a row has content that fits in the page, but the content of
+                // the second cell does not fit this will assure that the isLast trait for the first cell
+                // will also be false
+                lastCellParts[i].pgu.getCellLM().setLastTrait(lastCellParts[i].isLastPart());
                 addAreasForCell(firstCellParts[i].pgu,
                         firstCellParts[i].start, lastCellParts[i].end,
                         actualRowHeight, borderBeforeWhich, borderAfterWhich,
                         lastOnPage);
-                firstCellParts[i] = null;
+                firstCellParts[i] = null; // why? what about the lastCellParts[i]?
                 Arrays.fill(firstCellOnPage, i, i + currentGU.getCell().getNumberColumnsSpanned(),
                         false);
             }
