@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TableCaptionLayoutManager.java 990148 2010-08-27 13:31:41Z vhennebert $ */
+/* $Id: TableCaptionLayoutManager.java 1537600 2013-10-31 19:44:25Z gadams $ */
 
 package org.apache.fop.layoutmgr.table;
 
@@ -76,7 +76,7 @@ public class TableCaptionLayoutManager extends BlockStackingLayoutManager {
             int ipd = context.getRefIPD();
             BreakPoss bp;
 
-            LayoutContext childLC = new LayoutContext(0);
+            LayoutContext childLC = LayoutContext.newInstance();
             // if line layout manager then set stack limit to ipd
             // line LM actually generates a LineArea which is a block
             childLC.setStackLimit(
@@ -141,7 +141,7 @@ public class TableCaptionLayoutManager extends BlockStackingLayoutManager {
         /* TODO: Reimplement using Knuth approach
         LayoutManager childLM;
         int iStartPos = 0;
-        LayoutContext lc = new LayoutContext(0);
+        LayoutContext lc = LayoutContext.newInstance();
         while (parentIter.hasNext()) {
             LeafPosition lfp = (LeafPosition) parentIter.next();
             // Add the block areas to Area
@@ -180,6 +180,7 @@ public class TableCaptionLayoutManager extends BlockStackingLayoutManager {
             Area parentArea = parentLayoutManager.getParentArea(curBlockArea);
             int referenceIPD = parentArea.getIPD();
             curBlockArea.setIPD(referenceIPD);
+            curBlockArea.setBidiLevel(getTableCaptionFO().getBidiLevel());
             // Get reference IPD from parentArea
             setCurrentArea(curBlockArea); // ??? for generic operations
         }
