@@ -15,12 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: TableCellLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
+/* $Id: TableCellLayoutManager.java 1657872 2015-02-06 15:48:44Z ssteiner $ */
 
 package org.apache.fop.layoutmgr.table;
 
 import java.util.LinkedList;
 import java.util.List;
+import java.util.Map;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -28,19 +29,23 @@ import org.apache.commons.logging.LogFactory;
 import org.apache.fop.area.Area;
 import org.apache.fop.area.Block;
 import org.apache.fop.area.Trait;
+import org.apache.fop.fo.flow.Marker;
 import org.apache.fop.fo.flow.table.ConditionalBorder;
 import org.apache.fop.fo.flow.table.GridUnit;
 import org.apache.fop.fo.flow.table.PrimaryGridUnit;
 import org.apache.fop.fo.flow.table.Table;
 import org.apache.fop.fo.flow.table.TableCell;
 import org.apache.fop.fo.flow.table.TableColumn;
+import org.apache.fop.fo.flow.table.TableFooter;
+import org.apache.fop.fo.flow.table.TableHeader;
 import org.apache.fop.fo.flow.table.TablePart;
 import org.apache.fop.fo.flow.table.TableRow;
 import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
 import org.apache.fop.fo.properties.CommonBorderPaddingBackground.BorderInfo;
+import org.apache.fop.layoutmgr.AbstractLayoutManager;
 import org.apache.fop.layoutmgr.AreaAdditionUtil;
-import org.apache.fop.layoutmgr.BlockLevelLayoutManager;
 import org.apache.fop.layoutmgr.BlockStackingLayoutManager;
+import org.apache.fop.layoutmgr.ElementListObserver;
 import org.apache.fop.layoutmgr.ElementListUtils;
 import org.apache.fop.layoutmgr.Keep;
 import org.apache.fop.layoutmgr.KnuthBox;
@@ -49,8 +54,10 @@ import org.apache.fop.layoutmgr.KnuthGlue;
 import org.apache.fop.layoutmgr.KnuthPenalty;
 import org.apache.fop.layoutmgr.LayoutContext;
 import org.apache.fop.layoutmgr.LayoutManager;
+import org.apache.fop.layoutmgr.LocalBreaker;
 import org.apache.fop.layoutmgr.Position;
 import org.apache.fop.layoutmgr.PositionIterator;
+import org.apache.fop.layoutmgr.RetrieveTableMarkerLayoutManager;
 import org.apache.fop.layoutmgr.SpaceResolver;
 import org.apache.fop.layoutmgr.TraitSetter;
 import org.apache.fop.traits.BorderProps;
@@ -61,8 +68,7 @@ import org.apache.fop.util.ListUtil;
  * LayoutManager for a table-cell FO.
  * A cell contains blocks. These blocks fill the cell.
  */
-public class TableCellLayoutManager extends BlockStackingLayoutManager
-            implements BlockLevelLayoutManager {
+public class TableCellLayoutManager extends BlockStackingLayoutManager {
 
     /**
      * logging instance
@@ -79,6 +85,29 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
     private int totalHeight;
     private int usedBPD;
     private boolean emptyCell = true;
+    private boolean isDescendantOfTableFooter;
+    private boolean isDescendantOfTableHeader;
+    private boolean hasRetrieveTableMarker;
+    private boolean hasRepeatedHeader;
+
+    // place holder for the addAreas arguments
+    private boolean savedAddAreasArguments;
+    private PositionIterator savedParentIter;
+    private LayoutContext savedLayoutContext;
+    private int[] savedSpannedGridRowHeights;
+    private int savedStartRow;
+    private int savedEndRow;
+    private int savedBorderBeforeWhich;
+    private int savedBorderAfterWhich;
+    private boolean savedFirstOnPage;
+    private boolean savedLastOnPage;
+    private RowPainter savedPainter;
+    private int savedFirstRowHeight;
+    // this is set to false when the table-cell has a retrieve-table-marker and is in the table-header
+    private boolean flushArea = true;
+
+    // this information is set by the RowPainter
+    private boolean isLastTrait;
 
     /**
      * Create a new Cell layout manager.
@@ -87,7 +116,13 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
      */
     public TableCellLayoutManager(TableCell node, PrimaryGridUnit pgu) {
         super(node);
+        setGeneratesBlockArea(true);
         this.primaryGridUnit = pgu;
+        this.isDescendantOfTableHeader = node.getParent().getParent() instanceof TableHeader
+                || node.getParent() instanceof TableHeader;
+        this.isDescendantOfTableFooter = node.getParent().getParent() instanceof TableFooter
+                || node.getParent() instanceof TableFooter;
+        this.hasRetrieveTableMarker = node.hasRetrieveTableMarker();
     }
 
     /** @return the table-cell FO */
@@ -106,6 +141,9 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
         return getTableCell().getTable();
     }
 
+    public void setHasRepeatedHeader(boolean hasRepeatedHeader) {
+        this.hasRepeatedHeader = hasRepeatedHeader;
+    }
 
     /** {@inheritDoc} */
     protected int getIPIndents() {
@@ -144,7 +182,7 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
         LayoutManager curLM; // currently active LM
         LayoutManager prevLM = null; // previously active LM
         while ((curLM = getChildLM()) != null) {
-            LayoutContext childLC = new LayoutContext(0);
+            LayoutContext childLC = LayoutContext.newInstance();
             // curLM is a ?
             childLC.setStackLimitBP(context.getStackLimitBP().minus(stackLimit));
             childLC.setRefIPD(cellIPD);
@@ -248,6 +286,84 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
         totalHeight = h;
     }
 
+    private void clearRetrieveTableMarkerChildNodes(List<LayoutManager> childrenLMs) {
+        if (childrenLMs == null) {
+            return;
+        }
+        int n = childrenLMs.size();
+        for (int j = 0; j < n; j++) {
+            LayoutManager lm = childrenLMs.get(j);
+            if (lm == null) {
+                return;
+            } else if (lm instanceof RetrieveTableMarkerLayoutManager) {
+                ((AbstractLayoutManager) lm).getFObj().clearChildNodes();
+            } else {
+                List<LayoutManager> lms = lm.getChildLMs();
+                clearRetrieveTableMarkerChildNodes(lms);
+            }
+        }
+    }
+
+    /**
+     * Checks whether the associated table cell of this LM is in a table header or footer.
+     * @return true if descendant of table header or footer
+     */
+    private boolean isDescendantOfTableHeaderOrFooter() {
+        return (isDescendantOfTableFooter || isDescendantOfTableHeader);
+    }
+
+    private void saveAddAreasArguments(PositionIterator parentIter, LayoutContext layoutContext,
+            int[] spannedGridRowHeights, int startRow, int endRow, int borderBeforeWhich,
+            int borderAfterWhich, boolean firstOnPage, boolean lastOnPage, RowPainter painter,
+            int firstRowHeight) {
+        // checks for savedAddAreasArguments and isDescendantOfTableHeader were already made but repeat them
+        if (savedAddAreasArguments) {
+            return;
+        }
+        if (isDescendantOfTableHeader) {
+            savedAddAreasArguments = true;
+            savedParentIter = null /* parentIter */;
+            savedLayoutContext = null /* layoutContext */;
+            savedSpannedGridRowHeights = spannedGridRowHeights;
+            savedStartRow = startRow;
+            savedEndRow = endRow;
+            savedBorderBeforeWhich = borderBeforeWhich;
+            savedBorderAfterWhich = borderAfterWhich;
+            savedFirstOnPage = firstOnPage;
+            savedLastOnPage = lastOnPage;
+            savedPainter = painter;
+            savedFirstRowHeight = firstRowHeight;
+            TableLayoutManager parentTableLayoutManager = getTableLayoutManager();
+            parentTableLayoutManager.saveTableHeaderTableCellLayoutManagers(this);
+            // this saving is done the first time the addArea() is called; since the retrieve-table-markers
+            // cannot be resolved at this time we do not want to flush the area; the area needs nevertheless
+            // be built so that space is allocated for it.
+            flushArea = false;
+        }
+    }
+
+    private TableLayoutManager getTableLayoutManager() {
+        LayoutManager parentLM = getParent();
+        while (!(parentLM instanceof TableLayoutManager)) {
+            parentLM = parentLM.getParent();
+        }
+        TableLayoutManager tlm = (TableLayoutManager) parentLM;
+        return tlm;
+    }
+
+    /**
+     * Calls the addAreas() using the original arguments.
+     */
+    protected void repeatAddAreas() {
+        if (savedAddAreasArguments) {
+            addAreas(savedParentIter, savedLayoutContext, savedSpannedGridRowHeights, savedStartRow,
+                    savedEndRow, savedBorderBeforeWhich, savedBorderAfterWhich, savedFirstOnPage,
+                    savedLastOnPage, savedPainter, savedFirstRowHeight);
+            // so that the arguments of the next table fragment header can be saved
+            savedAddAreasArguments = false;
+        }
+    }
+
     /**
      * Add the areas for the break points. The cell contains block stacking layout
      * managers that add block areas.
@@ -277,17 +393,9 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
      * this row is placed on a previous page). Used to calculate the placement of the
      * row's background image if any
      */
-    public void addAreas(PositionIterator parentIter,
-                         LayoutContext layoutContext,
-                         int[] spannedGridRowHeights,
-                         int startRow,
-                         int endRow,
-                         int borderBeforeWhich,
-                         int borderAfterWhich,
-                         boolean firstOnPage,
-                         boolean lastOnPage,
-                         RowPainter painter,
-                         int firstRowHeight) {
+    public void addAreas(PositionIterator parentIter, LayoutContext layoutContext, int[] spannedGridRowHeights,
+            int startRow, int endRow, int borderBeforeWhich, int borderAfterWhich,
+            boolean firstOnPage, boolean lastOnPage, RowPainter painter, int firstRowHeight) {
         getParentArea(null);
 
         addId();
@@ -336,12 +444,14 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
                 Block[][] blocks = new Block[getTableCell().getNumberRowsSpanned()][getTableCell()
                         .getNumberColumnsSpanned()];
                 GridUnit[] gridUnits = (GridUnit[]) primaryGridUnit.getRows().get(startRow);
+                int level = getTableCell().getBidiLevelRecursive();
                 for (int x = 0; x < getTableCell().getNumberColumnsSpanned(); x++) {
                     GridUnit gu = gridUnits[x];
                     BorderInfo border = gu.getBorderBefore(borderBeforeWhich);
                     int borderWidth = border.getRetainedWidth() / 2;
                     if (borderWidth > 0) {
-                        addBorder(blocks, startRow, x, Trait.BORDER_BEFORE, border, firstOnPage);
+                        addBorder(blocks, startRow, x, Trait.BORDER_BEFORE, border,
+                                  firstOnPage, level);
                         adjustYOffset(blocks[startRow][x], -borderWidth);
                         adjustBPD(blocks[startRow][x], -borderWidth);
                     }
@@ -352,7 +462,8 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
                     BorderInfo border = gu.getBorderAfter(borderAfterWhich);
                     int borderWidth = border.getRetainedWidth() / 2;
                     if (borderWidth > 0) {
-                        addBorder(blocks, endRow, x, Trait.BORDER_AFTER, border, lastOnPage);
+                        addBorder(blocks, endRow, x, Trait.BORDER_AFTER, border,
+                                  lastOnPage, level);
                         adjustBPD(blocks[endRow][x], -borderWidth);
                     }
                 }
@@ -361,7 +472,8 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
                     BorderInfo border = gridUnits[0].getBorderStart();
                     int borderWidth = border.getRetainedWidth() / 2;
                     if (borderWidth > 0) {
-                        addBorder(blocks, y, 0, Trait.BORDER_START, border, inFirstColumn);
+                        addBorder(blocks, y, 0, Trait.BORDER_START, border,
+                                  inFirstColumn, level);
                         adjustXOffset(blocks[y][0], borderWidth);
                         adjustIPD(blocks[y][0], -borderWidth);
                     }
@@ -369,7 +481,7 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
                     borderWidth = border.getRetainedWidth() / 2;
                     if (borderWidth > 0) {
                         addBorder(blocks, y, gridUnits.length - 1, Trait.BORDER_END, border,
-                                inLastColumn);
+                                  inLastColumn, level);
                         adjustIPD(blocks[y][gridUnits.length - 1], -borderWidth);
                     }
                 }
@@ -406,15 +518,44 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
             if (getTableCell().getDisplayAlign() == EN_CENTER) {
                 Block space = new Block();
                 space.setBPD((cellBPD - usedBPD) / 2);
+                space.setBidiLevel(getTableCell().getBidiLevelRecursive());
                 curBlockArea.addBlock(space);
             } else if (getTableCell().getDisplayAlign() == EN_AFTER) {
                 Block space = new Block();
                 space.setBPD(cellBPD - usedBPD);
+                space.setBidiLevel(getTableCell().getBidiLevelRecursive());
                 curBlockArea.addBlock(space);
             }
         }
 
-        AreaAdditionUtil.addAreas(this, parentIter, layoutContext);
+        if (isDescendantOfTableHeaderOrFooter()) {
+            if (hasRetrieveTableMarker) {
+                if (isDescendantOfTableHeader && !savedAddAreasArguments) {
+                    saveAddAreasArguments(parentIter, layoutContext, spannedGridRowHeights, startRow, endRow,
+                            borderBeforeWhich, borderAfterWhich, firstOnPage, lastOnPage, painter,
+                            firstRowHeight);
+                }
+                recreateChildrenLMs();
+                int displayAlign = ((TableCell) this.getFObj()).getDisplayAlign();
+                TableCellBreaker breaker = new TableCellBreaker(this, cellIPD, displayAlign);
+                breaker.setDescendantOfTableFooter(isDescendantOfTableHeader);
+                if (isDescendantOfTableHeader) {
+                    breaker.setRepeatedHeader(hasRepeatedHeader);
+                } else {
+                    breaker.setRepeatedFooter(layoutContext.treatAsArtifact());
+                }
+                breaker.doLayout(usedBPD, false);
+                // this is needed so the next time the LMs are recreated they look like the originals; this
+                // is due to the fact that during the doLayout() above the FO tree changes when the
+                // retrieve-table-markers are resolved
+                clearRetrieveTableMarkerChildNodes(getChildLMs());
+            }
+        }
+
+        // if hasRetrieveTableMarker == true the areas were already added when the re-layout was done above
+        if (!hasRetrieveTableMarker) {
+            AreaAdditionUtil.addAreas(this, parentIter, layoutContext);
+        }
         // Re-adjust the cell's bpd as it may have been modified by the previous call
         // for some reason (?)
         curBlockArea.setBPD(cellBPD);
@@ -425,7 +566,11 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
                     getTableCell().getCommonBorderPaddingBackground(), this);
         }
 
-        flush();
+        if (flushArea) {
+            flush();
+        } else {
+            flushArea = true;
+        }
 
         curBlockArea = null;
 
@@ -460,15 +605,16 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
     }
 
     private void addBorder(Block[][] blocks, int i, int j, Integer side, BorderInfo border,
-            boolean outer) {
+                           boolean outer, int level) {
         if (blocks[i][j] == null) {
             blocks[i][j] = new Block();
             blocks[i][j].addTrait(Trait.IS_REFERENCE_AREA, Boolean.TRUE);
             blocks[i][j].setPositioning(Block.ABSOLUTE);
+            blocks[i][j].setBidiLevel(level);
         }
-        blocks[i][j].addTrait(side, new BorderProps(border.getStyle(),
+        blocks[i][j].addTrait(side, BorderProps.makeRectangular(border.getStyle(),
                 border.getRetainedWidth(), border.getColor(),
-                outer ? BorderProps.COLLAPSE_OUTER : BorderProps.COLLAPSE_INNER));
+                outer ? BorderProps.Mode.COLLAPSE_OUTER : BorderProps.Mode.COLLAPSE_INNER));
     }
 
     private static void adjustXOffset(Block block, int amount) {
@@ -499,6 +645,7 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
         block.setBPD(bpd);
         block.setXOffset(xoffset + startIndent - paddingStart);
         block.setYOffset(yoffset + borderBeforeWidth);
+        block.setBidiLevel(getTableCell().getBidiLevelRecursive());
         return block;
     }
 
@@ -524,6 +671,7 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
             curBlockArea.setXOffset(xoffset + startIndent);
             curBlockArea.setYOffset(yoffset);
             curBlockArea.setIPD(cellIPD);
+            curBlockArea.setBidiLevel(getTableCell().getBidiLevelRecursive());
 
             /*Area parentArea =*/ parentLayoutManager.getParentArea(curBlockArea);
             // Get reference IPD from parentArea
@@ -560,12 +708,8 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
 
     /** {@inheritDoc} */
     public Keep getKeepTogether() {
-        Keep keep = Keep.KEEP_AUTO;
-        if (primaryGridUnit.getRow() != null) {
-            keep = Keep.getKeep(primaryGridUnit.getRow().getKeepTogether());
-        }
-        keep = keep.compare(getParentKeepTogether());
-        return keep;
+        // keep-together does not apply to fo:table-cell
+        return Keep.KEEP_AUTO;
     }
 
     /** {@inheritDoc} */
@@ -615,4 +759,49 @@ public class TableCellLayoutManager extends BlockStackingLayoutManager
         return true;
     }
 
+    private static class TableCellBreaker extends LocalBreaker {
+
+        public TableCellBreaker(TableCellLayoutManager lm, int ipd, int displayAlign) {
+            super(lm, ipd, displayAlign);
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        protected void observeElementList(List elementList) {
+            String elementListID = lm.getParent().getFObj().getId() + "-" + lm.getFObj().getId();
+            ElementListObserver.observe(elementList, "table-cell", elementListID);
+        }
+
+    }
+
+    /**
+     * Registers the FO's markers on the current PageViewport and parent Table.
+     *
+     * @param isStarting    boolean indicating whether the markers qualify as 'starting'
+     * @param isFirst   boolean indicating whether the markers qualify as 'first'
+     * @param isLast    boolean indicating whether the markers qualify as 'last'
+     */
+    protected void registerMarkers(boolean isStarting, boolean isFirst, boolean isLast) {
+        Map<String, Marker> markers = getTableCell().getMarkers();
+        if (markers != null) {
+            getCurrentPV().registerMarkers(markers, isStarting, isFirst, isLast && isLastTrait);
+            if (!isDescendantOfTableHeaderOrFooter()) {
+                getTableLayoutManager().registerMarkers(markers, isStarting, isFirst, isLast && isLastTrait);
+            }
+        }
+    }
+
+    void setLastTrait(boolean isLast) {
+        isLastTrait = isLast;
+    }
+
+    /** {@inheritDoc} */
+    public void setParent(LayoutManager lm) {
+        this.parentLayoutManager = lm;
+        if (this.hasRetrieveTableMarker) {
+            this.getTableLayoutManager().flagAsHavingRetrieveTableMarker();
+        }
+    }
+
 }
