@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TableContentLayoutManager.java 807014 2009-08-23 20:27:48Z adelmelle $ */
+/* $Id: TableContentLayoutManager.java 1657872 2015-02-06 15:48:44Z ssteiner $ */
 
 package org.apache.fop.layoutmgr.table;
 
@@ -31,19 +31,25 @@ import org.apache.commons.logging.LogFactory;
 import org.apache.fop.datatypes.PercentBaseContext;
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FObj;
+import org.apache.fop.fo.flow.Marker;
 import org.apache.fop.fo.flow.table.EffRow;
 import org.apache.fop.fo.flow.table.PrimaryGridUnit;
 import org.apache.fop.fo.flow.table.Table;
+import org.apache.fop.fo.flow.table.TableBody;
 import org.apache.fop.fo.flow.table.TablePart;
 import org.apache.fop.layoutmgr.BreakElement;
 import org.apache.fop.layoutmgr.ElementListUtils;
+import org.apache.fop.layoutmgr.FootenoteUtil;
+import org.apache.fop.layoutmgr.FootnoteBodyLayoutManager;
 import org.apache.fop.layoutmgr.Keep;
+import org.apache.fop.layoutmgr.KnuthBlockBox;
 import org.apache.fop.layoutmgr.KnuthBox;
 import org.apache.fop.layoutmgr.KnuthElement;
 import org.apache.fop.layoutmgr.KnuthGlue;
 import org.apache.fop.layoutmgr.KnuthPossPosIter;
 import org.apache.fop.layoutmgr.LayoutContext;
 import org.apache.fop.layoutmgr.ListElement;
+import org.apache.fop.layoutmgr.PageBreaker;
 import org.apache.fop.layoutmgr.Position;
 import org.apache.fop.layoutmgr.PositionIterator;
 import org.apache.fop.layoutmgr.SpaceResolver.SpaceHandlingBreakPosition;
@@ -55,7 +61,7 @@ import org.apache.fop.util.BreakUtil;
 public class TableContentLayoutManager implements PercentBaseContext {
 
     /** Logger **/
-    private static Log log = LogFactory.getLog(TableContentLayoutManager.class);
+    private static final Log LOG = LogFactory.getLog(TableContentLayoutManager.class);
 
     private TableLayoutManager tableLM;
     private TableRowIterator bodyIter;
@@ -63,14 +69,17 @@ public class TableContentLayoutManager implements PercentBaseContext {
     private TableRowIterator footerIter;
     private LinkedList headerList;
     private LinkedList footerList;
-    private int headerNetHeight = 0;
-    private int footerNetHeight = 0;
+    private int headerNetHeight;
+    private int footerNetHeight;
 
     private int startXOffset;
     private int usedBPD;
 
     private TableStepper stepper;
 
+    private boolean headerIsBeingRepeated;
+    private boolean  atLeastOnce;
+
     /**
      * Main constructor
      * @param parent Parent layout manager
@@ -127,31 +136,53 @@ public class TableContentLayoutManager implements PercentBaseContext {
         return this.footerList;
     }
 
-    /** {@inheritDoc} */
-    public LinkedList getNextKnuthElements(LayoutContext context, int alignment) {
-        if (log.isDebugEnabled()) {
-            log.debug("==> Columns: " + getTableLM().getColumns());
+    /**
+     * Get a sequence of KnuthElements representing the content
+     * of the node assigned to the LM.
+     *
+     * @param context   the LayoutContext used to store layout information
+     * @param alignment the desired text alignment
+     * @return          the list of KnuthElements
+     * @see org.apache.fop.layoutmgr.LayoutManager#getNextKnuthElements(LayoutContext, int)
+     */
+    public List getNextKnuthElements(LayoutContext context, int alignment) {
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("==> Columns: " + getTableLM().getColumns());
         }
         KnuthBox headerAsFirst = null;
         KnuthBox headerAsSecondToLast = null;
         KnuthBox footerAsLast = null;
+        LinkedList returnList = new LinkedList();
+        int headerFootnoteBPD = 0;
         if (headerIter != null && headerList == null) {
             this.headerList = getKnuthElementsForRowIterator(
                     headerIter, context, alignment, TableRowIterator.HEADER);
             this.headerNetHeight
                     = ElementListUtils.calcContentLength(this.headerList);
-            if (log.isDebugEnabled()) {
-                log.debug("==> Header: "
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("==> Header: "
                         + headerNetHeight + " - " + this.headerList);
             }
             TableHeaderFooterPosition pos = new TableHeaderFooterPosition(
                     getTableLM(), true, this.headerList);
-            KnuthBox box = new KnuthBox(headerNetHeight, pos, false);
+            List<FootnoteBodyLayoutManager> footnoteList = FootenoteUtil.getFootnotes(headerList);
+            KnuthBox box = (footnoteList.isEmpty() || !getTableLM().getTable().omitHeaderAtBreak())
+                    ? new KnuthBox(headerNetHeight, pos, false)
+                    : new KnuthBlockBox(headerNetHeight, footnoteList, pos, false);
             if (getTableLM().getTable().omitHeaderAtBreak()) {
                 //We can simply add the table header at the start
                 //of the whole list
                 headerAsFirst = box;
             } else {
+                if (!footnoteList.isEmpty()) {
+                    List<List<KnuthElement>> footnotes = PageBreaker.getFootnoteKnuthElements(
+                            getTableLM().getPSLM().getFlowLayoutManager(), context, footnoteList);
+                    getTableLM().setHeaderFootnotes(footnotes);
+                    headerFootnoteBPD = getFootnotesBPD(footnotes);
+                    returnList.add(new KnuthBlockBox(-headerFootnoteBPD, footnoteList,
+                            new Position(getTableLM()), true));
+                    headerNetHeight += headerFootnoteBPD;
+                }
                 headerAsSecondToLast = box;
             }
         }
@@ -160,18 +191,26 @@ public class TableContentLayoutManager implements PercentBaseContext {
                     footerIter, context, alignment, TableRowIterator.FOOTER);
             this.footerNetHeight
                     = ElementListUtils.calcContentLength(this.footerList);
-            if (log.isDebugEnabled()) {
-                log.debug("==> Footer: "
+            if (LOG.isDebugEnabled()) {
+                LOG.debug("==> Footer: "
                         + footerNetHeight + " - " + this.footerList);
             }
             //We can simply add the table footer at the end of the whole list
             TableHeaderFooterPosition pos = new TableHeaderFooterPosition(
                     getTableLM(), false, this.footerList);
-            KnuthBox box = new KnuthBox(footerNetHeight, pos, false);
-            footerAsLast = box;
+            List<FootnoteBodyLayoutManager> footnoteList = FootenoteUtil.getFootnotes(footerList);
+            footerAsLast = footnoteList.isEmpty()
+                    ? new KnuthBox(footerNetHeight, pos, false)
+                    : new KnuthBlockBox(footerNetHeight, footnoteList, pos, false);
+            if (!(getTableLM().getTable().omitFooterAtBreak() || footnoteList.isEmpty())) {
+                List<List<KnuthElement>> footnotes = PageBreaker.getFootnoteKnuthElements(
+                        getTableLM().getPSLM().getFlowLayoutManager(), context, footnoteList);
+                getTableLM().setFooterFootnotes(footnotes);
+                footerNetHeight += getFootnotesBPD(footnotes);
+            }
         }
-        LinkedList returnList = getKnuthElementsForRowIterator(
-                bodyIter, context, alignment, TableRowIterator.BODY);
+        returnList.addAll(getKnuthElementsForRowIterator(
+                bodyIter, context, alignment, TableRowIterator.BODY));
         if (headerAsFirst != null) {
             int insertionPoint = 0;
             if (returnList.size() > 0 && ((ListElement)returnList.getFirst()).isForcedBreak()) {
@@ -192,9 +231,20 @@ public class TableContentLayoutManager implements PercentBaseContext {
             }
             returnList.add(insertionPoint, footerAsLast);
         }
+        if (headerFootnoteBPD != 0) {
+            returnList.add(new KnuthBox(headerFootnoteBPD, new Position(getTableLM()), true));
+        }
         return returnList;
     }
 
+    private int getFootnotesBPD(List<List<KnuthElement>> footnotes) {
+        int bpd = 0;
+        for (List<KnuthElement> footnote : footnotes) {
+            bpd += ElementListUtils.calcContentLength(footnote);
+        }
+        return bpd;
+    }
+
     /**
      * Creates Knuth elements by iterating over a TableRowIterator.
      * @param iter TableRowIterator instance to fetch rows from
@@ -309,16 +359,17 @@ public class TableContentLayoutManager implements PercentBaseContext {
      * @return the requested X offset
      */
     protected int getXOffsetOfGridUnit(PrimaryGridUnit gu) {
-        return getXOffsetOfGridUnit(gu.getColIndex());
+        return getXOffsetOfGridUnit(gu.getColIndex(), gu.getCell().getNumberColumnsSpanned());
     }
 
     /**
      * Returns the X offset of the grid unit in the given column.
      * @param colIndex the column index (zero-based)
+     * @param nrColSpan number columns spanned
      * @return the requested X offset
      */
-    protected int getXOffsetOfGridUnit(int colIndex) {
-        return startXOffset + getTableLM().getColumns().getXOffset(colIndex + 1, getTableLM());
+    protected int getXOffsetOfGridUnit(int colIndex, int nrColSpan) {
+        return startXOffset + getTableLM().getColumns().getXOffset(colIndex + 1, nrColSpan, getTableLM());
     }
 
     /**
@@ -337,7 +388,7 @@ public class TableContentLayoutManager implements PercentBaseContext {
         Position lastPos = null;
         Position lastCheckPos = null;
         while (parentIter.hasNext()) {
-            Position pos = (Position)parentIter.next();
+            Position pos = parentIter.next();
             if (pos instanceof SpaceHandlingBreakPosition) {
                 //This position has only been needed before addAreas was called, now we need the
                 //original one created by the layout manager.
@@ -369,14 +420,15 @@ public class TableContentLayoutManager implements PercentBaseContext {
             } else if (pos instanceof TableContentPosition) {
                 tablePositions.add(pos);
             } else {
-                if (log.isDebugEnabled()) {
-                    log.debug("Ignoring position: " + pos);
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("Ignoring position: " + pos);
                 }
             }
         }
+        boolean treatFooterAsArtifact = layoutContext.treatAsArtifact();
         if (lastPos instanceof TableHFPenaltyPosition) {
             TableHFPenaltyPosition penaltyPos = (TableHFPenaltyPosition)lastPos;
-            log.debug("Break at penalty!");
+            LOG.debug("Break at penalty!");
             if (penaltyPos.headerElements != null) {
                 //Header positions for the penalty position are in the last element and need to
                 //be handled first before all other TableContentPositions
@@ -384,41 +436,68 @@ public class TableContentLayoutManager implements PercentBaseContext {
             }
             if (penaltyPos.footerElements != null) {
                 footerElements = penaltyPos.footerElements;
+                treatFooterAsArtifact = true;
             }
         }
 
-        Map markers = getTableLM().getTable().getMarkers();
+        // there may be table fragment markers stored; clear them since we are starting a new fragment
+        tableLM.clearTableFragmentMarkers();
+
+        // note: markers at table level are to be retrieved by the page, not by the table itself
+        Map<String, Marker> markers = getTableLM().getTable().getMarkers();
         if (markers != null) {
-            getTableLM().getCurrentPV().addMarkers(markers,
+            getTableLM().getCurrentPV().registerMarkers(markers,
                     true, getTableLM().isFirst(firstPos), getTableLM().isLast(lastCheckPos));
         }
 
         if (headerElements != null) {
+            boolean ancestorTreatAsArtifact = layoutContext.treatAsArtifact();
+            if (headerIsBeingRepeated) {
+                layoutContext.setTreatAsArtifact(true);
+                if (!getTableLM().getHeaderFootnotes().isEmpty()) {
+                    getTableLM().getPSLM().addTableHeaderFootnotes(getTableLM().getHeaderFootnotes());
+                }
+            }
             //header positions for the last part are the second-to-last element and need to
             //be handled first before all other TableContentPositions
             addHeaderFooterAreas(headerElements, tableLM.getTable().getTableHeader(), painter,
                     false);
+            if (!ancestorTreatAsArtifact) {
+                headerIsBeingRepeated = true;
+            }
+            layoutContext.setTreatAsArtifact(ancestorTreatAsArtifact);
         }
 
         if (tablePositions.isEmpty()) {
             // TODO make sure this actually never happens
-            log.error("tablePositions empty."
+            LOG.error("tablePositions empty."
                     + " Please send your FO file to fop-users@xmlgraphics.apache.org");
         } else {
             // Here we are sure that posIter iterates only over TableContentPosition instances
             addBodyAreas(tablePositions.iterator(), painter, footerElements == null);
         }
 
+        // if there are TCLMs saved because they have a RetrieveTableMarker, we repeat the header areas now;
+        // this can also be done after the areas for the footer are added but should be the same as here
+        tableLM.setRepeateHeader(atLeastOnce);
+        tableLM.repeatAddAreasForSavedTableHeaderTableCellLayoutManagers();
+        atLeastOnce = true;
+
         if (footerElements != null) {
+            boolean ancestorTreatAsArtifact = layoutContext.treatAsArtifact();
+            layoutContext.setTreatAsArtifact(treatFooterAsArtifact);
             //Positions for footers are simply added at the end
-            addHeaderFooterAreas(footerElements, tableLM.getTable().getTableFooter(), painter,
-                    true);
+            addHeaderFooterAreas(footerElements, tableLM.getTable().getTableFooter(), painter, true);
+            if (lastPos instanceof TableHFPenaltyPosition && !tableLM.getFooterFootnotes().isEmpty()) {
+                tableLM.getPSLM().addTableFooterFootnotes(getTableLM().getFooterFootnotes());
+            }
+            layoutContext.setTreatAsArtifact(ancestorTreatAsArtifact);
         }
 
         this.usedBPD += painter.getAccumulatedBPD();
 
         if (markers != null) {
-            getTableLM().getCurrentPV().addMarkers(markers,
+            getTableLM().getCurrentPV().registerMarkers(markers,
                     false, getTableLM().isFirst(firstPos), getTableLM().isLast(lastCheckPos));
         }
     }
@@ -479,14 +558,20 @@ public class TableContentLayoutManager implements PercentBaseContext {
      */
     private void addTablePartAreas(List positions, RowPainter painter, TablePart body,
             boolean isFirstPos, boolean isLastPos, boolean lastInBody, boolean lastOnPage) {
-        getTableLM().getCurrentPV().addMarkers(body.getMarkers(),
+        getTableLM().getCurrentPV().registerMarkers(body.getMarkers(),
                 true, isFirstPos, isLastPos);
+        if (body instanceof TableBody) {
+            getTableLM().registerMarkers(body.getMarkers(), true, isFirstPos, isLastPos);
+        }
         painter.startTablePart(body);
         for (Iterator iter = positions.iterator(); iter.hasNext();) {
             painter.handleTableContentPosition((TableContentPosition) iter.next());
         }
-        getTableLM().getCurrentPV().addMarkers(body.getMarkers(),
+        getTableLM().getCurrentPV().registerMarkers(body.getMarkers(),
                 false, isFirstPos, isLastPos);
+        if (body instanceof TableBody) {
+            getTableLM().registerMarkers(body.getMarkers(), false, isFirstPos, isLastPos);
+        }
         painter.endTablePart(lastInBody, lastOnPage);
     }
 
