@@ -15,14 +15,16 @@
  * limitations under the License.
  */
 
-/* $Id: TableLayoutManager.java 669118 2008-06-18 09:02:45Z jeremias $ */
+/* $Id: TableLayoutManager.java 1658340 2015-02-09 09:53:37Z ssteiner $ */
 
 package org.apache.fop.layoutmgr.table;
 
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.Map;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -33,17 +35,24 @@ import org.apache.fop.datatypes.LengthBase;
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.FObj;
+import org.apache.fop.fo.flow.Marker;
+import org.apache.fop.fo.flow.Markers;
+import org.apache.fop.fo.flow.RetrieveTableMarker;
 import org.apache.fop.fo.flow.table.Table;
 import org.apache.fop.fo.flow.table.TableColumn;
-import org.apache.fop.layoutmgr.BlockStackingLayoutManager;
+import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
+import org.apache.fop.fo.properties.KeepProperty;
+import org.apache.fop.layoutmgr.BlockLevelEventProducer;
 import org.apache.fop.layoutmgr.BreakElement;
-import org.apache.fop.layoutmgr.ConditionalElementListener;
+import org.apache.fop.layoutmgr.BreakOpportunity;
 import org.apache.fop.layoutmgr.KnuthElement;
 import org.apache.fop.layoutmgr.KnuthGlue;
 import org.apache.fop.layoutmgr.LayoutContext;
+import org.apache.fop.layoutmgr.LeafPosition;
 import org.apache.fop.layoutmgr.ListElement;
+import org.apache.fop.layoutmgr.Position;
 import org.apache.fop.layoutmgr.PositionIterator;
-import org.apache.fop.layoutmgr.RelSide;
+import org.apache.fop.layoutmgr.SpacedBorderedPaddedBlockLayoutManager;
 import org.apache.fop.layoutmgr.TraitSetter;
 import org.apache.fop.traits.MinOptMax;
 import org.apache.fop.traits.SpaceVal;
@@ -57,38 +66,48 @@ import org.apache.fop.util.BreakUtil;
  * The table then creates areas for the columns, bodies and rows
  * the render background.
  */
-public class TableLayoutManager extends BlockStackingLayoutManager 
-                implements ConditionalElementListener {
+public class TableLayoutManager extends SpacedBorderedPaddedBlockLayoutManager
+        implements BreakOpportunity {
 
     /**
      * logging instance
      */
     private static Log log = LogFactory.getLog(TableLayoutManager.class);
-    
-    private TableContentLayoutManager contentLM; 
-    private ColumnSetup columns = null;
+
+    private TableContentLayoutManager contentLM;
+    private ColumnSetup columns;
 
     private Block curBlockArea;
 
     private double tableUnit;
     private boolean autoLayout = true;
 
-    private boolean discardBorderBefore;
-    private boolean discardBorderAfter;
-    private boolean discardPaddingBefore;
-    private boolean discardPaddingAfter;
-    private MinOptMax effSpaceBefore;
-    private MinOptMax effSpaceAfter;
-    
     private int halfBorderSeparationBPD;
     private int halfBorderSeparationIPD;
 
     /** See {@link TableLayoutManager#registerColumnBackgroundArea(TableColumn, Block, int)}. */
     private List columnBackgroundAreas;
 
+    private Position auxiliaryPosition;
+
+    // this holds a possible list of TCLMs that needed to have their addAreas() repeated
+    private List<TableCellLayoutManager> savedTCLMs;
+    private boolean areAllTCLMsSaved;
+
+    private Markers tableMarkers;
+    private Markers tableFragmentMarkers;
+
+    private boolean hasRetrieveTableMarker;
+
+    private boolean repeatedHeader;
+
+    private List<List<KnuthElement>> headerFootnotes = Collections.emptyList();
+
+    private List<List<KnuthElement>> footerFootnotes = Collections.emptyList();
+
     /**
      * Temporary holder of column background informations for a table-cell's area.
-     * 
+     *
      * @see TableLayoutManager#registerColumnBackgroundArea(TableColumn, Block, int)
      */
     private static final class ColumnBackgroundInfo {
@@ -112,18 +131,25 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         this.columns = new ColumnSetup(node);
     }
 
+
+    @Override
+    protected CommonBorderPaddingBackground getCommonBorderPaddingBackground() {
+        return getTable().getCommonBorderPaddingBackground();
+    }
+
+
     /** @return the table FO */
     public Table getTable() {
         return (Table)this.fobj;
     }
-    
+
     /**
      * @return the column setup for this table.
      */
     public ColumnSetup getColumns() {
         return this.columns;
     }
-    
+
     /** {@inheritDoc} */
     public void initialize() {
         foSpaceBefore = new SpaceVal(
@@ -131,8 +157,8 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         foSpaceAfter = new SpaceVal(
                 getTable().getCommonMarginBlock().spaceAfter, this).getSpace();
         startIndent = getTable().getCommonMarginBlock().startIndent.getValue(this);
-        endIndent = getTable().getCommonMarginBlock().endIndent.getValue(this); 
-        
+        endIndent = getTable().getCommonMarginBlock().endIndent.getValue(this);
+
         if (getTable().isSeparateBorderModel()) {
             this.halfBorderSeparationBPD = getTable().getBorderSeparation().getBPD().getLength()
                     .getValue(this) / 2;
@@ -142,23 +168,23 @@ public class TableLayoutManager extends BlockStackingLayoutManager
             this.halfBorderSeparationBPD = 0;
             this.halfBorderSeparationIPD = 0;
         }
-        
-        if (!getTable().isAutoLayout() 
-                && getTable().getInlineProgressionDimension().getOptimum(this).getEnum() 
+
+        if (!getTable().isAutoLayout()
+                && getTable().getInlineProgressionDimension().getOptimum(this).getEnum()
                     != EN_AUTO) {
             autoLayout = false;
         }
     }
 
     private void resetSpaces() {
-        this.discardBorderBefore = false;        
-        this.discardBorderAfter = false;        
-        this.discardPaddingBefore = false;        
+        this.discardBorderBefore = false;
+        this.discardBorderAfter = false;
+        this.discardPaddingBefore = false;
         this.discardPaddingAfter = false;
         this.effSpaceBefore = null;
         this.effSpaceAfter = null;
     }
-    
+
     /**
      * @return half the value of border-separation.block-progression-dimension, or 0 if
      * border-collapse="collapse".
@@ -176,9 +202,9 @@ public class TableLayoutManager extends BlockStackingLayoutManager
     }
 
     /** {@inheritDoc} */
-    public LinkedList getNextKnuthElements(LayoutContext context, int alignment) {
-        
-        LinkedList returnList = new LinkedList();
+    public List getNextKnuthElements(LayoutContext context, int alignment) {
+
+        List returnList = new LinkedList();
 
         /*
          * Compute the IPD and adjust it if necessary (overconstrained)
@@ -190,28 +216,28 @@ public class TableLayoutManager extends BlockStackingLayoutManager
             updateContentAreaIPDwithOverconstrainedAdjust(contentIPD);
         } else {
             if (!getTable().isAutoLayout()) {
-                log.info("table-layout=\"fixed\" and width=\"auto\", "
-                        + "but auto-layout not supported " 
-                        + "=> assuming width=\"100%\"");
+                BlockLevelEventProducer eventProducer = BlockLevelEventProducer.Provider.get(
+                        getTable().getUserAgent().getEventBroadcaster());
+                eventProducer.tableFixedAutoWidthNotSupported(this, getTable().getLocator());
             }
             updateContentAreaIPDwithOverconstrainedAdjust();
         }
         int sumOfColumns = columns.getSumOfColumnWidths(this);
         if (!autoLayout && sumOfColumns > getContentAreaIPD()) {
             log.debug(FONode.decorateWithContextInfo(
-                    "The sum of all column widths is larger than the specified table width.", 
+                    "The sum of all column widths is larger than the specified table width.",
                     getTable()));
             updateContentAreaIPDwithOverconstrainedAdjust(sumOfColumns);
         }
         int availableIPD = referenceIPD - getIPIndents();
         if (getContentAreaIPD() > availableIPD) {
-            log.warn(FONode.decorateWithContextInfo(
-                    "The extent in inline-progression-direction (width) of a table is"
-                    + " bigger than the available space (" 
-                    + getContentAreaIPD() + "mpt > " + context.getRefIPD() + "mpt)", 
-                    getTable()));
+            BlockLevelEventProducer eventProducer = BlockLevelEventProducer.Provider.get(
+                    getTable().getUserAgent().getEventBroadcaster());
+            eventProducer.objectTooWide(this, getTable().getName(),
+                    getContentAreaIPD(), context.getRefIPD(),
+                    getTable().getLocator());
         }
-        
+
         /* initialize unit to determine computed values
          * for proportional-column-width()
          */
@@ -222,7 +248,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         if (!firstVisibleMarkServed) {
             addKnuthElementsForSpaceBefore(returnList, alignment);
         }
-        
+
         if (getTable().isSeparateBorderModel()) {
             addKnuthElementsForBorderPaddingBefore(returnList, !firstVisibleMarkServed);
             firstVisibleMarkServed = true;
@@ -234,9 +260,9 @@ public class TableLayoutManager extends BlockStackingLayoutManager
 
 
         // Elements for the table-header/footer/body
-        LinkedList contentKnuthElements = null;
+        List contentKnuthElements;
         contentLM = new TableContentLayoutManager(this);
-        LayoutContext childLC = new LayoutContext(0);
+        LayoutContext childLC = LayoutContext.newInstance();
         /*
         childLC.setStackLimit(
               MinOptMax.subtract(context.getStackLimit(),
@@ -254,12 +280,11 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         log.debug(contentKnuthElements);
         wrapPositionElements(contentKnuthElements, returnList);
 
-        if (mustKeepWithPrevious() || childLC.isKeepWithPreviousPending()) {
-            context.setFlags(LayoutContext.KEEP_WITH_PREVIOUS_PENDING);
-        }
-        if (mustKeepWithNext() || childLC.isKeepWithNextPending()) {
-            context.setFlags(LayoutContext.KEEP_WITH_NEXT_PENDING);
-        }
+        context.updateKeepWithPreviousPending(getKeepWithPrevious());
+        context.updateKeepWithPreviousPending(childLC.getKeepWithPreviousPending());
+
+        context.updateKeepWithNextPending(getKeepWithNext());
+        context.updateKeepWithNextPending(childLC.getKeepWithNextPending());
 
         if (getTable().isSeparateBorderModel()) {
             addKnuthElementsForBorderPaddingAfter(returnList, true);
@@ -271,8 +296,8 @@ public class TableLayoutManager extends BlockStackingLayoutManager
             int breakBefore = BreakUtil.compareBreakClasses(getTable().getBreakBefore(),
                     childLC.getBreakBefore());
             if (breakBefore != Constants.EN_AUTO) {
-                returnList.addFirst(new BreakElement(getAuxiliaryPosition(), 
-                        0, -KnuthElement.INFINITE, breakBefore, context));
+                returnList.add(0, new BreakElement(new LeafPosition(getParent(), 0), 0,
+                        -KnuthElement.INFINITE, breakBefore, context));
             }
         }
 
@@ -280,7 +305,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         int breakAfter = BreakUtil.compareBreakClasses(getTable().getBreakAfter(),
                 childLC.getBreakAfter());
         if (breakAfter != Constants.EN_AUTO) {
-            returnList.add(new BreakElement(getAuxiliaryPosition(), 
+            returnList.add(new BreakElement(new LeafPosition(getParent(), 0),
                     0, -KnuthElement.INFINITE, breakAfter, context));
         }
 
@@ -289,6 +314,20 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         return returnList;
     }
 
+    /** {@inheritDoc} */
+    public Position getAuxiliaryPosition() {
+        /*
+         * Redefined to return a LeafPosition instead of a NonLeafPosition. The
+         * SpaceResolver.SpaceHandlingBreakPosition constructors unwraps all
+         * NonLeafPositions, which can lead to a NPE when a break in a table occurs at a
+         * page with different ipd.
+         */
+        if (auxiliaryPosition == null) {
+            auxiliaryPosition = new LeafPosition(this, 0);
+        }
+        return auxiliaryPosition;
+    }
+
     /**
      * Registers the given area, that will be used to render the part of column background
      * covered by a table-cell. If percentages are used to place the background image, the
@@ -296,7 +335,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
      * be known. The traits can't then be set when the areas for the cell are created
      * since at that moment this bpd is yet unknown. So they will instead be set in
      * TableLM's {@link #addAreas(PositionIterator, LayoutContext)} method.
-     * 
+     *
      * @param column the table-column element from which the cell gets background
      * informations
      * @param backgroundArea the block of the cell's dimensions that will hold the column
@@ -322,21 +361,21 @@ public class TableLayoutManager extends BlockStackingLayoutManager
     public void addAreas(PositionIterator parentIter,
                          LayoutContext layoutContext) {
         getParentArea(null);
-        getPSLM().addIDToPage(getTable().getId());
+        addId();
 
         // add space before, in order to implement display-align = "center" or "after"
         if (layoutContext.getSpaceBefore() != 0) {
-            addBlockSpacing(0.0, new MinOptMax(layoutContext.getSpaceBefore()));
+            addBlockSpacing(0.0, MinOptMax.getInstance(layoutContext.getSpaceBefore()));
         }
 
         int startXOffset = getTable().getCommonMarginBlock().startIndent.getValue(this);
-        
+
         // add column, body then row areas
 
         // BPD of the table, i.e., height of its content; table's borders and paddings not counted
         int tableHeight = 0;
         //Body childLM;
-        LayoutContext lc = new LayoutContext(0);
+        LayoutContext lc = LayoutContext.offspringOf(layoutContext);
 
 
         lc.setRefIPD(getContentAreaIPD());
@@ -358,31 +397,31 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         }
 
         if (getTable().isSeparateBorderModel()) {
-            TraitSetter.addBorders(curBlockArea, 
-                    getTable().getCommonBorderPaddingBackground(), 
+            TraitSetter.addBorders(curBlockArea,
+                    getTable().getCommonBorderPaddingBackground(),
                     discardBorderBefore, discardBorderAfter, false, false, this);
-            TraitSetter.addPadding(curBlockArea, 
-                    getTable().getCommonBorderPaddingBackground(), 
+            TraitSetter.addPadding(curBlockArea,
+                    getTable().getCommonBorderPaddingBackground(),
                     discardPaddingBefore, discardPaddingAfter, false, false, this);
         }
-        TraitSetter.addBackground(curBlockArea, 
+        TraitSetter.addBackground(curBlockArea,
                 getTable().getCommonBorderPaddingBackground(),
                 this);
         TraitSetter.addMargins(curBlockArea,
-                getTable().getCommonBorderPaddingBackground(), 
+                getTable().getCommonBorderPaddingBackground(),
                 startIndent, endIndent,
                 this);
-        TraitSetter.addBreaks(curBlockArea, 
+        TraitSetter.addBreaks(curBlockArea,
                 getTable().getBreakBefore(), getTable().getBreakAfter());
-        TraitSetter.addSpaceBeforeAfter(curBlockArea, layoutContext.getSpaceAdjust(), 
+        TraitSetter.addSpaceBeforeAfter(curBlockArea, layoutContext.getSpaceAdjust(),
                 effSpaceBefore, effSpaceAfter);
 
         flush();
 
         resetSpaces();
         curBlockArea = null;
-        
-        getPSLM().notifyEndOfLayout(((Table)getFObj()).getId());
+
+        notifyEndOfLayout();
     }
 
     /**
@@ -403,12 +442,12 @@ public class TableLayoutManager extends BlockStackingLayoutManager
             curBlockArea = new Block();
             // Set up dimensions
             // Must get dimensions from parent area
-            /*Area parentArea =*/ parentLM.getParentArea(curBlockArea);
-            
+            /*Area parentArea =*/ parentLayoutManager.getParentArea(curBlockArea);
+
             TraitSetter.setProducerID(curBlockArea, getTable().getId());
 
             curBlockArea.setIPD(getContentAreaIPD());
-            
+
             setCurrentArea(curBlockArea);
         }
         return curBlockArea;
@@ -427,7 +466,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
 
     /**
      * Adds the given area to this layout manager's area, without updating the used bpd.
-     * 
+     *
      * @param background an area
      */
     void addBackgroundArea(Block background) {
@@ -443,39 +482,28 @@ public class TableLayoutManager extends BlockStackingLayoutManager
     /** {@inheritDoc} */
     public void discardSpace(KnuthGlue spaceGlue) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public boolean mustKeepTogether() {
-        //TODO Keeps will have to be more sophisticated sooner or later
-        return super.mustKeepTogether() 
-                || !getTable().getKeepTogether().getWithinPage().isAuto()
-                || !getTable().getKeepTogether().getWithinColumn().isAuto();
+    /** {@inheritDoc} */
+    public KeepProperty getKeepTogetherProperty() {
+        return getTable().getKeepTogether();
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public boolean mustKeepWithPrevious() {
-        return !getTable().getKeepWithPrevious().getWithinPage().isAuto()
-                || !getTable().getKeepWithPrevious().getWithinColumn().isAuto();
+    /** {@inheritDoc} */
+    public KeepProperty getKeepWithPreviousProperty() {
+        return getTable().getKeepWithPrevious();
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public boolean mustKeepWithNext() {
-        return !getTable().getKeepWithNext().getWithinPage().isAuto()
-                || !getTable().getKeepWithNext().getWithinColumn().isAuto();
+    /** {@inheritDoc} */
+    public KeepProperty getKeepWithNextProperty() {
+        return getTable().getKeepWithNext();
     }
 
     // --------- Property Resolution related functions --------- //
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
     public int getBaseLength(int lengthBase, FObj fobj) {
         // Special handler for TableColumn width specifications
@@ -498,50 +526,143 @@ public class TableLayoutManager extends BlockStackingLayoutManager
             }
         }
     }
-    
+
     /** {@inheritDoc} */
-    public void notifySpace(RelSide side, MinOptMax effectiveLength) {
-        if (RelSide.BEFORE == side) {
-            if (log.isDebugEnabled()) {
-                log.debug(this + ": Space " + side + ", " 
-                        + this.effSpaceBefore + "-> " + effectiveLength);
+    public void reset() {
+        super.reset();
+        curBlockArea = null;
+        tableUnit = 0.0;
+    }
+
+    /**
+     * Saves a TableCellLayoutManager for later use.
+     *
+     * @param tclm a TableCellLayoutManager that has a RetrieveTableMarker
+     */
+    protected void saveTableHeaderTableCellLayoutManagers(TableCellLayoutManager tclm) {
+        if (savedTCLMs == null) {
+            savedTCLMs = new ArrayList<TableCellLayoutManager>();
+        }
+        if (!areAllTCLMsSaved) {
+            savedTCLMs.add(tclm);
+        }
+    }
+
+    /**
+     * Calls addAreas() for each of the saved TableCellLayoutManagers.
+     */
+    protected void repeatAddAreasForSavedTableHeaderTableCellLayoutManagers() {
+        if (savedTCLMs == null) {
+            return;
+        }
+        // if we get to this stage then we are at the footer of the table fragment; this means that no more
+        // different TCLM need to be saved (we already have all); we flag the list as being complete then
+        areAllTCLMsSaved = true;
+        for (int i = 0; i < savedTCLMs.size(); i++) {
+            TableCellLayoutManager tclm = savedTCLMs.get(i);
+            if (this.repeatedHeader) {
+                tclm.setHasRepeatedHeader(true);
             }
-            this.effSpaceBefore = effectiveLength;
+            tclm.repeatAddAreas();
+        }
+    }
+
+    /**
+     * Resolves a RetrieveTableMarker by finding a qualifying Marker to which it is bound to.
+     * @param rtm the RetrieveTableMarker to be resolved
+     * @return a bound RetrieveTableMarker instance or null if no qualifying Marker found
+     */
+    public RetrieveTableMarker resolveRetrieveTableMarker(RetrieveTableMarker rtm) {
+        String name = rtm.getRetrieveClassName();
+        int originalPosition = rtm.getPosition();
+        boolean changedPosition = false;
+
+        Marker mark = null;
+        // try the primary retrieve scope area, which is the same as table-fragment
+        mark = (tableFragmentMarkers == null) ? null : tableFragmentMarkers.resolve(rtm);
+        if (mark == null && rtm.getBoundary() != Constants.EN_TABLE_FRAGMENT) {
+            rtm.changePositionTo(Constants.EN_LAST_ENDING);
+            changedPosition = true;
+            // try the page scope area
+            mark = getCurrentPV().resolveMarker(rtm);
+            if (mark == null && rtm.getBoundary() != Constants.EN_PAGE) {
+                // try the table scope area
+                mark = (tableMarkers == null) ? null : tableMarkers.resolve(rtm);
+            }
+        }
+        if (changedPosition) {
+            // so that the next time it is called looks unchanged
+            rtm.changePositionTo(originalPosition);
+        }
+        if (mark == null) {
+            log.debug("found no marker with name: " + name);
+            return null;
         } else {
-            if (log.isDebugEnabled()) {
-                log.debug(this + ": Space " + side + ", " 
-                        + this.effSpaceAfter + "-> " + effectiveLength);
-            }
-            this.effSpaceAfter = effectiveLength;
+            rtm.bindMarker(mark);
+            return rtm;
         }
     }
 
-    /** {@inheritDoc} */
-    public void notifyBorder(RelSide side, MinOptMax effectiveLength) {
-        if (effectiveLength == null) {
-            if (RelSide.BEFORE == side) {
-                this.discardBorderBefore = true;
-            } else {
-                this.discardBorderAfter = true;
-            }
+    /**
+     * Register the markers for this table.
+     *
+     * @param marks the map of markers to add
+     * @param starting if the area being added is starting or ending
+     * @param isfirst if the area being added has is-first trait
+     * @param islast if the area being added has is-last trait
+     */
+    public void registerMarkers(Map<String, Marker> marks, boolean starting, boolean isfirst,
+            boolean islast) {
+        if (tableMarkers == null) {
+            tableMarkers = new Markers();
         }
-        if (log.isDebugEnabled()) {
-            log.debug(this + ": Border " + side + " -> " + effectiveLength);
+        tableMarkers.register(marks, starting, isfirst, islast);
+        if (tableFragmentMarkers == null) {
+            tableFragmentMarkers = new Markers();
         }
+        tableFragmentMarkers.register(marks, starting, isfirst, islast);
     }
 
-    /** {@inheritDoc} */
-    public void notifyPadding(RelSide side, MinOptMax effectiveLength) {
-        if (effectiveLength == null) {
-            if (RelSide.BEFORE == side) {
-                this.discardPaddingBefore = true;
-            } else {
-                this.discardPaddingAfter = true;
-            }
-        }
-        if (log.isDebugEnabled()) {
-            log.debug(this + ": Padding " + side + " -> " + effectiveLength);
+    /**
+     * Clears the list of markers in the current table fragment. Should be called just before starting a new
+     * header (that belongs to the next table fragment).
+     */
+    protected void clearTableFragmentMarkers() {
+        tableFragmentMarkers = null;
+    }
+
+    public void flagAsHavingRetrieveTableMarker() {
+        hasRetrieveTableMarker = true;
+    }
+
+    protected void possiblyRegisterMarkersForTables(Map<String, Marker> markers, boolean isStarting,
+            boolean isFirst, boolean isLast) {
+        // note: if we allow table-footer after a table-body this check should not be made and the markers
+        // should be registered regardless because the retrieval may be done only in the footer
+        if (hasRetrieveTableMarker) {
+            registerMarkers(markers, isStarting, isFirst, isLast);
         }
+        super.possiblyRegisterMarkersForTables(markers, isStarting, isFirst, isLast);
+    }
+
+    void setHeaderFootnotes(List<List<KnuthElement>> footnotes) {
+        this.headerFootnotes = footnotes;
+    }
+
+    List<List<KnuthElement>> getHeaderFootnotes() {
+        return headerFootnotes;
+    }
+
+    void setFooterFootnotes(List<List<KnuthElement>> footnotes) {
+        this.footerFootnotes = footnotes;
+    }
+
+    public void setRepeateHeader(boolean repeateHeader) {
+        this.repeatedHeader = repeateHeader;
+    }
+
+    List<List<KnuthElement>> getFooterFootnotes() {
+        return footerFootnotes;
     }
 
 }
