@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TableStepper.java 1309636 2012-04-05 02:41:59Z gadams $ */
+/* $Id: TableStepper.java 1681435 2015-05-24 11:14:22Z adelmelle $ */
 
 package org.apache.fop.layoutmgr.table;
 
@@ -158,6 +158,7 @@ public class TableStepper {
         for (int i = 0; i < columnCount; i++) {
             GridUnit gu = row.getGridUnit(i);
             if (!gu.isEmpty() && gu.isPrimary()) {
+                assert (gu instanceof PrimaryGridUnit);
                 activeCellList.add(new ActiveCell((PrimaryGridUnit) gu, row, rowIndex,
                         previousRowsLength, getTableLM()));
             }
@@ -419,7 +420,9 @@ public class TableStepper {
         for (Iterator iter = activeCells.iterator(); iter.hasNext();) {
             ActiveCell activeCell = (ActiveCell) iter.next();
             if (activeCell.endsOnRow(activeRowIndex)) {
-                rowFinished &= activeCell.finishes(step);
+                if (!activeCell.finishes(step)) {
+                    rowFinished = false;
+                }
             }
         }
         if (rowFinished) {
