@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BitmapImage.java 1357883 2012-07-05 20:29:53Z gadams $ */
+/* $Id: BitmapImage.java 1661887 2015-02-24 11:23:44Z ssteiner $ */
 
 package org.apache.fop.pdf;
 
@@ -34,7 +34,7 @@ public class BitmapImage implements PDFImage {
     private PDFDeviceColorSpace colorSpace;
     private byte[] bitmaps;
     private PDFReference maskRef;
-    private PDFColor transparent = null;
+    private PDFColor transparent;
     private String key;
     private PDFDocument pdfDoc;
     private PDFFilter pdfFilter;
@@ -51,7 +51,7 @@ public class BitmapImage implements PDFImage {
      * @param mask the transparency mask reference if any
      */
     public BitmapImage(String k, int width, int height, byte[] data,
-                  String mask) {
+                  PDFReference mask) {
         this.key = k;
         this.height = height;
         this.width = width;
@@ -59,7 +59,7 @@ public class BitmapImage implements PDFImage {
         this.colorSpace = new PDFDeviceColorSpace(PDFDeviceColorSpace.DEVICE_RGB);
         this.bitmaps = data;
         if (mask != null) {
-            maskRef = new PDFReference(mask);
+            maskRef = mask;
         }
     }
 
