@@ -15,15 +15,15 @@
  * limitations under the License.
  */
 
-/* $Id: PDFArray.java 830293 2009-10-27 19:07:52Z vhennebert $ */
+/* $Id: PDFArray.java 1661887 2015-02-24 11:23:44Z ssteiner $ */
 
 package org.apache.fop.pdf;
 
 import java.io.IOException;
 import java.io.OutputStream;
-import java.io.Writer;
-import java.util.Collection;
+import java.util.ArrayList;
 import java.util.List;
+import java.util.Set;
 
 import org.apache.commons.io.output.CountingOutputStream;
 
@@ -34,7 +34,7 @@ public class PDFArray extends PDFObject {
     /**
      * List holding the values of this array
      */
-    protected List values = new java.util.ArrayList();
+    protected List<Object> values = new java.util.ArrayList<Object>();
 
     /**
      * Create a new, empty array object
@@ -49,7 +49,7 @@ public class PDFArray extends PDFObject {
      * Create a new, empty array object with no parent.
      */
     public PDFArray() {
-        this(null);
+        this((PDFObject) null);
     }
 
     /**
@@ -62,7 +62,7 @@ public class PDFArray extends PDFObject {
         super(parent);
 
         for (int i = 0, c = values.length; i < c; i++) {
-            this.values.add(new Integer(values[i]));
+            this.values.add(Integer.valueOf(values[i]));
         }
     }
 
@@ -85,13 +85,31 @@ public class PDFArray extends PDFObject {
      * @param parent the array's parent if any
      * @param values the actual values wrapped by this object
      */
-    public PDFArray(PDFObject parent, Collection values) {
+    public PDFArray(PDFObject parent, List<?> values) {
         /* generic creation of PDF object */
         super(parent);
 
         this.values.addAll(values);
     }
 
+    /**
+     * Creates an array object made of the given elements.
+     *
+     * @param elements the array content
+     */
+    public PDFArray(Object... elements) {
+        this(null, elements);
+    }
+
+    /**
+     * Creates an array object made of the given elements.
+     *
+     * @param elements the array content
+     */
+    public PDFArray(List<?> elements) {
+        this(null, elements);
+    }
+
     /**
      * Create the array object
      * @param parent the array's parent if any
@@ -172,30 +190,39 @@ public class PDFArray extends PDFObject {
         this.values.add(new Double(value));
     }
 
-    /** {@inheritDoc} */
-    protected int output(OutputStream stream) throws IOException {
-        CountingOutputStream cout = new CountingOutputStream(stream);
-        Writer writer = PDFDocument.getWriterFor(cout);
-        if (hasObjectNumber()) {
-            writer.write(getObjectID());
-        }
+    /**
+     * Clears the PDF array.
+     */
+    public void clear() {
+        this.values.clear();
+    }
 
-        writer.write('[');
+    /** {@inheritDoc} */
+    @Override
+    public int output(OutputStream stream) throws IOException {
+        CountingOutputStream cout = new CountingOutputStream(stream);
+        StringBuilder textBuffer = new StringBuilder(64);
+        textBuffer.append('[');
         for (int i = 0; i < values.size(); i++) {
             if (i > 0) {
-                writer.write(' ');
+                textBuffer.append(' ');
             }
             Object obj = this.values.get(i);
-            formatObject(obj, cout, writer);
+            formatObject(obj, cout, textBuffer);
         }
-        writer.write(']');
-
-        if (hasObjectNumber()) {
-            writer.write("\nendobj\n");
-        }
-
-        writer.flush();
+        textBuffer.append(']');
+        PDFDocument.flushTextBuffer(textBuffer, cout);
         return cout.getCount();
     }
 
+    @Override
+    public void getChildren(Set<PDFObject> children) {
+        List<Object> contents = new ArrayList<Object>();
+        for (Object c : values) {
+            if (!(c instanceof PDFReference)) {
+                contents.add(c);
+            }
+        }
+        PDFDictionary.getChildren(contents, children);
+    }
 }
