@@ -15,10 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: PDFCIDSystemInfo.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: PDFCIDSystemInfo.java 1484190 2013-05-18 22:25:52Z lbernardo $ */
 
 package org.apache.fop.pdf;
 
+import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+
 // based on work by Takayuki Takeuchi
 
 /**
@@ -64,5 +67,24 @@ public class PDFCIDSystemInfo extends PDFObject {
         return p.toString();
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    public byte[] toPDF() {
+        ByteArrayOutputStream bout = new ByteArrayOutputStream(128);
+        try {
+            bout.write(encode("<< /Registry "));
+            bout.write(encodeText(registry));
+            bout.write(encode(" /Ordering "));
+            bout.write(encodeText(ordering));
+            bout.write(encode(" /Supplement "));
+            bout.write(encode(Integer.toString(supplement)));
+            bout.write(encode(" >>"));
+        } catch (IOException ioe) {
+            log.error("Ignored I/O exception", ioe);
+        }
+        return bout.toByteArray();
+    }
+
 }
 
