@@ -15,13 +15,12 @@
  * limitations under the License.
  */
 
-/* $Id: PDFDocument.java 1305467 2012-03-26 17:39:20Z vhennebert $ */
+/* $Id: PDFDocument.java 1661887 2015-02-24 11:23:44Z ssteiner $ */
 
 package org.apache.fop.pdf;
 
 // Java
 import java.io.IOException;
-import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
 import java.security.NoSuchAlgorithmException;
@@ -38,6 +37,7 @@ import java.util.Map;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.fop.pdf.StandardStructureAttributes.Table.Scope;
 import org.apache.fop.pdf.xref.CrossReferenceStream;
 import org.apache.fop.pdf.xref.CrossReferenceTable;
 import org.apache.fop.pdf.xref.TrailerDictionary;
@@ -78,18 +78,18 @@ public class PDFDocument {
     private Log log = LogFactory.getLog("org.apache.fop.pdf");
 
     /** the current character position */
-    private long position;
+    protected long position;
 
     /** the character position of each object */
-    private List<Long> indirectObjectOffsets = new ArrayList<Long>();
+    protected List<Long> indirectObjectOffsets = new ArrayList<Long>();
 
-    private Collection<PDFStructElem> structureTreeElements;
+    protected List<PDFStructElem> structureTreeElements;
 
     /** List of objects to write in the trailer */
-    private List<PDFObject> trailerObjects = new ArrayList<PDFObject>();
+    protected List<PDFObject> trailerObjects = new ArrayList<PDFObject>();
 
     /** the objects themselves */
-    private List<PDFObject> objects = new LinkedList<PDFObject>();
+    protected List<PDFObject> objects = new LinkedList<PDFObject>();
 
     /** Controls the PDF version of this document */
     private VersionController versionController;
@@ -128,6 +128,8 @@ public class PDFDocument {
     /** the counter for XObject numbering */
     private int xObjectCount;
 
+    protected int gStateObjectCount;
+
     /* TODO: Should be modified (works only for image subtype) */
     private Map<String, PDFXObject> xObjectsMap = new HashMap<String, PDFXObject>();
 
@@ -155,12 +157,26 @@ public class PDFDocument {
 
     private List<PDFLaunch> launches = new ArrayList<PDFLaunch>();
 
+    protected List<PDFPage> pageObjs = new ArrayList<PDFPage>();
+
+    private List<PDFLayer> layers;
+
+    private List<PDFNavigator> navigators;
+
+    private List<PDFNavigatorAction> navigatorActions;
+
     private PDFFactory factory;
 
     private FileIDGenerator fileIDGenerator;
 
     private boolean accessibilityEnabled;
 
+    private boolean mergeFontsEnabled;
+
+    private boolean linearizationEnabled;
+
+    protected boolean outputStarted;
+
     /**
      * Creates an empty PDF document.
      *
@@ -339,6 +355,14 @@ public class PDFDocument {
         return this.root;
     }
 
+    /**
+     * Get the Structural Tree Collection for this document
+     * @return
+     */
+    public List<PDFStructElem> getStructureTreeElements() {
+        return structureTreeElements;
+    }
+
     /**
      * Creates and returns a StructTreeRoot object.
      *
@@ -355,19 +379,20 @@ public class PDFDocument {
     }
 
     /**
-     * Creates and returns a structure element.
-     *
-     * @param structureType the structure type of the new element (value for the
-     * S entry)
-     * @param parent the parent of the new structure element in the structure
-     * hierarchy
-     * @return a dictionary of type StructElem
+     * Adds the given element to the structure tree.
      */
-    public PDFStructElem makeStructureElement(PDFName structureType, PDFObject parent) {
-        PDFStructElem structElem = new PDFStructElem(parent, structureType);
+    public void registerStructureElement(PDFStructElem structElem) {
         assignObjectNumber(structElem);
         structureTreeElements.add(structElem);
-        return structElem;
+    }
+
+    /**
+     * Assigns the given scope to the given element and adds it to the structure tree. The
+     * scope may not be added if it's not compatible with this document's PDF version.
+     */
+    public void registerStructureElement(PDFStructElem structElem, Scope scope) {
+        registerStructureElement(structElem);
+        versionController.addTableHeaderScopeAttribute(structElem, scope);
     }
 
     /**
@@ -389,6 +414,22 @@ public class PDFDocument {
     public PDFObject registerObject(PDFObject obj) {
         assignObjectNumber(obj);
         addObject(obj);
+        if (obj instanceof AbstractPDFStream) {
+            ((AbstractPDFStream) obj).registerChildren();
+        }
+        return obj;
+    }
+
+    /**
+     * Registers a {@link PDFObject} in this PDF document at end.
+     * The object is assigned a new object number.
+     *
+     * @param obj {@link PDFObject} to add
+     * @return the added {@link PDFObject} added (with its object number set)
+     */
+    <T extends PDFObject> T registerTrailerObject(T obj) {
+        assignObjectNumber(obj);
+        addTrailerObject(obj);
         return obj;
     }
 
@@ -399,6 +440,9 @@ public class PDFDocument {
      * @param obj {@link PDFObject} to assign a number to
      */
     public void assignObjectNumber(PDFObject obj) {
+        if (outputStarted && isLinearizationEnabled()) {
+            throw new IllegalStateException("Can't assign number after start of output");
+        }
         if (obj == null) {
             throw new NullPointerException("obj must not be null");
         }
@@ -414,7 +458,7 @@ public class PDFDocument {
                     + "PDFObject already has a parent PDFDocument");
         }
 
-        obj.setObjectNumber(++this.objectcount);
+        obj.setObjectNumber(this);
 
         if (currentParent == null) {
             obj.setDocument(this);
@@ -463,6 +507,7 @@ public class PDFDocument {
         }
         if (obj instanceof PDFPage) {
             this.pages.notifyKidRegistered((PDFPage)obj);
+            pageObjs.add((PDFPage) obj);
         }
         if (obj instanceof PDFLaunch) {
             this.launches.add((PDFLaunch) obj);
@@ -476,6 +521,24 @@ public class PDFDocument {
         if (obj instanceof PDFGoToRemote) {
             this.gotoremotes.add((PDFGoToRemote) obj);
         }
+        if (obj instanceof PDFLayer) {
+            if (this.layers == null) {
+                this.layers = new ArrayList<PDFLayer>();
+            }
+            this.layers.add((PDFLayer) obj);
+        }
+        if (obj instanceof PDFNavigator) {
+            if (this.navigators == null) {
+                this.navigators = new ArrayList<PDFNavigator>();
+            }
+            this.navigators.add((PDFNavigator) obj);
+        }
+        if (obj instanceof PDFNavigatorAction) {
+            if (this.navigatorActions == null) {
+                this.navigatorActions = new ArrayList<PDFNavigatorAction>();
+            }
+            this.navigatorActions.add((PDFNavigatorAction) obj);
+        }
     }
 
     /**
@@ -511,14 +574,25 @@ public class PDFDocument {
     public void setEncryption(PDFEncryptionParams params) {
         getProfile().verifyEncryptionAllowed();
         fileIDGenerator = FileIDGenerator.getRandomFileIDGenerator();
-        this.encryption = PDFEncryptionManager.newInstance(++this.objectcount, params, this);
+        this.encryption = PDFEncryptionManager.newInstance(params, this);
         if (this.encryption != null) {
             PDFObject pdfObject = (PDFObject)this.encryption;
             addTrailerObject(pdfObject);
+            try {
+                if (encryption.getPDFVersion().compareTo(versionController.getPDFVersion()) > 0) {
+                    versionController.setPDFVersion(encryption.getPDFVersion());
+                }
+            } catch (IllegalStateException ise) {
+                log.warn("Configured encryption requires PDF version " + encryption.getPDFVersion()
+                        + " but version has been set to " + versionController.getPDFVersion() + ".");
+                throw ise;
+            }
         } else {
-            log.warn(
-                "PDF encryption is unavailable. PDF will be "
-                    + "generated without encryption.");
+            log.warn("PDF encryption is unavailable. PDF will be generated without encryption.");
+            if (params.getEncryptionLengthInBits() == 256) {
+                log.warn("Make sure the JCE Unlimited Strength Jurisdiction Policy files are available."
+                        + "AES 256 encryption cannot be performed without them.");
+            }
         }
     }
 
@@ -718,24 +792,6 @@ public class PDFDocument {
         return this.fontMap;
     }
 
-    /**
-     * Resolve a URI.
-     *
-     * @param uri the uri to resolve
-     * @throws java.io.FileNotFoundException if the URI could not be resolved
-     * @return the InputStream from the URI.
-     */
-    protected InputStream resolveURI(String uri)
-        throws java.io.FileNotFoundException {
-        try {
-            /* TODO: Temporary hack to compile, improve later */
-            return new java.net.URL(uri).openStream();
-        } catch (Exception e) {
-            throw new java.io.FileNotFoundException(
-                "URI could not be resolved (" + e.getMessage() + "): " + uri);
-        }
-    }
-
     /**
      * Get an image from the image map.
      *
@@ -807,7 +863,7 @@ public class PDFDocument {
         PDFImageXObject xObject = (PDFImageXObject)this.xObjectsMap.get(key);
         if (xObject != null) {
             if (res != null) {
-                res.getPDFResources().addXObject(xObject);
+                res.addXObject(xObject);
             }
             return xObject;
         }
@@ -819,7 +875,7 @@ public class PDFDocument {
         registerObject(xObject);
         this.resources.addXObject(xObject);
         if (res != null) {
-            res.getPDFResources().addXObject(xObject);
+            res.addXObject(xObject);
         }
         this.xObjectsMap.put(key, xObject);
         return xObject;
@@ -847,7 +903,7 @@ public class PDFDocument {
         PDFFormXObject xObject = (PDFFormXObject)xObjectsMap.get(key);
         if (xObject != null) {
             if (res != null) {
-                res.getPDFResources().addXObject(xObject);
+                res.addXObject(xObject);
             }
             return xObject;
         }
@@ -859,7 +915,7 @@ public class PDFDocument {
         registerObject(xObject);
         this.resources.addXObject(xObject);
         if (res != null) {
-            res.getPDFResources().addXObject(xObject);
+            res.addXObject(xObject);
         }
         this.xObjectsMap.put(key, xObject);
         return xObject;
@@ -897,6 +953,34 @@ public class PDFDocument {
         this.accessibilityEnabled = enableAccessibility;
     }
 
+    /**
+     *
+     */
+    public PDFReference resolveExtensionReference(String id) {
+        if (layers != null) {
+            for (PDFLayer layer : layers) {
+                if (layer.hasId(id)) {
+                    return layer.makeReference();
+                }
+            }
+        }
+        if (navigators != null) {
+            for (PDFNavigator navigator : navigators) {
+                if (navigator.hasId(id)) {
+                    return navigator.makeReference();
+                }
+            }
+        }
+        if (navigatorActions != null) {
+            for (PDFNavigatorAction action : navigatorActions) {
+                if (action.hasId(id)) {
+                    return action.makeReference();
+                }
+            }
+        }
+        return null;
+    }
+
     /**
      * Writes out the entire document
      *
@@ -904,6 +988,7 @@ public class PDFDocument {
      * @throws IOException if there is an exception writing to the output stream
      */
     public void output(OutputStream stream) throws IOException {
+        outputStarted = true;
         //Write out objects until the list is empty. This approach (used with a
         //LinkedList) allows for output() methods to create and register objects
         //on the fly even during serialization.
@@ -913,9 +998,29 @@ public class PDFDocument {
         }
     }
 
-    private void streamIndirectObject(PDFObject o, OutputStream stream) throws IOException {
+    protected void writeTrailer(OutputStream stream, int first, int last, int size, long mainOffset, long startxref)
+            throws IOException {
+        TrailerOutputHelper trailerOutputHelper = mayCompressStructureTreeElements()
+                ? new CompressedTrailerOutputHelper()
+                : new UncompressedTrailerOutputHelper();
+        if (structureTreeElements != null) {
+            trailerOutputHelper.outputStructureTreeElements(stream);
+        }
+        TrailerDictionary trailerDictionary = createTrailerDictionary(mainOffset != 0);
+        if (mainOffset != 0) {
+            trailerDictionary.getDictionary().put("Prev", mainOffset);
+        }
+        trailerOutputHelper.outputCrossReferenceObject(stream, trailerDictionary, first, last, size);
+        String trailer = "\nstartxref\n" + startxref + "\n%%EOF\n";
+        stream.write(encode(trailer));
+    }
+
+    protected int streamIndirectObject(PDFObject o, OutputStream stream) throws IOException {
+        outputStarted = true;
         recordObjectOffset(o);
-        this.position += outputIndirectObject(o, stream);
+        int len = outputIndirectObject(o, stream);
+        this.position += len;
+        return len;
     }
 
     private void streamIndirectObjects(Collection<? extends PDFObject> objects, OutputStream stream)
@@ -926,7 +1031,7 @@ public class PDFDocument {
     }
 
     private void recordObjectOffset(PDFObject object) {
-        int index = object.getObjectNumber() - 1;
+        int index = object.getObjectNumber().getNumber() - 1;
         while (indirectObjectOffsets.size() <= index) {
             indirectObjectOffsets.add(null);
         }
@@ -1015,29 +1120,43 @@ public class PDFDocument {
             trailerOutputHelper.outputStructureTreeElements(stream);
         }
         streamIndirectObjects(trailerObjects, stream);
-        TrailerDictionary trailerDictionary = createTrailerDictionary();
-        long startxref = trailerOutputHelper.outputCrossReferenceObject(stream, trailerDictionary);
-        String trailer = "startxref\n" + startxref + "\n%%EOF\n";
+        TrailerDictionary trailerDictionary = createTrailerDictionary(true);
+        long startxref = trailerOutputHelper.outputCrossReferenceObject(stream, trailerDictionary, 0,
+                indirectObjectOffsets.size(), indirectObjectOffsets.size());
+        String trailer = "\nstartxref\n" + startxref + "\n%%EOF\n";
         stream.write(encode(trailer));
     }
 
     private boolean mayCompressStructureTreeElements() {
         return accessibilityEnabled
-                && versionController.getPDFVersion().compareTo(Version.V1_5) >= 0;
+                && versionController.getPDFVersion().compareTo(Version.V1_5) >= 0
+                && !isLinearizationEnabled();
     }
 
-    private TrailerDictionary createTrailerDictionary() {
+    private TrailerDictionary createTrailerDictionary(boolean addRoot) {
         FileIDGenerator gen = getFileIDGenerator();
-        TrailerDictionary trailerDictionary = new TrailerDictionary(this)
-                .setRoot(root)
-                .setInfo(info)
-                .setFileID(gen.getOriginalFileID(), gen.getUpdatedFileID());
+        TrailerDictionary trailerDictionary = new TrailerDictionary(this);
+        if (addRoot) {
+            trailerDictionary.setRoot(root).setInfo(info);
+        }
+        trailerDictionary.setFileID(gen.getOriginalFileID(), gen.getUpdatedFileID());
         if (isEncryptionActive()) {
             trailerDictionary.setEncryption(encryption);
         }
         return trailerDictionary;
     }
 
+    public boolean isMergeFontsEnabled() {
+        return mergeFontsEnabled;
+    }
+
+    public void setMergeFontsEnabled(boolean mergeFontsEnabled) {
+        this.mergeFontsEnabled = mergeFontsEnabled;
+        if (mergeFontsEnabled) {
+            getResources().createFontsAsObj();
+        }
+    }
+
     private interface TrailerOutputHelper {
 
         void outputStructureTreeElements(OutputStream stream) throws IOException;
@@ -1045,7 +1164,8 @@ public class PDFDocument {
         /**
          * @return the offset of the cross-reference object (the value of startxref)
          */
-        long outputCrossReferenceObject(OutputStream stream, TrailerDictionary trailerDictionary)
+        long outputCrossReferenceObject(OutputStream stream, TrailerDictionary trailerDictionary,
+                                        int first, int last, int size)
                 throws IOException;
     }
 
@@ -1057,9 +1177,9 @@ public class PDFDocument {
         }
 
         public long outputCrossReferenceObject(OutputStream stream,
-                TrailerDictionary trailerDictionary) throws IOException {
+                TrailerDictionary trailerDictionary, int first, int last, int size) throws IOException {
             new CrossReferenceTable(trailerDictionary, position,
-                    indirectObjectOffsets).output(stream);
+                    indirectObjectOffsets, first, last, size).output(stream);
             return position;
         }
     }
@@ -1078,7 +1198,7 @@ public class PDFDocument {
         }
 
         public long outputCrossReferenceObject(OutputStream stream,
-                TrailerDictionary trailerDictionary) throws IOException {
+                TrailerDictionary trailerDictionary, int first, int last, int size) throws IOException {
             // Outputting the object streams should not have created new indirect objects
             assert objects.isEmpty();
             new CrossReferenceStream(PDFDocument.this, ++objectcount, trailerDictionary, position,
@@ -1103,4 +1223,13 @@ public class PDFDocument {
         }
         return fileIDGenerator;
     }
+
+    public boolean isLinearizationEnabled() {
+        return linearizationEnabled;
+    }
+
+    public void setLinearizationEnabled(boolean b) {
+        linearizationEnabled = b;
+    }
+
 }
