@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFEncryptionParams.java 1154998 2011-08-08 15:51:43Z vhennebert $ */
+/* $Id: PDFEncryptionParams.java 1484190 2013-05-18 22:25:52Z lbernardo $ */
 
 package org.apache.fop.pdf;
 
@@ -35,8 +35,9 @@ public class PDFEncryptionParams {
     private boolean allowAccessContent = true;
     private boolean allowAssembleDocument = true;
     private boolean allowPrintHq = true;
+    private boolean encryptMetadata = true;
 
-    private int encryptionLengthInBits = 40;
+    private int encryptionLengthInBits = 128;
 
     /**
      * Creates a new instance.
@@ -51,13 +52,15 @@ public class PDFEncryptionParams {
                                boolean allowPrint,
                                boolean allowCopyContent,
                                boolean allowEditContent,
-                               boolean allowEditAnnotations) {
+                               boolean allowEditAnnotations,
+                               boolean encryptMetadata) {
         setUserPassword(userPassword);
         setOwnerPassword(ownerPassword);
         setAllowPrint(allowPrint);
         setAllowCopyContent(allowCopyContent);
         setAllowEditContent(allowEditContent);
         setAllowEditAnnotations(allowEditAnnotations);
+        this.encryptMetadata = encryptMetadata;
     }
 
     /**
@@ -84,6 +87,7 @@ public class PDFEncryptionParams {
         setAllowFillInForms(source.isAllowFillInForms());
         setAllowPrintHq(source.isAllowPrintHq());
         setEncryptionLengthInBits(source.getEncryptionLengthInBits());
+        encryptMetadata = source.encryptMetadata();
     }
 
     /**
@@ -150,6 +154,14 @@ public class PDFEncryptionParams {
         return allowPrintHq;
     }
 
+    /**
+     * Indicates whether Metadata should be encrypted.
+     * @return true or false
+     */
+    public boolean encryptMetadata() {
+        return encryptMetadata;
+    }
+
     /**
      * Returns the owner password.
      * @return the owner password, an empty string if no password applies
@@ -230,6 +242,14 @@ public class PDFEncryptionParams {
         this.allowPrintHq = allowPrintHq;
     }
 
+    /**
+     * Whether the Metadata should be encrypted or not; default is true;
+     * @param encryptMetadata true or false
+     */
+    public void setEncryptMetadata(boolean encryptMetadata) {
+        this.encryptMetadata = encryptMetadata;
+    }
+
     /**
      * Sets the owner password.
      * @param ownerPassword The owner password to set, null or an empty String
@@ -273,4 +293,18 @@ public class PDFEncryptionParams {
         this.encryptionLengthInBits = encryptionLength;
     }
 
+    public String toString() {
+        return "userPassword = " + userPassword + "\n"
+                + "ownerPassword = " + ownerPassword + "\n"
+                + "allowPrint = " + allowPrint + "\n"
+                + "allowCopyContent = " + allowCopyContent + "\n"
+                + "allowEditContent = " + allowEditContent + "\n"
+                + "allowEditAnnotations = " + allowEditAnnotations + "\n"
+                + "allowFillInForms  = " + allowFillInForms + "\n"
+                + "allowAccessContent = " + allowAccessContent + "\n"
+                + "allowAssembleDocument = " + allowAssembleDocument + "\n"
+                + "allowPrintHq = " + allowPrintHq + "\n"
+                + "encryptMetadata = " + encryptMetadata;
+    }
+
 }
