@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFFilterList.java 1305467 2012-03-26 17:39:20Z vhennebert $ */
+/* $Id: PDFFilterList.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.pdf;
 
@@ -50,9 +50,9 @@ public class PDFFilterList {
 
     private List<PDFFilter> filters = new java.util.ArrayList<PDFFilter>();
 
-    private boolean ignoreASCIIFilters = false;
+    private boolean ignoreASCIIFilters;
 
-    private boolean disableAllFilters = false;
+    private boolean disableAllFilters;
 
     /**
      * Default constructor.
@@ -250,7 +250,7 @@ public class PDFFilterList {
         // run the filters
         int nonNullParams = 0;
         for (int count = 0; count < filters.size(); count++) {
-            PDFFilter filter = (PDFFilter)filters.get(count);
+            PDFFilter filter = filters.get(count);
             // place the names in our local vector in reverse order
             if (filter.getName().length() > 0) {
                 names.add(0, filter.getName());
@@ -365,7 +365,7 @@ public class PDFFilterList {
         OutputStream out = stream;
         if (!isDisableAllFilters()) {
             for (int count = filters.size() - 1; count >= 0; count--) {
-                PDFFilter filter = (PDFFilter)filters.get(count);
+                PDFFilter filter = filters.get(count);
                 out = filter.applyFilter(out);
             }
         }
