@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: PDFFontNonBase14.java 627679 2008-02-14 08:12:34Z jeremias $ */
- 
+/* $Id: PDFFontNonBase14.java 828632 2009-10-22 10:15:11Z jeremias $ */
+
 package org.apache.fop.pdf;
 
 import org.apache.fop.fonts.FontType;
@@ -70,23 +70,15 @@ public abstract class PDFFontNonBase14 extends PDFFont {
     public PDFFontDescriptor getDescriptor() {
         return (PDFFontDescriptor)get("FontDescriptor");
     }
-    
-    /**
-     * Sets a ToUnicode CMap.
-     * @param cmap the ToUnicode character map
-     */
-    public void setToUnicode(PDFCMap cmap) {
-        put("ToUnicode", cmap);
-    }
-    
+
     /** {@inheritDoc} */
     protected void validate() {
         if (getDocumentSafely().getProfile().isFontEmbeddingRequired()) {
             if (this.getDescriptor().getFontFile() == null) {
-                throw new PDFConformanceException("For " + getDocumentSafely().getProfile() 
-                    + ", all fonts have to be embedded!");
+                throw new PDFConformanceException("For " + getDocumentSafely().getProfile()
+                    + ", all fonts have to be embedded! Offending font: " + getBaseFont());
             }
         }
     }
-    
+
 }
