@@ -15,12 +15,12 @@
  * limitations under the License.
  */
 
-/* $Id: PDFGState.java 815358 2009-09-15 15:07:51Z maxberger $ */
+/* $Id: PDFGState.java 1661887 2015-02-24 11:23:44Z ssteiner $ */
 
 package org.apache.fop.pdf;
 
 import java.util.Map;
-import java.util.Iterator;
+import java.util.Set;
 
 /**
  * Class representing a /ExtGState object.
@@ -105,13 +105,17 @@ public class PDFGState extends PDFObject {
     }
 
     private Map values = new java.util.HashMap();
+    private int objNum;
 
     /**
      * Returns the name of this object
      * @return the name
      */
     public String getName() {
-        return "GS" + getObjectNumber();
+        if (objNum == 0) {
+            objNum = ++getDocument().gStateObjectCount;
+        }
+        return "GS" + objNum;
     }
 
     /**
@@ -149,12 +153,10 @@ public class PDFGState extends PDFObject {
      */
     public String toPDFString() {
         StringBuffer sb = new StringBuffer(64);
-        sb.append(getObjectID());
         sb.append("<<\n/Type /ExtGState\n");
         appendVal(sb, GSTATE_ALPHA_NONSTROKE);
         appendVal(sb, GSTATE_ALPHA_STROKE);
-
-        sb.append(">>\nendobj\n");
+        sb.append(">>");
         return sb.toString();
     }
 
@@ -188,10 +190,10 @@ public class PDFGState extends PDFObject {
         if (vals1.size() != vals2.size()) {
             return false;
         }
-        for (Iterator iter = vals1.keySet().iterator(); iter.hasNext();) {
-            Object str = iter.next();
+        for (Map.Entry<Object, Object> e : (Set<Map.Entry<Object, Object>>) vals2.entrySet()) {
+            Object str = e.getKey();
             Object obj1 = vals1.get(str);
-            if (!obj1.equals(vals2.get(str))) {
+            if (!obj1.equals(e.getValue())) {
                 return false;
             }
         }
