@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFGoTo.java 1305467 2012-03-26 17:39:20Z vhennebert $ */
+/* $Id: PDFGoTo.java 1661887 2015-02-24 11:23:44Z ssteiner $ */
 
 package org.apache.fop.pdf;
 
@@ -31,10 +31,10 @@ public class PDFGoTo extends PDFAction {
     /**
      * the pageReference
      */
-    private String pageReference;
-    private String destination = null;
-    private float xPosition = 0;
-    private float yPosition = 0;
+    private PDFReference pageReference;
+    private String destination;
+    private float xPosition;
+    private float yPosition;
 
     /**
      * create a /GoTo object.
@@ -43,7 +43,9 @@ public class PDFGoTo extends PDFAction {
      */
     public PDFGoTo(String pageReference) {
         super();
-        setPageReference(pageReference);
+        if (pageReference != null) {
+            setPageReference(new PDFReference(pageReference));
+        }
     }
 
     /**
@@ -63,7 +65,7 @@ public class PDFGoTo extends PDFAction {
      *
      * @param pageReference the new page reference to use
      */
-    public void setPageReference(String pageReference) {
+    public void setPageReference(PDFReference pageReference) {
         this.pageReference = pageReference;
     }
 
