@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,15 +15,18 @@
  * limitations under the License.
  */
 
-/* $Id: PDFNames.java 610704 2008-01-10 07:38:47Z jeremias $ */
- 
+/* $Id: PDFNames.java 986203 2010-08-17 06:56:46Z jeremias $ */
+
 package org.apache.fop.pdf;
 
 /**
  * Class representing a PDF Names object
  */
 public class PDFNames extends PDFDictionary {
-    
+
+    private static final String DESTS = "Dests";
+    private static final String EMBEDDED_FILES = "EmbeddedFiles";
+
     /**
      * Create the Names object
      */
@@ -37,15 +40,31 @@ public class PDFNames extends PDFDictionary {
      * @return the Dests object, or null if it's not used
      */
     public PDFDests getDests() {
-        return (PDFDests)get("Dests");
+        return (PDFDests)get(DESTS);
     }
-    
+
     /**
      * Set the Dests object
      * @param dests the Dests object
      */
     public void setDests(PDFDests dests) {
-        put("Dests", dests);
+        put(DESTS, dests);
     }
-    
+
+    /**
+     * Returns the EmbeddedFiles object
+     * @return the EmbeddedFiles object, or null if it's not used
+     */
+    public PDFEmbeddedFiles getEmbeddedFiles() {
+        return (PDFEmbeddedFiles)get(EMBEDDED_FILES);
+    }
+
+    /**
+     * Set the EmbeddedFiles object
+     * @param embeddedFiles the EmbeddedFiles object
+     */
+    public void setEmbeddedFiles(PDFEmbeddedFiles embeddedFiles) {
+        put(EMBEDDED_FILES, embeddedFiles);
+    }
+
 }
