@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFProfile.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: PDFProfile.java 1685770 2015-06-16 11:46:50Z ssteiner $ */
 
 package org.apache.fop.pdf;
 
@@ -26,25 +26,29 @@ import java.text.MessageFormat;
  * the libarary and its users to enable the generation of PDFs conforming to the enabled PDF
  * profiles.
  * <p>
- * Some profile from PDF/X and PDF/A can be active simultaneously (example: PDF/A-1 and 
+ * Some profile from PDF/X and PDF/A can be active simultaneously (example: PDF/A-1 and
  * PDF/X-3:2003).
  */
 public class PDFProfile {
 
     /**
-     * Indicates the PDF/A mode currently active. Defaults to "no restrictions", i.e. 
+     * Indicates the PDF/A mode currently active. Defaults to "no restrictions", i.e.
      * PDF/A not active.
      */
     protected PDFAMode pdfAMode = PDFAMode.DISABLED;
-    
+
+    protected PDFUAMode pdfUAMode = PDFUAMode.DISABLED;
+
     /**
-     * Indicates the PDF/X mode currently active. Defaults to "no restrictions", i.e. 
+     * Indicates the PDF/X mode currently active. Defaults to "no restrictions", i.e.
      * PDF/X not active.
      */
     protected PDFXMode pdfXMode = PDFXMode.DISABLED;
-    
+
+    protected PDFVTMode pdfVTMode = PDFVTMode.DISABLED;
+
     private PDFDocument doc;
-    
+
     /**
      * Main constructor
      * @param doc the PDF document
@@ -52,39 +56,43 @@ public class PDFProfile {
     public PDFProfile(PDFDocument doc) {
         this.doc = doc;
     }
-    
+
     /**
      * Validates if the requested profile combination is compatible.
      */
     protected void validateProfileCombination() {
         if (pdfAMode != PDFAMode.DISABLED) {
-            if (pdfAMode == PDFAMode.PDFA_1A) {
-                throw new UnsupportedOperationException("PDF/A-1a is not implemented, yet");
-            }
             if (pdfAMode == PDFAMode.PDFA_1B) {
-                if (pdfXMode != PDFXMode.DISABLED && pdfXMode != PDFXMode.PDFX_3_2003) {
+                if (pdfXMode != PDFXMode.DISABLED && pdfXMode != PDFXMode.PDFX_3_2003 && pdfXMode != PDFXMode.PDFX_4) {
                     throw new PDFConformanceException(
                             pdfAMode + " and " + pdfXMode + " are not compatible!");
                 }
             }
         }
+        if (pdfVTMode != PDFVTMode.DISABLED && pdfXMode != PDFXMode.PDFX_4) {
+            throw new PDFConformanceException(pdfVTMode.name() + " requires " + PDFXMode.PDFX_4.getName() + " enabled");
+        }
     }
-    
+
     /** @return the PDFDocument this profile is attached to */
     public PDFDocument getDocument() {
         return this.doc;
     }
-    
+
     /** @return the PDF/A mode */
     public PDFAMode getPDFAMode() {
         return this.pdfAMode;
     }
-    
+
+    public PDFUAMode getPDFUAMode() {
+        return this.pdfUAMode;
+    }
+
     /** @return true if any PDF/A mode is active */
     public boolean isPDFAActive() {
         return getPDFAMode() != PDFAMode.DISABLED;
     }
-    
+
     /**
      * Sets the PDF/A mode
      * @param mode the PDF/A mode
@@ -96,17 +104,33 @@ public class PDFProfile {
         this.pdfAMode = mode;
         validateProfileCombination();
     }
-    
+
+    public void setPDFUAMode(PDFUAMode mode) {
+        if (mode == null) {
+            mode = PDFUAMode.DISABLED;
+        }
+        this.pdfUAMode = mode;
+        validateProfileCombination();
+    }
+
     /** @return the PDF/X mode */
     public PDFXMode getPDFXMode() {
         return this.pdfXMode;
     }
-    
+
+    public PDFVTMode getPDFVTMode() {
+        return this.pdfVTMode;
+    }
+
     /** @return true if any PDF/X mode is active */
     public boolean isPDFXActive() {
         return getPDFXMode() != PDFXMode.DISABLED;
     }
-    
+
+    public boolean isPDFVTActive() {
+        return getPDFVTMode() != PDFVTMode.DISABLED;
+    }
+
     /**
      * Sets the PDF/X mode
      * @param mode the PDF/X mode
@@ -119,6 +143,18 @@ public class PDFProfile {
         validateProfileCombination();
     }
 
+    /**
+     * Sets the PDF/X mode
+     * @param mode the PDF/X mode
+     */
+    public void setPDFVTMode(PDFVTMode mode) {
+        if (mode == null) {
+            mode = PDFVTMode.DISABLED;
+        }
+        this.pdfVTMode = mode;
+        validateProfileCombination();
+    }
+
     /** {@inheritDoc} */
     public String toString() {
         StringBuffer sb = new StringBuffer();
@@ -128,18 +164,24 @@ public class PDFProfile {
             sb.append(getPDFAMode());
         } else if (isPDFXActive()) {
             sb.append(getPDFXMode());
+        } else if (getPDFUAMode().isEnabled()) {
+            sb.append(getPDFUAMode());
         } else {
             sb.append(super.toString());
         }
         return sb.toString();
     }
-    
+
     //---------=== Info and validation methods ===---------
-    
-    private String format(String pattern, Object arg) {
-        return MessageFormat.format(pattern, new Object[] {arg});
+
+    private String format(String pattern, Object[] args) {
+        return MessageFormat.format(pattern, args);
     }
-    
+
+    private String format(String pattern, Object arg) {
+        return format(pattern, new Object[] {arg});
+    }
+
     /** Checks if encryption is allowed. */
     public void verifyEncryptionAllowed() {
         final String err = "{0} doesn't allow encrypted PDFs";
@@ -168,30 +210,72 @@ public class PDFProfile {
      * @param context Context information for the user to identify the problem spot
      */
     public void verifyTransparencyAllowed(String context) {
-        final String err = "{0} does not allow the use of transparency. ({1})";
-        if (isPDFAActive()) {
-            throw new PDFConformanceException(MessageFormat.format(err, 
-                    new Object[] {getPDFAMode(), context}));
+        Object profile = isTransparencyAllowed();
+        if (profile != null) {
+            throw new TransparencyDisallowedException(profile, context);
         }
-        if (isPDFXActive()) {
-            throw new PDFConformanceException(MessageFormat.format(err, 
-                    new Object[] {getPDFXMode(), context}));
+    }
+
+    /**
+     * Returns {@code null} if transparency is allowed, otherwise returns the profile that
+     * prevents it.
+     *
+     * @return {@code null}, or an object whose {@code toString} method returns the name
+     * of the profile that disallows transparency
+     */
+    public Object isTransparencyAllowed() {
+        if (pdfAMode.isPart1()) {
+            return getPDFAMode();
         }
+        if (getPDFXMode() == PDFXMode.PDFX_3_2003) {
+            return getPDFXMode();
+        }
+        return null;
     }
 
     /** Checks if the right PDF version is set. */
     public void verifyPDFVersion() {
-        final String err = "PDF version must be 1.4 for {0}";
-        if (getPDFAMode().isPDFA1LevelB() 
-                && getDocument().getPDFVersion() != PDFDocument.PDF_VERSION_1_4) {
+        String err = "PDF version must be 1.4 for {0}";
+        if (getPDFAMode().isPart1()
+                && !Version.V1_4.equals(getDocument().getPDFVersion())) {
             throw new PDFConformanceException(format(err, getPDFAMode()));
         }
-        if (getPDFXMode() == PDFXMode.PDFX_3_2003 
-                && getDocument().getPDFVersion() != PDFDocument.PDF_VERSION_1_4) {
+        if (getPDFXMode() == PDFXMode.PDFX_3_2003
+                && !Version.V1_4.equals(getDocument().getPDFVersion())) {
             throw new PDFConformanceException(format(err, getPDFXMode()));
         }
     }
-    
+
+    /**
+     * Checks a few things required for tagged PDF.
+     */
+    public void verifyTaggedPDF() {
+        if (getPDFAMode().isLevelA() || getPDFUAMode().isEnabled()) {
+            final String err = "{0} requires the {1} dictionary entry to be set";
+            String mode = getPDFAMode().toString();
+            if (getPDFUAMode().isEnabled()) {
+                mode = getPDFUAMode().toString();
+            }
+            PDFDictionary markInfo = getDocument().getRoot().getMarkInfo();
+            if (markInfo == null) {
+                throw new PDFConformanceException(format(
+                        "{0} requires that the accessibility option in the configuration file be enabled", mode));
+            }
+            if (!Boolean.TRUE.equals(markInfo.get("Marked"))) {
+                throw new PDFConformanceException(format(err,
+                        new Object[] {mode, "Marked"}));
+            }
+            if (getDocument().getRoot().getStructTreeRoot() == null) {
+                throw new PDFConformanceException(format(err,
+                        new Object[] {mode, "StructTreeRoot"}));
+            }
+            if (getDocument().getRoot().getLanguage() == null) {
+                throw new PDFConformanceException(format(err,
+                        new Object[] {mode, "Lang"}));
+            }
+        }
+    }
+
     /** @return true if the ID entry must be present in the trailer. */
     public boolean isIDEntryRequired() {
         return isPDFAActive() || isPDFXActive();
@@ -199,32 +283,35 @@ public class PDFProfile {
 
     /** @return true if all fonts need to be embedded. */
     public boolean isFontEmbeddingRequired() {
-        return isPDFAActive() || isPDFXActive();
+        return isPDFAActive() || isPDFXActive() || getPDFUAMode().isEnabled();
     }
 
     /** Checks if a title may be absent. */
     public void verifyTitleAbsent() {
+        final String err = "{0} requires the title to be set.";
+        if (getPDFUAMode().isEnabled()) {
+            throw new PDFConformanceException(format(err, getPDFUAMode()));
+        }
         if (isPDFXActive()) {
-            final String err = "{0} requires the title to be set.";
             throw new PDFConformanceException(format(err, getPDFXMode()));
         }
     }
 
     /** @return true if the ModDate Info entry must be present. */
     public boolean isModDateRequired() {
-        return getPDFXMode() == PDFXMode.PDFX_3_2003;
+        return getPDFXMode() != PDFXMode.DISABLED;
     }
 
     /** @return true if the Trapped Info entry must be present. */
     public boolean isTrappedEntryRequired() {
-        return getPDFXMode() == PDFXMode.PDFX_3_2003;
+        return getPDFXMode() != PDFXMode.DISABLED;
     }
 
     /** @return true if annotations are allowed */
     public boolean isAnnotationAllowed() {
         return !isPDFXActive();
     }
-    
+
     /** Checks if annotations are allowed. */
     public void verifyAnnotAllowed() {
         if (!isAnnotationAllowed()) {
@@ -242,4 +329,16 @@ public class PDFProfile {
         }
     }
 
+    /** Checks if embedded files are allowed. */
+    public void verifyEmbeddedFilesAllowed() {
+        final String err = "{0} does not allow embedded files.";
+        if (isPDFAActive()) {
+            throw new PDFConformanceException(format(err, getPDFAMode()));
+        }
+        if (isPDFXActive()) {
+            //Implicit since file specs are forbidden
+            throw new PDFConformanceException(format(err, getPDFXMode()));
+        }
+    }
+
 }
