@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFResources.java 1357883 2012-07-05 20:29:53Z gadams $ */
+/* $Id: PDFResources.java 1661887 2015-02-24 11:23:44Z ssteiner $ */
 
 package org.apache.fop.pdf;
 
@@ -37,51 +37,54 @@ import org.apache.fop.fonts.base14.ZapfDingbats;
 /**
  * Class representing a /Resources object.
  *
- * /Resources object contain a list of references to the fonts for the
- * document
+ * /Resources object contain a list of references to the fonts, patterns,
+ * shadings, etc.,  for the document.
  */
 public class PDFResources extends PDFDictionary {
 
     /**
      * /Font objects keyed by their internal name
      */
-    protected Map<String, PDFFont> fonts = new LinkedHashMap<String, PDFFont>();
+    protected Map<String, PDFDictionary> fonts = new LinkedHashMap<String, PDFDictionary>();
 
     /**
      * Set of XObjects
      */
     protected Set<PDFXObject> xObjects = new LinkedHashSet<PDFXObject>();
-
-    /**
-     * Set of patterns
-     */
-    protected Set<PDFPattern> patterns = new LinkedHashSet<PDFPattern>();
-
-    /**
-     * Set of shadings
-     */
-    protected Set<PDFShading> shadings = new LinkedHashSet<PDFShading>();
-
-    /**
-     * Set of ExtGStates
-     */
-    protected Set<PDFGState> gstates = new LinkedHashSet<PDFGState>();
-
     /** Map of color spaces (key: color space name) */
-    protected Map<PDFName, PDFColorSpace> colorSpaces = new LinkedHashMap<PDFName, PDFColorSpace>();
+    protected Map<LazyName, PDFColorSpace> colorSpaces = new LinkedHashMap<LazyName, PDFColorSpace>();
 
     /** Map of ICC color spaces (key: ICC profile description) */
     protected Map<String, PDFICCBasedColorSpace> iccColorSpaces = new LinkedHashMap<String, PDFICCBasedColorSpace>();
 
+    private PDFResources parent;
+    private PDFDictionary fontsObj;
+    private Map<String, PDFDictionary> fontsObjDict = new LinkedHashMap<String, PDFDictionary>();
+
+    /** Named properties */
+    protected Map<String, PDFReference> properties = new LinkedHashMap<String, PDFReference>();
+
+    protected Set<PDFResourceContext> contexts = new LinkedHashSet<PDFResourceContext>();
+
     /**
      * create a /Resources object.
-     *
-     * @param objnum the object's number
      */
-    public PDFResources(int objnum) {
+    public PDFResources(PDFDocument doc) {
         /* generic creation of object */
         super();
-        setObjectNumber(objnum);
+        setObjectNumber(doc);
+    }
+
+    public void addContext(PDFResourceContext c) {
+        contexts.add(c);
+    }
+
+    public void setParentResources(PDFResources p) {
+        parent = p;
+    }
+
+    public PDFResources getParentResources() {
+        return parent;
     }
 
     /**
@@ -90,7 +93,22 @@ public class PDFResources extends PDFDictionary {
      * @param font the PDFFont to add
      */
     public void addFont(PDFFont font) {
-        this.fonts.put(font.getName(), font);
+        addFont(font.getName(), font);
+    }
+
+    public void addFont(String name, PDFDictionary font) {
+        if (fontsObj != null) {
+            fontsObj.put(name, font);
+            fontsObjDict.put(name, font);
+        } else {
+            fonts.put(name, font);
+        }
+    }
+
+    public void createFontsAsObj() {
+        fontsObj = new PDFDictionary();
+        getDocument().registerTrailerObject(fontsObj);
+        put("Font", fontsObj);
     }
 
     /**
@@ -101,8 +119,9 @@ public class PDFResources extends PDFDictionary {
      */
    public void addFonts(PDFDocument doc, FontInfo fontInfo) {
         Map<String, Typeface> usedFonts = fontInfo.getUsedFonts();
-        for (String f : usedFonts.keySet()) {
-            Typeface font = usedFonts.get(f);
+        for (Map.Entry<String, Typeface> e : usedFonts.entrySet()) {
+            String f = e.getKey();
+            Typeface font = e.getValue();
 
             //Check if the font actually had any mapping operations. If not, it is an indication
             //that it has never actually been used and therefore doesn't have to be embedded.
@@ -119,34 +138,7 @@ public class PDFResources extends PDFDictionary {
                     f, font.getEmbedFontName(), encoding, font, desc));
             }
         }
-    }
-
-    /**
-     * Add a PDFGState to the resources.
-     *
-     * @param gs the PDFGState to add
-     */
-    public void addGState(PDFGState gs) {
-        this.gstates.add(gs);
-    }
-
-    /**
-     * Add a Shading to the resources.
-     *
-     * @param theShading the shading to add
-     */
-    public void addShading(PDFShading theShading) {
-        this.shadings.add(theShading);
-    }
-
-    /**
-     * Add the pattern to the resources.
-     *
-     * @param thePattern the pattern to add
-     */
-    public void addPattern(PDFPattern thePattern) {
-        this.patterns.add(thePattern);
-    }
+   }
 
     /**
      * Add an XObject to the resources.
@@ -162,7 +154,7 @@ public class PDFResources extends PDFDictionary {
      * @param colorSpace the color space
      */
     public void addColorSpace(PDFColorSpace colorSpace) {
-        this.colorSpaces.put(new PDFName(colorSpace.getName()), colorSpace);
+        this.colorSpaces.put(new LazyName(colorSpace), colorSpace);
         if (colorSpace instanceof PDFICCBasedColorSpace) {
             PDFICCBasedColorSpace icc = (PDFICCBasedColorSpace)colorSpace;
             String desc = ColorProfileUtil.getICCProfileDescription(
@@ -171,6 +163,16 @@ public class PDFResources extends PDFDictionary {
         }
     }
 
+    static class LazyName {
+        private PDFColorSpace colorSpace;
+        public LazyName(PDFColorSpace colorSpace) {
+            this.colorSpace = colorSpace;
+        }
+        public PDFName getName() {
+            return new PDFName(colorSpace.getName());
+        }
+    }
+
     /**
      * Returns a ICCBased color space by profile name.
      * @param desc the name of the color space
@@ -187,8 +189,31 @@ public class PDFResources extends PDFDictionary {
      * @return the requested color space or null if it wasn't found
      */
     public PDFColorSpace getColorSpace(PDFName name) {
-        PDFColorSpace cs = this.colorSpaces.get(name);
-        return cs;
+        for (Map.Entry<LazyName, PDFColorSpace> x : colorSpaces.entrySet()) {
+            if (x.getKey().getName().equals(name)) {
+                return x.getValue();
+            }
+        }
+        return null;
+    }
+
+    /**
+     * Add a named property.
+     *
+     * @param name name of property
+     * @param property reference to property value
+     */
+    public void addProperty(String name, PDFReference property) {
+        this.properties.put(name, property);
+    }
+
+    /**
+     * Get a named property.
+     *
+     * @param name name of property
+     */
+    public PDFReference getProperty(String name) {
+        return this.properties.get(name);
     }
 
     @Override
@@ -198,16 +223,40 @@ public class PDFResources extends PDFDictionary {
     }
 
     private void populateDictionary() {
-        if (!this.fonts.isEmpty()) {
+        if (parent != null && parent.fontsObj != null) {
+            put("Font", parent.fontsObj);
+        }
+        if (!this.fonts.isEmpty() || (parent != null && !parent.fonts.isEmpty())) {
             PDFDictionary dict = new PDFDictionary(this);
             /* construct PDF dictionary of font object references */
-            for (Map.Entry<String, PDFFont> entry : fonts.entrySet()) {
+            for (Map.Entry<String, PDFDictionary> entry : fonts.entrySet()) {
                 dict.put(entry.getKey(), entry.getValue());
             }
+            if (parent != null) {
+                for (Map.Entry<String, PDFDictionary> entry : parent.fonts.entrySet()) {
+                    dict.put(entry.getKey(), entry.getValue());
+                }
+                for (Map.Entry<String, PDFDictionary> entry : parent.fontsObjDict.entrySet()) {
+                    dict.put(entry.getKey(), entry.getValue());
+                }
+            }
             put("Font", dict);
         }
 
-        if (!this.shadings.isEmpty()) {
+        Set<PDFPattern> patterns = new LinkedHashSet<PDFPattern>();
+        Set<PDFShading> shadings = new LinkedHashSet<PDFShading>();
+        Set<PDFGState> gstates = new LinkedHashSet<PDFGState>();
+        for (PDFResourceContext c : contexts) {
+            xObjects.addAll(c.getXObjects());
+            patterns.addAll(c.getPatterns());
+            shadings.addAll(c.getShadings());
+            gstates.addAll(c.getGStates());
+        }
+        if (parent != null) {
+            xObjects.addAll(parent.xObjects);
+        }
+
+        if (!shadings.isEmpty()) {
             PDFDictionary dict = new PDFDictionary(this);
             for (PDFShading shading : shadings) {
                 dict.put(shading.getName(), shading);
@@ -215,7 +264,7 @@ public class PDFResources extends PDFDictionary {
             put("Shading", dict);
         }
 
-        if (!this.patterns.isEmpty()) {
+        if (!patterns.isEmpty()) {
             PDFDictionary dict = new PDFDictionary(this);
             for (PDFPattern pattern : patterns) {
                 dict.put(pattern.getName(), pattern);
@@ -230,15 +279,18 @@ public class PDFResources extends PDFDictionary {
         procset.add(new PDFName("Text"));
         put("ProcSet", procset);
 
-        if (this.xObjects != null && !this.xObjects.isEmpty()) {
-            PDFDictionary dict = new PDFDictionary(this);
+        if (!xObjects.isEmpty()) {
+            PDFDictionary dict = (PDFDictionary) get("XObject");
+            if (dict == null) {
+                dict = new PDFDictionary(this);
+            }
             for (PDFXObject xObject : xObjects) {
                 dict.put(xObject.getName().toString(), xObject);
             }
             put("XObject", dict);
         }
 
-        if (!this.gstates.isEmpty()) {
+        if (!gstates.isEmpty()) {
             PDFDictionary dict = new PDFDictionary(this);
             for (PDFGState gstate : gstates) {
                 dict.put(gstate.getName(), gstate);
@@ -253,6 +305,53 @@ public class PDFResources extends PDFDictionary {
             }
             put("ColorSpace", dict);
         }
+
+        if (!properties.isEmpty()) {
+            PDFDictionary dict = new PDFDictionary(this);
+            for (String name : properties.keySet()) {
+                dict.put(name, properties.get(name));
+            }
+            put("Properties", dict);
+        }
     }
 
+    @Override
+    public void getChildren(Set<PDFObject> children) {
+        getChildren(children, false);
+    }
+
+    private void getChildren(Set<PDFObject> children, boolean isParent) {
+        super.getChildren(children);
+        for (PDFDictionary f : fonts.values()) {
+            children.add(f);
+            f.getChildren(children);
+        }
+        for (PDFResourceContext c : contexts) {
+            for (PDFXObject x : c.getXObjects()) {
+                children.add(x);
+                x.getChildren(children);
+            }
+            for (PDFPattern x : c.getPatterns()) {
+                children.add(x);
+                x.getChildren(children);
+            }
+            for (PDFShading x : c.getShadings()) {
+                children.add(x);
+                x.getChildren(children);
+            }
+            for (PDFGState x : c.getGStates()) {
+                children.add(x);
+                x.getChildren(children);
+            }
+        }
+        if (!isParent) {
+            for (PDFColorSpace x : colorSpaces.values()) {
+                children.add((PDFObject)x);
+                ((PDFObject)x).getChildren(children);
+            }
+        }
+        if (parent != null) {
+            parent.getChildren(children, true);
+        }
+    }
 }
