@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,10 +15,16 @@
  * limitations under the License.
  */
 
-/* $Id: PDFRoot.java 615906 2008-01-28 15:04:39Z jeremias $ */
- 
+/* $Id: PDFRoot.java 1685770 2015-06-16 11:46:50Z ssteiner $ */
+
 package org.apache.fop.pdf;
 
+import java.io.IOException;
+import java.io.OutputStream;
+import java.util.Locale;
+
+import org.apache.fop.util.LanguageTags;
+
 /**
  * Class representing a Root (/Catalog) object.
  */
@@ -44,28 +50,45 @@ public class PDFRoot extends PDFDictionary {
      */
     public static final int PAGEMODE_FULLSCREEN = 3;
 
+    private final PDFDocument document;
+
+    private PDFDPartRoot dPartRoot;
+
+
     private static final PDFName[] PAGEMODE_NAMES = new PDFName[] {
         new PDFName("UseNone"),
         new PDFName("UseOutlines"),
         new PDFName("UseThumbs"),
         new PDFName("FullScreen"),
     };
-    
+
     /**
      * create a Root (/Catalog) object. NOTE: The PDFRoot
      * object must be created before the PDF document is
      * generated, but it is not assigned an object ID until
      * it is about to be written (immediately before the xref
-     * table as part of the trsailer). (mark-fop@inomial.com)
+     * table as part of the trailer). (mark-fop@inomial.com)
      *
      * @param objnum the object's number
      * @param pages the PDFPages object
      */
-    public PDFRoot(int objnum, PDFPages pages) {
-        super();
-        setObjectNumber(objnum);
+    public PDFRoot(PDFDocument document, PDFPages pages) {
+        this.document = document;
+        setObjectNumber(document);
         put("Type", new PDFName("Catalog"));
         setRootPages(pages);
+        setLanguage("x-unknown");
+    }
+
+    /** {@inheritDoc} */
+    public int output(OutputStream stream) throws IOException {
+        if (document.getProfile().getPDFUAMode().isEnabled()) {
+            PDFDictionary d = new PDFDictionary();
+            d.put("DisplayDocTitle", true);
+            put("ViewerPreferences", d);
+        }
+        getDocument().getProfile().verifyTaggedPDF();
+        return super.output(stream);
     }
 
     /**
@@ -94,7 +117,7 @@ public class PDFRoot extends PDFDictionary {
             return PAGEMODE_USENONE;
         }
     }
-    
+
     /**
      * add a /Page object to the root /Pages object
      *
@@ -123,7 +146,7 @@ public class PDFRoot extends PDFDictionary {
         PDFReference ref = (PDFReference)get("Pages");
         return (ref != null ? (PDFPages)ref.getObject() : null);
     }
-    
+
     /**
      * Sets the /PageLabels object.
      * @param pageLabels the /PageLabels object
@@ -131,7 +154,7 @@ public class PDFRoot extends PDFDictionary {
     public void setPageLabels(PDFPageLabels pageLabels) {
         put("PageLabels", pageLabels.makeReference());
     }
-    
+
     /**
      * Returns the /PageLabels object.
      * @return the /PageLabels object if set, null otherwise.
@@ -141,7 +164,7 @@ public class PDFRoot extends PDFDictionary {
         PDFReference ref = (PDFReference)get("PageLabels");
         return (ref != null ? (PDFPageLabels)ref.getObject() : null);
     }
-    
+
     /**
      * Set the root outline for the PDF document.
      *
@@ -149,7 +172,7 @@ public class PDFRoot extends PDFDictionary {
      */
     public void setRootOutline(PDFOutline out) {
         put("Outlines", out.makeReference());
-        
+
         //Set /PageMode to /UseOutlines by default if no other mode has been set
         PDFName mode = (PDFName)get("PageMode");
         if (mode == null) {
@@ -166,7 +189,7 @@ public class PDFRoot extends PDFDictionary {
         PDFReference ref = (PDFReference)get("Outlines");
         return (ref != null ? (PDFOutline)ref.getObject() : null);
     }
-    
+
     /**
      * Set the /Names object.
      * @param names the Names object
@@ -175,7 +198,7 @@ public class PDFRoot extends PDFDictionary {
     public void setNames(PDFNames names) {
         put("Names", names.makeReference());
     }
-    
+
     /**
      * Returns the /Names object.
      * @return the Names object if set, null otherwise.
@@ -185,18 +208,18 @@ public class PDFRoot extends PDFDictionary {
         PDFReference ref = (PDFReference)get("Names");
         return (ref != null ? (PDFNames)ref.getObject() : null);
     }
-    
+
     /**
      * Set the optional Metadata object.
      * @param meta the Metadata object
      * @since PDF 1.4
      */
     public void setMetadata(PDFMetadata meta) {
-        if (getDocumentSafely().getPDFVersion() >= PDFDocument.PDF_VERSION_1_4) {
+        if (getDocumentSafely().getPDFVersion().compareTo(Version.V1_4) >= 0) {
             put("Metadata", meta.makeReference());
         }
     }
-    
+
     /**
      * Returns the /Metadata object
      * @return the /Metadata object if set, null otherwise.
@@ -215,15 +238,15 @@ public class PDFRoot extends PDFDictionary {
     public PDFArray getOutputIntents() {
         return (PDFArray)get("OutputIntents");
     }
-    
+
     /**
      * Adds an OutputIntent to the PDF
      * @param outputIntent the OutputIntent dictionary
      * @since PDF 1.4
      */
     public void addOutputIntent(PDFOutputIntent outputIntent) {
-        if (getDocumentSafely().getPDFVersion() >= PDFDocument.PDF_VERSION_1_4) {
-            PDFArray outputIntents = getOutputIntents(); 
+        if (getDocumentSafely().getPDFVersion().compareTo(Version.V1_4) >= 0) {
+            PDFArray outputIntents = getOutputIntents();
             if (outputIntents == null) {
                 outputIntents = new PDFArray(this);
                 put("OutputIntents", outputIntents);
@@ -231,7 +254,18 @@ public class PDFRoot extends PDFDictionary {
             outputIntents.add(outputIntent);
         }
     }
-    
+
+    /**
+     * Sets the "Version" entry. If this version is greater than that specified in the header, this
+     * version takes precedence.
+     *
+     * @param version the PDF document version
+     * @since PDF 1.4
+     */
+    void setVersion(Version version) {
+        put("Version", new PDFName(version.toString()));
+    }
+
     /**
      * Returns the language identifier of the document.
      * @return the language identifier of the document (or null if not set or undefined)
@@ -240,16 +274,63 @@ public class PDFRoot extends PDFDictionary {
     public String getLanguage() {
         return (String)get("Lang");
     }
-    
+
     /**
-     * Sets the language identifier of the document.
-     * @param lang the language identifier of the document.
+     * Sets the locale of the document.
+     * @param locale the locale of the document.
      */
-    public void setLanguage(String lang) {
-        if (lang == null) {
-            throw new NullPointerException("lang must not be null");
+    public void setLanguage(Locale locale) {
+        if (locale == null) {
+            throw new NullPointerException("locale must not be null");
         }
+        setLanguage(LanguageTags.toLanguageTag(locale));
+    }
+
+    private void setLanguage(String lang) {
         put("Lang", lang);
     }
-    
+
+    /**
+     * Sets the StructTreeRoot object. Used for accessibility.
+     * @param structTreeRoot of this document
+     */
+    public void setStructTreeRoot(PDFStructTreeRoot structTreeRoot) {
+        if (structTreeRoot == null) {
+            throw new NullPointerException("structTreeRoot must not be null");
+        }
+        put("StructTreeRoot", structTreeRoot);
+    }
+
+    /**
+     * Returns the StructTreeRoot object.
+     * @return the structure tree root (or null if accessibility is not enabled)
+     */
+    public PDFStructTreeRoot getStructTreeRoot() {
+        return (PDFStructTreeRoot)get("StructTreeRoot");
+    }
+
+    /**
+     * Marks this document as conforming to the Tagged PDF conventions.
+     */
+    public void makeTagged() {
+        PDFDictionary dict = new PDFDictionary();
+        dict.put("Marked", Boolean.TRUE);
+        put("MarkInfo", dict);  //new PDFMarkInfo()
+    }
+
+    /**
+     * Returns the MarkInfo dictionary.
+     * @return the MarkInfo dictionary (or null if it's not present)
+     */
+    public PDFDictionary getMarkInfo() {
+        return (PDFDictionary)get("MarkInfo");
+    }
+
+    public PDFDPartRoot getDPartRoot() {
+        if (dPartRoot == null) {
+            dPartRoot = getDocument().getFactory().makeDPartRoot();
+            put("DPartRoot", dPartRoot.makeReference());
+        }
+        return dPartRoot;
+    }
 }
