@@ -15,14 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id: PDFStructTreeRoot.java 1297404 2012-03-06 10:17:54Z vhennebert $ */
+/* $Id: PDFStructTreeRoot.java 1376923 2012-08-24 14:10:39Z vhennebert $ */
 
 package org.apache.fop.pdf;
 
 /**
  * Class representing a PDF /StructTreeRoot dictionary.
  */
-public class PDFStructTreeRoot extends PDFDictionary {
+public class PDFStructTreeRoot extends StructureHierarchyMember {
 
     /**
      * Creates a new /StructTreeRoot dictionary.
@@ -49,6 +49,7 @@ public class PDFStructTreeRoot extends PDFDictionary {
      *
      * @param kid an object to be added to the K entry
      */
+    @Override
     public void addKid(PDFObject kid) {
         getKids().add(kid);
     }
