@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFText.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: PDFText.java 1435269 2013-01-18 17:45:40Z gadams $ */
 
 package org.apache.fop.pdf;
 
@@ -60,11 +60,9 @@ public class PDFText extends PDFObject {
                 "The text of this PDFText must not be empty");
         }
         StringBuffer sb = new StringBuffer(64);
-        sb.append(getObjectID());
         sb.append("(");
         sb.append(escapeText(getText()));
         sb.append(")");
-        sb.append("\nendobj\n");
         return sb.toString();
     }
 
@@ -207,6 +205,18 @@ public class PDFText extends PDFObject {
         return buf.toString();
     }
 
+    /**
+     * Convert a char to a multibyte hex representation appending to string buffer.
+     * Since Java always stores strings in UTF-16, we don't have to do any conversion.
+     * @param c character to encode
+     * @param sb the string buffer to append output
+     */
+    public static final void toUnicodeHex(char c, StringBuffer sb) {
+        for (int i = 0; i < 4; ++i) {
+            sb.append(DIGITS[(c >> (12 - 4 * i)) & 0x0F]);
+        }
+    }
+
     /**
      * Escaped a String as described in section 4.4 in the PDF 1.3 specs.
      * @param s String to escape
@@ -320,5 +330,41 @@ public class PDFText extends PDFObject {
         return bout.toByteArray();
     }
 
+    /**
+     * Converts a text to PDF's "string" data type. Unsupported characters get converted to '?'
+     * characters (similar to what the Java "US-ASCII" encoding does).
+     * @see #toPDFString(CharSequence, char)
+     * @param text the text to convert
+     * @return the converted string
+     */
+    public static String toPDFString(CharSequence text) {
+        return toPDFString(text, '?');
+    }
+
+    /**
+     * Converts a text to PDF's "string" data type. Unsupported characters get converted to the
+     * given replacement character.
+     * <p>
+     * The PDF library currently doesn't properly distinguish between the PDF
+     * data types "string" and "text string", so we currently restrict "string" to US-ASCII, also
+     * because "string" seems somewhat under-specified concerning the upper 128 bytes.
+     * @param text the text to convert
+     * @param replacement the replacement character used when substituting a character
+     * @return the converted string
+     */
+    public static String toPDFString(CharSequence text, char replacement) {
+        StringBuffer sb = new StringBuffer();
+        for (int i = 0, c = text.length(); i < c; i++) {
+            char ch = text.charAt(i);
+            if (ch > 127) {
+                //TODO Revisit the restriction to US-ASCII once "string" and "text string" are
+                //"disentangled".
+                sb.append(replacement);
+            } else {
+                sb.append(ch);
+            }
+        }
+        return sb.toString();
+    }
 }
 
