@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractRenderer.java 1297008 2012-03-05 11:19:47Z vhennebert $ */
+/* $Id: AbstractRenderer.java 1683758 2015-06-05 13:57:57Z ssteiner $ */
 
 package org.apache.fop.render;
 
@@ -28,6 +28,7 @@ import java.io.OutputStream;
 import java.util.List;
 import java.util.Locale;
 import java.util.Set;
+import java.util.Stack;
 
 import org.w3c.dom.Document;
 
@@ -85,35 +86,39 @@ public abstract class AbstractRenderer
     /**
      * user agent
      */
-    protected FOUserAgent userAgent = null;
+    protected FOUserAgent userAgent;
 
     /**
      * block progression position
      */
-    protected int currentBPPosition = 0;
+    protected int currentBPPosition;
 
     /**
      * inline progression position
      */
-    protected int currentIPPosition = 0;
+    protected int currentIPPosition;
 
     /**
      * the block progression position of the containing block used for
      * absolutely positioned blocks
      */
-    protected int containingBPPosition = 0;
+    protected int containingBPPosition;
 
     /**
      * the inline progression position of the containing block used for
      * absolutely positioned blocks
      */
-    protected int containingIPPosition = 0;
+    protected int containingIPPosition;
 
     /** the currently active PageViewport */
     protected PageViewport currentPageViewport;
 
+    /* warned XML handlers */
     private Set warnedXMLHandlers;
 
+    /* layers stack */
+    private Stack<String> layers;
+
     /** {@inheritDoc} */
     public abstract void setupFontInfo(FontInfo fontInfo) throws FOPException;
 
@@ -260,6 +265,10 @@ public abstract class AbstractRenderer
         if (viewport != null) {
             renderRegionViewport(viewport);
         }
+        viewport = page.getRegionViewport(FO_REGION_BODY);
+        if (viewport != null) {
+            renderRegionViewport(viewport);
+        }
         viewport = page.getRegionViewport(FO_REGION_END);
         if (viewport != null) {
             renderRegionViewport(viewport);
@@ -268,10 +277,6 @@ public abstract class AbstractRenderer
         if (viewport != null) {
             renderRegionViewport(viewport);
         }
-        viewport = page.getRegionViewport(FO_REGION_BODY);
-        if (viewport != null) {
-            renderRegionViewport(viewport);
-        }
     }
 
     /**
@@ -296,6 +301,7 @@ public abstract class AbstractRenderer
         startVParea(regionReference.getCTM(), port.getClipRectangle());
         // do after starting viewport area
         if (regionReference.getRegionClass() == FO_REGION_BODY) {
+            assert (regionReference instanceof BodyRegion);
             renderBodyRegion((BodyRegion) regionReference);
         } else {
             renderRegion(regionReference);
@@ -410,10 +416,10 @@ public abstract class AbstractRenderer
         for (int count = 0; count < spans.size(); count++) {
             span = (Span) spans.get(count);
             int level = span.getBidiLevel();
-            if ( level < 0 ) {
+            if (level < 0) {
                 level = 0;
             }
-            if ( ( level & 1 ) == 1 ) {
+            if ((level & 1) == 1) {
                 currentIPPosition += span.getIPD();
                 currentIPPosition += mr.getColumnGap();
             }
@@ -421,12 +427,12 @@ public abstract class AbstractRenderer
                 NormalFlow flow = span.getNormalFlow(c);
                 if (flow != null) {
                     currentBPPosition = saveSpanBPPos;
-                    if ( ( level & 1 ) == 1 ) {
+                    if ((level & 1) == 1) {
                         currentIPPosition -= flow.getIPD();
                         currentIPPosition -= mr.getColumnGap();
                     }
                     renderFlow(flow);
-                    if ( ( level & 1 ) == 0 ) {
+                    if ((level & 1) == 0) {
                         currentIPPosition += flow.getIPD();
                         currentIPPosition += mr.getColumnGap();
                     }
@@ -471,6 +477,10 @@ public abstract class AbstractRenderer
      * @param children  The children to render within the block viewport
      */
     protected void renderBlockViewport(BlockViewport bv, List children) {
+        boolean inNewLayer = false;
+        if (maybeStartLayer(bv)) {
+            inNewLayer = true;
+        }
         // clip and position viewport if necessary
         if (bv.getPositioning() == Block.ABSOLUTE) {
             // save positions
@@ -506,6 +516,7 @@ public abstract class AbstractRenderer
             currentIPPosition = saveIP;
             currentBPPosition = saveBP + bv.getAllocBPD();
         }
+        maybeEndLayer(bv, inNewLayer);
     }
 
     /**
@@ -550,9 +561,9 @@ public abstract class AbstractRenderer
                 // a line area is rendered from the top left position
                 // of the line, each inline object is offset from there
                 LineArea line = (LineArea) obj;
-                if ( parent != null ) {
+                if (parent != null) {
                     int level = parent.getBidiLevel();
-                    if ( ( level == -1 ) || ( ( level & 1 ) == 0 ) ) {
+                    if ((level == -1) || ((level & 1) == 0)) {
                         currentIPPosition += parent.getStartIndent();
                     } else {
                         currentIPPosition += parent.getEndIndent();
@@ -573,6 +584,10 @@ public abstract class AbstractRenderer
     protected void renderBlock(Block block) {
         assert block != null;
         List children = block.getChildAreas();
+        boolean inNewLayer = false;
+        if (maybeStartLayer(block)) {
+            inNewLayer = true;
+        }
         if (block instanceof BlockViewport) {
             if (children != null) {
                 renderBlockViewport((BlockViewport) block, children);
@@ -607,6 +622,45 @@ public abstract class AbstractRenderer
                 currentBPPosition = saveBP + block.getAllocBPD();
             }
         }
+        maybeEndLayer(block, inNewLayer);
+    }
+
+    /**
+     * Establish new optional content group layer.
+     * @param layer name of layer
+     */
+    protected abstract void startLayer(String layer);
+
+    /**
+     * Finish current optional content group layer.
+     */
+    protected abstract void endLayer();
+
+    protected boolean maybeStartLayer(Area area) {
+        String layer = (String) area.getTrait(Trait.LAYER);
+        if (layer != null) {
+            if (layers == null) {
+                layers = new Stack<String>();
+            }
+            if (layers.empty() || !layers.peek().equals(layer)) {
+                layers.push(layer);
+                startLayer(layer);
+                return true;
+            }
+        }
+        return false;
+    }
+
+    protected void maybeEndLayer(Area area, boolean inNewLayer) {
+        if (inNewLayer) {
+            assert layers != null;
+            assert !layers.empty();
+            String layer = (String) area.getTrait(Trait.LAYER);
+            assert layer != null;
+            assert layers.peek().equals(layer);
+            endLayer();
+            layers.pop();
+        }
     }
 
     /**
@@ -622,18 +676,11 @@ public abstract class AbstractRenderer
         int saveBP = currentBPPosition;
         currentBPPosition += line.getSpaceBefore();
         int bl = line.getBidiLevel();
-        if ( bl >= 0 ) {
-            if ( ( bl & 1 ) == 0 ) {
+        if (bl >= 0) {
+            if ((bl & 1) == 0) {
                 currentIPPosition += line.getStartIndent();
             } else {
                 currentIPPosition += line.getEndIndent();
-                // if line's content overflows line area, then
-                // ensure that overflow is drawn (extends)
-                // outside of left side of line area
-                int overflow = computeInlinesOverflow ( line );
-                if ( overflow > 0 ) {
-                    currentIPPosition -= overflow;
-                }
             }
         } else {
             currentIPPosition += line.getStartIndent();
@@ -645,16 +692,6 @@ public abstract class AbstractRenderer
         currentBPPosition = saveBP;
     }
 
-    private int computeInlinesOverflow ( LineArea line ) {
-        List children = line.getInlineAreas();
-        int ipdConsumed = 0;
-        for (int i = 0, l = children.size(); i < l; i++) {
-            InlineArea inline = (InlineArea) children.get(i);
-            ipdConsumed += inline.getIPD();
-        }
-        return ipdConsumed - line.getIPD();
-    }
-
     /**
      * Render the given InlineArea.
      * @param inlineArea inline area text to render
@@ -746,6 +783,10 @@ public abstract class AbstractRenderer
      * @param ip the inline parent to render
      */
     protected void renderInlineParent(InlineParent ip) {
+        boolean inNewLayer = false;
+        if (maybeStartLayer(ip)) {
+            inNewLayer = true;
+        }
         int level = ip.getBidiLevel();
         List children = ip.getChildAreas();
         renderInlineAreaBackAndBorders(ip);
@@ -755,7 +796,7 @@ public abstract class AbstractRenderer
         // it is right-to-left, then adjust starting ip position in order to
         // align children to starting (right) edge of filled area
         int ipAdjust;
-        if ( ( ip instanceof FilledArea ) && ( ( level & 1 ) != 0 ) ) {
+        if ((ip instanceof FilledArea) && ((level & 1) != 0)) {
             int ipdChildren = 0;
             for (int i = 0, l = children.size(); i < l; i++) {
                 InlineArea inline = (InlineArea) children.get(i);
@@ -766,11 +807,11 @@ public abstract class AbstractRenderer
             ipAdjust = 0;
         }
         // perform inline position adjustments
-        if ( ( level == -1 ) || ( ( level & 1 ) == 0 ) ) {
+        if ((level == -1) || ((level & 1) == 0)) {
             currentIPPosition += ip.getBorderAndPaddingWidthStart();
         } else {
             currentIPPosition += ip.getBorderAndPaddingWidthEnd();
-            if ( ipAdjust > 0 ) {
+            if (ipAdjust > 0) {
                 currentIPPosition += ipAdjust;
             }
         }
@@ -782,6 +823,7 @@ public abstract class AbstractRenderer
         }
         currentIPPosition = saveIP + ip.getAllocIPD();
         currentBPPosition = saveBP;
+        maybeEndLayer(ip, inNewLayer);
     }
 
     /**
@@ -791,7 +833,7 @@ public abstract class AbstractRenderer
     protected void renderInlineBlockParent(InlineBlockParent ibp) {
         int level = ibp.getBidiLevel();
         renderInlineAreaBackAndBorders(ibp);
-        if ( ( level == -1 ) || ( ( level & 1 ) == 0 ) ) {
+        if ((level == -1) || ((level & 1) == 0)) {
             currentIPPosition += ibp.getBorderAndPaddingWidthStart();
         } else {
             currentIPPosition += ibp.getBorderAndPaddingWidthEnd();
