@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: XMLHandlerConfigurator.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: XMLHandlerConfigurator.java 1664570 2015-03-06 09:41:07Z lbernardo $ */
 
 package org.apache.fop.render;
 
@@ -23,17 +23,18 @@ import org.apache.avalon.framework.configuration.Configuration;
 import org.apache.avalon.framework.configuration.ConfigurationException;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
 
 /**
- * Configurator for XMLHandler objects. 
+ * Configurator for XMLHandler objects.
  */
 public class XMLHandlerConfigurator extends AbstractRendererConfigurator {
 
     /** logger instance */
-    protected static Log log = LogFactory.getLog(XMLHandlerConfigurator.class);
-    
+    protected static final Log log = LogFactory.getLog(XMLHandlerConfigurator.class);
+
     /**
      * Default constructor
      * @param userAgent the user agent
@@ -81,7 +82,7 @@ public class XMLHandlerConfigurator extends AbstractRendererConfigurator {
      */
     public void configure(RendererContext context, String ns) throws FOPException {
         //Optional XML handler configuration
-        Configuration cfg = getRendererConfig(context.getRenderer());
+        Configuration cfg = userAgent.getRendererConfiguration(context.getRenderer().getMimeType());
         if (cfg != null) {
             cfg = getHandlerConfig(cfg, ns);
             if (cfg != null) {
