@@ -15,11 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id: AFPImageHandler.java 1195952 2011-11-01 12:20:21Z phancock $ */
+/* $Id: AFPImageHandler.java 1681435 2015-05-24 11:14:22Z adelmelle $ */
 
 package org.apache.fop.render.afp;
 
+import java.awt.Point;
 import java.awt.Rectangle;
+import java.awt.geom.Rectangle2D;
+import java.io.IOException;
 import java.util.Map;
 
 import org.apache.fop.afp.AFPDataObjectInfo;
@@ -28,6 +31,7 @@ import org.apache.fop.afp.AFPPaintingState;
 import org.apache.fop.afp.AFPResourceInfo;
 import org.apache.fop.afp.AFPUnitConverter;
 import org.apache.fop.render.ImageHandlerBase;
+import org.apache.fop.render.RendererContext;
 
 /**
  * A base abstract AFP image handler
@@ -37,21 +41,61 @@ public abstract class AFPImageHandler implements ImageHandlerBase {
     private static final int Y = 1;
 
     /** foreign attribute reader */
-    private final AFPForeignAttributeReader foreignAttributeReader
+    private static final AFPForeignAttributeReader FOREIGN_ATTRIBUTE_READER
         = new AFPForeignAttributeReader();
 
+    /**
+     * Generates an intermediate AFPDataObjectInfo that is later used to construct
+     * the appropriate data object in the AFP DataStream.
+     *
+     * @param rendererImageInfo the renderer image info
+     * @return a data object info object
+     * @throws IOException thrown if an I/O exception of some sort has occurred.
+     */
+    public AFPDataObjectInfo generateDataObjectInfo(
+            AFPRendererImageInfo rendererImageInfo) throws IOException {
+        AFPDataObjectInfo dataObjectInfo = createDataObjectInfo();
+
+        // set resource information
+        dataObjectInfo.setResourceInfo(createResourceInformation(
+                rendererImageInfo.getURI(),
+                rendererImageInfo.getForeignAttributes()));
+
+
+        Point origin = rendererImageInfo.getOrigin();
+        Rectangle2D position = rendererImageInfo.getPosition();
+        int srcX = Math.round(origin.x + (float)position.getX());
+        int srcY = Math.round(origin.y + (float)position.getY());
+        Rectangle targetRect = new Rectangle(
+                srcX,
+                srcY,
+                (int)Math.round(position.getWidth()),
+                (int)Math.round(position.getHeight()));
+
+        RendererContext context = rendererImageInfo.getRendererContext();
+        assert (context instanceof AFPRendererContext);
+        AFPRendererContext rendererContext = (AFPRendererContext) context;
+        AFPInfo afpInfo = rendererContext.getInfo();
+        AFPPaintingState paintingState = afpInfo.getPaintingState();
+
+        dataObjectInfo.setObjectAreaInfo(createObjectAreaInfo(paintingState, targetRect));
+
+        return dataObjectInfo;
+    }
+
     /**
      * Sets resource information on the data object info.
-     * @param dataObjectInfo the data object info instance
      * @param uri the image's URI (or null if no URI is available)
      * @param foreignAttributes a Map of foreign attributes (or null)
+     * @return the resource information object
      */
-    protected void setResourceInformation(AFPDataObjectInfo dataObjectInfo,
+    public static AFPResourceInfo createResourceInformation(
             String uri, Map foreignAttributes) {
         AFPResourceInfo resourceInfo
-            = foreignAttributeReader.getResourceInfo(foreignAttributes);
+            = FOREIGN_ATTRIBUTE_READER.getResourceInfo(foreignAttributes);
         resourceInfo.setUri(uri);
-        dataObjectInfo.setResourceInfo(resourceInfo);
+
+       return resourceInfo;
     }
 
     /**
