@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AFPPageOverlay.java 1357883 2012-07-05 20:29:53Z gadams $ */
+/* $Id: AFPPageOverlay.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.render.afp.extensions;
 
@@ -39,12 +39,12 @@ public class AFPPageOverlay extends AFPExtensionAttachment {
     /**
      * The x coordinate
      */
-    private int x = 0;
+    private int x;
 
     /**
      * The y coordinate
      */
-    private int y = 0;
+    private int y;
 
     /**
      * Default constructor.
