@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AFPPageOverlayElement.java 1296526 2012-03-03 00:18:45Z gadams $ */
+/* $Id: AFPPageOverlayElement.java 1465599 2013-04-08 11:51:52Z vhennebert $ */
 
 package org.apache.fop.render.afp.extensions;
 
@@ -57,7 +57,7 @@ public class AFPPageOverlayElement extends AbstractAFPExtensionObject {
     }
 
     /** {@inheritDoc} */
-    protected void startOfNode() throws FOPException {
+    public void startOfNode() throws FOPException {
         super.startOfNode();
         if (AFPElementMapping.INCLUDE_PAGE_OVERLAY.equals(getLocalName())) {
             if (parent.getNameId() != Constants.FO_SIMPLE_PAGE_MASTER
