@@ -15,18 +15,19 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractAFPExtensionObject.java 560936 2007-07-30 12:20:15Z cbowditch $ */
+/* $Id: AbstractAFPExtensionObject.java 1465599 2013-04-08 11:51:52Z vhennebert $ */
 
 package org.apache.fop.render.afp.extensions;
 
 // FOP
+import org.xml.sax.Attributes;
+import org.xml.sax.Locator;
+
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.ValidationException;
 import org.apache.fop.fo.extensions.ExtensionAttachment;
-import org.xml.sax.Attributes;
-import org.xml.sax.Locator;
 
 /**
  * Base class for the AFP-specific extension elements.
@@ -34,12 +35,15 @@ import org.xml.sax.Locator;
 public abstract class AbstractAFPExtensionObject extends FONode {
 
     /**
-     * AFP setup code
+     * the AFP extension attachment
      */
-    private AFPPageSetup setupCode;
-    
-    private String name;
-            
+    protected AFPExtensionAttachment extensionAttachment;
+
+    /**
+     * the element name of this extension
+     */
+    protected String name;
+
     /**
      * @see org.apache.fop.fo.FONode#FONode(FONode)
      * @param parent the parent formatting object
@@ -48,7 +52,6 @@ public abstract class AbstractAFPExtensionObject extends FONode {
     public AbstractAFPExtensionObject(FONode parent, String name) {
         super(parent);
         this.name = name;
-        this.setupCode = new AFPPageSetup(name);
     }
 
     /** {@inheritDoc} */
@@ -59,12 +62,6 @@ public abstract class AbstractAFPExtensionObject extends FONode {
         }
     }
 
-    /** {@inheritDoc} */
-    protected void addCharacters(char[] data, int start, int end,
-                                 PropertyList pList, Locator locator) {
-        setupCode.setContent(new String(data, start, end - start));       
-    }
-
     /** {@inheritDoc} */
     public String getNamespaceURI() {
         return AFPElementMapping.NAMESPACE;
@@ -79,37 +76,32 @@ public abstract class AbstractAFPExtensionObject extends FONode {
     public void processNode(String elementName, Locator locator,
                             Attributes attlist, PropertyList propertyList)
                                 throws FOPException {
-        String name = attlist.getValue("name");
-        if (name != null && name.length() > 0) {
-            setupCode.setName(name);
+        getExtensionAttachment();
+        String attr = attlist.getValue("name");
+        if (attr != null && attr.length() > 0) {
+            extensionAttachment.setName(attr);
         } else {
             throw new FOPException(elementName + " must have a name attribute.");
         }
-        if (AFPElementMapping.INCLUDE_PAGE_SEGMENT.equals(elementName)) {
-            name = attlist.getValue("src");
-            if (name != null && name.length() > 0) {
-                setupCode.setValue(name);
-            } else {
-                throw new FOPException(elementName + " must have a src attribute.");
-            }
-        } else if (AFPElementMapping.TAG_LOGICAL_ELEMENT.equals(elementName)) {
-            name = attlist.getValue("value");
-            if (name != null && name.length() > 0) {
-                setupCode.setValue(name);
-            } else {
-                throw new FOPException(elementName + " must have a value attribute.");
-            }
-        }
     }
 
     /** {@inheritDoc} */
-    protected void endOfNode() throws FOPException {
+    public void endOfNode() throws FOPException {
         super.endOfNode();
     }
 
+    /**
+     * Instantiates extension attachment object
+     * @return extension attachment
+     */
+    protected abstract ExtensionAttachment instantiateExtensionAttachment();
+
     /** {@inheritDoc} */
     public ExtensionAttachment getExtensionAttachment() {
-        return this.setupCode;
+        if (extensionAttachment == null) {
+            this.extensionAttachment = (AFPExtensionAttachment)instantiateExtensionAttachment();
+        }
+        return this.extensionAttachment;
     }
 
     /** {@inheritDoc} */
