@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TIFFRenderer.java 746664 2009-02-22 12:40:44Z jeremias $ */
+/* $Id: TIFFRenderer.java 1616668 2014-08-08 03:48:54Z gadams $ */
 
 package org.apache.fop.render.bitmap;
 
@@ -31,20 +31,25 @@ import java.awt.image.SinglePixelPackedSampleModel;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.util.Iterator;
+import java.util.NoSuchElementException;
 
 import org.apache.commons.logging.Log;
 
 import org.apache.xmlgraphics.image.GraphicsUtil;
 import org.apache.xmlgraphics.image.rendered.FormatRed;
 import org.apache.xmlgraphics.image.writer.ImageWriter;
-import org.apache.xmlgraphics.image.writer.ImageWriterParams;
 import org.apache.xmlgraphics.image.writer.ImageWriterRegistry;
 import org.apache.xmlgraphics.image.writer.MultiImageWriter;
 
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
+import org.apache.fop.apps.MimeConstants;
 import org.apache.fop.render.java2d.Java2DRenderer;
 
+import static org.apache.fop.render.bitmap.TIFFCompressionValue.CCITT_T4;
+import static org.apache.fop.render.bitmap.TIFFCompressionValue.CCITT_T6;
+import static org.apache.fop.render.bitmap.TIFFCompressionValue.PACKBITS;
+
 /**
  * <p>
  * This class represents renderer to TIFF (Tagged Image File Format) format. It
@@ -66,37 +71,29 @@ import org.apache.fop.render.java2d.Java2DRenderer;
  * <code>org.apache.fop.render.java2D.Java2DRenderer</code> and just encode
  * rendering results into TIFF format using Batik's image codec
  */
-public class TIFFRenderer extends Java2DRenderer implements TIFFConstants {
+public class TIFFRenderer extends Java2DRenderer {
 
-    /** ImageWriter parameters */
-    private ImageWriterParams writerParams;
-
-    /** Image Type as parameter for the BufferedImage constructor (see BufferedImage.TYPE_*) */
-    private int bufferedImageType = BufferedImage.TYPE_INT_ARGB;
+    private BitmapRenderingSettings imageSettings;
 
     private OutputStream outputStream;
 
     /** {@inheritDoc} */
     public String getMimeType() {
-        return MIME_TYPE;
-    }
-
-    /** Creates TIFF renderer. */
-    public TIFFRenderer() {
-        writerParams = new ImageWriterParams();
-        writerParams.setCompressionMethod(COMPRESSION_PACKBITS);
+        return MimeConstants.MIME_TIFF;
     }
 
     /**
-     * {@inheritDoc}
-     *          org.apache.fop.apps.FOUserAgent)
+     * Creates TIFF renderer.
+     *
+     * @param userAgent the user agent that contains configuration details. This cannot be null.
      */
-    public void setUserAgent(FOUserAgent foUserAgent) {
-        super.setUserAgent(foUserAgent);
-
-        //Set target resolution
+    public TIFFRenderer(FOUserAgent userAgent) {
+        super(userAgent);
+        imageSettings = new BitmapRenderingSettings();
+        imageSettings.setCompressionMethod(PACKBITS.getName());
+        imageSettings.setBufferedImageType(BufferedImage.TYPE_INT_ARGB);
         int dpi = Math.round(userAgent.getTargetResolution());
-        writerParams.setResolution(dpi);
+        imageSettings.setResolution(dpi);
     }
 
     /** {@inheritDoc} */
@@ -120,37 +117,41 @@ public class TIFFRenderer extends Java2DRenderer implements TIFFConstants {
                 = BitmapRendererEventProducer.Provider.get(
                         getUserAgent().getEventBroadcaster());
             eventProducer.noImageWriterFound(this, getMimeType());
-        }
-        if (writer.supportsMultiImageWriter()) {
-            MultiImageWriter multiWriter = writer.createMultiImageWriter(outputStream);
-            try {
-                // Write all pages/images
-                while (pageImagesItr.hasNext()) {
-                    RenderedImage img = (RenderedImage) pageImagesItr.next();
-                    multiWriter.writeImage(img, writerParams);
-                }
-            } finally {
-                multiWriter.close();
-            }
         } else {
-            writer.writeImage((RenderedImage) pageImagesItr.next(), outputStream, writerParams);
-            if (pageImagesItr.hasNext()) {
-                BitmapRendererEventProducer eventProducer
-                    = BitmapRendererEventProducer.Provider.get(
-                            getUserAgent().getEventBroadcaster());
-                eventProducer.stoppingAfterFirstPageNoFilename(this);
+            if (writer.supportsMultiImageWriter()) {
+                MultiImageWriter multiWriter = writer.createMultiImageWriter(outputStream);
+                try {
+                    // Write all pages/images
+                    while (pageImagesItr.hasNext()) {
+                        RenderedImage img = (RenderedImage) pageImagesItr.next();
+                        multiWriter.writeImage(img, imageSettings.getWriterParams());
+                    }
+                } finally {
+                    multiWriter.close();
+                }
+            } else {
+                RenderedImage renderedImage = null;
+                if (pageImagesItr.hasNext()) {
+                    renderedImage = (RenderedImage) pageImagesItr.next();
+                }
+                writer.writeImage(renderedImage, outputStream, imageSettings.getWriterParams());
+                if (pageImagesItr.hasNext()) {
+                    BitmapRendererEventProducer eventProducer
+                        = BitmapRendererEventProducer.Provider.get(
+                                getUserAgent().getEventBroadcaster());
+                    eventProducer.stoppingAfterFirstPageNoFilename(this);
+                }
             }
+            // Cleaning
+            outputStream.flush();
+            clearViewportList();
         }
-
-        // Cleaning
-        outputStream.flush();
-        clearViewportList();
         log.debug("TIFF encoding done.");
     }
 
     /** {@inheritDoc} */
     protected BufferedImage getBufferedImage(int bitmapWidth, int bitmapHeight) {
-        return new BufferedImage(bitmapWidth, bitmapHeight, bufferedImageType);
+        return new BufferedImage(bitmapWidth, bitmapHeight, imageSettings.getBufferedImageType());
     }
 
     /** Private inner class to lazy page rendering. */
@@ -160,7 +161,7 @@ public class TIFFRenderer extends Java2DRenderer implements TIFFConstants {
 
         private int count;
 
-        private int current = 0;
+        private int current;
 
         /**
          * Main constructor
@@ -186,12 +187,11 @@ public class TIFFRenderer extends Java2DRenderer implements TIFFConstants {
             try {
                 pageImage = getPageImage(current++);
             } catch (FOPException e) {
-                log.error(e);
-                return null;
+                throw new NoSuchElementException(e.getMessage());
             }
 
-            if (COMPRESSION_CCITT_T4.equalsIgnoreCase(writerParams.getCompressionMethod())
-                   || COMPRESSION_CCITT_T6.equalsIgnoreCase(writerParams.getCompressionMethod())) {
+            TIFFCompressionValue compression = TIFFCompressionValue.getType(imageSettings.getCompressionMethod());
+            if (compression == CCITT_T4 || compression == CCITT_T6) {
                 return pageImage;
             } else {
                 //Decorate the image with a packed sample model for encoding by the codec
@@ -219,11 +219,16 @@ public class TIFFRenderer extends Java2DRenderer implements TIFFConstants {
         }
     }
 
+    /** @param bufferedImageType an image type */
     public void setBufferedImageType(int bufferedImageType) {
-        this.bufferedImageType = bufferedImageType;
+        imageSettings.setBufferedImageType(bufferedImageType);
     }
 
-    public ImageWriterParams getWriterParams() {
-        return writerParams;
+    /**
+     * Returns the settings for the image rendering.
+     * @return the image rendering settings
+     */
+    public BitmapRenderingSettings getRenderingSettings() {
+        return imageSettings;
     }
 }
