@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractIFPainter.java 1293736 2012-02-26 02:29:01Z gadams $ */
+/* $Id: AbstractIFPainter.java 1717946 2015-12-04 12:18:50Z ssteiner $ */
 
 package org.apache.fop.render.intermediate;
 
@@ -45,8 +45,9 @@ import org.apache.xmlgraphics.image.loader.util.ImageUtil;
 
 import org.apache.fop.ResourceEventProducer;
 import org.apache.fop.apps.FOUserAgent;
-import org.apache.fop.apps.FopFactory;
 import org.apache.fop.fo.Constants;
+import org.apache.fop.fonts.FontInfo;
+import org.apache.fop.fonts.FontTriplet;
 import org.apache.fop.render.ImageHandler;
 import org.apache.fop.render.ImageHandlerRegistry;
 import org.apache.fop.render.ImageHandlerUtil;
@@ -57,7 +58,7 @@ import org.apache.fop.traits.RuleStyle;
 /**
  * Abstract base class for IFPainter implementations.
  */
-public abstract class AbstractIFPainter implements IFPainter {
+public abstract class AbstractIFPainter<T extends IFDocumentHandler> implements IFPainter {
 
     /** logging instance */
     private static Log log = LogFactory.getLog(AbstractIFPainter.class);
@@ -68,18 +69,39 @@ public abstract class AbstractIFPainter implements IFPainter {
     /** Holds the intermediate format state */
     protected IFState state;
 
+    private final T documentHandler;
 
     /**
      * Default constructor.
      */
-    public AbstractIFPainter() {
+    public AbstractIFPainter(T documentHandler) {
+        this.documentHandler = documentHandler;
+    }
+
+    protected String getFontKey(FontTriplet triplet) throws IFException {
+        String key = getFontInfo().getInternalFontKey(triplet);
+        if (key == null) {
+            throw new IFException("The font triplet is not available: \"" + triplet + "\" "
+                    + "for the MIME type: \"" + documentHandler.getMimeType() + "\"");
+        }
+        return key;
     }
 
     /**
      * Returns the intermediate format context object.
      * @return the context object
      */
-    protected abstract IFContext getContext();
+    protected IFContext getContext() {
+        return documentHandler.getContext();
+    }
+
+    protected FontInfo getFontInfo() {
+        return documentHandler.getFontInfo();
+    }
+
+    protected T getDocumentHandler() {
+        return documentHandler;
+    }
 
     /**
      * Returns the user agent.
@@ -89,14 +111,6 @@ public abstract class AbstractIFPainter implements IFPainter {
         return getContext().getUserAgent();
     }
 
-    /**
-     * Returns the FOP factory.
-     * @return the FOP factory.
-     */
-    protected FopFactory getFopFactory() {
-        return getUserAgent().getFactory();
-    }
-
     private AffineTransform combine(AffineTransform[] transforms) {
         AffineTransform at = new AffineTransform();
         for (int i = 0, c = transforms.length; i < c; i++) {
@@ -112,8 +126,8 @@ public abstract class AbstractIFPainter implements IFPainter {
     }
 
     /** {@inheritDoc} */
-    public void startGroup(AffineTransform[] transforms) throws IFException {
-        startGroup(combine(transforms));
+    public void startGroup(AffineTransform[] transforms, String layer) throws IFException {
+        startGroup(combine(transforms), layer);
     }
 
     /**
@@ -131,9 +145,9 @@ public abstract class AbstractIFPainter implements IFPainter {
      */
     protected void drawImageUsingImageHandler(ImageInfo info, Rectangle rect)
                     throws ImageException, IOException {
-        ImageManager manager = getFopFactory().getImageManager();
+        ImageManager manager = getUserAgent().getImageManager();
         ImageSessionContext sessionContext = getUserAgent().getImageSessionContext();
-        ImageHandlerRegistry imageHandlerRegistry = getFopFactory().getImageHandlerRegistry();
+        ImageHandlerRegistry imageHandlerRegistry = getUserAgent().getImageHandlerRegistry();
 
         //Load and convert the image to a supported format
         RenderingContext context = createRenderingContext();
@@ -141,10 +155,17 @@ public abstract class AbstractIFPainter implements IFPainter {
         context.putHints(hints);
 
         ImageFlavor[] flavors = imageHandlerRegistry.getSupportedFlavors(context);
+        info.getCustomObjects().put("warningincustomobject", true);
         org.apache.xmlgraphics.image.loader.Image img = manager.getImage(
                     info, flavors,
                     hints, sessionContext);
 
+        if (info.getCustomObjects().get("warning") != null) {
+            ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
+                    getUserAgent().getEventBroadcaster());
+            eventProducer.imageWarning(this, (String)info.getCustomObjects().get("warning"));
+        }
+
         try {
             drawImage(img, rect, context);
         } catch (IOException ioe) {
@@ -167,7 +188,7 @@ public abstract class AbstractIFPainter implements IFPainter {
         if (conversionMode != null) {
             hints.put(ImageHandlerUtil.CONVERSION_MODE, conversionMode);
         }
-
+        hints.put("page-number", documentHandler.getContext().getPageNumber());
         return hints;
     }
 
@@ -197,8 +218,8 @@ public abstract class AbstractIFPainter implements IFPainter {
     protected void drawImage(Image image, Rectangle rect,
             RenderingContext context, boolean convert, Map additionalHints)
                     throws IOException, ImageException {
-        ImageManager manager = getFopFactory().getImageManager();
-        ImageHandlerRegistry imageHandlerRegistry = getFopFactory().getImageHandlerRegistry();
+        ImageManager manager = getUserAgent().getImageManager();
+        ImageHandlerRegistry imageHandlerRegistry = getUserAgent().getImageHandlerRegistry();
 
         Image effImage;
         context.putHints(additionalHints);
@@ -224,7 +245,7 @@ public abstract class AbstractIFPainter implements IFPainter {
         if (log.isTraceEnabled()) {
             log.trace("Using ImageHandler: " + handler.getClass().getName());
         }
-
+        context.putHint("fontinfo", getFontInfo());
         handler.handleImage(context, effImage, rect);
     }
 
@@ -236,7 +257,7 @@ public abstract class AbstractIFPainter implements IFPainter {
      * @return the ImageInfo instance or null if there has been an error.
      */
     protected ImageInfo getImageInfo(String uri) {
-        ImageManager manager = getFopFactory().getImageManager();
+        ImageManager manager = getUserAgent().getImageManager();
         try {
             ImageSessionContext sessionContext = getUserAgent().getImageSessionContext();
             return manager.getImageInfo(uri, sessionContext);
@@ -262,7 +283,7 @@ public abstract class AbstractIFPainter implements IFPainter {
      * @param rect the rectangle in which to paint the image
      */
     protected void drawImageUsingURI(String uri, Rectangle rect) {
-        ImageManager manager = getFopFactory().getImageManager();
+        ImageManager manager = getUserAgent().getImageManager();
         ImageInfo info = null;
         try {
             ImageSessionContext sessionContext = getUserAgent().getImageSessionContext();
@@ -290,7 +311,7 @@ public abstract class AbstractIFPainter implements IFPainter {
      * @param rect the rectangle in which to paint the image
      */
     protected void drawImageUsingDocument(Document doc, Rectangle rect) {
-        ImageManager manager = getFopFactory().getImageManager();
+        ImageManager manager = getUserAgent().getImageManager();
         ImageInfo info = null;
         try {
             info = manager.preloadImage(null, new DOMSource(doc));
@@ -316,7 +337,7 @@ public abstract class AbstractIFPainter implements IFPainter {
 
     /** {@inheritDoc} */
     public void drawBorderRect(Rectangle rect, BorderProps top, BorderProps bottom,
-            BorderProps left, BorderProps right) throws IFException {
+            BorderProps left, BorderProps right, Color innerBackgroundColor) throws IFException {
         if (top != null) {
             Rectangle b = new Rectangle(
                     rect.x, rect.y,
@@ -438,4 +459,10 @@ public abstract class AbstractIFPainter implements IFPainter {
         return new AffineTransform(matrix);
     }
 
+    /** {@inheritDoc} */
+    public boolean isBackgroundRequired(BorderProps bpsBefore, BorderProps bpsAfter,
+            BorderProps bpsStart, BorderProps bpsEnd) {
+        return true;
+    }
+
 }
