@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: IFParser.java 1342680 2012-05-25 15:15:28Z vhennebert $ */
+/* $Id: IFParser.java 1717946 2015-12-04 12:18:50Z ssteiner $ */
 
 package org.apache.fop.render.intermediate;
 
@@ -107,14 +107,16 @@ public class IFParser implements IFConstants {
 
             transformer.transform(src, res);
         } catch (TransformerException te) {
+            Throwable cause = te.getCause();
             //Unpack original IFException if applicable
-            if (te.getCause() instanceof SAXException) {
-                SAXException se = (SAXException)te.getCause();
-                if (se.getCause() instanceof IFException) {
-                    throw (IFException)se.getCause();
+            if (cause instanceof SAXException) {
+                SAXException se = (SAXException) cause;
+                cause = se.getCause();
+                if (cause instanceof IFException) {
+                    throw (IFException) cause;
                 }
-            } else if (te.getCause() instanceof IFException) {
-                throw (IFException)te.getCause();
+            } else if (cause instanceof IFException) {
+                throw (IFException) cause;
             }
             throw te;
         }
@@ -130,7 +132,7 @@ public class IFParser implements IFConstants {
     public ContentHandler getContentHandler(IFDocumentHandler documentHandler,
                     FOUserAgent userAgent) {
         ElementMappingRegistry elementMappingRegistry
-            = userAgent.getFactory().getElementMappingRegistry();
+            = userAgent.getElementMappingRegistry();
         return new Handler(documentHandler, userAgent, elementMappingRegistry);
     }
 
@@ -163,25 +165,15 @@ public class IFParser implements IFConstants {
         private Map<String, StructureTreeElement> structureTreeElements
                 = new HashMap<String, StructureTreeElement>();
 
-        private final class StructureTreeHandler extends DefaultHandler {
+        private class StructureTreeHandler extends DefaultHandler {
 
-            private final Locale pageSequenceLanguage;
+            protected final StructureTreeEventHandler structureTreeEventHandler;
 
-            private final StructureTreeEventHandler structureTreeEventHandler;
-
-            private StructureTreeHandler(StructureTreeEventHandler structureTreeEventHandler,
-                    Locale pageSequenceLanguage) throws SAXException {
-                this.pageSequenceLanguage = pageSequenceLanguage;
+            StructureTreeHandler(StructureTreeEventHandler structureTreeEventHandler) {
                 this.structureTreeEventHandler = structureTreeEventHandler;
             }
 
             void startStructureTree(String type) {
-                structureTreeEventHandler.startPageSequence(pageSequenceLanguage, type);
-            }
-
-            public void endDocument() throws SAXException {
-                startIFElement(EL_PAGE_SEQUENCE, pageSequenceAttributes);
-                pageSequenceAttributes = null;
             }
 
             @Override
@@ -191,18 +183,19 @@ public class IFParser implements IFConstants {
                     if (localName.equals("marked-content")) {
                         localName = "#PCDATA";
                     }
+                    StructureTreeElement parent = getStructureTreeElement(attributes);
                     String structID = attributes.getValue(InternalElementMapping.URI,
                             InternalElementMapping.STRUCT_ID);
                     if (structID == null) {
-                        structureTreeEventHandler.startNode(localName, attributes);
+                        structureTreeEventHandler.startNode(localName, attributes, parent);
                     } else if (localName.equals("external-graphic")
                             || localName.equals("instream-foreign-object")) {
                         StructureTreeElement structureTreeElement
-                                = structureTreeEventHandler.startImageNode(localName, attributes);
+                                = structureTreeEventHandler.startImageNode(localName, attributes, parent);
                         structureTreeElements.put(structID, structureTreeElement);
                     } else {
                         StructureTreeElement structureTreeElement = structureTreeEventHandler
-                                    .startReferencedNode(localName, attributes);
+                                    .startReferencedNode(localName, attributes, parent);
                         structureTreeElements.put(structID, structureTreeElement);
                     }
                 }
@@ -217,6 +210,28 @@ public class IFParser implements IFConstants {
             }
         }
 
+        private class MainStructureTreeHandler extends StructureTreeHandler {
+
+            private final Locale pageSequenceLanguage;
+
+            MainStructureTreeHandler(StructureTreeEventHandler structureTreeEventHandler,
+                    Locale pageSequenceLanguage) throws SAXException {
+                super(structureTreeEventHandler);
+                this.pageSequenceLanguage = pageSequenceLanguage;
+            }
+
+            @Override
+            void startStructureTree(String type) {
+                structureTreeEventHandler.startPageSequence(pageSequenceLanguage, type);
+            }
+
+            public void endDocument() throws SAXException {
+                startIFElement(EL_PAGE_SEQUENCE, pageSequenceAttributes);
+                pageSequenceAttributes = null;
+            }
+
+        }
+
         public Handler(IFDocumentHandler documentHandler, FOUserAgent userAgent,
                 ElementMappingRegistry elementMappingRegistry) {
             this.documentHandler = documentHandler;
@@ -264,7 +279,7 @@ public class IFParser implements IFConstants {
                     if (localName.equals(EL_PAGE_SEQUENCE) && userAgent.isAccessibilityEnabled()) {
                         pageSequenceAttributes = new AttributesImpl(attributes);
                         Locale language = getLanguage(attributes);
-                        structureTreeHandler = new StructureTreeHandler(
+                        structureTreeHandler = new MainStructureTreeHandler(
                                 userAgent.getStructureTreeEventHandler(), language);
 
                     } else if (localName.equals(EL_STRUCTURE_TREE)) {
@@ -306,7 +321,7 @@ public class IFParser implements IFConstants {
                     delegate.startElement(uri, localName, qName, attributes);
                 } else {
                     ContentHandlerFactoryRegistry registry
-                            = userAgent.getFactory().getContentHandlerFactoryRegistry();
+                            = userAgent.getContentHandlerFactoryRegistry();
                     ContentHandlerFactory factory = registry.getFactory(uri);
                     if (factory == null) {
                         DOMImplementation domImplementation
@@ -362,9 +377,10 @@ public class IFParser implements IFConstants {
         }
 
         private void handleIFException(IFException ife) throws SAXException {
-            if (ife.getCause() instanceof SAXException) {
+            Throwable cause = ife.getCause();
+            if (cause instanceof SAXException) {
                 //unwrap
-                throw (SAXException)ife.getCause();
+                throw (SAXException) cause;
             } else {
                 //wrap
                 throw new SAXException(ife);
@@ -507,6 +523,7 @@ public class IFParser implements IFConstants {
                 establishForeignAttributes(foreignAttributes);
                 documentHandler.startPage(index, name, pageMasterName,
                         new Dimension(width, height));
+                documentHandler.getContext().setPageNumber(Integer.parseInt(name));
                 resetForeignAttributes();
             }
 
@@ -520,6 +537,7 @@ public class IFParser implements IFConstants {
 
             public void startElement(Attributes attributes) throws IFException {
                 documentHandler.startPageHeader();
+                structureTreeHandler = new StructureTreeHandler(userAgent.getStructureTreeEventHandler());
             }
 
             public void endElement() throws IFException {
@@ -578,7 +596,8 @@ public class IFParser implements IFConstants {
                 String transform = attributes.getValue("transform");
                 AffineTransform[] transforms
                     = AffineTransformArrayParser.createAffineTransform(transform);
-                painter.startGroup(transforms);
+                String layer = attributes.getValue("layer");
+                painter.startGroup(transforms, layer);
             }
 
             public void endElement() throws IFException {
@@ -629,11 +648,16 @@ public class IFParser implements IFConstants {
                 int[][] dp = XMLUtil.getAttributeAsPositionAdjustments(lastAttributes, "dp");
                 // if only DX present, then convert DX to DP; otherwise use only DP,
                 // effectively ignoring DX
-                if ( ( dp == null ) && ( dx != null ) ) {
-                    dp = IFUtil.convertDXToDP ( dx );
+                if ((dp == null) && (dx != null)) {
+                    dp = IFUtil.convertDXToDP(dx);
                 }
                 establishStructureTreeElement(lastAttributes);
+                boolean isHyphenated = Boolean.valueOf(lastAttributes.getValue("hyphenated"));
+                if (isHyphenated) {
+                    documentHandler.getContext().setHyphenated(isHyphenated);
+                }
                 painter.drawText(x, y, letterSpacing, wordSpacing, dp, content.toString());
+                documentHandler.getContext().setHyphenated(false);
                 resetStructureTreeElement();
             }
 
@@ -650,6 +674,19 @@ public class IFParser implements IFConstants {
                 int y = Integer.parseInt(attributes.getValue("y"));
                 int width = Integer.parseInt(attributes.getValue("width"));
                 int height = Integer.parseInt(attributes.getValue("height"));
+                BorderProps[] borders = new BorderProps[4];
+                for (int i = 0; i < 4; i++) {
+                    String b = attributes.getValue(SIDES[i]);
+                    if (b != null) {
+                        borders[i] = BorderProps.valueOf(userAgent, b);
+                    }
+                }
+
+                if (!(borders[0] == null && borders[1] == null
+                        && borders[2] == null && borders[3] == null)) {
+                    painter.clipBackground(new Rectangle(x, y, width, height),
+                            borders[0], borders[1], borders[2], borders[3]);
+                }
                 painter.clipRect(new Rectangle(x, y, width, height));
             }
 
@@ -668,7 +705,26 @@ public class IFParser implements IFConstants {
                 } catch (PropertyException pe) {
                     throw new IFException("Error parsing the fill attribute", pe);
                 }
-                painter.fillRect(new Rectangle(x, y, width, height), fillColor);
+
+                Rectangle rectangularArea = new Rectangle(x, y, width, height);
+
+                //TODO should rect be overloaded to include rounded corners
+                // or should we introduce a new IF element?
+                BorderProps[] borders = new BorderProps[4];
+                for (int i = 0; i < 4; i++) {
+                    String b = attributes.getValue(SIDES[i]);
+                    if (b != null) {
+                        borders[i] = BorderProps.valueOf(userAgent, b);
+                    }
+                }
+
+                if (!(borders[0] == null && borders[1] == null
+                        && borders[2] == null && borders[3] == null)) {
+                    painter.clipBackground(rectangularArea,
+                            borders[0], borders[1], borders[2], borders[3]);
+                }
+
+                painter.fillRect(rectangularArea , fillColor);
             }
 
         }
@@ -709,9 +765,16 @@ public class IFParser implements IFConstants {
                         borders[i] = BorderProps.valueOf(userAgent, b);
                     }
                 }
+                Color backgroundColor;
+
+                try {
+                    backgroundColor = getAttributeAsColor(attributes, "inner-background-color");
+                } catch (PropertyException pe) {
+                    throw new IFException("Error parsing the color attribute", pe);
+                }
 
                 painter.drawBorderRect(new Rectangle(x, y, width, height),
-                        borders[0], borders[1], borders[2], borders[3]);
+                        borders[0], borders[1], borders[2], borders[3], backgroundColor);
             }
 
         }
@@ -742,8 +805,8 @@ public class IFParser implements IFConstants {
                     }
                     painter.drawImage(uri, new Rectangle(x, y, width, height));
                 }
-                resetForeignAttributes();
                 resetStructureTreeElement();
+                resetForeignAttributes();
                 inForeignObject = false;
             }
 
@@ -798,12 +861,19 @@ public class IFParser implements IFConstants {
         }
 
         private void establishStructureTreeElement(Attributes attributes) {
-            String structRef = attributes.getValue(InternalElementMapping.URI,
-                    InternalElementMapping.STRUCT_REF);
+            StructureTreeElement element = getStructureTreeElement(attributes);
+            if (element != null) {
+                documentHandler.getContext().setStructureTreeElement(element);
+            }
+        }
+
+        private StructureTreeElement getStructureTreeElement(Attributes attributes) {
+            String structRef = attributes.getValue(InternalElementMapping.URI, InternalElementMapping.STRUCT_REF);
             if (structRef != null && structRef.length() > 0) {
                 assert structureTreeElements.containsKey(structRef);
-                StructureTreeElement structureTreeElement = structureTreeElements.get(structRef);
-                documentHandler.getContext().setStructureTreeElement(structureTreeElement);
+                return structureTreeElements.get(structRef);
+            } else {
+                return null;
             }
         }
 
