@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: IFRenderer.java 1297008 2012-03-05 11:19:47Z vhennebert $ */
+/* $Id: IFRenderer.java 1717946 2015-12-04 12:18:50Z ssteiner $ */
 
 package org.apache.fop.render.intermediate;
 
@@ -27,7 +27,6 @@ import java.awt.geom.AffineTransform;
 import java.awt.geom.Rectangle2D;
 import java.io.IOException;
 import java.io.OutputStream;
-import java.util.Arrays;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Locale;
@@ -123,7 +122,7 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
     /** If not null, the XMLRenderer will mimic another renderer by using its font setup. */
     protected Renderer mimic;
 
-    private boolean inPageSequence = false;
+    private boolean inPageSequence;
 
     private Stack graphicContextStack = new Stack();
     private Stack viewportDimensionStack = new Stack();
@@ -198,8 +197,9 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
     }
 
     private void handleIFExceptionWithIOException(IFException ife) throws IOException {
-        if (ife.getCause() instanceof IOException) {
-            throw (IOException)ife.getCause();
+        Throwable cause = ife.getCause();
+        if (cause instanceof IOException) {
+            throw (IOException) cause;
         } else {
             handleIFException(ife);
         }
@@ -232,9 +232,8 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
      * @return the default IFDocumentHandler
      */
     protected IFDocumentHandler createDefaultDocumentHandler() {
-        IFSerializer serializer = new IFSerializer();
         FOUserAgent userAgent = getUserAgent();
-        serializer.setContext(new IFContext(userAgent));
+        IFSerializer serializer = new IFSerializer(new IFContext(userAgent));
         if (userAgent.isAccessibilityEnabled()) {
             userAgent.setStructureTreeEventHandler(serializer.getStructureTreeEventHandler());
         }
@@ -402,9 +401,9 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
         GoToXYAction action = (GoToXYAction)actionSet.get(targetID);
         //GoToXYAction action = (GoToXYAction)idGoTos.get(targetID);
         if (action == null) {
-            if (pageIndex < 0) {
+            /* if (pageIndex < 0) {
                 //pageIndex = page
-            }
+            } */
             Point position = (Point)idPositions.get(targetID);
             // can the GoTo already be fully filled in?
             if (pageIndex >= 0 && position != null) {
@@ -521,7 +520,7 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
                 this.inPageSequence = true;
             }
             establishForeignAttributes(pageSequence.getForeignAttributes());
-            documentHandler.getContext().setLanguage(toLocale(pageSequence));
+            documentHandler.getContext().setLanguage(pageSequence.getLocale());
             documentHandler.startPageSequence(null);
             resetForeignAttributes();
             processExtensionAttachments(pageSequence);
@@ -530,17 +529,6 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
         }
     }
 
-    private Locale toLocale(PageSequence pageSequence) {
-        if (pageSequence.getLanguage() != null) {
-            if (pageSequence.getCountry() != null) {
-                return new Locale(pageSequence.getLanguage(), pageSequence.getCountry());
-            } else {
-                return new Locale(pageSequence.getLanguage());
-            }
-        }
-        return null;
-    }
-
     private Metadata createDefaultDocumentMetadata() {
         Metadata xmp = new Metadata();
         DublinCoreAdapter dc = DublinCoreSchema.getAdapter(xmp);
@@ -584,6 +572,8 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
             Dimension dim = new Dimension(viewArea.width, viewArea.height);
 
             establishForeignAttributes(page.getForeignAttributes());
+            documentHandler.getContext().setPageIndex(page.getPageIndex());
+            documentHandler.getContext().setPageNumber(page.getPageNumber());
             documentHandler.startPage(page.getPageIndex(), page.getPageNumberString(),
                     page.getSimplePageMasterName(), dim);
             resetForeignAttributes();
@@ -611,6 +601,7 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
 
             establishForeignAttributes(page.getForeignAttributes());
             documentHandler.endPage();
+            documentHandler.getContext().setPageIndex(-1);
             resetForeignAttributes();
         } catch (IFException e) {
             handleIFException(e);
@@ -753,6 +744,12 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
     protected void renderBlockViewport(BlockViewport bv, List children) {
         //Essentially the same code as in the super class but optimized for the IF
 
+        // Handle new layer.
+        boolean inNewLayer = false;
+        if (maybeStartLayer(bv)) {
+            inNewLayer = true;
+        }
+
         //This is the content-rect
         Dimension dim = new Dimension(bv.getIPD(), bv.getBPD());
         viewportDimensionStack.push(dim);
@@ -853,6 +850,7 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
             currentBPPosition += bv.getAllocBPD();
         }
         viewportDimensionStack.pop();
+        maybeEndLayer(bv, inNewLayer);
     }
 
     /** {@inheritDoc} */
@@ -860,7 +858,7 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
         StructureTreeElement structElem
                 = (StructureTreeElement) viewport.getTrait(Trait.STRUCTURE_TREE_ELEMENT);
         establishStructureTreeElement(structElem);
-        pushdID(viewport);
+        pushID(viewport);
         Dimension dim = new Dimension(viewport.getIPD(), viewport.getBPD());
         viewportDimensionStack.push(dim);
         super.renderInlineViewport(viewport);
@@ -906,10 +904,27 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
         restoreGraphicsState();
     }
 
+    /** {@inheritDoc} */
+    protected void startLayer(String layer) {
+        if (log.isTraceEnabled()) {
+            log.trace("startLayer() layer=" + layer);
+        }
+        saveGraphicsState();
+        pushGroup(new IFGraphicContext.Group(layer));
+    }
+
+    /** {@inheritDoc} */
+    protected void endLayer() {
+        if (log.isTraceEnabled()) {
+            log.trace("endLayer()");
+        }
+        restoreGraphicsState();
+    }
+
     /** {@inheritDoc} */
     protected void renderInlineArea(InlineArea inlineArea) {
         saveInlinePosIfTargetable(inlineArea);
-        pushdID(inlineArea);
+        pushID(inlineArea);
         super.renderInlineArea(inlineArea);
         popID(inlineArea);
     }
@@ -976,12 +991,19 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
             log.trace("renderBlock() " + block);
         }
         saveBlockPosIfTargetable(block);
-        pushdID(block);
+        pushID(block);
+        IFContext context = documentHandler.getContext();
+        Locale oldLocale = context.getLanguage();
+        context.setLanguage(block.getLocale());
+        String oldLocation = context.getLocation();
+        context.setLocation(block.getLocation());
         super.renderBlock(block);
+        context.setLocation(oldLocation);
+        context.setLanguage(oldLocale);
         popID(block);
     }
 
-    private void pushdID(Area area) {
+    private void pushID(Area area) {
         String prodID = (String) area.getTrait(Trait.PROD_ID);
         if (prodID != null) {
             ids.push(prodID);
@@ -998,7 +1020,7 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
     }
 
     private Typeface getTypeface(String fontName) {
-        Typeface tf = (Typeface) fontInfo.getFonts().get(fontName);
+        Typeface tf = fontInfo.getFonts().get(fontName);
         if (tf instanceof LazyFont) {
             tf = ((LazyFont)tf).getRealFont();
         }
@@ -1037,10 +1059,12 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
         textUtil.flush();
         textUtil.setStartPosition(rx, bl);
         textUtil.setSpacing(text.getTextLetterSpaceAdjust(), text.getTextWordSpaceAdjust());
+        documentHandler.getContext().setHyphenated(text.isHyphenated());
         super.renderText(text);
 
         textUtil.flush();
         renderTextDecoration(tf, size, text, bl, rx);
+        documentHandler.getContext().setHyphenated(false);
         resetStructurePointer();
     }
 
@@ -1050,13 +1074,19 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
         String s = word.getWord();
 
         int[][] dp = word.getGlyphPositionAdjustments();
-        if ( dp == null ) {
-            dp = IFUtil.convertDXToDP ( word.getLetterAdjustArray() );
+        Area parentArea = word.getParentArea();
+        assert (parentArea instanceof AbstractTextArea);
+        if (dp == null) {
+            renderTextWithAdjustments(s, word.getLetterAdjustArray(), word.isReversed(),
+                    font, (AbstractTextArea) parentArea);
+        } else if (IFUtil.isDPOnlyDX(dp)) {
+            renderTextWithAdjustments(s, IFUtil.convertDPToDX(dp), word.isReversed(),
+                    font, (AbstractTextArea) parentArea);
+        } else {
+            renderTextWithAdjustments(s, dp, word.isReversed(),
+                    font, (AbstractTextArea) parentArea);
         }
 
-        renderText(s, dp, word.isReversed(),
-                font, (AbstractTextArea)word.getParentArea());
-
         super.renderWord(word);
     }
 
@@ -1065,10 +1095,13 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
         Font font = getFontFromArea(space.getParentArea());
         String s = space.getSpace();
 
-        AbstractTextArea textArea = (AbstractTextArea)space.getParentArea();
-        renderText(s, null, false, font, textArea);
+        Area parentArea = space.getParentArea();
+        assert (parentArea instanceof AbstractTextArea);
+        AbstractTextArea textArea = (AbstractTextArea) parentArea;
+        renderTextWithAdjustments(s, (int[]) null, false, font, textArea);
 
-        if (textUtil.combined && space.isAdjustable()) {
+        /* COMBINED is always false
+        if (textUtil.COMBINED && space.isAdjustable()) {
             //Used for justified text, for example
             int tws = textArea.getTextWordSpaceAdjust()
                          + 2 * textArea.getTextLetterSpaceAdjust();
@@ -1076,20 +1109,10 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
                 textUtil.adjust(tws);
             }
         }
+        */
         super.renderSpace(space);
     }
 
-    private void renderText(String s,
-                              int[][] dp, boolean reversed,
-                              Font font, AbstractTextArea parentArea) {
-        if ( ( dp == null ) || IFUtil.isDPOnlyDX ( dp ) ) {
-            int[] dx = IFUtil.convertDPToDX ( dp );
-            renderTextWithAdjustments ( s, dx, reversed, font, parentArea );
-        } else {
-            renderTextWithAdjustments ( s, dp, reversed, font, parentArea );
-        }
-    }
-
     /**
      * Does low-level rendering of text using DX only position adjustments.
      * @param s text to render
@@ -1109,10 +1132,12 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
             char ch = s.charAt(i);
             textUtil.addChar(ch);
             int glyphAdjust = 0;
-            if (textUtil.combined && font.hasChar(ch)) {
+            /* COMBINED is always false
+            if (textUtil.COMBINED && font.hasChar(ch)) {
                 int tls = (i < l - 1 ? parentArea.getTextLetterSpaceAdjust() : 0);
                 glyphAdjust += tls;
             }
+            */
             if (dx != null && i < l) {
                 glyphAdjust += dx[i];
             }
@@ -1132,65 +1157,64 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
     private void renderTextWithAdjustments(String s,
                               int[][] dp, boolean reversed,
                               Font font, AbstractTextArea parentArea) {
-        assert !textUtil.combined;
-        for ( int i = 0, n = s.length(); i < n; i++ ) {
-            textUtil.addChar ( s.charAt ( i ) );
-            if ( dp != null ) {
-                textUtil.adjust ( dp[i] );
+        // assert !textUtil.COMBINED;
+        for (int i = 0, n = s.length(); i < n; i++) {
+            textUtil.addChar(s.charAt(i));
+            if (dp != null) {
+                textUtil.adjust(dp[i]);
             }
         }
     }
 
     private class TextUtil {
         private static final int INITIAL_BUFFER_SIZE = 16;
-        private int[][] dp = new int[INITIAL_BUFFER_SIZE][4];
-        // private int lastDPPos = 0; // TBD - not yet used
+        private int[][] dp = new int[INITIAL_BUFFER_SIZE][];
         private final StringBuffer text = new StringBuffer();
         private int startx;
         private int starty;
         private int tls;
         private int tws;
-        private final boolean combined = false;
+        // private final static boolean COMBINED = false; // no longer referenced
 
         void addChar(char ch) {
             text.append(ch);
         }
 
         void adjust(int dx) {
-            adjust ( new int[] {
-                    dx,                         // xPlaAdjust
-                    0,                          // yPlaAdjust
-                    dx,                         // xAdvAdjust
-                    0                           // yAdvAdjust
-                } );
+            if (dx != 0) {
+                adjust(new int[] {
+                        dx,                         // xPlaAdjust
+                        0,                          // yPlaAdjust
+                        dx,                         // xAdvAdjust
+                        0                           // yAdvAdjust
+                    });
+            }
         }
 
         void adjust(int[] pa) {
-            if ( !IFUtil.isPAIdentity ( pa ) ) {
+            if (!IFUtil.isPAIdentity(pa)) {
                 int idx = text.length();
                 if (idx > dp.length - 1) {
                     int newSize = Math.max(dp.length, idx + 1) + INITIAL_BUFFER_SIZE;
                     int[][] newDP = new int[newSize][];
-                    // reuse prior PA[0]...PA[dp.length-1]
+                    // reuse prior DP[0]...DP[dp.length-1]
                     System.arraycopy(dp, 0, newDP, 0, dp.length);
-                    // populate new PA[dp.length]...PA[newDP.length-1]
-                    for ( int i = dp.length, n = newDP.length; i < n; i++ ) {
-                        newDP[i] = new int[4];
-                    }
+                    // switch to new DP, leaving DP[dp.length]...DP[newDP.length-1] unpopulated
                     dp = newDP;
                 }
-                IFUtil.adjustPA ( dp[idx - 1], pa );
-                // lastDPPos = idx;
+                if (dp[idx - 1] == null) {
+                    dp[idx - 1] = new int[4];
+                }
+                IFUtil.adjustPA(dp[idx - 1], pa);
             }
         }
 
         void reset() {
             if (text.length() > 0) {
                 text.setLength(0);
-                for ( int i = 0, n = dp.length; i < n; i++ ) {
-                    Arrays.fill(dp[i], 0);
+                for (int i = 0, n = dp.length; i < n; i++) {
+                    dp[i] = null;
                 }
-                // lastDPPos = 0;
             }
         }
 
@@ -1207,13 +1231,13 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
         void flush() {
             if (text.length() > 0) {
                 try {
-                    if (combined) {
+                    /* if (COMBINED) { // COMBINED is always false
                         painter.drawText(startx, starty, 0, 0,
-                                         trimAdjustments ( dp, text.length() ), text.toString());
-                    } else {
+                                         trimAdjustments(dp, text.length()), text.toString());
+                    } else { */
                         painter.drawText(startx, starty, tls, tws,
-                                         trimAdjustments ( dp, text.length() ), text.toString());
-                    }
+                                         trimAdjustments(dp, text.length()), text.toString());
+                    /* } */
                 } catch (IFException e) {
                     handleIFException(e);
                 }
@@ -1231,23 +1255,23 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
          * no greater than text length, and the last entry has a non-zero
          * adjustment.
          */
-        private int[][] trimAdjustments ( int[][] dp, int textLength ) {
-            if ( dp != null ) {
+        private int[][] trimAdjustments(int[][] dp, int textLength) {
+            if (dp != null) {
                 int tl = textLength;
                 int pl = dp.length;
-                int i  = ( tl < pl ) ? tl : pl;
-                while ( i > 0 ) {
+                int i  = (tl < pl) ? tl : pl;
+                while (i > 0) {
                     int[] pa = dp [ i - 1 ];
-                    if ( !IFUtil.isPAIdentity ( pa ) ) {
+                    if ((pa != null) && !IFUtil.isPAIdentity(pa)) {
                         break;
                     } else {
                         i--;
                     }
                 }
-                if ( i == 0 ) {
+                if (i == 0) {
                     dp = null;
-                } else if ( i < pl ) {
-                    dp = IFUtil.copyDP ( dp, 0, i );
+                } else if (i < pl) {
+                    dp = IFUtil.copyDP(dp, 0, i);
                 }
             }
             return dp;
@@ -1332,40 +1356,64 @@ public class IFRenderer extends AbstractPathOrientedRenderer {
             handleIFException(ife);
         }
     }
-
+    /** {@inheritDoc} */
+    protected void clipBackground(float startx, float starty,
+            float width, float height,
+            BorderProps bpsBefore, BorderProps bpsAfter,
+            BorderProps bpsStart, BorderProps bpsEnd) {
+        pushGroup(new IFGraphicContext.Group());
+        Rectangle rect = toMillipointRectangle(startx, starty, width, height);
+        try {
+            painter.clipBackground(rect,
+                 bpsBefore,  bpsAfter, bpsStart,  bpsEnd);
+        } catch (IFException ife) {
+            handleIFException(ife);
+        }
+    }
     /** {@inheritDoc} */
     protected void closePath() {
         throw new IllegalStateException("Not used");
     }
 
     /** {@inheritDoc} */
-    protected void drawBorders(                                  // CSOK: ParameterNumber
-            float startx, float starty,
+    protected void drawBackground(float startx, float starty,
+            float width, float height,
+            Trait.Background back,
+            BorderProps bpsBefore, BorderProps bpsAfter,
+            BorderProps bpsStart, BorderProps bpsEnd) {
+        if (painter.isBackgroundRequired(bpsBefore, bpsAfter, bpsStart, bpsEnd)) {
+            super.drawBackground(startx, starty, width, height,
+                     back,  bpsBefore,  bpsAfter,
+                     bpsStart,  bpsEnd);
+        }
+    }
+
+    /** {@inheritDoc} */
+    protected void drawBorders(float startx, float starty,
             float width, float height,
             BorderProps bpsBefore, BorderProps bpsAfter,
-            BorderProps bpsStart, BorderProps bpsEnd, int level) {
+            BorderProps bpsStart, BorderProps bpsEnd, int level, Color innerBackgroundColor) {
         Rectangle rect = toMillipointRectangle(startx, starty, width, height);
         try {
             BorderProps bpsTop = bpsBefore;
             BorderProps bpsBottom = bpsAfter;
             BorderProps bpsLeft;
             BorderProps bpsRight;
-            if ( ( level == -1 ) || ( ( level & 1 ) == 0 ) ) {
+            if ((level == -1) || ((level & 1) == 0)) {
                 bpsLeft = bpsStart;
                 bpsRight = bpsEnd;
             } else {
                 bpsLeft = bpsEnd;
                 bpsRight = bpsStart;
             }
-            painter.drawBorderRect(rect, bpsTop, bpsBottom, bpsLeft, bpsRight);
+            painter.drawBorderRect(rect, bpsTop, bpsBottom, bpsLeft, bpsRight, innerBackgroundColor);
         } catch (IFException ife) {
             handleIFException(ife);
         }
     }
 
     /** {@inheritDoc} */
-    protected void drawBorderLine(                               // CSOK: ParameterNumber
-            float x1, float y1, float x2, float y2, boolean horz,
+    protected void drawBorderLine(float x1, float y1, float x2, float y2, boolean horz,
             boolean startOrBefore, int style, Color col) {
         //Simplified implementation that is only used by renderTextDecoration()
         //drawBorders() is overridden and uses the Painter's high-level method drawBorderRect()
