@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Java2DRenderer.java 932497 2010-04-09 16:34:29Z vhennebert $ */
+/* $Id: Java2DRenderer.java 1616590 2014-08-07 20:27:59Z gadams $ */
 
 package org.apache.fop.render.java2d;
 
@@ -53,11 +53,12 @@ import org.apache.xmlgraphics.image.loader.impl.ImageGraphics2D;
 import org.apache.xmlgraphics.image.loader.impl.ImageRendered;
 import org.apache.xmlgraphics.image.loader.impl.ImageXMLDOM;
 import org.apache.xmlgraphics.image.loader.util.ImageUtil;
+import org.apache.xmlgraphics.util.UnitConv;
 
 import org.apache.fop.ResourceEventProducer;
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
-import org.apache.fop.apps.FopFactoryConfigurator;
+import org.apache.fop.apps.FopFactoryConfig;
 import org.apache.fop.area.CTM;
 import org.apache.fop.area.PageViewport;
 import org.apache.fop.area.Trait;
@@ -72,6 +73,7 @@ import org.apache.fop.fo.Constants;
 import org.apache.fop.fonts.Font;
 import org.apache.fop.fonts.FontCollection;
 import org.apache.fop.fonts.FontInfo;
+import org.apache.fop.fonts.FontManager;
 import org.apache.fop.fonts.Typeface;
 import org.apache.fop.render.AbstractPathOrientedRenderer;
 import org.apache.fop.render.Graphics2DAdapter;
@@ -82,6 +84,8 @@ import org.apache.fop.render.pdf.CTMHelper;
 import org.apache.fop.util.CharUtilities;
 import org.apache.fop.util.ColorUtil;
 
+import static org.apache.fop.render.java2d.Java2DRendererOption.JAVA2D_TRANSPARENT_PAGE_BACKGROUND;
+
 /**
  * The <code>Java2DRenderer</code> class provides the abstract technical
  * foundation for all rendering with the Java2D API. Renderers like
@@ -108,23 +112,20 @@ import org.apache.fop.util.ColorUtil;
  */
 public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implements Printable {
 
-    /** Rendering Options key for the controlling the transparent page background option. */
-    public static final String JAVA2D_TRANSPARENT_PAGE_BACKGROUND = "transparent-page-background";
-
     /** The scale factor for the image size, values: ]0 ; 1] */
     protected double scaleFactor = 1;
 
     /** The page width in pixels */
-    protected int pageWidth = 0;
+    protected int pageWidth;
 
     /** The page height in pixels */
-    protected int pageHeight = 0;
+    protected int pageHeight;
 
     /** List of Viewports */
     protected List pageViewportList = new java.util.ArrayList();
 
     /** The 0-based current page number */
-    private int currentPageNumber = 0;
+    private int currentPageNumber;
 
     /** true if anti-aliasing is set */
     protected boolean antialiasing = true;
@@ -133,7 +134,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
     protected boolean qualityRendering = true;
 
     /** false: paints a non-transparent white background, true: for a transparent background */
-    protected boolean transparentPageBackground = false;
+    protected boolean transparentPageBackground;
 
     /** The current state, holds a Graphics2D and its context */
     protected Java2DGraphicsState state;
@@ -143,18 +144,20 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
     /** true if the renderer has finished rendering all the pages */
     private boolean renderingDone;
 
-    private GeneralPath currentPath = null;
+    private GeneralPath currentPath;
 
-    /** Default constructor */
-    public Java2DRenderer() {
-    }
+    /**
+     * Default constructor
+     *
+     * @param userAgent the user agent that contains configuration details. This cannot be null.
+     */
+    public Java2DRenderer(FOUserAgent userAgent) {
+        super(userAgent);
 
-    /** {@inheritDoc} */
-    public void setUserAgent(FOUserAgent foUserAgent) {
-        super.setUserAgent(foUserAgent);
+        // MH: necessary? the caller has access to FOUserAgent
         userAgent.setRendererOverride(this); // for document regeneration
 
-        String s = (String)userAgent.getRendererOptions().get(JAVA2D_TRANSPARENT_PAGE_BACKGROUND);
+        String s = (String) userAgent.getRendererOption(JAVA2D_TRANSPARENT_PAGE_BACKGROUND);
         if (s != null) {
             this.transparentPageBackground = "true".equalsIgnoreCase(s);
         }
@@ -170,15 +173,17 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         //Don't call super.setupFontInfo() here! Java2D needs a special font setup
         // create a temp Image to test font metrics on
         this.fontInfo = inFontInfo;
-        Graphics2D graphics2D = Java2DFontMetrics.createFontMetricsGraphics2D();
+        final Java2DFontMetrics java2DFontMetrics = new Java2DFontMetrics();
+
+        FontManager fontManager = userAgent.getFontManager();
 
         FontCollection[] fontCollections = new FontCollection[] {
-                new Base14FontCollection(graphics2D),
-                new InstalledFontCollection(graphics2D),
-                new ConfiguredFontCollection(getFontResolver(), getFontList())
+                new Base14FontCollection(java2DFontMetrics),
+                new InstalledFontCollection(java2DFontMetrics),
+                new ConfiguredFontCollection(fontManager.getResourceResolver(), getFontList(),
+                        userAgent.isComplexScriptFeaturesEnabled())
         };
-        userAgent.getFactory().getFontManager().setup(
-                getFontInfo(), fontCollections);
+        fontManager.setup(getFontInfo(), fontCollections);
     }
 
     /** {@inheritDoc} */
@@ -211,9 +216,6 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         renderingDone = true;
         int numberOfPages = currentPageNumber;
         // TODO set all vars to null for gc
-        if (numberOfPages == 0) {
-            new FOPException("No page could be rendered");
-        }
     }
 
     /** @return true if the renderer is not currently processing */
@@ -263,10 +265,15 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
      * @param pageViewport the <code>PageViewport</code> object supplied by
      * the Area Tree
      * @throws IOException In case of an I/O error
+     * @throws FOPException if cloning of pageViewport is not supported
      * @see org.apache.fop.render.Renderer
      */
-    public void renderPage(PageViewport pageViewport) throws IOException {
-        rememberPage((PageViewport)pageViewport.clone());
+    public void renderPage(PageViewport pageViewport) throws IOException, FOPException {
+        try {
+            rememberPage((PageViewport)pageViewport.clone());
+        } catch (CloneNotSupportedException e) {
+            throw new FOPException(e);
+        }
         //The clone() call is necessary as we store the page for later. Otherwise, the
         //RenderPagesModel calls PageViewport.clear() to release memory as early as possible.
         currentPageNumber++;
@@ -308,7 +315,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
             // set scale factor
             double scaleX = scaleFactor;
             double scaleY = scaleFactor;
-            String scale = (String) currentPageViewport.getForeignAttributes().get(
+            String scale = currentPageViewport.getForeignAttributes().get(
                     PageScale.EXT_PAGE_SCALE);
             Point2D scales = PageScale.getScale(scale);
             if (scales != null) {
@@ -317,10 +324,10 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
             }
 
             scaleX = scaleX
-                * (25.4f / FopFactoryConfigurator.DEFAULT_TARGET_RESOLUTION)
+                    * (UnitConv.IN2MM / FopFactoryConfig.DEFAULT_TARGET_RESOLUTION)
                 / userAgent.getTargetPixelUnitToMillimeter();
             scaleY = scaleY
-                * (25.4f / FopFactoryConfigurator.DEFAULT_TARGET_RESOLUTION)
+                    * (UnitConv.IN2MM / FopFactoryConfig.DEFAULT_TARGET_RESOLUTION)
                 / userAgent.getTargetPixelUnitToMillimeter();
             int bitmapWidth = (int) ((pageWidth * scaleX) + 0.5);
             int bitmapHeight = (int) ((pageHeight * scaleY) + 0.5);
@@ -444,7 +451,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
     }
 
     /** {@inheritDoc} */
-    protected void startVParea(CTM ctm, Rectangle2D clippingRect) {
+    protected void startVParea(CTM ctm, Rectangle clippingRect) {
 
         saveGraphicsState();
 
@@ -465,6 +472,14 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         restoreGraphicsState();
     }
 
+    /** {@inheritDoc} */
+    protected void startLayer(String layer) {
+    }
+
+    /** {@inheritDoc} */
+    protected void endLayer() {
+    }
+
     /** {@inheritDoc} */
     protected List breakOutOfStateStack() {
         log.debug("Block.FIXED --> break out");
@@ -711,7 +726,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         renderInlineAreaBackAndBorders(text);
 
         int rx = currentIPPosition + text.getBorderAndPaddingWidthStart();
-        int bl = currentBPPosition + text.getOffset() + text.getBaselineOffset();
+        int bl = currentBPPosition + text.getBlockProgressionOffset() + text.getBaselineOffset();
         int saveIP = currentIPPosition;
 
         Font font = getFontFromArea(text);
@@ -727,7 +742,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         //super.renderText(text);
 
         // rendering text decorations
-        Typeface tf = (Typeface) fontInfo.getFonts().get(font.getFontName());
+        Typeface tf = fontInfo.getFonts().get(font.getFontName());
         int fontsize = text.getTraitAsInteger(Trait.FONT_SIZE);
         renderTextDecoration(tf, fontsize, text, bl, rx);
     }
@@ -823,7 +838,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         // TODO Colors do not work on Leaders yet
 
         float startx = (currentIPPosition + area.getBorderAndPaddingWidthStart()) / 1000f;
-        float starty = ((currentBPPosition + area.getOffset()) / 1000f);
+        float starty = ((currentBPPosition + area.getBlockProgressionOffset()) / 1000f);
         float endx = (currentIPPosition + area.getBorderAndPaddingWidthStart()
                 + area.getIPD()) / 1000f;
 
@@ -878,11 +893,13 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
             closePath();
             state.getGraph().fill(currentPath);
             currentPath = null;
+            break;
 
         case EN_NONE:
             // No rule is drawn
             break;
         default:
+            break;
         } // end switch
 
         super.renderLeader(area);
@@ -908,7 +925,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         int y = currentBPPosition + (int)Math.round(pos.getY());
         uri = URISpecification.getURL(uri);
 
-        ImageManager manager = getUserAgent().getFactory().getImageManager();
+        ImageManager manager = getUserAgent().getImageManager();
         ImageInfo info = null;
         try {
             ImageSessionContext sessionContext = getUserAgent().getImageSessionContext();
