@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Java2DSVGHandler.java 746664 2009-02-22 12:40:44Z jeremias $ */
+/* $Id: Java2DSVGHandler.java 1664570 2015-03-06 09:41:07Z lbernardo $ */
 
 package org.apache.fop.render.java2d;
 
@@ -25,12 +25,14 @@ import java.util.Map;
 
 import org.w3c.dom.Document;
 
-import org.apache.batik.bridge.BridgeContext;
-import org.apache.batik.bridge.GVTBuilder;
-import org.apache.batik.gvt.GraphicsNode;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.batik.bridge.BridgeContext;
+import org.apache.batik.bridge.DefaultFontFamilyResolver;
+import org.apache.batik.bridge.GVTBuilder;
+import org.apache.batik.gvt.GraphicsNode;
+
 import org.apache.fop.image.loader.batik.BatikUtil;
 import org.apache.fop.render.AbstractGenericSVGHandler;
 import org.apache.fop.render.ImageHandlerUtil;
@@ -90,6 +92,7 @@ public class Java2DSVGHandler extends AbstractGenericSVGHandler
         public int currentXPosition;
         /** see Java2D_YPOS */
         public int currentYPosition;
+        /** paint as bitmap */
         public boolean paintAsBitmap;
 
         /** {@inheritDoc} */
@@ -126,8 +129,8 @@ public class Java2DSVGHandler extends AbstractGenericSVGHandler
 
         int x = info.currentXPosition;
         int y = info.currentYPosition;
-
-        SVGUserAgent ua = new SVGUserAgent(context.getUserAgent(), new AffineTransform());
+        SVGUserAgent ua = new SVGUserAgent(context.getUserAgent(), DefaultFontFamilyResolver.SINGLETON,
+                new AffineTransform());
 
         BridgeContext ctx = new BridgeContext(ua);
 
