@@ -15,26 +15,18 @@
  * limitations under the License.
  */
 
-/* $Id: PCLGenerator.java 1067881 2011-02-07 08:39:46Z jeremias $ */
+/* $Id: PCLGenerator.java 1695082 2015-08-10 14:15:48Z rmeyer $ */
 
 package org.apache.fop.render.pcl;
 
 import java.awt.Color;
 import java.awt.Dimension;
-import java.awt.Graphics2D;
-import java.awt.geom.AffineTransform;
-import java.awt.image.BufferedImage;
-import java.awt.image.BufferedImageOp;
-import java.awt.image.ByteLookupTable;
-import java.awt.image.ColorModel;
 import java.awt.image.DataBuffer;
 import java.awt.image.DataBufferByte;
 import java.awt.image.IndexColorModel;
-import java.awt.image.LookupOp;
 import java.awt.image.MultiPixelPackedSampleModel;
 import java.awt.image.Raster;
 import java.awt.image.RenderedImage;
-import java.awt.image.WritableRaster;
 import java.io.DataOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
@@ -42,9 +34,9 @@ import java.text.DecimalFormat;
 import java.text.DecimalFormatSymbols;
 import java.util.Locale;
 
+import org.apache.commons.io.IOUtils;
 import org.apache.commons.io.output.ByteArrayOutputStream;
 
-import org.apache.xmlgraphics.image.GraphicsUtil;
 import org.apache.xmlgraphics.util.UnitConv;
 
 import org.apache.fop.util.bitmap.BitmapImageUtil;
@@ -81,7 +73,7 @@ public class PCLGenerator {
      * true: Standard PCL shades are used (poor quality). false: user-defined pattern are used
      * to create custom dither patterns for better grayscale quality.
      */
-    private final boolean usePCLShades = false;
+    private static final boolean USE_PCL_SHADES = false;
 
     /**
      * Main constructor.
@@ -148,6 +140,15 @@ public class PCLGenerator {
         out.write(s.getBytes(ISO_8859_1));
     }
 
+    /**
+     * Writes raw bytes to the output stream
+     * @param bytes The bytes
+     * @throws IOException In case of an I/O error
+     */
+    public void writeBytes(byte[] bytes) throws IOException {
+        out.write(bytes);
+    }
+
     /**
      * Formats a double value with two decimal positions for PCL output.
      *
@@ -389,7 +390,7 @@ public class PCLGenerator {
             //y += h;
         }
         setPatternTransparencyMode(false);
-        if (usePCLShades
+        if (USE_PCL_SHADES
                 || Color.black.equals(col)
                 || Color.white.equals(col)) {
             writeCommand("*c" + formatDouble4(w / 100.0) + "h"
@@ -447,6 +448,8 @@ public class PCLGenerator {
         writeCommand("*c" + patternID + "G");
         writeCommand("*c" + baout.size() + "W");
         baout.writeTo(this.out);
+        IOUtils.closeQuietly(data);
+        IOUtils.closeQuietly(baout);
         writeCommand("*c4Q"); //temporary pattern
     }
 
@@ -518,7 +521,7 @@ public class PCLGenerator {
         } else if (Color.white.equals(col)) {
             selectCurrentPattern(0, 1); //white
         } else {
-            if (usePCLShades) {
+            if (USE_PCL_SHADES) {
                 selectCurrentPattern(convertToPCLShade(col), 2);
             } else {
                 defineGrayscalePattern(col, 32, DitherUtil.DITHER_MATRIX_4X4);
@@ -644,6 +647,7 @@ public class PCLGenerator {
         }
     }
 
+    /* not used
     private RenderedImage getMask(RenderedImage img, Dimension targetDim) {
         ColorModel cm = img.getColorModel();
         if (cm.hasAlpha()) {
@@ -677,18 +681,12 @@ public class PCLGenerator {
             } finally {
                 g2d.dispose();
             }
-            /*
-            try {
-                BatchDiffer.saveAsPNG(alpha, new java.io.File("D:/out-alpha.png"));
-                BatchDiffer.saveAsPNG(mask, new java.io.File("D:/out-mask.png"));
-            } catch (IOException e) {
-                e.printStackTrace();
-            }*/
             return mask;
         } else {
             return null;
         }
     }
+    */
 
     /**
      * Paint a bitmap at the current cursor position. The bitmap is converted to a monochrome
@@ -715,11 +713,11 @@ public class PCLGenerator {
                     (int)Math.ceil(UnitConv.mpt2px(targetDim.height, effResolution)));
         }
         boolean scaled = !orgDim.equals(effDim);
-        //ImageWriterUtil.saveAsPNG(img, new java.io.File("D:/text-0-org.png"));
 
         boolean monochrome = isMonochromeImage(img);
         if (!monochrome) {
             //Transparency mask disabled. Doesn't work reliably
+            /*
             final boolean transparencyDisabled = true;
             RenderedImage mask = (transparencyDisabled ? null : getMask(img, effDim));
             if (mask != null) {
@@ -729,11 +727,12 @@ public class PCLGenerator {
                 paintMonochromeBitmap(mask, effResolution);
                 popCursorPos();
             }
+            */
 
             RenderedImage red = BitmapImageUtil.convertToMonochrome(
                     img, effDim, this.ditheringQuality);
             selectCurrentPattern(0, 0); //Solid black
-            setTransparencyMode(sourceTransparency || mask != null, true);
+            setTransparencyMode(sourceTransparency /*|| mask != null*/, true);
             paintMonochromeBitmap(red, effResolution);
         } else {
             RenderedImage effImg = img;
@@ -836,10 +835,10 @@ public class PCLGenerator {
         private byte[] rle; //compressed (RLE)
         private byte[] uncompressed; //uncompressed
         private int lastcount = -1;
-        private byte lastbyte = 0;
-        private int rlewidth = 0;
-        private byte ib = 0; //current image bits
-        private int x = 0;
+        private byte lastbyte;
+        private int rlewidth;
+        private byte ib; //current image bits
+        private int x;
         private boolean zeroRow = true;
 
         public Encoder(RenderedImage img) {
