@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PCLImageHandlerGraphics2D.java 1067881 2011-02-07 08:39:46Z jeremias $ */
+/* $Id: PCLImageHandlerGraphics2D.java 1356646 2012-07-03 09:46:41Z mehdi $ */
 
 package org.apache.fop.render.pcl;
 
@@ -127,7 +127,7 @@ public class PCLImageHandlerGraphics2D implements ImageHandler {
         if (!painted) {
             //Fallback solution: Paint to a BufferedImage
             FOUserAgent ua = context.getUserAgent();
-            ImageManager imageManager = ua.getFactory().getImageManager();
+            ImageManager imageManager = ua.getImageManager();
             ImageRendered imgRend;
             try {
                 imgRend = (ImageRendered)imageManager.convertImage(
