@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ImageRenderedAdapter.java 772943 2009-05-08 11:21:41Z vhennebert $ */
+/* $Id: ImageRenderedAdapter.java 1716758 2015-11-26 18:53:01Z matthias $ */
 
 package org.apache.fop.render.pdf;
 import java.awt.color.ColorSpace;
@@ -23,26 +23,25 @@ import java.awt.color.ICC_ColorSpace;
 import java.awt.color.ICC_Profile;
 import java.awt.image.ColorModel;
 import java.awt.image.IndexColorModel;
+import java.awt.image.Raster;
 import java.awt.image.RenderedImage;
 import java.io.IOException;
 import java.io.OutputStream;
 
-import org.apache.commons.io.output.ByteArrayOutputStream;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.xmlgraphics.image.GraphicsUtil;
 import org.apache.xmlgraphics.image.loader.impl.ImageRendered;
 import org.apache.xmlgraphics.ps.ImageEncodingHelper;
 
 import org.apache.fop.pdf.AlphaRasterImage;
-import org.apache.fop.pdf.PDFArray;
 import org.apache.fop.pdf.PDFColor;
 import org.apache.fop.pdf.PDFDeviceColorSpace;
 import org.apache.fop.pdf.PDFDictionary;
 import org.apache.fop.pdf.PDFDocument;
 import org.apache.fop.pdf.PDFFilter;
 import org.apache.fop.pdf.PDFFilterList;
-import org.apache.fop.pdf.PDFName;
 import org.apache.fop.pdf.PDFReference;
 
 /**
@@ -55,7 +54,7 @@ public class ImageRenderedAdapter extends AbstractImageAdapter {
 
     private ImageEncodingHelper encodingHelper;
 
-    private PDFFilter pdfFilter = null;
+    private PDFFilter pdfFilter;
     private String maskRef;
     private PDFReference softMask;
 
@@ -78,12 +77,14 @@ public class ImageRenderedAdapter extends AbstractImageAdapter {
     }
 
     /** {@inheritDoc} */
+    @Override
     public int getWidth() {
         RenderedImage ri = getImage().getRenderedImage();
         return ri.getWidth();
     }
 
     /** {@inheritDoc} */
+    @Override
     public int getHeight() {
         RenderedImage ri = getImage().getRenderedImage();
         return ri.getHeight();
@@ -94,11 +95,13 @@ public class ImageRenderedAdapter extends AbstractImageAdapter {
     }
 
     /** {@inheritDoc} */
+    @Override
     protected ColorSpace getImageColorSpace() {
         return getEffectiveColorModel().getColorSpace();
     }
 
     /** {@inheritDoc} */
+    @Override
     protected ICC_Profile getEffectiveICCProfile() {
         ColorSpace cs = getImageColorSpace();
         if (cs instanceof ICC_ColorSpace) {
@@ -110,6 +113,7 @@ public class ImageRenderedAdapter extends AbstractImageAdapter {
     }
 
     /** {@inheritDoc} */
+    @Override
     public void setup(PDFDocument doc) {
         RenderedImage ri = getImage().getRenderedImage();
 
@@ -122,8 +126,11 @@ public class ImageRenderedAdapter extends AbstractImageAdapter {
             //TODO Implement code to combine image with background color if transparency is not
             //allowed (need BufferedImage support for that)
 
-            AlphaRasterImage alphaImage = new AlphaRasterImage("Mask:" + getKey(), ri);
-            this.softMask = doc.addImage(null, alphaImage).makeReference();
+            Raster raster = GraphicsUtil.getAlphaRaster(ri);
+            if (raster != null) {
+                AlphaRasterImage alphaImage = new AlphaRasterImage("Mask:" + getKey(), raster);
+                this.softMask = doc.addImage(null, alphaImage).makeReference();
+            }
         }
     }
 
@@ -145,6 +152,7 @@ public class ImageRenderedAdapter extends AbstractImageAdapter {
     }
 
     /** {@inheritDoc} */
+    @Override
     public boolean isTransparent() {
         ColorModel cm = getEffectiveColorModel();
         if (cm instanceof IndexColorModel) {
@@ -155,31 +163,8 @@ public class ImageRenderedAdapter extends AbstractImageAdapter {
         return (getImage().getTransparentColor() != null);
     }
 
-    private static Integer getIndexOfFirstTransparentColorInPalette(RenderedImage image) {
-        ColorModel cm = image.getColorModel();
-        if (cm instanceof IndexColorModel) {
-            IndexColorModel icm = (IndexColorModel)cm;
-            //Identify the transparent color in the palette
-            byte[] alphas = new byte[icm.getMapSize()];
-            byte[] reds = new byte[icm.getMapSize()];
-            byte[] greens = new byte[icm.getMapSize()];
-            byte[] blues = new byte[icm.getMapSize()];
-            icm.getAlphas(alphas);
-            icm.getReds(reds);
-            icm.getGreens(greens);
-            icm.getBlues(blues);
-            for (int i = 0;
-                    i < ((IndexColorModel) cm).getMapSize();
-                    i++) {
-                if ((alphas[i] & 0xFF) == 0) {
-                    return new Integer(i);
-                }
-            }
-        }
-        return null;
-    }
-
     /** {@inheritDoc} */
+    @Override
     public PDFColor getTransparentColor() {
         ColorModel cm = getEffectiveColorModel();
         if (cm instanceof IndexColorModel) {
@@ -196,11 +181,13 @@ public class ImageRenderedAdapter extends AbstractImageAdapter {
     }
 
     /** {@inheritDoc} */
+    @Override
     public String getMask() {
         return maskRef;
     }
 
     /** {@inheritDoc} */
+    @Override
     public PDFReference getSoftMaskReference() {
         return softMask;
     }
@@ -212,55 +199,22 @@ public class ImageRenderedAdapter extends AbstractImageAdapter {
 
     /** {@inheritDoc} */
     public void outputContents(OutputStream out) throws IOException {
+        long start = System.currentTimeMillis();
+        encodingHelper.setBWInvert(true);
         encodingHelper.encode(out);
+        long duration = System.currentTimeMillis() - start;
+        if (log.isDebugEnabled()) {
+            log.debug("Image encoding took " + duration + "ms");
+        }
     }
 
-    private static final int MAX_HIVAL = 255;
-
     /** {@inheritDoc} */
+    @Override
     public void populateXObjectDictionary(PDFDictionary dict) {
         ColorModel cm = getEffectiveColorModel();
         if (cm instanceof IndexColorModel) {
-            IndexColorModel icm = (IndexColorModel)cm;
-            PDFArray indexed = new PDFArray(dict);
-            indexed.add(new PDFName("Indexed"));
-
-            if (icm.getColorSpace().getType() != ColorSpace.TYPE_RGB) {
-                log.warn("Indexed color space is not using RGB as base color space."
-                        + " The image may not be handled correctly."
-                        + " Base color space: " + icm.getColorSpace()
-                        + " Image: " + image.getInfo());
-            }
-            indexed.add(new PDFName(toPDFColorSpace(icm.getColorSpace()).getName()));
-            int c = icm.getMapSize();
-            int hival = c - 1;
-            if (hival > MAX_HIVAL) {
-                throw new UnsupportedOperationException("hival must not go beyond " + MAX_HIVAL);
-            }
-            indexed.add(new Integer(hival));
-            int[] palette = new int[c];
-            icm.getRGBs(palette);
-            ByteArrayOutputStream baout = new ByteArrayOutputStream();
-            for (int i = 0; i < c; i++) {
-                //TODO Probably doesn't work for non RGB based color spaces
-                //See log warning above
-                int entry = palette[i];
-                baout.write((entry & 0xFF0000) >> 16);
-                baout.write((entry & 0xFF00) >> 8);
-                baout.write(entry & 0xFF);
-            }
-            indexed.add(baout.toByteArray());
-
-            dict.put("ColorSpace", indexed);
-            dict.put("BitsPerComponent", icm.getPixelSize());
-
-            Integer index = getIndexOfFirstTransparentColorInPalette(getImage().getRenderedImage());
-            if (index != null) {
-                PDFArray mask = new PDFArray(dict);
-                mask.add(index);
-                mask.add(index);
-                dict.put("Mask", mask);
-            }
+            IndexColorModel icm = (IndexColorModel) cm;
+            super.populateXObjectDictionaryForIndexColorModel(dict, icm);
         }
     }
 
