@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFContentGenerator.java 1088231 2011-04-03 09:40:27Z adelmelle $ */
+/* $Id: PDFContentGenerator.java 1661887 2015-02-24 11:23:44Z ssteiner $ */
 
 package org.apache.fop.render.pdf;
 
@@ -28,10 +28,13 @@ import java.io.OutputStream;
 import org.apache.fop.pdf.PDFColorHandler;
 import org.apache.fop.pdf.PDFDocument;
 import org.apache.fop.pdf.PDFFilterList;
+import org.apache.fop.pdf.PDFLinearization;
 import org.apache.fop.pdf.PDFNumber;
 import org.apache.fop.pdf.PDFPaintingState;
+import org.apache.fop.pdf.PDFReference;
 import org.apache.fop.pdf.PDFResourceContext;
 import org.apache.fop.pdf.PDFStream;
+import org.apache.fop.pdf.PDFText;
 import org.apache.fop.pdf.PDFTextUtil;
 import org.apache.fop.pdf.PDFXObject;
 
@@ -54,12 +57,13 @@ public class PDFContentGenerator {
     private PDFColorHandler colorHandler;
 
     /** drawing state */
-    protected PDFPaintingState currentState = null;
+    protected PDFPaintingState currentState;
     /** Text generation utility holding the current font status */
     protected PDFTextUtil textutil;
 
     private boolean inMarkedContentSequence;
     private boolean inArtifactMode;
+    private AffineTransform transform;
 
     /**
      * Main constructor. Creates a new PDF stream and additional helper classes for text painting
@@ -79,12 +83,19 @@ public class PDFContentGenerator {
             protected void write(String code) {
                 currentStream.add(code);
             }
+            protected void write(StringBuffer code) {
+                currentStream.add(code);
+            }
         };
 
         this.currentState = new PDFPaintingState();
         this.colorHandler = new PDFColorHandler(document.getResources());
     }
 
+    public AffineTransform getAffineTransform() {
+        return transform;
+    }
+
     /**
      * Returns the applicable resource context for the generator.
      * @return the resource context
@@ -138,6 +149,9 @@ public class PDFContentGenerator {
      * @throws IOException if an error occurs while flushing the PDF objects
      */
     public void flushPDFDoc() throws IOException {
+        if (document.isLinearizationEnabled()) {
+            new PDFLinearization(document).outputPages(outputStream);
+        }
         this.document.output(this.outputStream);
     }
 
@@ -147,15 +161,23 @@ public class PDFContentGenerator {
      */
     protected void comment(String text) {
         if (WRITE_COMMENTS) {
-            currentStream.add("% " + text + "\n");
+            getStream().add("% " + text + "\n");
         }
     }
 
     /** Save graphics state. */
     protected void saveGraphicsState() {
         endTextObject();
-        currentState.save();
-        currentStream.add("q\n");
+        getState().save();
+        getStream().add("q\n");
+    }
+
+    /** Save graphics state with optional layer. */
+    protected void saveGraphicsState(String layer) {
+        endTextObject();
+        getState().save();
+        maybeBeginLayer(layer);
+        getStream().add("q\n");
     }
 
     /**
@@ -165,33 +187,52 @@ public class PDFContentGenerator {
      */
     protected void saveGraphicsState(String structElemType, int sequenceNum) {
         endTextObject();
-        currentState.save();
+        getState().save();
         beginMarkedContentSequence(structElemType, sequenceNum);
-        currentStream.add("q\n");
+        getStream().add("q\n");
     }
 
     /**
-     * Begins a new marked content sequence (BDC or BMC). If the parameter structElemType is null,
-     * the sequenceNum is ignored and instead of a BDC with the MCID as parameter, an "Artifact"
-     * and a BMC command is generated.
-     * @param structElemType Structure Element Type
-     * @param mcid    Sequence number
+     * Begins a new marked content sequence (BDC or BMC). If {@code structElemType} is
+     * null, a BMC operator with an "Artifact" tag is generated. Otherwise, a BDC operator
+     * with {@code structElemType} as a tag is generated, and the given mcid stored in its
+     * property list.
+     *
+     * @param structElemType the type of the associated structure element
+     * @param mcid the marked content identifier
      */
     protected void beginMarkedContentSequence(String structElemType, int mcid) {
+        beginMarkedContentSequence(structElemType, mcid, null);
+    }
+
+    /**
+     * Begins a new marked content sequence (BDC or BMC). If {@code structElemType} is
+     * null, a BMC operator with an "Artifact" tag is generated. Otherwise, a BDC operator
+     * with {@code structElemType} as a tag is generated, and the given mcid and actual
+     * text are stored in its property list.
+     *
+     * @param structElemType the type of the associated structure element
+     * @param mcid the marked content identifier
+     * @param actualText the replacement text for the marked content
+     */
+    protected void beginMarkedContentSequence(String structElemType, int mcid, String actualText) {
         assert !this.inMarkedContentSequence;
         assert !this.inArtifactMode;
         if (structElemType != null) {
-            currentStream.add(structElemType + " <</MCID " + String.valueOf(mcid) + ">>\n"
+            String actualTextProperty = actualText == null ? ""
+                    : " /ActualText " + PDFText.escapeText(actualText);
+            getStream().add(structElemType + " <</MCID " + String.valueOf(mcid)
+                    + actualTextProperty + ">>\n"
                     + "BDC\n");
         } else {
-            currentStream.add("/Artifact\nBMC\n");
+            getStream().add("/Artifact\nBMC\n");
             this.inArtifactMode = true;
         }
         this.inMarkedContentSequence = true;
     }
 
     void endMarkedContentSequence() {
-        currentStream.add("EMC\n");
+        getStream().add("EMC\n");
         this.inMarkedContentSequence = false;
         this.inArtifactMode = false;
     }
@@ -203,9 +244,10 @@ public class PDFContentGenerator {
      */
     protected void restoreGraphicsState(boolean popState) {
         endTextObject();
-        currentStream.add("Q\n");
+        getStream().add("Q\n");
+        maybeEndLayer();
         if (popState) {
-            currentState.restore();
+            getState().restore();
         }
     }
 
@@ -223,26 +265,42 @@ public class PDFContentGenerator {
      */
     protected void restoreGraphicsStateAccess() {
         endTextObject();
-        currentStream.add("Q\n");
+        getStream().add("Q\n");
         if (this.inMarkedContentSequence) {
             endMarkedContentSequence();
         }
-        currentState.restore();
+        getState().restore();
     }
 
-    /**
-     * Separates 2 text elements, ending the current marked content sequence and
-     * starting a new one.
-     *
-     * @param structElemType structure element type
-     * @param mcid sequence number
-     * @see #beginMarkedContentSequence(String, int)
-     */
-    protected void separateTextElements(String structElemType, int mcid) {
-        textutil.endTextObject();
-        endMarkedContentSequence();
-        beginMarkedContentSequence(structElemType, mcid);
-        textutil.beginTextObject();
+    private void maybeBeginLayer(String layer) {
+        if ((layer != null) && (layer.length() > 0)) {
+            getState().setLayer(layer);
+            beginOptionalContent(layer);
+        }
+    }
+
+    private void maybeEndLayer() {
+        if (getState().getLayerChanged()) {
+            endOptionalContent();
+        }
+    }
+
+    private int ocNameIndex;
+
+    private void beginOptionalContent(String layerId) {
+        String name;
+        PDFReference layer = document.resolveExtensionReference(layerId);
+        if (layer != null) {
+            name = "oc" + ++ocNameIndex;
+            document.getResources().addProperty(name, layer);
+        } else {
+            name = "unknown";
+        }
+        getStream().add("/OC /" + name + " BDC\n");
+    }
+
+    private void endOptionalContent() {
+        getStream().add("EMC\n");
     }
 
     /** Indicates the beginning of a text object. */
@@ -261,8 +319,21 @@ public class PDFContentGenerator {
      * @see #beginMarkedContentSequence(String, int)
      */
     protected void beginTextObject(String structElemType, int mcid) {
+        beginTextObject(structElemType, mcid, null);
+    }
+
+    /**
+     * Indicates the beginning of a marked-content text object.
+     *
+     * @param structElemType structure element type
+     * @param mcid sequence number
+     * @param actualText the replacement text for the marked content
+     * @see #beginTextObject()
+     * @see #beginMarkedContentSequence(String, int, String))
+     */
+    protected void beginTextObject(String structElemType, int mcid, String actualText) {
         if (!textutil.isInTextObject()) {
-            beginMarkedContentSequence(structElemType, mcid);
+            beginMarkedContentSequence(structElemType, mcid, actualText);
             textutil.beginTextObject();
         }
     }
@@ -282,9 +353,10 @@ public class PDFContentGenerator {
      * @param transform the transformation matrix (in points)
      */
     public void concatenate(AffineTransform transform) {
+        this.transform = transform;
         if (!transform.isIdentity()) {
-            currentState.concatenate(transform);
-            currentStream.add(CTMHelper.toPDFString(transform, false) + " cm\n");
+            getState().concatenate(transform);
+            getStream().add(CTMHelper.toPDFString(transform, false) + " cm\n");
         }
     }
 
@@ -306,7 +378,7 @@ public class PDFContentGenerator {
      * @param content the PDF content
      */
     public void add(String content) {
-        currentStream.add(content);
+        getStream().add(content);
     }
 
     /**
@@ -323,9 +395,9 @@ public class PDFContentGenerator {
      * @param width line width in points
      */
     public void updateLineWidth(float width) {
-        if (currentState.setLineWidth(width)) {
+        if (getState().setLineWidth(width)) {
             //Only write if value has changed WRT the current line width
-            currentStream.add(format(width) + " w\n");
+            getStream().add(format(width) + " w\n");
         }
     }
 
@@ -335,7 +407,7 @@ public class PDFContentGenerator {
      */
     public void updateCharacterSpacing(float value) {
         if (getState().setCharacterSpacing(value)) {
-            currentStream.add(format(value) + " Tc\n");
+            getStream().add(format(value) + " Tc\n");
         }
     }
 
@@ -373,7 +445,7 @@ public class PDFContentGenerator {
         if (pdf != null) {
             colorHandler.establishColor(pdf, col, fill);
         } else {
-            setColor(col, fill, this.currentStream);
+            setColor(col, fill, getStream());
         }
     }
 
@@ -418,6 +490,13 @@ public class PDFContentGenerator {
         restoreGraphicsState();
     }
 
+    public void placeImage(AffineTransform at, String stream) {
+        saveGraphicsState();
+        concatenate(at);
+        add(stream);
+        restoreGraphicsState();
+    }
+
     /**
      * Places a previously registered image at a certain place on the page,
      * bracketing it as a marked-content sequence.
