@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractPSTranscoder.java 1297284 2012-03-05 23:29:29Z gadams $ */
+/* $Id: AbstractPSTranscoder.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.render.ps;
 
@@ -40,6 +40,7 @@ import org.apache.fop.apps.FOPException;
 import org.apache.fop.fonts.FontInfo;
 import org.apache.fop.svg.AbstractFOPTranscoder;
 import org.apache.fop.svg.PDFDocumentGraphics2DConfigurator;
+import org.apache.fop.svg.font.FOPFontFamilyResolverImpl;
 
 /**
  * <p>This class enables to transcode an input to a PostScript document.</p>
@@ -70,7 +71,7 @@ import org.apache.fop.svg.PDFDocumentGraphics2DConfigurator;
 public abstract class AbstractPSTranscoder extends AbstractFOPTranscoder {
 
     /** the root Graphics2D instance for generating PostScript */
-    protected AbstractPSDocumentGraphics2D graphics = null;
+    protected AbstractPSDocumentGraphics2D graphics;
 
     private FontInfo fontInfo;
 
@@ -115,6 +116,8 @@ public abstract class AbstractPSTranscoder extends AbstractFOPTranscoder {
                 this.fontInfo = PDFDocumentGraphics2DConfigurator.createFontInfo(
                         getEffectiveConfiguration(), useComplexScriptFeatures);
                 graphics.setCustomTextHandler(new NativeTextHandler(graphics, fontInfo));
+                ((FOPTranscoderUserAgent) userAgent).setFontFamilyResolver(
+                        new FOPFontFamilyResolverImpl(fontInfo));
             } catch (FOPException fe) {
                 throw new TranscoderException(fe);
             }
@@ -144,11 +147,11 @@ public abstract class AbstractPSTranscoder extends AbstractFOPTranscoder {
             graphics.setViewportDimension(width, height);
 
             if (hints.containsKey(ImageTranscoder.KEY_BACKGROUND_COLOR)) {
-                graphics.setBackgroundColor
-                    ((Color)hints.get(ImageTranscoder.KEY_BACKGROUND_COLOR));
+                graphics.setBackgroundColor(
+                    (Color)hints.get(ImageTranscoder.KEY_BACKGROUND_COLOR));
         }
-            graphics.setGraphicContext
-                (new org.apache.xmlgraphics.java2d.GraphicContext());
+            graphics.setGraphicContext(
+                new org.apache.xmlgraphics.java2d.GraphicContext());
             graphics.setTransform(curTxf);
 
             this.root.paint(graphics);
