@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PSImageHandlerGraphics2D.java 1296526 2012-03-03 00:18:45Z gadams $ */
+/* $Id: PSImageHandlerGraphics2D.java 1658710 2015-02-10 13:21:17Z ssteiner $ */
 
 package org.apache.fop.render.ps;
 
@@ -30,6 +30,7 @@ import org.apache.xmlgraphics.image.loader.Image;
 import org.apache.xmlgraphics.image.loader.ImageFlavor;
 import org.apache.xmlgraphics.image.loader.ImageInfo;
 import org.apache.xmlgraphics.image.loader.impl.ImageGraphics2D;
+import org.apache.xmlgraphics.java2d.GeneralGraphics2DImagePainter;
 import org.apache.xmlgraphics.java2d.Graphics2DImagePainter;
 import org.apache.xmlgraphics.java2d.ps.PSGraphics2D;
 import org.apache.xmlgraphics.ps.FormGenerator;
@@ -65,8 +66,8 @@ public class PSImageHandlerGraphics2D implements PSImageHandler {
         float imw = (float)dim.getWidth() / 1000f;
         float imh = (float)dim.getHeight() / 1000f;
 
-        float sx = fwidth / (float)imw;
-        float sy = fheight / (float)imh;
+        float sx = fwidth / imw;
+        float sy = fheight / imh;
 
         gen.commentln("%FOPBeginGraphics2D");
         gen.saveGraphicsState();
@@ -84,7 +85,9 @@ public class PSImageHandlerGraphics2D implements PSImageHandler {
         gen.concatMatrix(sx, 0, 0, sy, fx, fy);
 
         final boolean textAsShapes = false;
-        PSGraphics2D graphics = new PSGraphics2D(textAsShapes, gen);
+        PSGraphics2D graphics = (painter instanceof GeneralGraphics2DImagePainter)
+                ? (PSGraphics2D) ((GeneralGraphics2DImagePainter) painter).getGraphics(textAsShapes, gen)
+                : new PSGraphics2D(textAsShapes, gen);
         graphics.setGraphicContext(new org.apache.xmlgraphics.java2d.GraphicContext());
         AffineTransform transform = new AffineTransform();
         // scale to viewbox
