@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PSPainter.java 1357883 2012-07-05 20:29:53Z gadams $ */
+/* $Id: PSPainter.java 1693613 2015-07-31 15:04:31Z ssteiner $ */
 
 package org.apache.fop.render.ps;
 
@@ -40,8 +40,8 @@ import org.apache.xmlgraphics.image.loader.ImageSessionContext;
 import org.apache.xmlgraphics.ps.PSGenerator;
 import org.apache.xmlgraphics.ps.PSResource;
 
+import org.apache.fop.fonts.EmbeddingMode;
 import org.apache.fop.fonts.Font;
-import org.apache.fop.fonts.FontInfo;
 import org.apache.fop.fonts.FontTriplet;
 import org.apache.fop.fonts.LazyFont;
 import org.apache.fop.fonts.MultiByteFont;
@@ -49,10 +49,10 @@ import org.apache.fop.fonts.SingleByteFont;
 import org.apache.fop.fonts.Typeface;
 import org.apache.fop.render.RenderingContext;
 import org.apache.fop.render.intermediate.AbstractIFPainter;
-import org.apache.fop.render.intermediate.IFContext;
+import org.apache.fop.render.intermediate.BorderPainter;
+import org.apache.fop.render.intermediate.GraphicsPainter;
 import org.apache.fop.render.intermediate.IFException;
 import org.apache.fop.render.intermediate.IFState;
-import org.apache.fop.render.intermediate.IFUtil;
 import org.apache.fop.traits.BorderProps;
 import org.apache.fop.traits.RuleStyle;
 import org.apache.fop.util.CharUtilities;
@@ -61,15 +61,16 @@ import org.apache.fop.util.HexEncoder;
 /**
  * IFPainter implementation that produces PostScript.
  */
-public class PSPainter extends AbstractIFPainter {
+public class PSPainter extends AbstractIFPainter<PSDocumentHandler> {
 
     /** logging instance */
     private static Log log = LogFactory.getLog(PSPainter.class);
 
-    private PSDocumentHandler documentHandler;
-    private PSBorderPainter borderPainter;
+    private final GraphicsPainter graphicsPainter;
 
-    private boolean inTextMode = false;
+    private BorderPainter borderPainter;
+
+    private boolean inTextMode;
 
     /**
      * Default constructor.
@@ -80,27 +81,14 @@ public class PSPainter extends AbstractIFPainter {
     }
 
     protected PSPainter(PSDocumentHandler documentHandler, IFState state) {
-        super();
-        this.documentHandler = documentHandler;
-        this.borderPainter = new PSBorderPainter(documentHandler.gen);
+        super(documentHandler);
+        this.graphicsPainter = new PSGraphicsPainter(getGenerator());
+        this.borderPainter = new BorderPainter(graphicsPainter);
         this.state = state;
     }
 
-    /** {@inheritDoc} */
-    protected IFContext getContext() {
-        return this.documentHandler.getContext();
-    }
-
-    PSRenderingUtil getPSUtil() {
-        return this.documentHandler.psUtil;
-    }
-
-    FontInfo getFontInfo() {
-        return this.documentHandler.getFontInfo();
-    }
-
     private PSGenerator getGenerator() {
-        return this.documentHandler.gen;
+        return getDocumentHandler().getGenerator();
     }
 
     /** {@inheritDoc} */
@@ -128,7 +116,7 @@ public class PSPainter extends AbstractIFPainter {
     }
 
     /** {@inheritDoc} */
-    public void startGroup(AffineTransform transform) throws IFException {
+    public void startGroup(AffineTransform transform, String layer) throws IFException {
         try {
             PSGenerator generator = getGenerator();
             saveGraphicsState();
@@ -168,7 +156,7 @@ public class PSPainter extends AbstractIFPainter {
     /** {@inheritDoc} */
     protected void drawImageUsingImageHandler(ImageInfo info, Rectangle rect)
             throws ImageException, IOException {
-        if (!getPSUtil().isOptimizeResources()
+        if (!getDocumentHandler().getPSUtil().isOptimizeResources()
                 || PSImageUtils.isImageInlined(info,
                         (PSRenderingContext)createRenderingContext())) {
             super.drawImageUsingImageHandler(info, rect);
@@ -177,7 +165,7 @@ public class PSPainter extends AbstractIFPainter {
                 log.debug("Image " + info + " is embedded as a form later");
             }
             //Don't load image at this time, just put a form placeholder in the stream
-            PSResource form = documentHandler.getFormForImage(info.getOriginalURI());
+            PSResource form = getDocumentHandler().getFormForImage(info.getOriginalURI());
             PSImageUtils.drawForm(form, info, rect, getGenerator());
         }
     }
@@ -215,6 +203,20 @@ public class PSPainter extends AbstractIFPainter {
         }
     }
 
+    /** {@inheritDoc} */
+    public void clipBackground(Rectangle rect,
+            BorderProps bpsBefore, BorderProps bpsAfter,
+            BorderProps bpsStart, BorderProps bpsEnd) throws IFException {
+
+        try {
+            borderPainter.clipBackground(rect,
+                    bpsBefore,  bpsAfter, bpsStart,  bpsEnd);
+        } catch (IOException ioe) {
+            throw new IFException("I/O error while clipping background", ioe);
+        }
+
+    }
+
     /** {@inheritDoc} */
     public void fillRect(Rectangle rect, Paint fill) throws IFException {
         if (fill == null) {
@@ -242,15 +244,15 @@ public class PSPainter extends AbstractIFPainter {
 
     /** {@inheritDoc} */
     public void drawBorderRect(Rectangle rect, BorderProps top, BorderProps bottom,
-            BorderProps left, BorderProps right) throws IFException {
+            BorderProps left, BorderProps right, Color innerBackgroundColor) throws IFException {
         if (top != null || bottom != null || left != null || right != null) {
             try {
                 endTextObject();
-                if (getPSUtil().getRenderingMode() == PSRenderingMode.SIZE
+                if (getDocumentHandler().getPSUtil().getRenderingMode() == PSRenderingMode.SIZE
                     && hasOnlySolidBorders(top, bottom, left, right)) {
-                    super.drawBorderRect(rect, top, bottom, left, right);
+                    super.drawBorderRect(rect, top, bottom, left, right, innerBackgroundColor);
                 } else {
-                    this.borderPainter.drawBorders(rect, top, bottom, left, right);
+                    this.borderPainter.drawBorders(rect, top, bottom, left, right, innerBackgroundColor);
                 }
             } catch (IOException ioe) {
                 throw new IFException("I/O error in drawBorderRect()", ioe);
@@ -263,7 +265,7 @@ public class PSPainter extends AbstractIFPainter {
                 throws IFException {
         try {
             endTextObject();
-            this.borderPainter.drawLine(start, end, width, color, style);
+            this.graphicsPainter.drawLine(start, end, width, color, style);
         } catch (IOException ioe) {
             throw new IFException("I/O error in drawLine()", ioe);
         }
@@ -273,7 +275,7 @@ public class PSPainter extends AbstractIFPainter {
         if (fontName == null) {
             throw new NullPointerException("fontName must not be null");
         }
-        Typeface tf = (Typeface)getFontInfo().getFonts().get(fontName);
+        Typeface tf = getFontInfo().getFonts().get(fontName);
         if (tf instanceof LazyFont) {
             tf = ((LazyFont)tf).getRealFont();
         }
@@ -363,10 +365,7 @@ public class PSPainter extends AbstractIFPainter {
                     state.getFontFamily(), state.getFontStyle(), state.getFontWeight());
             //TODO Ignored: state.getFontVariant()
             //TODO Opportunity for font caching if font state is more heavily used
-            String fontKey = getFontInfo().getInternalFontKey(triplet);
-            if (fontKey == null) {
-                throw new IFException("Font not available: " + triplet, null);
-            }
+            String fontKey = getFontKey(triplet);
             int sizeMillipoints = state.getFontSize();
 
             // This assumes that *all* CIDFonts use a /ToUnicode mapping
@@ -377,8 +376,14 @@ public class PSPainter extends AbstractIFPainter {
             }
             Font font = getFontInfo().getFontInstance(triplet, sizeMillipoints);
 
-            useFont(fontKey, sizeMillipoints);
+            PSFontResource res = getDocumentHandler().getPSResourceForFontKey(fontKey);
+            boolean otf = tf instanceof MultiByteFont && ((MultiByteFont)tf).isOTFFile();
+            useFont(fontKey, sizeMillipoints, otf);
 
+            if (dp != null && dp[0] != null) {
+                x += dp[0][0];
+                y -= dp[0][1];
+            }
             generator.writeln("1 0 0 -1 " + formatMptAsPt(generator, x)
                     + " " + formatMptAsPt(generator, y) + " Tm");
 
@@ -397,16 +402,40 @@ public class PSPainter extends AbstractIFPainter {
                                     letterSpacing, wordSpacing, dp, font, tf, false);
                         }
                         if (encoding == 0) {
-                            useFont(fontKey, sizeMillipoints);
+                            useFont(fontKey, sizeMillipoints, false);
                         } else {
-                            useFont(fontKey + "_" + Integer.toString(encoding), sizeMillipoints);
+                            useFont(fontKey + "_" + Integer.toString(encoding), sizeMillipoints, false);
                         }
                         currentEncoding = encoding;
                         start = i;
                     }
                 }
             } else {
-                useFont(fontKey, sizeMillipoints);
+                if (tf instanceof MultiByteFont && ((MultiByteFont)tf).isOTFFile()) {
+                    //Analyze string and split up in order to paint in different sub-fonts/encodings
+                    int curEncoding = 0;
+                    for (int i = start; i < textLen; i++) {
+                        char orgChar = text.charAt(i);
+
+                        MultiByteFont mbFont = (MultiByteFont)tf;
+                        mbFont.mapChar(orgChar);
+                        int origGlyphIdx = mbFont.findGlyphIndex(orgChar);
+                        int newGlyphIdx = mbFont.getUsedGlyphs().get(origGlyphIdx);
+                        int encoding = newGlyphIdx / 256;
+                        if (encoding != curEncoding) {
+                            if (i != 0) {
+                                writeText(text, start, i - start, letterSpacing, wordSpacing, dp, font, tf,
+                                        true);
+                                start = i;
+                            }
+                            generator.writeln("/" + res.getName() + "." + encoding + " "
+                                    + generator.formatDouble(sizeMillipoints / 1000f) + " F");
+                            curEncoding = encoding;
+                        }
+                    }
+                } else {
+                    useFont(fontKey, sizeMillipoints, false);
+                }
             }
             writeText(text, start, textLen - start, letterSpacing, wordSpacing, dp, font, tf,
                     tf instanceof MultiByteFont);
@@ -429,36 +458,49 @@ public class PSPainter extends AbstractIFPainter {
         int lineStart = 0;
         StringBuffer accText = new StringBuffer(initialSize);
         StringBuffer sb = new StringBuffer(initialSize);
-        int[] dx = IFUtil.convertDPToDX ( dp );
-        int dxl = (dx != null ? dx.length : 0);
+        boolean isOTF = multiByte && ((MultiByteFont)tf).isOTFFile();
         for (int i = start; i < end; i++) {
             char orgChar = text.charAt(i);
             char ch;
             int cw;
-            int glyphAdjust = 0;
+            int xGlyphAdjust = 0;
+            int yGlyphAdjust = 0;
+
             if (CharUtilities.isFixedWidthSpace(orgChar)) {
                 //Fixed width space are rendered as spaces so copy/paste works in a reader
                 ch = font.mapChar(CharUtilities.SPACE);
                 cw = font.getCharWidth(orgChar);
-                glyphAdjust = font.getCharWidth(ch) - cw;
+                xGlyphAdjust = font.getCharWidth(ch) - cw;
             } else {
                 if ((wordSpacing != 0) && CharUtilities.isAdjustableSpace(orgChar)) {
-                    glyphAdjust -= wordSpacing;
+                    xGlyphAdjust -= wordSpacing;
                 }
                 ch = font.mapChar(orgChar);
-                cw = font.getCharWidth(orgChar);
+                cw = font.getCharWidth(orgChar); // this is never used?
             }
 
-            if (dx != null && i < dxl - 1) {
-                glyphAdjust -= dx[i + 1];
+            if (dp != null && i < dp.length && dp[i] != null) {
+                // get x advancement adjust
+                xGlyphAdjust -= dp[i][2] - dp[i][0];
+                yGlyphAdjust += dp[i][3] - dp[i][1];
             }
-            if (multiByte) {
-                accText.append(HexEncoder.encode(ch));
-            } else {
+            if (dp != null && i < dp.length - 1 && dp[i + 1] != null) {
+                // get x placement adjust for next glyph
+                xGlyphAdjust -= dp[i + 1][0];
+                yGlyphAdjust += dp[i + 1][1];
+            }
+            if (!multiByte || isOTF) {
                 char codepoint = (char)(ch % 256);
-                PSGenerator.escapeChar(codepoint, accText); //add character to accumulated text
+                if (isOTF) {
+                    codepoint -= (((MultiByteFont)tf).getEmbeddingMode() == EmbeddingMode.FULL) ? 0 : 1;
+                    accText.append(HexEncoder.encode(codepoint, 2));
+                } else {
+                    PSGenerator.escapeChar(codepoint, accText); //add character to accumulated text
+                }
+            } else {
+                accText.append(HexEncoder.encode(ch));
             }
-            if (glyphAdjust != 0) {
+            if (xGlyphAdjust != 0 || yGlyphAdjust != 0) {
                 needTJ = true;
                 if (sb.length() == 0) {
                     sb.append('['); //Need to start TJ
@@ -472,7 +514,13 @@ public class PSPainter extends AbstractIFPainter {
                     sb.append(' ');
                     accText.setLength(0); //reset accumulated text
                 }
-                sb.append(Integer.toString(glyphAdjust)).append(' ');
+                if (yGlyphAdjust == 0) {
+                    sb.append(Integer.toString(xGlyphAdjust)).append(' ');
+                } else {
+                    sb.append('[');
+                    sb.append(Integer.toString(yGlyphAdjust)).append(' ');
+                    sb.append(Integer.toString(xGlyphAdjust)).append(']').append(' ');
+                }
             }
         }
         if (needTJ) {
@@ -528,11 +576,16 @@ public class PSPainter extends AbstractIFPainter {
         return lineStart;
     }
 
-    private void useFont(String key, int size) throws IOException {
-        PSFontResource res = this.documentHandler.getPSResourceForFontKey(key);
+    private void useFont(String key, int size, boolean otf) throws IOException {
+        PSFontResource res = getDocumentHandler().getPSResourceForFontKey(key);
         PSGenerator generator = getGenerator();
-        generator.useFont("/" + res.getName(), size / 1000f);
+        if (otf) {
+            String name = "/" + res.getName() + ".0";
+            generator.getCurrentState().useFont(name, size);
+            generator.writeln(name + ' ' + generator.formatDouble(size / 1000f) + " F");
+        } else {
+            generator.useFont("/" + res.getName(), size / 1000f);
+        }
         res.notifyResourceUsageOnPage(generator.getResourceTracker());
     }
-
 }
