@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PSRenderingUtil.java 1095878 2011-04-22 07:15:43Z jeremias $ */
+/* $Id: PSRenderingUtil.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.render.ps;
 
@@ -33,16 +33,20 @@ import org.apache.fop.render.ps.extensions.PSCommentBefore;
 import org.apache.fop.render.ps.extensions.PSExtensionAttachment;
 import org.apache.fop.render.ps.extensions.PSSetupCode;
 
+import static org.apache.fop.render.ps.PSRendererOption.AUTO_ROTATE_LANDSCAPE;
+import static org.apache.fop.render.ps.PSRendererOption.LANGUAGE_LEVEL;
+import static org.apache.fop.render.ps.PSRendererOption.OPTIMIZE_RESOURCES;
+
 /**
  * Utility class which enables all sorts of features that are not directly connected to the
  * normal rendering process.
  */
-public class PSRenderingUtil implements PSConfigurationConstants {
+public class PSRenderingUtil {
 
     private FOUserAgent userAgent;
 
     /** Whether or not the safe set page device macro will be used or not */
-    private boolean safeSetPageDevice = false;
+    private boolean safeSetPageDevice;
 
     /**
      * Whether or not PostScript Document Structuring Conventions (DSC) compliant output are
@@ -50,11 +54,11 @@ public class PSRenderingUtil implements PSConfigurationConstants {
      */
     private boolean dscCompliant = true;
 
-    private boolean autoRotateLandscape = false;
+    private boolean autoRotateLandscape;
     private int languageLevel = PSGenerator.DEFAULT_LANGUAGE_LEVEL;
 
     /** Determines whether the PS file is generated in two passes to minimize file size */
-    private boolean optimizeResources = false;
+    private boolean optimizeResources;
 
     /**
      * Determines whether the generated PostScript code is optimized for minimum file size
@@ -64,20 +68,21 @@ public class PSRenderingUtil implements PSConfigurationConstants {
 
     PSRenderingUtil(FOUserAgent userAgent) {
         this.userAgent = userAgent;
+        //PSRendererConfig confi = userAgent.getRendererConfig(rendererConfiguration, configCreator)
         initialize();
     }
 
     private void initialize() {
         Object obj;
-        obj = userAgent.getRendererOptions().get(AUTO_ROTATE_LANDSCAPE);
+        obj = userAgent.getRendererOptions().get(AUTO_ROTATE_LANDSCAPE.getName());
         if (obj != null) {
             setAutoRotateLandscape(booleanValueOf(obj));
         }
-        obj = userAgent.getRendererOptions().get(LANGUAGE_LEVEL);
+        obj = userAgent.getRendererOptions().get(LANGUAGE_LEVEL.getName());
         if (obj != null) {
             setLanguageLevel(intValueOf(obj));
         }
-        obj = userAgent.getRendererOptions().get(OPTIMIZE_RESOURCES);
+        obj = userAgent.getRendererOptions().get(OPTIMIZE_RESOURCES.getName());
         if (obj != null) {
             setOptimizeResources(booleanValueOf(obj));
         }
