@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PSSVGHandler.java 766594 2009-04-20 06:50:59Z jeremias $ */
+/* $Id: PSSVGHandler.java 1616590 2014-08-07 20:27:59Z gadams $ */
 
 package org.apache.fop.render.ps;
 
@@ -27,11 +27,10 @@ import java.util.Map;
 import org.w3c.dom.Document;
 
 import org.apache.avalon.framework.configuration.Configuration;
+
 import org.apache.batik.bridge.BridgeContext;
 import org.apache.batik.bridge.GVTBuilder;
 import org.apache.batik.gvt.GraphicsNode;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 
 import org.apache.xmlgraphics.java2d.ps.PSGraphics2D;
 import org.apache.xmlgraphics.ps.PSGenerator;
@@ -52,14 +51,11 @@ import org.apache.fop.svg.SVGUserAgent;
  * It renders SVG to the PostScript document using the PSGraphics2D.
  * The properties from the PostScript renderer are subject to change.
  *
- * @version $Id: PSSVGHandler.java 766594 2009-04-20 06:50:59Z jeremias $
+ * @version $Id: PSSVGHandler.java 1616590 2014-08-07 20:27:59Z gadams $
  */
 public class PSSVGHandler extends AbstractGenericSVGHandler
             implements PSRendererContextConstants {
 
-    /** logging instance */
-    private static Log log = LogFactory.getLog(PSSVGHandler.class);
-
     /**
      * Create a new PostScript XML handler for use by the PostScript renderer.
      */
@@ -223,18 +219,16 @@ public class PSSVGHandler extends AbstractGenericSVGHandler
      * @param context the renderer context
      * @param doc the svg document
      */
-    protected void renderSVGDocument(RendererContext context,
-            Document doc) {
+    protected void renderSVGDocument(RendererContext context, Document doc) {
+        assert context != null;
         PSInfo psInfo = getPSInfo(context);
         int xOffset = psInfo.currentXPosition;
         int yOffset = psInfo.currentYPosition;
         PSGenerator gen = psInfo.psGenerator;
 
         boolean paintAsBitmap = false;
-        if (context != null) {
-            Map foreign = (Map)context.getProperty(RendererContextConstants.FOREIGN_ATTRIBUTES);
-            paintAsBitmap = ImageHandlerUtil.isConversionModeBitmap(foreign);
-        }
+        Map foreign = (Map)context.getProperty(RendererContextConstants.FOREIGN_ATTRIBUTES);
+        paintAsBitmap = ImageHandlerUtil.isConversionModeBitmap(foreign);
         if (paintAsBitmap) {
             try {
                 super.renderSVGDocument(context, doc);
@@ -253,15 +247,14 @@ public class PSSVGHandler extends AbstractGenericSVGHandler
             strokeText = cfg.getChild("stroke-text", true).getValueAsBoolean(strokeText);
         }
 
-        SVGUserAgent ua
-             = new SVGUserAgent(context.getUserAgent(), new AffineTransform());
+        SVGUserAgent ua = new SVGUserAgent(context.getUserAgent(), null /* TODO */, new AffineTransform());
 
         PSGraphics2D graphics = new PSGraphics2D(strokeText, gen);
         graphics.setGraphicContext(new org.apache.xmlgraphics.java2d.GraphicContext());
 
         BridgeContext ctx = new PSBridgeContext(ua,
                 (strokeText ? null : psInfo.fontInfo),
-                context.getUserAgent().getFactory().getImageManager(),
+                context.getUserAgent().getImageManager(),
                 context.getUserAgent().getImageSessionContext());
 
         //Cloning SVG DOM as Batik attaches non-thread-safe facilities (like the CSS engine)
@@ -285,8 +278,6 @@ public class PSSVGHandler extends AbstractGenericSVGHandler
         float sx = psInfo.getWidth() / w;
         float sy = psInfo.getHeight() / h;
 
-        ctx = null;
-
         try {
             gen.commentln("%FOPBeginSVG");
             gen.saveGraphicsState();
@@ -338,7 +329,7 @@ public class PSSVGHandler extends AbstractGenericSVGHandler
 
     /** {@inheritDoc} */
     public boolean supportsRenderer(Renderer renderer) {
-        return (renderer instanceof PSRenderer);
+        return false;
     }
 
 }
