@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,12 +15,10 @@
  * limitations under the License.
  */
 
-/* $Id: ResourceHandler.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: ResourceHandler.java 1617052 2014-08-10 06:55:01Z gadams $ */
 
 package org.apache.fop.render.ps;
 
-import java.awt.geom.Dimension2D;
-import java.awt.image.RenderedImage;
 import java.awt.geom.Rectangle2D;
 import java.io.IOException;
 import java.io.InputStream;
@@ -29,27 +27,21 @@ import java.util.Iterator;
 import java.util.Map;
 import java.util.Set;
 
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+
 import org.apache.xmlgraphics.image.loader.ImageException;
 import org.apache.xmlgraphics.image.loader.ImageFlavor;
 import org.apache.xmlgraphics.image.loader.ImageInfo;
 import org.apache.xmlgraphics.image.loader.ImageManager;
 import org.apache.xmlgraphics.image.loader.ImageSessionContext;
-import org.apache.xmlgraphics.image.loader.impl.ImageGraphics2D;
-import org.apache.xmlgraphics.image.loader.impl.ImageRawCCITTFax;
-import org.apache.xmlgraphics.image.loader.impl.ImageRawEPS;
-import org.apache.xmlgraphics.image.loader.impl.ImageRawJPEG;
-import org.apache.xmlgraphics.image.loader.impl.ImageRawStream;
-import org.apache.xmlgraphics.image.loader.impl.ImageRendered;
-import org.apache.xmlgraphics.image.loader.impl.ImageXMLDOM;
 import org.apache.xmlgraphics.image.loader.util.ImageUtil;
 import org.apache.xmlgraphics.ps.DSCConstants;
-import org.apache.xmlgraphics.ps.FormGenerator;
-import org.apache.xmlgraphics.ps.ImageEncoder;
-import org.apache.xmlgraphics.ps.ImageFormGenerator;
 import org.apache.xmlgraphics.ps.PSGenerator;
-import org.apache.xmlgraphics.ps.PSProcSets;
+import org.apache.xmlgraphics.ps.PSResource;
 import org.apache.xmlgraphics.ps.dsc.DSCException;
 import org.apache.xmlgraphics.ps.dsc.DSCFilter;
+import org.apache.xmlgraphics.ps.dsc.DSCListener;
 import org.apache.xmlgraphics.ps.dsc.DSCParser;
 import org.apache.xmlgraphics.ps.dsc.DSCParserConstants;
 import org.apache.xmlgraphics.ps.dsc.DefaultNestedDocumentHandler;
@@ -59,16 +51,21 @@ import org.apache.xmlgraphics.ps.dsc.events.DSCCommentBoundingBox;
 import org.apache.xmlgraphics.ps.dsc.events.DSCCommentDocumentNeededResources;
 import org.apache.xmlgraphics.ps.dsc.events.DSCCommentDocumentSuppliedResources;
 import org.apache.xmlgraphics.ps.dsc.events.DSCCommentHiResBoundingBox;
+import org.apache.xmlgraphics.ps.dsc.events.DSCCommentIncludeResource;
 import org.apache.xmlgraphics.ps.dsc.events.DSCCommentLanguageLevel;
 import org.apache.xmlgraphics.ps.dsc.events.DSCCommentPage;
 import org.apache.xmlgraphics.ps.dsc.events.DSCCommentPages;
 import org.apache.xmlgraphics.ps.dsc.events.DSCEvent;
 import org.apache.xmlgraphics.ps.dsc.events.DSCHeaderComment;
 import org.apache.xmlgraphics.ps.dsc.events.PostScriptComment;
+import org.apache.xmlgraphics.ps.dsc.events.PostScriptLine;
 import org.apache.xmlgraphics.ps.dsc.tools.DSCTools;
 
+import org.apache.fop.ResourceEventProducer;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.fonts.FontInfo;
+import org.apache.fop.render.ImageHandler;
+import org.apache.fop.render.ImageHandlerRegistry;
 
 /**
  * This class is used when two-pass production is used to generate the PostScript file (setting
@@ -78,33 +75,89 @@ import org.apache.fop.fonts.FontInfo;
  */
 public class ResourceHandler implements DSCParserConstants, PSSupportedFlavors {
 
+    /** logging instance */
+    private static Log log = LogFactory.getLog(ResourceHandler.class);
+
+    private FOUserAgent userAgent;
+    private FontInfo fontInfo;
+
+    private PSEventProducer eventProducer;
+
+    private ResourceTracker resTracker;
+
+    //key: URI, values PSImageFormResource
+    private Map globalFormResources = new java.util.HashMap();
+    //key: PSResource, values PSImageFormResource
+    private Map inlineFormResources = new java.util.HashMap();
+
     /**
-     * Rewrites the temporary PostScript file generated by PSRenderer adding all needed resources
-     * (fonts and images).
+     * Main constructor.
      * @param userAgent the FO user agent
-     * @param in the InputStream for the temporary PostScript file
-     * @param out the OutputStream to write the finished file to
+     * @param eventProducer the event producer
      * @param fontInfo the font information
      * @param resTracker the resource tracker to use
      * @param formResources Contains all forms used by this document (maintained by PSRenderer)
+     */
+    public ResourceHandler(FOUserAgent userAgent, PSEventProducer eventProducer,
+            FontInfo fontInfo, ResourceTracker resTracker, Map formResources) {
+        this.userAgent = userAgent;
+        this.eventProducer = eventProducer;
+        this.fontInfo = fontInfo;
+        this.resTracker = resTracker;
+        determineInlineForms(formResources);
+    }
+
+    /**
+     * This method splits up the form resources map into two. One for global forms which
+     * have been referenced more than once, and one for inline forms which have only been
+     * used once. The latter is to conserve memory in the PostScript interpreter.
+     * @param formResources the original form resources map
+     */
+    private void determineInlineForms(Map formResources) {
+        if (formResources == null) {
+            return;
+        }
+        Iterator iter = formResources.entrySet().iterator();
+        while (iter.hasNext()) {
+            Map.Entry entry = (Map.Entry)iter.next();
+            PSResource res = (PSResource)entry.getValue();
+            long count = resTracker.getUsageCount(res);
+            if (count > 1) {
+                //Make global form
+                this.globalFormResources.put(entry.getKey(), res);
+            } else {
+                //Inline resource
+                this.inlineFormResources.put(res, res);
+                resTracker.declareInlined(res);
+            }
+        }
+    }
+
+    /**
+     * Rewrites the temporary PostScript file generated by PSRenderer adding all needed resources
+     * (fonts and images).
+     * @param in the InputStream for the temporary PostScript file
+     * @param out the OutputStream to write the finished file to
      * @param pageCount the number of pages (given here because PSRenderer writes an "(atend)")
      * @param documentBoundingBox the document's bounding box
      *                                  (given here because PSRenderer writes an "(atend)")
      * @throws DSCException If there's an error in the DSC structure of the PS file
      * @throws IOException In case of an I/O error
      */
-    public static void process(FOUserAgent userAgent, InputStream in, OutputStream out, 
-            FontInfo fontInfo, ResourceTracker resTracker, Map formResources,
+    public void process(InputStream in, OutputStream out,
             int pageCount, Rectangle2D documentBoundingBox)
                     throws DSCException, IOException {
         DSCParser parser = new DSCParser(in);
+        parser.setCheckEOF(false);
+
         PSGenerator gen = new PSGenerator(out);
-        parser.setNestedDocumentHandler(new DefaultNestedDocumentHandler(gen));
-        
+        parser.addListener(new DefaultNestedDocumentHandler(gen));
+        parser.addListener(new IncludeResourceListener(gen));
+
         //Skip DSC header
         DSCHeaderComment header = DSCTools.checkAndSkipDSC30Header(parser);
         header.generate(gen);
-        
+
         parser.setFilter(new DSCFilter() {
             private final Set filtered = new java.util.HashSet();
             {
@@ -139,16 +192,16 @@ public class ResourceHandler implements DSCParserConstants, PSSupportedFlavors {
                 new DSCCommentHiResBoundingBox(documentBoundingBox).generate(gen);
 
                 PSFontUtils.determineSuppliedFonts(resTracker, fontInfo, fontInfo.getUsedFonts());
-                registerSuppliedForms(resTracker, formResources);
-                
+                registerSuppliedForms(resTracker, globalFormResources);
+
                 //Supplied Resources
-                DSCCommentDocumentSuppliedResources supplied 
+                DSCCommentDocumentSuppliedResources supplied
                     = new DSCCommentDocumentSuppliedResources(
                             resTracker.getDocumentSuppliedResources());
                 supplied.generate(gen);
-                
+
                 //Needed Resources
-                DSCCommentDocumentNeededResources needed 
+                DSCCommentDocumentNeededResources needed
                     = new DSCCommentDocumentNeededResources(
                             resTracker.getDocumentNeededResources());
                 needed.generate(gen);
@@ -166,21 +219,21 @@ public class ResourceHandler implements DSCParserConstants, PSSupportedFlavors {
             }
             event.generate(gen);
         }
-        
+
         //Skip to the FOPFontSetup
         PostScriptComment fontSetupPlaceholder = parser.nextPSComment("FOPFontSetup", gen);
         if (fontSetupPlaceholder == null) {
             throw new DSCException("Didn't find %FOPFontSetup comment in stream");
         }
-        PSFontUtils.writeFontDict(gen, fontInfo, fontInfo.getUsedFonts());
-        generateForms(resTracker, userAgent, formResources, gen);
+        PSFontUtils.writeFontDict(gen, fontInfo, fontInfo.getUsedFonts(), eventProducer);
+        generateForms(globalFormResources, gen);
 
         //Skip the prolog and to the first page
         DSCComment pageOrTrailer = parser.nextDSCComment(DSCConstants.PAGE, gen);
         if (pageOrTrailer == null) {
             throw new DSCException("Page expected, but none found");
         }
-        
+
         //Process individual pages (and skip as necessary)
         while (true) {
             DSCCommentPage page = (DSCCommentPage)pageOrTrailer;
@@ -193,12 +246,13 @@ public class ResourceHandler implements DSCParserConstants, PSSupportedFlavors {
                 break;
             }
         }
-        
+
         //Write the rest
         while (parser.hasNext()) {
             DSCEvent event = parser.nextEvent();
             event.generate(gen);
         }
+        gen.flush();
     }
 
     private static void reportInvalidDSC() throws DSCException {
@@ -217,115 +271,68 @@ public class ResourceHandler implements DSCParserConstants, PSSupportedFlavors {
         }
     }
 
-    private static void generateForms(ResourceTracker resTracker, FOUserAgent userAgent, 
-            Map formResources, PSGenerator gen) throws IOException {
+    private void generateForms(Map formResources, PSGenerator gen) throws IOException {
         if (formResources == null) {
             return;
         }
         Iterator iter = formResources.values().iterator();
         while (iter.hasNext()) {
             PSImageFormResource form = (PSImageFormResource)iter.next();
-            final String uri = form.getImageURI();
-            
-            ImageManager manager = userAgent.getFactory().getImageManager();
-            ImageInfo info = null;
-            try {
-                ImageSessionContext sessionContext = userAgent.getImageSessionContext();
-                info = manager.getImageInfo(uri, sessionContext);
-                
-                ImageFlavor[] flavors;
-                if (gen.getPSLevel() >= 3) {
-                    flavors = LEVEL_3_FLAVORS_FORM;
-                } else {
-                    flavors = LEVEL_2_FLAVORS_FORM;
-                }
-                Map hints = ImageUtil.getDefaultHints(sessionContext);
-                org.apache.xmlgraphics.image.loader.Image img = manager.getImage(
-                        info, flavors, hints, sessionContext);
-                
-                String imageDescription = info.getMimeType() + " " + info.getOriginalURI();
-                final Dimension2D dimensionsPt = info.getSize().getDimensionPt();
-                final Dimension2D dimensionsMpt = info.getSize().getDimensionMpt();
-
-                if (img instanceof ImageGraphics2D) {
-                    final ImageGraphics2D imageG2D = (ImageGraphics2D)img;
-                    FormGenerator formGen = new FormGenerator(
-                            form.getName(), imageDescription, dimensionsPt) {
-
-                        protected void generatePaintProc(PSGenerator gen)
-                                throws IOException {
-                            gen.getResourceTracker().notifyResourceUsageOnPage(
-                                    PSProcSets.EPS_PROCSET);
-                            gen.writeln("BeginEPSF");
-                            PSGraphics2DAdapter adapter = new PSGraphics2DAdapter(gen, false);
-                            adapter.paintImage(imageG2D.getGraphics2DImagePainter(),
-                                    null,
-                                    0, 0, 
-                                    (int)Math.round(dimensionsMpt.getWidth()),
-                                    (int)Math.round(dimensionsMpt.getHeight()));
-                            gen.writeln("EndEPSF");
-                        }
-                        
-                    };
-                    formGen.generate(gen);
-                } else if (img instanceof ImageRendered) {
-                    ImageRendered imgRend = (ImageRendered)img;
-                    RenderedImage ri = imgRend.getRenderedImage();
-                    FormGenerator formGen = new ImageFormGenerator(
-                            form.getName(), imageDescription,
-                            info.getSize().getDimensionPt(),
-                            ri, false);
-                    formGen.generate(gen);
-                } else if (img instanceof ImageXMLDOM) {
-                    throw new UnsupportedOperationException(
-                            "Embedding an ImageXMLDOM as a form isn't supported, yet");
-                } else if (img instanceof ImageRawStream) {
-                    final ImageRawStream raw = (ImageRawStream)img;
-                    if (raw instanceof ImageRawEPS) {
-                        final ImageRawEPS eps = (ImageRawEPS)raw;
-                        throw new UnsupportedOperationException(
-                                "Embedding EPS as forms isn't supported, yet");
-                        /*
-                        InputStream in = eps.createInputStream();
-                        try {
-                            FormGenerator formGen = new EPSFormGenerator(form.getName(),
-                                    imageDescription, dimensions, in);
-                            formGen.generate(gen);
-                        } finally {
-                            IOUtils.closeQuietly(in);
-                        }*/
-                    } else if (raw instanceof ImageRawCCITTFax) {
-                        ImageRawCCITTFax jpeg = (ImageRawCCITTFax)raw;
-                        ImageEncoder encoder = new ImageEncoderCCITTFax(jpeg);
-                        FormGenerator formGen = new ImageFormGenerator(
-                                form.getName(), imageDescription,
-                                info.getSize().getDimensionPt(),
-                                info.getSize().getDimensionPx(),
-                                encoder,
-                                jpeg.getColorSpace(), 1, false);
-                        formGen.generate(gen);
-                    } else if (raw instanceof ImageRawJPEG) {
-                        ImageRawJPEG jpeg = (ImageRawJPEG)raw;
-                        ImageEncoder encoder = new ImageEncoderJPEG(jpeg);
-                        FormGenerator formGen = new ImageFormGenerator(
-                                form.getName(), imageDescription,
-                                info.getSize().getDimensionPt(),
-                                info.getSize().getDimensionPx(),
-                                encoder,
-                                jpeg.getColorSpace(), jpeg.isInverted());
-                        formGen.generate(gen);
-                    } else {
-                        throw new UnsupportedOperationException("Unsupported raw image: " + info);
-                    }
-                } else {
-                    throw new UnsupportedOperationException("Unsupported image type: " + img);
-                }
-            } catch (ImageException ie) {
-                throw new IOException("Error while generating form for image: " + ie.getMessage());
-            }
+            generateFormForImage(gen, form);
         }
     }
 
+    private void generateFormForImage(PSGenerator gen, PSImageFormResource form)
+                throws IOException {
+        final String uri = form.getImageURI();
+
+        ImageManager manager = userAgent.getImageManager();
+        ImageInfo info = null;
+        try {
+            ImageSessionContext sessionContext = userAgent.getImageSessionContext();
+            info = manager.getImageInfo(uri, sessionContext);
+
+            //Create a rendering context for form creation
+            PSRenderingContext formContext = new PSRenderingContext(
+                    userAgent, gen, fontInfo, true);
+
+            ImageFlavor[] flavors;
+            ImageHandlerRegistry imageHandlerRegistry
+                = userAgent.getImageHandlerRegistry();
+            flavors = imageHandlerRegistry.getSupportedFlavors(formContext);
+
+            Map hints = ImageUtil.getDefaultHints(sessionContext);
+            org.apache.xmlgraphics.image.loader.Image img = manager.getImage(
+                    info, flavors, hints, sessionContext);
+
+            ImageHandler basicHandler = imageHandlerRegistry.getHandler(formContext, img);
+            if (basicHandler == null) {
+                throw new UnsupportedOperationException(
+                        "No ImageHandler available for image: "
+                            + img.getInfo() + " (" + img.getClass().getName() + ")");
+            }
+
+            if (!(basicHandler instanceof PSImageHandler)) {
+                throw new IllegalStateException(
+                        "ImageHandler implementation doesn't behave properly."
+                        + " It should have returned false in isCompatible(). Class: "
+                        + basicHandler.getClass().getName());
+            }
+            PSImageHandler handler = (PSImageHandler)basicHandler;
+            if (log.isTraceEnabled()) {
+                log.trace("Using ImageHandler: " + handler.getClass().getName());
+            }
+            handler.generateForm(formContext, img, form);
+
+        } catch (ImageException ie) {
+            ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
+                    userAgent.getEventBroadcaster());
+            eventProducer.imageError(resTracker, (info != null ? info.toString() : uri),
+                    ie, null);
+        }
+    }
+
+    /* not used
     private static FormGenerator createMissingForm(String formName, final Dimension2D dimensions) {
         FormGenerator formGen = new FormGenerator(formName, null, dimensions) {
 
@@ -345,9 +352,56 @@ public class ResourceHandler implements DSCParserConstants, PSSupportedFlavors {
                 gen.writeln("1 0 lineto");
                 gen.writeln("stroke");
             }
-            
+
         };
         return formGen;
     }
-    
+    */
+
+    private class IncludeResourceListener implements DSCListener {
+
+        private PSGenerator gen;
+
+        public IncludeResourceListener(PSGenerator gen) {
+            this.gen = gen;
+        }
+
+        /** {@inheritDoc} */
+        public void processEvent(DSCEvent event, DSCParser parser)
+                    throws IOException, DSCException {
+            if (event.isDSCComment() && event instanceof DSCCommentIncludeResource) {
+                DSCCommentIncludeResource include = (DSCCommentIncludeResource)event;
+                PSResource res = include.getResource();
+                if (res.getType().equals(PSResource.TYPE_FORM)) {
+                    if (inlineFormResources.containsValue(res)) {
+                        PSImageFormResource form = (PSImageFormResource)
+                                    inlineFormResources.get(res);
+                        //Create an inline form
+                        //Wrap in save/restore pair to release memory
+                        gen.writeln("save");
+                        generateFormForImage(gen, form);
+                        boolean execformFound = false;
+                        DSCEvent next = parser.nextEvent();
+                        if (next.isLine()) {
+                            PostScriptLine line = next.asLine();
+                            if (line.getLine().endsWith(" execform")) {
+                                line.generate(gen);
+                                execformFound = true;
+                            }
+                        }
+                        if (!execformFound) {
+                            throw new IOException(
+                                "Expected a PostScript line in the form: <form> execform");
+                        }
+                        gen.writeln("restore");
+                    } else {
+                        //Do nothing
+                    }
+                    parser.next();
+                }
+            }
+        }
+
+    }
+
 }
