@@ -15,22 +15,22 @@
  * limitations under the License.
  */
 
-/* $Id: PSPageSetupCodeElement.java 588749 2007-10-26 18:58:15Z acumiskey $ */
+/* $Id: PSPageSetupCodeElement.java 1465599 2013-04-08 11:51:52Z vhennebert $ */
 
 package org.apache.fop.render.ps.extensions;
 
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FONode;
-import org.apache.fop.fo.ValidationException;
 
 /**
- * Extension element for fox:ps-page-setup-code. 
+ * Extension element for fox:ps-page-setup-code.
  */
 public class PSPageSetupCodeElement extends AbstractPSExtensionObject {
 
+    /** The element name */
     protected static final String ELEMENT = "ps-page-setup-code";
-    
+
     /**
      * Main constructor
      * @param parent parent FO node
@@ -40,13 +40,14 @@ public class PSPageSetupCodeElement extends AbstractPSExtensionObject {
     }
 
     /** {@inheritDoc} */
-    protected void startOfNode() throws FOPException {
+    public void startOfNode() throws FOPException {
         super.startOfNode();
         if (parent.getNameId() != Constants.FO_SIMPLE_PAGE_MASTER) {
-            throw new ValidationException(getName() + " must be a child of fo:simple-page-master.");
+            invalidChildError(getLocator(), parent.getName(), getNamespaceURI(), getName(),
+                "rule.childOfSPM");
         }
     }
-    
+
     /** {@inheritDoc} */
     public String getLocalName() {
         return ELEMENT;
