@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PSSetPageDevice.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: PSSetPageDevice.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.render.ps.extensions;
 
@@ -37,7 +37,7 @@ public class PSSetPageDevice extends PSExtensionAttachment {
     /**
      * name attribute
      */
-    protected String name = null;
+    protected String name;
 
     /**
      * default constructor
