@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PSSetupCodeElement.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: PSSetupCodeElement.java 1465599 2013-04-08 11:51:52Z vhennebert $ */
 
 package org.apache.fop.render.ps.extensions;
 
@@ -40,7 +40,7 @@ public class PSSetupCodeElement extends AbstractPSExtensionObject {
     }
 
     /** {@inheritDoc} */
-    protected void startOfNode() throws FOPException {
+    public void startOfNode() throws FOPException {
         super.startOfNode();
         if (parent.getNameId() != Constants.FO_DECLARATIONS) {
             invalidChildError(getLocator(), parent.getName(), getNamespaceURI(), getName(),
