@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RTFHandler.java 932497 2010-04-09 16:34:29Z vhennebert $ */
+/* $Id: RTFHandler.java 1681438 2015-05-24 12:14:23Z adelmelle $ */
 
 package org.apache.fop.render.rtf;
 
@@ -32,7 +32,6 @@ import java.util.Iterator;
 import java.util.Map;
 
 import org.w3c.dom.Document;
-
 import org.xml.sax.SAXException;
 
 import org.apache.commons.io.IOUtils;
@@ -78,13 +77,14 @@ import org.apache.fop.fo.flow.ListItemBody;
 import org.apache.fop.fo.flow.ListItemLabel;
 import org.apache.fop.fo.flow.PageNumber;
 import org.apache.fop.fo.flow.PageNumberCitation;
+import org.apache.fop.fo.flow.PageNumberCitationLast;
 import org.apache.fop.fo.flow.table.Table;
 import org.apache.fop.fo.flow.table.TableBody;
-import org.apache.fop.fo.flow.table.TableFooter;
-import org.apache.fop.fo.flow.table.TablePart;
 import org.apache.fop.fo.flow.table.TableCell;
 import org.apache.fop.fo.flow.table.TableColumn;
+import org.apache.fop.fo.flow.table.TableFooter;
 import org.apache.fop.fo.flow.table.TableHeader;
+import org.apache.fop.fo.flow.table.TablePart;
 import org.apache.fop.fo.flow.table.TableRow;
 import org.apache.fop.fo.pagination.Flow;
 import org.apache.fop.fo.pagination.PageSequence;
@@ -97,8 +97,8 @@ import org.apache.fop.fo.properties.EnumLength;
 import org.apache.fop.fonts.FontSetup;
 import org.apache.fop.layoutmgr.inline.ImageLayout;
 import org.apache.fop.layoutmgr.table.ColumnSetup;
-import org.apache.fop.render.DefaultFontResolver;
 import org.apache.fop.render.RendererEventProducer;
+import org.apache.fop.render.rtf.rtflib.exceptions.RtfException;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.IRtfAfterContainer;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.IRtfBeforeContainer;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.IRtfListContainer;
@@ -116,13 +116,14 @@ import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfFootnote;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfHyperLink;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfList;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfListItem;
+import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfListItem.RtfListItemLabel;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfPage;
+import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfParagraphBreak;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfSection;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTable;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTableCell;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTableRow;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTextrun;
-import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfListItem.RtfListItemLabel;
 import org.apache.fop.render.rtf.rtflib.tools.BuilderContext;
 import org.apache.fop.render.rtf.rtflib.tools.PercentContext;
 import org.apache.fop.render.rtf.rtflib.tools.TableContext;
@@ -130,12 +131,6 @@ import org.apache.fop.render.rtf.rtflib.tools.TableContext;
 /**
  * RTF Handler: generates RTF output using the structure events from
  * the FO Tree sent to this structure handler.
- *
- * @author Bertrand Delacretaz <bdelacretaz@codeconsult.ch>
- * @author Trembicki-Guy, Ed <GuyE@DNB.com>
- * @author Boris Poudérous <boris.pouderous@eads-telecom.com>
- * @author Peter Herweg <pherweg@web.de>
- * @author Andreas Putz <a.putz@skynamics.com>
  */
 public class RTFHandler extends FOEventHandler {
 
@@ -146,13 +141,13 @@ public class RTFHandler extends FOEventHandler {
     private RtfDocumentArea docArea;
     private boolean bDefer;              //true, if each called handler shall be
                                          //processed at later time.
-    private boolean bPrevHeaderSpecified = false; //true, if there has been a
+    private boolean bPrevHeaderSpecified; //true, if there has been a
                                                   //header in any page-sequence
-    private boolean bPrevFooterSpecified = false; //true, if there has been a
+    private boolean bPrevFooterSpecified; //true, if there has been a
                                                   //footer in any page-sequence
-    private boolean bHeaderSpecified = false;  //true, if there is a header
+    private boolean bHeaderSpecified;  //true, if there is a header
                                                //in current page-sequence
-    private boolean bFooterSpecified = false;  //true, if there is a footer
+    private boolean bFooterSpecified;  //true, if there is a footer
                                                //in current page-sequence
     private BuilderContext builderContext = new BuilderContext(null);
 
@@ -162,6 +157,7 @@ public class RTFHandler extends FOEventHandler {
 
     private PercentContext percentManager = new PercentContext();
 
+
     /**
      * Creates a new RTF structure handler.
      * @param userAgent the FOUserAgent for this process
@@ -172,7 +168,8 @@ public class RTFHandler extends FOEventHandler {
         this.os = os;
         bDefer = true;
 
-        FontSetup.setup(fontInfo, null, new DefaultFontResolver(userAgent));
+        boolean base14Kerning = false;
+        FontSetup.setup(fontInfo, null, userAgent.getResourceResolver(), base14Kerning);
     }
 
     /**
@@ -221,8 +218,10 @@ public class RTFHandler extends FOEventHandler {
                     eventProducer.onlySPMSupported(this, reference, pageSeq.getLocator());
                     PageSequenceMaster master
                         = pageSeq.getRoot().getLayoutMasterSet().getPageSequenceMaster(reference);
-                    this.pagemaster = master.getNextSimplePageMaster(
-                            false, false, false, false);
+                    if (pageSeq.getMainFlow() != null) {
+                        this.pagemaster = master.getNextSimplePageMaster(
+                                false, false, false, false, pageSeq.getMainFlow().getFlowName());
+                    }
                 }
             }
 
@@ -274,7 +273,7 @@ public class RTFHandler extends FOEventHandler {
 
             return;
         } else {
-            builderContext.popContainer();
+            builderContext.popContainer(RtfSection.class, this);
             this.pagemaster = null;
         }
     }
@@ -299,8 +298,8 @@ public class RTFHandler extends FOEventHandler {
                     attr.set(RtfBefore.HEADER);
 
                     final IRtfBeforeContainer contBefore
-                        = (IRtfBeforeContainer)builderContext.getContainer
-                                (IRtfBeforeContainer.class, true, this);
+                        = (IRtfBeforeContainer)builderContext.getContainer(
+                                IRtfBeforeContainer.class, true, this);
                     contBefore.newBefore(attr);
                 }
 
@@ -311,8 +310,8 @@ public class RTFHandler extends FOEventHandler {
                     attr.set(RtfAfter.FOOTER);
 
                     final IRtfAfterContainer contAfter
-                        = (IRtfAfterContainer)builderContext.getContainer
-                                (IRtfAfterContainer.class, true, this);
+                        = (IRtfAfterContainer)builderContext.getContainer(
+                                IRtfAfterContainer.class, true, this);
                     contAfter.newAfter(attr);
                 }
                 handled = true;
@@ -381,10 +380,10 @@ public class RTFHandler extends FOEventHandler {
                 //just do nothing
             } else if (regionBefore != null
                     && fl.getFlowName().equals(regionBefore.getRegionName())) {
-                builderContext.popContainer();
+                builderContext.popContainer(RtfBefore.class, this);
             } else if (regionAfter != null
                     && fl.getFlowName().equals(regionAfter.getRegionName())) {
-                builderContext.popContainer();
+                builderContext.popContainer(RtfAfter.class, this);
             }
         } catch (Exception e) {
             log.error("endFlow: " + e.getMessage());
@@ -434,9 +433,16 @@ public class RTFHandler extends FOEventHandler {
                     true, this);
 
             RtfTextrun textrun = container.getTextrun();
+            RtfParagraphBreak par = textrun.addParagraphBreak();
 
-            textrun.addParagraphBreak();
-            textrun.popBlockAttributes();
+            RtfTableCell cellParent = (RtfTableCell)textrun.getParentOfClass(RtfTableCell.class);
+            if (cellParent != null && par != null) {
+                int iDepth = cellParent.findChildren(textrun);
+                cellParent.setLastParagraph(par, iDepth);
+            }
+
+            int breakValue = toRtfBreakValue(bl.getBreakAfter());
+            textrun.popBlockAttributes(breakValue);
 
         } catch (IOException ioe) {
             handleIOTrouble(ioe);
@@ -488,7 +494,8 @@ public class RTFHandler extends FOEventHandler {
             RtfTextrun textrun = container.getTextrun();
 
             textrun.addParagraphBreak();
-            textrun.popBlockAttributes();
+            int breakValue = toRtfBreakValue(bl.getBreakAfter());
+            textrun.popBlockAttributes(breakValue);
 
         } catch (IOException ioe) {
             handleIOTrouble(ioe);
@@ -498,6 +505,21 @@ public class RTFHandler extends FOEventHandler {
         }
     }
 
+    private int toRtfBreakValue(int foBreakValue) {
+        switch (foBreakValue) {
+        case Constants.EN_PAGE:
+            return RtfTextrun.BREAK_PAGE;
+        case Constants.EN_EVEN_PAGE:
+            return RtfTextrun.BREAK_EVEN_PAGE;
+        case Constants.EN_ODD_PAGE:
+            return RtfTextrun.BREAK_ODD_PAGE;
+        case Constants.EN_COLUMN:
+            return RtfTextrun.BREAK_COLUMN;
+        default:
+            return RtfTextrun.BREAK_NONE;
+        }
+    }
+
     /** {@inheritDoc} */
     public void startTable(Table tbl) {
         if (bDefer) {
@@ -552,7 +574,7 @@ public class RTFHandler extends FOEventHandler {
 
         nestedTableDepth--;
         builderContext.popTableContext();
-        builderContext.popContainer();
+        builderContext.popContainer(RtfTable.class, this);
     }
 
     /** {@inheritDoc} */
@@ -586,21 +608,25 @@ public class RTFHandler extends FOEventHandler {
 
     /** {@inheritDoc} */
     public void startHeader(TableHeader header) {
+        builderContext.pushPart(header);
         startPart(header);
     }
 
     /** {@inheritDoc} */
     public void endHeader(TableHeader header) {
+        builderContext.popPart(header.getClass(), this);
         endPart(header);
     }
 
     /** {@inheritDoc} */
     public void startFooter(TableFooter footer) {
+        builderContext.pushPart(footer);
         startPart(footer);
     }
 
     /** {@inheritDoc} */
     public void endFooter(TableFooter footer) {
+        builderContext.popPart(footer.getClass(), this);
         endPart(footer);
     }
 
@@ -661,6 +687,7 @@ public class RTFHandler extends FOEventHandler {
         try {
             RtfAttributes atts = TableAttributesConverter.convertTablePartAttributes(part);
 
+            // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
             RtfTable tbl = (RtfTable)builderContext.getContainer(RtfTable.class, true, this);
             tbl.setHeaderAttribs(atts);
         } catch (IOException ioe) {
@@ -677,6 +704,7 @@ public class RTFHandler extends FOEventHandler {
         }
 
         try {
+            // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
             RtfTable tbl = (RtfTable)builderContext.getContainer(RtfTable.class, true, this);
             tbl.setHeaderAttribs(null);
         } catch (IOException ioe) {
@@ -692,11 +720,13 @@ public class RTFHandler extends FOEventHandler {
      * {@inheritDoc}
      */
     public void startBody(TableBody body) {
+        builderContext.pushPart(body);
         startPart(body);
     }
 
     /** {@inheritDoc} */
     public void endBody(TableBody body) {
+        builderContext.popPart(TableBody.class, this);
         endPart(body);
     }
 
@@ -710,6 +740,7 @@ public class RTFHandler extends FOEventHandler {
 
         try {
             // create an RtfTableRow in the current RtfTable
+            // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
             final RtfTable tbl = (RtfTable)builderContext.getContainer(RtfTable.class,
                     true, null);
 
@@ -740,6 +771,7 @@ public class RTFHandler extends FOEventHandler {
 
         try {
             TableContext tctx = builderContext.getTableContext();
+            // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
             final RtfTableRow row = (RtfTableRow)builderContext.getContainer(RtfTableRow.class,
                     true, null);
 
@@ -765,7 +797,7 @@ public class RTFHandler extends FOEventHandler {
         }
 
 
-        builderContext.popContainer();
+        builderContext.popContainer(RtfTableRow.class, this);
         builderContext.getTableContext().decreaseRowSpannings();
     }
 
@@ -777,6 +809,7 @@ public class RTFHandler extends FOEventHandler {
 
         try {
             TableContext tctx = builderContext.getTableContext();
+            // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
             final RtfTableRow row = (RtfTableRow)builderContext.getContainer(RtfTableRow.class,
                     true, null);
 
@@ -865,8 +898,17 @@ public class RTFHandler extends FOEventHandler {
         if (bDefer) {
             return;
         }
+        try {
+            // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
+            RtfTableCell cell = (RtfTableCell)builderContext.getContainer(RtfTableCell.class, false, this);
+            cell.finish();
 
-        builderContext.popContainer();
+        } catch (Exception e) {
+            log.error("endCell: " + e.getMessage());
+            throw new RuntimeException(e.getMessage());
+        }
+
+        builderContext.popContainer(RtfTableCell.class, this);
         builderContext.getTableContext().selectNextColumn();
     }
 
@@ -902,7 +944,7 @@ public class RTFHandler extends FOEventHandler {
             return;
         }
 
-        builderContext.popContainer();
+        builderContext.popContainer(RtfList.class, this);
     }
 
     /** {@inheritDoc} */
@@ -913,6 +955,7 @@ public class RTFHandler extends FOEventHandler {
 
         // create an RtfListItem in the current RtfList
         try {
+            // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
             RtfList list = (RtfList)builderContext.getContainer(
                     RtfList.class, true, this);
 
@@ -925,10 +968,10 @@ public class RTFHandler extends FOEventHandler {
              */
             //TODO: do this only, if the labels content <> previous labels content
             if (list.getChildCount() > 0) {
-                this.endListBody();
+                this.endListBody(null);
                 this.endList((ListBlock) li.getParent());
                 this.startList((ListBlock) li.getParent());
-                this.startListBody();
+                this.startListBody(null);
 
                 list = (RtfList)builderContext.getContainer(
                         RtfList.class, true, this);
@@ -949,16 +992,17 @@ public class RTFHandler extends FOEventHandler {
             return;
         }
 
-        builderContext.popContainer();
+        builderContext.popContainer(RtfListItem.class, this);
     }
 
     /** {@inheritDoc} */
-    public void startListLabel() {
+    public void startListLabel(ListItemLabel listItemLabel) {
         if (bDefer) {
             return;
         }
 
         try {
+            // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
             RtfListItem item
                 = (RtfListItem)builderContext.getContainer(RtfListItem.class, true, this);
 
@@ -973,29 +1017,29 @@ public class RTFHandler extends FOEventHandler {
     }
 
     /** {@inheritDoc} */
-    public void endListLabel() {
+    public void endListLabel(ListItemLabel listItemLabel) {
         if (bDefer) {
             return;
         }
 
-        builderContext.popContainer();
+        builderContext.popContainer(RtfListItemLabel.class, this);
     }
 
     /** {@inheritDoc} */
-    public void startListBody() {
+    public void startListBody(ListItemBody listItemBody) {
     }
 
     /** {@inheritDoc} */
-    public void endListBody() {
+    public void endListBody(ListItemBody listItemBody) {
     }
 
     // Static Regions
     /** {@inheritDoc} */
-    public void startStatic() {
+    public void startStatic(StaticContent staticContent) {
     }
 
     /** {@inheritDoc} */
-    public void endStatic() {
+    public void endStatic(StaticContent statisContent) {
     }
 
     /** {@inheritDoc} */
@@ -1038,12 +1082,12 @@ public class RTFHandler extends FOEventHandler {
     }
 
     /** {@inheritDoc} */
-    public void endLink() {
+    public void endLink(BasicLink basicLink) {
         if (bDefer) {
             return;
         }
 
-        builderContext.popContainer();
+        builderContext.popContainer(RtfHyperLink.class, this);
     }
 
     /** {@inheritDoc} */
@@ -1058,14 +1102,14 @@ public class RTFHandler extends FOEventHandler {
 
             //set image data
             FOUserAgent userAgent = eg.getUserAgent();
-            ImageManager manager = userAgent.getFactory().getImageManager();
+            ImageManager manager = userAgent.getImageManager();
             info = manager.getImageInfo(uri, userAgent.getImageSessionContext());
 
             putGraphic(eg, info);
         } catch (ImageException ie) {
             ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
                     getUserAgent().getEventBroadcaster());
-            eventProducer.imageError(this, (info != null ? info.toString() : uri), ie, null);
+            eventProducer.imageError(this, uri, ie, null);
         } catch (FileNotFoundException fe) {
             ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
                     getUserAgent().getEventBroadcaster());
@@ -1078,7 +1122,7 @@ public class RTFHandler extends FOEventHandler {
     }
 
     /** {@inheritDoc} */
-    public void foreignObject(InstreamForeignObject ifo) {
+    public void endInstreamForeignObject(InstreamForeignObject ifo) {
         if (bDefer) {
             return;
         }
@@ -1112,7 +1156,7 @@ public class RTFHandler extends FOEventHandler {
             ImageXMLDOM image = new ImageXMLDOM(info, doc, ns);
 
             FOUserAgent userAgent = ifo.getUserAgent();
-            ImageManager manager = userAgent.getFactory().getImageManager();
+            ImageManager manager = userAgent.getImageManager();
             Map hints = ImageUtil.getDefaultHints(ua.getImageSessionContext());
             Image converted = manager.convertImage(image, FLAVORS, hints);
             putGraphic(ifo, converted);
@@ -1142,7 +1186,7 @@ public class RTFHandler extends FOEventHandler {
             throws IOException {
         try {
             FOUserAgent userAgent = abstractGraphic.getUserAgent();
-            ImageManager manager = userAgent.getFactory().getImageManager();
+            ImageManager manager = userAgent.getImageManager();
             ImageSessionContext sessionContext = userAgent.getImageSessionContext();
             Map hints = ImageUtil.getDefaultHints(sessionContext);
             Image image = manager.getImage(info, FLAVORS, hints, sessionContext);
@@ -1279,7 +1323,7 @@ public class RTFHandler extends FOEventHandler {
             return;
         }
 
-        builderContext.popContainer();
+        builderContext.popContainer(RtfFootnote.class, this);
     }
 
     /** {@inheritDoc} */
@@ -1289,6 +1333,7 @@ public class RTFHandler extends FOEventHandler {
         }
 
         try {
+            // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
             RtfFootnote rtfFootnote
                 = (RtfFootnote)builderContext.getContainer(
                     RtfFootnote.class,
@@ -1310,6 +1355,7 @@ public class RTFHandler extends FOEventHandler {
         }
 
         try {
+            // @SuppressFBWarnings("BC_UNCONFIRMED_CAST_OF_RETURN_VALUE")
             RtfFootnote rtfFootnote
                 = (RtfFootnote)builderContext.getContainer(
                     RtfFootnote.class,
@@ -1325,7 +1371,7 @@ public class RTFHandler extends FOEventHandler {
     }
 
     /** {@inheritDoc} */
-    public void leader(Leader l) {
+    public void startLeader(Leader l) {
         if (bDefer) {
             return;
         }
@@ -1341,8 +1387,10 @@ public class RTFHandler extends FOEventHandler {
             RtfTextrun textrun = container.getTextrun();
 
             textrun.addLeader(rtfAttr);
-
-        } catch (Exception e) {
+        } catch (IOException e) {
+            log.error("startLeader: " + e.getMessage());
+            throw new RuntimeException(e.getMessage());
+        } catch (FOPException e) {
             log.error("startLeader: " + e.getMessage());
             throw new RuntimeException(e.getMessage());
         }
@@ -1350,11 +1398,9 @@ public class RTFHandler extends FOEventHandler {
 
     /**
      * @param text FOText object
-     * @param data Array of characters to process.
-     * @param start Offset for characters to process.
-     * @param length Portion of array to process.
+     * @param characters CharSequence of the characters to process.
      */
-    public void text(FOText text, char[] data, int start, int length) {
+    public void text(FOText text, CharSequence characters) {
         if (bDefer) {
             return;
         }
@@ -1369,7 +1415,7 @@ public class RTFHandler extends FOEventHandler {
                 = TextAttributesConverter.convertCharacterAttributes(text);
 
             textrun.pushInlineAttributes(rtfAttr);
-            textrun.addString(new String(data, start, length - start));
+            textrun.addString(characters.toString());
             textrun.popInlineAttributes();
         } catch (IOException ioe) {
             handleIOTrouble(ioe);
@@ -1431,6 +1477,29 @@ public class RTFHandler extends FOEventHandler {
         }
     }
 
+    /** {@inheritDoc} */
+    public void startPageNumberCitationLast(PageNumberCitationLast l) {
+        if (bDefer) {
+            return;
+        }
+        try {
+
+            IRtfTextrunContainer container
+                  = (IRtfTextrunContainer)builderContext.getContainer(
+                      IRtfTextrunContainer.class, true, this);
+            RtfTextrun textrun = container.getTextrun();
+
+            textrun.addPageNumberCitation(l.getRefId());
+
+        } catch (RtfException e) {
+            log.error("startPageNumberCitationLast: " + e.getMessage());
+            throw new RuntimeException(e.getMessage());
+        } catch (IOException e) {
+            log.error("startPageNumberCitationLast: " + e.getMessage());
+            throw new RuntimeException(e.getMessage());
+        }
+    }
+
     private void prepareTable(Table tab) {
         // Allows to receive the available width of the table
         percentManager.setDimension(tab);
@@ -1471,59 +1540,59 @@ public class RTFHandler extends FOEventHandler {
     private void invokeDeferredEvent(FONode foNode, boolean bStart) {
         if (foNode instanceof PageSequence) {
             if (bStart) {
-                startPageSequence( (PageSequence) foNode);
+                startPageSequence((PageSequence) foNode);
             } else {
-                endPageSequence( (PageSequence) foNode);
+                endPageSequence((PageSequence) foNode);
             }
         } else if (foNode instanceof Flow) {
             if (bStart) {
-                startFlow( (Flow) foNode);
+                startFlow((Flow) foNode);
             } else {
-                endFlow( (Flow) foNode);
+                endFlow((Flow) foNode);
             }
         } else if (foNode instanceof StaticContent) {
             if (bStart) {
-                startStatic();
+                startStatic(null);
             } else {
-                endStatic();
+                endStatic(null);
             }
         } else if (foNode instanceof ExternalGraphic) {
             if (bStart) {
-                image( (ExternalGraphic) foNode );
+                image((ExternalGraphic) foNode);
             }
         } else if (foNode instanceof InstreamForeignObject) {
             if (bStart) {
-                foreignObject( (InstreamForeignObject) foNode );
+                endInstreamForeignObject((InstreamForeignObject) foNode);
             }
         } else if (foNode instanceof Block) {
             if (bStart) {
-                startBlock( (Block) foNode);
+                startBlock((Block) foNode);
             } else {
-                endBlock( (Block) foNode);
+                endBlock((Block) foNode);
             }
         } else if (foNode instanceof BlockContainer) {
             if (bStart) {
-                startBlockContainer( (BlockContainer) foNode);
+                startBlockContainer((BlockContainer) foNode);
             } else {
-                endBlockContainer( (BlockContainer) foNode);
+                endBlockContainer((BlockContainer) foNode);
             }
         } else if (foNode instanceof BasicLink) {
             //BasicLink must be placed before Inline
             if (bStart) {
-                startLink( (BasicLink) foNode);
+                startLink((BasicLink) foNode);
             } else {
-                endLink();
+                endLink(null);
             }
         } else if (foNode instanceof Inline) {
             if (bStart) {
-                startInline( (Inline) foNode);
+                startInline((Inline) foNode);
             } else {
-                endInline( (Inline) foNode);
+                endInline((Inline) foNode);
             }
         } else if (foNode instanceof FOText) {
             if (bStart) {
                 FOText text = (FOText) foNode;
-                text(text, text.getCharArray(), 0, text.length());
+                text(text, text.getCharSequence());
             }
         } else if (foNode instanceof Character) {
             if (bStart) {
@@ -1532,91 +1601,91 @@ public class RTFHandler extends FOEventHandler {
             }
         } else if (foNode instanceof PageNumber) {
             if (bStart) {
-                startPageNumber( (PageNumber) foNode);
+                startPageNumber((PageNumber) foNode);
             } else {
-                endPageNumber( (PageNumber) foNode);
+                endPageNumber((PageNumber) foNode);
             }
         } else if (foNode instanceof Footnote) {
             if (bStart) {
-                startFootnote( (Footnote) foNode);
+                startFootnote((Footnote) foNode);
             } else {
-                endFootnote( (Footnote) foNode);
+                endFootnote((Footnote) foNode);
             }
         } else if (foNode instanceof FootnoteBody) {
             if (bStart) {
-                startFootnoteBody( (FootnoteBody) foNode);
+                startFootnoteBody((FootnoteBody) foNode);
             } else {
-                endFootnoteBody( (FootnoteBody) foNode);
+                endFootnoteBody((FootnoteBody) foNode);
             }
         } else if (foNode instanceof ListBlock) {
             if (bStart) {
-                startList( (ListBlock) foNode);
+                startList((ListBlock) foNode);
             } else {
-                endList( (ListBlock) foNode);
+                endList((ListBlock) foNode);
             }
         } else if (foNode instanceof ListItemBody) {
             if (bStart) {
-                startListBody();
+                startListBody(null);
             } else {
-                endListBody();
+                endListBody(null);
             }
         } else if (foNode instanceof ListItem) {
             if (bStart) {
-                startListItem( (ListItem) foNode);
+                startListItem((ListItem) foNode);
             } else {
-                endListItem( (ListItem) foNode);
+                endListItem((ListItem) foNode);
             }
         } else if (foNode instanceof ListItemLabel) {
             if (bStart) {
-                startListLabel();
+                startListLabel(null);
             } else {
-                endListLabel();
+                endListLabel(null);
             }
         } else if (foNode instanceof Table) {
             if (bStart) {
-                startTable( (Table) foNode);
+                startTable((Table) foNode);
             } else {
-                endTable( (Table) foNode);
+                endTable((Table) foNode);
             }
         } else if (foNode instanceof TableHeader) {
             if (bStart) {
-                startHeader( (TableHeader) foNode);
+                startHeader((TableHeader) foNode);
             } else {
-                endHeader( (TableHeader) foNode);
+                endHeader((TableHeader) foNode);
             }
         } else if (foNode instanceof TableFooter) {
             if (bStart) {
-                startFooter( (TableFooter) foNode);
+                startFooter((TableFooter) foNode);
             } else {
-                endFooter( (TableFooter) foNode);
+                endFooter((TableFooter) foNode);
             }
         } else if (foNode instanceof TableBody) {
             if (bStart) {
-                startBody( (TableBody) foNode);
+                startBody((TableBody) foNode);
             } else {
-                endBody( (TableBody) foNode);
+                endBody((TableBody) foNode);
             }
         } else if (foNode instanceof TableColumn) {
             if (bStart) {
-                startColumn( (TableColumn) foNode);
+                startColumn((TableColumn) foNode);
             } else {
-                endColumn( (TableColumn) foNode);
+                endColumn((TableColumn) foNode);
             }
         } else if (foNode instanceof TableRow) {
             if (bStart) {
-                startRow( (TableRow) foNode);
+                startRow((TableRow) foNode);
             } else {
-                endRow( (TableRow) foNode);
+                endRow((TableRow) foNode);
             }
         } else if (foNode instanceof TableCell) {
             if (bStart) {
-                startCell( (TableCell) foNode);
+                startCell((TableCell) foNode);
             } else {
-                endCell( (TableCell) foNode);
+                endCell((TableCell) foNode);
             }
         } else if (foNode instanceof Leader) {
             if (bStart) {
-                leader((Leader) foNode);
+                startLeader((Leader) foNode);
             }
         } else if (foNode instanceof PageNumberCitation) {
             if (bStart) {
@@ -1624,6 +1693,12 @@ public class RTFHandler extends FOEventHandler {
             } else {
                 endPageNumberCitation((PageNumberCitation) foNode);
             }
+        } else if (foNode instanceof PageNumberCitationLast) {
+            if (bStart) {
+                startPageNumberCitationLast((PageNumberCitationLast) foNode);
+            } else {
+                endPageNumberCitationLast((PageNumberCitationLast) foNode);
+            }
         } else {
             RTFEventProducer eventProducer = RTFEventProducer.Provider.get(
                     getUserAgent().getEventBroadcaster());
@@ -1631,6 +1706,19 @@ public class RTFHandler extends FOEventHandler {
         }
     }
 
+    /**
+     * Closes any mismatched tags that are detected in the RTF structure.
+     * @param containerClass The class representing the tag to close.
+     * @return Determines whether the tag mismatch has been handled.
+     */
+    public boolean endContainer(Class containerClass) {
+        if (containerClass == RtfTableRow.class) {
+            endRow(null);
+            return true;
+        }
+        return false;
+    }
+
     /**
      * Calls the event handlers for the passed FONode and all its elements.
      *
@@ -1644,7 +1732,7 @@ public class RTFHandler extends FOEventHandler {
 
             Region regionBefore = pagemaster.getRegion(Constants.FO_REGION_BEFORE);
             if (regionBefore != null) {
-                FONode staticBefore = (FONode) pageSequence.getFlowMap().get(
+                FONode staticBefore = pageSequence.getFlowMap().get(
                         regionBefore.getRegionName());
                 if (staticBefore != null) {
                     recurseFONode(staticBefore);
@@ -1652,7 +1740,7 @@ public class RTFHandler extends FOEventHandler {
             }
             Region regionAfter = pagemaster.getRegion(Constants.FO_REGION_AFTER);
             if (regionAfter != null) {
-                FONode staticAfter = (FONode) pageSequence.getFlowMap().get(
+                FONode staticAfter = pageSequence.getFlowMap().get(
                         regionAfter.getRegionName());
                 if (staticAfter != null) {
                     recurseFONode(staticAfter);
@@ -1660,7 +1748,7 @@ public class RTFHandler extends FOEventHandler {
             }
 
 
-            recurseFONode( pageSequence.getMainFlow() );
+            recurseFONode(pageSequence.getMainFlow());
         } else if (foNode instanceof Table) {
             Table table = (Table) foNode;
 
@@ -1670,7 +1758,7 @@ public class RTFHandler extends FOEventHandler {
               prepareTable(table);
 
               for (Iterator it = table.getColumns().iterator(); it.hasNext();) {
-                  recurseFONode( (FONode) it.next() );
+                  recurseFONode((FONode) it.next());
               }
             } else {
                 //TODO Implement implicit column setup handling!
@@ -1681,17 +1769,17 @@ public class RTFHandler extends FOEventHandler {
 
             //recurse table-header
             if (table.getTableHeader() != null) {
-                recurseFONode( table.getTableHeader() );
+                recurseFONode(table.getTableHeader());
             }
 
             //recurse table-footer
             if (table.getTableFooter() != null) {
-                recurseFONode( table.getTableFooter() );
+                recurseFONode(table.getTableFooter());
             }
 
             if (foNode.getChildNodes() != null) {
                 for (Iterator it = foNode.getChildNodes(); it.hasNext();) {
-                    recurseFONode( (FONode) it.next() );
+                    recurseFONode((FONode) it.next());
                 }
             }
         } else if (foNode instanceof ListItem) {
