@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ParagraphKeeptogetherContext.java 1036179 2010-11-17 19:45:27Z spepping $ */
+/* $Id: ParagraphKeeptogetherContext.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -35,8 +35,8 @@ package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
 public final class ParagraphKeeptogetherContext {
 
-    private static int paraKeepTogetherOpen = 0;
-    private static boolean paraResetProperties = false;
+    private static int paraKeepTogetherOpen;
+    private static boolean paraResetProperties;
 
     private ParagraphKeeptogetherContext() {
     }
