@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfBookmarkContainerImpl.java 1297404 2012-03-06 10:17:54Z vhennebert $ */
+/* $Id: RtfBookmarkContainerImpl.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -42,7 +42,7 @@ public class RtfBookmarkContainerImpl extends RtfContainer implements IRtfBookma
     //////////////////////////////////////////////////
 
     /** Rtf bookmark */
-    private RtfBookmark mBookmark = null;
+    private RtfBookmark mBookmark;
 
 
     //////////////////////////////////////////////////
@@ -58,8 +58,8 @@ public class RtfBookmarkContainerImpl extends RtfContainer implements IRtfBookma
      *
      * @exception IOException On error
      */
-    RtfBookmarkContainerImpl (RtfContainer parent, Writer w) throws IOException {
-        super (parent, w, null);
+    RtfBookmarkContainerImpl(RtfContainer parent, Writer w) throws IOException {
+        super(parent, w, null);
     }
 
     /**
@@ -72,9 +72,9 @@ public class RtfBookmarkContainerImpl extends RtfContainer implements IRtfBookma
      *
      * @exception IOException On error
      */
-    RtfBookmarkContainerImpl (RtfContainer parent, Writer w, RtfAttributes attr) throws IOException
+    RtfBookmarkContainerImpl(RtfContainer parent, Writer w, RtfAttributes attr) throws IOException
     {
-        super (parent, w, attr);
+        super(parent, w, attr);
     }
 
 
@@ -91,12 +91,12 @@ public class RtfBookmarkContainerImpl extends RtfContainer implements IRtfBookma
      *
      * @throws IOException On eror
      */
-    public RtfBookmark newBookmark (String bookmark) throws IOException {
+    public RtfBookmark newBookmark(String bookmark) throws IOException {
         if (mBookmark != null) {
-            mBookmark.close ();
+            mBookmark.close();
         }
 
-        mBookmark = new RtfBookmark (this, writer, bookmark);
+        mBookmark = new RtfBookmark(this, writer, bookmark);
 
         return mBookmark;
     }
