@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfExternalGraphic.java 1297284 2012-03-05 23:29:29Z gadams $ */
+/* $Id: RtfExternalGraphic.java 1681384 2015-05-23 21:07:13Z adelmelle $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -226,18 +226,13 @@ public class RtfExternalGraphic extends RtfElement {
     /**
      * The url of the image
      */
-    protected URL url = null;
+    protected URL url;
 
     /**
      * The height of the image (in pixels)
      */
     protected int height = -1;
 
-    /**
-     * The desired percent value of the height
-     */
-    protected int heightPercent = -1;
-
     /**
      * The desired height (in twips)
      */
@@ -246,18 +241,13 @@ public class RtfExternalGraphic extends RtfElement {
     /**
      * Flag whether the desired height is a percentage
      */
-    protected boolean perCentH = false;
+    protected boolean perCentH;
 
     /**
      * The width of the image (in pixels)
      */
     protected int width = -1;
 
-    /**
-     * The desired percent value of the width
-     */
-    protected int widthPercent = -1;
-
     /**
      * The desired width (in twips)
      */
@@ -266,12 +256,12 @@ public class RtfExternalGraphic extends RtfElement {
     /**
      * Flag whether the desired width is a percentage
      */
-    protected boolean perCentW = false;
+    protected boolean perCentW;
 
     /**
      * Flag whether the image size shall be adjusted
      */
-    protected boolean scaleUniform = false;
+    protected boolean scaleUniform;
 
     /** cropping on left/top/right/bottom edges for \piccrop*N */
     private int[] cropValues = new int[4];
@@ -282,7 +272,7 @@ public class RtfExternalGraphic extends RtfElement {
      protected int graphicCompressionRate = 80;
 
      /** The image data */
-     private byte[] imagedata = null;
+     private byte[] imagedata;
 
      /** The image format */
      private FormatBase imageformat;
@@ -301,7 +291,7 @@ public class RtfExternalGraphic extends RtfElement {
      * @throws IOException for I/O problems
      */
     public RtfExternalGraphic(RtfContainer container, Writer writer) throws IOException {
-        super (container, writer);
+        super(container, writer);
     }
 
     /**
@@ -314,7 +304,7 @@ public class RtfExternalGraphic extends RtfElement {
      */
     public RtfExternalGraphic(RtfContainer container, Writer writer,
     RtfAttributes attributes) throws IOException {
-        super (container, writer, attributes);
+        super(container, writer, attributes);
     }
 
 
@@ -443,7 +433,7 @@ public class RtfExternalGraphic extends RtfElement {
         writeGroupMark(false);
     }
 
-    private void computeImageSize () {
+    private void computeImageSize() {
         if (imageformat.getType() == ImageConstants.I_PNG) {
             width = ImageUtil.getIntFromByteArray(imagedata, 16, 4, true);
             height = ImageUtil.getIntFromByteArray(imagedata, 20, 4, true);
@@ -475,19 +465,19 @@ public class RtfExternalGraphic extends RtfElement {
             int i = 0;
 
             i = ImageUtil.getIntFromByteArray(imagedata, 151, 4, false);
-            if (i != 0 ) {
+            if (i != 0) {
                 width = i;
             }
 
             i = ImageUtil.getIntFromByteArray(imagedata, 155, 4, false);
-            if (i != 0 ) {
+            if (i != 0) {
                 height = i;
             }
 
         }
     }
 
-    private void writeSizeInfo () throws IOException {
+    private void writeSizeInfo() throws IOException {
         // Set image size
         if (width != -1) {
             writeControlWord("picw" + width);
@@ -641,10 +631,10 @@ public class RtfExternalGraphic extends RtfElement {
     public void setURL(String urlString) throws IOException {
         URL tmpUrl = null;
         try {
-            tmpUrl = new URL (urlString);
+            tmpUrl = new URL(urlString);
         } catch (MalformedURLException e) {
             try {
-                tmpUrl = new File (urlString).toURI().toURL ();
+                tmpUrl = new File(urlString).toURI().toURL();
             } catch (MalformedURLException ee) {
                 throw new ExternalGraphicException("The attribute 'src' of "
                         + "<fo:external-graphic> has a invalid value: '"
@@ -658,7 +648,7 @@ public class RtfExternalGraphic extends RtfElement {
      * Gets  the compression rate for the image in percent.
      * @return Compression rate
      */
-    public int getCompressionRate () {
+    public int getCompressionRate() {
         return graphicCompressionRate;
     }
 
@@ -668,7 +658,7 @@ public class RtfExternalGraphic extends RtfElement {
      * @param percent Compression rate
      * @return true if the compression rate is valid (0..100), false if invalid
      */
-    public boolean setCompressionRate (int percent) {
+    public boolean setCompressionRate(int percent) {
         if (percent < 1 || percent > 100) {
             return false;
         }
