@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfExtraRowSet.java 1297284 2012-03-05 23:29:29Z gadams $ */
+/* $Id: RtfExtraRowSet.java 1616312 2014-08-06 19:19:31Z gadams $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -58,7 +58,7 @@ public class RtfExtraRowSet extends RtfContainer {
     /** Parent table context
      * (added by Boris Poudérous on july 2002 in order to process nested tables)
      */
-    private ITableColumnsInfo parentITableColumnsInfo = null;
+    private ITableColumnsInfo parentITableColumnsInfo;
 
     /** While a top-level RtfTableRow is being rendered, we build a list of
      *  RtfTableCells that must be rendered in extra rows.
@@ -107,13 +107,18 @@ public class RtfExtraRowSet extends RtfContainer {
 
         public int hashCode() {
             int hc = super.hashCode();
-            hc ^= ( hc * 11 ) + xOffset;
-            hc ^= ( hc * 19 ) + rowIndex;
+            hc ^= (hc * 11) + xOffset;
+            hc ^= (hc * 19) + rowIndex;
             return hc;
         }
 
         public boolean equals(Object o) {
-            return o != null && this.compareTo(o) == 0;
+            if (o instanceof PositionedCell) {
+                PositionedCell pc = (PositionedCell) o;
+                return (pc.rowIndex == rowIndex) && (pc.xOffset == xOffset);
+            } else {
+                return false;
+            }
         }
     }
 
@@ -313,7 +318,7 @@ public class RtfExtraRowSet extends RtfContainer {
       *
       * @param parentITableColumnsInfo table context to set
       */
-     public void setParentITableColumnsInfo (ITableColumnsInfo parentITableColumnsInfo) {
+     public void setParentITableColumnsInfo(ITableColumnsInfo parentITableColumnsInfo) {
        this.parentITableColumnsInfo = parentITableColumnsInfo;
      }
      /** - end - */
