@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfListItem.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: RtfListItem.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -26,14 +26,14 @@ package org.apache.fop.render.rtf.rtflib.rtfdoc;
  * the FOP project.
  */
 
-import java.io.Writer;
 import java.io.IOException;
+import java.io.Writer;
 
-import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTextrun;
-
-/**  Model of an RTF list item, which can contain RTF paragraphs
- *  @author Bertrand Delacretaz bdelacretaz@codeconsult.ch
- *  @author Andreas Putz a.putz@skynamics.com
+/**
+ * <p>Model of an RTF list item, which can contain RTF paragraphs.</p>
+ *
+ * <p>This work was authored by Bertrand Delacretaz (bdelacretaz@codeconsult.ch)
+ * and Andreas Putz (a.putz@skynamics.com).</p>
  */
 public class RtfListItem extends RtfContainer
         implements IRtfTextrunContainer,
@@ -43,7 +43,7 @@ public class RtfListItem extends RtfContainer
     private RtfList parentList;
     private RtfParagraph paragraph;
     private RtfListStyle listStyle;
-    private int number = 0;
+    private int number;
 
     /**
      * special RtfParagraph that writes list item setup code before its content
