@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfParagraph.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: RtfParagraph.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -26,14 +26,18 @@ package org.apache.fop.render.rtf.rtflib.rtfdoc;
  * the FOP project.
  */
 
-import java.io.Writer;
 import java.io.IOException;
+import java.io.Writer;
 import java.util.List;
 
-/**  Model of an RTF paragraph, which can contain RTF text elements.
- *  @author Bertrand Delacretaz bdelacretaz@codeconsult.ch
- *  @author Andreas Putz a.putz@skynamics.com
- *  @author Boris Poudérous, boris.pouderous@free.fr
+import org.apache.fop.apps.FOPException;
+
+/**
+ * <p>Model of an RTF paragraph, which can contain RTF text elements.</p>
+ *
+ * <p>This work was authored by Bertrand Delacretaz (bdelacretaz@codeconsult.ch),
+ * Andreas Putz (a.putz@skynamics.com), and
+ * Boris Poudérous (boris.pouderous@free.fr).</p>
  */
 
 public class RtfParagraph extends RtfBookmarkContainerImpl
@@ -46,8 +50,8 @@ implements IRtfTextContainer, IRtfPageBreakContainer, IRtfHyperLinkContainer,
     private RtfPageNumber pageNumber;
     private RtfPageNumberCitation pageNumberCitation;
     // Above line added by Boris POUDEROUS on 2002/07/09
-    private boolean keepn = false;
-    private boolean resetProperties = false;
+    private boolean keepn;
+    private boolean resetProperties;
 
     /* needed for importing Rtf into FrameMaker
        FrameMaker is not as forgiving as word in rtf
@@ -55,7 +59,7 @@ implements IRtfTextContainer, IRtfPageBreakContainer, IRtfHyperLinkContainer,
            after a table.  /pard is probably needed in other places
            also, this is just a hack to make FrameMaker import Jfor rtf
            correctly */
-    private boolean writeForBreak = false;
+    private boolean writeForBreak;
 
     /** Set of attributes that must be copied at the start of a paragraph */
     private static final String[] PARA_ATTRIBUTES = {"intbl"};
@@ -91,12 +95,17 @@ implements IRtfTextContainer, IRtfPageBreakContainer, IRtfHyperLinkContainer,
     /**
      * IRtfTextContainer requirement: return a copy of our attributes
      * @return a copy of this paragraphs attributes
+     * @throws FOPException if attributes cannot be cloned
      */
-    public RtfAttributes getTextContainerAttributes() {
+    public RtfAttributes getTextContainerAttributes() throws FOPException {
         if (attrib == null) {
             return null;
         }
-        return (RtfAttributes)this.attrib.clone();
+        try {
+            return (RtfAttributes)this.attrib.clone();
+        } catch (CloneNotSupportedException e) {
+            throw new FOPException(e);
+        }
     }
 
     /**
