@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfSpaceManager.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: RtfSpaceManager.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -40,7 +40,7 @@ public class RtfSpaceManager {
      * inside, then the next block should has increased value of space-before
      * property.
      */
-    private int accumulatedSpace = 0;
+    private int accumulatedSpace;
 
     /**
      * Construct a newly allocated <code>RtfSpaceManager</code> object.
