@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,9 @@
  * limitations under the License.
  */
 
-/* $Id: RtfStyleSheetTable.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: RtfStyleSheetTable.java 1617052 2014-08-10 06:55:01Z gadams $ */
 
+package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
 /*
  * This file is part of the RTF library of the FOP project, which was originally
@@ -25,19 +26,18 @@
  * the FOP project.
  */
 
-package org.apache.fop.render.rtf.rtflib.rtfdoc;
-
-import java.util.Vector;
-import java.util.Hashtable;
 import java.io.IOException;
+import java.util.Hashtable;
 import java.util.Iterator;
+import java.util.Vector;
 
 /**
- * Singelton of the RTF style sheet table.
- * This class belongs to the <jfor:stylesheet> tag processing.
- * @author <a href="mailto:a.putz@skynamics.com">Andreas Putz</a>
+ * <p>Singelton of the RTF style sheet table.
+ * This class belongs to the <jfor:stylesheet> tag processing.</p>
+ *
+ * <p>This work was authored by Andreas Putz (a.putz@skynamics.com).</p>
  */
-public class RtfStyleSheetTable {
+public final class RtfStyleSheetTable {
     //////////////////////////////////////////////////
     // @@ Symbolic constants
     //////////////////////////////////////////////////
@@ -60,7 +60,7 @@ public class RtfStyleSheetTable {
     //////////////////////////////////////////////////
 
     /** Singelton instance */
-    private static RtfStyleSheetTable instance = null;
+    private static RtfStyleSheetTable instance = new RtfStyleSheetTable();
 
 
     //////////////////////////////////////////////////
@@ -69,13 +69,13 @@ public class RtfStyleSheetTable {
 
 
     /** Table of styles */
-    private Hashtable styles = null;
+    private Hashtable styles;
 
     /** Used, style attributes to this vector */
-    private Hashtable attrTable = null;
+    private Hashtable attrTable;
 
     /** Used, style names to this vector */
-    private Vector nameTable = null;
+    private Vector nameTable;
 
     /** Default style */
     private String defaultStyleName = STANDARD_STYLE;
@@ -88,10 +88,10 @@ public class RtfStyleSheetTable {
     /**
      * Constructor.
      */
-    private RtfStyleSheetTable () {
-        styles = new Hashtable ();
-        attrTable = new Hashtable ();
-        nameTable = new Vector ();
+    private RtfStyleSheetTable() {
+        styles = new Hashtable();
+        attrTable = new Hashtable();
+        nameTable = new Vector();
     }
 
     /**
@@ -99,11 +99,7 @@ public class RtfStyleSheetTable {
      *
      * @return The instance of RtfStyleSheetTable
      */
-    public static RtfStyleSheetTable getInstance () {
-        if (instance == null) {
-            instance = new RtfStyleSheetTable ();
-        }
-
+    public static RtfStyleSheetTable getInstance() {
         return instance;
     }
 
@@ -116,7 +112,7 @@ public class RtfStyleSheetTable {
      * Sets the default style.
      * @param styleName Name of the default style, defined in the stylesheet
      */
-    public void setDefaultStyle (String styleName) {
+    public void setDefaultStyle(String styleName) {
         this.defaultStyleName = styleName;
     }
 
@@ -124,12 +120,12 @@ public class RtfStyleSheetTable {
      * Gets the name of the default style.
      * @return Default style name.
      */
-    public String getDefaultStyleName () {
-        if (attrTable.get (defaultStyleName) != null) {
+    public String getDefaultStyleName() {
+        if (attrTable.get(defaultStyleName) != null) {
             return defaultStyleName;
         }
 
-        if (attrTable.get (STANDARD_STYLE) != null) {
+        if (attrTable.get(STANDARD_STYLE) != null) {
             defaultStyleName = STANDARD_STYLE;
             return defaultStyleName;
         }
@@ -147,12 +143,12 @@ public class RtfStyleSheetTable {
      * @param name Name of style to add
      * @param attrs Rtf attributes which defines the style
      */
-    public void addStyle (String name, RtfAttributes attrs) {
-        nameTable.addElement (name);
+    public void addStyle(String name, RtfAttributes attrs) {
+        nameTable.addElement(name);
         if (attrs != null) {
-            attrTable.put (name, attrs);
+            attrTable.put(name, attrs);
         }
-        styles.put (name, new Integer (nameTable.size () - 1 + startIndex));
+        styles.put(name, new Integer(nameTable.size() - 1 + startIndex));
     }
 
     /**
@@ -161,17 +157,17 @@ public class RtfStyleSheetTable {
      * @param attr Default rtf attributes
      * @return Status value
      */
-    public int addStyleToAttributes (String name, RtfAttributes attr) {
+    public int addStyleToAttributes(String name, RtfAttributes attr) {
         // Sets status to ok
         int status = STATUS_OK;
 
         // Gets the style number from table
-        Integer style  = (Integer) styles.get (name);
+        Integer style  = (Integer) styles.get(name);
 
-        if (style == null && !name.equals (defaultStyleName)) {
+        if (style == null && !name.equals(defaultStyleName)) {
             // If style not found, and style was not the default style, try the default style
             name = defaultStyleName;
-            style = (Integer) styles.get (name);
+            style = (Integer) styles.get(name);
             // set status for default style setting
             status = STATUS_DEFAULT;
         }
@@ -182,20 +178,20 @@ public class RtfStyleSheetTable {
         }
 
         // Adds the attributes to default attributes, if not available in default attributes
-        attr.set ("cs", style.intValue ());
+        attr.set("cs", style.intValue());
 
-        Object o = attrTable.get (name);
+        Object o = attrTable.get(name);
         if (o != null) {
             RtfAttributes rtfAttr = (RtfAttributes) o;
 
-            for (Iterator names = rtfAttr.nameIterator (); names.hasNext ();) {
-                String attrName = (String) names.next ();
-                if (!attr.isSet (attrName)) {
-                    Integer i = (Integer) rtfAttr.getValue (attrName);
+            for (Iterator names = rtfAttr.nameIterator(); names.hasNext();) {
+                String attrName = (String) names.next();
+                if (!attr.isSet(attrName)) {
+                    Integer i = (Integer) rtfAttr.getValue(attrName);
                     if (i == null) {
-                        attr.set (attrName);
+                        attr.set(attrName);
                     } else {
-                        attr.set (attrName, i.intValue ());
+                        attr.set(attrName, i.intValue());
                     }
                 }
             }
@@ -208,29 +204,29 @@ public class RtfStyleSheetTable {
      * @param header Rtf header is the parent
      * @throws IOException On write error
      */
-    public void writeStyleSheet (RtfHeader header) throws IOException {
-        if (styles == null || styles.size () == 0) {
+    public void writeStyleSheet(RtfHeader header) throws IOException {
+        if (styles == null || styles.size() == 0) {
             return;
         }
-        header.writeGroupMark (true);
-        header.writeControlWord ("stylesheet");
+        header.writeGroupMark(true);
+        header.writeControlWord("stylesheet");
 
-        int number = nameTable.size ();
+        int number = nameTable.size();
         for (int i = 0; i < number; i++) {
-            String name = (String) nameTable.elementAt (i);
-            header.writeGroupMark (true);
-            header.writeControlWord ("*\\" + this.getRtfStyleReference (name));
+            String name = (String) nameTable.elementAt(i);
+            header.writeGroupMark(true);
+            header.writeControlWord("*\\" + this.getRtfStyleReference(name));
 
-            Object o = attrTable.get (name);
+            Object o = attrTable.get(name);
             if (o != null) {
-                header.writeAttributes ((RtfAttributes) o, RtfText.ATTR_NAMES);
-                header.writeAttributes ((RtfAttributes) o, RtfText.ALIGNMENT);
+                header.writeAttributes((RtfAttributes) o, RtfText.ATTR_NAMES);
+                header.writeAttributes((RtfAttributes) o, RtfText.ALIGNMENT);
             }
 
-            header.write (name + ";");
-            header.writeGroupMark (false);
+            header.write(name + ";");
+            header.writeGroupMark(false);
         }
-        header.writeGroupMark (false);
+        header.writeGroupMark(false);
     }
 
     /**
@@ -238,7 +234,7 @@ public class RtfStyleSheetTable {
      * @param name Name of Style
      * @return Rtf attribute of the style reference
      */
-    private String getRtfStyleReference (String name) {
-        return "cs" + styles.get (name).toString ();
+    private String getRtfStyleReference(String name) {
+        return "cs" + styles.get(name).toString();
     }
-}
\ No newline at end of file
+}
