@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfTableCell.java 693742 2008-09-10 09:00:16Z jeremias $ */
+/* $Id: RtfTableCell.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -30,8 +30,10 @@ import java.io.IOException;
 import java.io.Writer;
 import java.util.Iterator;
 
-/**  A cell in an RTF table, container for paragraphs, lists, etc.
- *  @author Bertrand Delacretaz bdelacretaz@codeconsult.ch
+/**
+ * <p>A cell in an RTF table, container for paragraphs, lists, etc.</p>
+ *
+ * <p>This work was authored by Bertrand Delacretaz (bdelacretaz@codeconsult.ch).</p>
  */
 
 public class RtfTableCell
@@ -46,6 +48,11 @@ public class RtfTableCell
     private boolean setCenter;
     private boolean setRight;
     private int id;
+    private RtfParagraphBreak lastBreak;
+    private int lastBreakDepth = Integer.MIN_VALUE;
+
+    private static final String TABLE_CELL_PARAGRAPH = "cell";
+    private static final String TABLE_CELL_NESTED_PARAGRAPH = "nestcell";
 
     /** default cell width (in twips ??) */
     public static final int DEFAULT_CELL_WIDTH = 2000;
@@ -181,8 +188,8 @@ public class RtfTableCell
     }
 
     /** used by RtfTableRow to write the <celldef> cell definition control words
-     *  @param widthOffset sum of the widths of preceeding cells in same row
-     *  @return widthOffset + width of this cell
+     *  @param offset sum of the widths of preceeding cells in same row
+     *  @return offset + width of this cell
      */
     int writeCellDef(int offset) throws IOException {
         /*
@@ -213,12 +220,12 @@ public class RtfTableCell
          * Added by Boris POUDEROUS on 2002/06/26
          */
         // Cell background color processing :
-        writeAttributes (attrib, ITableAttributes.CELL_COLOR);
+        writeAttributes(attrib, ITableAttributes.CELL_COLOR);
         /** - end - */
 
-        writeAttributes (attrib, ITableAttributes.ATTRIB_CELL_PADDING);
-        writeAttributes (attrib, ITableAttributes.CELL_BORDER);
-        writeAttributes (attrib, IBorderAttributes.BORDERS);
+        writeAttributes(attrib, ITableAttributes.ATTRIB_CELL_PADDING);
+        writeAttributes(attrib, ITableAttributes.CELL_BORDER);
+        writeAttributes(attrib, IBorderAttributes.BORDERS);
 
         // determine cell width
         int iCurrentWidth = this.cellWidth;
@@ -263,7 +270,7 @@ public class RtfTableCell
         } else {
             writeControlWord("trql");
         }
-        writeAttributes (attrib, ITableAttributes.CELL_VERT_ALIGN);
+        writeAttributes(attrib, ITableAttributes.CELL_VERT_ALIGN);
 
         writeControlWord("cellx" + xPos);
 
@@ -310,7 +317,9 @@ public class RtfTableCell
 
         if (getRow().getTable().isNestedTable()) {
             //nested table
-            writeControlWordNS("nestcell");
+            if (lastBreak == null) {
+                writeControlWordNS("nestcell");
+            }
             writeGroupMark(true);
             writeControlWord("nonesttables");
             writeControlWord("par");
@@ -350,7 +359,9 @@ public class RtfTableCell
                 //writeControlWord("par");
             }
 
-            writeControlWord("cell");
+            if (lastBreak == null) {
+                writeControlWord("cell");
+            }
         }
     }
 
@@ -533,4 +544,38 @@ public class RtfTableCell
 
         return null;
     }
+
+    /**
+     * The table cell decides whether or not a newly added paragraph break
+     * will be used to write the cell-end control word.
+     * For nested tables it is not necessary.
+     *
+     * @param parBreak the paragraph break element
+     * @param breakDepth The depth is necessary for picking the correct break element.
+     * If it is deeper inside the whole cell it will be used, and if there is something on
+     * the same level (depth) it is also set because the method is called for all breaks
+     * in the correct order.
+     */
+    public void setLastParagraph(RtfParagraphBreak parBreak, int breakDepth) {
+        if (parBreak != null && breakDepth >= lastBreakDepth) {
+            lastBreak = parBreak;
+            lastBreakDepth = breakDepth;
+        }
+    }
+
+    /**
+     * The last paragraph break was just stored before,
+     * now the control word is really switched
+     */
+    public void finish() {
+      //If it is nested and contains another table do not set it
+      if (getRow().getTable().isNestedTable() && table != null) {
+          lastBreak = null;
+      } else if (lastBreak != null) {
+              lastBreak.switchControlWord(
+                                          getRow().getTable().isNestedTable()
+                                          ? TABLE_CELL_NESTED_PARAGRAPH
+                                          : TABLE_CELL_PARAGRAPH);
+      }
+    }
 }
