@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfTemplate.java 985537 2010-08-14 17:17:00Z jeremias $ */
+/* $Id: RtfTemplate.java 1617052 2014-08-10 06:55:01Z gadams $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -36,14 +36,14 @@ import java.io.IOException;
 public final class RtfTemplate  {
 
     /** Singelton instance */
-    private static RtfTemplate instance = null;
+    private static RtfTemplate instance = new RtfTemplate();
 
-    private String templateFilePath = null;
+    private String templateFilePath;
 
     /**
      * Constructor.
      */
-    private RtfTemplate () {
+    private RtfTemplate() {
 
     }
 
@@ -53,11 +53,7 @@ public final class RtfTemplate  {
      *
      * @return The instance of RtfTemplate
      */
-    public static RtfTemplate getInstance () {
-        if (instance == null) {
-            instance = new RtfTemplate();
-        }
-
+    public static RtfTemplate getInstance() {
         return instance;
     }
 
@@ -81,19 +77,19 @@ public final class RtfTemplate  {
      * @param header Rtf header is the parent
      * @throws IOException On write error
      */
-    public void writeTemplate (RtfHeader header) throws IOException {
+    public void writeTemplate(RtfHeader header) throws IOException {
         if (templateFilePath == null || templateFilePath.length() == 0) {
             return;
         }
 
-        header.writeGroupMark (true);
-        header.writeControlWord ("template");
+        header.writeGroupMark(true);
+        header.writeControlWord("template");
         header.writeRtfString(this.templateFilePath);
-        header.writeGroupMark (false);
+        header.writeGroupMark(false);
 
-        header.writeGroupMark (true);
-        header.writeControlWord ("linkstyles");
-        header.writeGroupMark (false);
+        header.writeGroupMark(true);
+        header.writeControlWord("linkstyles");
+        header.writeGroupMark(false);
     }
 }
 
