@@ -15,8 +15,9 @@
  * limitations under the License.
  */
 
-/* $Id: RtfText.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: RtfText.java 1479969 2013-05-07 16:23:13Z vhennebert $ */
 
+package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
 /*
  * This file is part of the RTF library of the FOP project, which was originally
@@ -25,13 +26,15 @@
  * the FOP project.
  */
 
-package org.apache.fop.render.rtf.rtflib.rtfdoc;
-
 import java.io.IOException;
 import java.io.Writer;
 
-/**  Model of a text run (a piece of text with attributes) in an RTF document
- *  @author Bertrand Delacretaz bdelacretaz@codeconsult.ch
+import org.apache.fop.apps.FOPException;
+
+/**
+ * <p>Model of a text run (a piece of text with attributes) in an RTF document.</p>
+ *
+ * <p>This work was authored by Bertrand Delacretaz (bdelacretaz@codeconsult.ch).</p>
  */
 
 public class RtfText extends RtfElement {
@@ -239,12 +242,18 @@ public class RtfText extends RtfElement {
     }
 
     /** IRtfTextContainer requirement:
-     * @return a copy of our attributes */
-    public RtfAttributes getTextContainerAttributes() {
+     * @return a copy of our attributes
+     * @throws FOPException if attributes cannot be cloned
+     */
+    public RtfAttributes getTextContainerAttributes() throws FOPException {
         if (attrib == null) {
             return null;
         }
-        return (RtfAttributes)this.attrib.clone();
+        try {
+            return (RtfAttributes)this.attrib.clone();
+        } catch (CloneNotSupportedException e) {
+            throw new FOPException(e);
+        }
     }
 
     /** direct access to our text */
@@ -263,7 +272,7 @@ public class RtfText extends RtfElement {
      * @return true    If m_text is null\n
      *         false   m_text is set
      */
-    public boolean isEmpty () {
+    public boolean isEmpty() {
         return text == null || text.trim().length() == 0;
     }
 
@@ -275,9 +284,9 @@ public class RtfText extends RtfElement {
      * @return true    If m_text is character 160\n
      *         false   m_text is not a nbsp
      */
-    public boolean isNbsp () {
-        if (!isEmpty ()) {
-            if (text.trim ().length () == 1 && text.charAt (0) == CHAR_NBSP) {
+    public boolean isNbsp() {
+        if (!isEmpty()) {
+            if (text.trim().length() == 1 && text.charAt(0) == CHAR_NBSP) {
                 return true;
             }
         }
