@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfTextrun.java 1310948 2012-04-08 03:57:07Z gadams $ */
+/* $Id: RtfTextrun.java 1616590 2014-08-07 20:27:59Z gadams $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -49,7 +49,7 @@ public class RtfTextrun extends RtfContainer {
     /** Constant for a odd page break */
     public static final int BREAK_ODD_PAGE = 4;
 
-    private boolean bSuppressLastPar = false;
+    private boolean bSuppressLastPar;
     private RtfListItem rtfListItem;
 
     /**
@@ -331,8 +331,8 @@ public class RtfTextrun extends RtfContainer {
      * @throws IOException for I/O problems
      */
     public void addBookmark(String id) throws IOException {
-       if (id != "") {
-            // if id is not empty, add boormark
+       if (id.length() > 0) {
+            // if id is not empty, add bookmark
            new RtfBookmark(this, writer, id);
        }
     }
