@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TXTRenderer.java 1326144 2012-04-14 16:48:59Z gadams $ */
+/* $Id: TXTRenderer.java 1617052 2014-08-10 06:55:01Z gadams $ */
 
 package org.apache.fop.render.txt;
 
@@ -91,13 +91,13 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
      * Every line except the last line on a page (which will end with
      * pageEnding) will be terminated with this string.
      */
-    private final String lineEnding = "\r\n";
+    private static final String LINE_ENDING = "\r\n";
 
     /** Every page except the last one will end with this string. */
-    private final String pageEnding = "\f";
+    private static final String PAGE_ENDING = "\f";
 
     /** Equals true, if current page is first. */
-    private boolean firstPage = false;
+    private boolean firstPage;
 
     /** Manager for storing border's information. */
     private BorderManager bm;
@@ -211,7 +211,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
         if (firstPage) {
             firstPage = false;
         } else {
-            currentStream.add(pageEnding);
+            currentStream.add(PAGE_ENDING);
         }
 
         Rectangle2D bounds = page.getViewArea();
@@ -286,7 +286,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
                 currentStream.add(outr.toString());
             }
             if (row < pageHeight) {
-                currentStream.add(lineEnding);
+                currentStream.add(LINE_ENDING);
             }
         }
     }
@@ -501,8 +501,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
     /**
      * {@inheritDoc}
      */
-    protected void drawBorderLine(                               // CSOK: ParameterNumber
-            float x1, float y1, float x2, float y2,
+    protected void drawBorderLine(float x1, float y1, float x2, float y2,
             boolean horz, boolean startOrBefore, int style, Color col) {
 
         int borderHeight = bm.getHeight();
@@ -583,6 +582,14 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
         currentState.pop();
     }
 
+    /** {@inheritDoc} */
+    protected void startLayer(String layer) {
+    }
+
+    /** {@inheritDoc} */
+    protected void endLayer() {
+    }
+
     /** {@inheritDoc} */
     protected void concatenateTransformationMatrix(AffineTransform at) {
         currentState.push(new CTM(UnitConv.ptToMpt(at)));
