@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractFOPTextElementBridge.java 1297284 2012-03-05 23:29:29Z gadams $ */
+/* $Id: AbstractFOPTextElementBridge.java 1664570 2015-03-06 09:41:07Z lbernardo $ */
 
 package org.apache.fop.svg;
 
@@ -23,9 +23,9 @@ import org.w3c.dom.Element;
 
 import org.apache.batik.bridge.BridgeContext;
 import org.apache.batik.bridge.SVGTextElementBridge;
+import org.apache.batik.bridge.TextNode;
+import org.apache.batik.bridge.TextPainter;
 import org.apache.batik.gvt.GraphicsNode;
-import org.apache.batik.gvt.TextNode;
-import org.apache.batik.gvt.TextPainter;
 
 /**
  * <p>Bridge class for the &lt;text> element.
