@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractFOPTranscoder.java 1296526 2012-03-03 00:18:45Z gadams $ */
+/* $Id: AbstractFOPTranscoder.java 1664570 2015-03-06 09:41:07Z lbernardo $ */
 
 package org.apache.fop.svg;
 
@@ -32,12 +32,11 @@ import org.apache.avalon.framework.configuration.Configurable;
 import org.apache.avalon.framework.configuration.Configuration;
 import org.apache.avalon.framework.configuration.ConfigurationException;
 import org.apache.avalon.framework.configuration.DefaultConfiguration;
-
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.impl.SimpleLog;
 
-import org.apache.batik.bridge.UserAgent;
-import org.apache.batik.dom.svg.SVGDOMImplementation;
+import org.apache.batik.anim.dom.SVGDOMImplementation;
+import org.apache.batik.bridge.FontFamilyResolver;
 import org.apache.batik.dom.util.DocumentFactory;
 import org.apache.batik.transcoder.ErrorHandler;
 import org.apache.batik.transcoder.SVGAbstractTranscoder;
@@ -56,6 +55,8 @@ import org.apache.xmlgraphics.image.loader.ImageSessionContext;
 import org.apache.xmlgraphics.image.loader.impl.AbstractImageSessionContext;
 import org.apache.xmlgraphics.util.UnitConv;
 
+import org.apache.fop.svg.font.FOPFontFamilyResolver;
+
 /**
  * This is the common base class of all of FOP's transcoders.
  */
@@ -86,14 +87,9 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder implem
     /** The value to turn off text stroking. */
     public static final Boolean VALUE_FORMAT_OFF = Boolean.FALSE;
 
-    /**
-     * The user agent dedicated to this Transcoder.
-     */
-    protected UserAgent userAgent = createUserAgent();
-
     private Log logger;
     private EntityResolver resolver;
-    private Configuration cfg = null;
+    private Configuration cfg;
     private ImageManager imageManager;
     private ImageSessionContext imageSessionContext;
 
@@ -113,7 +109,7 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder implem
      * this method if you need non-default behaviour.
      * @return UserAgent the newly created user agent
      */
-    protected UserAgent createUserAgent() {
+    protected FOPTranscoderUserAgent createUserAgent() {
         return new FOPTranscoderUserAgent();
     }
 
@@ -331,6 +327,8 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder implem
      */
     protected class FOPTranscoderUserAgent extends SVGAbstractTranscoderUserAgent {
 
+        private FOPFontFamilyResolver fontFamilyResolver;
+
         /**
          * Displays the specified error message using the {@link ErrorHandler}.
          * @param message the message to display
@@ -386,6 +384,15 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder implem
             return "print";
         }
 
+        public void setFontFamilyResolver(FOPFontFamilyResolver resolver) {
+            fontFamilyResolver = resolver;
+        }
+
+        @Override
+        public FontFamilyResolver getFontFamilyResolver() {
+            return fontFamilyResolver;
+        }
+
     }
 
 }
