@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,48 +15,50 @@
  * limitations under the License.
  */
 
-/* $Id: PDFDocumentGraphics2D.java 557337 2007-07-18 17:37:14Z adelmelle $ */
+/* $Id: PDFDocumentGraphics2D.java 1661887 2015-02-24 11:23:44Z ssteiner $ */
 
 package org.apache.fop.svg;
 
-import org.apache.fop.Version;
-import org.apache.fop.pdf.PDFDocument;
-import org.apache.fop.pdf.PDFFilterList;
-import org.apache.fop.pdf.PDFPage;
-import org.apache.fop.pdf.PDFStream;
-import org.apache.fop.pdf.PDFState;
-import org.apache.fop.pdf.PDFNumber;
-import org.apache.fop.pdf.PDFResources;
-import org.apache.fop.pdf.PDFColor;
-import org.apache.fop.pdf.PDFAnnotList;
-import org.apache.fop.fonts.FontInfo;
-import org.apache.fop.fonts.FontSetup;
-
-import java.awt.Graphics;
-import java.awt.Font;
 import java.awt.Color;
+import java.awt.Font;
+import java.awt.Graphics;
 import java.awt.Shape;
 import java.awt.font.FontRenderContext;
 import java.awt.font.GlyphVector;
 import java.awt.geom.AffineTransform;
-import java.io.OutputStream;
 import java.io.IOException;
+import java.io.OutputStream;
 import java.io.StringWriter;
 
+import org.apache.xmlgraphics.image.GraphicsConstants;
+
+import org.apache.fop.Version;
+import org.apache.fop.fonts.FontInfo;
+import org.apache.fop.fonts.FontSetup;
+import org.apache.fop.pdf.PDFAnnotList;
+import org.apache.fop.pdf.PDFColorHandler;
+import org.apache.fop.pdf.PDFDocument;
+import org.apache.fop.pdf.PDFFilterList;
+import org.apache.fop.pdf.PDFNumber;
+import org.apache.fop.pdf.PDFPage;
+import org.apache.fop.pdf.PDFPaintingState;
+import org.apache.fop.pdf.PDFResources;
+import org.apache.fop.pdf.PDFStream;
+
 /**
- * This class is a wrapper for the <tt>PDFGraphics2D</tt> that
- * is used to create a full document around the pdf rendering from
- * <tt>PDFGraphics2D</tt>.
+ * This class is a wrapper for the {@link PDFGraphics2D} that
+ * is used to create a full document around the PDF rendering from
+ * {@link PDFGraphics2D}.
  *
  * @see org.apache.fop.svg.PDFGraphics2D
  */
 public class PDFDocumentGraphics2D extends PDFGraphics2D {
 
-    private PDFContext pdfContext;
+    private final PDFContext pdfContext;
 
     private int width;
     private int height;
-    
+
     //for SVG scaling
     private float svgWidth;
     private float svgHeight;
@@ -64,8 +66,8 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
     /** Normal PDF resolution (72dpi) */
     public static final int NORMAL_PDF_RESOLUTION = 72;
     /** Default device resolution (300dpi is a resonable quality for most purposes) */
-    public static final int DEFAULT_NATIVE_DPI = 300;
-  
+    public static final int DEFAULT_NATIVE_DPI = GraphicsConstants.DEFAULT_SAMPLE_DPI;
+
     /**
      * The device resolution may be different from the normal target resolution. See
      * http://issues.apache.org/bugzilla/show_bug.cgi?id=37305
@@ -96,9 +98,10 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
     public PDFDocumentGraphics2D(boolean textAsShapes) {
         super(textAsShapes);
 
-        this.pdfDoc = new PDFDocument("Apache FOP Version " + Version.getVersion() 
+        this.pdfDoc = new PDFDocument("Apache FOP Version " + Version.getVersion()
                 + ": PDFDocumentGraphics2D");
         this.pdfContext = new PDFContext();
+        this.colorHandler = new PDFColorHandler(this.pdfDoc.getResources());
     }
 
     /**
@@ -110,8 +113,8 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
      * @param textAsShapes set this to true so that text will be rendered
      * using curves and not the font.
      * @param stream the stream that the final document should be written to.
-     * @param width the width of the document
-     * @param height the height of the document
+     * @param width the width of the document (in points)
+     * @param height the height of the document (in points)
      * @throws IOException an io exception if there is a problem
      *         writing to the output stream
      */
@@ -150,17 +153,18 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
     }
 
     /**
-     * Setup a default FontInfo instance if none has been setup before. 
+     * Setup a default FontInfo instance if none has been setup before.
      */
     public void setupDefaultFontInfo() {
         if (fontInfo == null) {
             //Default minimal fonts
             FontInfo fontInfo = new FontInfo();
-            FontSetup.setup(fontInfo, null, null);
+            boolean base14Kerning = false;
+            FontSetup.setup(fontInfo, base14Kerning);
             setFontInfo(fontInfo);
         }
     }
-    
+
     /**
      * Set the device resolution for rendering.  Will take effect at the
      * start of the next page.
@@ -184,7 +188,7 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
     public void setFontInfo(FontInfo fontInfo) {
         this.fontInfo = fontInfo;
     }
-    
+
     /**
      * Get the font info for this pdf document.
      * @return the font information
@@ -208,7 +212,7 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
     public PDFContext getPDFContext() {
         return this.pdfContext;
     }
-    
+
     /**
      * Set the dimensions of the svg document that will be drawn.
      * This is useful if the dimensions of the svg document are different
@@ -230,15 +234,15 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
      * @param col the background colour to fill
      */
     public void setBackgroundColor(Color col) {
-        Color c = col;
-        PDFColor currentColour = new PDFColor(c.getRed(), c.getGreen(), c.getBlue());
-        currentStream.write("q\n");
-        currentStream.write(currentColour.getColorSpaceOut(true));
+        StringBuffer sb = new StringBuffer();
+        sb.append("q\n");
+        this.colorHandler.establishColor(sb, col, true);
 
-        currentStream.write("0 0 " + width + " " + height + " re\n");
+        sb.append("0 0 ").append(width).append(" ").append(height).append(" re\n");
 
-        currentStream.write("f\n");
-        currentStream.write("Q\n");
+        sb.append("f\n");
+        sb.append("Q\n");
+        currentStream.write(sb.toString());
     }
 
     /**
@@ -248,19 +252,32 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
     public void nextPage() {
         closePage();
     }
-    
+
     /**
-     * Closes the current page and adds it to the PDF file. 
+     * Is called to prepare the PDFDocumentGraphics2D for the next page to be painted. Basically,
+     * this closes the current page. A new page is prepared as soon as painting starts.
+     * This method allows to start the new page (and following pages) with a different page size.
+     * @param width the width of the new page (in points)
+     * @param height the height of the new page (in points)
+     */
+    public void nextPage(int width, int height) {
+        this.width = width;
+        this.height = height;
+        nextPage();
+    }
+
+    /**
+     * Closes the current page and adds it to the PDF file.
      */
     protected void closePage() {
         if (!pdfContext.isPagePending()) {
             return; //ignore
         }
+        currentStream.write("Q\n");
         //Finish page
         PDFStream pdfStream = this.pdfDoc.getFactory().makeStream(
                 PDFFilterList.CONTENT_FILTER, false);
         pdfStream.add(getString());
-        currentStream = null;
         this.pdfDoc.registerObject(pdfStream);
         pdfContext.getCurrentPage().setContents(pdfStream);
         PDFAnnotList annots = pdfContext.getCurrentPage().getAnnotations();
@@ -270,8 +287,9 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
         this.pdfDoc.addObject(pdfContext.getCurrentPage());
         pdfContext.clearCurrentPage();
     }
-    
+
     /** {@inheritDoc} */
+    @Override
     protected void preparePainting() {
         if (pdfContext.isPagePending()) {
             return;
@@ -296,11 +314,11 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
             throw new IllegalStateException("Close page first before starting another");
         }
         //Start page
-        graphicsState = new PDFState();
+        paintingState = new PDFPaintingState();
         if (this.initialTransform == null) {
             //Save initial transformation matrix
             this.initialTransform = getTransform();
-            this.initialClip = getClip();      
+            this.initialClip = getClip();
         } else {
             //Reset transformation matrix
             setTransform(this.initialTransform);
@@ -313,16 +331,17 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
         if (currentStream == null) {
             currentStream = new StringWriter();
         }
-        
+
         PDFResources pdfResources = this.pdfDoc.getResources();
         PDFPage page = this.pdfDoc.getFactory().makePage(pdfResources,
                 width, height);
         resourceContext = page;
         pdfContext.setCurrentPage(page);
-        pageRef = page.referencePDF();
+        pageRef = page.makeReference();
 
-        AffineTransform at = new AffineTransform(1.0, 0.0, 0.0, -1.0, 
-                                                 0.0, (double)height);
+        currentStream.write("q\n");
+        AffineTransform at = new AffineTransform(1.0, 0.0, 0.0, -1.0,
+                                                 0.0, height);
         currentStream.write("1 0 0 -1 0 " + height + " cm\n");
         if (svgWidth != 0) {
             double scaleX = width / svgWidth;
@@ -336,16 +355,16 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
             at.scale(s, s);
             currentStream.write("" + PDFNumber.doubleOut(s) + " 0 0 "
                                 + PDFNumber.doubleOut(s) + " 0 0 cm\n");
-            
+
             scale(1 / s, 1 / s);
         }
         // Remember the transform we installed.
-        graphicsState.concatenate(at);
+        paintingState.concatenate(at);
 
         pdfContext.increasePageCount();
     }
-        
-    
+
+
     /**
      * The rendering process has finished.
      * This should be called after the rendering has completed as there is
@@ -386,7 +405,9 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
      * @return     a new graphics context that is a copy of
      * this graphics context.
      */
+    @Override
     public Graphics create() {
+        preparePainting();
         return new PDFDocumentGraphics2D(this);
     }
 
@@ -398,6 +419,7 @@ public class PDFDocumentGraphics2D extends PDFGraphics2D {
      * @param x the x position
      * @param y the y position
      */
+    @Override
     public void drawString(String s, float x, float y) {
         if (super.textAsShapes) {
             Font font = super.getFont();
