@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFGraphics2D.java 1357883 2012-07-05 20:29:53Z gadams $ */
+/* $Id: PDFGraphics2D.java 1681435 2015-05-24 11:14:22Z adelmelle $ */
 
 package org.apache.fop.svg;
 
@@ -36,7 +36,6 @@ import java.awt.Stroke;
 import java.awt.color.ColorSpace;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.PathIterator;
-import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
 import java.awt.image.BufferedImage;
 import java.awt.image.ColorModel;
@@ -50,6 +49,7 @@ import java.awt.image.renderable.RenderableImage;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.io.StringWriter;
+import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 
@@ -68,6 +68,7 @@ import org.apache.xmlgraphics.image.loader.impl.ImageRawJPEG;
 import org.apache.xmlgraphics.image.loader.impl.ImageRendered;
 import org.apache.xmlgraphics.java2d.AbstractGraphics2D;
 import org.apache.xmlgraphics.java2d.GraphicContext;
+import org.apache.xmlgraphics.java2d.GraphicsConfigurationWithTransparency;
 
 import org.apache.fop.fonts.Font;
 import org.apache.fop.fonts.FontInfo;
@@ -79,6 +80,7 @@ import org.apache.fop.pdf.PDFColorHandler;
 import org.apache.fop.pdf.PDFConformanceException;
 import org.apache.fop.pdf.PDFDeviceColorSpace;
 import org.apache.fop.pdf.PDFDocument;
+import org.apache.fop.pdf.PDFFunction;
 import org.apache.fop.pdf.PDFGState;
 import org.apache.fop.pdf.PDFImage;
 import org.apache.fop.pdf.PDFImageXObject;
@@ -86,10 +88,16 @@ import org.apache.fop.pdf.PDFLink;
 import org.apache.fop.pdf.PDFNumber;
 import org.apache.fop.pdf.PDFPaintingState;
 import org.apache.fop.pdf.PDFPattern;
+import org.apache.fop.pdf.PDFReference;
 import org.apache.fop.pdf.PDFResourceContext;
 import org.apache.fop.pdf.PDFResources;
+import org.apache.fop.pdf.PDFShading;
 import org.apache.fop.pdf.PDFText;
 import org.apache.fop.pdf.PDFXObject;
+import org.apache.fop.render.gradient.Function;
+import org.apache.fop.render.gradient.GradientMaker;
+import org.apache.fop.render.gradient.Pattern;
+import org.apache.fop.render.gradient.Shading;
 import org.apache.fop.render.pdf.ImageRawCCITTFaxAdapter;
 import org.apache.fop.render.pdf.ImageRawJPEGAdapter;
 import org.apache.fop.render.pdf.ImageRenderedAdapter;
@@ -125,7 +133,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
     /**
      * The PDF reference of the current page.
      */
-    protected String pageRef;
+    protected PDFReference pageRef;
 
     /**
      * The PDF painting state
@@ -138,13 +146,13 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
     /**
      * The PDF graphics state level that this svg is being drawn into.
      */
-    protected int baseLevel = 0;
+    protected int baseLevel;
 
     /**
      * The count of natively handled images added to document so they receive
      * unique keys.
      */
-    protected int nativeCount = 0;
+    protected int nativeCount;
 
     /**
      * The current font information.
@@ -155,7 +163,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
      * The override font state used when drawing text and the font cannot be
      * set using java fonts.
      */
-    protected Font ovFontState = null;
+    protected Font ovFontState;
 
     /**
      * the current stream to add PDF commands to
@@ -178,7 +186,18 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
      * the pdf document objects to reduce memory.
      * Especially with images.
      */
-    protected OutputStream outputStream = null;
+    protected OutputStream outputStream;
+
+    private TransparencyIgnoredEventListener transparencyIgnoredEventListener;
+
+    /**
+     * May be used to give proper feedback to the user when a particular PDF profile is
+     * being used that disallows transparency.
+     */
+    public interface TransparencyIgnoredEventListener {
+
+        void transparencyIgnored(Object pdfProfile);
+    }
 
     /**
      * Create a new PDFGraphics2D with the given pdf document info.
@@ -194,7 +213,8 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
      * @param size the current font size
      */
     public PDFGraphics2D(boolean textAsShapes, FontInfo fi, PDFDocument doc,
-                         PDFResourceContext page, String pref, String font, float size) {
+                         PDFResourceContext page, PDFReference pref, String font, float size,
+                         TransparencyIgnoredEventListener listener) {
         this(textAsShapes);
         pdfDoc = doc;
         this.colorHandler = new PDFColorHandler(doc.getResources());
@@ -204,6 +224,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
         fontInfo = fi;
         pageRef = pref;
         paintingState = new PDFPaintingState();
+        this.transparencyIgnoredEventListener = listener;
     }
 
     /**
@@ -235,6 +256,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
         this.nativeCount = g.nativeCount;
         this.outputStream = g.outputStream;
         this.ovFontState = g.ovFontState;
+        this.transparencyIgnoredEventListener = g.transparencyIgnoredEventListener;
     }
 
     /**
@@ -273,7 +295,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
      */
     public void setPaintingState(PDFPaintingState state) {
         paintingState = state;
-        baseLevel = paintingState.getStackLevel();
+        baseLevel = paintingState.getStackLevel(); // @SuppressFBWarnings("URF_UNREAD_PUBLIC_OR_PROTECTED_FIELD")
     }
 
     /**
@@ -310,7 +332,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
      * Gets the PDF reference of the current page.
      * @return the PDF reference of the current page
      */
-    public String getPageReference() {
+    public PDFReference getPageReference() {
         return this.pageRef;
     }
 
@@ -401,7 +423,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
             if (linkType != PDFLink.EXTERNAL) {
                 String pdfdest = "/FitR " + dest;
                 resourceContext.addAnnotation(
-                    pdfDoc.getFactory().makeLink(rect, getPageReference(), pdfdest));
+                    pdfDoc.getFactory().makeLink(rect, getPageReference().toString(), pdfdest));
             } else {
                 resourceContext.addAnnotation(
                     pdfDoc.getFactory().makeLink(rect, dest, linkType, 0));
@@ -453,7 +475,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
     }
 
     private void flushPDFDocument() {
-        if (outputStream != null) {
+        if (outputStream != null && !pdfDoc.isLinearizationEnabled()) {
             try {
                 this.pdfDoc.output(outputStream);
             } catch (IOException ioe) {
@@ -543,7 +565,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
 
             xObject = addRenderedImage(key, buf);
         } else {
-            resourceContext.getPDFResources().addXObject(xObject);
+            resourceContext.addXObject(xObject);
         }
 
         AffineTransform at = new AffineTransform();
@@ -742,7 +764,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
         //TODO Handle this in PDFColorHandler by automatically converting the color.
         //This won't work properly anyway after the redesign of ColorExt
         if (col.getColorSpace().getType() == ColorSpace.TYPE_CMYK) {
-             if (pdfDoc.getProfile().getPDFAMode().isPDFA1LevelB()) {
+             if (pdfDoc.getProfile().getPDFAMode().isPart1()) {
                  //See PDF/A-1, ISO 19005:1:2005(E), 6.2.3.3
                  //FOP is currently restricted to DeviceRGB if PDF/A-1 is active.
                  throw new PDFConformanceException(
@@ -777,7 +799,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
      * @param fill true if the paint should be set for filling
      * @return true if the paint is handled natively, false if the paint should be rasterized
      */
-    protected boolean applyPaint(Paint paint, boolean fill) {   // CSOK: MethodLength
+    protected boolean applyPaint(Paint paint, boolean fill) {
         preparePainting();
 
         if (paint instanceof Color) {
@@ -795,163 +817,18 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
                     new Color[] {gpaint.getColor1(), gpaint.getColor2()},
                     gpaint.isCyclic() ? LinearGradientPaint.REPEAT : LinearGradientPaint.NO_CYCLE);
         }
-        if (paint instanceof LinearGradientPaint) {
-            LinearGradientPaint gp = (LinearGradientPaint)paint;
-
-            // This code currently doesn't support 'repeat'.
-            // For linear gradients it is possible to construct
-            // a 'tile' that is repeated with a PDF pattern, but
-            // it would be very tricky as you would have to rotate
-            // the coordinate system so the repeat was axially
-            // aligned.  At this point I'm just going to rasterize it.
-            MultipleGradientPaint.CycleMethodEnum cycle = gp.getCycleMethod();
-            if (cycle != MultipleGradientPaint.NO_CYCLE) {
-                return false;
-            }
-
-            Color[] cols = gp.getColors();
-            float[] fractions = gp.getFractions();
-
-            // Build proper transform from gradient space to page space
-            // ('Patterns' don't get userspace transform).
-            AffineTransform transform;
-            transform = new AffineTransform(getBaseTransform());
-            transform.concatenate(getTransform());
-            transform.concatenate(gp.getTransform());
-
-            List<Double> theMatrix = new java.util.ArrayList<Double>();
-            double [] mat = new double[6];
-            transform.getMatrix(mat);
-            for (int idx = 0; idx < mat.length; idx++) {
-                theMatrix.add(new Double(mat[idx]));
-            }
-
-            Point2D p1 = gp.getStartPoint();
-            Point2D p2 = gp.getEndPoint();
-            List<Double> theCoords = new java.util.ArrayList<Double>();
-            theCoords.add(new Double(p1.getX()));
-            theCoords.add(new Double(p1.getY()));
-            theCoords.add(new Double(p2.getX()));
-            theCoords.add(new Double(p2.getY()));
-
-            List<Boolean> theExtend = new java.util.ArrayList<Boolean>();
-            theExtend.add(Boolean.TRUE);
-            theExtend.add(Boolean.TRUE);
-
-            List<Double> theDomain = new java.util.ArrayList<Double>();
-            theDomain.add(new Double(0));
-            theDomain.add(new Double(1));
-
-            List<Double> theEncode = new java.util.ArrayList<Double>();
-            theEncode.add(new Double(0));
-            theEncode.add(new Double(1));
-            theEncode.add(new Double(0));
-            theEncode.add(new Double(1));
-
-            List<Double> theBounds = new java.util.ArrayList<Double>();
-
-            List<Color> someColors = new java.util.ArrayList<Color>();
-
-            for (int count = 0; count < cols.length; count++) {
-                Color c1 = cols[count];
-                if (c1.getAlpha() != 255) {
-                    return false;  // PDF can't do alpha
-                }
-
-                //PDFColor color1 = new PDFColor(c1.getRed(), c1.getGreen(),
-                //                               c1.getBlue());
-                someColors.add(c1);
-                if (count > 0 && count < cols.length - 1) {
-                    theBounds.add(new Double(fractions[count]));
-                }
-            }
-
-            //Gradients are currently restricted to sRGB
-            PDFDeviceColorSpace aColorSpace;
-            aColorSpace = new PDFDeviceColorSpace(PDFDeviceColorSpace.DEVICE_RGB);
-            PDFPattern myPat = pdfDoc.getFactory().makeGradient(
-                    resourceContext, false, aColorSpace,
-                    someColors, theBounds, theCoords, theMatrix);
-            currentStream.write(myPat.getColorSpaceOut(fill));
-
+        if (paint instanceof LinearGradientPaint && gradientSupported((LinearGradientPaint) paint)) {
+            Pattern pattern = GradientMaker.makeLinearGradient((LinearGradientPaint) paint,
+                    getBaseTransform(), getTransform());
+            PDFPattern pdfPattern = createPDFPattern(pattern);
+            currentStream.write(pdfPattern.getColorSpaceOut(fill));
             return true;
         }
-        if (paint instanceof RadialGradientPaint) {
-            RadialGradientPaint rgp = (RadialGradientPaint)paint;
-
-            // There is essentially no way to support repeats
-            // in PDF for radial gradients (the one option would
-            // be to 'grow' the outer circle until it fully covered
-            // the bounds and then grow the stops accordingly, the
-            // problem is that this may require an extremely large
-            // number of stops for cases where the focus is near
-            // the edge of the outer circle).  so we rasterize.
-            MultipleGradientPaint.CycleMethodEnum cycle = rgp.getCycleMethod();
-            if (cycle != MultipleGradientPaint.NO_CYCLE) {
-                return false;
-            }
-
-            AffineTransform transform;
-            transform = new AffineTransform(getBaseTransform());
-            transform.concatenate(getTransform());
-            transform.concatenate(rgp.getTransform());
-
-            List<Double> theMatrix = new java.util.ArrayList<Double>();
-            double [] mat = new double[6];
-            transform.getMatrix(mat);
-            for (int idx = 0; idx < mat.length; idx++) {
-                theMatrix.add(new Double(mat[idx]));
-            }
-
-            double ar = rgp.getRadius();
-            Point2D ac = rgp.getCenterPoint();
-            Point2D af = rgp.getFocusPoint();
-
-            List<Double> theCoords = new java.util.ArrayList<Double>();
-            double dx = af.getX() - ac.getX();
-            double dy = af.getY() - ac.getY();
-            double d = Math.sqrt(dx * dx + dy * dy);
-            if (d > ar) {
-                // the center point af must be within the circle with
-                // radius ar centered at ac so limit it to that.
-                double scale = (ar * .9999) / d;
-                dx = dx * scale;
-                dy = dy * scale;
-            }
-
-            theCoords.add(new Double(ac.getX() + dx)); // Fx
-            theCoords.add(new Double(ac.getY() + dy)); // Fy
-            theCoords.add(new Double(0));
-            theCoords.add(new Double(ac.getX()));
-            theCoords.add(new Double(ac.getY()));
-            theCoords.add(new Double(ar));
-
-            Color[] cols = rgp.getColors();
-            List<Color> someColors = new java.util.ArrayList<Color>();
-            for (int count = 0; count < cols.length; count++) {
-                Color cc = cols[count];
-                if (cc.getAlpha() != 255) {
-                    return false;  // PDF can't do alpha
-                }
-
-                someColors.add(cc);
-            }
-
-            float[] fractions = rgp.getFractions();
-            List<Double> theBounds = new java.util.ArrayList<Double>();
-            for (int count = 1; count < fractions.length - 1; count++) {
-                float offset = fractions[count];
-                theBounds.add(new Double(offset));
-            }
-            PDFDeviceColorSpace colSpace;
-            colSpace = new PDFDeviceColorSpace(PDFDeviceColorSpace.DEVICE_RGB);
-
-            PDFPattern myPat = pdfDoc.getFactory().makeGradient
-                (resourceContext, true, colSpace,
-                 someColors, theBounds, theCoords, theMatrix);
-
-            currentStream.write(myPat.getColorSpaceOut(fill));
-
+        if (paint instanceof RadialGradientPaint && gradientSupported((RadialGradientPaint) paint)) {
+            Pattern pattern = GradientMaker.makeRadialGradient((RadialGradientPaint) paint,
+                    getBaseTransform(), getTransform());
+            PDFPattern pdfPattern = createPDFPattern(pattern);
+            currentStream.write(pdfPattern.getColorSpaceOut(fill));
             return true;
         }
         if (paint instanceof PatternPaint) {
@@ -961,6 +838,47 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
         return false; // unknown paint
     }
 
+    private PDFPattern createPDFPattern(Pattern pattern) {
+        Shading shading = pattern.getShading();
+        Function function = shading.getFunction();
+        List<PDFFunction> pdfFunctions = new ArrayList<PDFFunction>(function.getFunctions().size());
+        for (Function f : function.getFunctions()) {
+            pdfFunctions.add(registerFunction(new PDFFunction(f)));
+        }
+        PDFFunction pdfFunction = registerFunction(new PDFFunction(function, pdfFunctions));
+        PDFShading pdfShading = new PDFShading(shading.getShadingType(), shading.getColorSpace(), shading.getCoords(),
+                pdfFunction);
+        pdfShading = registerShading(pdfShading);
+        PDFPattern pdfPattern = new PDFPattern(pattern.getPatternType(), pdfShading, null, null, pattern.getMatrix());
+        return registerPattern(pdfPattern);
+    }
+
+    private boolean gradientSupported(MultipleGradientPaint gradient) {
+        return !(gradientContainsTransparency(gradient) || gradientIsRepeated(gradient));
+    }
+
+    private boolean gradientContainsTransparency(MultipleGradientPaint gradient) {
+        for (Color color : gradient.getColors()) {
+            if (color.getAlpha() != 255) {
+                return true;
+            }
+        }
+        return false;
+    }
+
+    private boolean gradientIsRepeated(MultipleGradientPaint gradient) {
+         // For linear gradients it is possible to construct a 'tile' that is repeated with
+         // a PDF pattern, but it would be very tricky as the coordinate system would have
+         // to be rotated so the repeat is axially aligned.
+
+         // For radial gradients there is essentially no way to support repeats in PDF (the
+         // one option would be to 'grow' the outer circle until it fully covers the
+         // bounds and then grow the stops accordingly, the problem is that this may
+         // require an extremely large number of stops for cases where the focus is near
+         // the edge of the outer circle).
+        return (gradient.getCycleMethod() != MultipleGradientPaint.NO_CYCLE);
+    }
+
     private boolean createPattern(PatternPaint pp, boolean fill) {
         preparePainting();
 
@@ -972,7 +890,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
         PDFResourceContext context = new PDFResourceContext(res);
         PDFGraphics2D pattGraphic = new PDFGraphics2D(textAsShapes, specialFontInfo,
                                         pdfDoc, context, getPageReference(),
-                                        "", 0);
+                                        "", 0, transparencyIgnoredEventListener);
         pattGraphic.setGraphicContext(new GraphicContext());
         pattGraphic.gc.validateTransformStack();
         pattGraphic.setRenderingHints(this.getRenderingHints());
@@ -1099,24 +1017,25 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
         int devH = devBounds.height;
 
         ColorSpace rgbCS = ColorSpace.getInstance(ColorSpace.CS_sRGB);
-        ColorModel rgbCM = new DirectColorModel
-            (rgbCS, 32, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000,
+        ColorModel rgbCM = new DirectColorModel(
+            rgbCS, 32, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000,
              false, DataBuffer.TYPE_BYTE);
 
         PaintContext pctx = paint.createContext(rgbCM, devBounds, usrBounds,
                                                 at, getRenderingHints());
-        PDFXObject imageInfo = pdfDoc.getXObject
-            ("TempImage:" + pctx.toString());
+        PDFXObject imageInfo = pdfDoc.getXObject(
+            "TempImage:" + pctx.toString());
         if (imageInfo != null) {
-            resourceContext.getPDFResources().addXObject(imageInfo);
+            resourceContext.addXObject(imageInfo);
         } else {
             Raster r = pctx.getRaster(devX, devY, devW, devH);
-            WritableRaster wr = (WritableRaster)r;
+            assert (r instanceof WritableRaster);
+            WritableRaster wr = (WritableRaster) r;
             wr = wr.createWritableTranslatedChild(0, 0);
 
             ColorModel pcm = pctx.getColorModel();
-            BufferedImage bi = new BufferedImage
-                (pcm, wr, pcm.isAlphaPremultiplied(), null);
+            BufferedImage bi = new BufferedImage(
+                pcm, wr, pcm.isAlphaPremultiplied(), null);
             final byte[] rgb  = new byte[devW * devH * 3];
             final int[]  line = new int[devW];
             final byte[] mask;
@@ -1134,8 +1053,8 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
                         val = line[x];
                         mask[maskIdx++] = (byte)(val >>> 24);
                         rgb[rgbIdx++]   = (byte)((val >> 16) & 0x0FF);
-                        rgb[rgbIdx++]   = (byte)((val >> 8 ) & 0x0FF);
-                        rgb[rgbIdx++]   = (byte)((val      ) & 0x0FF);
+                        rgb[rgbIdx++]   = (byte)((val >> 8) & 0x0FF);
+                        rgb[rgbIdx++]   = (byte)(val & 0x0FF);
                     }
                 }
             } else {
@@ -1145,19 +1064,19 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
                     for (x = 0; x < devW; x++) {
                         val = line[x];
                         rgb[rgbIdx++]  = (byte)((val >> 16) & 0x0FF);
-                        rgb[rgbIdx++]  = (byte)((val >> 8 ) & 0x0FF);
-                        rgb[rgbIdx++]  = (byte)((val      ) & 0x0FF);
+                        rgb[rgbIdx++]  = (byte)((val >> 8) & 0x0FF);
+                        rgb[rgbIdx++]  = (byte)(val & 0x0FF);
                     }
                 }
             }
 
-            String maskRef = null;
+            PDFReference maskRef = null;
             if (mask != null) {
-                BitmapImage fopimg = new BitmapImage
-                    ("TempImageMask:" + pctx.toString(), devW, devH, mask, null);
+                BitmapImage fopimg = new BitmapImage(
+                    "TempImageMask:" + pctx.toString(), devW, devH, mask, null);
                 fopimg.setColorSpace(new PDFDeviceColorSpace(PDFDeviceColorSpace.DEVICE_GRAY));
                 PDFImageXObject xobj = pdfDoc.addImage(resourceContext, fopimg);
-                maskRef = xobj.referencePDF();
+                maskRef = xobj.makeReference();
 
                 flushPDFDocument();
             }
@@ -1236,7 +1155,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
             float lw = bs.getLineWidth();
             currentStream.write(PDFNumber.doubleOut(lw) + " w\n");
 
-            float ml = bs.getMiterLimit();
+            float ml = Math.max(1.0f, bs.getMiterLimit());
             currentStream.write(PDFNumber.doubleOut(ml) + " M\n");
         }
     }
@@ -1259,7 +1178,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
         if (xObject == null) {
             xObject = addRenderedImage(key, img);
         } else {
-            resourceContext.getPDFResources().addXObject(xObject);
+            resourceContext.addXObject(xObject);
         }
 
         useXObject(xObject, xform, img.getWidth(), img.getHeight());
@@ -1425,18 +1344,21 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
      */
     protected void applyAlpha(int fillAlpha, int strokeAlpha) {
         if (fillAlpha != OPAQUE || strokeAlpha != OPAQUE) {
-            checkTransparencyAllowed();
-            Map<String, Float> vals = new java.util.HashMap<String, Float>();
-            if (fillAlpha != OPAQUE) {
-                vals.put(PDFGState.GSTATE_ALPHA_NONSTROKE, new Float(fillAlpha / 255f));
+            Object profile = isTransparencyAllowed();
+            if (profile == null) {
+                Map<String, Float> vals = new java.util.HashMap<String, Float>();
+                if (fillAlpha != OPAQUE) {
+                    vals.put(PDFGState.GSTATE_ALPHA_NONSTROKE, new Float(fillAlpha / 255f));
+                }
+                if (strokeAlpha != OPAQUE) {
+                    vals.put(PDFGState.GSTATE_ALPHA_STROKE, new Float(strokeAlpha / 255f));
+                }
+                PDFGState gstate = pdfDoc.getFactory().makeGState(vals, paintingState.getGState());
+                resourceContext.addGState(gstate);
+                currentStream.write("/" + gstate.getName() + " gs\n");
+            } else if (transparencyIgnoredEventListener != null) {
+                transparencyIgnoredEventListener.transparencyIgnored(profile);
             }
-            if (strokeAlpha != OPAQUE) {
-                vals.put(PDFGState.GSTATE_ALPHA_STROKE, new Float(strokeAlpha / 255f));
-            }
-            PDFGState gstate = pdfDoc.getFactory().makeGState(
-                    vals, paintingState.getGState());
-            resourceContext.addGState(gstate);
-            currentStream.write("/" + gstate.getName() + " gs\n");
         }
     }
 
@@ -1681,8 +1603,8 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
     }
 
     /** Checks whether the use of transparency is allowed. */
-    protected void checkTransparencyAllowed() {
-        pdfDoc.getProfile().verifyTransparencyAllowed("Java2D graphics");
+    protected Object isTransparencyAllowed() {
+        return pdfDoc.getProfile().isTransparencyAllowed();
     }
 
     /**
@@ -1690,11 +1612,15 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
      * @param iter PathIterator to process
      */
     public void processPathIterator(PathIterator iter) {
+        double lastX = 0.0;
+        double lastY = 0.0;
         while (!iter.isDone()) {
             double[] vals = new double[6];
             int type = iter.currentSegment(vals);
             switch (type) {
             case PathIterator.SEG_CUBICTO:
+                lastX = vals[4];
+                lastY = vals[5];
                 currentStream.write(PDFNumber.doubleOut(vals[0], DEC) + " "
                                     + PDFNumber.doubleOut(vals[1], DEC) + " "
                                     + PDFNumber.doubleOut(vals[2], DEC) + " "
@@ -1703,18 +1629,30 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
                                     + PDFNumber.doubleOut(vals[5], DEC) + " c\n");
                 break;
             case PathIterator.SEG_LINETO:
+                lastX = vals[0];
+                lastY = vals[1];
                 currentStream.write(PDFNumber.doubleOut(vals[0], DEC) + " "
                                     + PDFNumber.doubleOut(vals[1], DEC) + " l\n");
                 break;
             case PathIterator.SEG_MOVETO:
+                lastX = vals[0];
+                lastY = vals[1];
                 currentStream.write(PDFNumber.doubleOut(vals[0], DEC) + " "
                                     + PDFNumber.doubleOut(vals[1], DEC) + " m\n");
                 break;
             case PathIterator.SEG_QUADTO:
-                currentStream.write(PDFNumber.doubleOut(vals[0], DEC) + " "
-                                    + PDFNumber.doubleOut(vals[1], DEC) + " "
-                                    + PDFNumber.doubleOut(vals[2], DEC) + " "
-                                    + PDFNumber.doubleOut(vals[3], DEC) + " y\n");
+                double controlPointAX = lastX + ((2.0 / 3.0) * (vals[0] - lastX));
+                double controlPointAY = lastY + ((2.0 / 3.0) * (vals[1] - lastY));
+                double controlPointBX = vals[2] + ((2.0 / 3.0) * (vals[0] - vals[2]));
+                double controlPointBY = vals[3] + ((2.0 / 3.0) * (vals[1] - vals[3]));
+                currentStream.write(PDFNumber.doubleOut(controlPointAX, DEC) + " "
+                        + PDFNumber.doubleOut(controlPointAY, DEC) + " "
+                        + PDFNumber.doubleOut(controlPointBX, DEC) + " "
+                        + PDFNumber.doubleOut(controlPointBY, DEC) + " "
+                        + PDFNumber.doubleOut(vals[2], DEC) + " "
+                        + PDFNumber.doubleOut(vals[3], DEC) + " c\n");
+                lastX = vals[2];
+                lastY = vals[3];
                 break;
             case PathIterator.SEG_CLOSE:
                 currentStream.write("h\n");
@@ -1765,7 +1703,7 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
      */
     @Override
     public GraphicsConfiguration getDeviceConfiguration() {
-        return new PDFGraphicsConfiguration();
+        return new GraphicsConfigurationWithTransparency();
     }
 
     /**
@@ -1839,4 +1777,34 @@ public class PDFGraphics2D extends AbstractGraphics2D implements NativeImageHand
         //NYI
     }
 
+    /**
+     * Registers a function object against the output format document
+     * @param function The function object to register
+     * @return Returns either the function which has already been registered
+     * or the current new registered object.
+     */
+    public PDFFunction registerFunction(PDFFunction function) {
+        return pdfDoc.getFactory().registerFunction(function);
+    }
+
+    /**
+     * Registers a shading object against the otuput format document
+     * @param shading The shading object to register
+     * @return Returs either the shading which has already been registered
+     * or the current new registered object
+     */
+    public PDFShading registerShading(PDFShading shading) {
+        return pdfDoc.getFactory().registerShading(resourceContext, shading);
+    }
+
+    /**
+     * Registers a pattern object against the output format document
+     * @param pattern The pattern object to register
+     * @return Returns either the pattern which has already been registered
+     * or the current new registered object
+     */
+    public PDFPattern registerPattern(PDFPattern pattern) {
+        return pdfDoc.getFactory().registerPattern(resourceContext, pattern);
+    }
+
 }
