@@ -15,18 +15,18 @@
  * limitations under the License.
  */
 
-/* $Id: PDFTextElementBridge.java 721430 2008-11-28 11:13:12Z acumiskey $ */
+/* $Id: PDFTextElementBridge.java 1664570 2015-03-06 09:41:07Z lbernardo $ */
 
 package org.apache.fop.svg;
 
-import org.apache.batik.gvt.TextPainter;
+import org.apache.batik.bridge.TextPainter;
 
 /**
- * Bridge class for the &lt;text> element.
+ * <p>Bridge class for the &lt;text> element.
  * This bridge will use the direct text painter if the text
- * for the element is simple.
+ * for the element is simple.</p>
  *
- * @author <a href="mailto:keiron@aftexsw.com">Keiron Liddle</a>
+ * <p>This work was authored by Keiron Liddle (keiron@aftexsw.com).</p>
  */
 public class PDFTextElementBridge extends AbstractFOPTextElementBridge {
 
