@@ -15,11 +15,22 @@
  * limitations under the License.
  */
 
-/* $Id: Fop.java 828999 2009-10-23 11:22:53Z acumiskey $ */
+/* $Id: Fop.java 1616668 2014-08-08 03:48:54Z gadams $ */
 
 package org.apache.fop.tools.anttasks;
 
 // Ant
+import java.io.BufferedOutputStream;
+import java.io.File;
+import java.io.IOException;
+import java.io.OutputStream;
+import java.net.URI;
+import java.util.List;
+
+import org.xml.sax.SAXException;
+
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.impl.SimpleLog;
 import org.apache.tools.ant.BuildException;
 import org.apache.tools.ant.DirectoryScanner;
 import org.apache.tools.ant.Project;
@@ -27,26 +38,14 @@ import org.apache.tools.ant.Task;
 import org.apache.tools.ant.types.FileSet;
 import org.apache.tools.ant.util.GlobPatternMapper;
 
-// Java
-import java.io.BufferedOutputStream;
-import java.io.File;
-import java.io.IOException;
-import java.io.OutputStream;
-import java.net.MalformedURLException;
-import java.util.List;
-import java.util.Vector;
-
-// FOP
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
+import org.apache.fop.apps.FopConfParser;
 import org.apache.fop.apps.FopFactory;
+import org.apache.fop.apps.FopFactoryBuilder;
 import org.apache.fop.apps.MimeConstants;
 import org.apache.fop.cli.InputHandler;
 
-import org.apache.commons.logging.impl.SimpleLog;
-import org.apache.commons.logging.Log;
-import org.xml.sax.SAXException;
-
 /**
  * Wrapper for FOP which allows it to be accessed from within an Ant task.
  * Accepts the inputs:
@@ -79,8 +78,8 @@ public class Fop extends Task {
     private File userConfig;
     private int messageType = Project.MSG_VERBOSE;
     private boolean logFiles = true;
-    private boolean force = false;
-    private boolean relativebase = false;
+    private boolean force;
+    private boolean relativebase;
     private boolean throwExceptions = true;
 
     /**
@@ -156,7 +155,7 @@ public class Fop extends Task {
     }
 
     /**
-     * Sets the XSLT parameters 
+     * Sets the XSLT parameters
      * @param xsltParams the XSLT parameters
      */
     public void setXsltParams(String xsltParams) {
@@ -267,8 +266,7 @@ public class Fop extends Task {
     /**
      * Set whether exceptions are thrown.
      * default is false.
-     *
-     * @param force true if always generate.
+     * @param throwExceptions true if exceptions should be thrown
      */
     public void setThrowexceptions(boolean throwExceptions) {
         this.throwExceptions = throwExceptions;
@@ -308,6 +306,7 @@ public class Fop extends Task {
     /**
      * Returns the message type corresponding to Project.MSG_*
      * representing the current message level.
+     * @return message type
      * @see org.apache.tools.ant.Project
      */
     public int getMessageType() {
@@ -381,15 +380,15 @@ public class Fop extends Task {
 class FOPTaskStarter {
 
     // configure fopFactory as desired
-    private FopFactory fopFactory = FopFactory.newInstance();
+    private final FopFactory fopFactory;
 
     private Fop task;
-    private String baseURL = null;
+    private URI baseUri;
 
     /**
      * logging instance
      */
-    protected Log logger = null;
+    protected Log logger;
 
 
     /**
@@ -410,8 +409,21 @@ class FOPTaskStarter {
 
     FOPTaskStarter(Fop task) throws SAXException, IOException {
         this.task = task;
+        //Set base directory
+        if (task.getBasedir() != null) {
+            this.baseUri = task.getBasedir().toURI();
+        } else {
+            if (task.getFofile() != null) {
+                this.baseUri = task.getFofile().getParentFile().toURI();
+            }
+        }
         if (task.getUserconfig() != null) {
-            fopFactory.setUserConfig(task.getUserconfig());
+            FopFactoryBuilder confBuilder = new FopConfParser(
+                    task.getUserconfig()).getFopFactoryBuilder();
+            confBuilder.setBaseURI(baseUri);
+            fopFactory = confBuilder.build();
+        } else {
+            fopFactory = FopFactory.newInstance(baseUri);
         }
     }
 
@@ -489,29 +501,8 @@ class FOPTaskStarter {
         return new File(file.getParentFile(), name);
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public void run() throws FOPException {
-        //Set base directory
-        if (task.getBasedir() != null) {
-            try {
-                this.baseURL = task.getBasedir().toURI().toURL().toExternalForm();
-            } catch (MalformedURLException mfue) {
-                logger.error("Error creating base URL from base directory", mfue);
-            }
-        } else {
-            try {
-                if (task.getFofile() != null) {
-                    this.baseURL =  task.getFofile().getParentFile().toURI().toURL().
-                                      toExternalForm();
-                }
-            } catch (MalformedURLException mfue) {
-                logger.error("Error creating base URL from XSL-FO input file", mfue);
-            }
-        }
-
-        task.log("Using base URL: " + baseURL, Project.MSG_DEBUG);
+        task.log("Using base URI: " + baseUri, Project.MSG_DEBUG);
 
         String outputFormat = normalizeOutputFormat(task.getFormat());
         String newExtension = determineExtension(outputFormat);
@@ -535,11 +526,11 @@ class FOPTaskStarter {
                 // OR output file doesn't exist OR
                 // output file is older than input file
                 if (task.getForce() || !outf.exists()
-                    || (task.getFofile().lastModified() > outf.lastModified() )) {
+                    || (task.getFofile().lastModified() > outf.lastModified())) {
                     render(task.getFofile(), outf, outputFormat);
                     actioncount++;
                 } else if (outf.exists()
-                        && (task.getFofile().lastModified() <= outf.lastModified() )) {
+                        && (task.getFofile().lastModified() <= outf.lastModified())) {
                     skippedcount++;
                 }
             }
@@ -556,13 +547,13 @@ class FOPTaskStarter {
                 // OR output file doesn't exist OR
                 // output file is older than input file
                 if (task.getForce() || !outf.exists()
-                    || (task.getXmlFile().lastModified() > outf.lastModified() ||
-                            task.getXsltFile().lastModified() > outf.lastModified())) {
+                        || (task.getXmlFile().lastModified() > outf.lastModified()
+                        || task.getXsltFile().lastModified() > outf.lastModified())) {
                     render(task.getXmlFile(), task.getXsltFile(), outf, outputFormat);
                     actioncount++;
                 } else if (outf.exists()
-                        && (task.getXmlFile().lastModified() <= outf.lastModified() ||
-                                task.getXsltFile().lastModified() <= outf.lastModified())) {
+                        && (task.getXmlFile().lastModified() <= outf.lastModified()
+                            || task.getXsltFile().lastModified() <= outf.lastModified())) {
                     skippedcount++;
                 }
             }
@@ -603,12 +594,10 @@ class FOPTaskStarter {
                 }
                 try {
                     if (task.getRelativebase()) {
-                        this.baseURL = f.getParentFile().toURI().toURL().
-                                         toExternalForm();
+                        this.baseUri = f.getParentFile().toURI();
                     }
-                    if (this.baseURL == null) {
-                        this.baseURL = fs.getDir(task.getProject()).toURI().toURL().
-                                          toExternalForm();
+                    if (this.baseUri == null) {
+                        this.baseUri = fs.getDir(task.getProject()).toURI();
                     }
 
                 } catch (Exception e) {
@@ -619,14 +608,14 @@ class FOPTaskStarter {
                 // OR output file doesn't exist OR
                 // output file is older than input file
                 if (task.getForce() || !outf.exists()
-                    || (f.lastModified() > outf.lastModified() )) {
+                    || (f.lastModified() > outf.lastModified())) {
                     if (xsltFile != null) {
                         render(f, xsltFile, outf, outputFormat);
                     } else {
                         render(f, outf, outputFormat);
                     }
                     actioncount++;
-                } else if (outf.exists() && (f.lastModified() <= outf.lastModified() )) {
+                } else if (outf.exists() && (f.lastModified() <= outf.lastModified())) {
                     skippedcount++;
                 }
             }
@@ -642,7 +631,8 @@ class FOPTaskStarter {
         }
     }
 
-    private void renderInputHandler(InputHandler inputHandler, File outFile, String outputFormat) throws Exception {
+    private void renderInputHandler(InputHandler inputHandler, File outFile, String outputFormat)
+            throws Exception {
         OutputStream out = null;
         try {
             out = new java.io.FileOutputStream(outFile);
@@ -653,7 +643,6 @@ class FOPTaskStarter {
         boolean success = false;
         try {
             FOUserAgent userAgent = fopFactory.newFOUserAgent();
-            userAgent.setBaseURL(this.baseURL);
             inputHandler.renderTo(userAgent, outputFormat, out);
             success = true;
         } catch (Exception ex) {
@@ -688,15 +677,16 @@ class FOPTaskStarter {
 
     private void render(File xmlFile, File xsltFile, File outFile, String outputFormat) {
         //TODO: implement support for XSLT params
-        final Vector xsltParams = null;
-        InputHandler inputHandler = new InputHandler(xmlFile, xsltFile, xsltParams);
+        InputHandler inputHandler = new InputHandler(xmlFile, xsltFile, null);
         try {
             renderInputHandler(inputHandler, outFile, outputFormat);
         } catch (Exception ex) {
-            logger.error("Error rendering xml/xslt files: " + xmlFile + ", " + xsltFile, ex);
+            logger.error("Error rendering xml/xslt files: "
+                         + xmlFile + ", " + xsltFile, ex);
         }
         if (task.getLogFiles()) {
-            task.log("xml: " + xmlFile + ", xslt: " + xsltFile + " -> " + outFile, Project.MSG_INFO);
+            task.log("xml: " + xmlFile + ", xslt: "
+                     + xsltFile + " -> " + outFile, Project.MSG_INFO);
         }
     }
 }
