@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: SpaceVal.java 893238 2009-12-22 17:20:51Z vhennebert $ */
+/* $Id: SpaceVal.java 1494364 2013-06-18 23:04:16Z lbernardo $ */
 
 package org.apache.fop.traits;
 
@@ -59,15 +59,25 @@ public class SpaceVal {
         int min = spaceprop.getMinimum(context).getLength().getValue(context);
         int opt = spaceprop.getOptimum(context).getLength().getValue(context);
         int max = spaceprop.getMaximum(context).getLength().getValue(context);
+        // if a value is not set defaults to zero
+        if (min > max && max == 0) {
+            max = opt > min ? opt : min;
+        }
+        if (min > opt && opt == 0) {
+            opt = (min + max) / 2;
+        }
+        if (opt > max && max == 0) {
+            max = opt;
+        }
         return MinOptMax.getInstance(min, opt, max);
     }
 
     /**
      * Constructor for SpaceVal objects based on the full set of properties.
      * @param space space to use
-     * @param bConditional Conditionality value
-     * @param bForcing Forcing value
-     * @param iPrecedence Precedence value
+     * @param conditional Conditionality value
+     * @param forcing Forcing value
+     * @param precedence Precedence value
      */
     public SpaceVal(MinOptMax space, boolean conditional, boolean forcing, int precedence) {
         this.space = space;
@@ -76,6 +86,12 @@ public class SpaceVal {
         this.precedence = precedence;
     }
 
+    /**
+     * @param wordSpacing property
+     * @param letterSpacing space value
+     * @param fs font
+     * @return space value
+     */
     public static SpaceVal makeWordSpacing(Property wordSpacing, SpaceVal letterSpacing, Font fs) {
         if (wordSpacing.getEnum() == Constants.EN_NORMAL) {
             // give word spaces the possibility to shrink by a third,
@@ -89,6 +105,10 @@ public class SpaceVal {
         }
     }
 
+    /**
+     * @param letterSpacing property
+     * @return space value
+     */
     public static SpaceVal makeLetterSpacing(Property letterSpacing) {
         if (letterSpacing.getEnum() == Constants.EN_NORMAL) {
             // letter spaces are set to zero (or use different values?)
