@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,19 +15,22 @@
  * limitations under the License.
  */
 
-/* $Id: ColorExt.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: ColorExt.java 1466146 2013-04-09 17:31:41Z vhennebert $ */
 
 package org.apache.fop.util;
 
 import java.awt.Color;
 import java.awt.color.ColorSpace;
+import java.util.Arrays;
 
 /**
  * Color helper class.
  * <p>
  * This class extends java.awt.Color class keeping track of the original color
  * property values specified by the fo user in a rgb-icc call.
+ * @deprecated Replaced by {@link org.apache.xmlgraphics.java2d.color.ColorWithAlternatives}
  */
+@Deprecated
 public final class ColorExt extends Color {
     //
     private static final long serialVersionUID = 1L;
@@ -36,11 +39,11 @@ public final class ColorExt extends Color {
     private float rgbReplacementRed;
     private float rgbReplacementGreen;
     private float rgbReplacementBlue;
-    
+
     private String iccProfileName;
     private String iccProfileSrc;
     private ColorSpace colorSpace;
-    
+
     private float[] colorValues;
 
     /*
@@ -59,7 +62,7 @@ public final class ColorExt extends Color {
 
     /**
      * Create ColorExt object backup up FO's rgb-icc color function
-     * 
+     *
      * @param redReplacement
      *            Red part of RGB replacement color that will be used when ICC
      *            profile can not be loaded
@@ -95,7 +98,7 @@ public final class ColorExt extends Color {
 
     /**
      * Create ColorExt object backing up SVG's icc-color function.
-     * 
+     *
      * @param red
      *            Red value resulting from the conversion from the user provided
      *            (icc) color values to the batik (rgb) color space
@@ -120,6 +123,7 @@ public final class ColorExt extends Color {
     public static ColorExt createFromSvgIccColor(float red, float green,
             float blue, float opacity, String profileName, String profileHref,
             ColorSpace profileCS, float[] colorValues) {
+        //TODO this method is not referenced by FOP, can it be deleted?
         ColorExt ce = new ColorExt(red, green, blue, opacity);
         ce.rgbReplacementRed = -1;
         ce.rgbReplacementGreen = -1;
@@ -132,9 +136,62 @@ public final class ColorExt extends Color {
 
     }
 
+    /** {@inheritDoc} */
+    @Override
+    public int hashCode() {
+        //implementation from the superclass should be good enough for our purposes
+        return super.hashCode();
+    }
+
+    /** {@inheritDoc} */
+    @Override
+    public boolean equals(Object obj) {
+        if (this == obj) {
+            return true;
+        }
+        if (!super.equals(obj)) {
+            return false;
+        }
+        if (getClass() != obj.getClass()) {
+            return false;
+        }
+        ColorExt other = (ColorExt)obj;
+        //TODO maybe use super.getColorComponents() instead
+        if (!Arrays.equals(colorValues, other.colorValues)) {
+            return false;
+        }
+        if (iccProfileName == null) {
+            if (other.iccProfileName != null) {
+                return false;
+            }
+        } else if (!iccProfileName.equals(other.iccProfileName)) {
+            return false;
+        }
+        if (iccProfileSrc == null) {
+            if (other.iccProfileSrc != null) {
+                return false;
+            }
+        } else if (!iccProfileSrc.equals(other.iccProfileSrc)) {
+            return false;
+        }
+        if (Float.floatToIntBits(rgbReplacementBlue)
+                != Float.floatToIntBits(other.rgbReplacementBlue)) {
+            return false;
+        }
+        if (Float.floatToIntBits(rgbReplacementGreen)
+                != Float.floatToIntBits(other.rgbReplacementGreen)) {
+            return false;
+        }
+        if (Float.floatToIntBits(rgbReplacementRed)
+                != Float.floatToIntBits(other.rgbReplacementRed)) {
+            return false;
+        }
+        return true;
+    }
+
     /**
      * Get ICC profile name
-     * 
+     *
      * @return ICC profile name
      */
     public String getIccProfileName() {
@@ -143,7 +200,7 @@ public final class ColorExt extends Color {
 
     /**
      * Get ICC profile source
-     * 
+     *
      * @return ICC profile source
      */
     public String getIccProfileSrc() {
@@ -154,14 +211,19 @@ public final class ColorExt extends Color {
      * @return the original ColorSpace
      */
     public ColorSpace getOrigColorSpace() {
+        //TODO this method is probably unnecessary due to super.cs and getColorSpace()
         return this.colorSpace;
     }
 
     /**
+     * Returns the original color values.
      * @return the original color values
      */
     public float[] getOriginalColorComponents() {
-        return this.colorValues;
+        //TODO this method is probably unnecessary due to super.fvalue and getColorComponents()
+        float[] copy = new float[this.colorValues.length];
+        System.arraycopy(this.colorValues, 0, copy, 0, copy.length);
+        return copy;
     }
 
     /**
@@ -176,7 +238,9 @@ public final class ColorExt extends Color {
         sb.append(this.rgbReplacementGreen + ",");
         sb.append(this.rgbReplacementBlue + ",");
         sb.append(this.iccProfileName + ",");
-        sb.append("\"" + this.iccProfileSrc + "\"");
+        if (this.iccProfileSrc != null) {
+            sb.append("\"" + this.iccProfileSrc + "\"");
+        }
         float[] colorComponents = this.getColorComponents(null);
         for (int ix = 0; ix < colorComponents.length; ix++) {
             sb.append(",");
