@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AdvancedMessageFormat.java 1058988 2011-01-14 12:58:53Z spepping $ */
+/* $Id: AdvancedMessageFormat.java 1616312 2014-08-06 19:19:31Z gadams $ */
 
 package org.apache.fop.util.text;
 
@@ -146,10 +146,13 @@ public class AdvancedMessageFormat {
                     i++;
                     ch = pattern.charAt(i);
                 }
-                //no break here! Must be right before "default" section
+                sb.append(ch);
+                i++;
+                break;
             default:
                 sb.append(ch);
                 i++;
+                break;
             }
         }
         if (sb.length() > 0) {
@@ -170,7 +173,7 @@ public class AdvancedMessageFormat {
             }
         } else {
             String format = parts[1];
-            PartFactory factory = (PartFactory)PART_FACTORIES.get(format);
+            PartFactory factory = PART_FACTORIES.get(format);
             if (factory == null) {
                 throw new IllegalArgumentException(
                         "No PartFactory available under the name: " + format);
@@ -184,7 +187,7 @@ public class AdvancedMessageFormat {
     }
 
     private static Function getFunction(String functionName) {
-        return (Function)FUNCTIONS.get(functionName);
+        return FUNCTIONS.get(functionName);
     }
 
     /**
@@ -398,7 +401,7 @@ public class AdvancedMessageFormat {
 
         protected List<Part> parts = new java.util.ArrayList<Part>();
         private boolean conditional;
-        private boolean hasSections = false;
+        private boolean hasSections;
 
         public CompositePart(boolean conditional) {
             this.conditional = conditional;
