@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: MIFElement.java 985537 2010-08-14 17:17:00Z jeremias $ */
+/* $Id: MIFElement.java 1610839 2014-07-15 20:25:58Z vhennebert $ */
 
 package org.apache.fop.render.mif;
 
@@ -34,15 +34,15 @@ public class MIFElement {
     /** name */
     protected String name;
     /** value string */
-    protected String valueStr = null;
+    protected String valueStr;
     /** value elements */
-    protected List valueElements = null;
+    protected List valueElements;
     /** true if started */
-    protected boolean started = false;
+    protected boolean started;
     /** true if finishing */
-    protected boolean finish = false;
+    protected boolean finish;
     /** true if finished */
-    protected boolean finished = false;
+    protected boolean finished;
 
     /**
      * @param name a name
