@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: MIFHandler.java 1357883 2012-07-05 20:29:53Z gadams $ */
+/* $Id: MIFHandler.java 1479969 2013-05-07 16:23:13Z vhennebert $ */
 
 package org.apache.fop.render.mif;
 
@@ -57,7 +57,6 @@ import org.apache.fop.fo.pagination.PageSequenceMaster;
 import org.apache.fop.fo.pagination.SimplePageMaster;
 import org.apache.fop.fo.pagination.StaticContent;
 import org.apache.fop.fonts.FontSetup;
-import org.apache.fop.render.DefaultFontResolver;
 
 // TODO: do we really want every method throwing a SAXException
 
@@ -91,7 +90,7 @@ public class MIFHandler extends FOEventHandler {
         super(ua);
         outStream = os;
         boolean base14Kerning = false; //TODO - FIXME
-        FontSetup.setup(fontInfo, null, new DefaultFontResolver(ua), base14Kerning);
+        FontSetup.setup(fontInfo, null, ua.getResourceResolver(), base14Kerning);
     }
 
     /** {@inheritDoc} */
@@ -352,12 +351,12 @@ public class MIFHandler extends FOEventHandler {
     }
 
     public void character(Character c) {
-        appendCharacters ( new String ( new char[] {c.getCharacter()} ) );
+        appendCharacters(new String(new char[] {c.getCharacter()}));
     }
 
     /** {@inheritDoc} */
     public void characters(FOText foText) {
-        appendCharacters ( foText.getCharSequence().toString() );
+        appendCharacters(foText.getCharSequence().toString());
     }
 
     /** {@inheritDoc} */
@@ -368,7 +367,7 @@ public class MIFHandler extends FOEventHandler {
     public void endPageNumber(PageNumber pagenum) {
     }
 
-    private void appendCharacters ( String str ) {
+    private void appendCharacters(String str) {
         if (para != null) {
             str = str.trim();
             // break into nice length chunks
