@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: SVGPainter.java 1293736 2012-02-26 02:29:01Z gadams $ */
+/* $Id: SVGPainter.java 1537948 2013-11-01 14:34:18Z gadams $ */
 
 package org.apache.fop.render.svg;
 
@@ -30,7 +30,6 @@ import java.io.IOException;
 import java.util.Map;
 
 import org.w3c.dom.Document;
-
 import org.xml.sax.SAXException;
 import org.xml.sax.helpers.AttributesImpl;
 
@@ -46,7 +45,6 @@ import org.apache.fop.render.ImageHandlerUtil;
 import org.apache.fop.render.RenderingContext;
 import org.apache.fop.render.intermediate.AbstractIFPainter;
 import org.apache.fop.render.intermediate.IFConstants;
-import org.apache.fop.render.intermediate.IFContext;
 import org.apache.fop.render.intermediate.IFException;
 import org.apache.fop.render.intermediate.IFState;
 import org.apache.fop.render.intermediate.IFUtil;
@@ -60,12 +58,11 @@ import org.apache.fop.util.XMLUtil;
 /**
  * IFPainter implementation that writes SVG.
  */
-public class SVGPainter extends AbstractIFPainter implements SVGConstants {
-
-    private AbstractSVGDocumentHandler parent;
+public class SVGPainter extends AbstractIFPainter<AbstractSVGDocumentHandler>
+        implements SVGConstants {
 
     /** The SAX content handler that receives the generated XML events. */
-    protected GenerationHelperContentHandler handler;
+    private GenerationHelperContentHandler handler;
 
     private static final int MODE_NORMAL = 0;
     private static final int MODE_TEXT = 1;
@@ -79,17 +76,11 @@ public class SVGPainter extends AbstractIFPainter implements SVGConstants {
      */
     public SVGPainter(AbstractSVGDocumentHandler parent,
             GenerationHelperContentHandler contentHandler) {
-        super();
-        this.parent = parent;
+        super(parent);
         this.handler = contentHandler;
         this.state = IFState.create();
     }
 
-    /** {@inheritDoc} */
-    protected IFContext getContext() {
-        return parent.getContext();
-    }
-
     /** {@inheritDoc} */
     public void startViewport(AffineTransform transform, Dimension size, Rectangle clipRect)
             throws IFException {
@@ -155,16 +146,16 @@ public class SVGPainter extends AbstractIFPainter implements SVGConstants {
     }
 
     /** {@inheritDoc} */
-    public void startGroup(AffineTransform[] transforms) throws IFException {
-        startGroup(SVGUtil.formatAffineTransformsMptToPt(transforms));
+    public void startGroup(AffineTransform[] transforms, String layer) throws IFException {
+        startGroup(SVGUtil.formatAffineTransformsMptToPt(transforms), layer);
     }
 
     /** {@inheritDoc} */
-    public void startGroup(AffineTransform transform) throws IFException {
-        startGroup(SVGUtil.formatAffineTransformMptToPt(transform));
+    public void startGroup(AffineTransform transform, String layer) throws IFException {
+        startGroup(SVGUtil.formatAffineTransformMptToPt(transform), layer);
     }
 
-    private void startGroup(String transform) throws IFException {
+    private void startGroup(String transform, String layer) throws IFException {
         try {
             AttributesImpl atts = new AttributesImpl();
             if (transform != null && transform.length() > 0) {
@@ -191,7 +182,7 @@ public class SVGPainter extends AbstractIFPainter implements SVGConstants {
         try {
             establish(MODE_NORMAL);
 
-            ImageManager manager = getUserAgent().getFactory().getImageManager();
+            ImageManager manager = getUserAgent().getImageManager();
             ImageInfo info = null;
             try {
                 ImageSessionContext sessionContext = getUserAgent().getImageSessionContext();
@@ -268,6 +259,13 @@ public class SVGPainter extends AbstractIFPainter implements SVGConstants {
         //TODO Implement me!!!
     }
 
+    /** {@inheritDoc} */
+    public void clipBackground(Rectangle rect, BorderProps bpsBefore, BorderProps bpsAfter,
+            BorderProps bpsStart, BorderProps bpsEnd) throws IFException {
+        //TODO Implement me!!!
+
+    }
+
     /** {@inheritDoc} */
     public void fillRect(Rectangle rect, Paint fill) throws IFException {
         if (fill == null) {
@@ -408,4 +406,13 @@ public class SVGPainter extends AbstractIFPainter implements SVGConstants {
         }
     }
 
+    /** {@inheritDoc} */
+    public void fillBackground(Rectangle rect, Paint fill, BorderProps bpsBefore,
+            BorderProps bpsAfter, BorderProps bpsStart, BorderProps bpsEnd) throws IFException {
+        // Not supported in SVG
+
+    }
+
+
+
 }
