@@ -15,29 +15,21 @@
  * limitations under the License.
  */
 
-/* $Id: BasicTranscoderTestSuite.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: BasicTranscoderTestSuite.java 1178747 2011-10-04 10:09:01Z vhennebert $ */
 
 package org.apache.fop;
 
-import junit.framework.Test;
-import junit.framework.TestSuite;
+import org.junit.runner.RunWith;
+import org.junit.runners.Suite;
+import org.junit.runners.Suite.SuiteClasses;
 
 /**
  * Test suite for basic functionality of FOP's transcoders.
  */
+@RunWith(Suite.class)
+@SuiteClasses({
+    BasicPDFTranscoderTestCase.class,
+    BasicPSTranscoderTestCase.class
+})
 public class BasicTranscoderTestSuite {
-
-    /**
-     * Builds the test suite
-     * @return the test suite
-     */
-    public static Test suite() {
-        TestSuite suite = new TestSuite(
-            "Basic functionality test suite for FOP's transcoders");
-        //$JUnit-BEGIN$
-        suite.addTest(new TestSuite(BasicPDFTranscoderTestCase.class));
-        suite.addTest(new TestSuite(BasicPSTranscoderTestCase.class));
-        //$JUnit-END$
-        return suite;
-    }
 }
