@@ -15,17 +15,30 @@
  * limitations under the License.
  */
 
-/* $Id: FontAttributesMissingTestCase.java 1198853 2011-11-07 18:18:29Z vhennebert $ */
+/* $Id: FontAttributesMissingTestCase.java 1551536 2013-12-17 13:15:06Z vhennebert $ */
 
 package org.apache.fop.config;
 
+import java.io.IOException;
+
+import org.xml.sax.SAXException;
+
+import org.apache.fop.apps.FopConfBuilder;
+import org.apache.fop.apps.PDFRendererConfBuilder;
+
 /**
  * this font is without a metrics-url or an embed-url
  */
 public class FontAttributesMissingTestCase extends BaseDestructiveUserConfigTest {
 
-    @Override
-    public String getUserConfigFilename() {
-        return "test_font_attributes_missing.xconf";
+    public FontAttributesMissingTestCase() throws SAXException, IOException {
+        super(new FopConfBuilder().setStrictValidation(true)
+                                  .startRendererConfig(PDFRendererConfBuilder.class)
+                                      .startFontsConfig()
+                                          .startFont(null, null)
+                                              .addTriplet("Gladiator", "normal", "normal")
+                                          .endFont()
+                                      .endFontConfig()
+                                  .endRendererConfig().build());
     }
 }
