@@ -15,23 +15,35 @@
  * limitations under the License.
  */
 
-/* $Id: PDFAConformanceTestCase.java 1198853 2011-11-07 18:18:29Z vhennebert $ */
+/* $Id: PDFAConformanceTestCase.java 1584765 2014-04-04 15:52:30Z vhennebert $ */
 
 package org.apache.fop.render.pdf;
 
+import java.io.File;
+import java.io.IOException;
+import java.util.HashMap;
+import java.util.Map;
+
+import org.junit.Test;
+import org.xml.sax.SAXException;
+
 import static org.junit.Assert.fail;
 
-import java.io.File;
-
 import org.apache.fop.apps.FOUserAgent;
+import org.apache.fop.events.EventChecker;
+import org.apache.fop.pdf.PDFAMode;
 import org.apache.fop.pdf.PDFConformanceException;
-import org.junit.Test;
+import org.apache.fop.svg.SVGEventProducer;
 
 /**
  * Tests PDF/A-1 functionality.
  */
 public class PDFAConformanceTestCase extends BasePDFTest {
 
+    public PDFAConformanceTestCase() throws SAXException, IOException {
+        super(getDefaultConfFile());
+    }
+
     private File foBaseDir = new File("test/xml/pdf-a");
     private boolean dumpPDF = Boolean.getBoolean("PDFAConformanceTestCase.dumpPDF");
 
@@ -59,17 +71,12 @@ public class PDFAConformanceTestCase extends BasePDFTest {
      * Test exception when PDF/A-1 is enabled together with encryption.
      * @throws Exception if the test fails
      */
-    @Test
+    @Test(expected = PDFConformanceException.class)
     public void testNoEncryption() throws Exception {
         final FOUserAgent ua = getUserAgent();
         ua.getRendererOptions().put("owner-password", "mypassword"); //To enabled encryption
         File foFile = new File(foBaseDir, "minimal-pdf-a.fo");
-        try {
-            convertFO(foFile, ua, dumpPDF);
-            fail("Expected PDFConformanceException. PDF/A-1 and PDF encryption don't go together.");
-        } catch (PDFConformanceException e) {
-            //Good!
-        }
+        convertFO(foFile, ua, dumpPDF);
     }
 
     /**
@@ -106,4 +113,31 @@ public class PDFAConformanceTestCase extends BasePDFTest {
         }
     }
 
+    @Test
+    public void svgTransparency() throws Exception {
+        FOUserAgent ua = getUserAgent();
+        EventChecker eventChecker = setupEventChecker(ua, "transparencyIgnored");
+        File foFile = new File(foBaseDir, "svg-transparency.fo");
+        convertFO(foFile, ua, dumpPDF);
+        eventChecker.end();
+    }
+
+    @Test
+    public void svgContainingBitmapWithTransparency() throws Exception {
+        FOUserAgent ua = getUserAgent();
+        EventChecker eventChecker = setupEventChecker(ua, "bitmapWithTransparency");
+        File foFile = new File(foBaseDir, "svg-with-transparent-image.fo");
+        convertFO(foFile, ua, dumpPDF);
+        eventChecker.end();
+    }
+
+    private EventChecker setupEventChecker(FOUserAgent ua, String expectedEvent) {
+        Map<String, Object> params = new HashMap<String, Object>();
+        params.put("pdfProfile", PDFAMode.PDFA_1B);
+        EventChecker eventChecker = new EventChecker(SVGEventProducer.class.getName()
+                + "." + expectedEvent, params);
+        ua.getEventBroadcaster().addEventListener(eventChecker);
+        return eventChecker;
+    }
+
 }
