@@ -15,20 +15,22 @@
  * limitations under the License.
  */
 
-/* $Id: PDFEncodingTestCase.java 1198853 2011-11-07 18:18:29Z vhennebert $ */
+/* $Id: PDFEncodingTestCase.java 1551536 2013-12-17 13:15:06Z vhennebert $ */
 
 package org.apache.fop.render.pdf;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertTrue;
-
 import java.io.File;
 import java.io.IOException;
 import java.util.StringTokenizer;
 
-import org.apache.fop.apps.FOUserAgent;
 import org.junit.Ignore;
 import org.junit.Test;
+import org.xml.sax.SAXException;
+
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
+
+import org.apache.fop.apps.FOUserAgent;
 
 /** Test that characters are correctly encoded in a generated PDF file */
 public class PDFEncodingTestCase extends BasePDFTest {
@@ -37,6 +39,10 @@ public class PDFEncodingTestCase extends BasePDFTest {
     static final String INPUT_FILE = "test/xml/pdf-encoding/pdf-encoding-test.xconf";
     static final String TEST_MARKER = "PDFE_TEST_MARK_";
 
+    public PDFEncodingTestCase() throws SAXException, IOException {
+        super(INPUT_FILE);
+    }
+
 
     /**
      * create an FOUserAgent for our tests
@@ -47,11 +53,6 @@ public class PDFEncodingTestCase extends BasePDFTest {
         return a;
     }
 
-    /** @return our specific config */
-    protected File getUserConfigFile() {
-        return new File(INPUT_FILE);
-    }
-
     /**
      * Test using a standard FOP font
      * @throws Exception checkstyle wants a comment here, even a silly one
