@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: NestedTable.java 1297404 2012-03-06 10:17:54Z vhennebert $ */
+/* $Id: NestedTable.java 1551536 2013-12-17 13:15:06Z vhennebert $ */
 
 
 /*
@@ -30,14 +30,13 @@ package org.apache.fop.render.rtf.rtflib.testdocs;
 import java.io.IOException;
 
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfDocumentArea;
+import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfParagraph;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfSection;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTable;
-import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfParagraph;
-import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTableRow;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTableCell;
+import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTableRow;
 
 /**  Generates an RTF document to test nested tables with the jfor rtflib package.
- *  @author Bertrand Delacretaz bdelacretaz@codeconsult.ch
  */
 
 class NestedTable extends TestDocument {
@@ -78,8 +77,8 @@ class NestedTable extends TestDocument {
         // second row contains nested table
         {
             RtfTableRow r = tbl.newTableRow();
-            r.newTableCell(40 * MM_TO_TWIPS).newParagraph().newText
-                    ("cell 1,0, width 40mm, to the left of nested table.");
+            r.newTableCell(40 * MM_TO_TWIPS).newParagraph().newText(
+                    "cell 1,0, width 40mm, to the left of nested table.");
 
             final RtfTableCell c = r.newTableCell(80 * MM_TO_TWIPS);
             c.newParagraph().newText("cell 1,1, width 80mm, this text is "
@@ -88,17 +87,17 @@ class NestedTable extends TestDocument {
             fillNestedTable(c.newTable(new DummyTableColumnsInfo()), 1);
             c.newParagraph().newText("AFTER NESTED TABLE");
 
-            r.newTableCell(40 * MM_TO_TWIPS).newParagraph().newText
-                    ("cell 1,2, width 40mm, to the right of nested table.");
+            r.newTableCell(40 * MM_TO_TWIPS).newParagraph().newText(
+                    "cell 1,2, width 40mm, to the right of nested table.");
         }
 
         // third row, normal
         {
             RtfTableRow r = tbl.newTableRow();
-            r.newTableCell(80 * MM_TO_TWIPS).newParagraph().newText
-                    ("cell 2,0, width 80mm, this row has two cells.");
-            r.newTableCell(80 * MM_TO_TWIPS).newParagraph().newText
-                    ("cell 2,1, width 80mm, last cell.");
+            r.newTableCell(80 * MM_TO_TWIPS).newParagraph().newText(
+                    "cell 2,0, width 80mm, this row has two cells.");
+            r.newTableCell(80 * MM_TO_TWIPS).newParagraph().newText(
+                    "cell 2,1, width 80mm, last cell.");
         }
 
     }
@@ -118,8 +117,8 @@ class NestedTable extends TestDocument {
         // second row contains nested table
         {
             RtfTableRow r = tbl.newTableRow();
-            r.newTableCell(40 * MM_TO_TWIPS).newParagraph().newText
-                    ("cell 1,0, width 40mm, to the left of nested tables.");
+            r.newTableCell(40 * MM_TO_TWIPS).newParagraph().newText(
+                    "cell 1,0, width 40mm, to the left of nested tables.");
 
             final RtfTableCell c = r.newTableCell(80 * MM_TO_TWIPS);
             c.newParagraph().newText("cell 1,1, width 80mm, this text is "
@@ -130,17 +129,17 @@ class NestedTable extends TestDocument {
             fillNestedTable(c.newTable(new DummyTableColumnsInfo()), 3);
             c.newParagraph().newText("AFTER");
 
-            r.newTableCell(40 * MM_TO_TWIPS).newParagraph().newText
-                    ("cell 1,2, width 40mm, to the right of nested table.");
+            r.newTableCell(40 * MM_TO_TWIPS).newParagraph().newText(
+                    "cell 1,2, width 40mm, to the right of nested table.");
         }
 
         // third row, normal
         {
             RtfTableRow r = tbl.newTableRow();
-            r.newTableCell(80 * MM_TO_TWIPS).newParagraph().newText
-                    ("cell 2,0, width 80mm, this row has two cells.");
-            r.newTableCell(80 * MM_TO_TWIPS).newParagraph().newText
-                    ("cell 2,1, width 80mm, last cell.");
+            r.newTableCell(80 * MM_TO_TWIPS).newParagraph().newText(
+                    "cell 2,0, width 80mm, this row has two cells.");
+            r.newTableCell(80 * MM_TO_TWIPS).newParagraph().newText(
+                    "cell 2,1, width 80mm, last cell.");
         }
     }
 
@@ -162,8 +161,8 @@ class NestedTable extends TestDocument {
         // second row contains nested table
         {
             RtfTableRow r = tbl.newTableRow();
-            r.newTableCell(40 * MM_TO_TWIPS).newParagraph().newText
-                    ("cell 1,0, width 40mm, to the left of nested tables.");
+            r.newTableCell(40 * MM_TO_TWIPS).newParagraph().newText(
+                    "cell 1,0, width 40mm, to the left of nested tables.");
 
             final RtfTableCell c = r.newTableCell(80 * MM_TO_TWIPS);
             c.newParagraph().newText("cell 1,1, width 80mm, this text is "
@@ -174,17 +173,17 @@ class NestedTable extends TestDocument {
             fillNestedTable(c.newTable(new DummyTableColumnsInfo()), 6);
             c.newParagraph().newText("AFTER");
 
-            r.newTableCell(40 * MM_TO_TWIPS).newParagraph().newText
-                    ("cell 1,2, width 40mm, to the right of nested table.");
+            r.newTableCell(40 * MM_TO_TWIPS).newParagraph().newText(
+                    "cell 1,2, width 40mm, to the right of nested table.");
         }
 
         // third row, normal
         {
             RtfTableRow r = tbl.newTableRow();
-            r.newTableCell(80 * MM_TO_TWIPS).newParagraph().newText
-                    ("cell 2,0, width 80mm, this row has two cells.");
-            r.newTableCell(80 * MM_TO_TWIPS).newParagraph().newText
-                    ("cell 2,1, width 80mm, last cell.");
+            r.newTableCell(80 * MM_TO_TWIPS).newParagraph().newText(
+                    "cell 2,0, width 80mm, this row has two cells.");
+            r.newTableCell(80 * MM_TO_TWIPS).newParagraph().newText(
+                    "cell 2,1, width 80mm, last cell.");
         }
     }
 
