@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: SimpleTable.java 1297404 2012-03-06 10:17:54Z vhennebert $ */
+/* $Id: SimpleTable.java 1551536 2013-12-17 13:15:06Z vhennebert $ */
 
 
 /*
@@ -32,11 +32,10 @@ import java.io.IOException;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfDocumentArea;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfSection;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTable;
-import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTableRow;
 import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTableCell;
+import org.apache.fop.render.rtf.rtflib.rtfdoc.RtfTableRow;
 
 /**  Generates a simple RTF test document for the jfor rtflib package.
- *  @author Bertrand Delacretaz bdelacretaz@codeconsult.ch
  */
 class SimpleTable extends TestDocument {
     /** generate the body of the test document */
@@ -48,8 +47,8 @@ class SimpleTable extends TestDocument {
     protected void generateDocument(RtfDocumentArea rda, RtfSection sect)
     throws IOException {
         final RtfTable tbl = sect.newTable(new DummyTableColumnsInfo());
-        tbl.newTableRow().newTableCell(C1W * INCH_TO_TWIPS).newParagraph().newText
-                ("Here's a table row with just one cell, width " + C1W + "''");
+        tbl.newTableRow().newTableCell(C1W * INCH_TO_TWIPS).newParagraph().newText(
+                "Here's a table row with just one cell, width " + C1W + "''");
 
         for (int row = 0; row < MAX_ROW; row++) {
             final RtfTableRow r = tbl.newTableRow();
