@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BitmapProducer.java 426576 2006-07-28 15:44:37Z jeremias $ */
+/* $Id: BitmapProducer.java 746664 2009-02-22 12:40:44Z jeremias $ */
 
 package org.apache.fop.visual;
 
@@ -31,9 +31,10 @@ public interface BitmapProducer {
      * Produces a BufferedImage from the source file by invoking the FO processor and
      * converting the generated output file to a bitmap image if necessary.
      * @param src the source FO or XML file
+     * @param index the index of the producer inside the test set
      * @param context context information for the conversion
      * @return the generated BufferedImage
      */
-    BufferedImage produce(File src, ProducerContext context);
-    
+    BufferedImage produce(File src, int index, ProducerContext context);
+
 }
