@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ConvertUtils.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: ConvertUtils.java 1551536 2013-12-17 13:15:06Z vhennebert $ */
 
 package org.apache.fop.visual;
 
@@ -27,7 +27,10 @@ import org.apache.commons.logging.Log;
 /**
  * Utilities for converting files with external converters.
  */
-public class ConvertUtils {
+public final class ConvertUtils {
+
+    private ConvertUtils() {
+    }
 
     /**
      * Calls an external converter application (GhostScript, for example).
