@@ -15,12 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: ReferenceBitmapLoader.java 746664 2009-02-22 12:40:44Z jeremias $ */
+/* $Id: ReferenceBitmapLoader.java 1356646 2012-07-03 09:46:41Z mehdi $ */
 
 package org.apache.fop.visual;
 
 import java.awt.image.BufferedImage;
 import java.io.File;
+import java.net.URI;
 
 import org.apache.avalon.framework.configuration.Configurable;
 import org.apache.avalon.framework.configuration.Configuration;
@@ -42,6 +43,10 @@ public class ReferenceBitmapLoader extends AbstractBitmapProducer implements Con
 
     private File bitmapDirectory;
 
+    public ReferenceBitmapLoader(URI baseUri) {
+        super(baseUri);
+    }
+
     /** @see org.apache.avalon.framework.configuration.Configurable */
     public void configure(Configuration cfg) throws ConfigurationException {
         this.bitmapDirectory = new File(cfg.getChild("directory").getValue(null));
